=== modified file 'HACKING'
--- HACKING	2006-08-01 18:27:42 +0000
+++ HACKING	2006-10-13 07:22:23 +0000
@@ -56,7 +56,7 @@
 applies to modules and classes.
 
 If you wish to change the behaviour of a supported API in an incompatible
-way, you need to change its name as well. For instance, if I add a optional keyword
+way, you need to change its name as well. For instance, if I add an optional keyword
 parameter to branch.commit - that's fine. On the other hand, if I add a
 keyword parameter to branch.commit which is a *required* transaction
 object, I should rename the API - i.e. to 'branch.commit_transaction'. 
@@ -67,7 +67,7 @@
 details for you - such as updating the docstring, and issuing a warning
 when the old api is used.
 
-For unsupported API's, it does not hurt to follow this discipline, but its
+For unsupported API's, it does not hurt to follow this discipline, but it's
 not required. Minimally though, please try to rename things so that
 callers will at least get an AttributeError rather than weird results.
 
@@ -78,7 +78,7 @@
 There are some common requirements in the library: some parameters need to be
 unicode safe, some need byte strings, and so on. At the moment we have
 only codified one specific pattern: Parameters that need to be unicode
-should be check via 'bzrlib.osutils.safe_unicode'. This will coerce the
+should be checked via 'bzrlib.osutils.safe_unicode'. This will coerce the
 input into unicode in a consistent fashion, allowing trivial strings to be
 used for programmer convenience, but not performing unpredictably in the
 presence of different locales.
@@ -209,6 +209,78 @@
 > factory, then yes, foo_factory is what I would use.
 
 
+Registries
+----------
+
+Several places in Bazaar use (or will use) a registry, which is a 
+mapping from names to objects or classes.  The registry allows for 
+loading in registered code only when it's needed, and keeping
+associated information such as a help string or description.
+
+
+Lazy Imports
+------------
+
+To make startup time faster, we use the ``bzrlib.lazy_import`` module to
+delay importing modules until they are actually used. ``lazy_import`` uses
+the same syntax as regular python imports. So to import a few modules in a
+lazy fashion do::
+
+  from bzrlib.lazy_import import lazy_import
+  lazy_import(globals(), """
+  import os
+  import subprocess
+  import sys
+  import time
+
+  from bzrlib import (
+     errors,
+     transport,
+     revision as _mod_revision,
+     )
+  import bzrlib.transport
+  import bzrlib.xml5
+  """)
+
+At this point, all of these exist as a ``ImportReplacer`` object, ready to
+be imported once a member is accessed. Also, when importing a module into
+the local namespace, which is likely to clash with variable names, it is
+recommended to prefix it as ``_mod_<module>``. This makes it clean that
+the variable is a module, and these object should be hidden anyway, since
+they shouldn't be imported into other namespaces.
+
+
+Modules versus Members
+~~~~~~~~~~~~~~~~~~~~~~
+
+While it is possible for ``lazy_import()`` to import members of a module
+when using the ``from module import member`` syntax, it is recommended to
+only use that syntax to load sub modules ``from module import submodule``.
+This is because variables and classes can frequently be used without
+needing a sub-member for example::
+
+  lazy_import(globals(), """
+  from module import MyClass
+  """)
+
+  def test(x):
+      return isinstance(x, MyClass)
+
+This will incorrectly fail, because ``MyClass`` is a ``ImportReplacer``
+object, rather than the real class.
+
+
+Passing to other variables
+~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+It also is incorrect to assign ``ImportReplacer`` objects to other variables.
+Because the replacer only knows about the original name, it is unable to
+replace other variables. The ``ImportReplacer`` class will raise an
+``IllegalUseOfScopeReplacer`` exception if it can figure out that this
+happened. But it requires accessing a member more than once from the new
+variable, so some bugs are not detected right away.
+
+
 Writing output
 ==============
 
@@ -248,6 +320,7 @@
 
 Writing tests
 =============
+
 In general tests should be placed in a file named test_FOO.py where 
 FOO is the logical thing under test. That file should be placed in the
 tests subdirectory under the package being tested.
@@ -279,9 +352,14 @@
     the command. We do this so that the library api has continual pressure
     on it to be as functional as the command line in a simple manner, and
     to isolate knock-on effects throughout the blackbox test suite when a
-    command changes it name or signature. Ideally only the tests for a
+    command changes its name or signature. Ideally only the tests for a
     given command are affected when a given command is changed.
 
+ 4. If you have a test which does actually require running bzr in a
+    subprocess you can use ``run_bzr_subprocess``. By default the spawned
+    process will not load plugins unless ``--allow-plugins`` is supplied.
+
+
 Doctests
 --------
 
@@ -404,7 +482,7 @@
     that cannot be displayed.
   
   strict
-    Attempting to print and unprintable character will cause a UnicodeError.
+    Attempting to print an unprintable character will cause a UnicodeError.
     This is for commands that are intended more as scripting support, rather
     than plain user review.
     For exampl: ``bzr ls`` is designed to be used with shell scripting. One

=== modified file 'NEWS'
--- NEWS	2006-09-27 05:12:54 +0000
+++ NEWS	2006-10-16 09:14:47 +0000
@@ -7,12 +7,167 @@
       mainline. These revision numbers are not yet accepted as input into bzr
       commands such as log, diff etc. (Robert Collins)
 
+    * ``bzr help commands`` output is now shorter (Aaron Bentley)
+
+    * New connection: ``bzr+http://`` which supports tunnelling the smart
+      protocol over an HTTP connection. If writing is enabled on the bzr
+      server, then you can write over the http connection.
+      (Andrew Bennetts)
+
+    * ``bzr`` now uses lazy importing to reduce the startup time. This has
+      a moderate effect on lots of actions, especially ones that have
+      little to do. For example ``bzr rocks`` time is down to 116ms from
+      283ms. (John Arbash Meinel)
+
+    * New Registry class to provide name-to-object registry-like support,
+      for example for schemes where plugins can register new classes to
+      do certain tasks (e.g. log formatters). Also provides lazy registration
+      to allow modules to be loaded on request. (John Arbash Meinel, Adeodato
+      Simó)
+
   API INCOMPATABILITY:
   
     * LogFormatter subclasses show now expect the 'revno' parameter to 
       show() to be a string rather than an int. (Robert Collins)
 
-  BUG FIXES:
+  INTERNALS:
+
+    * ``TestCase.run_bzr``, ``run_bzr_captured``, and ``run_bzr_subprocess``
+      can take a ``working_dir='foo'`` parameter, which will change directory 
+      for the command. (John Arbash Meinel)
+
+    * ``bzrlib.lazy_regex.lazy_compile`` can be used to create a proxy
+      around a regex, which defers compilation until first use. 
+      (John Arbash Meinel)
+
+    * ``TestCase.run_bzr_subprocess`` defaults to supplying the
+      ``--no-plugins`` parameter to ensure test reproducability, and avoid
+      problems with system-wide installed plugins. (John Arbash Meinel)
+
+    * Newly-initialized trees have unique root ids.  (Aaron Bentley)
+
+    * ``WorkingTree.set_root_id(None)`` is now deprecated. Please
+      pass in inventory.ROOT_ID if you want the default root id value.
+      (Robert Collins, John Arbash Meinel)
+
+    * New method ``WorkingTree.flush()`` which will write the current memory
+      inventory out to disk. At the same time, read_working_inventory will
+      no longer trash the current tree inventory if it has been modified within
+      the current lock, and the tree will now ``flush()`` automatically on
+      ``unlock()``. ``WorkingTree.set_root_id()`` has been updated to take
+      advantage of this functionality. (Robert Collins, John Arbash Meinel)
+
+  BUG FIXES:
+
+    * Don't require ``Content-Type`` in range responses. Assume they are a
+      single range if ``Content-Type`` does not exist.
+      (John Arbash Meinel, #62473)
+
+    * bzr branch/pull no longer complain about progress bar cleanup when
+      interrupted during fetch.  (Aaron Bentley, #54000)
+
+    * ``WorkingTree.set_parent_trees()`` uses the trees to directly write
+      the basis inventory, rather than going through the repository. This
+      allows us to have 1 inventory read, and 2 inventory writes when
+      committing a new tree. (John Arbash Meinel)
+
+    * When reverting, files that are not locally modified that do not exist
+      in the target are deleted, not just unversioned (Aaron Bentley)
+
+    * When trying to acquire a lock, don't fail immediately. Instead, try
+      a few times (up to 1 hour) before timing out. Also, report why the
+      lock is unavailable (John Arbash Meinel, #43521, #49556)
+
+    * Leave HttpTransportBase daughter classes decides how they
+      implement cloning. (Vincent Ladeuil, #61606)
+
+   * diff3 does not indicate conflicts on clean merge. (Aaron Bentley)
+
+  TESTING:
+
+    * New test base class TestCaseWithMemoryTransport offers memory-only
+      testing facilities: its not suitable for tests that need to mutate disk
+      state, but most tests should not need that and should be converted to
+      TestCaseWithMemoryTransport. (Robert Collins)
+
+    * ``TestCase.make_branch_and_memory_tree`` now takes a format
+      option to set the BzrDir, Repository and Branch formats of the
+      created objects. (Robert Collins, John Arbash Meinel)
+
+bzr 0.11  2006-10-02
+
+    * Smart server transport test failures on windows fixed. (Lukáš Lalinský).
+
+bzr 0.11rc2  2006-09-27
+
+  BUG FIXES:
+
+    * Test suite hangs on windows fixed. (Andrew Bennets, Alexander Belchenko).
+    
+    * Commit performance regression fixed. (Aaron Bentley, Robert Collins, John
+      Arbash Meinel).
+
+bzr 0.11rc1  2006-09-25
+
+  IMPROVEMENTS:
+
+    * Knit files now wait to create their contents until the first data is
+      added. The old code used to create an empty .knit and a .kndx with just
+      the header. However, this caused a lot of extra round trips over sftp.
+      This can change the time for ``bzr push`` to create a new remote branch
+      from 160s down to 100s. This also affects ``bzr commit`` performance when
+      adding new files, ``bzr commit`` on a new kernel-like tree drops from 50s
+      down to 40s (John Arbash Meinel, #44692)
+
+    * When an entire subtree has been deleted, commit will now report that
+      just the top of the subtree has been deleted, rather than reporting
+      all the individual items. (Robert Collins)
+
+    * Commit performs one less XML parse. (Robert Collins)
+
+    * ``bzr checkout`` now operates on readonly branches as well
+      as readwrite branches. This fixes bug #39542. (Robert Collins)
+
+    * ``bzr bind`` no longer synchronises history with the master branch.
+      Binding should be followed by an update or push to synchronise the 
+      two branches. This is closely related to the fix for bug #39542.
+      (Robert Collins)
+
+    * ``bzrlib.lazy_import.lazy_import`` function to create on-demand 
+      objects.  This allows all imports to stay at the global scope, but
+      modules will not actually be imported if they are not used.
+      (John Arbash Meinel)
+
+    * Support bzr:// and bzr+ssh:// urls to work with the new RPC-based
+      transport which will be used with the upcoming high-performance smart
+      server. The new command ``bzr serve`` will invoke bzr in server mode,
+      which processes these requests. (Andrew Bennetts, Robert Collins, Martin
+      Pool)
+
+    * New command ``bzr version-info`` which can be used to get a summary
+      of the current state of the tree. This is especially useful as part
+      of a build commands. See ``doc/version_info.txt`` for more information 
+      (John Arbash Meinel)
+
+  BUG FIXES:
+
+    * 'bzr inventory [FILE...]' allows restricting the file list to a
+      specific set of files. (John Arbash Meinel, #3631)
+
+    * Don't abort when annotating empty files (John Arbash Meinel, #56814)
+
+    * Add ``Stanza.to_unicode()`` which can be passed to another Stanza
+      when nesting stanzas. Also, add ``read_stanza_unicode`` to handle when
+      reading a nested Stanza. (John Arbash Meinel)
+
+    * Transform._set_mode() needs to stat the right file. 
+      (John Arbash Meinel, #56549)
+
+    * Raise WeaveFormatError rather than StopIteration when trying to read
+      an empty Weave file. (John Arbash Meinel, #46871)
+
+    * Don't access e.code for generic URLErrors, only HTTPErrors have .code.
+      (Vincent Ladeuil, #59835)
 
     * Handle boundary="" lines properly to allow access through a Squid proxy.
       (John Arbash Meinel, #57723)
@@ -28,11 +183,42 @@
     * Handle when LANG is not recognized by python. Emit a warning, but
       just revert to using 'ascii'. (John Arbash Meinel, #35392)
 
+    * Don't use preexec_fn on win32, as it is not supported by subprocess.
+      (John Arbash Meinel)
+
+    * Skip specific tests when the dependencies aren't met. This includes
+      some ``setup.py`` tests when ``python-dev`` is not available, and
+      some tests that depend on paramiko. (John Arbash Meinel, Mattheiu Moy)
+
+    * Fallback to Paramiko properly, if no ``ssh`` executable exists on
+      the system. (Andrew Bennetts, John Arbash Meinel)
+
+    * ``Branch.bind(other_branch)`` no longer takes a write lock on the
+      other branch, and will not push or pull between the two branches.
+      API users will need to perform a push or pull or update operation if they
+      require branch synchronisation to take place. (Robert Collins, #47344)
+
+    * When creating a tarball or zipfile export, export unicode names as utf-8
+      paths. This may not work perfectly on all platforms, but has the best
+      chance of working in the common case. (John Arbash Meinel, #56816)
+
+    * When committing, only files that exist in working tree or basis tree
+      may be specified (Aaron Bentley, #50793)
+
+  PORTABILITY:
+
+    * Fixes to run on Python 2.5 (Brian M. Carlson, Martin Pool, Marien Zwart)
+
   INTERNALS:
 
+    * TestCaseInTempDir now creates a separate directory for HOME, rather
+      than having HOME set to the same location as the working directory.
+      (John Arbash Meinel)
+
     * run_bzr_subprocess() can take an optional 'env_changes={}' parameter,
-      which will update os.environ inside the spawned child.
-      (John Arbash Meinel)
+      which will update os.environ inside the spawned child. It also can
+      take a 'universal_newlines=True', which helps when checking the output
+      of the command. (John Arbash Meinel)
 
     * Refactor SFTP vendors to allow easier re-use when ssh is used. 
       (Andrew Bennetts)
@@ -58,6 +244,59 @@
       NoSuchFile and forcing the caller to repeat their request.
       (John Arbash Meinel)
 
+<<<<<<< TREE
+    * WorkingTree has a new api ``unversion`` which allow the unversioning of
+      entries by their file id. (Robert Collins)
+
+    * WorkingTree.pending_merges is deprecated.  Please use the get_parent_ids
+      (introduced in 0.10) method instead. (Robert Collins)
+
+    * WorkingTree has a new lock_tree_write method which locks the branch for
+      read rather than write. This is appropriate for actions which only need
+      the branch data for reference rather than mutation. A new decorator
+      needs_tree_write_lock is provided in the workingtree module. Like the
+      needs_read_lock and needs_write_lock decorators this allows static 
+      declaration of the locking requirements of a function to ensure that
+      a lock is taken out for casual scripts. (Robert Collins, #54107)
+
+    * All WorkingTree methods which write to the tree, but not to the branch
+      have been converted to use ``needs_tree_write_lock`` rather than 
+      ``needs_write_lock``. Also converted is the revert, conflicts and tree
+      transform modules. This provides a modest performance improvement on 
+      metadir style trees, due to the reduce lock-acquisition, and a more
+      significant performance improvement on lightweight checkouts from 
+      remote branches, where trivial operations used to pay a significant 
+      penalty. It also provides the basis for allowing readonly checkouts.
+      (Robert Collins)
+
+    * Special case importing the standard library 'copy' module. This shaves
+      off 40ms of startup time, while retaining compatibility. See:
+      ``bzrlib/inspect_for_copy.py`` for more details. (John Arbash Meinel)
+
+    * WorkingTree has a new parent class MutableTree which represents the 
+      specialisations of Tree which are able to be altered. (Robert Collins)
+
+    * New methods mkdir and put_file_bytes_non_atomic on MutableTree that
+      mutate the tree and its contents. (Robert Collins)
+
+    * Transport behaviour at the root of the URL is now defined and tested.
+      (Andrew Bennetts, Robert Collins)
+
+  TESTING:
+
+    * New test helper classs MemoryTree. This is typically accessed via
+      ``self.make_branch_and_memory_tree()`` in test cases. (Robert Collins)
+      
+    * Add start_bzr_subprocess and stop_bzr_subprocess to allow test code to
+      continue running concurrently with a subprocess of bzr. (Andrew Bennetts,
+      Robert Collins)
+
+    * Add a new method ``Transport.get_smart_client()``. This is provided to
+      allow upgrades to a richer interface than the VFS one provided by
+      Transport. (Andrew Bennetts, Martin Pool)
+
+bzr 0.10  2006-08-29
+=======
     * bzrlib.tsort.merge_sorted now accepts 'generate_revnos'. This parameter
       will cause it to add another column to its output, which contains the
       dotted-decimal revno for each revision, as a tuple. (Robert Collins)
@@ -66,6 +305,7 @@
       LogFormatter.show_merge_revno. (Robert Collins)
 
 bzr 0.10.0RC1  2006-08-28
+>>>>>>> MERGE-SOURCE
   
   IMPROVEMENTS:
     * 'merge' now takes --uncommitted, to apply uncommitted changes from a
@@ -124,6 +364,9 @@
       in handling of revision properties. (John Arbash Meinel, Holger Krekel,
       #54723)
 
+    * The bzr selftest was failing on installed versions due to a bug in a new
+      test helper. (John Arbash Meinel, Robert Collins, #58057)
+
   INTERNALS:
 
     * ``bzrlib.cache_utf8`` contains ``encode()`` and ``decode()`` functions

=== modified file 'README'
--- README	2006-03-16 20:20:32 +0000
+++ README	2006-09-25 00:03:15 +0000
@@ -1,8 +1,8 @@
-*******************************
-Release notes for Bazaar-NG 0.8
-*******************************
+**********************
+README for Bazaar 0.11
+**********************
 
-bazaar-ng is a distributed/decentralized version control system.
+Bazaar is a distributed/decentralized version control system.
 
 Features include:
 
@@ -26,7 +26,7 @@
  * vibrant and active community
 
 For usage instructions see 
-http://bazaar.canonical.com/IntroductionToBzr
+http://doc.bazaar-vcs.org/
 or run "bzr help"
 
 For installation instructions see ./INSTALL

=== modified file 'TODO'
--- TODO	2006-06-03 16:23:09 +0000
+++ TODO	2006-10-11 01:17:15 +0000
@@ -371,6 +371,11 @@
   This has been implemented, but it does not remove the texts from
   the store.
 
+* ``bzrlib.lazy_import.lazy_import`` could check to see if a given
+  module has already been imported (look at ``sys.modules``). If it
+  has, then just import it directly, rather than creating a lazy
+  object.
+
 
 Large things
 ------------

=== modified file 'bzr'
--- bzr	2006-08-28 01:09:52 +0000
+++ bzr	2006-10-11 00:23:23 +0000
@@ -32,7 +32,7 @@
 
 REINVOKE = "__BZR_REINVOKE"
 NEED_VERS = (2, 4)
-KNOWN_PYTHONS = ('python2.4',)
+KNOWN_PYTHONS = ('python2.4', 'python2.5')
 
 if version_info < NEED_VERS:
     if not os.environ.has_key(REINVOKE):
@@ -50,7 +50,6 @@
     os.unsetenv(REINVOKE)
 
 
-
 profiling = False
 if '--profile-imports' in sys.argv:
     sys.argv.remove('--profile-imports')
@@ -61,8 +60,6 @@
 
 try:
     import bzrlib
-    import bzrlib.commands
-    import bzrlib.trace
 except ImportError, e:
     sys.stderr.write("bzr: ERROR: "
                      "Couldn't import bzrlib and dependencies.\n"
@@ -70,7 +67,16 @@
                      "\n")
     raise
 
-if bzrlib.version_info[:3] != (0, 11, 0):
+import bzrlib.inspect_for_copy
+bzrlib.inspect_for_copy.import_copy_with_hacked_inspect()
+
+import bzrlib.lazy_regex
+bzrlib.lazy_regex.install_lazy_compile()
+
+import bzrlib.commands
+import bzrlib.trace
+
+if bzrlib.version_info[:3] != (0, 12, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-08-28 15:04:19 +0000
+++ bzrlib/__init__.py	2006-09-25 00:03:15 +0000
@@ -35,7 +35,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (0, 11, 0, 'dev', 0)
+version_info = (0, 12, 0, 'dev', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2006-08-16 15:31:34 +0000
+++ bzrlib/add.py	2006-09-12 06:30:01 +0000
@@ -296,7 +296,7 @@
                 else:
                     this_ie = inv[this_id]
 
-            for subf in os.listdir(abspath):
+            for subf in sorted(os.listdir(abspath)):
                 # here we could use TreeDirectory rather than 
                 # string concatenation.
                 subp = bzrlib.osutils.pathjoin(directory.raw_path, subf)

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2006-06-20 03:57:11 +0000
+++ bzrlib/annotate.py	2006-09-29 20:24:37 +0000
@@ -29,7 +29,7 @@
 import time
 
 from bzrlib.config import extract_email_address
-from bzrlib.errors import BzrError
+from bzrlib.errors import NoEmailInUsername
 
 
 def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
@@ -39,7 +39,10 @@
 
     prevanno=''
     annotation = list(_annotate_file(branch, rev_id, file_id))
-    max_origin_len = max(len(origin) for origin in set(x[1] for x in annotation))
+    if len(annotation) == 0:
+        max_origin_len = 0
+    else:
+        max_origin_len = max(len(origin) for origin in set(x[1] for x in annotation))
     for (revno_str, author, date_str, line_rev_id, text ) in annotation:
         if verbose:
             anno = '%5s %-*s %8s ' % (revno_str, max_origin_len, author, date_str)
@@ -78,6 +81,6 @@
             author = rev.committer
             try:
                 author = extract_email_address(author)
-            except BzrError:
+            except NoEmailInUsername:
                 pass        # use the whole name
         yield (revno_str, author, date_str, origin, text)

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2006-08-09 15:57:23 +0000
+++ bzrlib/atomicfile.py	2006-09-13 02:51:49 +0000
@@ -15,12 +15,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-import codecs
-import errno
 import os
+import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import stat
 import socket
-import sys
 import warnings
 
 from bzrlib import (
@@ -28,11 +29,11 @@
     osutils,
     symbol_versioning,
     )
-from bzrlib.osutils import rename
+""")
 
 # not forksafe - but we dont fork.
 _pid = os.getpid()
-_hostname = socket.gethostname()
+_hostname = None
 
 
 class AtomicFile(object):
@@ -47,10 +48,15 @@
     __slots__ = ['tmpfilename', 'realfilename', '_fd']
 
     def __init__(self, filename, mode='wb', new_mode=None):
+        global _hostname
+
         self._fd = None
         assert mode in ('wb', 'wt'), \
             "invalid AtomicFile mode %r" % mode
 
+        if _hostname is None:
+            _hostname = socket.gethostname()
+
         self.tmpfilename = '%s.%d.%s.tmp' % (filename, _pid, _hostname)
 
         self.realfilename = filename
@@ -102,7 +108,7 @@
     def commit(self):
         """Close the file and move to final name."""
         self._close_tmpfile('commit')
-        rename(self.tmpfilename, self.realfilename)
+        osutils.rename(self.tmpfilename, self.realfilename)
 
     def abort(self):
         """Discard temporary file without committing changes."""

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2006-08-18 15:21:29 +0000
+++ bzrlib/benchmarks/__init__.py	2006-10-10 06:28:41 +0000
@@ -127,36 +127,22 @@
         creator = HeavilyMergedTreeCreator(self, link_bzr=hardlink)
         return creator.create(root=directory_name)
 
-    def create_with_commits(self, num_files, num_commits, directory_name='.'):
+    def create_with_commits(self, num_files, num_commits, directory_name='.',
+                            hardlink=False):
         """Create a tree with many files and many commits. Every commit changes
         exactly one file.
         
         :param num_files: number of files to be created
         :param num_commits: number of commits in the newly created tree
         """
-        files = ["%s/%s" % (directory_name, i) for i in range(num_files)]
-        for fn in files:
-            f = open(fn, "wb")
-            try:
-                f.write("some content\n")
-            finally:
-                f.close()
-        tree = bzrdir.BzrDir.create_standalone_workingtree(directory_name)
-        tree.add([str(i) for i in range(num_files)])
-        tree.lock_write()
-        try:
-            tree.commit('initial commit')
-            for i in range(num_commits):
-                fn = files[i % len(files)]
-                content = range(i) + [i, i, i, ""]
-                f = open(fn, "wb")
-                try:
-                    f.write("\n".join([str(i) for i in content]))
-                finally:
-                    f.close()
-                tree.commit("changing file %s" % fn)
-        finally:
-            tree.unlock()
+        from bzrlib.benchmarks.tree_creator.many_commit import (
+            ManyCommitTreeCreator,
+            )
+        creator = ManyCommitTreeCreator(self, link_bzr=hardlink,
+                                        num_files=num_files,
+                                        num_commits=num_commits)
+        tree = creator.create(root=directory_name)
+        files = ["%s/%s" % (directory_name, fn) for fn in creator.files]
         return tree, files
 
     def commit_some_revisions(self, tree, files, num_commits,
@@ -187,13 +173,16 @@
     testmod_names = [ \
                    'bzrlib.benchmarks.bench_add',
                    'bzrlib.benchmarks.bench_bench',
+                   'bzrlib.benchmarks.bench_bundle',
                    'bzrlib.benchmarks.bench_cache_utf8',
                    'bzrlib.benchmarks.bench_checkout',
                    'bzrlib.benchmarks.bench_commit',
+                   'bzrlib.benchmarks.bench_info',
                    'bzrlib.benchmarks.bench_inventory',
                    'bzrlib.benchmarks.bench_log',
                    'bzrlib.benchmarks.bench_osutils',
                    'bzrlib.benchmarks.bench_rocks',
+                   'bzrlib.benchmarks.bench_startup',
                    'bzrlib.benchmarks.bench_status',
                    'bzrlib.benchmarks.bench_transform',
                    'bzrlib.benchmarks.bench_workingtree',

=== added file 'bzrlib/benchmarks/bench_bundle.py'
--- bzrlib/benchmarks/bench_bundle.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_bundle.py	2006-10-04 10:17:53 +0000
@@ -0,0 +1,305 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzr bundle performance."""
+
+from cStringIO import StringIO
+import os
+import shutil
+
+from bzrlib import bzrdir
+from bzrlib.add import smart_add
+from bzrlib.benchmarks import Benchmark
+from bzrlib.branch import Branch
+from bzrlib.bundle import read_bundle
+from bzrlib.bundle.apply_bundle import install_bundle
+from bzrlib.bundle.serializer import write_bundle
+from bzrlib.revision import NULL_REVISION
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.workingtree import WorkingTree
+
+
+class BundleBenchmark(Benchmark):
+    """Benchmarks for bzr bundle performance and bzr merge with a bundle."""
+   
+    def test_create_bundle_known_kernel_like_tree(self):
+        """Create a bundle for a kernel sized tree with no ignored, unknowns,
+        or added and one commit.
+        """ 
+        self.make_kernel_like_committed_tree()
+        self.time(self.run_bzr, 'bundle', '--revision', '..-1')
+
+    def test_create_bundle_many_commit_tree (self):
+        """Create a bundle for a tree with many commits but no changes.""" 
+        self.make_many_commit_tree()
+        self.time(self.run_bzr, 'bundle', '--revision', '..-1')
+
+    def test_create_bundle_heavily_merged_tree(self):
+        """Create a bundle for a heavily merged tree.""" 
+        self.make_heavily_merged_tree()
+        self.time(self.run_bzr, 'bundle', '--revision', '..-1')
+        
+    def test_apply_bundle_known_kernel_like_tree(self):
+        """Create a bundle for a kernel sized tree with no ignored, unknowns,
+        or added and one commit.
+        """ 
+        tree = self.make_kernel_like_committed_tree('tree')
+
+        f = open('bundle', 'wb')
+        try:
+            write_bundle(tree.branch.repository, tree.last_revision(),
+                         NULL_REVISION, f)
+        finally:
+            f.close()
+
+        tree2 = self.make_branch_and_tree('branch_a')
+        os.chdir('branch_a')
+        self.time(self.run_bzr, 'merge', '../bundle')
+ 
+class BundleLibraryLevelWriteBenchmark(Benchmark):
+    """ Benchmarks for the write_bundle library function. """
+
+    def _time_read_write(self):
+        print "timing"
+        branch, relpath = Branch.open_containing("a")
+        revision_history = branch.revision_history()
+        bundle_text = StringIO()
+        print "starting write bundle"
+        self.time(write_bundle, branch.repository, revision_history[-1],
+                  NULL_REVISION, bundle_text)
+        print "stopped writing bundle"
+        bundle_text.seek(0)
+        target_tree = self.make_branch_and_tree('b')
+        print "starting reading bundle"
+        bundle = self.time(read_bundle, bundle_text)
+        print "starting installing bundle"
+        self.time(install_bundle, target_tree.branch.repository, bundle)
+
+    def test_few_files_small_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(5, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1, 1)
+        self._time_read_write()
+
+    def test_few_files_small_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(5, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 100, 1)
+        self._time_read_write()
+
+    def test_few_files_small_tree_1000_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(5, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1000, 1)
+        self._time_read_write()
+
+    def test_few_files_moderate_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1, 1)
+        self._time_read_write()
+
+    def test_few_files_moderate_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 100, 1)
+        self._time_read_write()
+
+    def test_few_files_moderate_tree_1000_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1000, 1)
+        self._time_read_write()
+
+    def test_some_files_moderate_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 1, 1)
+        self._time_read_write()
+
+    def test_some_files_moderate_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 100, 1)
+        self._time_read_write()
+
+    def test_some_files_moderate_tree_1000_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 1000, 1)
+        self._time_read_write()
+
+    def test_few_files_big_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1, 1)
+        self._time_read_write()
+
+    def test_few_files_big_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 100, 1)
+        self._time_read_write()
+
+    def test_few_files_big_tree_1000_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1000, 1)
+        self._time_read_write()
+
+    def test_some_files_big_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 1, 1)
+        self._time_read_write()
+
+    def test_some_files_big_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 100, 1)
+        self._time_read_write()
+
+    def test_some_files_big_tree_1000_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 1000, 1)
+        self._time_read_write()
+
+    def test_many_files_big_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:1000], 1, 1)
+        self._time_read_write()
+
+    def test_many_files_big_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:1000], 100, 1)
+        self._time_read_write()
+
+    def test_many_files_big_tree_1000_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:1000], 1000, 1)
+        self._time_read_write()
+
+class BundleLibraryLevelInstallBenchmark(Benchmark):
+    """ Benchmarks for the install_bundle library function. """
+
+    def _time_read_write(self):
+        branch, relpath = Branch.open_containing("a")
+        revision_history = branch.revision_history()
+        bundle_text = StringIO()
+        write_bundle(branch.repository, revision_history[-1],
+                     NULL_REVISION, bundle_text)
+        bundle_text.seek(0)
+        target_tree = self.make_branch_and_tree('b')
+        bundle = self.time(read_bundle, bundle_text)
+        self.time(install_bundle, target_tree.branch.repository, bundle)
+
+    def test_few_files_small_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(5, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1, 1)
+        self._time_read_write()
+
+    def test_few_files_small_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(5, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 100, 1)
+        self._time_read_write()
+
+    def test_few_files_moderate_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1, 1)
+        self._time_read_write()
+
+    def test_few_files_moderate_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 100, 1)
+        self._time_read_write()
+
+    def test_some_files_moderate_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 1, 1)
+        self._time_read_write()
+
+    def test_some_files_moderate_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(100, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 100, 1)
+        self._time_read_write()
+
+    def test_few_files_big_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 1, 1)
+        self._time_read_write()
+
+    def test_few_files_big_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:5], 100, 1)
+        self._time_read_write()
+
+    def test_some_files_big_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 1, 1)
+        self._time_read_write()
+
+    def test_some_files_big_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:100], 100, 1)
+        self._time_read_write()
+
+    def test_many_files_big_tree_1_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:1000], 1, 1)
+        self._time_read_write()
+
+    def test_many_files_big_tree_100_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(1000, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:1000], 100, 1)
+        self._time_read_write()
+
+
+if __name__ == '__main__':
+    # USE the following if you want to regenerate the above test functions 
+    for treesize, treesize_h in [(5, "small"), (100, "moderate"),
+                                 (1000, "big")]:
+        for bundlefiles, bundlefiles_h in [(5, "few"), (100, "some"),
+                                           (1000, "many")]:
+            if bundlefiles > treesize:
+                continue
+            for num_revisions in [1, 100, 1000]:
+                code = """\
+    def test_%s_files_%s_tree_%s_revision(self):
+        os.mkdir("a")
+        tree, files = self.create_with_commits(%s, 1, directory_name="a")
+        self.commit_some_revisions(tree, files[:%s], %s, 1)
+        self._time_read_write()
+""" % (bundlefiles_h, treesize_h, num_revisions,
+       treesize, bundlefiles, num_revisions)
+                print code
+
+
+

=== added file 'bzrlib/benchmarks/bench_info.py'
--- bzrlib/benchmarks/bench_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_info.py	2006-08-30 19:44:06 +0000
@@ -0,0 +1,36 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzr info performance."""
+
+
+from bzrlib.benchmarks import Benchmark
+
+
+class InfoBenchmark(Benchmark):
+    """This is a stub. Use this benchmark with a network transport.
+    Currently "bzr info sftp://..." takes > 4 min"""
+
+    def test_no_ignored_unknown_kernel_like_tree(self):
+        """Info in a kernel sized tree with no ignored or unknowns. """
+        self.make_kernel_like_added_tree()
+        self.time(self.run_bzr, 'info')
+
+    def test_no_changes_known_kernel_like_tree(self):
+        """Info in a kernel sized tree with no ignored, unknowns, or added.""" 
+        self.make_kernel_like_committed_tree()
+        self.time(self.run_bzr, 'info')
+
+     

=== modified file 'bzrlib/benchmarks/bench_rocks.py'
--- bzrlib/benchmarks/bench_rocks.py	2006-06-11 21:26:20 +0000
+++ bzrlib/benchmarks/bench_rocks.py	2006-10-09 23:44:19 +0000
@@ -13,7 +13,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Benchmarks of bzr commit."""
+"""Benchmarks of bzr rocks."""
 
 
 from bzrlib.benchmarks import Benchmark

=== modified file 'bzrlib/benchmarks/bench_sftp.py'
--- bzrlib/benchmarks/bench_sftp.py	2006-08-18 16:48:53 +0000
+++ bzrlib/benchmarks/bench_sftp.py	2006-09-22 03:29:29 +0000
@@ -21,7 +21,7 @@
     bzrdir,
     )
 from bzrlib.benchmarks import Benchmark
-from bzrlib.tests import test_sftp_transport
+from bzrlib.tests import test_sftp_transport, TestSkipped
 
 try:
     import paramiko

=== added file 'bzrlib/benchmarks/bench_startup.py'
--- bzrlib/benchmarks/bench_startup.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_startup.py	2006-10-09 23:44:19 +0000
@@ -0,0 +1,89 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Benchmarks of bzr startup time, for some simple operations."""
+
+
+from bzrlib.benchmarks import Benchmark
+
+
+class StartupBenchmark(Benchmark):
+
+    def make_simple_tree(self):
+        """A small, simple tree. No caching needed"""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+        tree.add(['a', 'b', 'b/c'])
+        return tree
+
+    def make_simple_committed_tree(self):
+        tree = self.make_simple_tree()
+        tree.commit('simple commit')
+        return tree
+
+    def test___version(self):
+        """Test the startup overhead of plain bzr --version"""
+        self.time(self.run_bzr_subprocess, '--version')
+
+    def test_branch(self):
+        """Test the time to branch this into other"""
+        tree = self.make_simple_committed_tree()
+        self.time(self.run_bzr_subprocess, 'branch', '.', 'other')
+
+    def test_commit(self):
+        """Test execution of simple commit"""
+        tree = self.make_simple_tree()
+        self.time(self.run_bzr_subprocess, 'commit', '-m', 'init simple tree')
+
+    def test_diff(self):
+        """Test simple diff time"""
+        tree = self.make_simple_committed_tree()
+        self.time(self.run_bzr_subprocess, 'diff')
+
+    def test_help(self):
+        """Test the startup overhead of plain bzr help"""
+        self.time(self.run_bzr_subprocess, 'help')
+
+    def test_help_commands(self):
+        """startup time for bzr help commands, which has to load more"""
+        self.time(self.run_bzr_subprocess, 'help', 'commands')
+
+    def test_log(self):
+        """Test simple log time"""
+        tree = self.make_simple_committed_tree()
+        self.time(self.run_bzr_subprocess, 'log')
+
+    def test_missing(self):
+        """Test simple missing time"""
+        tree = self.make_simple_committed_tree()
+        other = tree.bzrdir.sprout('other')
+        self.time(self.run_bzr_subprocess, 'missing', working_dir='other')
+
+    def test_pull(self):
+        """Test simple pull time"""
+        tree = self.make_simple_committed_tree()
+        other = tree.bzrdir.sprout('other')
+        # There should be nothing to pull, and this should be determined
+        # quickly
+        self.time(self.run_bzr_subprocess, 'pull', working_dir='other')
+
+    def test_rocks(self):
+        """Test the startup overhead by running a do-nothing command"""
+        self.time(self.run_bzr_subprocess, 'rocks')
+
+    def test_status(self):
+        """Test simple status time"""
+        tree = self.make_simple_committed_tree()
+        self.time(self.run_bzr_subprocess, 'status')

=== modified file 'bzrlib/benchmarks/bench_status.py'
--- bzrlib/benchmarks/bench_status.py	2006-08-15 03:19:49 +0000
+++ bzrlib/benchmarks/bench_status.py	2006-08-18 16:42:15 +0000
@@ -34,3 +34,10 @@
         """Status in a kernel sized tree with no ignored, unknowns, or added.""" 
         self.make_kernel_like_committed_tree(link_bzr=True)
         self.time(self.run_bzr, 'status')
+
+    def test_single_file_no_changes_known_kernel_like_tree(self):
+        """Status in a kernel sized tree with no ignored, unknowns, or added
+        of a single file."""
+        self.make_kernel_like_committed_tree()
+        #XXX depends on the filenames generated by make_kernel_like_tree
+        self.time(self.run_bzr, 'status', '7/4/0/16')

=== added file 'bzrlib/benchmarks/tree_creator/many_commit.py'
--- bzrlib/benchmarks/tree_creator/many_commit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/tree_creator/many_commit.py	2006-08-30 19:44:06 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tree creator for many commits, including file changes."""
+
+from bzrlib import (
+    bzrdir,
+    )
+
+from bzrlib.benchmarks.tree_creator import TreeCreator
+
+
+class ManyCommitTreeCreator(TreeCreator):
+    """Create an tree many files and many commits."""
+
+    def __init__(self, test, link_bzr=False, num_files=10, num_commits=10):
+        tree_name = 'many_files_many_commit_tree_%d_%d' % (
+            num_files, num_commits)
+        super(ManyCommitTreeCreator, self).__init__(test,
+            tree_name=tree_name,
+            link_bzr=link_bzr,
+            link_working=False,
+            hot_cache=True)
+        self.files = ["%s" % (i, ) for i in range(num_files)]
+        self.num_files = num_files
+        self.num_commits = num_commits
+
+    def _create_tree(self, root, in_cache=False):
+        num_files = self.num_files
+        num_commits = self.num_commits
+        files = ["%s/%s" % (root, fn) for fn in self.files]
+        for fn in files:
+            f = open(fn, "wb")
+            try:
+                f.write("some content\n")
+            finally:
+                f.close()
+        tree = bzrdir.BzrDir.create_standalone_workingtree(root)
+        tree.add(self.files)
+        tree.lock_write()
+        try:
+            tree.commit('initial commit')
+            for i in range(num_commits):
+                fn = files[i % len(files)]
+                content = range(i) + [i, i, i, ""]
+                f = open(fn, "wb")
+                try:
+                    f.write("\n".join([str(i) for i in content]))
+                finally:
+                    f.close()
+                tree.commit("changing file %s" % fn)
+        finally:
+            tree.unlock()
+        return tree
+

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-09-05 22:33:10 +0000
+++ bzrlib/branch.py	2006-10-11 00:23:23 +0000
@@ -15,8 +15,11 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+from cStringIO import StringIO
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from copy import deepcopy
-from cStringIO import StringIO
 from unittest import TestSuite
 from warnings import warn
 
@@ -26,24 +29,26 @@
         cache_utf8,
         errors,
         lockdir,
+        lockable_files,
         osutils,
-        revision,
+        revision as _mod_revision,
         transport,
         tree,
         ui,
         urlutils,
         )
-from bzrlib.config import TreeConfig
+from bzrlib.config import BranchConfig, TreeConfig
+from bzrlib.lockable_files import LockableFiles, TransportLock
+""")
+
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-import bzrlib.errors as errors
-from bzrlib.errors import (BzrError, BzrCheckError, DivergedBranches, 
-                           HistoryMissing, InvalidRevisionId, 
-                           InvalidRevisionNumber, LockError, NoSuchFile, 
+from bzrlib.errors import (BzrError, BzrCheckError, DivergedBranches,
+                           HistoryMissing, InvalidRevisionId,
+                           InvalidRevisionNumber, LockError, NoSuchFile,
                            NoSuchRevision, NoWorkingTree, NotVersionedError,
-                           NotBranchError, UninitializableFormat, 
-                           UnlistableStore, UnlistableBranch, 
+                           NotBranchError, UninitializableFormat,
+                           UnlistableStore, UnlistableBranch,
                            )
-from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.symbol_versioning import (deprecated_function,
                                       deprecated_method,
                                       DEPRECATED_PARAMETER,
@@ -138,16 +143,17 @@
         """
         return bzrdir.BzrDir.create_standalone_workingtree(base).branch
 
+    @deprecated_function(zero_eight)
     def setup_caching(self, cache_root):
         """Subclasses that care about caching should override this, and set
         up cached stores located under cache_root.
+        
+        NOTE: This is unused.
         """
-        # seems to be unused, 2006-01-13 mbp
-        warn('%s is deprecated' % self.setup_caching)
-        self.cache_root = cache_root
+        pass
 
     def get_config(self):
-        return bzrlib.config.BranchConfig(self)
+        return BranchConfig(self)
 
     def _get_nick(self):
         return self.get_config().get_nickname()
@@ -221,7 +227,7 @@
                     last_revision = from_history[-1]
                 else:
                     # no history in the source branch
-                    last_revision = revision.NULL_REVISION
+                    last_revision = _mod_revision.NULL_REVISION
             return self.repository.fetch(from_branch.repository,
                                          revision_id=last_revision,
                                          pb=nested_pb)
@@ -293,7 +299,7 @@
 
     def revision_history(self):
         """Return sequence of revision hashes on to this branch."""
-        raise NotImplementedError('revision_history is abstract')
+        raise NotImplementedError(self.revision_history)
 
     def revno(self):
         """Return current revision number for this branch.
@@ -580,6 +586,18 @@
             mainline_parent_id = revision_id
         return BranchCheckResult(self)
 
+    def _get_checkout_format(self):
+        """Return the most suitable metadir for a checkout of this branch.
+        Weaves are used if this branch's repostory uses weaves.
+        """
+        if isinstance(self.bzrdir, bzrdir.BzrDirPreSplitOut):
+            from bzrlib import repository
+            format = bzrdir.BzrDirMetaFormat1()
+            format.repository_format = repository.RepositoryFormat7()
+        else:
+            format = self.repository.bzrdir.cloning_metadir()
+        return format
+
     def create_checkout(self, to_location, revision_id=None, 
                         lightweight=False):
         """Create a checkout of a branch.
@@ -590,23 +608,23 @@
         produce a bound branch (heavyweight checkout)
         :return: The tree of the created checkout
         """
+        t = transport.get_transport(to_location)
+        try:
+            t.mkdir('.')
+        except errors.FileExists:
+            pass
         if lightweight:
-            t = transport.get_transport(to_location)
-            try:
-                t.mkdir('.')
-            except errors.FileExists:
-                pass
             checkout = bzrdir.BzrDirMetaFormat1().initialize_on_transport(t)
             BranchReferenceFormat().initialize(checkout, self)
         else:
+            format = self._get_checkout_format()
             checkout_branch = bzrdir.BzrDir.create_branch_convenience(
-                to_location, force_new_tree=False)
+                to_location, force_new_tree=False, format=format)
             checkout = checkout_branch.bzrdir
             checkout_branch.bind(self)
-            if revision_id is not None:
-                rh = checkout_branch.revision_history()
-                new_rh = rh[:rh.index(revision_id) + 1]
-                checkout_branch.set_revision_history(new_rh)
+            # pull up to the specified revision_id to set the initial 
+            # branch tip correctly, and seed it with history.
+            checkout_branch.pull(self, stop_revision=revision_id)
         return checkout.create_workingtree(revision_id)
 
 
@@ -716,8 +734,8 @@
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        control_files = LockableFiles(branch_transport, 'branch-lock',
-                                      TransportLock)
+        control_files = lockable_files.LockableFiles(branch_transport,
+                             'branch-lock', lockable_files.TransportLock)
         control_files.create_lock()
         control_files.lock_write()
         try:
@@ -777,7 +795,8 @@
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        control_files = LockableFiles(branch_transport, 'lock', lockdir.LockDir)
+        control_files = lockable_files.LockableFiles(branch_transport, 'lock',
+                                                     lockdir.LockDir)
         control_files.create_lock()
         control_files.lock_write()
         control_files.put_utf8('format', self.get_format_string())
@@ -802,7 +821,8 @@
             format = BranchFormat.find_format(a_bzrdir)
             assert format.__class__ == self.__class__
         transport = a_bzrdir.get_branch_transport(None)
-        control_files = LockableFiles(transport, 'lock', lockdir.LockDir)
+        control_files = lockable_files.LockableFiles(transport, 'lock',
+                                                     lockdir.LockDir)
         return BzrBranch5(_format=self,
                           _control_files=control_files,
                           a_bzrdir=a_bzrdir,
@@ -958,21 +978,6 @@
 
     __repr__ = __str__
 
-    def __del__(self):
-        # TODO: It might be best to do this somewhere else,
-        # but it is nice for a Branch object to automatically
-        # cache it's information.
-        # Alternatively, we could have the Transport objects cache requests
-        # See the earlier discussion about how major objects (like Branch)
-        # should never expect their __del__ function to run.
-        # XXX: cache_root seems to be unused, 2006-01-13 mbp
-        if hasattr(self, 'cache_root') and self.cache_root is not None:
-            try:
-                osutils.rmtree(self.cache_root)
-            except:
-                pass
-            self.cache_root = None
-
     def _get_base(self):
         return self._base
 
@@ -1131,7 +1136,7 @@
         # make a new revision history from the graph
         current_rev_id = revision_id
         new_history = []
-        while current_rev_id not in (None, revision.NULL_REVISION):
+        while current_rev_id not in (None, _mod_revision.NULL_REVISION):
             new_history.append(current_rev_id)
             current_rev_id_parents = stop_graph[current_rev_id]
             try:
@@ -1187,7 +1192,7 @@
         try:
             old_count = len(self.revision_history())
             try:
-                self.update_revisions(source,stop_revision)
+                self.update_revisions(source, stop_revision)
             except DivergedBranches:
                 if not overwrite:
                     raise
@@ -1328,8 +1333,13 @@
 
     @needs_write_lock
     def bind(self, other):
-        """Bind the local branch the other branch.
+        """Bind this branch to the branch other.
 
+        This does not push or pull data between the branches, though it does
+        check for divergence to raise an error when the branches are not
+        either the same, or one a prefix of the other. That behaviour may not
+        be useful, so that check may be removed in future.
+        
         :param other: The branch to bind to
         :type other: Branch
         """
@@ -1340,35 +1350,27 @@
         #       but binding itself may not be.
         #       Since we *have* to check at commit time, we don't
         #       *need* to check here
-        self.pull(other)
-
-        # we are now equal to or a suffix of other.
-
-        # Since we have 'pulled' from the remote location,
-        # now we should try to pull in the opposite direction
-        # in case the local tree has more revisions than the
-        # remote one.
-        # There may be a different check you could do here
-        # rather than actually trying to install revisions remotely.
-        # TODO: capture an exception which indicates the remote branch
-        #       is not writable. 
-        #       If it is up-to-date, this probably should not be a failure
-        
-        # lock other for write so the revision-history syncing cannot race
-        other.lock_write()
-        try:
-            other.pull(self)
-            # if this does not error, other now has the same last rev we do
-            # it can only error if the pull from other was concurrent with
-            # a commit to other from someone else.
-
-            # until we ditch revision-history, we need to sync them up:
-            self.set_revision_history(other.revision_history())
-            # now other and self are up to date with each other and have the
-            # same revision-history.
-        finally:
-            other.unlock()
-
+
+        # we want to raise diverged if:
+        # last_rev is not in the other_last_rev history, AND
+        # other_last_rev is not in our history, and do it without pulling
+        # history around
+        last_rev = self.last_revision()
+        if last_rev is not None:
+            other.lock_read()
+            try:
+                other_last_rev = other.last_revision()
+                if other_last_rev is not None:
+                    # neither branch is new, we have to do some work to
+                    # ascertain diversion.
+                    remote_graph = other.repository.get_revision_graph(
+                        other_last_rev)
+                    local_graph = self.repository.get_revision_graph(last_rev)
+                    if (last_rev not in remote_graph and
+                        other_last_rev not in local_graph):
+                        raise errors.DivergedBranches(self, other)
+            finally:
+                other.unlock()
         self.set_bound_location(other.base)
 
     @needs_write_lock

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-08-31 18:18:09 +0000
+++ bzrlib/builtins.py	2006-10-15 16:22:42 +0000
@@ -16,11 +16,12 @@
 
 """builtin bzr commands"""
 
+import os
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import codecs
 import errno
-import os
-import os.path
 import sys
 
 import bzrlib
@@ -32,37 +33,34 @@
     errors,
     ignores,
     log,
+    merge as _mod_merge,
     osutils,
     repository,
     transport,
+    tree as _mod_tree,
     ui,
     urlutils,
     )
-from bzrlib.branch import Branch, BranchReferenceFormat
-from bzrlib.bundle import read_bundle_from_url
+from bzrlib.branch import Branch
 from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.conflicts import ConflictList
+from bzrlib.revision import common_ancestor
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.workingtree import WorkingTree
+""")
+
 from bzrlib.commands import Command, display_command
-from bzrlib.errors import (BzrError, BzrCheckError, BzrCommandError, 
-                           NotBranchError, DivergedBranches, NotConflicted,
-                           NoSuchFile, NoWorkingTree, FileInWrongBranch,
-                           NotVersionedError, NotABundle)
-from bzrlib.merge import Merge3Merger
 from bzrlib.option import Option
 from bzrlib.progress import DummyProgress, ProgressPhase
-from bzrlib.revision import common_ancestor
-from bzrlib.revisionspec import RevisionSpec
 from bzrlib.trace import mutter, note, log_error, warning, is_quiet, info
-from bzrlib.transport.local import LocalTransport
-from bzrlib.workingtree import WorkingTree
 
 
 def tree_files(file_list, default_branch=u'.'):
     try:
         return internal_tree_files(file_list, default_branch)
-    except FileInWrongBranch, e:
-        raise BzrCommandError("%s is not in the same branch as %s" %
-                             (e.path, file_list[0]))
+    except errors.FileInWrongBranch, e:
+        raise errors.BzrCommandError("%s is not in the same branch as %s" %
+                                     (e.path, file_list[0]))
 
 
 # XXX: Bad function name; should possibly also be a class method of
@@ -89,7 +87,7 @@
         try:
             new_list.append(tree.relpath(filename))
         except errors.PathNotChild:
-            raise FileInWrongBranch(tree.branch, filename)
+            raise errors.FileInWrongBranch(tree.branch, filename)
     return tree, new_list
 
 
@@ -107,9 +105,13 @@
         format = bzrdir.BzrDirMetaFormat1()
         format.repository_format = repository.RepositoryFormatKnit1()
         return format
+    if typestring == "experimental-knit2":
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        return format
     msg = "Unknown bzr format %s. Current formats are: default, knit,\n" \
           "metaweave and weave" % typestring
-    raise BzrCommandError(msg)
+    raise errors.BzrCommandError(msg)
 
 
 # TODO: Make sure no commands unconditionally use the working directory as a
@@ -190,9 +192,11 @@
     def run(self, revision_id=None, revision=None):
 
         if revision_id is not None and revision is not None:
-            raise BzrCommandError('You can only supply one of revision_id or --revision')
+            raise errors.BzrCommandError('You can only supply one of'
+                                         ' revision_id or --revision')
         if revision_id is None and revision is None:
-            raise BzrCommandError('You must supply either --revision or a revision_id')
+            raise errors.BzrCommandError('You must supply either'
+                                         ' --revision or a revision_id')
         b = WorkingTree.open_containing(u'.')[0].branch
 
         # TODO: jam 20060112 should cat-revision always output utf-8?
@@ -201,7 +205,8 @@
         elif revision is not None:
             for rev in revision:
                 if rev is None:
-                    raise BzrCommandError('You cannot specify a NULL revision.')
+                    raise errors.BzrCommandError('You cannot specify a NULL'
+                                                 ' revision.')
                 revno, rev_id = rev.in_history(b)
                 self.outf.write(b.repository.get_revision_xml(rev_id).decode('utf-8'))
     
@@ -237,7 +242,7 @@
             for rev in revision_info_list:
                 revs.append(RevisionSpec.from_string(rev))
         if len(revs) == 0:
-            raise BzrCommandError('You must supply a revision identifier')
+            raise errors.BzrCommandError('You must supply a revision identifier')
 
         b = WorkingTree.open_containing(u'.')[0].branch
 
@@ -294,7 +299,7 @@
                 base_tree, base_path = WorkingTree.open_containing(
                                             file_ids_from)
             except errors.NoWorkingTree:
-                base_branch, base_path = branch.Branch.open_containing(
+                base_branch, base_path = Branch.open_containing(
                                             file_ids_from)
                 base_tree = base_branch.basis_tree()
 
@@ -357,26 +362,49 @@
     """Show inventory of the current working copy or a revision.
 
     It is possible to limit the output to a particular entry
-    type using the --kind option.  For example; --kind file.
+    type using the --kind option.  For example: --kind file.
+
+    It is also possible to restrict the list of files to a specific
+    set. For example: bzr inventory --show-ids this/file
     """
 
     takes_options = ['revision', 'show-ids', 'kind']
-    
+    takes_args = ['file*']
+
     @display_command
-    def run(self, revision=None, show_ids=False, kind=None):
+    def run(self, revision=None, show_ids=False, kind=None, file_list=None):
         if kind and kind not in ['file', 'directory', 'symlink']:
-            raise BzrCommandError('invalid kind specified')
-        tree = WorkingTree.open_containing(u'.')[0]
-        if revision is None:
-            inv = tree.read_working_inventory()
-        else:
+            raise errors.BzrCommandError('invalid kind specified')
+
+        work_tree, file_list = tree_files(file_list)
+
+        if revision is not None:
             if len(revision) > 1:
-                raise BzrCommandError('bzr inventory --revision takes'
-                    ' exactly one revision identifier')
-            inv = tree.branch.repository.get_revision_inventory(
-                revision[0].in_history(tree.branch).rev_id)
-
-        for path, entry in inv.entries():
+                raise errors.BzrCommandError('bzr inventory --revision takes'
+                                             ' exactly one revision identifier')
+            revision_id = revision[0].in_history(work_tree.branch).rev_id
+            tree = work_tree.branch.repository.revision_tree(revision_id)
+                        
+            # We include work_tree as well as 'tree' here
+            # So that doing '-r 10 path/foo' will lookup whatever file
+            # exists now at 'path/foo' even if it has been renamed, as
+            # well as whatever files existed in revision 10 at path/foo
+            trees = [tree, work_tree]
+        else:
+            tree = work_tree
+            trees = [tree]
+
+        if file_list is not None:
+            file_ids = _mod_tree.find_ids_across_trees(file_list, trees,
+                                                      require_versioned=True)
+            # find_ids_across_trees may include some paths that don't
+            # exist in 'tree'.
+            entries = sorted((tree.id2path(file_id), tree.inventory[file_id])
+                             for file_id in file_ids if file_id in tree)
+        else:
+            entries = tree.inventory.entries()
+
+        for path, entry in entries:
             if kind and kind != entry.kind:
                 continue
             if show_ids:
@@ -409,7 +437,7 @@
             names_list = []
 
         if len(names_list) < 2:
-            raise BzrCommandError("missing file argument")
+            raise errors.BzrCommandError("missing file argument")
         tree, rel_names = tree_files(names_list)
         
         if os.path.isdir(names_list[-1]):
@@ -418,8 +446,8 @@
                 self.outf.write("%s => %s\n" % pair)
         else:
             if len(names_list) != 2:
-                raise BzrCommandError('to mv multiple files the destination '
-                                      'must be a versioned directory')
+                raise errors.BzrCommandError('to mv multiple files the destination '
+                                             'must be a versioned directory')
             tree.rename_one(rel_names[0], rel_names[1])
             self.outf.write("%s => %s\n" % (rel_names[0], rel_names[1]))
             
@@ -453,7 +481,7 @@
         try:
             tree_to = WorkingTree.open_containing(u'.')[0]
             branch_to = tree_to.branch
-        except NoWorkingTree:
+        except errors.NoWorkingTree:
             tree_to = None
             branch_to = Branch.open_containing(u'.')[0]
 
@@ -461,13 +489,14 @@
         if location is not None:
             try:
                 reader = bundle.read_bundle_from_url(location)
-            except NotABundle:
+            except errors.NotABundle:
                 pass # Continue on considering this url a Branch
 
         stored_loc = branch_to.get_parent()
         if location is None:
             if stored_loc is None:
-                raise BzrCommandError("No pull location known or specified.")
+                raise errors.BzrCommandError("No pull location known or"
+                                             " specified.")
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
@@ -491,7 +520,7 @@
         elif len(revision) == 1:
             rev_id = revision[0].in_history(branch_from).rev_id
         else:
-            raise BzrCommandError('bzr pull --revision takes one value.')
+            raise errors.BzrCommandError('bzr pull --revision takes one value.')
 
         old_rh = branch_to.revision_history()
         if tree_to is not None:
@@ -551,7 +580,7 @@
         stored_loc = br_from.get_push_location()
         if location is None:
             if stored_loc is None:
-                raise BzrCommandError("No push location known or specified.")
+                raise errors.BzrCommandError("No push location known or specified.")
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
@@ -565,7 +594,7 @@
         try:
             dir_to = bzrdir.BzrDir.open(location_url)
             br_to = dir_to.open_branch()
-        except NotBranchError:
+        except errors.NotBranchError:
             # create a branch.
             to_transport = to_transport.clone('..')
             if not create_prefix:
@@ -573,9 +602,9 @@
                     relurl = to_transport.relpath(location_url)
                     mutter('creating directory %s => %s', location_url, relurl)
                     to_transport.mkdir(relurl)
-                except NoSuchFile:
-                    raise BzrCommandError("Parent directory of %s "
-                                          "does not exist." % location)
+                except errors.NoSuchFile:
+                    raise errors.BzrCommandError("Parent directory of %s "
+                                                 "does not exist." % location)
             else:
                 current = to_transport.base
                 needed = [(to_transport, to_transport.relpath(location_url))]
@@ -584,13 +613,13 @@
                         to_transport, relpath = needed[-1]
                         to_transport.mkdir(relpath)
                         needed.pop()
-                    except NoSuchFile:
+                    except errors.NoSuchFile:
                         new_transport = to_transport.clone('..')
                         needed.append((new_transport,
                                        new_transport.relpath(to_transport.base)))
                         if new_transport.base == to_transport.base:
-                            raise BzrCommandError("Could not create "
-                                                  "path prefix.")
+                            raise errors.BzrCommandError("Could not create "
+                                                         "path prefix.")
             dir_to = br_from.bzrdir.clone(location_url,
                 revision_id=br_from.last_revision())
             br_to = dir_to.open_branch()
@@ -611,13 +640,13 @@
                     warning('This transport does not update the working '
                             'tree of: %s' % (br_to.base,))
                     count = br_to.pull(br_from, overwrite)
-                except NoWorkingTree:
+                except errors.NoWorkingTree:
                     count = br_to.pull(br_from, overwrite)
                 else:
                     count = tree_to.pull(br_from, overwrite)
-            except DivergedBranches:
-                raise BzrCommandError("These branches have diverged."
-                                      "  Try a merge then push with overwrite.")
+            except errors.DivergedBranches:
+                raise errors.BzrCommandError('These branches have diverged.'
+                                        '  Try using "merge" and then "push".')
         note('%d revision(s) pushed.' % (count,))
 
         if verbose:
@@ -650,14 +679,14 @@
         if revision is None:
             revision = [None]
         elif len(revision) > 1:
-            raise BzrCommandError(
+            raise errors.BzrCommandError(
                 'bzr branch --revision takes exactly 1 revision value')
         try:
             br_from = Branch.open(from_location)
         except OSError, e:
             if e.errno == errno.ENOENT:
-                raise BzrCommandError('Source location "%s" does not'
-                                      ' exist.' % to_location)
+                raise errors.BzrCommandError('Source location "%s" does not'
+                                             ' exist.' % to_location)
             else:
                 raise
         br_from.lock_read()
@@ -683,11 +712,11 @@
             try:
                 to_transport.mkdir('.')
             except errors.FileExists:
-                raise BzrCommandError('Target directory "%s" already'
-                                      ' exists.' % to_location)
+                raise errors.BzrCommandError('Target directory "%s" already'
+                                             ' exists.' % to_location)
             except errors.NoSuchFile:
-                raise BzrCommandError('Parent of "%s" does not exist.' %
-                                      to_location)
+                raise errors.BzrCommandError('Parent of "%s" does not exist.'
+                                             % to_location)
             try:
                 # preserve whatever source format we have.
                 dir = br_from.bzrdir.sprout(to_transport.base,
@@ -696,11 +725,11 @@
             except errors.NoSuchRevision:
                 to_transport.delete_tree('.')
                 msg = "The branch %s has no revision %s." % (from_location, revision[0])
-                raise BzrCommandError(msg)
+                raise errors.BzrCommandError(msg)
             except errors.UnlistableBranch:
                 osutils.rmtree(to_location)
                 msg = "The branch %s cannot be used as a --basis" % (basis,)
-                raise BzrCommandError(msg)
+                raise errors.BzrCommandError(msg)
             if name:
                 branch.control_files.put_utf8('branch-name', name)
             note('Branched %d revision(s).' % branch.revno())
@@ -745,7 +774,7 @@
         if revision is None:
             revision = [None]
         elif len(revision) > 1:
-            raise BzrCommandError(
+            raise errors.BzrCommandError(
                 'bzr checkout --revision takes exactly 1 revision value')
         if branch_location is None:
             branch_location = osutils.getcwd()
@@ -760,7 +789,7 @@
         # if the source and to_location are the same, 
         # and there is no working tree,
         # then reconstitute a branch
-        if (osutils.abspath(to_location) == 
+        if (osutils.abspath(to_location) ==
             osutils.abspath(branch_location)):
             try:
                 source.bzrdir.open_workingtree()
@@ -771,11 +800,11 @@
             os.mkdir(to_location)
         except OSError, e:
             if e.errno == errno.EEXIST:
-                raise BzrCommandError('Target directory "%s" already'
-                                      ' exists.' % to_location)
+                raise errors.BzrCommandError('Target directory "%s" already'
+                                             ' exists.' % to_location)
             if e.errno == errno.ENOENT:
-                raise BzrCommandError('Parent of "%s" does not exist.' %
-                                      to_location)
+                raise errors.BzrCommandError('Parent of "%s" does not exist.'
+                                             % to_location)
             else:
                 raise
         old_format = bzrdir.BzrDirFormat.get_default_format()
@@ -796,11 +825,10 @@
 
     @display_command
     def run(self, dir=u'.'):
-        from bzrlib.tree import find_renames
         tree = WorkingTree.open_containing(dir)[0]
         old_inv = tree.basis_tree().inventory
         new_inv = tree.read_working_inventory()
-        renames = list(find_renames(old_inv, new_inv))
+        renames = list(_mod_tree.find_renames(old_inv, new_inv))
         renames.sort()
         for old_name, new_name in renames:
             self.outf.write("%s => %s\n" % (old_name, new_name))
@@ -823,7 +851,7 @@
         tree = WorkingTree.open_containing(dir)[0]
         tree.lock_write()
         try:
-            existing_pending_merges = tree.pending_merges()
+            existing_pending_merges = tree.get_parent_ids()[1:]
             last_rev = tree.last_revision()
             if last_rev == tree.branch.last_revision():
                 # may be up to date, check master too.
@@ -835,7 +863,7 @@
             conflicts = tree.update()
             revno = tree.branch.revision_id_to_revno(tree.last_revision())
             note('Updated to revision %d.' % (revno,))
-            if tree.pending_merges() != existing_pending_merges:
+            if tree.get_parent_ids()[1:] != existing_pending_merges:
                 note('Your local commits will now show as pending merges with '
                      "'bzr status', and can be committed with 'bzr commit'.")
             if conflicts != 0:
@@ -885,14 +913,14 @@
         tree, file_list = tree_files(file_list)
         if new is False:
             if file_list is None:
-                raise BzrCommandError('Specify one or more files to remove, or'
-                                      ' use --new.')
+                raise errors.BzrCommandError('Specify one or more files to'
+                                             ' remove, or use --new.')
         else:
             added = tree.changes_from(tree.basis_tree(),
                 specific_files=file_list).added
             file_list = sorted([f[0] for f in added], reverse=True)
             if len(file_list) == 0:
-                raise BzrCommandError('No matching files.')
+                raise errors.BzrCommandError('No matching files.')
         tree.remove(file_list, verbose=verbose, to_file=self.outf)
 
 
@@ -911,8 +939,8 @@
     def run(self, filename):
         tree, relpath = WorkingTree.open_containing(filename)
         i = tree.inventory.path2id(relpath)
-        if i == None:
-            raise BzrError("%r is not a versioned file" % filename)
+        if i is None:
+            raise errors.BzrError("%r is not a versioned file" % filename)
         else:
             self.outf.write(i + '\n')
 
@@ -932,8 +960,8 @@
         tree, relpath = WorkingTree.open_containing(filename)
         inv = tree.inventory
         fid = inv.path2id(relpath)
-        if fid == None:
-            raise BzrError("%r is not a versioned file" % filename)
+        if fid is None:
+            raise errors.BzrError("%r is not a versioned file" % filename)
         for fip in inv.get_idpath(fid):
             self.outf.write(fip + '\n')
 
@@ -996,7 +1024,7 @@
             last_revision = wt.last_revision()
 
         revision_ids = b.repository.get_ancestry(last_revision)
-        assert revision_ids[0] == None
+        assert revision_ids[0] is None
         revision_ids.pop(0)
         for revision_id in revision_ids:
             self.outf.write(revision_id + '\n')
@@ -1053,10 +1081,11 @@
                     
         try:
             existing_bzrdir = bzrdir.BzrDir.open(location)
-        except NotBranchError:
+        except errors.NotBranchError:
             # really a NotBzrDir error...
             bzrdir.BzrDir.create_branch_convenience(location, format=format)
         else:
+            from bzrlib.transport.local import LocalTransport
             if existing_bzrdir.has_branch():
                 if (isinstance(to_transport, LocalTransport)
                     and not existing_bzrdir.has_workingtree()):
@@ -1163,7 +1192,8 @@
             new_label = 'new/'
         else:
             if not ':' in prefix:
-                 raise BzrError("--diff-prefix expects two values separated by a colon")
+                 raise errors.BzrError("--diff-prefix expects two values"
+                                       " separated by a colon")
             old_label, new_label = prefix.split(":")
         
         try:
@@ -1171,17 +1201,17 @@
             tree2 = None
             b = None
             b2 = None
-        except FileInWrongBranch:
+        except errors.FileInWrongBranch:
             if len(file_list) != 2:
-                raise BzrCommandError("Files are in different branches")
+                raise errors.BzrCommandError("Files are in different branches")
 
             tree1, file1 = WorkingTree.open_containing(file_list[0])
             tree2, file2 = WorkingTree.open_containing(file_list[1])
             if file1 != "" or file2 != "":
                 # FIXME diff those two files. rbc 20051123
-                raise BzrCommandError("Files are in different branches")
+                raise errors.BzrCommandError("Files are in different branches")
             file_list = None
-        except NotBranchError:
+        except errors.NotBranchError:
             if (revision is not None and len(revision) == 2
                 and not revision[0].needs_branch()
                 and not revision[1].needs_branch()):
@@ -1192,7 +1222,7 @@
                 raise
         if revision is not None:
             if tree2 is not None:
-                raise BzrCommandError("Can't specify -r with two branches")
+                raise errors.BzrCommandError("Can't specify -r with two branches")
             if (len(revision) == 1) or (revision[1].spec is None):
                 return diff_cmd_helper(tree1, file_list, diff_options,
                                        revision[0], 
@@ -1202,7 +1232,8 @@
                                        revision[0], revision[1],
                                        old_label=old_label, new_label=new_label)
             else:
-                raise BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
+                raise errors.BzrCommandError('bzr diff --revision takes exactly'
+                                             ' one or two revision identifiers')
         else:
             if tree2 is not None:
                 return show_diff_trees(tree1, tree2, sys.stdout, 
@@ -1260,6 +1291,8 @@
         for file_id in inv:
             if file_id in basis_inv:
                 continue
+            if inv.is_root(file_id) and len(basis_inv) == 0:
+                continue
             path = inv.id2path(file_id)
             if not os.access(osutils.abspath(path), os.F_OK):
                 continue
@@ -1346,7 +1379,12 @@
         else:
             # local dir only
             # FIXME ? log the current subdir only RBC 20060203 
-            dir, relpath = bzrdir.BzrDir.open_containing('.')
+            if revision is not None \
+                    and len(revision) > 0 and revision[0].get_branch():
+                location = revision[0].get_branch()
+            else:
+                location = '.'
+            dir, relpath = bzrdir.BzrDir.open_containing(location)
             b = dir.open_branch()
 
         if revision is None:
@@ -1355,6 +1393,12 @@
         elif len(revision) == 1:
             rev1 = rev2 = revision[0].in_history(b).revno
         elif len(revision) == 2:
+            if revision[1].get_branch() != revision[0].get_branch():
+                # b is taken from revision[0].get_branch(), and
+                # show_log will use its revision_history. Having
+                # different branches will lead to weird behaviors.
+                raise errors.BzrCommandError(
+                    "Log doesn't accept two revisions in different branches.")
             if revision[0].spec is None:
                 # missing begin-range means first revision
                 rev1 = 1
@@ -1367,7 +1411,7 @@
             else:
                 rev2 = revision[1].in_history(b).revno
         else:
-            raise BzrCommandError('bzr log --revision takes one or two values.')
+            raise errors.BzrCommandError('bzr log --revision takes one or two values.')
 
         # By this point, the revision numbers are converted to the +ve
         # form if they were supplied in the -ve form, so we can do
@@ -1375,7 +1419,7 @@
         if rev1 > rev2:
             (rev2, rev1) = (rev1, rev2)
 
-        if (log_format == None):
+        if (log_format is None):
             default = b.get_config().log_format()
             log_format = get_log_format(long=long, short=short, line=line, 
                                         default=default)
@@ -1447,7 +1491,7 @@
             null=False):
 
         if verbose and null:
-            raise BzrCommandError('Cannot set both --verbose and --null')
+            raise errors.BzrCommandError('Cannot set both --verbose and --null')
         all = not (unknown or versioned or ignored)
 
         selection = {'I':ignored, '?':unknown, 'V':versioned}
@@ -1461,7 +1505,7 @@
             tree = tree.branch.repository.revision_tree(
                 revision[0].in_history(tree.branch).rev_id)
 
-        for fp, fc, kind, fid, entry in tree.list_files():
+        for fp, fc, kind, fid, entry in tree.list_files(include_root=False):
             if fp.startswith(relpath):
                 fp = fp[len(relpath):]
                 if non_recursive and '/' in fp:
@@ -1519,7 +1563,7 @@
                 print pattern
             return
         if name_pattern is None:
-            raise BzrCommandError("ignore requires a NAME_PATTERN")
+            raise errors.BzrCommandError("ignore requires a NAME_PATTERN")
         tree, relpath = WorkingTree.open_containing(u'.')
         ifn = tree.abspath('.bzrignore')
         if os.path.exists(ifn):
@@ -1582,7 +1626,7 @@
         try:
             revno = int(revno)
         except ValueError:
-            raise BzrCommandError("not a valid revision-number: %r" % revno)
+            raise errors.BzrCommandError("not a valid revision-number: %r" % revno)
 
         print WorkingTree.open_containing(u'.')[0].branch.get_rev_id(revno)
 
@@ -1620,13 +1664,14 @@
             rev_id = b.last_revision()
         else:
             if len(revision) != 1:
-                raise BzrError('bzr export --revision takes exactly 1 argument')
+                raise errors.BzrError('bzr export --revision takes exactly'
+                                      ' 1 argument')
             rev_id = revision[0].in_history(b).rev_id
         t = b.repository.revision_tree(rev_id)
         try:
             export(t, dest, format, root)
         except errors.NoSuchExportFormat, e:
-            raise BzrCommandError('Unsupported export format: %s' % e.format)
+            raise errors.BzrCommandError('Unsupported export format: %s' % e.format)
 
 
 class cmd_cat(Command):
@@ -1638,16 +1683,18 @@
     @display_command
     def run(self, filename, revision=None):
         if revision is not None and len(revision) != 1:
-            raise BzrCommandError("bzr cat --revision takes exactly one number")
+            raise errors.BzrCommandError("bzr cat --revision takes exactly one number")
         tree = None
         try:
             tree, relpath = WorkingTree.open_containing(filename)
             b = tree.branch
-        except NotBranchError:
+        except errors.NotBranchError:
             pass
 
         if tree is None:
             b, relpath = Branch.open_containing(filename)
+        if revision is not None and revision[0].get_branch() is not None:
+            b = Branch.open(revision[0].get_branch())
         if revision is None:
             revision_id = b.last_revision()
         else:
@@ -1735,16 +1782,16 @@
             template = make_commit_message_template(tree, selected_list)
             message = edit_commit_message(template)
             if message is None:
-                raise BzrCommandError("please specify a commit message"
-                                      " with either --message or --file")
+                raise errors.BzrCommandError("please specify a commit message"
+                                             " with either --message or --file")
         elif message and file:
-            raise BzrCommandError("please specify either --message or --file")
+            raise errors.BzrCommandError("please specify either --message or --file")
         
         if file:
             message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
 
         if message == "":
-            raise BzrCommandError("empty commit message specified")
+            raise errors.BzrCommandError("empty commit message specified")
         
         if verbose:
             reporter = ReportCommitToLog()
@@ -1758,16 +1805,16 @@
         except PointlessCommit:
             # FIXME: This should really happen before the file is read in;
             # perhaps prepare the commit; get the message; then actually commit
-            raise BzrCommandError("no changes to commit."
-                                  " use --unchanged to commit anyhow")
+            raise errors.BzrCommandError("no changes to commit."
+                                         " use --unchanged to commit anyhow")
         except ConflictsInTree:
-            raise BzrCommandError("Conflicts detected in working tree.  "
+            raise errors.BzrCommandError("Conflicts detected in working tree.  "
                 'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
         except StrictCommitFailed:
-            raise BzrCommandError("Commit refused because there are unknown "
-                                  "files in the working tree.")
+            raise errors.BzrCommandError("Commit refused because there are unknown "
+                                         "files in the working tree.")
         except errors.BoundBranchOutOfDate, e:
-            raise BzrCommandError(str(e) + "\n"
+            raise errors.BzrCommandError(str(e) + "\n"
                 'To commit to master branch, run update and then commit.\n'
                 'You can also pass --local to commit to continue working '
                 'disconnected.')
@@ -1857,7 +1904,7 @@
             # use branch if we're inside one; otherwise global config
             try:
                 c = Branch.open_containing('.')[0].get_config()
-            except NotBranchError:
+            except errors.NotBranchError:
                 c = config.GlobalConfig()
             if email:
                 self.outf.write(c.user_email() + '\n')
@@ -1868,7 +1915,7 @@
         # display a warning if an email address isn't included in the given name.
         try:
             config.extract_email_address(name)
-        except BzrError, e:
+        except errors.NoEmailInUsername, e:
             warning('"%s" does not seem to contain an email address.  '
                     'This is allowed, but not recommended.', name)
         
@@ -1936,7 +1983,7 @@
             return FakeNFSServer
         msg = "No known transport type %s. Supported types are: sftp\n" %\
             (typestring)
-        raise BzrCommandError(msg)
+        raise errors.BzrCommandError(msg)
 
     hidden = True
     takes_args = ['testspecs*']
@@ -1985,6 +2032,7 @@
                 test_suite_factory = benchmarks.test_suite
                 if verbose is None:
                     verbose = True
+                # TODO: should possibly lock the history file...
                 benchfile = open(".perf_history", "at")
             else:
                 test_suite_factory = None
@@ -2040,7 +2088,7 @@
     
     @display_command
     def run(self, branch, other):
-        from bzrlib.revision import common_ancestor, MultipleRevisionSources
+        from bzrlib.revision import MultipleRevisionSources
         
         branch1 = Branch.open_containing(branch)[0]
         branch2 = Branch.open_containing(other)[0]
@@ -2108,22 +2156,21 @@
                             ' from a working copy, instead of branch changes')]
 
     def help(self):
-        from merge import merge_type_help
         from inspect import getdoc
-        return getdoc(self) + '\n' + merge_type_help() 
+        return getdoc(self) + '\n' + _mod_merge.merge_type_help()
 
     def run(self, branch=None, revision=None, force=False, merge_type=None,
             show_base=False, reprocess=False, remember=False, 
             uncommitted=False):
         if merge_type is None:
-            merge_type = Merge3Merger
+            merge_type = _mod_merge.Merge3Merger
 
         tree = WorkingTree.open_containing(u'.')[0]
 
         if branch is not None:
             try:
                 reader = bundle.read_bundle_from_url(branch)
-            except NotABundle:
+            except errors.NotABundle:
                 pass # Continue on considering this url a Branch
             else:
                 conflicts = merge_bundle(reader, tree, not force, merge_type,
@@ -2133,7 +2180,9 @@
                 else:
                     return 1
 
-        branch = self._get_remembered_parent(tree, branch, 'Merging from')
+        if revision is None \
+                or len(revision) < 1 or revision[0].needs_branch():
+            branch = self._get_remembered_parent(tree, branch, 'Merging from')
 
         if revision is None or len(revision) < 1:
             if uncommitted:
@@ -2145,8 +2194,9 @@
             other_branch, path = Branch.open_containing(branch)
         else:
             if uncommitted:
-                raise BzrCommandError('Cannot use --uncommitted and --revision'
-                                      ' at the same time.')
+                raise errors.BzrCommandError('Cannot use --uncommitted and'
+                                             ' --revision at the same time.')
+            branch = revision[0].get_branch() or branch
             if len(revision) == 1:
                 base = [None, None]
                 other_branch, path = Branch.open_containing(branch)
@@ -2155,12 +2205,17 @@
             else:
                 assert len(revision) == 2
                 if None in revision:
-                    raise BzrCommandError(
-                        "Merge doesn't permit that revision specifier.")
-                other_branch, path = Branch.open_containing(branch)
+                    raise errors.BzrCommandError(
+                        "Merge doesn't permit empty revision specifier.")
+                base_branch, path = Branch.open_containing(branch)
+                branch1 = revision[1].get_branch() or branch
+                other_branch, path1 = Branch.open_containing(branch1)
+                if revision[0].get_branch() is not None:
+                    # then path was obtained from it, and is None.
+                    path = path1
 
-                base = [branch, revision[0].in_history(other_branch).revno]
-                other = [branch, revision[1].in_history(other_branch).revno]
+                base = [branch, revision[0].in_history(base_branch).revno]
+                other = [branch1, revision[1].in_history(other_branch).revno]
 
         if tree.branch.get_parent() is None or remember:
             tree.branch.set_parent(other_branch.base)
@@ -2172,11 +2227,12 @@
         pb = ui.ui_factory.nested_progress_bar()
         try:
             try:
-                conflict_count = merge(other, base, check_clean=(not force),
-                                       merge_type=merge_type,
-                                       reprocess=reprocess,
-                                       show_base=show_base,
-                                       pb=pb, file_list=interesting_files)
+                conflict_count = _merge_helper(
+                    other, base, check_clean=(not force),
+                    merge_type=merge_type,
+                    reprocess=reprocess,
+                    show_base=show_base,
+                    pb=pb, file_list=interesting_files)
             finally:
                 pb.finished()
             if conflict_count != 0:
@@ -2203,7 +2259,7 @@
         stored_location = tree.branch.get_parent()
         mutter("%s", stored_location)
         if stored_location is None:
-            raise BzrCommandError("No location specified or remembered")
+            raise errors.BzrCommandError("No location specified or remembered")
         display_url = urlutils.unescape_for_display(stored_location, self.outf.encoding)
         self.outf.write("%s remembered location %s\n" % (verb_string, display_url))
         return stored_location
@@ -2236,23 +2292,21 @@
                             "conflicts")]
 
     def help(self):
-        from merge import merge_type_help
         from inspect import getdoc
-        return getdoc(self) + '\n' + merge_type_help() 
+        return getdoc(self) + '\n' + _mod_merge.merge_type_help()
 
     def run(self, file_list=None, merge_type=None, show_base=False,
             reprocess=False):
-        from bzrlib.merge import merge_inner, transform_tree
         if merge_type is None:
-            merge_type = Merge3Merger
+            merge_type = _mod_merge.Merge3Merger
         tree, file_list = tree_files(file_list)
         tree.lock_write()
         try:
             parents = tree.get_parent_ids()
             if len(parents) != 2:
-                raise BzrCommandError("Sorry, remerge only works after normal"
-                                      " merges.  Not cherrypicking or"
-                                      " multi-merges.")
+                raise errors.BzrCommandError("Sorry, remerge only works after normal"
+                                             " merges.  Not cherrypicking or"
+                                             " multi-merges.")
             repository = tree.branch.repository
             base_revision = common_ancestor(parents[0],
                                             parents[1], repository)
@@ -2266,7 +2320,7 @@
                 for filename in file_list:
                     file_id = tree.path2id(filename)
                     if file_id is None:
-                        raise NotVersionedError(filename)
+                        raise errors.NotVersionedError(filename)
                     interesting_ids.add(file_id)
                     if tree.kind(file_id) != "directory":
                         continue
@@ -2274,7 +2328,7 @@
                     for name, ie in tree.inventory.iter_entries(file_id):
                         interesting_ids.add(ie.file_id)
                 new_conflicts = conflicts.select_conflicts(tree, file_list)[0]
-            transform_tree(tree, tree.basis_tree(), interesting_ids)
+            _mod_merge.transform_tree(tree, tree.basis_tree(), interesting_ids)
             tree.set_conflicts(ConflictList(new_conflicts))
             if file_list is None:
                 restore_files = list(tree.iter_conflicts())
@@ -2283,15 +2337,16 @@
             for filename in restore_files:
                 try:
                     restore(tree.abspath(filename))
-                except NotConflicted:
+                except errors.NotConflicted:
                     pass
-            conflicts = merge_inner(tree.branch, other_tree, base_tree,
-                                    this_tree=tree,
-                                    interesting_ids=interesting_ids,
-                                    other_rev_id=parents[1],
-                                    merge_type=merge_type,
-                                    show_base=show_base,
-                                    reprocess=reprocess)
+            conflicts = _mod_merge.merge_inner(
+                                      tree.branch, other_tree, base_tree,
+                                      this_tree=tree,
+                                      interesting_ids=interesting_ids,
+                                      other_rev_id=parents[1],
+                                      merge_type=merge_type,
+                                      show_base=show_base,
+                                      reprocess=reprocess)
         finally:
             tree.unlock()
         if conflicts > 0:
@@ -2300,21 +2355,34 @@
             return 0
 
 class cmd_revert(Command):
-    """Reverse all changes since the last commit.
-
-    Only versioned files are affected.  Specify filenames to revert only 
-    those files.  By default, any files that are changed will be backed up
-    first.  Backup files have a '~' appended to their name.
+    """Revert files to a previous revision.
+
+    Giving a list of files will revert only those files.  Otherwise, all files
+    will be reverted.  If the revision is not specified with '--revision', the
+    last committed revision is used.
+
+    To remove only some changes, without reverting to a prior version, use
+    merge instead.  For example, "merge . --r-2..-3" will remove the changes
+    introduced by -2, without affecting the changes introduced by -1.  Or
+    to remove certain changes on a hunk-by-hunk basis, see the Shelf plugin.
+    
+    By default, any files that have been manually changed will be backed up
+    first.  (Files changed only by merge are not backed up.)  Backup files have
+    '.~#~' appended to their name, where # is a number.
+
+    When you provide files, you can use their current pathname or the pathname
+    from the target revision.  So you can use revert to "undelete" a file by
+    name.  If you name a directory, all the contents of that directory will be
+    reverted.
     """
     takes_options = ['revision', 'no-backup']
     takes_args = ['file*']
     aliases = ['merge-revert']
 
     def run(self, revision=None, no_backup=False, file_list=None):
-        from bzrlib.commands import parse_spec
         if file_list is not None:
             if len(file_list) == 0:
-                raise BzrCommandError("No files specified")
+                raise errors.BzrCommandError("No files specified")
         else:
             file_list = []
         
@@ -2323,7 +2391,7 @@
             # FIXME should be tree.last_revision
             rev_id = tree.last_revision()
         elif len(revision) != 1:
-            raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
+            raise errors.BzrCommandError('bzr revert --revision takes exactly 1 argument')
         else:
             rev_id = revision[0].in_history(tree.branch).rev_id
         pb = ui.ui_factory.nested_progress_bar()
@@ -2415,7 +2483,7 @@
         if other_branch is None:
             other_branch = parent
             if other_branch is None:
-                raise BzrCommandError("No peer location known or specified.")
+                raise errors.BzrCommandError("No peer location known or specified.")
             print "Using last location: " + local_branch.get_parent()
         remote_branch = Branch.open(other_branch)
         if remote_branch.base == local_branch.base:
@@ -2425,7 +2493,7 @@
             remote_branch.lock_read()
             try:
                 local_extra, remote_extra = find_unmerged(local_branch, remote_branch)
-                if (log_format == None):
+                if (log_format is None):
                     default = local_branch.get_config().log_format()
                     log_format = get_log_format(long=long, short=short, 
                                                 line=line, default=default)
@@ -2479,12 +2547,12 @@
         import bzrlib.plugin
         from inspect import getdoc
         for name, plugin in bzrlib.plugin.all_plugins().items():
-            if hasattr(plugin, '__path__'):
+            if getattr(plugin, '__path__', None) is not None:
                 print plugin.__path__[0]
-            elif hasattr(plugin, '__file__'):
+            elif getattr(plugin, '__file__', None) is not None:
                 print plugin.__file__
             else:
-                print `plugin`
+                print repr(plugin)
                 
             d = getdoc(plugin)
             if d:
@@ -2550,7 +2618,7 @@
             if revision is None:
                 revision_id = branch.last_revision()
             elif len(revision) != 1:
-                raise BzrCommandError('bzr annotate --revision takes exactly 1 argument')
+                raise errors.BzrCommandError('bzr annotate --revision takes exactly 1 argument')
             else:
                 revision_id = revision[0].in_history(branch).rev_id
             file_id = tree.inventory.path2id(relpath)
@@ -2572,9 +2640,9 @@
     def run(self, revision_id_list=None, revision=None):
         import bzrlib.gpg as gpg
         if revision_id_list is not None and revision is not None:
-            raise BzrCommandError('You can only supply one of revision_id or --revision')
+            raise errors.BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id_list is None and revision is None:
-            raise BzrCommandError('You must supply either --revision or a revision_id')
+            raise errors.BzrCommandError('You must supply either --revision or a revision_id')
         b = WorkingTree.open_containing(u'.')[0].branch
         gpg_strategy = gpg.GPGStrategy(b.get_config())
         if revision_id_list is not None:
@@ -2593,12 +2661,12 @@
                 if to_revid is None:
                     to_revno = b.revno()
                 if from_revno is None or to_revno is None:
-                    raise BzrCommandError('Cannot sign a range of non-revision-history revisions')
+                    raise errors.BzrCommandError('Cannot sign a range of non-revision-history revisions')
                 for revno in range(from_revno, to_revno + 1):
                     b.repository.sign_revision(b.get_rev_id(revno), 
                                                gpg_strategy)
             else:
-                raise BzrCommandError('Please supply either one revision, or a range.')
+                raise errors.BzrCommandError('Please supply either one revision, or a range.')
 
 
 class cmd_bind(Command):
@@ -2616,9 +2684,9 @@
         b_other = Branch.open(location)
         try:
             b.bind(b_other)
-        except DivergedBranches:
-            raise BzrCommandError('These branches have diverged.'
-                                  ' Try merging, and then bind again.')
+        except errors.DivergedBranches:
+            raise errors.BzrCommandError('These branches have diverged.'
+                                         ' Try merging, and then bind again.')
 
 
 class cmd_unbind(Command):
@@ -2634,7 +2702,7 @@
     def run(self):
         b, relpath = Branch.open_containing(u'.')
         if not b.unbind():
-            raise BzrCommandError('Local branch is not bound')
+            raise errors.BzrCommandError('Local branch is not bound')
 
 
 class cmd_uncommit(Command):
@@ -2742,13 +2810,74 @@
             pass
         
 
+class cmd_wait_until_signalled(Command):
+    """Test helper for test_start_and_stop_bzr_subprocess_send_signal.
+
+    This just prints a line to signal when it is ready, then blocks on stdin.
+    """
+
+    hidden = True
+
+    def run(self):
+        sys.stdout.write("running\n")
+        sys.stdout.flush()
+        sys.stdin.readline()
+
+
+class cmd_serve(Command):
+    """Run the bzr server."""
+
+    aliases = ['server']
+
+    takes_options = [
+        Option('inet',
+               help='serve on stdin/out for use from inetd or sshd'),
+        Option('port',
+               help='listen for connections on nominated port of the form '
+                    '[hostname:]portnumber. Passing 0 as the port number will '
+                    'result in a dynamically allocated port.',
+               type=str),
+        Option('directory',
+               help='serve contents of directory',
+               type=unicode),
+        Option('allow-writes',
+               help='By default the server is a readonly server. Supplying '
+                    '--allow-writes enables write access to the contents of '
+                    'the served directory and below. '
+                ),
+        ]
+
+    def run(self, port=None, inet=False, directory=None, allow_writes=False):
+        from bzrlib.transport import smart
+        from bzrlib.transport import get_transport
+        if directory is None:
+            directory = os.getcwd()
+        url = urlutils.local_path_to_url(directory)
+        if not allow_writes:
+            url = 'readonly+' + url
+        t = get_transport(url)
+        if inet:
+            server = smart.SmartServerPipeStreamMedium(sys.stdin, sys.stdout, t)
+        elif port is not None:
+            if ':' in port:
+                host, port = port.split(':')
+            else:
+                host = '127.0.0.1'
+            server = smart.SmartTCPServer(t, host=host, port=int(port))
+            print 'listening on port: ', server.port
+            sys.stdout.flush()
+        else:
+            raise errors.BzrCommandError("bzr serve requires one of --inet or --port")
+        server.serve()
+
 
 # command-line interpretation helper for merge-related commands
-def merge(other_revision, base_revision,
-          check_clean=True, ignore_zero=False,
-          this_dir=None, backup_files=False, merge_type=Merge3Merger,
-          file_list=None, show_base=False, reprocess=False,
-          pb=DummyProgress()):
+def _merge_helper(other_revision, base_revision,
+                  check_clean=True, ignore_zero=False,
+                  this_dir=None, backup_files=False,
+                  merge_type=None,
+                  file_list=None, show_base=False, reprocess=False,
+                  pb=DummyProgress()):
     """Merge changes into a tree.
 
     base_revision
@@ -2776,20 +2905,23 @@
     clients might prefer to call merge.merge_inner(), which has less magic 
     behavior.
     """
-    from bzrlib.merge import Merger
+    # Loading it late, so that we don't always have to import bzrlib.merge
+    if merge_type is None:
+        merge_type = _mod_merge.Merge3Merger
     if this_dir is None:
         this_dir = u'.'
     this_tree = WorkingTree.open_containing(this_dir)[0]
-    if show_base and not merge_type is Merge3Merger:
-        raise BzrCommandError("Show-base is not supported for this merge"
-                              " type. %s" % merge_type)
+    if show_base and not merge_type is _mod_merge.Merge3Merger:
+        raise errors.BzrCommandError("Show-base is not supported for this merge"
+                                     " type. %s" % merge_type)
     if reprocess and not merge_type.supports_reprocess:
-        raise BzrCommandError("Conflict reduction is not supported for merge"
-                              " type %s." % merge_type)
+        raise errors.BzrCommandError("Conflict reduction is not supported for merge"
+                                     " type %s." % merge_type)
     if reprocess and show_base:
-        raise BzrCommandError("Cannot do conflict reduction and show base.")
+        raise errors.BzrCommandError("Cannot do conflict reduction and show base.")
     try:
-        merger = Merger(this_tree.branch, this_tree=this_tree, pb=pb)
+        merger = _mod_merge.Merger(this_tree.branch, this_tree=this_tree,
+                                   pb=pb)
         merger.pp = ProgressPhase("Merge phase", 5, pb)
         merger.pp.next_phase()
         merger.check_basis(check_clean)
@@ -2812,11 +2944,16 @@
     return conflicts
 
 
+# Compatibility
+merge = _merge_helper
+
+
 # these get imported and then picked up by the scan for cmd_*
 # TODO: Some more consistent way to split command definitions across files;
 # we do need to load at least some information about them to know of 
 # aliases.  ideally we would avoid loading the implementation until the
 # details were needed.
+from bzrlib.cmd_version_info import cmd_version_info
 from bzrlib.conflicts import cmd_resolve, cmd_conflicts, restore
 from bzrlib.bundle.commands import cmd_bundle_revisions
 from bzrlib.sign_my_commits import cmd_sign_my_commits

=== modified file 'bzrlib/bundle/__init__.py'
--- bzrlib/bundle/__init__.py	2006-07-07 16:41:38 +0000
+++ bzrlib/bundle/__init__.py	2006-10-06 05:53:44 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,10 +14,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    errors,
+    urlutils,
+    )
 from bzrlib.bundle.serializer import read_bundle
-import bzrlib.errors as errors
-import bzrlib.urlutils
-import bzrlib.transport
+from bzrlib.transport import get_transport
+""")
 
 
 def read_bundle_from_url(url):
@@ -26,7 +31,7 @@
     :return: A BundleReader, may raise NotABundle if the target 
             is not a proper bundle.
     """
-    url, filename = bzrlib.urlutils.split(url, exclude_trailing_slash=False)
+    url, filename = urlutils.split(url, exclude_trailing_slash=False)
     if not filename:
         # A path to a directory was passed in
         # definitely not a bundle
@@ -36,7 +41,7 @@
     # Some transports cannot detect that we are trying to read a
     # directory until we actually issue read() on the handle.
     try:
-        t = bzrlib.transport.get_transport(url)
+        t = get_transport(url)
         f = t.get(filename)
         return read_bundle(f)
     except (errors.TransportError, errors.PathError), e:

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2006-08-29 20:47:02 +0000
+++ bzrlib/bundle/bundle_data.py	2006-10-12 03:19:14 +0000
@@ -223,7 +223,8 @@
             if repository.has_revision(revision_id):
                 testament = StrictTestament.from_revision(repository, 
                                                           revision_id)
-                local_sha1 = testament.as_sha1()
+                local_sha1 = self._testament_sha1_from_revision(repository,
+                                                                revision_id)
                 if sha1 != local_sha1:
                     raise BzrError('sha1 mismatch. For revision id {%s}' 
                             'local: %s, bundle: %s' % (revision_id, local_sha1, sha1))
@@ -281,7 +282,7 @@
         rev_info = self.get_revision_info(revision_id)
         assert rev.revision_id == rev_info.revision_id
         assert rev.revision_id == revision_id
-        sha1 = StrictTestament(rev, inventory).as_sha1()
+        sha1 = self._testament_sha1(rev, inventory)
         if sha1 != rev_info.sha1:
             raise TestamentMismatch(rev.revision_id, rev_info.sha1, sha1)
         if rev.revision_id in rev_to_sha1:
@@ -539,7 +540,7 @@
             return None
         if old_path in self.deleted:
             return None
-        if hasattr(self.base_tree, 'path2id'):
+        if getattr(self.base_tree, 'path2id', None) is not None:
             return self.base_tree.path2id(old_path)
         else:
             return self.base_tree.inventory.path2id(old_path)
@@ -577,7 +578,8 @@
                 then be cached.
         """
         base_id = self.old_contents_id(file_id)
-        if base_id is not None:
+        if (base_id is not None and
+            base_id != self.base_tree.inventory.root.file_id):
             patch_original = self.base_tree.get_file(base_id)
         else:
             patch_original = None
@@ -646,22 +648,16 @@
 
         assert self.base_tree is not None
         base_inv = self.base_tree.inventory
-        root_id = base_inv.root.file_id
-        try:
-            # New inventories have a unique root_id
-            inv = Inventory(root_id, self.revision_id)
-        except TypeError:
-            inv = Inventory(revision_id=self.revision_id)
-        inv.root.revision = self.get_last_changed(root_id)
+        inv = Inventory(None, self.revision_id)
 
         def add_entry(file_id):
             path = self.id2path(file_id)
             if path is None:
                 return
-            parent_path = dirname(path)
-            if parent_path == u'':
-                parent_id = root_id
+            if path == '':
+                parent_id = None
             else:
+                parent_path = dirname(path)
                 parent_id = self.path2id(parent_path)
 
             kind = self.get_kind(file_id)
@@ -688,8 +684,6 @@
 
         sorted_entries = self.sorted_path_id()
         for path, file_id in sorted_entries:
-            if file_id == inv.root.file_id:
-                continue
             add_entry(file_id)
 
         return inv

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2006-08-24 00:07:15 +0000
+++ bzrlib/bundle/commands.py	2006-09-12 22:12:30 +0000
@@ -8,14 +8,19 @@
 
 import sys
 
-from bzrlib.branch import Branch
-from bzrlib.commands import Command, register_command
-import bzrlib.errors as errors
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    branch,
+    errors,
+    urlutils,
+    )
+from bzrlib.revision import common_ancestor
+""")
+
+from bzrlib.commands import Command
 from bzrlib.option import Option
-from bzrlib.revision import (common_ancestor, MultipleRevisionSources,
-                             NULL_REVISION)
 from bzrlib.trace import note
-from bzrlib import urlutils
 
 
 class cmd_send_changeset(Command):
@@ -43,7 +48,7 @@
                 raise BzrCommandError('We do not support rollup-changesets yet.')
             revision = revision[0]
 
-        b = Branch.open_containing('.')
+        b = branch.Branch.open_containing('.')
 
         if not to:
             try:
@@ -84,7 +89,7 @@
         from bzrlib import user_encoding
         from bzrlib.bundle.serializer import write_bundle
 
-        target_branch = Branch.open_containing(u'.')[0]
+        target_branch = branch.Branch.open_containing(u'.')[0]
 
         if base is None:
             base_specified = False
@@ -118,7 +123,7 @@
                 # we must format with 'ascii' to be safe
                 note('Using saved location: %s',
                      urlutils.unescape_for_display(base, 'ascii'))
-            base_branch = Branch.open(base)
+            base_branch = branch.Branch.open(base)
 
             # We don't want to lock the same branch across
             # 2 different branches
@@ -159,7 +164,7 @@
         from read_changeset import read_changeset
         #from bzrlib.xml import serializer_v4
 
-        b, relpath = Branch.open_containing('.')
+        b, relpath = branch.Branch.open_containing('.')
 
         if filename is None or filename == '-':
             f = sys.stdin

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2006-08-29 20:47:02 +0000
+++ bzrlib/bundle/serializer/__init__.py	2006-09-27 20:16:08 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Development Ltd
+# (C) 2005-2006 Canonical Development Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,9 +37,7 @@
 
 
 def _get_filename(f):
-    if hasattr(f, 'name'):
-        return f.name
-    return '<unknown>'
+    return getattr(f, 'name', '<unknown>')
 
 
 def read_bundle(f):
@@ -98,23 +96,32 @@
         source.unlock()
 
 
-def write_bundle(repository, revision_id, base_revision_id, out):
+def write_bundle(repository, revision_id, base_revision_id, out, format=None):
     """"""
     repository.lock_read()
     try:
-        return _write_bundle(repository, revision_id, base_revision_id, out)
+        return _write_bundle(repository, revision_id, base_revision_id, out,
+                             format)
     finally:
         repository.unlock()
 
 
-def _write_bundle(repository, revision_id, base_revision_id, out):
+def _write_bundle(repository, revision_id, base_revision_id, out, format):
+    """Write a bundle of revisions.
+
+    :param repository: Repository containing revisions to serialize.
+    :param revision_id: Head revision_id of the bundle.
+    :param base_revision_id: Revision assumed to be present in repositories
+         applying the bundle.
+    :param out: Output file.
+    """
     if base_revision_id is NULL_REVISION:
         base_revision_id = None
     base_ancestry = set(repository.get_ancestry(base_revision_id))
     revision_ids = [r for r in repository.get_ancestry(revision_id) if r
                     not in base_ancestry]
     revision_ids = list(reversed(revision_ids))
-    write(repository, revision_ids, out, 
+    write(repository, revision_ids, out, format,
           forced_bases = {revision_id:base_revision_id})
     return revision_ids
 
@@ -151,7 +158,7 @@
     
     # This has to be formatted for "original" date, so that the
     # revision XML entry will be reproduced faithfully.
-    if offset == None:
+    if offset is None:
         offset = 0
     tt = time.gmtime(t + offset)
 
@@ -290,5 +297,6 @@
     to_file.write('\n')
 
 register_lazy('0.8', 'bzrlib.bundle.serializer.v08', 'BundleSerializerV08')
+register_lazy('0.9', 'bzrlib.bundle.serializer.v09', 'BundleSerializerV09')
 register_lazy(None, 'bzrlib.bundle.serializer.v08', 'BundleSerializerV08')
 

=== modified file 'bzrlib/bundle/serializer/v08.py'
--- bzrlib/bundle/serializer/v08.py	2006-08-15 20:43:59 +0000
+++ bzrlib/bundle/serializer/v08.py	2006-09-28 22:50:18 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Development Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -94,6 +94,10 @@
         """
         return BundleReader(f).info
 
+    def check_compatible(self):
+        if self.source.supports_rich_root():
+            raise errors.IncompatibleFormat('0.8', repr(self.source))
+
     def write(self, source, revision_ids, forced_bases, f):
         """Write the bundless to the supplied files.
 
@@ -106,6 +110,7 @@
         self.revision_ids = revision_ids
         self.forced_bases = forced_bases
         self.to_file = f
+        self.check_compatible()
         source.lock_read()
         try:
             self._write_main_header()
@@ -182,6 +187,10 @@
             last_rev_id = base_id
             last_rev_tree = base_tree
 
+    def _testament_sha1(self, revision_id):
+        return StrictTestament.from_revision(self.source, 
+                                             revision_id).as_sha1()
+
     def _write_revision(self, rev, rev_tree, base_rev, base_tree, 
                         explicit_base, force_binary):
         """Write out the information for a revision."""
@@ -196,8 +205,7 @@
         self._write_delta(rev_tree, base_tree, rev.revision_id, force_binary)
 
         w('revision id', rev.revision_id)
-        w('sha1', StrictTestament.from_revision(self.source, 
-                                                rev.revision_id).as_sha1())
+        w('sha1', self._testament_sha1(rev.revision_id))
         w('inventory sha1', rev.inventory_sha1)
         if rev.parent_ids:
             w('parent ids', rev.parent_ids)
@@ -267,7 +275,8 @@
             else:
                 action.write(self.to_file)
 
-        delta = new_tree.changes_from(old_tree, want_unchanged=True)
+        delta = new_tree.changes_from(old_tree, want_unchanged=True,
+                                      include_root=True)
         for path, file_id, kind in delta.removed:
             action = Action('removed', [kind, path]).write(self.to_file)
 
@@ -316,7 +325,7 @@
         self.from_file = iter(from_file)
         self._next_line = None
         
-        self.info = BundleInfo08()
+        self.info = self._get_info()
         # We put the actual inventory ids in the footer, so that the patch
         # is easier to read for humans.
         # Unfortunately, that means we need to read everything before we
@@ -324,6 +333,9 @@
         self._read()
         self._validate()
 
+    def _get_info(self):
+        return BundleInfo08()
+
     def _read(self):
         self._next().next()
         while self._next_line is not None:
@@ -411,7 +423,7 @@
             return
 
         revision_info = self.info.revisions[-1]
-        if hasattr(revision_info, key):
+        if key in revision_info.__dict__:
             if getattr(revision_info, key) is None:
                 setattr(revision_info, key, value)
             else:
@@ -502,6 +514,14 @@
 
 
 class BundleInfo08(BundleInfo):
+
     def _update_tree(self, bundle_tree, revision_id):
         bundle_tree.note_last_changed('', revision_id)
         BundleInfo._update_tree(self, bundle_tree, revision_id)
+
+    def _testament_sha1_from_revision(self, repository, revision_id):
+        testament = StrictTestament.from_revision(repository, revision_id)
+        return testament.as_sha1()
+
+    def _testament_sha1(self, revision, inventory):
+        return StrictTestament(revision, inventory).as_sha1()

=== added file 'bzrlib/bundle/serializer/v09.py'
--- bzrlib/bundle/serializer/v09.py	1970-01-01 00:00:00 +0000
+++ bzrlib/bundle/serializer/v09.py	2006-09-27 20:16:08 +0000
@@ -0,0 +1,74 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.bundle.serializer import BUNDLE_HEADER
+from bzrlib.bundle.serializer.v08 import BundleSerializerV08, BundleReader
+from bzrlib.testament import StrictTestament3
+from bzrlib.bundle.bundle_data import BundleInfo
+
+
+"""Serializer for bundle format 0.9"""
+
+
+class BundleSerializerV09(BundleSerializerV08):
+    """Serializer for bzr bundle format 0.9
+    
+    This format supports rich root data, for the nested-trees work, but also
+    supports repositories that don't have rich root data.  It cannot be
+    used to transfer from a knit2 repo into a knit1 repo, because that would
+    be lossy.
+    """
+
+    def check_compatible(self):
+        pass
+
+    def _write_main_header(self):
+        """Write the header for the changes"""
+        f = self.to_file
+        f.write(BUNDLE_HEADER + '0.9\n#\n')
+
+    def _testament_sha1(self, revision_id):
+        return StrictTestament3.from_revision(self.source, 
+                                              revision_id).as_sha1()
+
+    def read(self, f):
+        """Read the rest of the bundles from the supplied file.
+
+        :param f: The file to read from
+        :return: A list of bundles
+        """
+        return BundleReaderV09(f).info
+
+
+class BundleInfo09(BundleInfo):
+    """BundleInfo that uses StrictTestament3
+    
+    This means that the root data is included in the testament.
+    """
+
+    def _testament_sha1_from_revision(self, repository, revision_id):
+        testament = StrictTestament3.from_revision(repository, revision_id)
+        return testament.as_sha1()
+
+    def _testament_sha1(self, revision, inventory):
+        return StrictTestament3(revision, inventory).as_sha1()
+
+
+class BundleReaderV09(BundleReader):
+    """BundleReader for 0.9 bundles"""
+    
+    def _get_info(self):
+        return BundleInfo09()

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-09-05 18:48:28 +0000
+++ bzrlib/bzrdir.py	2006-10-12 03:19:14 +0000
@@ -22,35 +22,46 @@
 
 # TODO: remove unittest dependency; put that stuff inside the test suite
 
-from copy import deepcopy
+# TODO: The Format probe_transport seems a bit redundant with just trying to
+# open the bzrdir. -- mbp
+#
+# TODO: Can we move specific formats into separate modules to make this file
+# smaller?
+
 from cStringIO import StringIO
 import os
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from copy import deepcopy
 from stat import S_ISDIR
-from unittest import TestSuite
+import unittest
 
 import bzrlib
-import bzrlib.errors as errors
-from bzrlib.lockable_files import LockableFiles, TransportLock
-from bzrlib.lockdir import LockDir
+from bzrlib import (
+    errors,
+    lockable_files,
+    lockdir,
+    revision as _mod_revision,
+    urlutils,
+    xml4,
+    xml5,
+    )
 from bzrlib.osutils import (
-                            abspath,
-                            pathjoin,
-                            safe_unicode,
-                            sha_strings,
-                            sha_string,
-                            )
-import bzrlib.revision
+    safe_unicode,
+    sha_strings,
+    sha_string,
+    )
 from bzrlib.store.revision.text import TextRevisionStore
 from bzrlib.store.text import TextStore
 from bzrlib.store.versioned import WeaveStore
-from bzrlib.trace import mutter
 from bzrlib.transactions import WriteTransaction
 from bzrlib.transport import get_transport
+from bzrlib.weave import Weave
+""")
+
+from bzrlib.trace import mutter
 from bzrlib.transport.local import LocalTransport
-import bzrlib.urlutils as urlutils
-from bzrlib.weave import Weave
-from bzrlib.xml4 import serializer_v4
-import bzrlib.xml5
 
 
 class BzrDir(object):
@@ -87,6 +98,11 @@
         """Return true if this bzrdir is one whose format we can convert from."""
         return True
 
+    def check_conversion_target(self, target_format):
+        target_repo_format = target_format.repository_format
+        source_repo_format = self._format.repository_format
+        source_repo_format.check_conversion_target(target_repo_format)
+
     @staticmethod
     def _check_supported(format, allow_unsupported):
         """Check whether format is a supported format.
@@ -177,7 +193,7 @@
     def _make_tail(self, url):
         head, tail = urlutils.split(url)
         if tail and tail != '.':
-            t = bzrlib.transport.get_transport(head)
+            t = get_transport(head)
             try:
                 t.mkdir(tail)
             except errors.FileExists:
@@ -199,7 +215,7 @@
                     "not one of %r" % cls)
         head, tail = urlutils.split(base)
         if tail and tail != '.':
-            t = bzrlib.transport.get_transport(head)
+            t = get_transport(head)
             try:
                 t.mkdir(tail)
             except errors.FileExists:
@@ -293,7 +309,7 @@
         This will use the current default BzrDirFormat, and use whatever 
         repository format that that uses for bzrdirformat.create_repository.
 
-        ;param shared: Create a shared repository rather than a standalone
+        :param shared: Create a shared repository rather than a standalone
                        repository.
         The Repository object is returned.
 
@@ -314,7 +330,7 @@
         repository format that that uses for bzrdirformat.create_workingtree,
         create_branch and create_repository.
 
-        The WorkingTree object is returned.
+        :return: The WorkingTree object.
         """
         t = get_transport(safe_unicode(base))
         if not isinstance(t, LocalTransport):
@@ -330,6 +346,21 @@
         """
         raise NotImplementedError(self.create_workingtree)
 
+    def destroy_workingtree(self):
+        """Destroy the working tree at this BzrDir.
+
+        Formats that do not support this may raise UnsupportedOperation.
+        """
+        raise NotImplementedError(self.destroy_workingtree)
+
+    def destroy_workingtree_metadata(self):
+        """Destroy the control files for the working tree at this BzrDir.
+
+        The contents of working tree files are not affected.
+        Formats that do not support this may raise UnsupportedOperation.
+        """
+        raise NotImplementedError(self.destroy_workingtree_metadata)
+
     def find_repository(self):
         """Find the repository that should be used for a_bzrdir.
 
@@ -461,10 +492,18 @@
         _unsupported is a private parameter to the BzrDir class.
         """
         t = get_transport(base)
-        # mutter("trying to open %r with transport %r", base, t)
-        format = BzrDirFormat.find_format(t)
+        return BzrDir.open_from_transport(t, _unsupported=_unsupported)
+
+    @staticmethod
+    def open_from_transport(transport, _unsupported=False):
+        """Open a bzrdir within a particular directory.
+
+        :param transport: Transport containing the bzrdir.
+        :param _unsupported: private.
+        """
+        format = BzrDirFormat.find_format(transport)
         BzrDir._check_supported(format, _unsupported)
-        return format.open(t, _found=True)
+        return format.open(transport, _found=True)
 
     def open_branch(self, unsupported=False):
         """Open the branch object at this BzrDir if one is present.
@@ -504,11 +543,9 @@
         url = a_transport.base
         while True:
             try:
-                format = BzrDirFormat.find_format(a_transport)
-                BzrDir._check_supported(format, False)
-                return format.open(a_transport), urlutils.unescape(a_transport.relpath(url))
+                result = BzrDir.open_from_transport(a_transport)
+                return result, urlutils.unescape(a_transport.relpath(url))
             except errors.NotBranchError, e:
-                ## mutter('not a branch in: %r %s', a_transport.base, e)
                 pass
             new_t = a_transport.clone('..')
             if new_t.base == a_transport.base:
@@ -564,6 +601,28 @@
         except errors.NoWorkingTree:
             return False
 
+    def cloning_metadir(self, basis=None):
+        """Produce a metadir suitable for cloning with"""
+        def related_repository(bzrdir):
+            try:
+                branch = bzrdir.open_branch()
+                return branch.repository
+            except errors.NotBranchError:
+                source_branch = None
+                return bzrdir.open_repository()
+        result_format = self._format.__class__()
+        try:
+            try:
+                source_repository = related_repository(self)
+            except errors.NoRepositoryPresent:
+                if basis is None:
+                    raise
+                source_repository = related_repository(self)
+            result_format.repository_format = source_repository._format
+        except errors.NoRepositoryPresent:
+            pass
+        return result_format
+
     def sprout(self, url, revision_id=None, basis=None, force_new_repo=False):
         """Create a copy of this bzrdir prepared for use as a new line of
         development.
@@ -579,7 +638,8 @@
             itself to download less data.
         """
         self._make_tail(url)
-        result = self._format.initialize(url)
+        cloning_format = self.cloning_metadir(basis)
+        result = cloning_format.initialize(url)
         basis_repo, basis_branch, basis_tree = self._get_basis_components(basis)
         try:
             source_branch = self.open_branch()
@@ -625,7 +685,12 @@
         # TODO: jam 20060426 we probably need a test in here in the
         #       case that the newly sprouted branch is a remote one
         if result_repo is None or result_repo.make_working_trees():
-            result.create_workingtree()
+            wt = result.create_workingtree()
+            if wt.inventory.root is None:
+                try:
+                    wt.set_root_id(self.open_workingtree.get_root_id())
+                except errors.NoWorkingTree:
+                    pass
         return result
 
 
@@ -635,9 +700,10 @@
     def __init__(self, _transport, _format):
         """See BzrDir.__init__."""
         super(BzrDirPreSplitOut, self).__init__(_transport, _format)
-        assert self._format._lock_class == TransportLock
+        assert self._format._lock_class == lockable_files.TransportLock
         assert self._format._lock_file_name == 'branch-lock'
-        self._control_files = LockableFiles(self.get_branch_transport(None),
+        self._control_files = lockable_files.LockableFiles(
+                                            self.get_branch_transport(None),
                                             self._format._lock_file_name,
                                             self._format._lock_class)
 
@@ -687,12 +753,21 @@
         # done on this format anyway. So - acceptable wart.
         result = self.open_workingtree()
         if revision_id is not None:
-            if revision_id == bzrlib.revision.NULL_REVISION:
+            if revision_id == _mod_revision.NULL_REVISION:
                 result.set_parent_ids([])
             else:
                 result.set_parent_ids([revision_id])
         return result
 
+    def destroy_workingtree(self):
+        """See BzrDir.destroy_workingtree."""
+        raise errors.UnsupportedOperation(self.destroy_workingtree, self)
+
+    def destroy_workingtree_metadata(self):
+        """See BzrDir.destroy_workingtree_metadata."""
+        raise errors.UnsupportedOperation(self.destroy_workingtree_metadata, 
+                                          self)
+
     def get_branch_transport(self, branch_format):
         """See BzrDir.get_branch_transport()."""
         if branch_format is None:
@@ -838,9 +913,21 @@
         from bzrlib.workingtree import WorkingTreeFormat
         return WorkingTreeFormat.get_default_format().initialize(self, revision_id)
 
+    def destroy_workingtree(self):
+        """See BzrDir.destroy_workingtree."""
+        wt = self.open_workingtree()
+        repository = wt.branch.repository
+        empty = repository.revision_tree(bzrlib.revision.NULL_REVISION)
+        wt.revert([], old_tree=empty)
+        self.destroy_workingtree_metadata()
+
+    def destroy_workingtree_metadata(self):
+        self.transport.delete_tree('checkout')
+
     def _get_mkdir_mode(self):
         """Figure out the mode to use when creating a bzrdir subdir."""
-        temp_control = LockableFiles(self.transport, '', TransportLock)
+        temp_control = lockable_files.LockableFiles(self.transport, '',
+                                     lockable_files.TransportLock)
         return temp_control._dir_mode
 
     def get_branch_transport(self, branch_format):
@@ -1022,7 +1109,8 @@
         """Initialize a new bzrdir in the base directory of a Transport."""
         # Since we don't have a .bzr directory, inherit the
         # mode from the root directory
-        temp_control = LockableFiles(transport, '', TransportLock)
+        temp_control = lockable_files.LockableFiles(transport,
+                            '', lockable_files.TransportLock)
         temp_control._transport.mkdir('.bzr',
                                       # FIXME: RBC 20060121 don't peek under
                                       # the covers
@@ -1037,8 +1125,8 @@
                       ('branch-format', self.get_format_string()),
                       ]
         # NB: no need to escape relative paths that are url safe.
-        control_files = LockableFiles(control, self._lock_file_name, 
-                                      self._lock_class)
+        control_files = lockable_files.LockableFiles(control,
+                            self._lock_file_name, self._lock_class)
         control_files.create_lock()
         control_files.lock_write()
         try:
@@ -1057,6 +1145,10 @@
         """
         return True
 
+    def same_model(self, target_format):
+        return (self.repository_format.rich_root_data == 
+            target_format.rich_root_data)
+
     @classmethod
     def known_formats(klass):
         """Return all the known formats.
@@ -1143,7 +1235,7 @@
     removed in format 5; write support for this format has been removed.
     """
 
-    _lock_class = TransportLock
+    _lock_class = lockable_files.TransportLock
 
     def get_format_string(self):
         """See BzrDirFormat.get_format_string()."""
@@ -1178,7 +1270,7 @@
     def __return_repository_format(self):
         """Circular import protection."""
         from bzrlib.repository import RepositoryFormat4
-        return RepositoryFormat4(self)
+        return RepositoryFormat4()
     repository_format = property(__return_repository_format)
 
 
@@ -1193,7 +1285,7 @@
        Unhashed stores in the repository.
     """
 
-    _lock_class = TransportLock
+    _lock_class = lockable_files.TransportLock
 
     def get_format_string(self):
         """See BzrDirFormat.get_format_string()."""
@@ -1238,7 +1330,7 @@
     def __return_repository_format(self):
         """Circular import protection."""
         from bzrlib.repository import RepositoryFormat5
-        return RepositoryFormat5(self)
+        return RepositoryFormat5()
     repository_format = property(__return_repository_format)
 
 
@@ -1252,7 +1344,7 @@
      - Format 6 repositories [always]
     """
 
-    _lock_class = TransportLock
+    _lock_class = lockable_files.TransportLock
 
     def get_format_string(self):
         """See BzrDirFormat.get_format_string()."""
@@ -1297,7 +1389,7 @@
     def __return_repository_format(self):
         """Circular import protection."""
         from bzrlib.repository import RepositoryFormat6
-        return RepositoryFormat6(self)
+        return RepositoryFormat6()
     repository_format = property(__return_repository_format)
 
 
@@ -1312,7 +1404,7 @@
      - Format 7 repositories [optional]
     """
 
-    _lock_class = LockDir
+    _lock_class = lockdir.LockDir
 
     def get_converter(self, format=None):
         """See BzrDirFormat.get_converter()."""
@@ -1372,7 +1464,7 @@
         self._formats = formats
     
     def adapt(self, test):
-        result = TestSuite()
+        result = unittest.TestSuite()
         for format in self._formats:
             new_test = deepcopy(test)
             new_test.transport_server = self._transport_server
@@ -1476,8 +1568,9 @@
         self.bzrdir.transport.delete_tree('text-store')
 
     def _convert_working_inv(self):
-        inv = serializer_v4.read_inventory(self.branch.control_files.get('inventory'))
-        new_inv_xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
+        inv = xml4.serializer_v4.read_inventory(
+                    self.branch.control_files.get('inventory'))
+        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv)
         # FIXME inventory is a working tree change.
         self.branch.control_files.put('inventory', StringIO(new_inv_xml))
 
@@ -1509,7 +1602,7 @@
                                                       prefixed=False,
                                                       compressed=True))
         try:
-            transaction = bzrlib.transactions.WriteTransaction()
+            transaction = WriteTransaction()
             for i, rev_id in enumerate(self.converted_revs):
                 self.pb.update('write revision', i, len(self.converted_revs))
                 _revision_store.add_revision(self.revisions[rev_id], transaction)
@@ -1541,7 +1634,8 @@
     def _load_old_inventory(self, rev_id):
         assert rev_id not in self.converted_revs
         old_inv_xml = self.branch.repository.inventory_store.get(rev_id).read()
-        inv = serializer_v4.read_inventory_from_string(old_inv_xml)
+        inv = xml4.serializer_v4.read_inventory_from_string(old_inv_xml)
+        inv.revision_id = rev_id
         rev = self.revisions[rev_id]
         if rev.inventory_sha1:
             assert rev.inventory_sha1 == sha_string(old_inv_xml), \
@@ -1551,7 +1645,7 @@
     def _load_updated_inventory(self, rev_id):
         assert rev_id in self.converted_revs
         inv_xml = self.inv_weave.get_text(rev_id)
-        inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(inv_xml)
+        inv = xml5.serializer_v5.read_inventory_from_string(inv_xml)
         return inv
 
     def _convert_one_rev(self, rev_id):
@@ -1570,10 +1664,10 @@
             entries = inv.iter_entries()
             entries.next()
             for path, ie in entries:
-                assert hasattr(ie, 'revision'), \
+                assert getattr(ie, 'revision', None) is not None, \
                     'no revision on {%s} in {%s}' % \
                     (file_id, rev.revision_id)
-        new_inv_xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
+        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv)
         new_inv_sha1 = sha_string(new_inv_xml)
         self.inv_weave.add_lines(rev.revision_id, 
                                  present_parents,
@@ -1612,7 +1706,8 @@
                                                   entry_vf=w)
         for old_revision in previous_entries:
                 # if this fails, its a ghost ?
-                assert old_revision in self.converted_revs 
+                assert old_revision in self.converted_revs, \
+                    "Revision {%s} not in converted_revs" % old_revision
         self.snapshot_ie(previous_entries, ie, w, rev_id)
         del ie.text_id
         assert getattr(ie, 'revision', None) is not None
@@ -1791,10 +1886,10 @@
     def make_lock(self, name):
         """Make a lock for the new control dir name."""
         self.step('Make %s lock' % name)
-        ld = LockDir(self.bzrdir.transport, 
-                     '%s/lock' % name,
-                     file_modebits=self.file_mode,
-                     dir_modebits=self.dir_mode)
+        ld = lockdir.LockDir(self.bzrdir.transport,
+                             '%s/lock' % name,
+                             file_modebits=self.file_mode,
+                             dir_modebits=self.dir_mode)
         ld.create()
 
     def move_entry(self, new_dir, entry):

=== added file 'bzrlib/cmd_version_info.py'
--- bzrlib/cmd_version_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/cmd_version_info.py	2006-10-06 07:00:57 +0000
@@ -0,0 +1,90 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Commands for generating snapshot information about a bzr tree."""
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    branch,
+    errors,
+    version_info_formats,
+    workingtree,
+    )
+""")
+from bzrlib.commands import Command
+from bzrlib.option import Option
+
+
+def _parse_version_info_format(format):
+    """Convert a string passed by the user into a VersionInfoFormat.
+
+    This looks in the version info format registry, and if the format
+    cannot be found, generates a useful error exception.
+    """
+    try:
+        return version_info_formats.get_builder(format)
+    except KeyError:
+        formats = version_info_formats.get_builder_formats()
+        raise errors.BzrCommandError('No known version info format %s.'
+                                     ' Supported types are: %s'
+                                     % (format, formats))
+
+
+class cmd_version_info(Command):
+    """Generate version information about this tree."""
+
+    takes_options = [Option('format', type=_parse_version_info_format,
+                            help='Select the output format'),
+                     Option('all', help='include all possible information'),
+                     Option('check-clean', help='check if tree is clean'),
+                     Option('include-history',
+                            help='Include the revision-history'),
+                     Option('include-file-revisions',
+                            help='Include the last revision for each file')
+                     ]
+    takes_args = ['location?']
+
+    encoding_type = 'exact'
+
+    def run(self, location=None, format=None,
+            all=False, check_clean=False, include_history=False,
+            include_file_revisions=False):
+
+        if location is None:
+            location = '.'
+
+        if format is None:
+            format = version_info_formats.get_builder(None)
+
+        wt = None
+        try:
+            wt = workingtree.WorkingTree.open_containing(location)[0]
+        except errors.NoWorkingTree:
+            b = branch.Branch.open(location)
+        else:
+            b = wt.branch
+
+        if all:
+            include_history = True
+            check_clean = True
+            include_file_revisions=True
+
+        builder = format(b, working_tree=wt,
+                check_for_clean=check_clean,
+                include_revision_history=include_history,
+                include_file_revisions=include_file_revisions)
+        builder.generate(self.outf)

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-08-31 18:18:09 +0000
+++ bzrlib/commands.py	2006-10-06 05:53:44 +0000
@@ -28,24 +28,33 @@
 # TODO: "--profile=cum", to change sort order.  Is there any value in leaving
 # the profile output behind so it can be interactively examined?
 
+import os
+import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import codecs
 import errno
-import os
 from warnings import warn
-import sys
 
 import bzrlib
-import bzrlib.errors as errors
-from bzrlib.errors import (BzrError,
-                           BzrCommandError,
-                           BzrCheckError,
-                           NotBranchError)
-from bzrlib import option
+from bzrlib import (
+    errors,
+    option,
+    osutils,
+    trace,
+    )
+""")
+
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    deprecated_method,
+    zero_eight,
+    zero_eleven,
+    )
+# Compatibility
 from bzrlib.option import Option
-import bzrlib.osutils
-from bzrlib.symbol_versioning import (deprecated_method, zero_eight)
-import bzrlib.trace
-from bzrlib.trace import mutter, note, log_error, warning, be_quiet
+
 
 plugin_cmds = {}
 
@@ -66,7 +75,7 @@
         k_unsquished = k
     if k_unsquished not in plugin_cmds:
         plugin_cmds[k_unsquished] = cmd
-        mutter('registered plugin command %s', k_unsquished)
+        trace.mutter('registered plugin command %s', k_unsquished)
         if decorate and k_unsquished in builtin_command_names():
             return _builtin_commands()[k_unsquished]
     elif decorate:
@@ -74,8 +83,8 @@
         plugin_cmds[k_unsquished] = cmd
         return result
     else:
-        log_error('Two plugins defined the same command: %r' % k)
-        log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
+        trace.log_error('Two plugins defined the same command: %r' % k)
+        trace.log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
 
 
 def _squish_command_name(cmd):
@@ -150,7 +159,7 @@
     if cmd_obj:
         return cmd_obj
 
-    raise BzrCommandError('unknown command "%s"' % cmd_name)
+    raise errors.BzrCommandError('unknown command "%s"' % cmd_name)
 
 
 class Command(object):
@@ -222,10 +231,10 @@
 
         Maps from long option name to option object."""
         r = dict()
-        r['help'] = Option.OPTIONS['help']
+        r['help'] = option.Option.OPTIONS['help']
         for o in self.takes_options:
             if isinstance(o, basestring):
-                o = Option.OPTIONS[o]
+                o = option.Option.OPTIONS[o]
             r[o.name] = o
         return r
 
@@ -239,7 +248,7 @@
             self.outf = sys.stdout
             return
 
-        output_encoding = bzrlib.osutils.get_terminal_encoding()
+        output_encoding = osutils.get_terminal_encoding()
 
         # use 'replace' so that we don't abort if trying to write out
         # in e.g. the default C locale.
@@ -316,6 +325,9 @@
             return None
 
 
+# Technically, this function hasn't been use in a *really* long time
+# but we are only deprecating it now.
+@deprecated_function(zero_eleven)
 def parse_spec(spec):
     """
     >>> parse_spec(None)
@@ -385,29 +397,29 @@
                 argdict[argname + '_list'] = None
         elif ap[-1] == '+':
             if not args:
-                raise BzrCommandError("command %r needs one or more %s"
-                        % (cmd, argname.upper()))
+                raise errors.BzrCommandError("command %r needs one or more %s"
+                                             % (cmd, argname.upper()))
             else:
                 argdict[argname + '_list'] = args[:]
                 args = []
         elif ap[-1] == '$': # all but one
             if len(args) < 2:
-                raise BzrCommandError("command %r needs one or more %s"
-                        % (cmd, argname.upper()))
+                raise errors.BzrCommandError("command %r needs one or more %s"
+                                             % (cmd, argname.upper()))
             argdict[argname + '_list'] = args[:-1]
             args[:-1] = []
         else:
             # just a plain arg
             argname = ap
             if not args:
-                raise BzrCommandError("command %r requires argument %s"
-                        % (cmd, argname.upper()))
+                raise errors.BzrCommandError("command %r requires argument %s"
+                               % (cmd, argname.upper()))
             else:
                 argdict[argname] = args.pop(0)
             
     if args:
-        raise BzrCommandError("extra argument to command %s: %s"
-                              % (cmd, args[0]))
+        raise errors.BzrCommandError("extra argument to command %s: %s"
+                                     % (cmd, args[0]))
 
     return argdict
 
@@ -520,7 +532,7 @@
         elif a == '--builtin':
             opt_builtin = True
         elif a in ('--quiet', '-q'):
-            be_quiet()
+            trace.be_quiet()
         else:
             argv_copy.append(a)
         i += 1
@@ -574,7 +586,7 @@
         return ret or 0
     finally:
         # reset, in case we may do other commands later within the same process
-        be_quiet(False)
+        trace.be_quiet(False)
 
 def display_command(func):
     """Decorator that suppresses pipe/interrupt errors."""
@@ -584,7 +596,7 @@
             sys.stdout.flush()
             return result
         except IOError, e:
-            if not hasattr(e, 'errno'):
+            if getattr(e, 'errno', None) is None:
                 raise
             if e.errno != errno.EPIPE:
                 # Win32 raises IOError with errno=0 on a broken pipe
@@ -602,7 +614,7 @@
     bzrlib.ui.ui_factory = TextUIFactory()
     argv = [a.decode(bzrlib.user_encoding) for a in argv[1:]]
     ret = run_bzr_catch_errors(argv)
-    mutter("return code %d", ret)
+    trace.mutter("return code %d", ret)
     return ret
 
 
@@ -611,10 +623,10 @@
         return run_bzr(argv)
         # do this here inside the exception wrappers to catch EPIPE
         sys.stdout.flush()
-    except Exception, e:
+    except (KeyboardInterrupt, Exception), e:
         # used to handle AssertionError and KeyboardInterrupt
         # specially here, but hopefully they're handled ok by the logger now
-        bzrlib.trace.report_exception(sys.exc_info(), sys.stderr)
+        trace.report_exception(sys.exc_info(), sys.stderr)
         if os.environ.get('BZR_PDB'):
             print '**** entering debugger'
             import pdb

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-08-22 10:50:58 +0000
+++ bzrlib/commit.py	2006-10-15 15:55:32 +0000
@@ -61,6 +61,8 @@
 
 # TODO: If commit fails, leave the message in a file somewhere.
 
+# TODO: Change the parameter 'rev_id' to 'revision_id' to be consistent with
+# the rest of the code; add a deprecation of the old name.
 
 import os
 import re
@@ -69,8 +71,11 @@
 
 from cStringIO import StringIO
 
+from bzrlib import (
+    errors,
+    tree,
+    )
 import bzrlib.config
-import bzrlib.errors as errors
 from bzrlib.errors import (BzrError, PointlessCommit,
                            ConflictsInTree,
                            StrictCommitFailed
@@ -81,7 +86,7 @@
 from bzrlib.testament import Testament
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
-from bzrlib.inventory import Inventory, ROOT_ID, InventoryEntry
+from bzrlib.inventory import Inventory, InventoryEntry
 from bzrlib import symbol_versioning
 from bzrlib.symbol_versioning import (deprecated_passed,
         deprecated_function,
@@ -121,6 +126,8 @@
     def snapshot_change(self, change, path):
         if change == 'unchanged':
             return
+        if change == 'added' and path == '':
+            return
         note("%s %s", change, path)
 
     def completed(self, revno, rev_id):
@@ -245,11 +252,17 @@
             self._check_bound_branch()
 
             # check for out of date working trees
-            # if we are bound, then self.branch is the master branch and this
-            # test is thus all we need.
+            try:
+                first_tree_parent = self.work_tree.get_parent_ids()[0]
+            except IndexError:
+                # if there are no parents, treat our parent as 'None'
+                # this is so that we still consier the master branch
+                # - in a checkout scenario the tree may have no
+                # parents but the branch may do.
+                first_tree_parent = None
             master_last = self.master_branch.last_revision()
-            if (master_last is not None and 
-                master_last != self.work_tree.last_revision()):
+            if (master_last is not None and
+                master_last != first_tree_parent):
                 raise errors.OutOfDateTree(self.work_tree)
     
             if strict:
@@ -269,6 +282,11 @@
             self.work_inv = self.work_tree.inventory
             self.basis_tree = self.work_tree.basis_tree()
             self.basis_inv = self.basis_tree.inventory
+            if specific_files is not None:
+                # Ensure specified files are versioned
+                # (We don't actually need the ids here)
+                tree.find_ids_across_trees(specific_files, 
+                                           [self.basis_tree, self.work_tree])
             # one to finish, one for rev and inventory, and one for each
             # inventory entry, and the same for the new inventory.
             # note that this estimate is too long when we do a partial tree
@@ -289,10 +307,7 @@
             self._populate_new_inv()
             self._report_deletes()
 
-            if not (self.allow_pointless
-                    or len(self.parents) > 1
-                    or self.builder.new_inventory != self.basis_inv):
-                raise PointlessCommit()
+            self._check_pointless()
 
             self._emit_progress_update()
             # TODO: Now the new inventory is known, check for conflicts and
@@ -319,11 +334,8 @@
             # and now do the commit locally.
             self.branch.append_revision(self.rev_id)
 
-            # if the builder gave us the revisiontree it created back, we
-            # could use it straight away here.
-            # TODO: implement this.
-            self.work_tree.set_parent_trees([(self.rev_id,
-                self.branch.repository.revision_tree(self.rev_id))])
+            rev_tree = self.builder.revision_tree()
+            self.work_tree.set_parent_trees([(self.rev_id, rev_tree)])
             # now the work tree is up to date with the branch
             
             self.reporter.completed(self.branch.revno(), self.rev_id)
@@ -340,6 +352,60 @@
             self._cleanup()
         return self.rev_id
 
+    def _any_real_changes(self):
+        """Are there real changes between new_inventory and basis?
+
+        For trees without rich roots, inv.root.revision changes every commit.
+        But if that is the only change, we want to treat it as though there
+        are *no* changes.
+        """
+        new_entries = self.builder.new_inventory.iter_entries()
+        basis_entries = self.basis_inv.iter_entries()
+        new_path, new_root_ie = new_entries.next()
+        basis_path, basis_root_ie = basis_entries.next()
+
+        # This is a copy of InventoryEntry.__eq__ only leaving out .revision
+        def ie_equal_no_revision(this, other):
+            return ((this.file_id == other.file_id)
+                    and (this.name == other.name)
+                    and (this.symlink_target == other.symlink_target)
+                    and (this.text_sha1 == other.text_sha1)
+                    and (this.text_size == other.text_size)
+                    and (this.text_id == other.text_id)
+                    and (this.parent_id == other.parent_id)
+                    and (this.kind == other.kind)
+                    and (this.executable == other.executable)
+                    )
+        if not ie_equal_no_revision(new_root_ie, basis_root_ie):
+            return True
+
+        for new_ie, basis_ie in zip(new_entries, basis_entries):
+            if new_ie != basis_ie:
+                return True
+
+        # No actual changes present
+        return False
+
+    def _check_pointless(self):
+        if self.allow_pointless:
+            return
+        # A merge with no effect on files
+        if len(self.parents) > 1:
+            return
+        # work around the fact that a newly-initted tree does differ from its
+        # basis
+        if len(self.basis_inv) == 0 and len(self.builder.new_inventory) == 1:
+            raise PointlessCommit()
+        # Shortcut, if the number of entries changes, then we obviously have
+        # a change
+        if len(self.builder.new_inventory) != len(self.basis_inv):
+            return
+        # If length == 1, then we only have the root entry. Which means
+        # that there is no real difference (only the root could be different)
+        if (len(self.builder.new_inventory) != 1 and self._any_real_changes()):
+            return
+        raise PointlessCommit()
+
     def _check_bound_branch(self):
         """Check to see if the local branch is bound.
 
@@ -462,17 +528,18 @@
         """
         specific = self.specific_files
         deleted_ids = []
+        deleted_paths = set()
         for path, ie in self.work_inv.iter_entries():
+            if is_inside_any(deleted_paths, path):
+                # The tree will delete the required ids recursively.
+                continue
             if specific and not is_inside_any(specific, path):
                 continue
             if not self.work_tree.has_filename(path):
+                deleted_paths.add(path)
                 self.reporter.missing(path)
-                deleted_ids.append((path, ie.file_id))
-        if deleted_ids:
-            deleted_ids.sort(reverse=True)
-            for path, file_id in deleted_ids:
-                del self.work_inv[file_id]
-            self.work_tree._write_inventory(self.work_inv)
+                deleted_ids.append(ie.file_id)
+        self.work_tree.unversion(deleted_ids)
 
     def _populate_new_inv(self):
         """Build revision inventory.
@@ -489,6 +556,7 @@
         # in bugs like #46635.  Any reason not to use/enhance Tree.changes_from?
         # ADHB 11-07-2006
         mutter("Selecting files for commit with filter %s", self.specific_files)
+        assert self.work_inv.root is not None
         entries = self.work_inv.iter_entries()
         if not self.builder.record_root_entry:
             symbol_versioning.warn('CommitBuilders should support recording'
@@ -513,7 +581,6 @@
                 else:
                     # this entry is new and not being committed
                     continue
-
             self.builder.record_entry_contents(ie, self.parent_invs, 
                 path, self.work_tree)
             # describe the nature of the change that has occurred relative to

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2006-08-06 23:35:09 +0000
+++ bzrlib/config.py	2006-10-12 01:07:42 +0000
@@ -61,19 +61,26 @@
 up=pull
 """
 
+import os
+import sys
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import errno
 from fnmatch import fnmatch
-import os
 import re
-import sys
 from StringIO import StringIO
 
 import bzrlib
-from bzrlib import errors, urlutils
-from bzrlib.osutils import pathjoin
+from bzrlib import (
+    errors,
+    osutils,
+    urlutils,
+    )
+import bzrlib.util.configobj.configobj as configobj
+""")
+
 from bzrlib.trace import mutter, warning
-import bzrlib.util.configobj.configobj as configobj
 
 
 CHECK_IF_POSSIBLE=0
@@ -255,6 +262,18 @@
             raise errors.ParseConfigError(e.errors, e.config.filename)
         return self._parser
 
+    def _get_matching_sections(self):
+        """Return an ordered list of (section_name, extra_path) pairs.
+
+        If the section contains inherited configuration, extra_path is
+        a string containing the additional path components.
+        """
+        section = self._get_section()
+        if section is not None:
+            return [(section, '')]
+        else:
+            return []
+
     def _get_section(self):
         """Override this to define the section used by the config."""
         return "DEFAULT"
@@ -277,11 +296,13 @@
 
     def _get_user_option(self, option_name):
         """See Config._get_user_option."""
-        try:
-            return self._get_parser().get_value(self._get_section(),
-                                                option_name)
-        except KeyError:
-            pass
+        for (section, extra_path) in self._get_matching_sections():
+            try:
+                return self._get_parser().get_value(section, option_name)
+            except KeyError:
+                pass
+        else:
+            return None
 
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
@@ -379,13 +400,8 @@
             location = urlutils.local_path_from_url(location)
         self.location = location
 
-    def _get_section(self):
-        """Get the section we should look in for config items.
-
-        Returns None if none exists. 
-        TODO: perhaps return a NullSection that thunks through to the 
-              global config.
-        """
+    def _get_matching_sections(self):
+        """Return an ordered list of section names matching this location."""
         sections = self._get_parser()
         location_names = self.location.split('/')
         if self.location.endswith('/'):
@@ -422,11 +438,19 @@
                         continue
                 except KeyError:
                     pass
-            matches.append((len(section_names), section))
-        if not len(matches):
-            return None
+            matches.append((len(section_names), section,
+                            '/'.join(location_names[len(section_names):])))
         matches.sort(reverse=True)
-        return matches[0][1]
+        sections = []
+        for (length, section, extra_path) in matches:
+            sections.append((section, extra_path))
+            # should we stop looking for parent configs here?
+            try:
+                if self._get_parser()[section].as_bool('ignore_parents'):
+                    break
+            except KeyError:
+                pass
+        return sections
 
     def set_user_option(self, option, value):
         """Save option and its value in the configuration."""
@@ -599,32 +623,32 @@
             base = os.environ.get('HOME', None)
         if base is None:
             raise errors.BzrError('You must have one of BZR_HOME, APPDATA, or HOME set')
-        return pathjoin(base, 'bazaar', '2.0')
+        return osutils.pathjoin(base, 'bazaar', '2.0')
     else:
         # cygwin, linux, and darwin all have a $HOME directory
         if base is None:
             base = os.path.expanduser("~")
-        return pathjoin(base, ".bazaar")
+        return osutils.pathjoin(base, ".bazaar")
 
 
 def config_filename():
     """Return per-user configuration ini file filename."""
-    return pathjoin(config_dir(), 'bazaar.conf')
+    return osutils.pathjoin(config_dir(), 'bazaar.conf')
 
 
 def branches_config_filename():
     """Return per-user configuration ini file filename."""
-    return pathjoin(config_dir(), 'branches.conf')
+    return osutils.pathjoin(config_dir(), 'branches.conf')
 
 
 def locations_config_filename():
     """Return per-user configuration ini file filename."""
-    return pathjoin(config_dir(), 'locations.conf')
+    return osutils.pathjoin(config_dir(), 'locations.conf')
 
 
 def user_ignore_config_filename():
     """Return the user default ignore filename"""
-    return pathjoin(config_dir(), 'ignore')
+    return osutils.pathjoin(config_dir(), 'ignore')
 
 
 def _auto_user_id():
@@ -698,8 +722,7 @@
     """
     m = re.search(r'[\w+.-]+@[\w+.-]+', e)
     if not m:
-        raise errors.BzrError("%r doesn't seem to contain "
-                              "a reasonable email address" % e)
+        raise errors.NoEmailInUsername(e)
     return m.group(0)
 
 

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2006-09-06 12:00:01 +0000
+++ bzrlib/conflicts.py	2006-10-06 07:36:16 +0000
@@ -20,20 +20,25 @@
 # point down
 
 import os
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import errno
 
-import bzrlib
-from bzrlib.commands import register_command
-from bzrlib.errors import BzrCommandError, NotConflicted, UnsupportedOperation
+from bzrlib import (
+    commands,
+    errors,
+    osutils,
+    rio,
+    )
+""")
 from bzrlib.option import Option
-from bzrlib.osutils import rename, delete_any
-from bzrlib.rio import Stanza
 
 
 CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
 
 
-class cmd_conflicts(bzrlib.commands.Command):
+class cmd_conflicts(commands.Command):
     """List files with conflicts.
 
     Merge will do its best to combine the changes in two branches, but there
@@ -55,7 +60,7 @@
             print conflict
 
 
-class cmd_resolve(bzrlib.commands.Command):
+class cmd_resolve(commands.Command):
     """Mark a conflict as resolved.
 
     Merge will do its best to combine the changes in two branches, but there
@@ -76,19 +81,21 @@
         from bzrlib.workingtree import WorkingTree
         if all:
             if file_list:
-                raise BzrCommandError("If --all is specified, no FILE may be provided")
+                raise errors.BzrCommandError("If --all is specified,"
+                                             " no FILE may be provided")
             tree = WorkingTree.open_containing('.')[0]
             resolve(tree)
         else:
             if file_list is None:
-                raise BzrCommandError("command 'resolve' needs one or more FILE, or --all")
+                raise errors.BzrCommandError("command 'resolve' needs one or"
+                                             " more FILE, or --all")
             tree = WorkingTree.open_containing(file_list[0])[0]
             to_resolve = [tree.relpath(p) for p in file_list]
             resolve(tree, to_resolve)
 
 
 def resolve(tree, paths=None, ignore_misses=False):
-    tree.lock_write()
+    tree.lock_tree_write()
     try:
         tree_conflicts = tree.conflicts()
         if paths is None:
@@ -99,7 +106,7 @@
                 tree_conflicts.select_conflicts(tree, paths, ignore_misses)
         try:
             tree.set_conflicts(new_conflicts)
-        except UnsupportedOperation:
+        except errors.UnsupportedOperation:
             pass
         selected_conflicts.remove_files(tree)
     finally:
@@ -113,7 +120,7 @@
     """
     conflicted = False
     try:
-        rename(filename + ".THIS", filename)
+        osutils.rename(filename + ".THIS", filename)
         conflicted = True
     except OSError, e:
         if e.errno != errno.ENOENT:
@@ -131,7 +138,7 @@
         if e.errno != errno.ENOENT:
             raise
     if not conflicted:
-        raise NotConflicted(filename)
+        raise errors.NotConflicted(filename)
 
 
 class ConflictList(object):
@@ -198,7 +205,7 @@
                 continue
             for suffix in CONFLICT_SUFFIXES:
                 try:
-                    delete_any(tree.abspath(conflict.path+suffix))
+                    osutils.delete_any(tree.abspath(conflict.path+suffix))
                 except OSError, e:
                     if e.errno != errno.ENOENT:
                         raise
@@ -261,7 +268,7 @@
         self.file_id = file_id
 
     def as_stanza(self):
-        s = Stanza(type=self.typestring, path=self.path)
+        s = rio.Stanza(type=self.typestring, path=self.path)
         if self.file_id is not None:
             s.add('file_id', self.file_id)
         return s

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2006-08-15 22:56:53 +0000
+++ bzrlib/delta.py	2006-10-12 03:19:14 +0000
@@ -152,13 +152,14 @@
         want_unchanged=want_unchanged,
         specific_files=specific_files,
         extra_trees=extra_trees,
-        require_versioned=require_versioned)
-
-
-def _compare_trees(old_tree, new_tree, want_unchanged, specific_file_ids):
+        require_versioned=require_versioned,
+        include_root=False)
+
+
+def _compare_trees(old_tree, new_tree, want_unchanged, specific_file_ids,
+                   include_root):
 
     from osutils import is_inside_any
-    
     old_inv = old_tree.inventory
     new_inv = new_tree.inventory
     delta = TreeDelta()
@@ -167,8 +168,8 @@
     # TODO: Rather than iterating over the whole tree and then filtering, we
     # could diff just the specified files (if any) and their subtrees.  
 
-    old_files = old_tree.list_files()
-    new_files = new_tree.list_files()
+    old_files = old_tree.list_files(include_root)
+    new_files = new_tree.list_files(include_root)
 
     more_old = True
     more_new = True
@@ -251,7 +252,7 @@
             assert entry.file_id not in added
             added[entry.file_id] = path, entry
 
-    while old_path or new_path:
+    while old_path is not None or new_path is not None:
         # list_files() returns files in alphabetical path sorted order
         if old_path == new_path:
             if old_file_id == new_file_id:

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-08-25 20:43:30 +0000
+++ bzrlib/diff.py	2006-09-12 22:16:38 +0000
@@ -14,25 +14,32 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import errno
 import os
 import re
+import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import errno
 import subprocess
-import sys
 import tempfile
 import time
 
+from bzrlib import (
+    errors,
+    osutils,
+    patiencediff,
+    textfile,
+    )
+""")
+
 # compatability - plugins import compare_trees from diff!!!
 # deprecated as of 0.10
 from bzrlib.delta import compare_trees
-from bzrlib.errors import BzrError
-import bzrlib.errors as errors
-import bzrlib.osutils
-from bzrlib.patiencediff import unified_diff
-import bzrlib.patiencediff
-from bzrlib.symbol_versioning import (deprecated_function,
-        zero_eight)
-from bzrlib.textfile import check_text_lines
+from bzrlib.symbol_versioning import (
+        deprecated_function,
+        zero_eight,
+        )
 from bzrlib.trace import mutter, warning
 
 
@@ -60,12 +67,12 @@
         return
     
     if allow_binary is False:
-        check_text_lines(oldlines)
-        check_text_lines(newlines)
+        textfile.check_text_lines(oldlines)
+        textfile.check_text_lines(newlines)
 
     if sequence_matcher is None:
-        sequence_matcher = bzrlib.patiencediff.PatienceSequenceMatcher
-    ud = unified_diff(oldlines, newlines,
+        sequence_matcher = patiencediff.PatienceSequenceMatcher
+    ud = patiencediff.unified_diff(oldlines, newlines,
                       fromfile=old_filename.encode(path_encoding),
                       tofile=new_filename.encode(path_encoding),
                       sequencematcher=sequence_matcher)
@@ -90,7 +97,10 @@
 
 def _set_lang_C():
     """Set the env var LANG=C"""
-    os.environ['LANG'] = 'C'
+    osutils.set_or_unset_env('LANG', 'C')
+    osutils.set_or_unset_env('LC_ALL', None)
+    osutils.set_or_unset_env('LC_CTYPE', None)
+    osutils.set_or_unset_env('LANGUAGE', None)
 
 
 def _spawn_external_diff(diffcmd, capture_errors=True):
@@ -103,7 +113,12 @@
     :return: A Popen object.
     """
     if capture_errors:
-        preexec_fn = _set_lang_C
+        if sys.platform == 'win32':
+            # Win32 doesn't support preexec_fn, but that is
+            # okay, because it doesn't support LANG either.
+            preexec_fn = None
+        else:
+            preexec_fn = _set_lang_C
         stderr = subprocess.PIPE
     else:
         preexec_fn = None
@@ -202,7 +217,8 @@
             # Write out the new i18n diff response
             to_file.write(out+'\n')
             if pipe.returncode != 2:
-                raise BzrError('external diff failed with exit code 2'
+                raise errors.BzrError(
+                               'external diff failed with exit code 2'
                                ' when run with LANG=C, but not when run'
                                ' natively: %r' % (diffcmd,))
 
@@ -210,8 +226,8 @@
             # Starting with diffutils 2.8.4 the word "binary" was dropped.
             m = re.match('^(binary )?files.*differ$', first_line, re.I)
             if m is None:
-                raise BzrError('external diff failed with exit code 2;'
-                               ' command: %r' % (diffcmd,))
+                raise errors.BzrError('external diff failed with exit code 2;'
+                                      ' command: %r' % (diffcmd,))
             else:
                 # Binary files differ, just return
                 return
@@ -226,8 +242,8 @@
             else:
                 msg = 'exit code %d' % rc
                 
-            raise BzrError('external diff failed with %s; command: %r' 
-                           % (rc, diffcmd))
+            raise errors.BzrError('external diff failed with %s; command: %r' 
+                                  % (rc, diffcmd))
 
 
     finally:

=== modified file 'bzrlib/doc/api/branch.txt'
--- bzrlib/doc/api/branch.txt	2006-06-30 21:22:07 +0000
+++ bzrlib/doc/api/branch.txt	2006-10-05 22:42:09 +0000
@@ -4,8 +4,8 @@
 
 Make a temporary directory for these tests:
 
-   >>> from bzrlib.osutils import mkdtemp
-   >>> test_dir = mkdtemp(prefix='bzrlib_doc_api_branch_txt_')
+   >>> from bzrlib import osutils
+   >>> test_dir = osutils.mkdtemp(prefix='bzrlib_doc_api_branch_txt_')
 
 Branches are created by BzrDir's:
 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-09-04 08:00:48 +0000
+++ bzrlib/errors.py	2006-10-16 01:13:59 +0000
@@ -44,7 +44,7 @@
 >>> try:
 ...   raise NotBranchError(path='/foo/bar')
 ... except:
-...   print sys.exc_type
+...   print '%s.%s' % (sys.exc_type.__module__, sys.exc_type.__name__)
 ...   print sys.exc_value
 ...   path = getattr(sys.exc_value, 'path', None)
 ...   if path is not None:
@@ -96,7 +96,7 @@
 class BzrError(StandardError):
     
     is_user_error = True
-    
+
     def __str__(self):
         # XXX: Should we show the exception class in 
         # exceptions that don't provide their own message?  
@@ -120,7 +120,11 @@
     # base classes should override the docstring with their human-
     # readable explanation
 
-    def __init__(self, **kwds):
+    def __init__(self, *args, **kwds):
+        # XXX: Use the underlying BzrError to always generate the args attribute
+        # if it doesn't exist.  We can't use super here, because exceptions are
+        # old-style classes in python2.4 (but new in 2.5).  --bmc, 20060426
+        BzrError.__init__(self, *args)
         for key, value in kwds.items():
             setattr(self, key, value)
 
@@ -132,9 +136,14 @@
             if isinstance(s, unicode):
                 return s.encode('utf8')
             return s
-        except (NameError, ValueError, KeyError), e:
-            return 'Unprintable exception %s: %s' \
-                % (self.__class__.__name__, str(e))
+        except (TypeError, NameError, ValueError, KeyError), e:
+            return 'Unprintable exception %s(%r): %s' \
+                % (self.__class__.__name__,
+                   self.__dict__, str(e))
+
+
+class AlreadyBuilding(BzrNewError):
+    """The tree builder is already building a tree."""
 
 
 class BzrCheckError(BzrNewError):
@@ -174,6 +183,23 @@
         self.branch = branch
 
 
+class InventoryModified(BzrNewError):
+    """The current inventory for the tree %(tree)r has been modified, so a clean inventory cannot be read without data loss."""
+
+    def __init__(self, tree):
+        BzrNewError.__init__(self)
+        self.tree = tree
+
+
+class NoSuchId(BzrNewError):
+    """The file id %(file_id)s is not present in the tree %(tree)s."""
+    
+    def __init__(self, tree, file_id):
+        BzrNewError.__init__(self)
+        self.file_id = file_id
+        self.tree = tree
+
+
 class NoWorkingTree(BzrNewError):
     """No WorkingTree exists for %(base)s."""
     
@@ -182,6 +208,10 @@
         self.base = base
 
 
+class NotBuilding(BzrNewError):
+    """Not currently building a tree."""
+
+
 class NotLocalUrl(BzrNewError):
     """%(url)s is not a local path."""
     
@@ -190,6 +220,14 @@
         self.url = url
 
 
+class NotWriteLocked(BzrNewError):
+    """%(not_locked)r is not write locked but needs to be."""
+
+    def __init__(self, not_locked):
+        BzrNewError.__init__(self)
+        self.not_locked = not_locked
+
+
 class BzrCommandError(BzrNewError):
     """Error from user command"""
 
@@ -248,6 +286,16 @@
     """Directory not empty: %(path)r%(extra)s"""
 
 
+class ReadingCompleted(BzrNewError):
+    """The MediumRequest '%(request)s' has already had finish_reading called upon it - the request has been completed and no more data may be read."""
+
+    is_user_error = False
+
+    def __init__(self, request):
+        BzrNewError.__init__(self)
+        self.request = request
+
+
 class ResourceBusy(PathError):
     """Device or resource busy: %(path)r%(extra)s"""
 
@@ -265,8 +313,7 @@
 
     def __init__(self, msg, base, args):
         PathError.__init__(self, base, msg)
-        self.args = [base]
-        self.args.extend(args)
+        self.args = [base] + list(args)
 
 
 class UnsupportedProtocol(PathError):
@@ -276,6 +323,18 @@
         PathError.__init__(self, url, extra=extra)
 
 
+class ShortReadvError(PathError):
+    """readv() read %(actual)s bytes rather than %(length)s bytes at %(offset)s for %(path)s%(extra)s"""
+
+    is_user_error = False
+
+    def __init__(self, path, offset, length, actual, extra=None):
+        PathError.__init__(self, path, extra=extra)
+        self.offset = offset
+        self.length = length
+        self.actual = actual
+
+
 class PathNotChild(BzrNewError):
     """Path %(path)r is not a child of path %(base)r%(extra)s"""
 
@@ -365,6 +424,14 @@
         self.bzrdir = bzrdir_format
 
 
+class IncompatibleRevision(BzrNewError):
+    """Revision is not compatible with %(repo_format)s"""
+
+    def __init__(self, repo_format):
+        BzrNewError.__init__(self)
+        self.repo_format = repo_format
+
+
 class NotVersionedError(BzrNewError):
     """%(path)s is not versioned"""
     def __init__(self, path):
@@ -761,11 +828,21 @@
 
 class NoSuchExportFormat(BzrNewError):
     """Export format %(format)r not supported"""
+
     def __init__(self, format):
         BzrNewError.__init__(self)
         self.format = format
 
 
+
+class TooManyConcurrentRequests(BzrNewError):
+    """The medium '%(medium)s' has reached its concurrent request limit. Be sure to finish_writing and finish_reading on the current request that is open."""
+
+    def __init__(self, medium):
+        BzrNewError.__init__(self)
+        self.medium = medium
+
+
 class TransportError(BzrNewError):
     """Transport error: %(msg)s %(orig_error)s"""
 
@@ -781,9 +858,16 @@
         BzrNewError.__init__(self)
 
 
+class SmartProtocolError(TransportError):
+    """Generic bzr smart protocol error: %(details)s"""
+
+    def __init__(self, details):
+        self.details = details
+
+
 # A set of semi-meaningful errors which can be thrown
 class TransportNotPossible(TransportError):
-    """Transport operation not possible: %(msg)s %(orig_error)%"""
+    """Transport operation not possible: %(msg)s %(orig_error)s"""
 
 
 class ConnectionError(TransportError):
@@ -842,6 +926,14 @@
         BzrError.__init__(self, message)
 
 
+class NoEmailInUsername(BzrNewError):
+    """%(username)r does not seem to contain a reasonable email address"""
+
+    def __init__(self, username):
+        BzrNewError.__init__(self)
+        self.username = username
+
+
 class SigningFailed(BzrError):
     def __init__(self, command_line):
         BzrError.__init__(self, "Failed to gpg sign data with command '%s'"
@@ -855,6 +947,26 @@
                           " unchanged." % tree.basedir)
 
 
+class WritingCompleted(BzrNewError):
+    """The MediumRequest '%(request)s' has already had finish_writing called upon it - accept bytes may not be called anymore."""
+
+    is_user_error = False
+
+    def __init__(self, request):
+        BzrNewError.__init__(self)
+        self.request = request
+
+
+class WritingNotComplete(BzrNewError):
+    """The MediumRequest '%(request)s' has not has finish_writing called upon it - until the write phase is complete no data may be read."""
+
+    is_user_error = False
+
+    def __init__(self, request):
+        BzrNewError.__init__(self)
+        self.request = request
+
+
 class CantReprocessAndShowBase(BzrNewError):
     """Can't reprocess and show base.
 Reprocessing obscures relationship of conflicting lines to base."""
@@ -875,6 +987,14 @@
         self.filename = filename
 
 
+class MediumNotConnected(BzrNewError):
+    """The medium '%(medium)s' is not connected."""
+
+    def __init__(self, medium):
+        BzrNewError.__init__(self)
+        self.medium = medium
+
+
 class MustUseDecorated(Exception):
     """A decorating function has requested its original command be used.
     
@@ -916,6 +1036,18 @@
     """Tree transform is malformed %(conflicts)r"""
 
 
+class NoFinalPath(BzrNewError):
+    """No final name for trans_id %(trans_id)r
+    file-id: %(file_id)r"
+    root trans-id: %(root_trans_id)r 
+    """
+
+    def __init__(self, trans_id, transform):
+        self.trans_id = trans_id
+        self.file_id = transform.final_file_id(trans_id)
+        self.root_trans_id = transform.root
+
+
 class BzrBadParameter(BzrNewError):
     """A bad parameter : %(param)s is not usable.
     
@@ -981,6 +1113,15 @@
         self.format = format
 
 
+class BadConversionTarget(BzrNewError):
+    """Cannot convert to format %(format)s.  %(problem)s"""
+
+    def __init__(self, problem, format):
+        BzrNewError.__init__(self)
+        self.problem = problem
+        self.format = format
+
+
 class NoDiff(BzrNewError):
     """Diff is not installed on this machine: %(msg)s"""
 
@@ -1139,6 +1280,41 @@
         BzrNewError.__init__(self)
 
 
+class IncompatibleFormat(BzrNewError):
+    """Bundle format %(bundle_format)s is incompatible with %(other)s"""
+
+    def __init__(self, bundle_format, other):
+        BzrNewError.__init__(self)
+        self.bundle_format = bundle_format
+        self.other = other
+
+
+class BadInventoryFormat(BzrNewError):
+    """Root class for inventory serialization errors"""
+
+
+class UnexpectedInventoryFormat(BadInventoryFormat):
+    """The inventory was not in the expected format:\n %(msg)s"""
+
+    def __init__(self, msg):
+        BadInventoryFormat.__init__(self, msg=msg)
+
+
+class NoSmartMedium(BzrNewError):
+    """The transport '%(transport)s' cannot tunnel the smart protocol."""
+
+    def __init__(self, transport):
+        BzrNewError.__init__(self)
+        self.transport = transport
+
+
+class NoSmartServer(NotBranchError):
+    """No smart server available at %(url)s"""
+
+    def __init__(self, url):
+        self.url = url
+
+
 class UnknownSSH(BzrNewError):
     """Unrecognised value for BZR_SSH environment variable: %(vendor)s"""
 
@@ -1153,3 +1329,39 @@
     def __init__(self, revision_id):
         BzrNewError.__init__(self)
         self.revision_id = revision_id
+
+
+class IllegalUseOfScopeReplacer(BzrNewError):
+    """ScopeReplacer object %(name)r was used incorrectly: %(msg)s%(extra)s"""
+
+    is_user_error = False
+
+    def __init__(self, name, msg, extra=None):
+        BzrNewError.__init__(self)
+        self.name = name
+        self.msg = msg
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
+
+
+class InvalidImportLine(BzrNewError):
+    """Not a valid import statement: %(msg)\n%(text)s"""
+
+    is_user_error = False
+
+    def __init__(self, text, msg):
+        BzrNewError.__init__(self)
+        self.text = text
+        self.msg = msg
+
+
+class ImportNameCollision(BzrNewError):
+    """Tried to import an object to the same name as an existing object. %(name)s"""
+
+    is_user_error = False
+
+    def __init__(self, name):
+        BzrNewError.__init__(self)
+        self.name = name

=== modified file 'bzrlib/export/__init__.py'
--- bzrlib/export/__init__.py	2006-08-29 20:47:02 +0000
+++ bzrlib/export/__init__.py	2006-09-20 23:43:18 +0000
@@ -98,6 +98,8 @@
 def get_root_name(dest):
     """Get just the root name for an export.
 
+    >>> get_root_name('../mytest.tar')
+    'mytest'
     >>> get_root_name('mytar.tar')
     'mytar'
     >>> get_root_name('mytar.tar.bz2')

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2006-07-28 23:53:09 +0000
+++ bzrlib/export/zip_exporter.py	2006-09-23 19:05:44 +0000
@@ -18,8 +18,22 @@
 """
 
 import os
+import stat
+import zipfile
+
+from bzrlib import (
+    osutils,
+    )
 from bzrlib.trace import mutter
-import zipfile
+
+
+# Windows expects this bit to be set in the 'external_attr' section
+# Or it won't consider the entry a directory
+ZIP_DIRECTORY_BIT = (1 << 4)
+
+_FILE_ATTR = stat.S_IFREG
+_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT
+
 
 def zip_exporter(tree, dest, root):
     """ Export this tree to a new zip file.
@@ -49,23 +63,32 @@
             file_id = ie.file_id
             mutter("  export {%s} kind %s to %s", file_id, ie.kind, dest)
 
-            if ie.kind == "file": 
+            # zipfile.ZipFile switches all paths to forward
+            # slashes anyway, so just stick with that.
+            filename = osutils.pathjoin(root, dp).encode('utf8')
+            if ie.kind == "file":
                 zinfo = zipfile.ZipInfo(
-                            filename=str(os.path.join(root, dp)),
+                            filename=filename,
                             date_time=now)
                 zinfo.compress_type = compression
+                zinfo.external_attr = _FILE_ATTR
                 zipf.writestr(zinfo, tree.get_file_text(file_id))
             elif ie.kind == "directory":
+                # Directories must contain a trailing slash, to indicate
+                # to the zip routine that they are really directories and
+                # not just empty files.
                 zinfo = zipfile.ZipInfo(
-                            filename=str(os.path.join(root, dp)+os.sep),
+                            filename=filename + '/',
                             date_time=now)
                 zinfo.compress_type = compression
+                zinfo.external_attr = _DIR_ATTR
                 zipf.writestr(zinfo,'')
             elif ie.kind == "symlink":
                 zinfo = zipfile.ZipInfo(
-                            filename=str(os.path.join(root, dp+".lnk")),
+                            filename=(filename + '.lnk'),
                             date_time=now)
                 zinfo.compress_type = compression
+                zinfo.external_attr = _FILE_ATTR
                 zipf.writestr(zinfo, ie.symlink_target)
 
         zipf.close()

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-07-28 16:05:23 +0000
+++ bzrlib/fetch.py	2006-09-10 02:18:31 +0000
@@ -147,7 +147,7 @@
         if self._last_revision is NULL_REVISION:
             # explicit limit of no revisions needed
             return None
-        if (self._last_revision != None and
+        if (self._last_revision is not None and
             self.to_repository.has_revision(self._last_revision)):
             return None
             
@@ -281,6 +281,107 @@
         to_rf.join(from_rf, version_ids=revs)
 
 
+class Inter1and2Helper(object):
+    """Helper for operations that convert data from model 1 and 2
+    
+    This is for use by fetchers and converters.
+    """
+
+    def __init__(self, source, target):
+        """Constructor.
+
+        :param source: The repository data comes from
+        :param target: The repository data goes to
+        """
+        self.source = source
+        self.target = target
+
+    def iter_rev_trees(self, revs):
+        """Iterate through RevisionTrees efficiently.
+
+        Additionally, the inventory's revision_id is set if unset.
+
+        Trees are retrieved in batches of 100, and then yielded in the order
+        they were requested.
+
+        :param revs: A list of revision ids
+        """
+        while revs:
+            for tree in self.source.revision_trees(revs[:100]):
+                if tree.inventory.revision_id is None:
+                    tree.inventory.revision_id = tree.get_revision_id()
+                yield tree
+            revs = revs[100:]
+
+    def generate_root_texts(self, revs):
+        """Generate VersionedFiles for all root ids.
+        
+        :param revs: the revisions to include
+        """
+        inventory_weave = self.source.get_inventory_weave()
+        parent_texts = {}
+        versionedfile = {}
+        to_store = self.target.weave_store
+        for tree in self.iter_rev_trees(revs):
+            revision_id = tree.inventory.root.revision
+            root_id = tree.inventory.root.file_id
+            parents = inventory_weave.get_parents(revision_id)
+            if root_id not in versionedfile:
+                versionedfile[root_id] = to_store.get_weave_or_empty(root_id, 
+                    self.target.get_transaction())
+            parent_texts[root_id] = versionedfile[root_id].add_lines(
+                revision_id, parents, [], parent_texts)
+
+    def regenerate_inventory(self, revs):
+        """Generate a new inventory versionedfile in target, convertin data.
+        
+        The inventory is retrieved from the source, (deserializing it), and
+        stored in the target (reserializing it in a different format).
+        :param revs: The revisions to include
+        """
+        inventory_weave = self.source.get_inventory_weave()
+        for tree in self.iter_rev_trees(revs):
+            parents = inventory_weave.get_parents(tree.get_revision_id())
+            self.target.add_inventory(tree.get_revision_id(), tree.inventory,
+                                      parents)
+
+
+class Model1toKnit2Fetcher(GenericRepoFetcher):
+    """Fetch from a Model1 repository into a Knit2 repository
+    """
+    def __init__(self, to_repository, from_repository, last_revision=None, 
+                 pb=None):
+        self.helper = Inter1and2Helper(from_repository, to_repository)
+        GenericRepoFetcher.__init__(self, to_repository, from_repository,
+                                    last_revision, pb)
+
+    def _fetch_weave_texts(self, revs):
+        GenericRepoFetcher._fetch_weave_texts(self, revs)
+        # Now generate a weave for the tree root
+        self.helper.generate_root_texts(revs)
+
+    def _fetch_inventory_weave(self, revs):
+        self.helper.regenerate_inventory(revs)
+ 
+
+class Knit1to2Fetcher(KnitRepoFetcher):
+    """Fetch from a Knit1 repository into a Knit2 repository"""
+
+    def __init__(self, to_repository, from_repository, last_revision=None, 
+                 pb=None):
+        self.helper = Inter1and2Helper(from_repository, to_repository)
+        KnitRepoFetcher.__init__(self, to_repository, from_repository,
+                                 last_revision, pb)
+
+    def _fetch_weave_texts(self, revs):
+        KnitRepoFetcher._fetch_weave_texts(self, revs)
+        # Now generate a weave for the tree root
+        self.helper.generate_root_texts(revs)
+
+    def _fetch_inventory_weave(self, revs):
+        self.helper.regenerate_inventory(revs)
+        
+
 class Fetcher(object):
     """Backwards compatibility glue for branch.fetch()."""
 

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2006-08-25 14:07:28 +0000
+++ bzrlib/gpg.py	2006-09-12 20:02:29 +0000
@@ -17,8 +17,12 @@
 
 """GPG signing and checking logic."""
 
+import os
+import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import errno
-import os
 import subprocess
 
 from bzrlib import (
@@ -26,6 +30,7 @@
     trace,
     ui,
     )
+""")
 
 
 class DisabledGPGStrategy(object):
@@ -72,11 +77,16 @@
 
     def sign(self, content):
         ui.ui_factory.clear_term()
+
+        preexec_fn = _set_gpg_tty
+        if sys.platform == 'win32':
+            # Win32 doesn't support preexec_fn, but wouldn't support TTY anyway.
+            preexec_fn = None
         try:
             process = subprocess.Popen(self._command_line(),
                                        stdin=subprocess.PIPE,
                                        stdout=subprocess.PIPE,
-                                       preexec_fn=_set_gpg_tty)
+                                       preexec_fn=preexec_fn)
             try:
                 result = process.communicate(content)[0]
                 if process.returncode is None:

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2006-08-02 13:02:47 +0000
+++ bzrlib/help.py	2006-09-25 21:54:28 +0000
@@ -21,6 +21,7 @@
 
 # TODO: `help commands --all` should show hidden commands
 import textwrap
+from bzrlib import osutils 
 
 global_help = \
 """Bazaar -- a free distributed version-control tool
@@ -53,9 +54,9 @@
 
 
 def help(topic=None, outfile = None):
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
-    if topic == None:
+    if topic is None:
         outfile.write(global_help)
     elif topic == 'commands':
         help_commands(outfile = outfile)
@@ -102,13 +103,13 @@
 
     cmdname = str(cmdname)
 
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
 
     cmd_object = get_cmd_object(cmdname)
 
     doc = cmd_object.help()
-    if doc == None:
+    if doc is None:
         raise NotImplementedError("sorry, no detailed help yet for %r" % cmdname)
 
     print >>outfile, 'usage:', command_usage(cmd_object) 
@@ -141,27 +142,29 @@
     from bzrlib.commands import (builtin_command_names,
                                  plugin_command_names,
                                  get_cmd_object)
-
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
-
-    names = set()                       # to eliminate duplicates
-    names.update(builtin_command_names())
+    names = set(builtin_command_names()) # to eliminate duplicates
     names.update(plugin_command_names())
-    names = list(names)
-    names.sort()
-
-    for cmd_name in names:
-        cmd_object = get_cmd_object(cmd_name)
-        if cmd_object.hidden:
-            continue
-        print >>outfile, command_usage(cmd_object)
-
+    commands = ((n, get_cmd_object(n)) for n in names)
+    shown_commands = [(n, o) for n, o in commands if not o.hidden]
+    max_name = max(len(n) for n, o in shown_commands)
+    indent = ' ' * (max_name + 1)
+    width = osutils.terminal_width() - 1
+    for cmd_name, cmd_object in sorted(shown_commands):
         plugin_name = cmd_object.plugin_name()
-        print_command_plugin(cmd_object, outfile, '        %s\n')
+        if plugin_name is None:
+            plugin_name = ''
+        else:
+            plugin_name = ' [%s]' % plugin_name
 
         cmd_help = cmd_object.help()
         if cmd_help:
             firstline = cmd_help.split('\n', 1)[0]
-            print >>outfile, '        ' + firstline
-        
+        else:
+            firstline = ''
+        helpstring = '%-*s %s%s' % (max_name, cmd_name, firstline, plugin_name)
+        lines = textwrap.wrap(helpstring, subsequent_indent=indent,
+                              width=width)
+        for line in lines:
+            outfile.write(line + '\n')

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2006-08-08 00:11:35 +0000
+++ bzrlib/info.py	2006-09-10 19:04:48 +0000
@@ -31,7 +31,7 @@
 def plural(n, base='', pl=None):
     if n == 1:
         return base
-    elif pl != None:
+    elif pl is not None:
         return pl
     else:
         return 's'
@@ -173,7 +173,10 @@
     work_inv = working.inventory
     delta = working.changes_from(basis, want_unchanged=True)
     history = branch.revision_history()
-    tree_last_id = working.last_revision()
+    try:
+        tree_last_id = working.get_parent_ids()[0]
+    except IndexError:
+        tree_last_id = None
 
     if len(history) and tree_last_id != history[-1]:
         tree_last_revno = branch.revision_id_to_revno(tree_last_id)
@@ -207,9 +210,10 @@
     print '  %8d ignored' % ignore_cnt
 
     dir_cnt = 0
-    entries = work_inv.iter_entries()
-    entries.next()
-    dir_cnt = sum(1 for path, ie in entries if ie.kind == 'directory')
+    for file_id in work_inv:
+        if (work_inv.get_file_kind(file_id) == 'directory' and 
+            not work_inv.is_root(file_id)):
+            dir_cnt += 1
     print '  %8d versioned %s' \
           % (dir_cnt,
              plural(dir_cnt, 'subdirectory', 'subdirectories'))

=== added file 'bzrlib/inspect_for_copy.py'
--- bzrlib/inspect_for_copy.py	1970-01-01 00:00:00 +0000
+++ bzrlib/inspect_for_copy.py	2006-10-06 05:53:44 +0000
@@ -0,0 +1,74 @@
+# Copyright (C) 2005, 2006 Canonical Development Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A version of inspect that includes what 'copy' needs.
+
+Importing the python standard module 'copy' is far more expensive than it
+needs to be, because copy imports 'inspect' which imports 'tokenize'.
+And 'copy' only needs 2 small functions out of 'inspect', but has to
+load all of 'tokenize', which makes it horribly slow.
+
+This module is designed to use tricky hacks in import rules, to avoid this
+overhead.
+"""
+
+
+####
+# These are the only 2 functions that 'copy' needs from 'inspect'
+# As you can see, they are quite trivial, and don't justify the
+# 40ms spent to import 'inspect' because it is importing 'tokenize'
+# These are copied verbatim from the python standard library.
+
+# ----------------------------------------------------------- class helpers
+def _searchbases(cls, accum):
+    # Simulate the "classic class" search order.
+    if cls in accum:
+        return
+    accum.append(cls)
+    for base in cls.__bases__:
+        _searchbases(base, accum)
+
+
+def getmro(cls):
+    "Return tuple of base classes (including cls) in method resolution order."
+    if hasattr(cls, "__mro__"):
+        return cls.__mro__
+    else:
+        result = []
+        _searchbases(cls, result)
+        return tuple(result)
+
+
+def import_copy_with_hacked_inspect():
+    """Import the 'copy' module with a hacked 'inspect' module"""
+    # We don't actually care about 'getmro' but we need to pass
+    # something in the list so that we get the direct module,
+    # rather than getting the base module
+    import sys
+
+    # Don't hack around if 'inspect' already exists
+    if 'inspect' in sys.modules:
+        import copy
+        return
+
+    mod = __import__('bzrlib.inspect_for_copy',
+                     globals(), locals(), ['getmro'])
+
+    sys.modules['inspect'] = mod
+    try:
+        import copy
+    finally:
+        del sys.modules['inspect']

=== modified file 'bzrlib/inter.py'
--- bzrlib/inter.py	2006-07-29 08:46:23 +0000
+++ bzrlib/inter.py	2006-08-22 16:14:17 +0000
@@ -36,8 +36,8 @@
     the needs_read_lock and needs_write_lock decorators.)
     """
 
-    # _optimisers = set()
-    # Each concrete InterObject type should have its own optimisers set.
+    # _optimisers = list()
+    # Each concrete InterObject type should have its own optimisers list.
 
     def __init__(self, source, target):
         """Construct a default InterObject instance. Please use 'get'.
@@ -74,7 +74,7 @@
         If an optimised worker exists it will be used otherwise
         a default Inter worker instance will be created.
         """
-        for provider in klass._optimisers:
+        for provider in reversed(klass._optimisers):
             if provider.is_compatible(source, target):
                 return provider(source, target)
         return klass(source, target)
@@ -98,7 +98,7 @@
     @classmethod
     def register_optimiser(klass, optimiser):
         """Register an InterObject optimiser."""
-        klass._optimisers.add(optimiser)
+        klass._optimisers.append(optimiser)
 
     def unlock(self):
         """Release the locks on source and target."""

=== modified file 'bzrlib/intset.py'
--- bzrlib/intset.py	2006-07-28 16:05:23 +0000
+++ bzrlib/intset.py	2006-09-08 18:46:29 +0000
@@ -71,7 +71,7 @@
             If specified, an initial collection of values.
         """
         self._val = bitmask
-        if values != None:
+        if values is not None:
             self.update(values)
 
 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-08-30 13:54:51 +0000
+++ bzrlib/inventory.py	2006-10-12 01:50:46 +0000
@@ -27,21 +27,27 @@
 # created, but it's not for now.
 ROOT_ID = "TREE_ROOT"
 
-
-import collections
-import os.path
+import os
 import re
 import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import collections
 import tarfile
-import types
-from warnings import warn
 
 import bzrlib
-from bzrlib import errors, osutils
-from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
-                            pathjoin, sha_strings)
-from bzrlib.errors import (NotVersionedError, InvalidEntryName,
-                           BzrError, BzrCheckError, BinaryFile)
+from bzrlib import (
+    errors,
+    osutils,
+    symbol_versioning,
+    )
+""")
+
+from bzrlib.errors import (
+    BzrCheckError,
+    BzrError,
+    )
 from bzrlib.trace import mutter
 
 
@@ -82,7 +88,7 @@
     InventoryDirectory('123', 'src', parent_id='TREE_ROOT', revision=None)
     >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
     InventoryFile('2323', 'hello.c', parent_id='123', sha1=None, len=None)
-    >>> shouldbe = {0: '', 1: 'src', 2: pathjoin('src','hello.c')}
+    >>> shouldbe = {0: '', 1: 'src', 2: 'src/hello.c'}
     >>> for ix, j in enumerate(i.iter_entries()):
     ...   print (j[0] == shouldbe[ix], j[1])
     ... 
@@ -246,7 +252,7 @@
 
     def get_tar_item(self, root, dp, now, tree):
         """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(pathjoin(root, dp))
+        item = tarfile.TarInfo(osutils.pathjoin(root, dp).encode('utf8'))
         # TODO: would be cool to actually set it to the timestamp of the
         # revision it was last changed
         item.mtime = now
@@ -281,7 +287,7 @@
         """
         assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
-            raise InvalidEntryName(name=name)
+            raise errors.InvalidEntryName(name=name)
         self.executable = False
         self.revision = None
         self.text_sha1 = None
@@ -311,7 +317,7 @@
         
         This is a template method - implement _put_on_disk in subclasses.
         """
-        fullpath = pathjoin(dest, dp)
+        fullpath = osutils.pathjoin(dest, dp)
         self._put_on_disk(fullpath, tree)
         # mutter("  export {%s} kind %s to %s", self.file_id,
         #         self.kind, fullpath)
@@ -514,9 +520,9 @@
         self.parent_id = None
         self.name = u''
         self.revision = None
-        warn('RootEntry is deprecated as of bzr 0.10.  Please use '
-             'InventoryDirectory instead.',
-            DeprecationWarning, stacklevel=2)
+        symbol_versioning.warn('RootEntry is deprecated as of bzr 0.10.'
+                               '  Please use InventoryDirectory instead.',
+                               DeprecationWarning, stacklevel=2)
 
     def __eq__(self, other):
         if not isinstance(other, RootEntry):
@@ -645,7 +651,7 @@
             else:
                 text_diff(to_label, to_text,
                           from_label, from_text, output_to)
-        except BinaryFile:
+        except errors.BinaryFile:
             if reverse:
                 label_pair = (to_label, from_label)
             else:
@@ -677,7 +683,7 @@
 
     def _put_on_disk(self, fullpath, tree):
         """See InventoryEntry._put_on_disk."""
-        pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
+        osutils.pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
         if tree.is_executable(self.file_id):
             os.chmod(fullpath, 0755)
 
@@ -849,6 +855,9 @@
     ['', u'hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    Traceback (most recent call last):
+    BzrError: parent_id {TREE_ROOT} not in inventory
+    >>> inv.add(InventoryFile('123-123', 'hello.c', 'TREE_ROOT-12345678-12345678'))
     InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678', sha1=None, len=None)
     """
     def __init__(self, root_id=ROOT_ID, revision_id=None):
@@ -861,17 +870,11 @@
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch.create() create a unique inventory
-        # root id. Rather than generating a random one here.
-        #if root_id is None:
-        #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
         if root_id is not None:
             self._set_root(InventoryDirectory(root_id, '', None))
         else:
             self.root = None
             self._byid = {}
-        # FIXME: this isn't ever used, changing it to self.revision may break
-        # things. TODO make everything use self.revision_id
         self.revision_id = revision_id
 
     def _set_root(self, ie):
@@ -898,7 +901,8 @@
     def iter_entries(self, from_dir=None):
         """Return (path, entry) pairs, in order by name."""
         if from_dir is None:
-            assert self.root
+            if self.root is None:
+                return
             from_dir = self.root
             yield '', self.root
         elif isinstance(from_dir, basestring):
@@ -951,7 +955,8 @@
         # TODO? Perhaps this should return the from_dir so that the root is
         # yielded? or maybe an option?
         if from_dir is None:
-            assert self.root
+            if self.root is None:
+                return
             from_dir = self.root
             yield '', self.root
         elif isinstance(from_dir, basestring):
@@ -982,7 +987,7 @@
             kids = dir_ie.children.items()
             kids.sort()
             for name, ie in kids:
-                child_path = pathjoin(dir_path, name)
+                child_path = osutils.pathjoin(dir_path, name)
                 accum.append((child_path, ie))
                 if ie.kind == 'directory':
                     descend(ie, child_path)
@@ -1001,7 +1006,7 @@
             kids.sort()
 
             for name, child_ie in kids:
-                child_path = pathjoin(parent_path, name)
+                child_path = osutils.pathjoin(parent_path, name)
                 descend(child_ie, child_path)
         descend(self.root, u'')
         return accum
@@ -1057,10 +1062,6 @@
             assert self.root is None and len(self._byid) == 0
             self._set_root(entry)
             return entry
-        if entry.parent_id == ROOT_ID:
-            assert self.root is not None, self
-            entry.parent_id = self.root.file_id
-
         try:
             parent = self._byid[entry.parent_id]
         except KeyError:
@@ -1068,7 +1069,7 @@
 
         if entry.name in parent.children:
             raise BzrError("%s is already versioned" %
-                    pathjoin(self.id2path(parent.file_id), entry.name))
+                    osutils.pathjoin(self.id2path(parent.file_id), entry.name))
 
         self._byid[entry.file_id] = entry
         parent.children[entry.name] = entry
@@ -1093,7 +1094,7 @@
             parent_path = parts[:-1]
             parent_id = self.path2id(parent_path)
             if parent_id is None:
-                raise NotVersionedError(path=parent_path)
+                raise errors.NotVersionedError(path=parent_path)
         ie = make_entry(kind, parts[-1], parent_id, file_id)
         return self.add(ie)
 
@@ -1193,12 +1194,14 @@
 
         Returns None IFF the path is not found.
         """
-        if isinstance(name, types.StringTypes):
-            name = splitpath(name)
+        if isinstance(name, basestring):
+            name = osutils.splitpath(name)
 
         # mutter("lookup path %r" % name)
 
         parent = self.root
+        if parent is None:
+            return None
         for f in name:
             try:
                 cie = parent.children[f]
@@ -1217,6 +1220,24 @@
     def has_id(self, file_id):
         return (file_id in self._byid)
 
+    def remove_recursive_id(self, file_id):
+        """Remove file_id, and children, from the inventory.
+        
+        :param file_id: A file_id to remove.
+        """
+        to_find_delete = [self._byid[file_id]]
+        to_delete = []
+        while to_find_delete:
+            ie = to_find_delete.pop()
+            to_delete.append(ie.file_id)
+            if ie.kind == 'directory':
+                to_find_delete.extend(ie.children.values())
+        for file_id in reversed(to_delete):
+            ie = self[file_id]
+            del self._byid[file_id]
+            if ie.parent_id is not None:
+                del self[ie.parent_id].children[ie.name]
+
     def rename(self, file_id, new_parent_id, new_name):
         """Move a file within the inventory.
 
@@ -1246,6 +1267,9 @@
         file_ie.name = new_name
         file_ie.parent_id = new_parent_id
 
+    def is_root(self, file_id):
+        return self.root is not None and file_id == self.root.file_id
+
 
 def make_entry(kind, name, parent_id, file_id=None):
     """Create an inventory entry.

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-09-06 15:16:39 +0000
+++ bzrlib/knit.py	2006-09-27 20:27:46 +0000
@@ -77,6 +77,7 @@
 from bzrlib import (
     cache_utf8,
     errors,
+    progress,
     )
 from bzrlib.errors import FileExists, NoSuchFile, KnitError, \
         InvalidRevisionId, KnitCorrupt, KnitHeaderError, \
@@ -279,12 +280,18 @@
     stored and retrieved.
     """
 
-    def __init__(self, relpath, transport, file_mode=None, access_mode=None, 
+    def __init__(self, relpath, transport, file_mode=None, access_mode=None,
                  factory=None, basis_knit=DEPRECATED_PARAMETER, delta=True,
-                 create=False):
+                 create=False, create_parent_dir=False, delay_create=False,
+                 dir_mode=None):
         """Construct a knit at location specified by relpath.
         
         :param create: If not True, only open an existing knit.
+        :param create_parent_dir: If True, create the parent directory if 
+            creating the file fails. (This is used for stores with 
+            hash-prefixes that may not exist yet)
+        :param delay_create: The calling code is aware that the knit won't 
+            actually be created until the first data is stored.
         """
         if deprecated_passed(basis_knit):
             warnings.warn("KnitVersionedFile.__(): The basis_knit parameter is"
@@ -301,9 +308,13 @@
         self.delta = delta
 
         self._index = _KnitIndex(transport, relpath + INDEX_SUFFIX,
-            access_mode, create=create, file_mode=file_mode)
+            access_mode, create=create, file_mode=file_mode,
+            create_parent_dir=create_parent_dir, delay_create=delay_create,
+            dir_mode=dir_mode)
         self._data = _KnitData(transport, relpath + DATA_SUFFIX,
-            access_mode, create=create and not len(self), file_mode=file_mode)
+            access_mode, create=create and not len(self), file_mode=file_mode,
+            create_parent_dir=create_parent_dir, delay_create=delay_create,
+            dir_mode=dir_mode)
 
     def __repr__(self):
         return '%s(%s)' % (self.__class__.__name__, 
@@ -788,10 +799,13 @@
             text_map[version_id] = text 
         return text_map, final_content 
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+    def iter_lines_added_or_present_in_versions(self, version_ids=None, 
+                                                pb=None):
         """See VersionedFile.iter_lines_added_or_present_in_versions()."""
         if version_ids is None:
             version_ids = self.versions()
+        if pb is None:
+            pb = progress.DummyProgress()
         # we don't care about inclusions, the caller cares.
         # but we need to setup a list of records to visit.
         # we need version_id, position, length
@@ -809,33 +823,26 @@
                 data_pos, length = self._index.get_position(version_id)
                 version_id_records.append((version_id, data_pos, length))
 
-        pb = bzrlib.ui.ui_factory.nested_progress_bar()
         count = 0
         total = len(version_id_records)
-        try:
+        pb.update('Walking content.', count, total)
+        for version_id, data, sha_value in \
+            self._data.read_records_iter(version_id_records):
             pb.update('Walking content.', count, total)
-            for version_id, data, sha_value in \
-                self._data.read_records_iter(version_id_records):
-                pb.update('Walking content.', count, total)
-                method = self._index.get_method(version_id)
-                version_idx = self._index.lookup(version_id)
-                assert method in ('fulltext', 'line-delta')
-                if method == 'fulltext':
-                    content = self.factory.parse_fulltext(data, version_idx)
-                    for line in content.text():
+            method = self._index.get_method(version_id)
+            version_idx = self._index.lookup(version_id)
+            assert method in ('fulltext', 'line-delta')
+            if method == 'fulltext':
+                content = self.factory.parse_fulltext(data, version_idx)
+                for line in content.text():
+                    yield line
+            else:
+                delta = self.factory.parse_line_delta(data, version_idx)
+                for start, end, count, lines in delta:
+                    for origin, line in lines:
                         yield line
-                else:
-                    delta = self.factory.parse_line_delta(data, version_idx)
-                    for start, end, count, lines in delta:
-                        for origin, line in lines:
-                            yield line
-                count +=1
-            pb.update('Walking content.', total, total)
-            pb.finished()
-        except:
-            pb.update('Walking content.', total, total)
-            pb.finished()
-            raise
+            count +=1
+        pb.update('Walking content.', total, total)
         
     def num_versions(self):
         """See VersionedFile.num_versions()."""
@@ -948,16 +955,15 @@
 class _KnitComponentFile(object):
     """One of the files used to implement a knit database"""
 
-    def __init__(self, transport, filename, mode, file_mode=None):
+    def __init__(self, transport, filename, mode, file_mode=None,
+                 create_parent_dir=False, dir_mode=None):
         self._transport = transport
         self._filename = filename
         self._mode = mode
-        self._file_mode=file_mode
-
-    def write_header(self):
-        if self._transport.put_bytes_non_atomic(self._filename, self.HEADER,
-                mode=self._file_mode):
-            raise KnitCorrupt(self._filename, 'misaligned after writing header')
+        self._file_mode = file_mode
+        self._dir_mode = dir_mode
+        self._create_parent_dir = create_parent_dir
+        self._need_to_create = False
 
     def check_header(self, fp):
         line = fp.readline()
@@ -1050,8 +1056,12 @@
                                    parents,
                                    index)
 
-    def __init__(self, transport, filename, mode, create=False, file_mode=None):
-        _KnitComponentFile.__init__(self, transport, filename, mode, file_mode)
+    def __init__(self, transport, filename, mode, create=False, file_mode=None,
+                 create_parent_dir=False, delay_create=False, dir_mode=None):
+        _KnitComponentFile.__init__(self, transport, filename, mode,
+                                    file_mode=file_mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
         self._cache = {}
         # position in _history is the 'official' index for a revision
         # but the values may have come from a newer entry.
@@ -1126,7 +1136,12 @@
             except NoSuchFile, e:
                 if mode != 'w' or not create:
                     raise
-                self.write_header()
+                if delay_create:
+                    self._need_to_create = True
+                else:
+                    self._transport.put_bytes_non_atomic(self._filename,
+                        self.HEADER, mode=self._file_mode)
+
         finally:
             pb.update('read knit index', total, total)
             pb.finished()
@@ -1247,7 +1262,19 @@
             assert isinstance(line, str), \
                 'content must be utf-8 encoded: %r' % (line,)
             lines.append(line)
-        self._transport.append_bytes(self._filename, ''.join(lines))
+        if not self._need_to_create:
+            self._transport.append_bytes(self._filename, ''.join(lines))
+        else:
+            sio = StringIO()
+            sio.write(self.HEADER)
+            sio.writelines(lines)
+            sio.seek(0)
+            self._transport.put_file_non_atomic(self._filename, sio,
+                                create_parent_dir=self._create_parent_dir,
+                                mode=self._file_mode,
+                                dir_mode=self._dir_mode)
+            self._need_to_create = False
+
         # cache after writing, so that a failed write leads to missing cache
         # entries not extra ones. XXX TODO: RBC 20060502 in the event of a 
         # failure, reload the index or flush it or some such, to prevent
@@ -1298,10 +1325,13 @@
 class _KnitData(_KnitComponentFile):
     """Contents of the knit data file"""
 
-    HEADER = "# bzr knit data 8\n"
-
-    def __init__(self, transport, filename, mode, create=False, file_mode=None):
-        _KnitComponentFile.__init__(self, transport, filename, mode)
+    def __init__(self, transport, filename, mode, create=False, file_mode=None,
+                 create_parent_dir=False, delay_create=False,
+                 dir_mode=None):
+        _KnitComponentFile.__init__(self, transport, filename, mode,
+                                    file_mode=file_mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
         self._checked = False
         # TODO: jam 20060713 conceptually, this could spill to disk
         #       if the cached size gets larger than a certain amount
@@ -1310,8 +1340,11 @@
         self._cache = {}
         self._do_cache = False
         if create:
-            self._transport.put_bytes_non_atomic(self._filename, '',
-                                                 mode=file_mode)
+            if delay_create:
+                self._need_to_create = create
+            else:
+                self._transport.put_bytes_non_atomic(self._filename, '',
+                                                     mode=self._file_mode)
 
     def enable_cache(self):
         """Enable caching of reads."""
@@ -1356,14 +1389,30 @@
         :return: the offset in the data file raw_data was written.
         """
         assert isinstance(raw_data, str), 'data must be plain bytes'
-        return self._transport.append_bytes(self._filename, raw_data)
+        if not self._need_to_create:
+            return self._transport.append_bytes(self._filename, raw_data)
+        else:
+            self._transport.put_bytes_non_atomic(self._filename, raw_data,
+                                   create_parent_dir=self._create_parent_dir,
+                                   mode=self._file_mode,
+                                   dir_mode=self._dir_mode)
+            self._need_to_create = False
+            return 0
         
     def add_record(self, version_id, digest, lines):
         """Write new text record to disk.  Returns the position in the
         file where it was written."""
         size, sio = self._record_to_data(version_id, digest, lines)
         # write to disk
-        start_pos = self._transport.append_file(self._filename, sio)
+        if not self._need_to_create:
+            start_pos = self._transport.append_file(self._filename, sio)
+        else:
+            self._transport.put_file_non_atomic(self._filename, sio,
+                               create_parent_dir=self._create_parent_dir,
+                               mode=self._file_mode,
+                               dir_mode=self._dir_mode)
+            self._need_to_create = False
+            start_pos = 0
         if self._do_cache:
             self._cache[version_id] = sio.getvalue()
         return start_pos, size

=== added file 'bzrlib/lazy_import.py'
--- bzrlib/lazy_import.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lazy_import.py	2006-10-06 05:53:44 +0000
@@ -0,0 +1,381 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Functionality to create lazy evaluation objects.
+
+This includes waiting to import a module until it is actually used.
+
+Most commonly, the 'lazy_import' function is used to import other modules
+in an on-demand fashion. Typically use looks like:
+    from bzrlib.lazy_import import lazy_import
+    lazy_import(globals(), '''
+    from bzrlib import (
+        errors,
+        osutils,
+        branch,
+        )
+    import bzrlib.branch
+    ''')
+
+    Then 'errors, osutils, branch' and 'bzrlib' will exist as lazy-loaded
+    objects which will be replaced with a real object on first use.
+
+    In general, it is best to only load modules in this way. This is because
+    it isn't safe to pass these variables to other functions before they
+    have been replaced. This is especially true for constants, sometimes
+    true for classes or functions (when used as a factory, or you want
+    to inherit from them).
+"""
+
+
+class ScopeReplacer(object):
+    """A lazy object that will replace itself in the appropriate scope.
+
+    This object sits, ready to create the real object the first time it is
+    needed.
+    """
+
+    __slots__ = ('_scope', '_factory', '_name')
+
+    def __init__(self, scope, factory, name):
+        """Create a temporary object in the specified scope.
+        Once used, a real object will be placed in the scope.
+
+        :param scope: The scope the object should appear in
+        :param factory: A callable that will create the real object.
+            It will be passed (self, scope, name)
+        :param name: The variable name in the given scope.
+        """
+        self._scope = scope
+        self._factory = factory
+        self._name = name
+        scope[name] = self
+
+    def _replace(self):
+        """Actually replace self with other in the given scope"""
+        name = object.__getattribute__(self, '_name')
+        try:
+            factory = object.__getattribute__(self, '_factory')
+            scope = object.__getattribute__(self, '_scope')
+        except AttributeError, e:
+            # Because ScopeReplacer objects only replace a single
+            # item, passing them to another variable before they are
+            # replaced would cause them to keep getting replaced
+            # (only they are replacing the wrong variable). So we
+            # make it forbidden, and try to give a good error.
+            raise errors.IllegalUseOfScopeReplacer(
+                name, msg="Object already cleaned up, did you assign it"
+                          " to another variable?",
+                extra=e)
+        obj = factory(self, scope, name)
+        scope[name] = obj
+        return obj
+
+    def _cleanup(self):
+        """Stop holding on to all the extra stuff"""
+        del self._factory
+        del self._scope
+        # We keep _name, so that we can report errors
+        # del self._name
+
+    def __getattribute__(self, attr):
+        _replace = object.__getattribute__(self, '_replace')
+        obj = _replace()
+        _cleanup = object.__getattribute__(self, '_cleanup')
+        _cleanup()
+        return getattr(obj, attr)
+
+    def __call__(self, *args, **kwargs):
+        _replace = object.__getattribute__(self, '_replace')
+        obj = _replace()
+        _cleanup = object.__getattribute__(self, '_cleanup')
+        _cleanup()
+        return obj(*args, **kwargs)
+
+
+class ImportReplacer(ScopeReplacer):
+    """This is designed to replace only a portion of an import list.
+
+    It will replace itself with a module, and then make children
+    entries also ImportReplacer objects.
+
+    At present, this only supports 'import foo.bar.baz' syntax.
+    """
+
+    # '_import_replacer_children' is intentionally a long semi-unique name
+    # that won't likely exist elsewhere. This allows us to detect an
+    # ImportReplacer object by using
+    #       object.__getattribute__(obj, '_import_replacer_children')
+    # We can't just use 'isinstance(obj, ImportReplacer)', because that
+    # accesses .__class__, which goes through __getattribute__, and triggers
+    # the replacement.
+    __slots__ = ('_import_replacer_children', '_member', '_module_path')
+
+    def __init__(self, scope, name, module_path, member=None, children={}):
+        """Upon request import 'module_path' as the name 'module_name'.
+        When imported, prepare children to also be imported.
+
+        :param scope: The scope that objects should be imported into.
+            Typically this is globals()
+        :param name: The variable name. Often this is the same as the 
+            module_path. 'bzrlib'
+        :param module_path: A list for the fully specified module path
+            ['bzrlib', 'foo', 'bar']
+        :param member: The member inside the module to import, often this is
+            None, indicating the module is being imported.
+        :param children: Children entries to be imported later.
+            This should be a map of children specifications.
+            {'foo':(['bzrlib', 'foo'], None, 
+                {'bar':(['bzrlib', 'foo', 'bar'], None {})})
+            }
+        Examples:
+            import foo => name='foo' module_path='foo',
+                          member=None, children={}
+            import foo.bar => name='foo' module_path='foo', member=None,
+                              children={'bar':(['foo', 'bar'], None, {}}
+            from foo import bar => name='bar' module_path='foo', member='bar'
+                                   children={}
+            from foo import bar, baz would get translated into 2 import
+            requests. On for 'name=bar' and one for 'name=baz'
+        """
+        if member is not None:
+            assert not children, \
+                'Cannot supply both a member and children'
+
+        self._import_replacer_children = children
+        self._member = member
+        self._module_path = module_path
+
+        # Indirecting through __class__ so that children can
+        # override _import (especially our instrumented version)
+        cls = object.__getattribute__(self, '__class__')
+        ScopeReplacer.__init__(self, scope=scope, name=name,
+                               factory=cls._import)
+
+    def _import(self, scope, name):
+        children = object.__getattribute__(self, '_import_replacer_children')
+        member = object.__getattribute__(self, '_member')
+        module_path = object.__getattribute__(self, '_module_path')
+        module_python_path = '.'.join(module_path)
+        if member is not None:
+            module = __import__(module_python_path, scope, scope, [member])
+            return getattr(module, member)
+        else:
+            module = __import__(module_python_path, scope, scope, [])
+            for path in module_path[1:]:
+                module = getattr(module, path)
+
+        # Prepare the children to be imported
+        for child_name, (child_path, child_member, grandchildren) in \
+                children.iteritems():
+            # Using self.__class__, so that children get children classes
+            # instantiated. (This helps with instrumented tests)
+            cls = object.__getattribute__(self, '__class__')
+            cls(module.__dict__, name=child_name,
+                module_path=child_path, member=child_member,
+                children=grandchildren)
+        return module
+
+
+class ImportProcessor(object):
+    """Convert text that users input into lazy import requests"""
+
+    # TODO: jam 20060912 This class is probably not strict enough about
+    #       what type of text it allows. For example, you can do:
+    #       import (foo, bar), which is not allowed by python.
+    #       For now, it should be supporting a superset of python import
+    #       syntax which is all we really care about.
+
+    __slots__ = ['imports', '_lazy_import_class']
+
+    def __init__(self, lazy_import_class=None):
+        self.imports = {}
+        if lazy_import_class is None:
+            self._lazy_import_class = ImportReplacer
+        else:
+            self._lazy_import_class = lazy_import_class
+
+    def lazy_import(self, scope, text):
+        """Convert the given text into a bunch of lazy import objects.
+
+        This takes a text string, which should be similar to normal python
+        import markup.
+        """
+        self._build_map(text)
+        self._convert_imports(scope)
+
+    def _convert_imports(self, scope):
+        # Now convert the map into a set of imports
+        for name, info in self.imports.iteritems():
+            self._lazy_import_class(scope, name=name, module_path=info[0],
+                                    member=info[1], children=info[2])
+
+    def _build_map(self, text):
+        """Take a string describing imports, and build up the internal map"""
+        for line in self._canonicalize_import_text(text):
+            if line.startswith('import '):
+                self._convert_import_str(line)
+            elif line.startswith('from '):
+                self._convert_from_str(line)
+            else:
+                raise errors.InvalidImportLine(line,
+                    "doesn't start with 'import ' or 'from '")
+
+    def _convert_import_str(self, import_str):
+        """This converts a import string into an import map.
+
+        This only understands 'import foo, foo.bar, foo.bar.baz as bing'
+
+        :param import_str: The import string to process
+        """
+        assert import_str.startswith('import ')
+        import_str = import_str[len('import '):]
+
+        for path in import_str.split(','):
+            path = path.strip()
+            if not path:
+                continue
+            as_hunks = path.split(' as ')
+            if len(as_hunks) == 2:
+                # We have 'as' so this is a different style of import
+                # 'import foo.bar.baz as bing' creates a local variable
+                # named 'bing' which points to 'foo.bar.baz'
+                name = as_hunks[1].strip()
+                module_path = as_hunks[0].strip().split('.')
+                if name in self.imports:
+                    raise errors.ImportNameCollision(name)
+                # No children available in 'import foo as bar'
+                self.imports[name] = (module_path, None, {})
+            else:
+                # Now we need to handle
+                module_path = path.split('.')
+                name = module_path[0]
+                if name not in self.imports:
+                    # This is a new import that we haven't seen before
+                    module_def = ([name], None, {})
+                    self.imports[name] = module_def
+                else:
+                    module_def = self.imports[name]
+
+                cur_path = [name]
+                cur = module_def[2]
+                for child in module_path[1:]:
+                    cur_path.append(child)
+                    if child in cur:
+                        cur = cur[child][2]
+                    else:
+                        next = (cur_path[:], None, {})
+                        cur[child] = next
+                        cur = next[2]
+
+    def _convert_from_str(self, from_str):
+        """This converts a 'from foo import bar' string into an import map.
+
+        :param from_str: The import string to process
+        """
+        assert from_str.startswith('from ')
+        from_str = from_str[len('from '):]
+
+        from_module, import_list = from_str.split(' import ')
+
+        from_module_path = from_module.split('.')
+
+        for path in import_list.split(','):
+            path = path.strip()
+            if not path:
+                continue
+            as_hunks = path.split(' as ')
+            if len(as_hunks) == 2:
+                # We have 'as' so this is a different style of import
+                # 'import foo.bar.baz as bing' creates a local variable
+                # named 'bing' which points to 'foo.bar.baz'
+                name = as_hunks[1].strip()
+                module = as_hunks[0].strip()
+            else:
+                name = module = path
+            if name in self.imports:
+                raise errors.ImportNameCollision(name)
+            self.imports[name] = (from_module_path, module, {})
+
+    def _canonicalize_import_text(self, text):
+        """Take a list of imports, and split it into regularized form.
+
+        This is meant to take regular import text, and convert it to
+        the forms that the rest of the converters prefer.
+        """
+        out = []
+        cur = None
+        continuing = False
+
+        for line in text.split('\n'):
+            line = line.strip()
+            loc = line.find('#')
+            if loc != -1:
+                line = line[:loc].strip()
+
+            if not line:
+                continue
+            if cur is not None:
+                if line.endswith(')'):
+                    out.append(cur + ' ' + line[:-1])
+                    cur = None
+                else:
+                    cur += ' ' + line
+            else:
+                if '(' in line and ')' not in line:
+                    cur = line.replace('(', '')
+                else:
+                    out.append(line.replace('(', '').replace(')', ''))
+        if cur is not None:
+            raise errors.InvalidImportLine(cur, 'Unmatched parenthesis')
+        return out
+
+
+def lazy_import(scope, text, lazy_import_class=None):
+    """Create lazy imports for all of the imports in text.
+
+    This is typically used as something like:
+    from bzrlib.lazy_import import lazy_import
+    lazy_import(globals(), '''
+    from bzrlib import (
+        foo,
+        bar,
+        baz,
+        )
+    import bzrlib.branch
+    import bzrlib.transport
+    ''')
+
+    Then 'foo, bar, baz' and 'bzrlib' will exist as lazy-loaded
+    objects which will be replaced with a real object on first use.
+
+    In general, it is best to only load modules in this way. This is
+    because other objects (functions/classes/variables) are frequently
+    used without accessing a member, which means we cannot tell they
+    have been used.
+    """
+    # This is just a helper around ImportProcessor.lazy_import
+    proc = ImportProcessor(lazy_import_class=lazy_import_class)
+    return proc.lazy_import(scope, text)
+
+
+# The only module that this module depends on is 'bzrlib.errors'. But it
+# can actually be imported lazily, since we only need it if there is a
+# problem.
+
+lazy_import(globals(), """
+from bzrlib import errors
+""")

=== added file 'bzrlib/lazy_regex.py'
--- bzrlib/lazy_regex.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lazy_regex.py	2006-10-10 06:48:45 +0000
@@ -0,0 +1,106 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Lazily compiled regex objects.
+
+This module defines a class which creates proxy objects for regex compilation.
+This allows overriding re.compile() to return lazily compiled objects.
+"""
+
+import re
+
+
+class LazyRegex(object):
+    """A proxy around a real regex, which won't be compiled until accessed."""
+
+
+    # These are the parameters on a real _sre.SRE_Pattern object, which we
+    # will map to local members so that we don't have the proxy overhead.
+    _regex_attributes_to_copy = [
+                 '__copy__', '__deepcopy__', 'findall', 'finditer', 'match',
+                 'scanner', 'search', 'split', 'sub', 'subn'
+                 ]
+
+    # We use slots to keep the overhead low. But we need a slot entry for
+    # all of the attributes we will copy
+    __slots__ = ['_real_regex', '_regex_args', '_regex_kwargs',
+                ] + _regex_attributes_to_copy
+
+    def __init__(self, args=(), kwargs={}):
+        """Create a new proxy object, passing in the args to pass to re.compile
+
+        :param args: The *args to pass to re.compile
+        :param kwargs: The **kwargs to pass to re.compile
+        """
+        self._real_regex = None
+        self._regex_args = args
+        self._regex_kwargs = kwargs
+
+    def _compile_and_collapse(self):
+        """Actually compile the requested regex"""
+        self._real_regex = self._real_re_compile(*self._regex_args,
+                                                 **self._regex_kwargs)
+        for attr in self._regex_attributes_to_copy:
+            setattr(self, attr, getattr(self._real_regex, attr))
+
+    def _real_re_compile(self, *args, **kwargs):
+        """Thunk over to the original re.compile"""
+        return _real_re_compile(*args, **kwargs)
+
+    def __getattr__(self, attr):
+        """Return a member from the proxied regex object.
+
+        If the regex hasn't been compiled yet, compile it
+        """
+        if self._real_regex is None:
+            self._compile_and_collapse()
+        # Once we have compiled, the only time we should come here
+        # is actually if the attribute is missing.
+        return getattr(self._real_regex, attr)
+
+
+def lazy_compile(*args, **kwargs):
+    """Create a proxy object which will compile the regex on demand.
+
+    :return: a LazyRegex proxy object.
+    """
+    return LazyRegex(args, kwargs)
+
+
+def install_lazy_compile():
+    """Make lazy_compile the default compile mode for regex compilation.
+
+    This overrides re.compile with lazy_compile. To restore the original
+    functionality, call reset_compile().
+    """
+    re.compile = lazy_compile
+
+
+def reset_compile():
+    """Restore the original function to re.compile().
+    
+    It is safe to call reset_compile() multiple times, it will always
+    restore re.compile() to the value that existed at import time.
+    Though the first call will reset back to the original (it doesn't
+    track nesting level)
+    """
+    re.compile = _real_re_compile
+
+
+_real_re_compile = re.compile
+assert _real_re_compile is not lazy_compile, \
+    "re.compile has already been overridden as lazy_compile, but this would" \
+    " cause infinite recursion"

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-09-05 21:25:15 +0000
+++ bzrlib/lockdir.py	2006-09-29 20:24:37 +0000
@@ -96,8 +96,11 @@
 
 import os
 import time
-from StringIO import StringIO
+from cStringIO import StringIO
 
+from bzrlib import (
+    errors,
+    )
 import bzrlib.config
 from bzrlib.errors import (
         DirectoryNotEmpty,
@@ -111,10 +114,11 @@
         ResourceBusy,
         UnlockableTransport,
         )
-from bzrlib.trace import mutter
+from bzrlib.trace import mutter, note
 from bzrlib.transport import Transport
-from bzrlib.osutils import rand_chars
-from bzrlib.rio import RioWriter, read_stanza, Stanza
+from bzrlib.osutils import rand_chars, format_delta
+from bzrlib.rio import read_stanza, Stanza
+
 
 # XXX: At the moment there is no consideration of thread safety on LockDir
 # objects.  This should perhaps be updated - e.g. if two threads try to take a
@@ -130,8 +134,10 @@
 # TODO: Make sure to pass the right file and directory mode bits to all
 # files/dirs created.
 
+
 _DEFAULT_TIMEOUT_SECONDS = 300
-_DEFAULT_POLL_SECONDS = 0.5
+_DEFAULT_POLL_SECONDS = 1.0
+
 
 class LockDir(object):
     """Write-lock guarding access to data."""
@@ -160,6 +166,8 @@
         self._dir_modebits = dir_modebits
         self.nonce = rand_chars(20)
 
+        self._report_function = note
+
     def __repr__(self):
         return '%s(%s%s)' % (self.__class__.__name__,
                              self.transport.base,
@@ -200,14 +208,13 @@
                 # After creating the lock directory, try again
                 self.transport.mkdir(tmpname)
 
-            sio = StringIO()
-            self._prepare_info(sio)
-            sio.seek(0)
-            # append will create a new file; we use append rather than put
-            # because we don't want to write to a temporary file and rename
-            # into place, because that's going to happen to the whole
-            # directory
-            self.transport.append_file(tmpname + self.__INFO_NAME, sio)
+            info_bytes = self._prepare_info()
+            # We use put_file_non_atomic because we just created a new unique
+            # directory so we don't have to worry about files existing there.
+            # We'll rename the whole directory into place to get atomic
+            # properties
+            self.transport.put_bytes_non_atomic(tmpname + self.__INFO_NAME,
+                                                info_bytes)
 
             self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True
@@ -244,12 +251,8 @@
         self._check_not_locked()
         holder_info = self.peek()
         if holder_info is not None:
-            if bzrlib.ui.ui_factory.get_boolean(
-                "Break lock %s held by %s@%s [process #%s]" % (
-                    self.transport,
-                    holder_info["user"],
-                    holder_info["hostname"],
-                    holder_info["pid"])):
+            lock_info = '\n'.join(self._format_lock_info(holder_info))
+            if bzrlib.ui.ui_factory.get_boolean("Break %s" % lock_info):
                 self.force_break(holder_info)
         
     def force_break(self, dead_holder_info):
@@ -336,25 +339,28 @@
         except NoSuchFile, e:
             return None
 
-    def _prepare_info(self, outf):
+    def _prepare_info(self):
         """Write information about a pending lock to a temporary file.
         """
         import socket
         # XXX: is creating this here inefficient?
         config = bzrlib.config.GlobalConfig()
+        try:
+            user = config.user_email()
+        except errors.NoEmailInUsername:
+            user = config.username()
         s = Stanza(hostname=socket.gethostname(),
                    pid=str(os.getpid()),
                    start_time=str(int(time.time())),
                    nonce=self.nonce,
-                   user=config.user_email(),
+                   user=user,
                    )
-        RioWriter(outf).write_stanza(s)
+        return s.to_string()
 
     def _parse_info(self, info_file):
         return read_stanza(info_file.readlines()).as_dict()
 
-    def wait_lock(self, timeout=_DEFAULT_TIMEOUT_SECONDS,
-                  poll=_DEFAULT_POLL_SECONDS):
+    def wait_lock(self, timeout=None, poll=None):
         """Wait a certain period for a lock.
 
         If the lock can be acquired within the bounded time, it
@@ -363,15 +369,44 @@
         approximately `timeout` seconds.  (It may be a bit more if
         a transport operation takes a long time to complete.)
         """
+        if timeout is None:
+            timeout = _DEFAULT_TIMEOUT_SECONDS
+        if poll is None:
+            poll = _DEFAULT_POLL_SECONDS
+
         # XXX: the transport interface doesn't let us guard 
         # against operations there taking a long time.
         deadline = time.time() + timeout
+        deadline_str = None
+        last_info = None
         while True:
             try:
                 self.attempt_lock()
                 return
             except LockContention:
                 pass
+            new_info = self.peek()
+            mutter('last_info: %s, new info: %s', last_info, new_info)
+            if new_info is not None and new_info != last_info:
+                if last_info is None:
+                    start = 'Unable to obtain'
+                else:
+                    start = 'Lock owner changed for'
+                last_info = new_info
+                formatted_info = self._format_lock_info(new_info)
+                if deadline_str is None:
+                    deadline_str = time.strftime('%H:%M:%S',
+                                                 time.localtime(deadline))
+                self._report_function('%s %s\n'
+                                      '%s\n' # held by
+                                      '%s\n' # locked ... ago
+                                      'Will continue to try until %s\n',
+                                      start,
+                                      formatted_info[0],
+                                      formatted_info[1],
+                                      formatted_info[2],
+                                      deadline_str)
+
             if time.time() + poll < deadline:
                 time.sleep(poll)
             else:
@@ -379,7 +414,7 @@
 
     def lock_write(self):
         """Wait for and acquire the lock."""
-        self.attempt_lock()
+        self.wait_lock()
 
     def lock_read(self):
         """Compatibility-mode shared lock.
@@ -409,3 +444,13 @@
             else:
                 raise LockContention(self)
 
+    def _format_lock_info(self, info):
+        """Turn the contents of peek() into something for the user"""
+        lock_url = self.transport.abspath(self.path)
+        delta = time.time() - int(info['start_time'])
+        return [
+            'lock %s' % (lock_url,),
+            'held by %(user)s on host %(hostname)s [process #%(pid)s]' % info,
+            'locked %s' % (format_delta(delta),),
+            ]
+

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2006-09-07 04:28:05 +0000
+++ bzrlib/log.py	2006-10-16 09:14:47 +0000
@@ -386,7 +386,7 @@
             message = rev.message.rstrip('\r\n')
             for l in message.split('\n'):
                 print >>to_file,  indent+'  ' + l
-        if delta != None:
+        if delta is not None:
             delta.show(to_file, self.show_ids)
 
 
@@ -412,7 +412,7 @@
 
         # TODO: Why not show the modified files in a shorter form as
         # well? rewrap them single lines of appropriate length
-        if delta != None:
+        if delta is not None:
             delta.show(to_file, self.show_ids)
         print >>to_file, ''
 

=== modified file 'bzrlib/lsprof.py'
--- bzrlib/lsprof.py	2006-08-14 21:21:30 +0000
+++ bzrlib/lsprof.py	2006-09-08 18:46:29 +0000
@@ -165,7 +165,7 @@
         for k, v in sys.modules.items():
             if v is None:
                 continue
-            if not hasattr(v, '__file__'):
+            if getattr(v, '__file__', None) is None:
                 continue
             if not isinstance(v.__file__, str):
                 continue

=== added file 'bzrlib/memorytree.py'
--- bzrlib/memorytree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/memorytree.py	2006-09-22 04:52:17 +0000
@@ -0,0 +1,236 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""MemoryTree object.
+
+See MemoryTree for more details.
+"""
+
+
+from copy import deepcopy
+
+from bzrlib import errors, mutabletree
+from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.osutils import sha_file
+from bzrlib.mutabletree import needs_tree_write_lock
+from bzrlib.transport.memory import MemoryTransport
+
+
+class MemoryTree(mutabletree.MutableTree):
+    """A MemoryTree is a specialisation of MutableTree.
+    
+    It maintains nearly no state outside of read_lock and write_lock
+    transactions. (it keeps a reference to the branch, and its last-revision
+    only).
+    """
+
+    def __init__(self, branch, revision_id):
+        """Construct a MemoryTree for branch using revision_id."""
+        self.branch = branch
+        self.bzrdir = branch.bzrdir
+        self._branch_revision_id = revision_id
+        self._locks = 0
+        self._lock_mode = None
+
+    @needs_tree_write_lock
+    def _add(self, files, ids, kinds):
+        """See MutableTree._add."""
+        for f, file_id, kind in zip(files, ids, kinds):
+            if kind is None:
+                kind = 'file'
+            if file_id is None:
+                self._inventory.add_path(f, kind=kind)
+            else:
+                self._inventory.add_path(f, kind=kind, file_id=file_id)
+
+    def basis_tree(self):
+        """See Tree.basis_tree()."""
+        return self._basis_tree
+
+    @staticmethod
+    def create_on_branch(branch):
+        """Create a MemoryTree for branch, using the last-revision of branch."""
+        return MemoryTree(branch, branch.last_revision())
+
+    def _gather_kinds(self, files, kinds):
+        """See MutableTree._gather_kinds.
+        
+        This implementation does not care about the file kind of
+        missing files, so is a no-op.
+        """
+
+    def get_file(self, file_id):
+        """See Tree.get_file."""
+        return self._file_transport.get(self.id2path(file_id))
+
+    def get_file_sha1(self, file_id, path=None):
+        """See Tree.get_file_sha1()."""
+        if path is None:
+            path = self.id2path(file_id)
+        stream = self._file_transport.get(path)
+        return sha_file(stream)
+
+    @needs_read_lock
+    def get_parent_ids(self):
+        """See Tree.get_parent_ids.
+
+        This implementation returns the current cached value from
+            self._parent_ids.
+        """
+        return list(self._parent_ids)
+
+    def has_filename(self, filename):
+        """See Tree.has_filename()."""
+        return self._file_transport.has(filename)
+
+    def is_executable(self, file_id, path=None):
+        return self._inventory[file_id].executable
+
+    def mkdir(self, path, file_id=None):
+        """See MutableTree.mkdir()."""
+        self.add(path, file_id, 'directory')
+        if file_id is None:
+            file_id = self.path2id(path)
+        self._file_transport.mkdir(path)
+        return file_id
+
+    @needs_read_lock
+    def last_revision(self):
+        """See MutableTree.last_revision."""
+        return self._branch_revision_id
+
+    def lock_read(self):
+        """Lock the memory tree for reading.
+
+        This triggers population of data from the branch for its revision.
+        """
+        self._locks += 1
+        try:
+            if self._locks == 1:
+                self.branch.lock_read()
+                self._lock_mode = "r"
+                self._populate_from_branch()
+        except:
+            self._locks -= 1
+            raise
+
+    def lock_tree_write(self):
+        """See MutableTree.lock_tree_write()."""
+        self._locks += 1
+        try:
+            if self._locks == 1:
+                self.branch.lock_read()
+                self._lock_mode = "w"
+                self._populate_from_branch()
+            elif self._lock_mode == "r":
+                raise errors.ReadOnlyError(self)
+        except:
+            self._locks -= 1
+            raise
+
+    def lock_write(self):
+        """See MutableTree.lock_write()."""
+        self._locks += 1
+        try:
+            if self._locks == 1:
+                self.branch.lock_write()
+                self._lock_mode = "w"
+                self._populate_from_branch()
+            elif self._lock_mode == "r":
+                raise errors.ReadOnlyError(self)
+        except:
+            self._locks -= 1
+            raise
+
+    def _populate_from_branch(self):
+        """Populate the in-tree state from the branch."""
+        self._basis_tree = self.branch.repository.revision_tree(
+            self._branch_revision_id)
+        if self._branch_revision_id is None:
+            self._parent_ids = []
+        else:
+            self._parent_ids = [self._branch_revision_id]
+        self._inventory = deepcopy(self._basis_tree._inventory)
+        self._file_transport = MemoryTransport()
+        # TODO copy the revision trees content, or do it lazy, or something.
+        inventory_entries = self._inventory.iter_entries()
+        for path, entry in inventory_entries:
+            if path == '':
+                continue
+            if entry.kind == 'directory':
+                self._file_transport.mkdir(path)
+            elif entry.kind == 'file':
+                self._file_transport.put_file(path,
+                    self._basis_tree.get_file(entry.file_id))
+            else:
+                raise NotImplementedError(self._populate_from_branch)
+
+    def put_file_bytes_non_atomic(self, file_id, bytes):
+        """See MutableTree.put_file_bytes_non_atomic."""
+        self._file_transport.put_bytes(self.id2path(file_id), bytes)
+
+    def unlock(self):
+        """Release a lock.
+
+        This frees all cached state when the last lock context for the tree is
+        left.
+        """
+        if self._locks == 1:
+            self._basis_tree = None
+            self._parent_ids = []
+            self._inventory = None
+            try:
+                self.branch.unlock()
+            finally:
+                self._locks = 0
+                self._lock_mode = None
+        else:
+            self._locks -= 1
+
+    @needs_tree_write_lock
+    def unversion(self, file_ids):
+        """Remove the file ids in file_ids from the current versioned set.
+
+        When a file_id is unversioned, all of its children are automatically
+        unversioned.
+
+        :param file_ids: The file ids to stop versioning.
+        :raises: NoSuchId if any fileid is not currently versioned.
+        """
+        # XXX: This should be in mutabletree, but the inventory-save action
+        # is not relevant to memory tree. Until that is done in unlock by
+        # working tree, we cannot share the implementation.
+        for file_id in file_ids:
+            if self._inventory.has_id(file_id):
+                self._inventory.remove_recursive_id(file_id)
+            else:
+                raise errors.NoSuchId(self, file_id)
+
+    def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
+        """See MutableTree.set_parent_trees()."""
+        if len(parents_list) == 0:
+            self._parent_ids = []
+            self._basis_tree = self.branch.repository.revisiontree(None)
+        else:
+            if parents_list[0][1] is None and not allow_leftmost_as_ghost:
+                # a ghost in the left most parent
+                raise errors.GhostRevisionUnusableHere(parents_list[0][0])
+            self._parent_ids = [parent_id for parent_id, tree in parents_list]
+            if parents_list[0][1] is None:
+                self._basis_tree = self.branch.repository.revisiontree(None)
+            else:
+                self._basis_tree = parents_list[0][1]
+            self._branch_revision_id = parents_list[0][0]

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-09-04 08:00:00 +0000
+++ bzrlib/merge.py	2006-10-12 01:50:46 +0000
@@ -17,9 +17,11 @@
 
 import os
 import errno
-from tempfile import mkdtemp
 import warnings
 
+from bzrlib import (
+    osutils,
+    )
 from bzrlib.branch import Branch
 from bzrlib.conflicts import ConflictList, Conflict
 from bzrlib.errors import (BzrCommandError,
@@ -37,13 +39,14 @@
                            )
 from bzrlib.merge3 import Merge3
 import bzrlib.osutils
-from bzrlib.osutils import rename, pathjoin, rmtree
+from bzrlib.osutils import rename, pathjoin
 from progress import DummyProgress, ProgressPhase
 from bzrlib.revision import common_ancestor, is_ancestor, NULL_REVISION
 from bzrlib.textfile import check_text_lines
 from bzrlib.trace import mutter, warning, note
 from bzrlib.transform import (TreeTransform, resolve_conflicts, cook_conflicts,
-                              FinalPaths, create_by_entry, unique_add)
+                              FinalPaths, create_by_entry, unique_add,
+                              ROOT_PARENT)
 from bzrlib.versionedfile import WeaveMerge
 from bzrlib import ui
 
@@ -386,7 +389,7 @@
                     self.merge_executable(file_id, file_status)
             finally:
                 child_pb.finished()
-                
+            self.fix_root()
             self.pp.next_phase()
             child_pb = ui.ui_factory.nested_progress_bar()
             try:
@@ -408,6 +411,33 @@
             working_tree.unlock()
             self.pb.clear()
 
+    def fix_root(self):
+        try:
+            self.tt.final_kind(self.tt.root)
+        except NoSuchFile:
+            self.tt.cancel_deletion(self.tt.root)
+        if self.tt.final_file_id(self.tt.root) is None:
+            self.tt.version_file(self.tt.tree_file_id(self.tt.root), 
+                                 self.tt.root)
+        if self.other_tree.inventory.root is None:
+            return
+        other_root_file_id = self.other_tree.inventory.root.file_id
+        other_root = self.tt.trans_id_file_id(other_root_file_id)
+        if other_root == self.tt.root:
+            return
+        try:
+            self.tt.final_kind(other_root)
+        except NoSuchFile:
+            return
+        self.reparent_children(self.other_tree.inventory.root, self.tt.root)
+        self.tt.cancel_creation(other_root)
+        self.tt.cancel_versioning(other_root)
+
+    def reparent_children(self, ie, target):
+        for thing, child in ie.children.iteritems():
+            trans_id = self.tt.trans_id_file_id(child.file_id)
+            self.tt.adjust_path(self.tt.final_name(trans_id), target, trans_id)
+
     def write_modified(self, results):
         modified_hashes = {}
         for path in results.modified_paths:
@@ -518,9 +548,10 @@
                         "conflict": other_entry}
         trans_id = self.tt.trans_id_file_id(file_id)
         parent_id = winner_entry[parent_id_winner].parent_id
-        parent_trans_id = self.tt.trans_id_file_id(parent_id)
-        self.tt.adjust_path(winner_entry[name_winner].name, parent_trans_id,
-                            trans_id)
+        if parent_id is not None:
+            parent_trans_id = self.tt.trans_id_file_id(parent_id)
+            self.tt.adjust_path(winner_entry[name_winner].name, 
+                                parent_trans_id, trans_id)
 
     def merge_contents(self, file_id):
         """Performa a merge on file_id contents."""
@@ -868,7 +899,7 @@
         will be dumped, and a will be conflict noted.
         """
         import bzrlib.patch
-        temp_dir = mkdtemp(prefix="bzr-")
+        temp_dir = osutils.mkdtemp(prefix="bzr-")
         try:
             new_file = pathjoin(temp_dir, "new")
             this = self.dump_file(temp_dir, "this", self.this_tree, file_id)
@@ -886,9 +917,9 @@
                 name = self.tt.final_name(trans_id)
                 parent_id = self.tt.final_parent(trans_id)
                 self._dump_conflicts(name, parent_id, file_id)
-            self._raw_conflicts.append(('text conflict', trans_id))
+                self._raw_conflicts.append(('text conflict', trans_id))
         finally:
-            rmtree(temp_dir)
+            osutils.rmtree(temp_dir)
 
 
 def merge_inner(this_branch, other_tree, base_tree, ignore_zero=False,

=== added file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/mutabletree.py	2006-09-22 04:52:17 +0000
@@ -0,0 +1,186 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""MutableTree object.
+
+See MutableTree for more details.
+"""
+
+
+from bzrlib import tree
+from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.osutils import splitpath
+from bzrlib.symbol_versioning import DEPRECATED_PARAMETER
+
+
+def needs_tree_write_lock(unbound):
+    """Decorate unbound to take out and release a tree_write lock."""
+    def tree_write_locked(self, *args, **kwargs):
+        self.lock_tree_write()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    tree_write_locked.__doc__ = unbound.__doc__
+    tree_write_locked.__name__ = unbound.__name__
+    return tree_write_locked
+
+
+class MutableTree(tree.Tree):
+    """A MutableTree is a specialisation of Tree which is able to be mutated.
+
+    Generally speaking these mutations are only possible within a lock_write
+    context, and will revert if the lock is broken abnormally - but this cannot
+    be guaranteed - depending on the exact implementation of the mutable state.
+
+    The most common form of Mutable Tree is WorkingTree, see bzrlib.workingtree.
+    For tests we also have MemoryTree which is a MutableTree whose contents are
+    entirely in memory.
+
+    For now, we are not treating MutableTree as an interface to provide
+    conformance tests for - rather we are testing MemoryTree specifically, and 
+    interface testing implementations of WorkingTree.
+
+    A mutable tree always has an associated Branch and BzrDir object - the
+    branch and bzrdir attributes.
+    """
+
+    @needs_write_lock
+    def add(self, files, ids=None, kinds=None):
+        """Add paths to the set of versioned paths.
+
+        Note that the command line normally calls smart_add instead,
+        which can automatically recurse.
+
+        This adds the files to the inventory, so that they will be
+        recorded by the next commit.
+
+        :param files: List of paths to add, relative to the base of the tree.
+        :param ids: If set, use these instead of automatically generated ids.
+            Must be the same length as the list of files, but may
+            contain None for ids that are to be autogenerated.
+        :param kinds: Optional parameter to specify the kinds to be used for
+            each file.
+
+        TODO: Perhaps callback with the ids and paths as they're added.
+        """
+        if isinstance(files, basestring):
+            assert(ids is None or isinstance(ids, basestring))
+            assert(kinds is None or isinstance(kinds, basestring))
+            files = [files]
+            if ids is not None:
+                ids = [ids]
+            if kinds is not None:
+                kinds = [kinds]
+
+        if ids is None:
+            ids = [None] * len(files)
+        else:
+            assert(len(ids) == len(files))
+
+        if kinds is None:
+            kinds = [None] * len(files)
+        else:
+            assert(len(kinds) == len(files))
+        for f in files:
+            # generic constraint checks:
+            if self.is_control_filename(f):
+                raise errors.ForbiddenControlFileError(filename=f)
+            fp = splitpath(f)
+        # fill out file kinds for all files [not needed when we stop 
+        # caring about the instantaneous file kind within a uncommmitted tree
+        #
+        self._gather_kinds(files, kinds)
+        self._add(files, ids, kinds)
+
+    def _add(self, files, ids, kinds):
+        """Helper function for add - updates the inventory."""
+        raise NotImplementedError(self._add)
+
+    @needs_write_lock
+    def commit(self, message=None, revprops=None, *args, **kwargs):
+        # avoid circular imports
+        from bzrlib import commit
+        if revprops is None:
+            revprops = {}
+        if not 'branch-nick' in revprops:
+            revprops['branch-nick'] = self.branch.nick
+        # args for wt.commit start at message from the Commit.commit method,
+        # but with branch a kwarg now, passing in args as is results in the
+        #message being used for the branch
+        args = (DEPRECATED_PARAMETER, message, ) + args
+        committed_id = commit.Commit().commit(working_tree=self,
+            revprops=revprops, *args, **kwargs)
+        return committed_id
+
+    def _gather_kinds(self, files, kinds):
+        """Helper function for add - sets the entries of kinds."""
+        raise NotImplementedError(self._gather_kinds)
+
+    @needs_read_lock
+    def last_revision(self):
+        """Return the revision id of the last commit performed in this tree.
+
+        In early tree formats the result of last_revision is the same as the
+        branch last_revision, but that is no longer the case for modern tree
+        formats.
+        
+        last_revision returns the left most parent id, or None if there are no
+        parents.
+
+        last_revision was deprecated as of 0.11. Please use get_parent_ids
+        instead.
+        """
+        raise NotImplementedError(self.last_revision)
+
+    def lock_tree_write(self):
+        """Lock the working tree for write, and the branch for read.
+
+        This is useful for operations which only need to mutate the working
+        tree. Taking out branch write locks is a relatively expensive process
+        and may fail if the branch is on read only media. So branch write locks
+        should only be taken out when we are modifying branch data - such as in
+        operations like commit, pull, uncommit and update.
+        """
+        raise NotImplementedError(self.lock_tree_write)
+
+    def lock_write(self):
+        """Lock the tree and its branch. This allows mutating calls to be made.
+
+        Some mutating methods will take out implicit write locks, but in 
+        general you should always obtain a write lock before calling mutating
+        methods on a tree.
+        """
+        raise NotImplementedError(self.lock_write)
+
+    @needs_write_lock
+    def mkdir(self, path, file_id=None):
+        """Create a directory in the tree. if file_id is None, one is assigned.
+
+        :param path: A unicode file path.
+        :param file_id: An optional file-id.
+        :return: the file id of the new directory.
+        """
+        raise NotImplementedError(self.mkdir)
+
+    def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
+        """Set the parents of the working tree.
+
+        :param parents_list: A list of (revision_id, tree) tuples. 
+            If tree is None, then that element is treated as an unreachable
+            parent tree - i.e. a ghost.
+        """
+        raise NotImplementedError(self.set_parent_trees)

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2006-08-28 16:19:47 +0000
+++ bzrlib/option.py	2006-10-06 07:13:51 +0000
@@ -17,12 +17,18 @@
 # TODO: For things like --diff-prefix, we want a way to customize the display
 # of the option argument.
 
+import re
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import optparse
-import re
 
+from bzrlib import (
+    errors,
+    revisionspec,
+    )
+""")
 from bzrlib.trace import warning
-from bzrlib.revisionspec import RevisionSpec
-from bzrlib.errors import BzrCommandError
 
 
 def _parse_revision_str(revstr):
@@ -82,7 +88,7 @@
     revs = []
     sep = re.compile("\\.\\.(?!/)")
     for x in sep.split(revstr):
-        revs.append(RevisionSpec.from_string(x or None))
+        revs.append(revisionspec.RevisionSpec.from_string(x or None))
     return revs
 
 
@@ -100,7 +106,7 @@
         type_list = '\n'.join(lines)
         msg = "No known merge type %s. Supported types are:\n%s" %\
             (typestring, type_list)
-        raise BzrCommandError(msg)
+        raise errors.BzrCommandError(msg)
 
 class Option(object):
     """Description of a command line option"""
@@ -191,7 +197,7 @@
     DEFAULT_VALUE = object()
 
     def error(self, message):
-        raise BzrCommandError(message)
+        raise errors.BzrCommandError(message)
 
 
 def get_optparser(options):

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-08-25 15:05:57 +0000
+++ bzrlib/osutils.py	2006-10-11 00:23:23 +0000
@@ -17,6 +17,16 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from cStringIO import StringIO
+import os
+import re
+import stat
+from stat import (S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE,
+                  S_ISCHR, S_ISBLK, S_ISFIFO, S_ISSOCK)
+import sys
+import time
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import errno
 from ntpath import (abspath as _nt_abspath,
                     join as _nt_join,
@@ -24,32 +34,29 @@
                     realpath as _nt_realpath,
                     splitdrive as _nt_splitdrive,
                     )
-import os
-from os import listdir
 import posixpath
-import re
 import sha
 import shutil
-from shutil import copyfile
-import stat
-from stat import (S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE,
-                  S_ISCHR, S_ISBLK, S_ISFIFO, S_ISSOCK)
+from shutil import (
+    rmtree,
+    )
 import string
-import sys
-import time
-import types
 import tempfile
+from tempfile import (
+    mkdtemp,
+    )
 import unicodedata
 
+from bzrlib import (
+    errors,
+    )
+""")
+
 import bzrlib
-from bzrlib.errors import (BzrError,
-                           BzrBadParameterNotUnicode,
-                           NoSuchFile,
-                           PathNotChild,
-                           IllegalPath,
-                           )
-from bzrlib.symbol_versioning import (deprecated_function, 
-        zero_nine)
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    zero_nine,
+    )
 from bzrlib.trace import mutter
 
 
@@ -84,7 +91,7 @@
     Windows."""
     # TODO: I'm not really sure this is the best format either.x
     global _QUOTE_RE
-    if _QUOTE_RE == None:
+    if _QUOTE_RE is None:
         _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/\\_~-])')
         
     if _QUOTE_RE.search(f):
@@ -122,7 +129,7 @@
         return _mapper(_lstat(f).st_mode)
     except OSError, e:
         if getattr(e, 'errno', None) == errno.ENOENT:
-            raise bzrlib.errors.NoSuchFile(f)
+            raise errors.NoSuchFile(f)
         raise
 
 
@@ -144,13 +151,13 @@
     elif kind == 'symlink':
         return '@'
     else:
-        raise BzrError('invalid file kind %r' % kind)
+        raise errors.BzrError('invalid file kind %r' % kind)
 
 lexists = getattr(os.path, 'lexists', None)
 if lexists is None:
     def lexists(f):
         try:
-            if hasattr(os, 'lstat'):
+            if getattr(os, 'lstat') is not None:
                 os.lstat(f)
             else:
                 os.stat(f)
@@ -159,7 +166,7 @@
             if e.errno == errno.ENOENT:
                 return False;
             else:
-                raise BzrError("lstat/stat of (%r): %r" % (f, e))
+                raise errors.BzrError("lstat/stat of (%r): %r" % (f, e))
 
 
 def fancy_rename(old, new, rename_func, unlink_func):
@@ -186,16 +193,16 @@
     file_existed = False
     try:
         rename_func(new, tmp_name)
-    except (NoSuchFile,), e:
+    except (errors.NoSuchFile,), e:
         pass
     except IOError, e:
         # RBC 20060103 abstraction leakage: the paramiko SFTP clients rename
-        # function raises an IOError with errno == None when a rename fails.
+        # function raises an IOError with errno is None when a rename fails.
         # This then gets caught here.
         if e.errno not in (None, errno.ENOENT, errno.ENOTDIR):
             raise
     except Exception, e:
-        if (not hasattr(e, 'errno') 
+        if (getattr(e, 'errno', None) is None
             or e.errno not in (errno.ENOENT, errno.ENOTDIR)):
             raise
     else:
@@ -302,11 +309,12 @@
 pathjoin = os.path.join
 normpath = os.path.normpath
 getcwd = os.getcwdu
-mkdtemp = tempfile.mkdtemp
 rename = os.rename
 dirname = os.path.dirname
 basename = os.path.basename
-rmtree = shutil.rmtree
+# These were already imported into local scope
+# mkdtemp = tempfile.mkdtemp
+# rmtree = shutil.rmtree
 
 MIN_ABS_PATHLENGTH = 1
 
@@ -330,7 +338,7 @@
         if function in (os.remove, os.rmdir) \
             and type_ == OSError \
             and value.errno == errno.EACCES:
-            bzrlib.osutils.make_writable(path)
+            make_writable(path)
             function(path)
         else:
             raise
@@ -370,7 +378,7 @@
 
 
 def normalizepath(f):
-    if hasattr(os.path, 'realpath'):
+    if getattr(os.path, 'realpath', None) is not None:
         F = realpath
     else:
         F = abspath
@@ -440,19 +448,6 @@
     
     The empty string as a dir name is taken as top-of-tree and matches 
     everything.
-    
-    >>> is_inside('src', pathjoin('src', 'foo.c'))
-    True
-    >>> is_inside('src', 'srccontrol')
-    False
-    >>> is_inside('src', pathjoin('src', 'a', 'a', 'a', 'foo.c'))
-    True
-    >>> is_inside('foo.c', 'foo.c')
-    True
-    >>> is_inside('foo.c', '')
-    False
-    >>> is_inside('', 'foo.c')
-    True
     """
     # XXX: Most callers of this can actually do something smarter by 
     # looking at the inventory
@@ -505,7 +500,7 @@
 
 
 def sha_file(f):
-    if hasattr(f, 'tell'):
+    if getattr(f, 'tell', None) is not None:
         assert f.tell() == 0
     s = sha.new()
     BUFSIZE = 128<<10
@@ -555,7 +550,7 @@
 def local_time_offset(t=None):
     """Return offset of local zone from GMT, either at present or at time t."""
     # python2.3 localtime() can't take None
-    if t == None:
+    if t is None:
         t = time.time()
         
     if time.localtime(t).tm_isdst and time.daylight:
@@ -574,15 +569,15 @@
         tt = time.gmtime(t)
         offset = 0
     elif timezone == 'original':
-        if offset == None:
+        if offset is None:
             offset = 0
         tt = time.gmtime(t + offset)
     elif timezone == 'local':
         tt = time.localtime(t)
         offset = local_time_offset(t)
     else:
-        raise BzrError("unsupported timezone format %r" % timezone,
-                       ['options are "utc", "original", "local"'])
+        raise errors.BzrError("unsupported timezone format %r" % timezone,
+                              ['options are "utc", "original", "local"'])
     if date_fmt is None:
         date_fmt = "%a %Y-%m-%d %H:%M:%S"
     if show_offset:
@@ -596,6 +591,54 @@
     return time.strftime('%Y%m%d%H%M%S', time.gmtime(when))
     
 
+def format_delta(delta):
+    """Get a nice looking string for a time delta.
+
+    :param delta: The time difference in seconds, can be positive or negative.
+        positive indicates time in the past, negative indicates time in the
+        future. (usually time.time() - stored_time)
+    :return: String formatted to show approximate resolution
+    """
+    delta = int(delta)
+    if delta >= 0:
+        direction = 'ago'
+    else:
+        direction = 'in the future'
+        delta = -delta
+
+    seconds = delta
+    if seconds < 90: # print seconds up to 90 seconds
+        if seconds == 1:
+            return '%d second %s' % (seconds, direction,)
+        else:
+            return '%d seconds %s' % (seconds, direction)
+
+    minutes = int(seconds / 60)
+    seconds -= 60 * minutes
+    if seconds == 1:
+        plural_seconds = ''
+    else:
+        plural_seconds = 's'
+    if minutes < 90: # print minutes, seconds up to 90 minutes
+        if minutes == 1:
+            return '%d minute, %d second%s %s' % (
+                    minutes, seconds, plural_seconds, direction)
+        else:
+            return '%d minutes, %d second%s %s' % (
+                    minutes, seconds, plural_seconds, direction)
+
+    hours = int(minutes / 60)
+    minutes -= 60 * hours
+    if minutes == 1:
+        plural_minutes = ''
+    else:
+        plural_minutes = 's'
+
+    if hours == 1:
+        return '%d hour, %d minute%s %s' % (hours, minutes,
+                                            plural_minutes, direction)
+    return '%d hours, %d minute%s %s' % (hours, minutes,
+                                         plural_minutes, direction)
 
 def filesize(f):
     """Return size of given open file."""
@@ -611,10 +654,10 @@
 except (NotImplementedError, AttributeError):
     # If python doesn't have os.urandom, or it doesn't work,
     # then try to first pull random data from /dev/urandom
-    if os.path.exists("/dev/urandom"):
+    try:
         rand_bytes = file('/dev/urandom', 'rb').read
     # Otherwise, use this hack as a last resort
-    else:
+    except (IOError, OSError):
         # not well seeded, but better than nothing
         def rand_bytes(n):
             import random
@@ -642,22 +685,8 @@
 ## decomposition (might be too tricksy though.)
 
 def splitpath(p):
-    """Turn string into list of parts.
-
-    >>> splitpath('a')
-    ['a']
-    >>> splitpath('a/b')
-    ['a', 'b']
-    >>> splitpath('a/./b')
-    ['a', 'b']
-    >>> splitpath('a/.b')
-    ['a', '.b']
-    >>> splitpath('a/../b')
-    Traceback (most recent call last):
-    ...
-    BzrError: sorry, '..' not allowed in path
-    """
-    assert isinstance(p, types.StringTypes)
+    """Turn string into list of parts."""
+    assert isinstance(p, basestring)
 
     # split on either delimiter because people might use either on
     # Windows
@@ -666,7 +695,7 @@
     rps = []
     for f in ps:
         if f == '..':
-            raise BzrError("sorry, %r not allowed in path" % f)
+            raise errors.BzrError("sorry, %r not allowed in path" % f)
         elif (f == '.') or (f == ''):
             pass
         else:
@@ -676,8 +705,8 @@
 def joinpath(p):
     assert isinstance(p, list)
     for f in p:
-        if (f == '..') or (f == None) or (f == ''):
-            raise BzrError("sorry, %r not allowed in path" % f)
+        if (f == '..') or (f is None) or (f == ''):
+            raise errors.BzrError("sorry, %r not allowed in path" % f)
     return pathjoin(*p)
 
 
@@ -705,14 +734,14 @@
 def link_or_copy(src, dest):
     """Hardlink a file, or copy it if it can't be hardlinked."""
     if not hardlinks_good():
-        copyfile(src, dest)
+        shutil.copyfile(src, dest)
         return
     try:
         os.link(src, dest)
     except (OSError, IOError), e:
         if e.errno != errno.EXDEV:
             raise
-        copyfile(src, dest)
+        shutil.copyfile(src, dest)
 
 def delete_any(full_path):
     """Delete a file or directory."""
@@ -726,7 +755,7 @@
 
 
 def has_symlinks():
-    if hasattr(os, 'symlink'):
+    if getattr(os, 'symlink', None) is not None:
         return True
     else:
         return False
@@ -776,7 +805,7 @@
         if tail:
             s.insert(0, tail)
     else:
-        raise PathNotChild(rp, base)
+        raise errors.PathNotChild(rp, base)
 
     if s:
         return pathjoin(*s)
@@ -797,7 +826,7 @@
     try:
         return unicode_or_utf8_string.decode('utf8')
     except UnicodeDecodeError:
-        raise BzrBadParameterNotUnicode(unicode_or_utf8_string)
+        raise errors.BzrBadParameterNotUnicode(unicode_or_utf8_string)
 
 
 _platform_normalizes_filenames = False
@@ -867,10 +896,30 @@
 
     return width
 
+
 def supports_executable():
     return sys.platform != "win32"
 
 
+def set_or_unset_env(env_variable, value):
+    """Modify the environment, setting or removing the env_variable.
+
+    :param env_variable: The environment variable in question
+    :param value: The value to set the environment to. If None, then
+        the variable will be removed.
+    :return: The original value of the environment variable.
+    """
+    orig_val = os.environ.get(env_variable)
+    if value is None:
+        if orig_val is not None:
+            del os.environ[env_variable]
+    else:
+        if isinstance(value, unicode):
+            value = value.encode(bzrlib.user_encoding)
+        os.environ[env_variable] = value
+    return orig_val
+
+
 _validWin32PathRE = re.compile(r'^([A-Za-z]:[/\\])?[^:<>*"?\|]*$')
 
 
@@ -882,7 +931,7 @@
     if sys.platform != "win32":
         return
     if _validWin32PathRE.match(path) is None:
-        raise IllegalPath(path)
+        raise errors.IllegalPath(path)
 
 
 def walkdirs(top, prefix=""):
@@ -921,7 +970,7 @@
     lstat = os.lstat
     pending = []
     _directory = _directory_kind
-    _listdir = listdir
+    _listdir = os.listdir
     pending = [(prefix, "", _directory, None, top)]
     while pending:
         dirblock = []
@@ -1032,7 +1081,7 @@
         _cached_user_encoding = locale.getpreferredencoding()
     except locale.Error, e:
         sys.stderr.write('bzr: warning: %s\n'
-                         '  Could not what text encoding to use.\n'
+                         '  Could not determine what text encoding to use.\n'
                          '  This error usually means your Python interpreter\n'
                          '  doesn\'t support the locale set by $LANG (%s)\n'
                          "  Continuing with ascii encoding.\n"

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2006-08-01 04:54:43 +0000
+++ bzrlib/plugin.py	2006-09-13 03:21:03 +0000
@@ -39,20 +39,34 @@
 # That could be either a separate Python interpreter or perhaps a new
 # namespace inside this interpreter.
 
-import imp
 import os
 import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import imp
 import types
 
-import bzrlib
-from bzrlib.config import config_dir
-from bzrlib.trace import log_error, mutter, warning, \
-        log_exception_quietly
-from bzrlib.errors import BzrError
-from bzrlib import plugins
-from bzrlib.osutils import pathjoin
-
-DEFAULT_PLUGIN_PATH = pathjoin(config_dir(), 'plugins')
+from bzrlib import (
+    config,
+    osutils,
+    plugins,
+    )
+""")
+
+from bzrlib.trace import mutter, warning, log_exception_quietly
+
+
+DEFAULT_PLUGIN_PATH = None
+
+
+def get_default_plugin_path():
+    """Get the DEFAULT_PLUGIN_PATH"""
+    global DEFAULT_PLUGIN_PATH
+    if DEFAULT_PLUGIN_PATH is None:
+        DEFAULT_PLUGIN_PATH = osutils.pathjoin(config.config_dir(), 'plugins')
+    return DEFAULT_PLUGIN_PATH
+
 
 _loaded = False
 
@@ -60,7 +74,7 @@
 def all_plugins():
     """Return a dictionary of the plugins."""
     result = {}
-    for name, plugin in bzrlib.plugins.__dict__.items():
+    for name, plugin in plugins.__dict__.items():
         if isinstance(plugin, types.ModuleType):
             result[name] = plugin
     return result
@@ -92,10 +106,10 @@
     if _loaded:
         # People can make sure plugins are loaded, they just won't be twice
         return
-        #raise BzrError("plugins already initialized")
     _loaded = True
 
-    dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(os.pathsep)
+    dirs = os.environ.get('BZR_PLUGIN_PATH',
+                          get_default_plugin_path()).split(os.pathsep)
     dirs.insert(0, os.path.dirname(plugins.__file__))
 
     load_from_dirs(dirs)
@@ -125,12 +139,12 @@
         if not os.path.isdir(d):
             continue
         for f in os.listdir(d):
-            path = pathjoin(d, f)
+            path = osutils.pathjoin(d, f)
             if os.path.isdir(path):
                 for entry in package_entries:
                     # This directory should be a package, and thus added to
                     # the list
-                    if os.path.isfile(pathjoin(path, entry)):
+                    if os.path.isfile(osutils.pathjoin(path, entry)):
                         break
                 else: # This directory is not a package
                     continue
@@ -143,7 +157,7 @@
                         break
                 else:
                     continue
-            if getattr(bzrlib.plugins, f, None):
+            if getattr(plugins, f, None):
                 mutter('Plugin name %s already loaded', f)
             else:
                 mutter('add plugin name %s', f)
@@ -158,7 +172,7 @@
                 try:
                     plugin = imp.load_module('bzrlib.plugins.' + name,
                                              *plugin_info)
-                    setattr(bzrlib.plugins, name, plugin)
+                    setattr(plugins, name, plugin)
                 finally:
                     if plugin_info[0] is not None:
                         plugin_info[0].close()

=== modified file 'bzrlib/plugins/launchpad/test_register.py'
--- bzrlib/plugins/launchpad/test_register.py	2006-05-16 02:31:06 +0000
+++ bzrlib/plugins/launchpad/test_register.py	2006-09-21 23:06:54 +0000
@@ -71,6 +71,9 @@
 
 class InstrumentedXMLRPCTransport(xmlrpclib.Transport):
 
+    # Python 2.5's xmlrpclib looks for this.
+    _use_datetime = False
+
     def __init__(self, testcase):
         self.testcase = testcase
 

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2006-07-10 23:00:44 +0000
+++ bzrlib/progress.py	2006-10-06 07:29:17 +0000
@@ -35,12 +35,17 @@
 # TODO: Optionally show elapsed time instead/as well as ETA; nicer
 # when the rate is unpredictable
 
-
 import sys
 import time
 import os
 
-import bzrlib.errors as errors
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    errors,
+    )
+""")
+
 from bzrlib.trace import mutter
 
 

=== added file 'bzrlib/registry.py'
--- bzrlib/registry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/registry.py	2006-10-13 00:55:26 +0000
@@ -0,0 +1,214 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Classes to provide name-to-object registry-like support."""
+
+
+class _ObjectGetter(object):
+    """Maintain a reference to an object, and return the object on request.
+
+    This is used by Registry to make plain objects function similarly
+    to lazily imported objects.
+
+    Objects can be any sort of python object (class, function, module,
+    instance, etc)
+    """
+
+    __slots__ = ['_obj']
+
+    def __init__(self, obj):
+        self._obj = obj
+
+    def get_obj(self):
+        """Get the object that was saved at creation time"""
+        return self._obj
+
+
+class _LazyObjectGetter(_ObjectGetter):
+    """Keep a record of a possible object.
+
+    When requested, load and return it.
+    """
+
+    __slots__ = ['_module_name', '_member_name', '_imported']
+
+    def __init__(self, module_name, member_name):
+        self._module_name = module_name
+        self._member_name = member_name
+        self._imported = False
+        super(_LazyObjectGetter, self).__init__(None)
+
+    def get_obj(self):
+        """Get the referenced object.
+
+        Upon first request, the object will be imported. Future requests will
+        return the imported object.
+        """
+        if not self._imported:
+            self._do_import()
+        return super(_LazyObjectGetter, self).get_obj()
+
+    def _do_import(self):
+        obj = __import__(self._module_name, globals(), locals(),
+                         [self._member_name])
+        if self._member_name:
+            obj = getattr(obj, self._member_name)
+        self._obj = obj
+        self._imported = True
+
+
+class Registry(object):
+    """A class that registers objects to a name.
+
+    There are many places that want to collect related objects and access them
+    by a key. This class is designed to allow registering the mapping from key
+    to object. It goes one step further, and allows registering a name to a
+    hypothetical object which has not been imported yet. It also supports
+    adding additional information at registration time so that decisions can be
+    made without having to import the object (which may be expensive).
+
+    The functions 'get', 'get_info', and 'get_help' also support a
+    'default_key' (settable through my_registry.default_key = XXX, XXX must
+    already be registered.) Calling my_registry.get() or my_registry.get(None),
+    will return the entry for the default key.
+    """
+
+    def __init__(self):
+        """Create a new Registry."""
+        self._default_key = None
+        # Map from key => (is_lazy, info)
+        self._dict = {}
+        self._help_dict = {}
+        self._info_dict = {}
+
+    def register(self, key, obj, help=None, info=None,
+                 override_existing=False):
+        """Register a new object to a name.
+
+        :param key: This is the key to use to request the object later.
+        :param obj: The object to register.
+        :param help: Help text for this entry. This may be a string or
+                a callable. If it is a callable, it should take two
+                parameters (registry, key): this registry and the key that 
+                the help was registered under.
+        :param info: More information for this entry. Registry.get_info()
+                can be used to get this information. Registry treats this as an
+                opaque storage location (it is defined by the caller).
+        :param override_existing: Raise KeyErorr if False and something has
+                already been registered for that key. If True, ignore if there
+                is an existing key (always register the new value).
+        """
+        if not override_existing:
+            if key in self._dict:
+                raise KeyError('Key %r already registered' % key)
+        self._dict[key] = _ObjectGetter(obj)
+        self._add_help_and_info(key, help=help, info=info)
+
+    def register_lazy(self, key, module_name, member_name,
+                      help=None, info=None,
+                      override_existing=False):
+        """Register a new object to be loaded on request.
+
+        :param module_name: The python path to the module. Such as 'os.path'.
+        :param member_name: The member of the module to return.  If empty or 
+                None, get() will return the module itself.
+        :param help: Help text for this entry. This may be a string or
+                a callable.
+        :param info: More information for this entry. Registry 
+        :param override_existing: If True, replace the existing object
+                with the new one. If False, if there is already something
+                registered with the same key, raise a KeyError
+        """
+        if not override_existing:
+            if key in self._dict:
+                raise KeyError('Key %r already registered' % key)
+        self._dict[key] = _LazyObjectGetter(module_name, member_name)
+        self._add_help_and_info(key, help=help, info=info)
+
+    def _add_help_and_info(self, key, help=None, info=None):
+        """Add the help and information about this key"""
+        self._help_dict[key] = help
+        self._info_dict[key] = info
+
+    def get(self, key=None):
+        """Return the object register()'ed to the given key.
+
+        May raise ImportError if the object was registered lazily and
+        there are any problems, or AttributeError if the module does not 
+        have the supplied member.
+
+        :param key: The key to obtain the object for. If no object has been
+            registered to that key, the object registered for self.default_key
+            will be returned instead, if it exists. Otherwise KeyError will be
+            raised.
+        :return: The previously registered object.
+        :raises ImportError: If the object was registered lazily, and there are
+            problems during import.
+        :raises AttributeError: If registered lazily, and the module does not
+            contain the registered member.
+        """
+        return self._dict[self._get_key_or_default(key)].get_obj()
+
+    def _get_key_or_default(self, key=None):
+        """Return either 'key' or the default key if key is None"""
+        if key is not None:
+            return key
+        if self.default_key is None:
+            raise KeyError('Key is None, and no default key is set')
+        else:
+            return self.default_key
+
+    def get_help(self, key=None):
+        """Get the help text associated with the given key"""
+        the_help = self._help_dict[self._get_key_or_default(key)]
+        if callable(the_help):
+            return the_help(self, key)
+        return the_help
+
+    def get_info(self, key=None):
+        """Get the extra information associated with the given key"""
+        return self._info_dict[self._get_key_or_default(key)]
+
+    def remove(self, key):
+        """Remove a registered entry.
+
+        This is mostly for the test suite, but it can be used by others
+        """
+        del self._dict[key]
+
+    def __contains__(self, key):
+        return key in self._dict
+
+    def keys(self):
+        """Get a list of registered entries"""
+        return sorted(self._dict.keys())
+
+    def iteritems(self):
+        for key, getter in self._dict.iteritems():
+            yield key, getter.get_obj()
+
+    def _set_default_key(self, key):
+        if not self._dict.has_key(key):
+            raise KeyError('No object registered under key %s.' % key)
+        else:
+            self._default_key = key
+
+    def _get_default_key(self):
+        return self._default_key
+
+    default_key = property(_get_default_key, _set_default_key,
+                            doc="Current value of the default key."
+                                " Can be set to any existing key.")

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-08-16 21:10:33 +0000
+++ bzrlib/repository.py	2006-10-12 01:50:46 +0000
@@ -14,36 +14,55 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from cStringIO import StringIO
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from binascii import hexlify
 from copy import deepcopy
-from cStringIO import StringIO
 import re
 import time
-from unittest import TestSuite
-
-from bzrlib import bzrdir, check, delta, gpg, errors, xml5, ui, transactions, osutils
+import unittest
+
+from bzrlib import (
+    bzrdir,
+    check,
+    delta,
+    errors,
+    gpg,
+    graph,
+    knit,
+    lockable_files,
+    lockdir,
+    osutils,
+    revision as _mod_revision,
+    symbol_versioning,
+    transactions,
+    ui,
+    weave,
+    weavefile,
+    xml5,
+    xml6,
+    )
+from bzrlib.osutils import (
+    rand_bytes,
+    compact_date, 
+    local_time_offset,
+    )
+from bzrlib.revisiontree import RevisionTree
+from bzrlib.store.versioned import VersionedFileStore
+from bzrlib.store.text import TextStore
+from bzrlib.testament import Testament
+""")
+
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.errors import InvalidRevisionId
-from bzrlib.graph import Graph
 from bzrlib.inter import InterObject
 from bzrlib.inventory import Inventory, InventoryDirectory, ROOT_ID
-from bzrlib.knit import KnitVersionedFile, KnitPlainFactory
-from bzrlib.lockable_files import LockableFiles, TransportLock
-from bzrlib.lockdir import LockDir
-from bzrlib.osutils import (safe_unicode, rand_bytes, compact_date, 
-                            local_time_offset)
-from bzrlib.revision import NULL_REVISION, Revision
-from bzrlib.revisiontree import RevisionTree
-from bzrlib.store.versioned import VersionedFileStore, WeaveStore
-from bzrlib.store.text import TextStore
-from bzrlib import symbol_versioning
-from bzrlib.symbol_versioning import (deprecated_method,
-        zero_nine, 
+from bzrlib.symbol_versioning import (
+        deprecated_method,
+        zero_nine,
         )
-from bzrlib.testament import Testament
 from bzrlib.trace import mutter, note, warning
-from bzrlib.tsort import topo_sort
-from bzrlib.weave import WeaveFile
 
 
 # Old formats display a warning, but only once
@@ -75,7 +94,7 @@
             "Mismatch between inventory revision" \
             " id and insertion revid (%r, %r)" % (inv.revision_id, revid)
         assert inv.root is not None
-        inv_text = xml5.serializer_v5.write_inventory_to_string(inv)
+        inv_text = self.serialise_inventory(inv)
         inv_sha1 = osutils.sha_string(inv_text)
         inv_vf = self.control_weaves.get_weave('inventory',
                                                self.get_transaction())
@@ -195,6 +214,7 @@
         # TODO: make sure to construct the right store classes, etc, depending
         # on whether escaping is required.
         self._warn_if_deprecated()
+        self._serializer = xml5.serializer_v5
 
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, 
@@ -305,7 +325,8 @@
         or testing the revision graph.
         """
         if not revision_id or not isinstance(revision_id, basestring):
-            raise InvalidRevisionId(revision_id=revision_id, branch=self)
+            raise errors.InvalidRevisionId(revision_id=revision_id,
+                                           branch=self)
         return self._revision_store.get_revisions([revision_id],
                                                   self.get_transaction())[0]
     @needs_read_lock
@@ -399,10 +420,7 @@
         revision_ids. Each altered file-ids has the exact revision_ids that
         altered it listed explicitly.
         """
-        assert isinstance(self._format, (RepositoryFormat5,
-                                         RepositoryFormat6,
-                                         RepositoryFormat7,
-                                         RepositoryFormatKnit1)), \
+        assert self._serializer.support_altered_by_hack, \
             ("fileids_altered_by_revision_ids only supported for branches " 
              "which store inventory as unnested xml, not on %r" % self)
         selected_revision_ids = set(revision_ids)
@@ -417,20 +435,25 @@
         # revisions. We don't need to see all lines in the inventory because
         # only those added in an inventory in rev X can contain a revision=X
         # line.
-        for line in w.iter_lines_added_or_present_in_versions(selected_revision_ids):
-            start = line.find('file_id="')+9
-            if start < 9: continue
-            end = line.find('"', start)
-            assert end>= 0
-            file_id = _unescape_xml(line[start:end])
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            for line in w.iter_lines_added_or_present_in_versions(
+                selected_revision_ids, pb=pb):
+                start = line.find('file_id="')+9
+                if start < 9: continue
+                end = line.find('"', start)
+                assert end>= 0
+                file_id = _unescape_xml(line[start:end])
 
-            start = line.find('revision="')+10
-            if start < 10: continue
-            end = line.find('"', start)
-            assert end>= 0
-            revision_id = _unescape_xml(line[start:end])
-            if revision_id in selected_revision_ids:
-                result.setdefault(file_id, set()).add(revision_id)
+                start = line.find('revision="')+10
+                if start < 10: continue
+                end = line.find('"', start)
+                assert end>= 0
+                revision_id = _unescape_xml(line[start:end])
+                if revision_id in selected_revision_ids:
+                    result.setdefault(file_id, set()).add(revision_id)
+        finally:
+            pb.finished()
         return result
 
     @needs_read_lock
@@ -450,10 +473,13 @@
         :param revision_id: The expected revision id of the inventory.
         :param xml: A serialised inventory.
         """
-        result = xml5.serializer_v5.read_inventory_from_string(xml)
+        result = self._serializer.read_inventory_from_string(xml)
         result.root.revision = revision_id
         return result
 
+    def serialise_inventory(self, inv):
+        return self._serializer.write_inventory_to_string(inv)
+
     @needs_read_lock
     def get_inventory_xml(self, revision_id):
         """Get inventory XML as a file object."""
@@ -480,11 +506,12 @@
         :return: a dictionary of revision_id->revision_parents_list.
         """
         # special case NULL_REVISION
-        if revision_id == NULL_REVISION:
+        if revision_id == _mod_revision.NULL_REVISION:
             return {}
-        weave = self.get_inventory_weave()
-        all_revisions = self._eliminate_revisions_not_present(weave.versions())
-        entire_graph = dict([(node, weave.get_parents(node)) for 
+        a_weave = self.get_inventory_weave()
+        all_revisions = self._eliminate_revisions_not_present(
+                                a_weave.versions())
+        entire_graph = dict([(node, a_weave.get_parents(node)) for 
                              node in all_revisions])
         if revision_id is None:
             return entire_graph
@@ -509,15 +536,15 @@
         :param revision_ids: an iterable of revisions to graph or None for all.
         :return: a Graph object with the graph reachable from revision_ids.
         """
-        result = Graph()
+        result = graph.Graph()
         if not revision_ids:
             pending = set(self.all_revision_ids())
             required = set([])
         else:
             pending = set(revision_ids)
             # special case NULL_REVISION
-            if NULL_REVISION in pending:
-                pending.remove(NULL_REVISION)
+            if _mod_revision.NULL_REVISION in pending:
+                pending.remove(_mod_revision.NULL_REVISION)
             required = set(pending)
         done = set([])
         while len(pending):
@@ -577,8 +604,9 @@
         """
         # TODO: refactor this to use an existing revision object
         # so we don't need to read it in twice.
-        if revision_id is None or revision_id == NULL_REVISION:
-            return RevisionTree(self, Inventory(), NULL_REVISION)
+        if revision_id is None or revision_id == _mod_revision.NULL_REVISION:
+            return RevisionTree(self, Inventory(root_id=None), 
+                                _mod_revision.NULL_REVISION)
         else:
             inv = self.get_revision_inventory(revision_id)
             return RevisionTree(self, inv, revision_id)
@@ -589,7 +617,7 @@
 
         `revision_id` may not be None or 'null:'"""
         assert None not in revision_ids
-        assert NULL_REVISION not in revision_ids
+        assert _mod_revision.NULL_REVISION not in revision_ids
         texts = self.get_inventory_weave().get_texts(revision_ids)
         for text, revision_id in zip(texts, revision_ids):
             inv = self.deserialise_inventory(revision_id, text)
@@ -697,6 +725,9 @@
         warning("Format %s for %s is deprecated - please use 'bzr upgrade' to get better performance"
                 % (self._format, self.bzrdir.transport.base))
 
+    def supports_rich_root(self):
+        return self._format.rich_root_data
+
 
 class AllInOneRepository(Repository):
     """Legacy support - the repository behaviour for all-in-one branches."""
@@ -765,10 +796,12 @@
             parent_trees[p_id] = repository.revision_tree(None)
 
     inv = revision_tree.inventory
-    
+    entries = inv.iter_entries()
     # backwards compatability hack: skip the root id.
-    entries = inv.iter_entries()
-    entries.next()
+    if not repository.supports_rich_root():
+        path, root = entries.next()
+        if root.revision != rev.revision_id:
+            raise errors.IncompatibleRevision(repr(repository))
     # Add the texts that are not already present
     for path, ie in entries:
         w = repository.weave_store.get_weave_or_empty(ie.file_id,
@@ -911,13 +944,13 @@
         :return: a dictionary of revision_id->revision_parents_list.
         """
         # special case NULL_REVISION
-        if revision_id == NULL_REVISION:
+        if revision_id == _mod_revision.NULL_REVISION:
             return {}
-        weave = self._get_revision_vf()
-        entire_graph = weave.get_graph()
+        a_weave = self._get_revision_vf()
+        entire_graph = a_weave.get_graph()
         if revision_id is None:
-            return weave.get_graph()
-        elif revision_id not in weave:
+            return a_weave.get_graph()
+        elif revision_id not in a_weave:
             raise errors.NoSuchRevision(self, revision_id)
         else:
             # add what can be reached from revision_id
@@ -925,7 +958,7 @@
             pending = set([revision_id])
             while len(pending) > 0:
                 node = pending.pop()
-                result[node] = weave.get_parents(node)
+                result[node] = a_weave.get_parents(node)
                 for revision_id in result[node]:
                     if revision_id not in result:
                         pending.add(revision_id)
@@ -938,7 +971,7 @@
         :param revision_ids: an iterable of revisions to graph or None for all.
         :return: a Graph object with the graph reachable from revision_ids.
         """
-        result = Graph()
+        result = graph.Graph()
         vf = self._get_revision_vf()
         versions = set(vf.versions())
         if not revision_ids:
@@ -947,8 +980,8 @@
         else:
             pending = set(revision_ids)
             # special case NULL_REVISION
-            if NULL_REVISION in pending:
-                pending.remove(NULL_REVISION)
+            if _mod_revision.NULL_REVISION in pending:
+                pending.remove(_mod_revision.NULL_REVISION)
             required = set(pending)
         done = set([])
         while len(pending):
@@ -989,6 +1022,52 @@
         return self._get_revision_vf().get_parents(revision_id)
 
 
+class KnitRepository2(KnitRepository):
+    """"""
+    def __init__(self, _format, a_bzrdir, control_files, _revision_store,
+                 control_store, text_store):
+        KnitRepository.__init__(self, _format, a_bzrdir, control_files,
+                              _revision_store, control_store, text_store)
+        self._serializer = xml6.serializer_v6
+
+    def deserialise_inventory(self, revision_id, xml):
+        """Transform the xml into an inventory object. 
+
+        :param revision_id: The expected revision id of the inventory.
+        :param xml: A serialised inventory.
+        """
+        result = self._serializer.read_inventory_from_string(xml)
+        assert result.root.revision is not None
+        return result
+
+    def serialise_inventory(self, inv):
+        """Transform the inventory object into XML text.
+
+        :param revision_id: The expected revision id of the inventory.
+        :param xml: A serialised inventory.
+        """
+        assert inv.revision_id is not None
+        assert inv.root.revision is not None
+        return KnitRepository.serialise_inventory(self, inv)
+
+    def get_commit_builder(self, branch, parents, config, timestamp=None, 
+                           timezone=None, committer=None, revprops=None, 
+                           revision_id=None):
+        """Obtain a CommitBuilder for this repository.
+        
+        :param branch: Branch to commit to.
+        :param parents: Revision ids of the parents of the new revision.
+        :param config: Configuration to use.
+        :param timestamp: Optional timestamp recorded for commit.
+        :param timezone: Optional timezone for timestamp.
+        :param committer: Optional committer to set for commit.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.
+        """
+        return RootCommitBuilder(self, parents, config, timestamp, timezone,
+                                 committer, revprops, revision_id)
+
+
 class RepositoryFormat(object):
     """A repository format.
 
@@ -1087,7 +1166,8 @@
                                   transport,
                                   control_files,
                                   prefixed=True,
-                                  versionedfile_class=WeaveFile,
+                                  versionedfile_class=weave.WeaveFile,
+                                  versionedfile_kwargs={},
                                   escaped=False):
         weave_transport = control_files._transport.clone(name)
         dir_mode = control_files._dir_mode
@@ -1096,6 +1176,7 @@
                                   dir_mode=dir_mode,
                                   file_mode=file_mode,
                                   versionedfile_class=versionedfile_class,
+                                  versionedfile_kwargs=versionedfile_kwargs,
                                   escaped=escaped)
 
     def initialize(self, a_bzrdir, shared=False):
@@ -1117,6 +1198,9 @@
         """
         return True
 
+    def check_conversion_target(self, target_format):
+        raise NotImplementedError(self.check_conversion_target)
+
     def open(self, a_bzrdir, _found=False):
         """Return an instance of this format for the bzrdir a_bzrdir.
         
@@ -1141,15 +1225,14 @@
 class PreSplitOutRepositoryFormat(RepositoryFormat):
     """Base class for the pre split out repository formats."""
 
+    rich_root_data = False
+
     def initialize(self, a_bzrdir, shared=False, _internal=False):
         """Create a weave repository.
         
         TODO: when creating split out bzr branch formats, move this to a common
         base for Format5, Format6. or something like that.
         """
-        from bzrlib.weavefile import write_weave_v5
-        from bzrlib.weave import Weave
-
         if shared:
             raise errors.IncompatibleFormat(self, a_bzrdir._format)
 
@@ -1159,7 +1242,7 @@
         
         # Create an empty weave
         sio = StringIO()
-        write_weave_v5(Weave(), sio)
+        weavefile.write_weave_v5(weave.Weave(), sio)
         empty_weave = sio.getvalue()
 
         mutter('creating repository in %s.', a_bzrdir.transport.base)
@@ -1169,8 +1252,8 @@
         
         # FIXME: RBC 20060125 don't peek under the covers
         # NB: no need to escape relative paths that are url safe.
-        control_files = LockableFiles(a_bzrdir.transport, 'branch-lock',
-                                      TransportLock)
+        control_files = lockable_files.LockableFiles(a_bzrdir.transport,
+                                'branch-lock', lockable_files.TransportLock)
         control_files.create_lock()
         control_files.lock_write()
         control_files._transport.mkdir_multi(dirs,
@@ -1210,6 +1293,9 @@
                                   control_store=control_store,
                                   text_store=text_store)
 
+    def check_conversion_target(self, target_format):
+        pass
+
 
 class RepositoryFormat4(PreSplitOutRepositoryFormat):
     """Bzr repository format 4.
@@ -1326,6 +1412,8 @@
 class MetaDirRepositoryFormat(RepositoryFormat):
     """Common base class for the new repositories using the metadir layout."""
 
+    rich_root_data = False
+
     def __init__(self):
         super(MetaDirRepositoryFormat, self).__init__()
         self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
@@ -1335,7 +1423,8 @@
         # FIXME: RBC 20060125 don't peek under the covers
         # NB: no need to escape relative paths that are url safe.
         repository_transport = a_bzrdir.get_repository_transport(self)
-        control_files = LockableFiles(repository_transport, 'lock', LockDir)
+        control_files = lockable_files.LockableFiles(repository_transport,
+                                'lock', lockdir.LockDir)
         control_files.create_lock()
         return control_files
 
@@ -1383,6 +1472,9 @@
         """See RepositoryFormat.get_format_description()."""
         return "Weave repository format 7"
 
+    def check_conversion_target(self, target_format):
+        pass
+
     def _get_revision_store(self, repo_transport, control_files):
         """See RepositoryFormat._get_revision_store()."""
         return self._get_text_rev_store(repo_transport,
@@ -1404,12 +1496,9 @@
         :param shared: If true the repository will be initialized as a shared
                        repository.
         """
-        from bzrlib.weavefile import write_weave_v5
-        from bzrlib.weave import Weave
-
         # Create an empty weave
         sio = StringIO()
-        write_weave_v5(Weave(), sio)
+        weavefile.write_weave_v5(weave.Weave(), sio)
         empty_weave = sio.getvalue()
 
         mutter('creating repository in %s.', a_bzrdir.transport.base)
@@ -1435,7 +1524,8 @@
             repo_transport = _override_transport
         else:
             repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = LockableFiles(repo_transport, 'lock', LockDir)
+        control_files = lockable_files.LockableFiles(repo_transport,
+                                'lock', lockdir.LockDir)
         text_store = self._get_text_store(repo_transport, control_files)
         control_store = self._get_control_store(repo_transport, control_files)
         _revision_store = self._get_revision_store(repo_transport, control_files)
@@ -1447,8 +1537,8 @@
                                  text_store=text_store)
 
 
-class RepositoryFormatKnit1(MetaDirRepositoryFormat):
-    """Bzr repository knit format 1.
+class RepositoryFormatKnit(MetaDirRepositoryFormat):
+    """Bzr repository knit format (generalized). 
 
     This repository format has:
      - knits for file texts and inventory
@@ -1459,8 +1549,6 @@
      - an optional 'shared-storage' flag
      - an optional 'no-working-trees' flag
      - a LockDir lock
-
-    This format was introduced in bzr 0.8.
     """
 
     def _get_control_store(self, repo_transport, control_files):
@@ -1469,18 +1557,10 @@
             repo_transport,
             prefixed=False,
             file_mode=control_files._file_mode,
-            versionedfile_class=KnitVersionedFile,
-            versionedfile_kwargs={'factory':KnitPlainFactory()},
+            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_kwargs={'factory':knit.KnitPlainFactory()},
             )
 
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar-NG Knit Repository Format 1"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Knit repository format 1"
-
     def _get_revision_store(self, repo_transport, control_files):
         """See RepositoryFormat._get_revision_store()."""
         from bzrlib.store.revision.knit import KnitRevisionStore
@@ -1489,8 +1569,10 @@
             file_mode=control_files._file_mode,
             prefixed=False,
             precious=True,
-            versionedfile_class=KnitVersionedFile,
-            versionedfile_kwargs={'delta':False, 'factory':KnitPlainFactory()},
+            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_kwargs={'delta':False,
+                                  'factory':knit.KnitPlainFactory(),
+                                 },
             escaped=True,
             )
         return KnitRevisionStore(versioned_file_store)
@@ -1498,10 +1580,15 @@
     def _get_text_store(self, transport, control_files):
         """See RepositoryFormat._get_text_store()."""
         return self._get_versioned_file_store('knits',
-                                              transport,
-                                              control_files,
-                                              versionedfile_class=KnitVersionedFile,
-                                              escaped=True)
+                                  transport,
+                                  control_files,
+                                  versionedfile_class=knit.KnitVersionedFile,
+                                  versionedfile_kwargs={
+                                      'create_parent_dir':True,
+                                      'delay_create':True,
+                                      'dir_mode':control_files._dir_mode,
+                                  },
+                                  escaped=True)
 
     def initialize(self, a_bzrdir, shared=False):
         """Create a knit format 1 repository.
@@ -1518,7 +1605,8 @@
         
         self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
         repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = LockableFiles(repo_transport, 'lock', LockDir)
+        control_files = lockable_files.LockableFiles(repo_transport,
+                                'lock', lockdir.LockDir)
         control_store = self._get_control_store(repo_transport, control_files)
         transaction = transactions.WriteTransaction()
         # trigger a write of the inventory store.
@@ -1542,7 +1630,8 @@
             repo_transport = _override_transport
         else:
             repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = LockableFiles(repo_transport, 'lock', LockDir)
+        control_files = lockable_files.LockableFiles(repo_transport,
+                                'lock', lockdir.LockDir)
         text_store = self._get_text_store(repo_transport, control_files)
         control_store = self._get_control_store(repo_transport, control_files)
         _revision_store = self._get_revision_store(repo_transport, control_files)
@@ -1554,11 +1643,99 @@
                               text_store=text_store)
 
 
+class RepositoryFormatKnit1(RepositoryFormatKnit):
+    """Bzr repository knit format 1.
+
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+
+    This format was introduced in bzr 0.8.
+    """
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar-NG Knit Repository Format 1"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Knit repository format 1"
+
+    def check_conversion_target(self, target_format):
+        pass
+
+
+class RepositoryFormatKnit2(RepositoryFormatKnit):
+    """Bzr repository knit format 2.
+
+    THIS FORMAT IS EXPERIMENTAL
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+     - Support for recording full info about the tree root
+
+    """
+    
+    rich_root_data = True
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar Knit Repository Format 2\n"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Knit repository format 2"
+
+    def check_conversion_target(self, target_format):
+        if not target_format.rich_root_data:
+            raise errors.BadConversionTarget(
+                'Does not support rich root data.', target_format)
+
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = lockable_files.LockableFiles(repo_transport, 'lock',
+                                                     lockdir.LockDir)
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return KnitRepository2(_format=self,
+                               a_bzrdir=a_bzrdir,
+                               control_files=control_files,
+                               _revision_store=_revision_store,
+                               control_store=control_store,
+                               text_store=text_store)
+
+
+
 # formats which have no format string are not discoverable
 # and not independently creatable, so are not registered.
 RepositoryFormat.register_format(RepositoryFormat7())
 _default_format = RepositoryFormatKnit1()
 RepositoryFormat.register_format(_default_format)
+RepositoryFormat.register_format(RepositoryFormatKnit2())
 RepositoryFormat.set_default_format(_default_format)
 _legacy_formats = [RepositoryFormat4(),
                    RepositoryFormat5(),
@@ -1577,38 +1754,16 @@
     InterRepository.get(other).method_name(parameters).
     """
 
-    _optimisers = set()
+    _optimisers = []
     """The available optimised InterRepository types."""
 
-    @needs_write_lock
     def copy_content(self, revision_id=None, basis=None):
-        """Make a complete copy of the content in self into destination.
-        
-        This is a destructive operation! Do not use it on existing 
-        repositories.
-
-        :param revision_id: Only copy the content needed to construct
-                            revision_id and its parents.
-        :param basis: Copy the needed data preferentially from basis.
-        """
-        try:
-            self.target.set_make_working_trees(self.source.make_working_trees())
-        except NotImplementedError:
-            pass
-        # grab the basis available data
-        if basis is not None:
-            self.target.fetch(basis, revision_id=revision_id)
-        # but don't bother fetching if we have the needed data now.
-        if (revision_id not in (None, NULL_REVISION) and 
-            self.target.has_revision(revision_id)):
-            return
-        self.target.fetch(self.source, revision_id=revision_id)
-
-    @needs_write_lock
+        raise NotImplementedError(self.copy_content)
+
     def fetch(self, revision_id=None, pb=None):
         """Fetch the content required to construct revision_id.
 
-        The content is copied from source to target.
+        The content is copied from self.source to self.target.
 
         :param revision_id: if None all content is copied, if NULL_REVISION no
                             content is copied.
@@ -1618,15 +1773,8 @@
         Returns the copied revision count and the failed revisions in a tuple:
         (copied, failures).
         """
-        from bzrlib.fetch import GenericRepoFetcher
-        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
-               self.source, self.source._format, self.target, self.target._format)
-        f = GenericRepoFetcher(to_repository=self.target,
-                               from_repository=self.source,
-                               last_revision=revision_id,
-                               pb=pb)
-        return f.count_copied, f.failed_revisions
-
+        raise NotImplementedError(self.fetch)
+   
     @needs_read_lock
     def missing_revision_ids(self, revision_id=None):
         """Return the revision ids that source has that target does not.
@@ -1640,7 +1788,7 @@
         target_ids = set(self.target.all_revision_ids())
         if revision_id is not None:
             source_ids = self.source.get_ancestry(revision_id)
-            assert source_ids[0] == None
+            assert source_ids[0] is None
             source_ids.pop(0)
         else:
             source_ids = self.source.all_revision_ids()
@@ -1651,7 +1799,65 @@
         return [rev_id for rev_id in source_ids if rev_id in result_set]
 
 
-class InterWeaveRepo(InterRepository):
+class InterSameDataRepository(InterRepository):
+    """Code for converting between repositories that represent the same data.
+    
+    Data format and model must match for this to work.
+    """
+
+    _matching_repo_format = RepositoryFormat4()
+    """Repository format for testing with."""
+
+    @staticmethod
+    def is_compatible(source, target):
+        if not isinstance(source, Repository):
+            return False
+        if not isinstance(target, Repository):
+            return False
+        if source._format.rich_root_data == target._format.rich_root_data:
+            return True
+        else:
+            return False
+
+    @needs_write_lock
+    def copy_content(self, revision_id=None, basis=None):
+        """Make a complete copy of the content in self into destination.
+        
+        This is a destructive operation! Do not use it on existing 
+        repositories.
+
+        :param revision_id: Only copy the content needed to construct
+                            revision_id and its parents.
+        :param basis: Copy the needed data preferentially from basis.
+        """
+        try:
+            self.target.set_make_working_trees(self.source.make_working_trees())
+        except NotImplementedError:
+            pass
+        # grab the basis available data
+        if basis is not None:
+            self.target.fetch(basis, revision_id=revision_id)
+        # but don't bother fetching if we have the needed data now.
+        if (revision_id not in (None, _mod_revision.NULL_REVISION) and 
+            self.target.has_revision(revision_id)):
+            return
+        self.target.fetch(self.source, revision_id=revision_id)
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import GenericRepoFetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, 
+               self.target._format)
+        f = GenericRepoFetcher(to_repository=self.target,
+                               from_repository=self.source,
+                               last_revision=revision_id,
+                               pb=pb)
+        return f.count_copied, f.failed_revisions
+
+
+class InterWeaveRepo(InterSameDataRepository):
     """Optimised code paths between Weave based repositories."""
 
     _matching_repo_format = RepositoryFormat7()
@@ -1743,7 +1949,7 @@
         # - RBC 20060209
         if revision_id is not None:
             source_ids = self.source.get_ancestry(revision_id)
-            assert source_ids[0] == None
+            assert source_ids[0] is None
             source_ids.pop(0)
         else:
             source_ids = self.source._all_possible_ids()
@@ -1769,7 +1975,7 @@
             return self.source._eliminate_revisions_not_present(required_topo_revisions)
 
 
-class InterKnitRepo(InterRepository):
+class InterKnitRepo(InterSameDataRepository):
     """Optimised code paths between Knit based repositories."""
 
     _matching_repo_format = RepositoryFormatKnit1()
@@ -1806,7 +2012,7 @@
         """See InterRepository.missing_revision_ids()."""
         if revision_id is not None:
             source_ids = self.source.get_ancestry(revision_id)
-            assert source_ids[0] == None
+            assert source_ids[0] is None
             source_ids.pop(0)
         else:
             source_ids = self.source._all_possible_ids()
@@ -1831,8 +2037,89 @@
             # that against the revision records.
             return self.source._eliminate_revisions_not_present(required_topo_revisions)
 
+
+class InterModel1and2(InterRepository):
+
+    _matching_repo_format = None
+
+    @staticmethod
+    def is_compatible(source, target):
+        if not isinstance(source, Repository):
+            return False
+        if not isinstance(target, Repository):
+            return False
+        if not source._format.rich_root_data and target._format.rich_root_data:
+            return True
+        else:
+            return False
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import Model1toKnit2Fetcher
+        f = Model1toKnit2Fetcher(to_repository=self.target,
+                                 from_repository=self.source,
+                                 last_revision=revision_id,
+                                 pb=pb)
+        return f.count_copied, f.failed_revisions
+
+    @needs_write_lock
+    def copy_content(self, revision_id=None, basis=None):
+        """Make a complete copy of the content in self into destination.
+        
+        This is a destructive operation! Do not use it on existing 
+        repositories.
+
+        :param revision_id: Only copy the content needed to construct
+                            revision_id and its parents.
+        :param basis: Copy the needed data preferentially from basis.
+        """
+        try:
+            self.target.set_make_working_trees(self.source.make_working_trees())
+        except NotImplementedError:
+            pass
+        # grab the basis available data
+        if basis is not None:
+            self.target.fetch(basis, revision_id=revision_id)
+        # but don't bother fetching if we have the needed data now.
+        if (revision_id not in (None, _mod_revision.NULL_REVISION) and 
+            self.target.has_revision(revision_id)):
+            return
+        self.target.fetch(self.source, revision_id=revision_id)
+
+
+class InterKnit1and2(InterKnitRepo):
+
+    _matching_repo_format = None
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with Knit1 source and Knit2 target"""
+        try:
+            return (isinstance(source._format, (RepositoryFormatKnit1)) and
+                    isinstance(target._format, (RepositoryFormatKnit2)))
+        except AttributeError:
+            return False
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import Knit1to2Fetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, 
+               self.target._format)
+        f = Knit1to2Fetcher(to_repository=self.target,
+                            from_repository=self.source,
+                            last_revision=revision_id,
+                            pb=pb)
+        return f.count_copied, f.failed_revisions
+
+
+InterRepository.register_optimiser(InterSameDataRepository)
 InterRepository.register_optimiser(InterWeaveRepo)
 InterRepository.register_optimiser(InterKnitRepo)
+InterRepository.register_optimiser(InterModel1and2)
+InterRepository.register_optimiser(InterKnit1and2)
 
 
 class RepositoryTestProviderAdapter(object):
@@ -1850,7 +2137,7 @@
         self._formats = formats
     
     def adapt(self, test):
-        result = TestSuite()
+        result = unittest.TestSuite()
         for repository_format, bzrdir_format in self._formats:
             new_test = deepcopy(test)
             new_test.transport_server = self._transport_server
@@ -1880,7 +2167,7 @@
         self._formats = formats
     
     def adapt(self, test):
-        result = TestSuite()
+        result = unittest.TestSuite()
         for interrepo_class, repository_format, repository_format_to in self._formats:
             new_test = deepcopy(test)
             new_test.transport_server = self._transport_server
@@ -1903,16 +2190,21 @@
         # default format.
         # XXX: robertc 20060220 reinstate this when there are two supported
         # formats which do not have an optimal code path between them.
-        result.append((InterRepository,
-                       RepositoryFormat6(),
-                       RepositoryFormatKnit1()))
+        #result.append((InterRepository,
+        #               RepositoryFormat6(),
+        #               RepositoryFormatKnit1()))
         for optimiser in InterRepository._optimisers:
-            result.append((optimiser,
-                           optimiser._matching_repo_format,
-                           optimiser._matching_repo_format
-                           ))
+            if optimiser._matching_repo_format is not None:
+                result.append((optimiser,
+                               optimiser._matching_repo_format,
+                               optimiser._matching_repo_format
+                               ))
         # if there are specific combinations we want to use, we can add them 
         # here.
+        result.append((InterModel1and2, RepositoryFormat5(),
+                       RepositoryFormatKnit2()))
+        result.append((InterKnit1and2, RepositoryFormatKnit1(),
+                       RepositoryFormatKnit2()))
         return result
 
 
@@ -1945,6 +2237,7 @@
         self.step('Moving repository to repository.backup')
         self.repo_dir.transport.move('repository', 'repository.backup')
         backup_transport =  self.repo_dir.transport.clone('repository.backup')
+        repo._format.check_conversion_target(self.target_format)
         self.source_repo = repo._format.open(self.repo_dir,
             _found=True,
             _override_transport=backup_transport)
@@ -2023,7 +2316,8 @@
 
         :return: The revision id of the recorded revision.
         """
-        rev = Revision(timestamp=self._timestamp,
+        rev = _mod_revision.Revision(
+                       timestamp=self._timestamp,
                        timezone=self._timezone,
                        committer=self._committer,
                        message=message,
@@ -2035,6 +2329,18 @@
             self.new_inventory, self._config)
         return self._new_revision_id
 
+    def revision_tree(self):
+        """Return the tree that was just committed.
+
+        After calling commit() this can be called to get a RevisionTree
+        representing the newly committed tree. This is preferred to
+        calling Repository.revision_tree() because that may require
+        deserializing the inventory, while we already have a copy in
+        memory.
+        """
+        return RevisionTree(self.repository, self.new_inventory,
+                            self._new_revision_id)
+
     def finish_inventory(self):
         """Tell the builder that the inventory is finished."""
         if self.new_inventory.root is None:
@@ -2096,10 +2402,9 @@
 
         # In this revision format, root entries have no knit or weave
         if ie is self.new_inventory.root:
-            if len(parent_invs):
-                ie.revision = parent_invs[0].root.revision
-            else:
-                ie.revision = None
+            # When serializing out to disk and back in
+            # root.revision is always _new_revision_id
+            ie.revision = self._new_revision_id
             return
         previous_entries = ie.find_previous_heads(
             parent_invs,
@@ -2175,6 +2480,41 @@
     record_root_entry = True
 
 
+class RootCommitBuilder(CommitBuilder):
+    """This commitbuilder actually records the root id"""
+    
+    record_root_entry = True
+
+    def record_entry_contents(self, ie, parent_invs, path, tree):
+        """Record the content of ie from tree into the commit if needed.
+
+        Side effect: sets ie.revision when unchanged
+
+        :param ie: An inventory entry present in the commit.
+        :param parent_invs: The inventories of the parent revisions of the
+            commit.
+        :param path: The path the entry is at in the tree.
+        :param tree: The tree which contains this entry and should be used to 
+        obtain content.
+        """
+        assert self.new_inventory.root is not None or ie.parent_id is None
+        self.new_inventory.add(ie)
+
+        # ie.revision is always None if the InventoryEntry is considered
+        # for committing. ie.snapshot will record the correct revision 
+        # which may be the sole parent if it is untouched.
+        if ie.revision is not None:
+            return
+
+        previous_entries = ie.find_previous_heads(
+            parent_invs,
+            self.repository.weave_store,
+            self.repository.get_transaction())
+        # we are creating a new revision for ie in the history store
+        # and inventory.
+        ie.snapshot(self._new_revision_id, path, previous_entries, tree, self)
+
+
 _unescape_map = {
     'apos':"'",
     'quot':'"',

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2006-08-28 16:28:10 +0000
+++ bzrlib/revisionspec.py	2006-09-03 15:58:15 +0000
@@ -224,6 +224,13 @@
         """
         return True
 
+    def get_branch(self):
+        """When the revision specifier contains a branch location, return it.
+        
+        Otherwise, return None.
+        """
+        return None
+
 
 # private API
 
@@ -273,6 +280,12 @@
     def needs_branch(self):
         return self.spec.find(':') == -1
 
+    def get_branch(self):
+        if self.spec.find(':') == -1:
+            return None
+        else:
+            return self.spec[self.spec.find(':')+1:]
+
 # Old compatibility 
 RevisionSpec_int = RevisionSpec_revno
 

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2006-08-15 03:09:14 +0000
+++ bzrlib/revisiontree.py	2006-09-28 01:53:23 +0000
@@ -40,7 +40,6 @@
         self._repository = branch
         self._weave_store = branch.weave_store
         self._inventory = inv
-        assert inv.root is not None
         self._revision_id = revision_id
 
     def get_parent_ids(self):
@@ -48,7 +47,11 @@
 
         A RevisionTree's parents match the revision graph.
         """
-        parent_ids = self._repository.get_revision(self._revision_id).parent_ids
+        if self._revision_id not in (None, 'null:'):
+            parent_ids = self._repository.get_revision(
+                self._revision_id).parent_ids
+        else:
+            parent_ids = []
         return parent_ids
         
     def get_revision_id(self):
@@ -93,11 +96,13 @@
     def has_filename(self, filename):
         return bool(self.inventory.path2id(filename))
 
-    def list_files(self):
+    def list_files(self, include_root=False):
         # The only files returned by this are those from the version
         entries = self.inventory.iter_entries()
         # skip the root for compatability with the current apis.
-        entries.next()
+        if self.inventory.root is not None and not include_root:
+            # skip the root for compatability with the current apis.
+            entries.next()
         for path, entry in entries:
             yield path, 'V', entry.kind, entry.file_id, entry
 

=== modified file 'bzrlib/rio.py'
--- bzrlib/rio.py	2006-03-09 18:47:40 +0000
+++ bzrlib/rio.py	2006-09-22 19:37:57 +0000
@@ -171,6 +171,28 @@
         """Return stanza as a single string"""
         return ''.join(self.to_lines())
 
+    def to_unicode(self):
+        """Return stanza as a single Unicode string.
+
+        This is most useful when adding a Stanza to a parent Stanza
+        """
+        if not self.items:
+            return u''
+
+        result = []
+        for tag, value in self.items:
+            if value == '':
+                result.append(tag + ': \n')
+            elif '\n' in value:
+                # don't want splitlines behaviour on empty lines
+                val_lines = value.split('\n')
+                result.append(tag + ': ' + val_lines[0] + '\n')
+                for line in val_lines[1:]:
+                    result.append('\t' + line + '\n')
+            else:
+                result.append(tag + ': ' + value + '\n')
+        return u''.join(result)
+
     def write(self, to_file):
         """Write stanza to a file"""
         to_file.writelines(self.to_lines())
@@ -223,17 +245,41 @@
 
     The raw lines must be in utf-8 encoding.
     """
-    items = []
+    unicode_iter = (line.decode('utf-8') for line in line_iter)
+    return read_stanza_unicode(unicode_iter)
+
+
+def read_stanza_unicode(unicode_iter):
+    """Read a Stanza from a list of lines or a file.
+
+    The lines should already be in unicode form. This returns a single
+    stanza that was read. If there is a blank line at the end of the Stanza,
+    it is consumed. It is not an error for there to be no blank line at
+    the end of the iterable. If there is a blank line at the beginning,
+    this is treated as an empty Stanza and None is returned.
+
+    Only the stanza lines and the trailing blank (if any) are consumed
+    from the unicode_iter
+
+    :param unicode_iter: A iterable, yeilding Unicode strings. See read_stanza
+        if you have a utf-8 encoded string.
+    :return: A Stanza object if there are any lines in the file.
+        None otherwise
+    """
     stanza = Stanza()
     tag = None
     accum_value = None
-    for line in line_iter:
-        if line == None or line == '':
+    
+    # TODO: jam 20060922 This code should raise real errors rather than
+    #       using 'assert' to process user input, or raising ValueError
+    #       rather than a more specific error.
+
+    for line in unicode_iter:
+        if line is None or line == '':
             break       # end of file
         if line == '\n':
             break       # end of stanza
-        line = line.decode('utf-8')
-        assert line[-1] == '\n'
+        assert line.endswith('\n')
         real_l = line
         if line[0] == '\t': # continues previous value
             if tag is None:
@@ -245,11 +291,13 @@
             try:
                 colon_index = line.index(': ')
             except ValueError:
-                raise ValueError('tag/value separator not found in line %r' % real_l)
+                raise ValueError('tag/value separator not found in line %r'
+                                 % real_l)
             tag = str(line[:colon_index])
             assert valid_tag(tag), \
                     "invalid rio tag %r" % tag
             accum_value = line[colon_index+2:-1]
+
     if tag is not None: # add last tag-value
         stanza.add(tag, accum_value)
         return stanza

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2005-11-18 08:10:07 +0000
+++ bzrlib/shellcomplete.py	2006-09-08 18:46:29 +0000
@@ -2,9 +2,9 @@
 
 
 def shellcomplete(context=None, outfile = None):
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
-    if context == None:
+    if context is None:
         shellcomplete_commands(outfile = outfile)
     else:
         shellcomplete_on_command(context, outfile = outfile)
@@ -12,7 +12,7 @@
 def shellcomplete_on_command(cmdname, outfile = None):
     cmdname = str(cmdname)
 
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
 
     from inspect import getdoc
@@ -20,7 +20,7 @@
     cmdobj = commands.get_cmd_object(cmdname)
 
     doc = getdoc(cmdobj)
-    if doc == None:
+    if doc is None:
         raise NotImplementedError("sorry, no detailed shellcomplete yet for %r" % cmdname)
 
     shellcomplete_on_option(cmdobj.takes_options, outfile = None)
@@ -32,7 +32,7 @@
     from bzrlib.option import Option
     if not options:
         return
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
     for on in options:
         for shortname, longname in Option.SHORT_OPTIONS.items():
@@ -50,7 +50,7 @@
     import commands
     from inspect import getdoc
     
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
     
     cmds = []
@@ -63,7 +63,7 @@
         if cmdclass.hidden:
             continue
         doc = getdoc(cmdclass)
-        if doc == None:
+        if doc is None:
 	    outfile.write(cmdname + '\n')
         else:
 	    doclines = doc.splitlines()

=== modified file 'bzrlib/sign_my_commits.py'
--- bzrlib/sign_my_commits.py	2006-07-28 16:05:23 +0000
+++ bzrlib/sign_my_commits.py	2006-09-13 02:44:23 +0000
@@ -17,9 +17,15 @@
 """Command which looks for unsigned commits by the current user, and signs them.
 """
 
-from bzrlib import config, gpg
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    config,
+    gpg,
+    )
+from bzrlib.bzrdir import BzrDir
+""")
 from bzrlib.commands import Command
-from bzrlib.bzrdir import BzrDir
 from bzrlib.option import Option
 
 

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2006-08-25 05:21:34 +0000
+++ bzrlib/status.py	2006-09-08 18:46:29 +0000
@@ -177,7 +177,7 @@
             m_revision = branch.repository.get_revision(merge)
             print >> to_file, ' ', line_log(m_revision, width - 3)
             inner_merges = branch.repository.get_ancestry(merge)
-            assert inner_merges[0] == None
+            assert inner_merges[0] is None
             inner_merges.pop(0)
             inner_merges.reverse()
             for mmerge in inner_merges:

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2006-09-05 20:58:28 +0000
+++ bzrlib/store/__init__.py	2006-09-08 18:52:17 +0000
@@ -94,7 +94,7 @@
 
     def listable(self):
         """Return True if this store is able to be listed."""
-        return hasattr(self, "__iter__")
+        return (getattr(self, "__iter__", None) is not None)
 
     def copy_all_ids(self, store_from, pb=None):
         """Copy all the file ids from store_from into self."""

=== modified file 'bzrlib/store/revision/__init__.py'
--- bzrlib/store/revision/__init__.py	2006-06-12 02:07:37 +0000
+++ bzrlib/store/revision/__init__.py	2006-09-12 21:19:54 +0000
@@ -20,14 +20,19 @@
 and ghosts information.
 """
 
+from cStringIO import StringIO
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from copy import deepcopy
-from cStringIO import StringIO
-from unittest import TestSuite
-
+import unittest
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    xml5,
+    )
+""")
 from bzrlib.trace import mutter
 
 
@@ -46,7 +51,7 @@
         self._factories = factories
     
     def adapt(self, test):
-        result = TestSuite()
+        result = unittest.TestSuite()
         for factory in self._factories:
             new_test = deepcopy(test)
             new_test.transport_server = self._transport_server
@@ -76,7 +81,7 @@
 
     def __init__(self, serializer=None):
         if serializer is None:
-            serializer = bzrlib.xml5.serializer_v5
+            serializer = xml5.serializer_v5
         self._serializer = serializer
 
     def add_revision(self, revision, transaction):

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2006-09-05 20:08:52 +0000
+++ bzrlib/store/text.py	2006-09-08 18:52:17 +0000
@@ -114,7 +114,7 @@
         # gzip.GzipFile.read() requires a tell() function
         # but some transports return objects that cannot seek
         # so buffer them in a StringIO instead
-        if hasattr(f, 'tell'):
+        if getattr(f, 'tell', None) is not None:
             return gzip.GzipFile(mode='rb', fileobj=f)
         else:
             from cStringIO import StringIO

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2006-09-05 21:35:51 +0000
+++ bzrlib/symbol_versioning.py	2006-09-25 01:10:49 +0000
@@ -30,6 +30,7 @@
            'zero_nine',
            'zero_ten',
            'zero_eleven',
+           'zero_twelve',
            ]
 
 from warnings import warn
@@ -41,6 +42,7 @@
 zero_nine = "%s was deprecated in version 0.9."
 zero_ten = "%s was deprecated in version 0.10."
 zero_eleven = "%s was deprecated in version 0.11."
+zero_twelve = "%s was deprecated in version 0.12."
 
 
 def set_warning_method(method):

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2006-08-16 16:55:54 +0000
+++ bzrlib/testament.py	2006-09-26 13:28:39 +0000
@@ -131,9 +131,7 @@
         for l in self.message.splitlines():
             a('  %s\n' % l)
         a('inventory:\n')
-        entries = self.inventory.iter_entries()
-        entries.next()
-        for path, ie in entries:
+        for path, ie in self._get_entries():
             a(self._entry_to_line(path, ie))
         r.extend(self._revprops_to_lines())
         if __debug__:
@@ -142,6 +140,11 @@
                     '%r of type %s is not a plain string' % (l, type(l))
         return [line.encode('utf-8') for line in r]
 
+    def _get_entries(self):
+        entries = self.inventory.iter_entries()
+        entries.next()
+        return entries
+
     def _escape_path(self, path):
         assert not contains_linebreaks(path)
         return unicode(path.replace('\\', '/').replace(' ', '\ '))
@@ -197,7 +200,7 @@
 
 
 class StrictTestament(Testament):
-    """This testament format is for use as a checksum in changesets"""
+    """This testament format is for use as a checksum in bundle format 0.8"""
 
     long_header = 'bazaar-ng testament version 2.1\n'
     short_header = 'bazaar-ng testament short form 2.1\n'
@@ -206,3 +209,21 @@
         l += ' ' + ie.revision
         l += {True: ' yes\n', False: ' no\n'}[ie.executable]
         return l
+
+
+class StrictTestament3(StrictTestament):
+    """This testament format is for use as a checksum in bundle format 0.9+
+    
+    It differs from StrictTestament by including data about the tree root.
+    """
+
+    long_header = 'bazaar testament version 3 strict\n'
+    short_header = 'bazaar testament short form 3 strict\n'
+    def _get_entries(self):
+        return self.inventory.iter_entries()
+
+    def _escape_path(self, path):
+        assert not contains_linebreaks(path)
+        if path == '':
+            path = '.'
+        return unicode(path.replace('\\', '/').replace(' ', '\ '))

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-09-06 13:01:07 +0000
+++ bzrlib/tests/__init__.py	2006-10-15 16:22:42 +0000
@@ -43,11 +43,13 @@
 import time
 
 
+from bzrlib import memorytree
 import bzrlib.branch
 import bzrlib.bzrdir as bzrdir
 import bzrlib.commands
 import bzrlib.bundle.serializer
 import bzrlib.errors as errors
+import bzrlib.export
 import bzrlib.inventory
 import bzrlib.iterablefile
 import bzrlib.lockdir
@@ -69,6 +71,7 @@
 from bzrlib.transport import get_transport
 import bzrlib.transport
 from bzrlib.transport.local import LocalRelpathServer
+from bzrlib.transport.memory import MemoryServer
 from bzrlib.transport.readonly import ReadonlyServer
 from bzrlib.trace import mutter
 from bzrlib.tests import TestUtil
@@ -84,17 +87,15 @@
 
 MODULES_TO_TEST = []
 MODULES_TO_DOCTEST = [
-                      bzrlib.branch,
                       bzrlib.bundle.serializer,
-                      bzrlib.commands,
                       bzrlib.errors,
+                      bzrlib.export,
                       bzrlib.inventory,
                       bzrlib.iterablefile,
                       bzrlib.lockdir,
                       bzrlib.merge3,
                       bzrlib.option,
-                      bzrlib.osutils,
-                      bzrlib.store
+                      bzrlib.store,
                       ]
 
 
@@ -150,7 +151,12 @@
             from bzrlib.version import _get_bzr_source_tree
             src_tree = _get_bzr_source_tree()
             if src_tree:
-                revision_id = src_tree.last_revision()
+                try:
+                    revision_id = src_tree.get_parent_ids()[0]
+                except IndexError:
+                    # XXX: if this is a brand new tree, do the same as if there
+                    # is no branch.
+                    revision_id = ''
             else:
                 # XXX: If there's no branch, what should we do?
                 revision_id = ''
@@ -233,6 +239,11 @@
         if isinstance(err[1], TestSkipped):
             return self.addSkipped(test, err)    
         unittest.TestResult.addError(self, test, err)
+        # We can only do this if we have one of our TestCases, not if
+        # we have a doctest.
+        setKeepLogfile = getattr(test, 'setKeepLogfile', None)
+        if setKeepLogfile is not None:
+            setKeepLogfile()
         self.extractBenchmarkTime(test)
         if self.showAll:
             self.stream.writeln("ERROR %s" % self._testTimeString())
@@ -249,6 +260,11 @@
 
     def addFailure(self, test, err):
         unittest.TestResult.addFailure(self, test, err)
+        # We can only do this if we have one of our TestCases, not if
+        # we have a doctest.
+        setKeepLogfile = getattr(test, 'setKeepLogfile', None)
+        if setKeepLogfile is not None:
+            setKeepLogfile()
         self.extractBenchmarkTime(test)
         if self.showAll:
             self.stream.writeln(" FAIL %s" % self._testTimeString())
@@ -375,7 +391,7 @@
         # This is still a little bogus, 
         # but only a little. Folk not using our testrunner will
         # have to delete their temp directories themselves.
-        test_root = TestCaseInTempDir.TEST_ROOT
+        test_root = TestCaseWithMemoryTransport.TEST_ROOT
         if result.wasSuccessful() or not self.keep_output:
             if test_root is not None:
                 # If LANG=C we probably have created some bogus paths
@@ -399,7 +415,7 @@
                 self.stream.writeln(
                     "Failed tests working directories are in '%s'\n" %
                     test_root)
-        TestCaseInTempDir.TEST_ROOT = None
+        TestCaseWithMemoryTransport.TEST_ROOT = None
         if self.pb is not None:
             self.pb.clear()
         return result
@@ -475,6 +491,7 @@
 
     _log_file_name = None
     _log_contents = ''
+    _keep_log_file = False
     # record lsprof data when performing benchmark calls.
     _gather_lsprof_in_benchmarks = False
 
@@ -655,16 +672,20 @@
     def _finishLogFile(self):
         """Finished with the log file.
 
-        Read contents into memory, close, and delete.
+        Close the file and delete it, unless setKeepLogfile was called.
         """
         if self._log_file is None:
             return
         bzrlib.trace.disable_test_log(self._log_nonce)
-        self._log_file.seek(0)
-        self._log_contents = self._log_file.read()
         self._log_file.close()
-        os.remove(self._log_file_name)
-        self._log_file = self._log_file_name = None
+        self._log_file = None
+        if not self._keep_log_file:
+            os.remove(self._log_file_name)
+            self._log_file_name = None
+
+    def setKeepLogfile(self):
+        """Make the logfile not be deleted when _finishLogFile is called."""
+        self._keep_log_file = True
 
     def addCleanup(self, callable):
         """Arrange to run a callable when this case is torn down.
@@ -679,38 +700,26 @@
 
     def _cleanEnvironment(self):
         new_env = {
+            'BZR_HOME': None, # Don't inherit BZR_HOME to all the tests.
             'HOME': os.getcwd(),
             'APPDATA': os.getcwd(),
             'BZR_EMAIL': None,
             'BZREMAIL': None, # may still be present in the environment
             'EMAIL': None,
+            'BZR_PROGRESS_BAR': None,
         }
         self.__old_env = {}
         self.addCleanup(self._restoreEnvironment)
         for name, value in new_env.iteritems():
             self._captureVar(name, value)
 
-
     def _captureVar(self, name, newvalue):
-        """Set an environment variable, preparing it to be reset when finished."""
-        self.__old_env[name] = os.environ.get(name, None)
-        if newvalue is None:
-            if name in os.environ:
-                del os.environ[name]
-        else:
-            os.environ[name] = newvalue
-
-    @staticmethod
-    def _restoreVar(name, value):
-        if value is None:
-            if name in os.environ:
-                del os.environ[name]
-        else:
-            os.environ[name] = value
+        """Set an environment variable, and reset it when finished."""
+        self.__old_env[name] = osutils.set_or_unset_env(name, newvalue)
 
     def _restoreEnvironment(self):
         for name, value in self.__old_env.iteritems():
-            self._restoreVar(name, value)
+            osutils.set_or_unset_env(name, value)
 
     def tearDown(self):
         self._runCleanups()
@@ -751,19 +760,34 @@
     def log(self, *args):
         mutter(*args)
 
-    def _get_log(self):
-        """Return as a string the log for this test"""
-        if self._log_file_name:
-            return open(self._log_file_name).read()
-        else:
+    def _get_log(self, keep_log_file=False):
+        """Return as a string the log for this test. If the file is still
+        on disk and keep_log_file=False, delete the log file and store the
+        content in self._log_contents."""
+        # flush the log file, to get all content
+        import bzrlib.trace
+        bzrlib.trace._trace_file.flush()
+        if self._log_contents:
             return self._log_contents
-        # TODO: Delete the log after it's been read in
+        if self._log_file_name is not None:
+            logfile = open(self._log_file_name)
+            try:
+                log_contents = logfile.read()
+            finally:
+                logfile.close()
+            if not keep_log_file:
+                self._log_contents = log_contents
+                os.remove(self._log_file_name)
+            return log_contents
+        else:
+            return "DELETED log file to reduce memory footprint"
 
     def capture(self, cmd, retcode=0):
         """Shortcut that splits cmd into words, runs, and returns stdout"""
         return self.run_bzr_captured(cmd.split(), retcode=retcode)[0]
 
-    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None):
+    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None,
+                         working_dir=None):
         """Invoke bzr and return (stdout, stderr).
 
         Useful for code that wants to check the contents of the
@@ -784,6 +808,7 @@
         :param retcode: expected return code, or None for don't-care.
         :param encoding: encoding for sys.stdout and sys.stderr
         :param stdin: A string to be used as stdin for the command.
+        :param working_dir: Change to this directory before running
         """
         if encoding is None:
             encoding = bzrlib.user_encoding
@@ -805,6 +830,12 @@
             stdout=stdout,
             stderr=stderr)
         bzrlib.ui.ui_factory.stdin = stdin
+
+        cwd = None
+        if working_dir is not None:
+            cwd = osutils.getcwd()
+            os.chdir(working_dir)
+
         try:
             result = self.apply_redirected(stdin, stdout, stderr,
                                            bzrlib.commands.run_bzr_catch_errors,
@@ -812,6 +843,8 @@
         finally:
             logger.removeHandler(handler)
             bzrlib.ui.ui_factory = old_ui_factory
+            if cwd is not None:
+                os.chdir(cwd)
 
         out = stdout.getvalue()
         err = stderr.getvalue()
@@ -838,7 +871,9 @@
         retcode = kwargs.pop('retcode', 0)
         encoding = kwargs.pop('encoding', None)
         stdin = kwargs.pop('stdin', None)
-        return self.run_bzr_captured(args, retcode=retcode, encoding=encoding, stdin=stdin)
+        working_dir = kwargs.pop('working_dir', None)
+        return self.run_bzr_captured(args, retcode=retcode, encoding=encoding,
+                                     stdin=stdin, working_dir=working_dir)
 
     def run_bzr_decode(self, *args, **kwargs):
         if 'encoding' in kwargs:
@@ -891,27 +926,130 @@
             variables. A value of None will unset the env variable.
             The values must be strings. The change will only occur in the
             child, so you don't need to fix the environment after running.
+        :param universal_newlines: Convert CRLF => LF
+        :param allow_plugins: By default the subprocess is run with
+            --no-plugins to ensure test reproducibility. Also, it is possible
+            for system-wide plugins to create unexpected output on stderr,
+            which can cause unnecessary test failures.
         """
         env_changes = kwargs.get('env_changes', {})
+        working_dir = kwargs.get('working_dir', None)
+        allow_plugins = kwargs.get('allow_plugins', False)
+        process = self.start_bzr_subprocess(args, env_changes=env_changes,
+                                            working_dir=working_dir,
+                                            allow_plugins=allow_plugins)
+        # We distinguish between retcode=None and retcode not passed.
+        supplied_retcode = kwargs.get('retcode', 0)
+        return self.finish_bzr_subprocess(process, retcode=supplied_retcode,
+            universal_newlines=kwargs.get('universal_newlines', False),
+            process_args=args)
+
+    def start_bzr_subprocess(self, process_args, env_changes=None,
+                             skip_if_plan_to_signal=False,
+                             working_dir=None,
+                             allow_plugins=False):
+        """Start bzr in a subprocess for testing.
+
+        This starts a new Python interpreter and runs bzr in there.
+        This should only be used for tests that have a justifiable need for
+        this isolation: e.g. they are testing startup time, or signal
+        handling, or early startup code, etc.  Subprocess code can't be
+        profiled or debugged so easily.
+
+        :param process_args: a list of arguments to pass to the bzr executable,
+            for example `['--version']`.
+        :param env_changes: A dictionary which lists changes to environment
+            variables. A value of None will unset the env variable.
+            The values must be strings. The change will only occur in the
+            child, so you don't need to fix the environment after running.
+        :param skip_if_plan_to_signal: raise TestSkipped when true and os.kill
+            is not available.
+        :param allow_plugins: If False (default) pass --no-plugins to bzr.
+
+        :returns: Popen object for the started process.
+        """
+        if skip_if_plan_to_signal:
+            if not getattr(os, 'kill', None):
+                raise TestSkipped("os.kill not available.")
+
+        if env_changes is None:
+            env_changes = {}
+        old_env = {}
+
         def cleanup_environment():
             for env_var, value in env_changes.iteritems():
-                if value is None:
-                    if env_var in os.environ:
-                        del os.environ[env_var]
-                else:
-                    os.environ[env_var] = value
-
+                old_env[env_var] = osutils.set_or_unset_env(env_var, value)
+
+        def restore_environment():
+            for env_var, value in old_env.iteritems():
+                osutils.set_or_unset_env(env_var, value)
+
+        bzr_path = self.get_bzr_path()
+
+        cwd = None
+        if working_dir is not None:
+            cwd = osutils.getcwd()
+            os.chdir(working_dir)
+
+        try:
+            # win32 subprocess doesn't support preexec_fn
+            # so we will avoid using it on all platforms, just to
+            # make sure the code path is used, and we don't break on win32
+            cleanup_environment()
+            command = [sys.executable, bzr_path]
+            if not allow_plugins:
+                command.append('--no-plugins')
+            command.extend(process_args)
+            process = self._popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        finally:
+            restore_environment()
+            if cwd is not None:
+                os.chdir(cwd)
+
+        return process
+
+    def _popen(self, *args, **kwargs):
+        """Place a call to Popen.
+
+        Allows tests to override this method to intercept the calls made to
+        Popen for introspection.
+        """
+        return Popen(*args, **kwargs)
+
+    def get_bzr_path(self):
+        """Return the path of the 'bzr' executable for this test suite."""
         bzr_path = os.path.dirname(os.path.dirname(bzrlib.__file__))+'/bzr'
-        args = list(args)
-        process = Popen([sys.executable, bzr_path]+args,
-                         stdout=PIPE, stderr=PIPE,
-                         preexec_fn=cleanup_environment)
-        out = process.stdout.read()
-        err = process.stderr.read()
-        retcode = process.wait()
-        supplied_retcode = kwargs.get('retcode', 0)
-        if supplied_retcode is not None:
-            assert supplied_retcode == retcode
+        if not os.path.isfile(bzr_path):
+            # We are probably installed. Assume sys.argv is the right file
+            bzr_path = sys.argv[0]
+        return bzr_path
+
+    def finish_bzr_subprocess(self, process, retcode=0, send_signal=None,
+                              universal_newlines=False, process_args=None):
+        """Finish the execution of process.
+
+        :param process: the Popen object returned from start_bzr_subprocess.
+        :param retcode: The status code that is expected.  Defaults to 0.  If
+            None is supplied, the status code is not checked.
+        :param send_signal: an optional signal to send to the process.
+        :param universal_newlines: Convert CRLF => LF
+        :returns: (stdout, stderr)
+        """
+        if send_signal is not None:
+            os.kill(process.pid, send_signal)
+        out, err = process.communicate()
+
+        if universal_newlines:
+            out = out.replace('\r\n', '\n')
+            err = err.replace('\r\n', '\n')
+
+        if retcode is not None and retcode != process.returncode:
+            if process_args is None:
+                process_args = "(unknown args)"
+            mutter('Output of bzr %s:\n%s', process_args, out)
+            mutter('Error for bzr %s:\n%s', process_args, err)
+            self.fail('Command bzr %s failed with retcode %s != %s'
+                      % (process_args, retcode, process.returncode))
         return [out, err]
 
     def check_inventory_shape(self, inv, shape):
@@ -986,8 +1124,213 @@
 
 BzrTestBase = TestCase
 
+
+class TestCaseWithMemoryTransport(TestCase):
+    """Common test class for tests that do not need disk resources.
+
+    Tests that need disk resources should derive from TestCaseWithTransport.
+
+    TestCaseWithMemoryTransport sets the TEST_ROOT variable for all bzr tests.
+
+    For TestCaseWithMemoryTransport the test_home_dir is set to the name of
+    a directory which does not exist. This serves to help ensure test isolation
+    is preserved. test_dir is set to the TEST_ROOT, as is cwd, because they
+    must exist. However, TestCaseWithMemoryTransport does not offer local
+    file defaults for the transport in tests, nor does it obey the command line
+    override, so tests that accidentally write to the common directory should
+    be rare.
+    """
+
+    TEST_ROOT = None
+    _TEST_NAME = 'test'
+
+
+    def __init__(self, methodName='runTest'):
+        # allow test parameterisation after test construction and before test
+        # execution. Variables that the parameteriser sets need to be 
+        # ones that are not set by setUp, or setUp will trash them.
+        super(TestCaseWithMemoryTransport, self).__init__(methodName)
+        self.transport_server = default_transport
+        self.transport_readonly_server = None
+
+    def failUnlessExists(self, path):
+        """Fail unless path, which may be abs or relative, exists."""
+        self.failUnless(osutils.lexists(path))
+
+    def failIfExists(self, path):
+        """Fail if path, which may be abs or relative, exists."""
+        self.failIf(osutils.lexists(path))
+        
+    def get_transport(self):
+        """Return a writeable transport for the test scratch space"""
+        t = get_transport(self.get_url())
+        self.assertFalse(t.is_readonly())
+        return t
+
+    def get_readonly_transport(self):
+        """Return a readonly transport for the test scratch space
+        
+        This can be used to test that operations which should only need
+        readonly access in fact do not try to write.
+        """
+        t = get_transport(self.get_readonly_url())
+        self.assertTrue(t.is_readonly())
+        return t
+
+    def get_readonly_server(self):
+        """Get the server instance for the readonly transport
+
+        This is useful for some tests with specific servers to do diagnostics.
+        """
+        if self.__readonly_server is None:
+            if self.transport_readonly_server is None:
+                # readonly decorator requested
+                # bring up the server
+                self.get_url()
+                self.__readonly_server = ReadonlyServer()
+                self.__readonly_server.setUp(self.__server)
+            else:
+                self.__readonly_server = self.transport_readonly_server()
+                self.__readonly_server.setUp()
+            self.addCleanup(self.__readonly_server.tearDown)
+        return self.__readonly_server
+
+    def get_readonly_url(self, relpath=None):
+        """Get a URL for the readonly transport.
+
+        This will either be backed by '.' or a decorator to the transport 
+        used by self.get_url()
+        relpath provides for clients to get a path relative to the base url.
+        These should only be downwards relative, not upwards.
+        """
+        base = self.get_readonly_server().get_url()
+        if relpath is not None:
+            if not base.endswith('/'):
+                base = base + '/'
+            base = base + relpath
+        return base
+
+    def get_server(self):
+        """Get the read/write server instance.
+
+        This is useful for some tests with specific servers that need
+        diagnostics.
+
+        For TestCaseWithMemoryTransport this is always a MemoryServer, and there
+        is no means to override it.
+        """
+        if self.__server is None:
+            self.__server = MemoryServer()
+            self.__server.setUp()
+            self.addCleanup(self.__server.tearDown)
+        return self.__server
+
+    def get_url(self, relpath=None):
+        """Get a URL (or maybe a path) for the readwrite transport.
+
+        This will either be backed by '.' or to an equivalent non-file based
+        facility.
+        relpath provides for clients to get a path relative to the base url.
+        These should only be downwards relative, not upwards.
+        """
+        base = self.get_server().get_url()
+        if relpath is not None and relpath != '.':
+            if not base.endswith('/'):
+                base = base + '/'
+            # XXX: Really base should be a url; we did after all call
+            # get_url()!  But sometimes it's just a path (from
+            # LocalAbspathServer), and it'd be wrong to append urlescaped data
+            # to a non-escaped local path.
+            if base.startswith('./') or base.startswith('/'):
+                base += relpath
+            else:
+                base += urlutils.escape(relpath)
+        return base
+
+    def _make_test_root(self):
+        if TestCaseWithMemoryTransport.TEST_ROOT is not None:
+            return
+        i = 0
+        while True:
+            root = u'test%04d.tmp' % i
+            try:
+                os.mkdir(root)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    i += 1
+                    continue
+                else:
+                    raise
+            # successfully created
+            TestCaseWithMemoryTransport.TEST_ROOT = osutils.abspath(root)
+            break
+        # make a fake bzr directory there to prevent any tests propagating
+        # up onto the source directory's real branch
+        bzrdir.BzrDir.create_standalone_workingtree(
+            TestCaseWithMemoryTransport.TEST_ROOT)
+
+    def makeAndChdirToTestDir(self):
+        """Create a temporary directories for this one test.
+        
+        This must set self.test_home_dir and self.test_dir and chdir to
+        self.test_dir.
+        
+        For TestCaseWithMemoryTransport we chdir to the TEST_ROOT for this test.
+        """
+        os.chdir(TestCaseWithMemoryTransport.TEST_ROOT)
+        self.test_dir = TestCaseWithMemoryTransport.TEST_ROOT
+        self.test_home_dir = self.test_dir + "/MemoryTransportMissingHomeDir"
+        
+    def make_branch(self, relpath, format=None):
+        """Create a branch on the transport at relpath."""
+        repo = self.make_repository(relpath, format=format)
+        return repo.bzrdir.create_branch()
+
+    def make_bzrdir(self, relpath, format=None):
+        try:
+            # might be a relative or absolute path
+            maybe_a_url = self.get_url(relpath)
+            segments = maybe_a_url.rsplit('/', 1)
+            t = get_transport(maybe_a_url)
+            if len(segments) > 1 and segments[-1] not in ('', '.'):
+                try:
+                    t.mkdir('.')
+                except errors.FileExists:
+                    pass
+            if format is None:
+                format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+            return format.initialize_on_transport(t)
+        except errors.UninitializableFormat:
+            raise TestSkipped("Format %s is not initializable." % format)
+
+    def make_repository(self, relpath, shared=False, format=None):
+        """Create a repository on our default transport at relpath."""
+        made_control = self.make_bzrdir(relpath, format=format)
+        return made_control.create_repository(shared=shared)
+
+    def make_branch_and_memory_tree(self, relpath, format=None):
+        """Create a branch on the default transport and a MemoryTree for it."""
+        b = self.make_branch(relpath, format=format)
+        return memorytree.MemoryTree.create_on_branch(b)
+
+    def overrideEnvironmentForTesting(self):
+        os.environ['HOME'] = self.test_home_dir
+        os.environ['APPDATA'] = self.test_home_dir
+        
+    def setUp(self):
+        super(TestCaseWithMemoryTransport, self).setUp()
+        self._make_test_root()
+        _currentdir = os.getcwdu()
+        def _leaveDirectory():
+            os.chdir(_currentdir)
+        self.addCleanup(_leaveDirectory)
+        self.makeAndChdirToTestDir()
+        self.overrideEnvironmentForTesting()
+        self.__readonly_server = None
+        self.__server = None
+
      
-class TestCaseInTempDir(TestCase):
+class TestCaseInTempDir(TestCaseWithMemoryTransport):
     """Derived class that runs a test within a temporary directory.
 
     This is useful for tests that need to create a branch, etc.
@@ -1000,8 +1343,6 @@
     InTempDir is an old alias for FunctionalTestCase.
     """
 
-    TEST_ROOT = None
-    _TEST_NAME = 'test'
     OVERRIDE_PYTHON = 'python'
 
     def check_file_contents(self, filename, expect):
@@ -1012,31 +1353,12 @@
             self.log("actually: %r" % contents)
             self.fail("contents of %s not as expected" % filename)
 
-    def _make_test_root(self):
-        if TestCaseInTempDir.TEST_ROOT is not None:
-            return
-        i = 0
-        while True:
-            root = u'test%04d.tmp' % i
-            try:
-                os.mkdir(root)
-            except OSError, e:
-                if e.errno == errno.EEXIST:
-                    i += 1
-                    continue
-                else:
-                    raise
-            # successfully created
-            TestCaseInTempDir.TEST_ROOT = osutils.abspath(root)
-            break
-        # make a fake bzr directory there to prevent any tests propagating
-        # up onto the source directory's real branch
-        bzrdir.BzrDir.create_standalone_workingtree(TestCaseInTempDir.TEST_ROOT)
-
-    def setUp(self):
-        super(TestCaseInTempDir, self).setUp()
-        self._make_test_root()
-        _currentdir = os.getcwdu()
+    def makeAndChdirToTestDir(self):
+        """See TestCaseWithMemoryTransport.makeAndChdirToTestDir().
+        
+        For TestCaseInTempDir we create a temporary directory based on the test
+        name and then create two subdirs - test and home under it.
+        """
         # shorten the name, to avoid test failures due to path length
         short_id = self.id().replace('bzrlib.tests.', '') \
                    .replace('__main__.', '')[-100:]
@@ -1052,16 +1374,14 @@
                 i = i + 1
                 continue
             else:
-                self.test_dir = candidate_dir
+                os.mkdir(candidate_dir)
+                self.test_home_dir = candidate_dir + '/home'
+                os.mkdir(self.test_home_dir)
+                self.test_dir = candidate_dir + '/work'
                 os.mkdir(self.test_dir)
                 os.chdir(self.test_dir)
                 break
-        os.environ['HOME'] = self.test_dir
-        os.environ['APPDATA'] = self.test_dir
-        def _leaveDirectory():
-            os.chdir(_currentdir)
-        self.addCleanup(_leaveDirectory)
-        
+
     def build_tree(self, shape, line_endings='native', transport=None):
         """Build a test tree according to a pattern.
 
@@ -1108,14 +1428,6 @@
     def build_tree_contents(self, shape):
         build_tree_contents(shape)
 
-    def failUnlessExists(self, path):
-        """Fail unless path, which may be abs or relative, exists."""
-        self.failUnless(osutils.lexists(path))
-
-    def failIfExists(self, path):
-        """Fail if path, which may be abs or relative, exists."""
-        self.failIf(osutils.lexists(path))
-        
     def assertFileEqual(self, content, path):
         """Fail if path does not contain 'content'."""
         self.failUnless(osutils.lexists(path))
@@ -1137,48 +1449,8 @@
     readwrite one must both define get_url() as resolving to os.getcwd().
     """
 
-    def __init__(self, methodName='testMethod'):
-        super(TestCaseWithTransport, self).__init__(methodName)
-        self.__readonly_server = None
-        self.__server = None
-        self.transport_server = default_transport
-        self.transport_readonly_server = None
-
-    def get_readonly_url(self, relpath=None):
-        """Get a URL for the readonly transport.
-
-        This will either be backed by '.' or a decorator to the transport 
-        used by self.get_url()
-        relpath provides for clients to get a path relative to the base url.
-        These should only be downwards relative, not upwards.
-        """
-        base = self.get_readonly_server().get_url()
-        if relpath is not None:
-            if not base.endswith('/'):
-                base = base + '/'
-            base = base + relpath
-        return base
-
-    def get_readonly_server(self):
-        """Get the server instance for the readonly transport
-
-        This is useful for some tests with specific servers to do diagnostics.
-        """
-        if self.__readonly_server is None:
-            if self.transport_readonly_server is None:
-                # readonly decorator requested
-                # bring up the server
-                self.get_url()
-                self.__readonly_server = ReadonlyServer()
-                self.__readonly_server.setUp(self.__server)
-            else:
-                self.__readonly_server = self.transport_readonly_server()
-                self.__readonly_server.setUp()
-            self.addCleanup(self.__readonly_server.tearDown)
-        return self.__readonly_server
-
     def get_server(self):
-        """Get the read/write server instance.
+        """See TestCaseWithMemoryTransport.
 
         This is useful for some tests with specific servers that need
         diagnostics.
@@ -1189,70 +1461,20 @@
             self.addCleanup(self.__server.tearDown)
         return self.__server
 
-    def get_url(self, relpath=None):
-        """Get a URL for the readwrite transport.
-
-        This will either be backed by '.' or to an equivalent non-file based
-        facility.
-        relpath provides for clients to get a path relative to the base url.
-        These should only be downwards relative, not upwards.
-        """
-        base = self.get_server().get_url()
-        if relpath is not None and relpath != '.':
-            if not base.endswith('/'):
-                base = base + '/'
-            base = base + urlutils.escape(relpath)
-        return base
-
-    def get_transport(self):
-        """Return a writeable transport for the test scratch space"""
-        t = get_transport(self.get_url())
-        self.assertFalse(t.is_readonly())
-        return t
-
-    def get_readonly_transport(self):
-        """Return a readonly transport for the test scratch space
-        
-        This can be used to test that operations which should only need
-        readonly access in fact do not try to write.
-        """
-        t = get_transport(self.get_readonly_url())
-        self.assertTrue(t.is_readonly())
-        return t
-
-    def make_branch(self, relpath, format=None):
-        """Create a branch on the transport at relpath."""
-        repo = self.make_repository(relpath, format=format)
-        return repo.bzrdir.create_branch()
-
-    def make_bzrdir(self, relpath, format=None):
-        try:
-            url = self.get_url(relpath)
-            mutter('relpath %r => url %r', relpath, url)
-            segments = url.split('/')
-            if segments and segments[-1] not in ('', '.'):
-                parent = '/'.join(segments[:-1])
-                t = get_transport(parent)
-                try:
-                    t.mkdir(segments[-1])
-                except errors.FileExists:
-                    pass
-            if format is None:
-                format=bzrlib.bzrdir.BzrDirFormat.get_default_format()
-            # FIXME: make this use a single transport someday. RBC 20060418
-            return format.initialize_on_transport(get_transport(relpath))
-        except errors.UninitializableFormat:
-            raise TestSkipped("Format %s is not initializable." % format)
-
-    def make_repository(self, relpath, shared=False, format=None):
-        """Create a repository on our default transport at relpath."""
-        made_control = self.make_bzrdir(relpath, format=format)
-        return made_control.create_repository(shared=shared)
-
     def make_branch_and_tree(self, relpath, format=None):
         """Create a branch on the transport and a tree locally.
 
-        Returns the tree.
+        If the transport is not a LocalTransport, the Tree can't be created on
+        the transport.  In that case the working tree is created in the local
+        directory, and the returned tree's branch and repository will also be
+        accessed locally.
+
+        This will fail if the original default transport for this test
+        case wasn't backed by the working directory, as the branch won't
+        be on disk for us to open it.  
+
+        :param format: The BzrDirFormat.
+        :returns: the WorkingTree.
         """
         # TODO: always use the local disk path for the working tree,
         # this obviously requires a format that supports branch references
@@ -1262,11 +1484,14 @@
         try:
             return b.bzrdir.create_workingtree()
         except errors.NotLocalUrl:
-            # new formats - catch No tree error and create
-            # a branch reference and a checkout.
-            # old formats at that point - raise TestSkipped.
-            # TODO: rbc 20060208
-            return WorkingTreeFormat2().initialize(bzrdir.BzrDir.open(relpath))
+            # We can only make working trees locally at the moment.  If the
+            # transport can't support them, then reopen the branch on a local
+            # transport, and create the working tree there.  
+            #
+            # Possibly we should instead keep
+            # the non-disk-backed branch and create a local checkout?
+            bd = bzrdir.BzrDir.open(relpath)
+            return bd.create_workingtree()
 
     def assertIsDirectory(self, relpath, transport):
         """Assert that relpath within transport is a directory.
@@ -1283,6 +1508,11 @@
             self.fail("path %s is not a directory; has mode %#o"
                       % (relpath, mode))
 
+    def setUp(self):
+        super(TestCaseWithTransport, self).setUp()
+        self.__server = None
+        self.transport_server = default_transport
+
 
 class ChrootedTestCase(TestCaseWithTransport):
     """A support class that provides readonly urls outside the local namespace.
@@ -1314,7 +1544,6 @@
 def run_suite(suite, name='test', verbose=False, pattern=".*",
               stop_on_failure=False, keep_output=False,
               transport=None, lsprof_timed=None, bench_history=None):
-    TestCaseInTempDir._TEST_NAME = name
     TestCase._gather_lsprof_in_benchmarks = lsprof_timed
     if verbose:
         verbosity = 2
@@ -1393,6 +1622,7 @@
                    'bzrlib.tests.test_errors',
                    'bzrlib.tests.test_escaped_store',
                    'bzrlib.tests.test_fetch',
+                   'bzrlib.tests.test_ftp_transport',
                    'bzrlib.tests.test_gpg',
                    'bzrlib.tests.test_graph',
                    'bzrlib.tests.test_hashcache',
@@ -1402,9 +1632,12 @@
                    'bzrlib.tests.test_ignores',
                    'bzrlib.tests.test_inv',
                    'bzrlib.tests.test_knit',
+                   'bzrlib.tests.test_lazy_import',
+                   'bzrlib.tests.test_lazy_regex',
                    'bzrlib.tests.test_lockdir',
                    'bzrlib.tests.test_lockable_files',
                    'bzrlib.tests.test_log',
+                   'bzrlib.tests.test_memorytree',
                    'bzrlib.tests.test_merge',
                    'bzrlib.tests.test_merge3',
                    'bzrlib.tests.test_merge_core',
@@ -1419,6 +1652,7 @@
                    'bzrlib.tests.test_plugins',
                    'bzrlib.tests.test_progress',
                    'bzrlib.tests.test_reconcile',
+                   'bzrlib.tests.test_registry',
                    'bzrlib.tests.test_repository',
                    'bzrlib.tests.test_revert',
                    'bzrlib.tests.test_revision',
@@ -1429,8 +1663,8 @@
                    'bzrlib.tests.test_selftest',
                    'bzrlib.tests.test_setup',
                    'bzrlib.tests.test_sftp_transport',
-                   'bzrlib.tests.test_ftp_transport',
                    'bzrlib.tests.test_smart_add',
+                   'bzrlib.tests.test_smart_transport',
                    'bzrlib.tests.test_source',
                    'bzrlib.tests.test_status',
                    'bzrlib.tests.test_store',
@@ -1443,6 +1677,7 @@
                    'bzrlib.tests.test_transform',
                    'bzrlib.tests.test_transport',
                    'bzrlib.tests.test_tree',
+                   'bzrlib.tests.test_treebuilder',
                    'bzrlib.tests.test_tsort',
                    'bzrlib.tests.test_tuned_gzip',
                    'bzrlib.tests.test_ui',
@@ -1450,6 +1685,7 @@
                    'bzrlib.tests.test_urlutils',
                    'bzrlib.tests.test_versionedfile',
                    'bzrlib.tests.test_version',
+                   'bzrlib.tests.test_version_info',
                    'bzrlib.tests.test_weave',
                    'bzrlib.tests.test_whitebox',
                    'bzrlib.tests.test_workingtree',
@@ -1470,7 +1706,11 @@
     for m in MODULES_TO_TEST:
         suite.addTest(loader.loadTestsFromModule(m))
     for m in MODULES_TO_DOCTEST:
-        suite.addTest(doctest.DocTestSuite(m))
+        try:
+            suite.addTest(doctest.DocTestSuite(m))
+        except ValueError, e:
+            print '**failed to get doctest for: %s\n%s' %(m,e)
+            raise
     for name, plugin in bzrlib.plugin.all_plugins().items():
         if getattr(plugin, 'test_suite', None) is not None:
             suite.addTest(plugin.test_suite())

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-08-25 15:01:52 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-09-23 06:39:46 +0000
@@ -60,6 +60,7 @@
                      'bzrlib.tests.blackbox.test_ignored',
                      'bzrlib.tests.blackbox.test_info',
                      'bzrlib.tests.blackbox.test_init',
+                     'bzrlib.tests.blackbox.test_inventory',
                      'bzrlib.tests.blackbox.test_locale',
                      'bzrlib.tests.blackbox.test_log',
                      'bzrlib.tests.blackbox.test_logformats',
@@ -79,6 +80,7 @@
                      'bzrlib.tests.blackbox.test_revision_history',
                      'bzrlib.tests.blackbox.test_revision_info',
                      'bzrlib.tests.blackbox.test_selftest',
+                     'bzrlib.tests.blackbox.test_serve',
                      'bzrlib.tests.blackbox.test_shared_repository',
                      'bzrlib.tests.blackbox.test_sign_my_commits',
                      'bzrlib.tests.blackbox.test_status',
@@ -87,6 +89,7 @@
                      'bzrlib.tests.blackbox.test_uncommit',
                      'bzrlib.tests.blackbox.test_update',
                      'bzrlib.tests.blackbox.test_upgrade',
+                     'bzrlib.tests.blackbox.test_version_info',
                      'bzrlib.tests.blackbox.test_versioning',
                      'bzrlib.tests.blackbox.test_whoami',
                      ]

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2006-08-16 08:03:47 +0000
+++ bzrlib/tests/blackbox/test_add.py	2006-09-07 21:31:58 +0000
@@ -19,8 +19,6 @@
 
 import os
 
-from bzrlib import ignores
-
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -28,8 +26,6 @@
         
     def test_add_reports(self):
         """add command prints the names of added files."""
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.runbzr('init')
         self.build_tree(['top.txt', 'dir/', 'dir/sub.txt', 'CVS'])
         self.build_tree_contents([('.bzrignore', 'CVS\n')])
@@ -42,13 +38,12 @@
                            'added dir',
                            'added dir/sub.txt',
                            'added top.txt',
-                           'ignored 2 file(s).'],
+                           'ignored 1 file(s).'],
                           results)
         out = self.run_bzr_captured(['add', '-v'], retcode=0)[0]
         results = sorted(out.rstrip('\n').split('\n'))
         self.assertEquals(['If you wish to add some of these files, please'\
                            ' add them by name.',
-                           'ignored .bazaar matching "./.bazaar"',
                            'ignored CVS matching "CVS"'],
                           results)
 
@@ -66,8 +61,6 @@
 
         "bzr add" should add the parent(s) as necessary.
         """
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.runbzr('init')
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
@@ -91,8 +84,6 @@
 
         "bzr add" should do this happily.
         """
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.runbzr('init')
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
@@ -105,7 +96,6 @@
         """Add in subdirectory should add only things from there down"""
         from bzrlib.workingtree import WorkingTree
 
-        ignores._set_user_ignores(['./.bazaar'])
         eq = self.assertEqual
         ass = self.assertTrue
         chdir = os.chdir

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2006-08-24 00:21:44 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2006-09-21 19:41:29 +0000
@@ -97,3 +97,13 @@
         self.assertEquals(out, '')
         self.assertEquals(err, 'bzr: ERROR: bzr annotate --revision takes'
                                ' exactly 1 argument\n')
+
+    def test_annotate_empty_file(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/empty', '')])
+        tree.add('empty')
+        tree.commit('add empty file')
+
+        os.chdir('tree')
+        out, err = self.run_bzr('annotate', 'empty')
+        self.assertEqual('', out)

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2006-08-25 05:03:17 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2006-09-11 01:03:59 +0000
@@ -15,8 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-"""Tests of bound branches (binding, unbinding, commit, etc) command.
-"""
+"""Tests of bound branches (binding, unbinding, commit, etc) command."""
 
 import os
 from cStringIO import StringIO
@@ -235,17 +234,22 @@
         bzr('commit', '-m', 'merged')
         self.check_revno(3)
 
+        # After binding, the revision history should be unaltered
+        base_branch = Branch.open('../base')
+        child_branch = Branch.open('.')
+        # take a copy before
+        base_history = base_branch.revision_history()
+        child_history = child_branch.revision_history()
+
         # After a merge, trying to bind again should succeed
-        # by pushing the new change to base
+        # keeping the new change as a local commit.
         bzr('bind', '../base')
         self.check_revno(3)
-        self.check_revno(3, '../base')
+        self.check_revno(2, '../base')
 
-        # After binding, the revision history should be identical
-        child_rh = bzr('revision-history')[0]
-        os.chdir('../base')
-        base_rh = bzr('revision-history')[0]
-        self.assertEquals(child_rh, base_rh)
+        # and compare the revision history now
+        self.assertEqual(base_history, base_branch.revision_history())
+        self.assertEqual(child_history, child_branch.revision_history())
 
     def test_bind_parent_ahead(self):
         bzr = self.run_bzr
@@ -261,7 +265,8 @@
         self.check_revno(1)
         bzr('bind', '../base')
 
-        self.check_revno(2)
+        # binding does not pull data:
+        self.check_revno(1)
         bzr('unbind')
 
         # Check and make sure it also works if parent is ahead multiple
@@ -272,16 +277,16 @@
         self.check_revno(5)
 
         os.chdir('../child')
-        self.check_revno(2)
+        self.check_revno(1)
         bzr('bind', '../base')
-        self.check_revno(5)
+        self.check_revno(1)
 
     def test_bind_child_ahead(self):
         # test binding when the master branches history is a prefix of the 
-        # childs
+        # childs - it should bind ok but the revision histories should not
+        # be altered
         bzr = self.run_bzr
         self.create_branches()
-        return
 
         os.chdir('child')
         bzr('unbind')
@@ -290,7 +295,7 @@
         self.check_revno(1, '../base')
 
         bzr('bind', '../base')
-        self.check_revno(2, '../base')
+        self.check_revno(1, '../base')
 
         # Check and make sure it also works if child is ahead multiple
         bzr('unbind')
@@ -299,9 +304,9 @@
         bzr('commit', '-m', 'child 5', '--unchanged')
         self.check_revno(5)
 
-        self.check_revno(2, '../base')
+        self.check_revno(1, '../base')
         bzr('bind', '../base')
-        self.check_revno(5, '../base')
+        self.check_revno(1, '../base')
 
     def test_commit_after_merge(self):
         bzr = self.run_bzr

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2006-09-05 08:14:23 +0000
@@ -67,7 +67,7 @@
         self.runbzr('branch source target --basis commit_tree')
         target = bzrdir.BzrDir.open('target')
         self.assertEqual('2', target.open_branch().last_revision())
-        self.assertEqual('2', target.open_workingtree().last_revision())
+        self.assertEqual(['2'], target.open_workingtree().get_parent_ids())
         self.assertTrue(target.open_branch().repository.has_revision('2'))
 
     def test_branch_only_copies_history(self):

=== modified file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2006-09-27 22:12:53 +0000
@@ -19,7 +19,10 @@
 import os
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    lockdir,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.tests.blackbox import ExternalBase
@@ -88,20 +91,34 @@
     def test_saying_no_leaves_it_locked(self):
         ### if 'no' is answered, objects should remain locked.
         self.wt.lock_write()
-        self.master_branch.lock_write()
-        # run the break-lock
-        # we need 5 yes's - wt, branch, repo, bound branch, bound repo.
-        self.run_bzr('break-lock', 'checkout', stdin="n\nn\nn\nn\nn\n")
-        # a new tree instance should not be lockable
-        wt = bzrlib.workingtree.WorkingTree.open('checkout')
-        self.assertRaises(errors.LockContention, wt.lock_write)
-        # and a new instance of the master branch 
-        mb = wt.branch.get_master_branch()
-        self.assertRaises(errors.LockContention, mb.lock_write)
-        # unlock our branches normally.
-        self.wt.unlock()
-        self.master_branch.unlock()
+        try:
+            self.master_branch.lock_write()
+            try:
+                # run the break-lock
+                # we need 5 yes's - wt, branch, repo, bound branch, bound repo.
+                self.run_bzr('break-lock', 'checkout', stdin="n\nn\nn\nn\nn\n")
 
+                # The default timeout to wait for LockContention is 5 minutes.
+                # we need to override this temporarily.
+                # TODO: jam 20060927 When we have per repository/branch/tree
+                #       timeouts set the value of the repository,
+                #       rather than setting the global default.
+                orig_default = lockdir._DEFAULT_TIMEOUT_SECONDS
+                try:
+                    lockdir._DEFAULT_TIMEOUT_SECONDS = 1
+                    # a new tree instance should not be lockable
+                    wt = bzrlib.workingtree.WorkingTree.open('checkout')
+                    self.assertRaises(errors.LockContention, wt.lock_write)
+                    # and a new instance of the master branch 
+                    mb = wt.branch.get_master_branch()
+                    self.assertRaises(errors.LockContention, mb.lock_write)
+                finally:
+                    lockdir._DEFAULT_TIMEOUT_SECONDS = orig_default
+                # unlock our branches normally.
+            finally:
+                self.master_branch.unlock()
+        finally:
+            self.wt.unlock()
 
 class TestBreakLockOldBranch(ExternalBase):
 

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2006-09-11 06:07:02 +0000
@@ -31,6 +31,8 @@
         def bzr(*args, **kwargs):
             return self.run_bzr(*args, **kwargs)[0]
 
+        os.mkdir('branch')
+        os.chdir('branch')
         bzr('init')
         open('a', 'wb').write('foo\n')
         bzr('add', 'a')
@@ -51,4 +53,11 @@
         rev_id = bzr('revision-history').strip().split('\n')[-1]
 
         self.assertEquals(bzr('cat', 'a', '-r', 'revid:%s' % rev_id), 'baz\n')
-
+        
+        os.chdir('..')
+        
+        self.assertEquals(bzr('cat', 'branch/a', '-r', 'revno:1:branch'),
+                          'foo\n')
+        bzr('cat', 'a', retcode=3)
+        bzr('cat', 'a', '-r', 'revno:1:branch-that-does-not-exist', retcode=3)
+        

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2006-09-05 08:14:23 +0000
@@ -58,7 +58,7 @@
         # the working tree should now be at revision '1' with the content
         # from 1.
         result = bzrdir.BzrDir.open('checkout')
-        self.assertEqual('1', result.open_workingtree().last_revision())
+        self.assertEqual(['1'], result.open_workingtree().get_parent_ids())
         self.failIfExists('checkout/added_in_2')
 
     def test_checkout_light_dash_r(self):
@@ -66,7 +66,7 @@
         # the working tree should now be at revision '1' with the content
         # from 1.
         result = bzrdir.BzrDir.open('checkout')
-        self.assertEqual('1', result.open_workingtree().last_revision())
+        self.assertEqual(['1'], result.open_workingtree().get_parent_ids())
         self.failIfExists('checkout/added_in_2')
 
     def test_checkout_reconstitutes_working_trees(self):

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2006-08-11 10:03:44 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2006-08-17 11:10:57 +0000
@@ -49,7 +49,8 @@
             os.mkdir('foo')
             bzrdir.BzrDirFormat5().initialize('foo')
             out, err = self.run_bzr("status", "foo")
-            self.assertContainsRe(self._get_log(), "bzr upgrade")
+            self.assertContainsRe(self._get_log(keep_log_file=True),
+                                  "bzr upgrade")
         finally:
             repository._deprecation_warning_done = True
 

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_export.py	2006-09-23 19:05:44 +0000
@@ -19,56 +19,225 @@
 """
 
 import os
+import stat
+import sys
+import tarfile
+import zipfile
 
-from bzrlib.branch import Branch
+from bzrlib.export import (
+    zip_exporter,
+    )
+from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
-import tarfile, zipfile
+
 
 class TestExport(ExternalBase):
 
     def test_tar_export(self):
+        tree = self.make_branch_and_tree('tar')
+        self.build_tree(['tar/a'])
+        tree.add('a')
 
-        os.mkdir('tar')
         os.chdir('tar')
+        self.run_bzr('ignore', 'something')
+        tree.commit('1')
 
-        self.runbzr('init')
-        open('a', 'wb').write('foo\n')
-        self.runbzr('add a')
-        self.runbzr('ignore something')
-        self.runbzr('commit -m 1')
-        self.runbzr('export test.tar.gz')
+        self.failUnless(tree.has_filename('.bzrignore'))
+        self.run_bzr('export', 'test.tar.gz')
         ball = tarfile.open('test.tar.gz')
-        for m in ball.getnames():
-            #print m
-            self.failIf(os.path.basename(m) == '.bzrignore', 'tar export contains .bzrignore')
+        # Make sure the tarball contains 'a', but does not contain
+        # '.bzrignore'.
+        self.assertEqual(['test/a'], sorted(ball.getnames()))
+
+    def test_tar_export_unicode(self):
+        tree = self.make_branch_and_tree('tar')
+        fname = u'\xe5.txt'
+        try:
+            self.build_tree(['tar/' + fname])
+        except UnicodeError:
+            raise TestSkipped('Unable to represent path %r' % (fname,))
+        tree.add([fname])
+        tree.commit('first')
+
+        os.chdir('tar')
+        self.run_bzr('export', 'test.tar')
+        ball = tarfile.open('test.tar')
+        # all paths are prefixed with the base name of the tarball
+        self.assertEqual(['test/' + fname.encode('utf8')],
+                         sorted(ball.getnames()))
 
     def test_zip_export(self):
-
-        os.mkdir('zip')
-        os.chdir('zip')
-
-        self.runbzr('init')
-        open('a', 'wb').write('foo\n')
-        self.runbzr('add a')
-        self.runbzr('ignore something')
-        self.runbzr('commit -m 1')
-        self.runbzr('export test.zip')
-        ball = zipfile.ZipFile('test.zip')
-        for m in ball.namelist():
-            #print m
-            self.failIf(os.path.basename(m) == '.bzrignore', 'zip export contains .bzrignore')
+        tree = self.make_branch_and_tree('zip')
+        self.build_tree(['zip/a'])
+        tree.add('a')
+
+        os.chdir('zip')
+        self.run_bzr('ignore', 'something')
+        tree.commit('1')
+
+        self.failUnless(tree.has_filename('.bzrignore'))
+        self.run_bzr('export', 'test.zip')
+
+        zfile = zipfile.ZipFile('test.zip')
+        # Make sure the zipfile contains 'a', but does not contain
+        # '.bzrignore'.
+        self.assertEqual(['test/a'], sorted(zfile.namelist()))
+
+    def test_zip_export_unicode(self):
+        tree = self.make_branch_and_tree('zip')
+        fname = u'\xe5.txt'
+        try:
+            self.build_tree(['zip/' + fname])
+        except UnicodeError:
+            raise TestSkipped('Unable to represent path %r' % (fname,))
+        tree.add([fname])
+        tree.commit('first')
+
+        os.chdir('zip')
+        self.run_bzr('export', 'test.zip')
+        zfile = zipfile.ZipFile('test.zip')
+        # all paths are prefixed with the base name of the zipfile
+        self.assertEqual(['test/' + fname.encode('utf8')],
+                         sorted(zfile.namelist()))
+
+    def test_zip_export_directories(self):
+        tree = self.make_branch_and_tree('zip')
+        self.build_tree(['zip/a', 'zip/b/', 'zip/b/c', 'zip/d/'])
+        tree.add(['a', 'b', 'b/c', 'd'])
+        tree.commit('init')
+
+        os.chdir('zip')
+        self.run_bzr('export', 'test.zip')
+        zfile = zipfile.ZipFile('test.zip')
+        names = sorted(zfile.namelist())
+
+        # even on win32, zipfile.ZipFile changes all names to use
+        # forward slashes
+        self.assertEqual(['test/a', 'test/b/', 'test/b/c', 'test/d/'], names)
+
+        file_attr = stat.S_IFREG
+        dir_attr = stat.S_IFDIR | zip_exporter.ZIP_DIRECTORY_BIT
+
+        a_info = zfile.getinfo(names[0])
+        self.assertEqual(file_attr, a_info.external_attr)
+
+        b_info = zfile.getinfo(names[1])
+        self.assertEqual(dir_attr, b_info.external_attr)
+
+        c_info = zfile.getinfo(names[2])
+        self.assertEqual(file_attr, c_info.external_attr)
+
+        d_info = zfile.getinfo(names[3])
+        self.assertEqual(dir_attr, d_info.external_attr)
 
     def test_dir_export(self):
+        tree = self.make_branch_and_tree('dir')
+        self.build_tree(['dir/a'])
+        tree.add('a')
 
-        os.mkdir('dir')
         os.chdir('dir')
+        self.run_bzr('ignore', 'something')
+        tree.commit('1')
 
-        self.runbzr('init')
-        open('a', 'wb').write('foo\n')
-        self.runbzr('add a')
-        self.runbzr('ignore something')
-        self.runbzr('commit -m 1')
-        self.runbzr('export direxport')
+        self.failUnless(tree.has_filename('.bzrignore'))
+        self.run_bzr('export', 'direxport')
 
         files = sorted(os.listdir('direxport'))
-        self.assertEqual(['a'], files, 'dir contains .bzrignore')
+        # Make sure the exported directory contains 'a', but does not contain
+        # '.bzrignore'.
+        self.assertEqual(['a'], files)
+
+    def example_branch(self):
+        tree = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/hello', 'foo')])
+        tree.add('hello')
+        tree.commit('setup')
+
+        self.build_tree_contents([('branch/goodbye', 'baz')])
+        tree.add('goodbye')
+        tree.commit('setup')
+        return tree
+        
+    def test_basic_directory_export(self):
+        self.example_branch()
+        os.chdir('branch')
+
+        # Directory exports
+        self.run_bzr('export', '../latest')
+        self.assertEqual(['goodbye', 'hello'], sorted(os.listdir('../latest')))
+        self.check_file_contents('../latest/goodbye', 'baz')
+        self.run_bzr('export', '../first', '-r', '1')
+        self.assertEqual(['hello'], sorted(os.listdir('../first')))
+        self.check_file_contents('../first/hello', 'foo')
+
+        # Even with .gz and .bz2 it is still a directory
+        self.run_bzr('export', '../first.gz', '-r', '1')
+        self.check_file_contents('../first.gz/hello', 'foo')
+        self.run_bzr('export', '../first.bz2', '-r', '1')
+        self.check_file_contents('../first.bz2/hello', 'foo')
+
+    def test_basic_tarfile_export(self):
+        self.example_branch()
+        os.chdir('branch')
+
+        self.run_bzr('export', '../first.tar', '-r', '1')
+        self.failUnless(os.path.isfile('../first.tar'))
+        tf = tarfile.open('../first.tar')
+        try:
+            self.assertEqual(['first/hello'], sorted(tf.getnames()))
+            self.assertEqual('foo', tf.extractfile('first/hello').read())
+        finally:
+            tf.close()
+
+        self.run_bzr('export', '../first.tar.gz', '-r', '1')
+        self.failUnless(os.path.isfile('../first.tar.gz'))
+        self.run_bzr('export', '../first.tbz2', '-r', '1')
+        self.failUnless(os.path.isfile('../first.tbz2'))
+        self.run_bzr('export', '../first.tar.bz2', '-r', '1')
+        self.failUnless(os.path.isfile('../first.tar.bz2'))
+        self.run_bzr('export', '../first.tar.tbz2', '-r', '1')
+        self.failUnless(os.path.isfile('../first.tar.tbz2'))
+
+        tf = tarfile.open('../first.tar.tbz2', 'r:bz2')
+        try:
+            self.assertEqual(['first.tar/hello'], sorted(tf.getnames()))
+            self.assertEqual('foo', tf.extractfile('first.tar/hello').read())
+        finally:
+            tf.close()
+        self.run_bzr('export', '../first2.tar', '-r', '1', '--root', 'pizza')
+        tf = tarfile.open('../first2.tar')
+        try:
+            self.assertEqual(['pizza/hello'], sorted(tf.getnames()))
+            self.assertEqual('foo', tf.extractfile('pizza/hello').read())
+        finally:
+            tf.close()
+
+    def test_basic_zipfile_export(self):
+        self.example_branch()
+        os.chdir('branch')
+
+        self.run_bzr('export', '../first.zip', '-r', '1')
+        self.failUnlessExists('../first.zip')
+        zf = zipfile.ZipFile('../first.zip')
+        try:
+            self.assertEqual(['first/hello'], sorted(zf.namelist()))
+            self.assertEqual('foo', zf.read('first/hello'))
+        finally:
+            zf.close()
+
+        self.run_bzr('export', '../first2.zip', '-r', '1', '--root', 'pizza')
+        zf = zipfile.ZipFile('../first2.zip')
+        try:
+            self.assertEqual(['pizza/hello'], sorted(zf.namelist()))
+            self.assertEqual('foo', zf.read('pizza/hello'))
+        finally:
+            zf.close()
+        
+        self.run_bzr('export', '../first-zip', '--format=zip', '-r', '1')
+        zf = zipfile.ZipFile('../first-zip')
+        try:
+            self.assertEqual(['first-zip/hello'], sorted(zf.namelist()))
+            self.assertEqual('foo', zf.read('first-zip/hello'))
+        finally:
+            zf.close()
+

=== modified file 'bzrlib/tests/blackbox/test_ignore.py'
--- bzrlib/tests/blackbox/test_ignore.py	2006-07-21 04:02:14 +0000
+++ bzrlib/tests/blackbox/test_ignore.py	2006-10-05 22:42:09 +0000
@@ -22,7 +22,10 @@
 import re
 import sys
 
-from bzrlib import ignores
+from bzrlib import (
+    ignores,
+    osutils,
+    )
 import bzrlib
 from bzrlib.branch import Branch
 import bzrlib.bzrdir as bzrdir
@@ -30,7 +33,6 @@
 from bzrlib.osutils import (
     has_symlinks,
     pathjoin,
-    rmtree,
     terminal_width,
     )
 from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
@@ -47,7 +49,7 @@
 
         # is_ignored() will now create the user global ignore file
         # if it doesn't exist, so make sure we ignore it in our tests
-        ignores._set_user_ignores(['*.tmp', './.bazaar'])
+        ignores._set_user_ignores(['*.tmp'])
 
         self.build_tree_contents(
             [('foo.tmp', '.tmp files are ignored by default'),

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2006-08-14 15:36:22 +0000
+++ bzrlib/tests/blackbox/test_info.py	2006-09-22 04:52:17 +0000
@@ -21,6 +21,7 @@
 import sys
 
 import bzrlib
+from bzrlib import repository
 from bzrlib.osutils import format_date
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
@@ -133,7 +134,9 @@
         # Branch and bind to standalone, needs upgrade to metadir
         # (creates backup as unknown)
         branch1.bzrdir.sprout('bound')
-        bzrlib.upgrade.upgrade('bound', bzrlib.bzrdir.BzrDirMetaFormat1())
+        knit1_format = bzrlib.bzrdir.BzrDirMetaFormat1()
+        knit1_format.repository_format = repository.RepositoryFormatKnit1()
+        bzrlib.upgrade.upgrade('bound', knit1_format)
         branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
         branch3.bind(branch1)
         bound_tree = branch3.bzrdir.open_workingtree()
@@ -186,7 +189,7 @@
 
         # Checkout standalone (same as above, but does not have parent set)
         old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(knit1_format)
         branch4 = bzrlib.bzrdir.BzrDir.create_branch_convenience('checkout')
         bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
         branch4.bind(branch1)
@@ -1200,7 +1203,7 @@
         transport.mkdir('tree')
         transport.mkdir('tree/checkout')
         co_branch = bzrlib.bzrdir.BzrDir.create_branch_convenience('tree/checkout',
-                                    format=bzrlib.bzrdir.BzrDirMetaFormat1())
+            format=bzrlib.bzrdir.BzrDirMetaFormat1())
         co_branch.bind(repo_branch)
         # Do a light checkout of the heavy one
         transport.mkdir('tree/lightcheckout')

=== added file 'bzrlib/tests/blackbox/test_inventory.py'
--- bzrlib/tests/blackbox/test_inventory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_inventory.py	2006-09-22 21:39:54 +0000
@@ -0,0 +1,116 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black-box tests for 'bzr inventory'."""
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestInventory(TestCaseWithTransport):
+
+    def setUp(self):
+        TestCaseWithTransport.setUp(self)
+
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        tree.commit('init', rev_id='one')
+        self.tree = tree
+
+    def assertInventoryEqual(self, expected, *args, **kwargs):
+        """Test that the output of 'bzr inventory' is as expected.
+
+        Any arguments supplied will be passed to run_bzr.
+        """
+        out, err = self.run_bzr('inventory', *args, **kwargs)
+        self.assertEqual(expected, out)
+        self.assertEqual('', err)
+        
+    def test_inventory(self):
+        self.assertInventoryEqual('a\nb\nb/c\n')
+
+    def test_inventory_kind(self):
+        self.assertInventoryEqual('a\nb/c\n', '--kind', 'file')
+        self.assertInventoryEqual('b\n', '--kind', 'directory')
+
+    def test_inventory_show_ids(self):
+        expected = ''.join(('%-50s %s\n' % (path, file_id))
+                                for path, file_id in
+                                [('a', 'a-id'),
+                                 ('b', 'b-id'),
+                                 ('b/c', 'c-id')
+                                ]
+                          )
+        self.assertInventoryEqual(expected, '--show-ids')
+
+    def test_inventory_specific_files(self):
+        self.assertInventoryEqual('a\n', 'a')
+        self.assertInventoryEqual('b\nb/c\n', 'b', 'b/c')
+        # 'bzr inventory' recurses into subdirectories
+        self.assertInventoryEqual('b\nb/c\n', 'b')
+
+    def test_inventory_mixed(self):
+        """Test that we get expected results when mixing parameters"""
+        a_line = '%-50s %s\n' % ('a', 'a-id')
+        b_line = '%-50s %s\n' % ('b', 'b-id')
+        c_line = '%-50s %s\n' % ('b/c', 'c-id')
+
+        self.assertInventoryEqual('', '--kind', 'directory', 'a')
+        self.assertInventoryEqual(a_line + c_line, '--kind', 'file',
+                                                   '--show-ids')
+        self.assertInventoryEqual(c_line, '--kind', 'file', '--show-ids',
+                                          'b', 'b/c')
+
+    def test_in_subdir(self):
+        os.chdir('b')
+        # TODO: jam 20060922 Maybe inventory should return the paths as
+        #       relative to '.', rather than relative to root
+
+        # a plain 'inventory' returns all files
+        self.assertInventoryEqual('a\nb\nb/c\n')
+        # But passing '.' will only return paths underneath here
+        self.assertInventoryEqual('b\nb/c\n', '.')
+
+
+    def test_inventory_revision(self):
+        self.build_tree(['b/d', 'e'])
+        self.tree.add(['b/d', 'e'], ['d-id', 'e-id'])
+        self.tree.commit('add files')
+
+        self.tree.rename_one('b/d', 'd')
+        self.tree.commit('rename b/d => d')
+
+        # Passing just -r returns the inventory of that revision
+        self.assertInventoryEqual('a\nb\nb/c\n', '-r', '1')
+        self.assertInventoryEqual('a\nb\nb/c\nb/d\ne\n', '-r', '2')
+
+        # Passing a path will lookup the path in the old and current locations
+        self.assertInventoryEqual('b/d\n', '-r', '2', 'b/d')
+        self.assertInventoryEqual('b/d\n', '-r', '2', 'd')
+
+        self.tree.rename_one('e', 'b/e')
+        self.tree.commit('rename e => b/e')
+
+        # When supplying just a directory paths that are now,
+        # or used to be, in that directory are shown
+        self.assertInventoryEqual('b\nb/c\nb/d\ne\n', '-r', '2', 'b')
+
+    def test_missing_file(self):
+        self.run_bzr_error([r'Path\(s\) are not versioned: no-such-file'],
+                           'inventory', 'no-such-file')

=== modified file 'bzrlib/tests/blackbox/test_locale.py'
--- bzrlib/tests/blackbox/test_locale.py	2006-09-05 11:24:56 +0000
+++ bzrlib/tests/blackbox/test_locale.py	2006-09-12 21:25:01 +0000
@@ -43,7 +43,8 @@
     def test_log_C(self):
         out, err = self.run_bzr_subprocess('--no-aliases', '--no-plugins',
                '-q', 'log', '--log-format=long', 'tree',
-               env_changes={'LANG':'C', 'BZR_PROGRESS_BAR':'none'})
+               env_changes={'LANG':'C', 'BZR_PROGRESS_BAR':'none',
+                            'LC_ALL':None, 'LC_CTYPE':None, 'LANGUAGE':None})
         self.assertEqual('', err)
         self.assertEqualDiff("""\
 ------------------------------------------------------------
@@ -58,12 +59,13 @@
     def test_log_BOGUS(self):
         out, err = self.run_bzr_subprocess('--no-aliases', '--no-plugins',
                '-q', 'log', '--log-format=long', 'tree',
-               env_changes={'LANG':'BOGUS', 'BZR_PROGRESS_BAR':'none'})
+               env_changes={'LANG':'BOGUS', 'BZR_PROGRESS_BAR':'none',
+                            'LC_ALL':None, 'LC_CTYPE':None, 'LANGUAGE':None})
         # XXX: This depends on the exact formatting of a locale.Error
         # as the first part of the string. It may be a little tempermental
         self.assertEqualDiff("""\
 bzr: warning: unsupported locale setting
-  Could not what text encoding to use.
+  Could not determine what text encoding to use.
   This error usually means your Python interpreter
   doesn't support the locale set by $LANG (BOGUS)
   Continuing with ascii encoding.

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2006-09-07 04:28:05 +0000
+++ bzrlib/tests/blackbox/test_log.py	2006-10-16 09:14:47 +0000
@@ -89,6 +89,25 @@
         log = self.runbzr("log -r 1..3")[0]
         self.assertEquals(self.full_log, log)
 
+    def test_log_revno_n_path(self):
+        os.mkdir('branch1')
+        os.chdir('branch1')
+        self._prepare()
+        os.chdir('..')
+        os.mkdir('branch2')
+        os.chdir('branch2')
+        self._prepare()
+        os.chdir('..')
+        log = self.runbzr("log -r revno:2:branch1..revno:3:branch2",
+                          retcode=3)[0]
+        log = self.runbzr("log -r revno:1:branch2..revno:3:branch2")[0]
+        self.assertEquals(self.full_log, log)
+        log = self.runbzr("log -r revno:1:branch2")[0]
+        self.assertTrue('revno: 1\n' in log)
+        self.assertTrue('revno: 2\n' not in log)
+        self.assertTrue('branch nick: branch2\n' in log)
+        self.assertTrue('branch nick: branch1\n' not in log)
+        
 
 class TestLogMerges(ExternalBase):
 

=== modified file 'bzrlib/tests/blackbox/test_ls.py'
--- bzrlib/tests/blackbox/test_ls.py	2006-07-21 04:02:14 +0000
+++ bzrlib/tests/blackbox/test_ls.py	2006-09-07 21:31:58 +0000
@@ -28,7 +28,7 @@
         super(TestLS, self).setUp()
         
         # Create a simple branch that can be used in testing
-        ignores._set_user_ignores(['./.bazaar', 'user-ignore'])
+        ignores._set_user_ignores(['user-ignore'])
 
         self.wt = self.make_branch_and_tree('.')
         self.build_tree_contents([
@@ -48,41 +48,36 @@
 
     def test_ls_basic(self):
         """Test the abilities of 'bzr ls'"""
-        self.ls_equals('.bazaar\n.bzrignore\na\n')
-        self.ls_equals('I        .bazaar/\n'
-                       '?        .bzrignore\n'
+        self.ls_equals('.bzrignore\na\n')
+        self.ls_equals('?        .bzrignore\n'
                        '?        a\n',
                        '--verbose')
         self.ls_equals('.bzrignore\n'
                        'a\n',
                        '--unknown')
-        self.ls_equals('.bazaar\n', '--ignored')
+        self.ls_equals('', '--ignored')
         self.ls_equals('', '--versioned')
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n',
                        '--unknown', '--ignored', '--versioned')
-        self.ls_equals('.bazaar\n', '--ignored', '--versioned')
-        self.ls_equals('.bazaar\0.bzrignore\0a\0', '--null')
+        self.ls_equals('', '--ignored', '--versioned')
+        self.ls_equals('.bzrignore\0a\0', '--null')
 
     def test_ls_added(self):
         self.wt.add(['a'])
-        self.ls_equals('I        .bazaar/\n'
-                       '?        .bzrignore\n'
+        self.ls_equals('?        .bzrignore\n'
                        'V        a\n',
                        '--verbose')
         self.wt.commit('add')
         
         self.build_tree(['subdir/'])
-        self.ls_equals('I        .bazaar/\n'
-                       '?        .bzrignore\n'
+        self.ls_equals('?        .bzrignore\n'
                        'V        a\n'
                        '?        subdir/\n'
                        , '--verbose')
         self.build_tree(['subdir/b'])
         self.wt.add(['subdir/', 'subdir/b', '.bzrignore'])
-        self.ls_equals('I        .bazaar/\n'
-                       'V        .bzrignore\n'
+        self.ls_equals('V        .bzrignore\n'
                        'V        a\n'
                        'V        subdir/\n'
                        'V        subdir/b\n'
@@ -92,14 +87,12 @@
         self.build_tree(['subdir/', 'subdir/b'])
         self.wt.add(['a', 'subdir/', 'subdir/b', '.bzrignore'])
 
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'subdir\n'
                        , '--non-recursive')
 
-        self.ls_equals('I        .bazaar/\n'
-                       'V        .bzrignore\n'
+        self.ls_equals('V        .bzrignore\n'
                        'V        a\n'
                        'V        subdir/\n'
                        , '--verbose', '--non-recursive')
@@ -109,20 +102,17 @@
         self.ls_equals('b\n')
         self.ls_equals('b\0'
                   , '--null')
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'subdir\n'
                        'subdir/b\n'
                        , '--from-root')
-        self.ls_equals('.bazaar\0'
-                       '.bzrignore\0'
+        self.ls_equals('.bzrignore\0'
                        'a\0'
                        'subdir\0'
                        'subdir/b\0'
                        , '--from-root', '--null')
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'subdir\n'
                        , '--from-root', '--non-recursive')
@@ -146,22 +136,19 @@
         self.wt.add(['a', '.bzrignore'])
 
         self.build_tree(['blah.py', 'blah.pyo', 'user-ignore'])
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'blah.py\n'
                        'blah.pyo\n'
                        'user-ignore\n'
                        )
-        self.ls_equals('I        .bazaar/\n'
-                       'V        .bzrignore\n'
+        self.ls_equals('V        .bzrignore\n'
                        'V        a\n'
                        '?        blah.py\n'
                        'I        blah.pyo\n'
                        'I        user-ignore\n'
                        , '--verbose')
-        self.ls_equals('.bazaar\n'
-                       'blah.pyo\n'
+        self.ls_equals('blah.pyo\n'
                        'user-ignore\n'
                        , '--ignored')
         self.ls_equals('blah.py\n'

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2006-08-25 05:03:17 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2006-09-19 19:30:15 +0000
@@ -52,6 +52,7 @@
         os.mkdir('a')
         os.chdir('a')
         self.example_branch()
+        ancestor = Branch.open('.').revno()
         os.chdir('..')
         self.runbzr('branch a b')
         os.chdir('b')
@@ -79,6 +80,18 @@
         b_tip = b.last_revision()
         self.failUnless(a.branch.repository.has_revision(b_tip))
         self.assertEqual([a_tip, b_tip], a.get_parent_ids())
+        self.runbzr('revert --no-backup')
+        out, err = self.runbzr('merge -r revno:1:./hello', retcode=3)
+        self.assertTrue("Not a branch" in err)
+        self.runbzr('merge -r revno:%d:./..revno:%d:../b'
+                    %(ancestor,b.revno()))
+        self.assertEquals(a.get_parent_ids(), 
+                          [a.branch.last_revision(), b.last_revision()])
+        self.check_file_contents('goodbye', 'quux')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge -r revno:%d:../b'%b.revno())
+        self.assertEquals(a.get_parent_ids(),
+                          [a.branch.last_revision(), b.last_revision()])
         a_tip = a.commit('merged')
         self.runbzr('merge ../b -r last:1')
         self.assertEqual([a_tip], a.get_parent_ids())
@@ -194,10 +207,10 @@
         file('../bundle', 'wb').write(self.runbzr('bundle ../branch_a')[0])
         os.chdir('../branch_a')
         self.runbzr('merge ../bundle', retcode=1)
-        testament_a = Testament.from_revision(tree_a.branch.repository, 
-                                              tree_b.last_revision())
+        testament_a = Testament.from_revision(tree_a.branch.repository,
+                                              tree_b.get_parent_ids()[0])
         testament_b = Testament.from_revision(tree_b.branch.repository,
-                                              tree_b.last_revision())
+                                              tree_b.get_parent_ids()[0])
         self.assertEqualDiff(testament_a.as_text(),
                          testament_b.as_text())
         tree_a.set_conflicts(ConflictList())

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2006-08-06 23:35:09 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2006-09-07 21:37:43 +0000
@@ -20,7 +20,7 @@
 import sys
 import os
 
-from bzrlib import ignores, osutils, urlutils
+from bzrlib import osutils, urlutils
 import bzrlib
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.trace import mutter, note
@@ -471,7 +471,6 @@
     def test_ignore(self):
         bzr = self.run_bzr_decode
 
-        ignores._set_user_ignores(['./.bazaar'])
         fname2 = self.info['filename'] + '2.txt'
         open(fname2, 'wb').write('ignored\n')
 

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2006-09-05 08:14:23 +0000
@@ -292,10 +292,10 @@
         self.assertEqualDiff(tree_a.branch.revision_history(),
                              tree_b.branch.revision_history())
 
-        testament_a = Testament.from_revision(tree_a.branch.repository, 
-                                              tree_a.last_revision())
+        testament_a = Testament.from_revision(tree_a.branch.repository,
+                                              tree_a.get_parent_ids()[0])
         testament_b = Testament.from_revision(tree_b.branch.repository,
-                                              tree_b.last_revision())
+                                              tree_b.get_parent_ids()[0])
         self.assertEqualDiff(testament_a.as_text(),
                              testament_b.as_text())
 

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_push.py	2006-10-10 00:15:25 +0000
@@ -71,7 +71,7 @@
         out = self.run_bzr('push', '../branch_b', retcode=3)
         self.assertEquals(out,
                 ('','bzr: ERROR: These branches have diverged.  '
-                    'Try a merge then push with overwrite.\n'))
+                    'Try using "merge" and then "push".\n'))
         self.assertEquals(abspath(branch_a.get_push_location()),
                           abspath(branch_b.bzrdir.root_transport.base))
 

=== modified file 'bzrlib/tests/blackbox/test_reconcile.py'
--- bzrlib/tests/blackbox/test_reconcile.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/blackbox/test_reconcile.py	2006-09-10 20:12:00 +0000
@@ -52,7 +52,8 @@
         t = bzrdir.BzrDir.create_standalone_workingtree('.')
         # an empty inventory with no revision will trigger reconciliation.
         repo = t.branch.repository
-        inv = Inventory()
+        inv = Inventory(revision_id='missing')
+        inv.root.revision='missing'
         repo.add_inventory('missing', inv, [])
         (out, err) = self.run_bzr_captured(['reconcile'])
         self.assertEqualDiff(out, "Reconciling repository %s\n"

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2006-09-02 15:07:48 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2006-10-13 07:41:06 +0000
@@ -16,6 +16,7 @@
 """UI tests for the test framework."""
 
 import os
+import signal
 import sys
 
 import bzrlib
@@ -25,7 +26,7 @@
 from bzrlib.errors import ParamikoNotPresent
 from bzrlib.tests import (
                           TestCase,
-                          TestCaseInTempDir,
+                          TestCaseWithMemoryTransport,
                           TestCaseWithTransport,
                           TestSkipped,
                           )
@@ -64,8 +65,8 @@
         except ParamikoNotPresent:
             raise TestSkipped("Paramiko not present")
         old_transport = bzrlib.tests.default_transport
-        old_root = TestCaseInTempDir.TEST_ROOT
-        TestCaseInTempDir.TEST_ROOT = None
+        old_root = TestCaseWithMemoryTransport.TEST_ROOT
+        TestCaseWithMemoryTransport.TEST_ROOT = None
         try:
             TestOptions.current_test = "test_transport_set_to_sftp"
             stdout = self.capture('selftest --transport=sftp test_transport_set_to_sftp')
@@ -80,13 +81,58 @@
         finally:
             bzrlib.tests.default_transport = old_transport
             TestOptions.current_test = None
-            TestCaseInTempDir.TEST_ROOT = old_root
+            TestCaseWithMemoryTransport.TEST_ROOT = old_root
 
 
 class TestRunBzr(ExternalBase):
 
-    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None):
+    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None,
+                         working_dir=None):
+        """Override run_bzr_captured to test how it is invoked by run_bzr.
+
+        We test how run_bzr_captured actually invokes bzr in another location.
+        Here we only need to test that it is run_bzr passes the right
+        parameters to run_bzr_captured.
+        """
+        self.argv = argv
+        self.retcode = retcode
+        self.encoding = encoding
         self.stdin = stdin
+        self.working_dir = working_dir
+
+    def test_args(self):
+        """Test that run_bzr passes args correctly to run_bzr_captured"""
+        self.run_bzr('arg1', 'arg2', 'arg3', retcode=1)
+        self.assertEqual(('arg1', 'arg2', 'arg3'), self.argv)
+
+    def test_encoding(self):
+        """Test that run_bzr passes encoding to run_bzr_captured"""
+        self.run_bzr('foo', 'bar')
+        self.assertEqual(None, self.encoding)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', encoding='baz')
+        self.assertEqual('baz', self.encoding)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+    def test_retcode(self):
+        """Test that run_bzr passes retcode to run_bzr_captured"""
+        # Default is retcode == 0
+        self.run_bzr('foo', 'bar')
+        self.assertEqual(0, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', retcode=1)
+        self.assertEqual(1, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', retcode=None)
+        self.assertEqual(None, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', retcode=3)
+        self.assertEqual(3, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
 
     def test_stdin(self):
         # test that the stdin keyword to run_bzr is passed through to
@@ -96,8 +142,21 @@
         # should invoke it.
         self.run_bzr('foo', 'bar', stdin='gam')
         self.assertEqual('gam', self.stdin)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
         self.run_bzr('foo', 'bar', stdin='zippy')
         self.assertEqual('zippy', self.stdin)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+    def test_working_dir(self):
+        """Test that run_bzr passes working_dir to run_bzr_captured"""
+        self.run_bzr('foo', 'bar')
+        self.assertEqual(None, self.working_dir)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', working_dir='baz')
+        self.assertEqual('baz', self.working_dir)
+        self.assertEqual(('foo', 'bar'), self.argv)
 
 
 class TestBenchmarkTests(TestCaseWithTransport):
@@ -106,12 +165,12 @@
         """bzr selftest --benchmark should not run the default test suite."""
         # We test this by passing a regression test name to --benchmark, which
         # should result in 0 rests run.
-        old_root = TestCaseInTempDir.TEST_ROOT
+        old_root = TestCaseWithMemoryTransport.TEST_ROOT
         try:
-            TestCaseInTempDir.TEST_ROOT = None
+            TestCaseWithMemoryTransport.TEST_ROOT = None
             out, err = self.run_bzr('selftest', '--benchmark', 'workingtree_implementations')
         finally:
-            TestCaseInTempDir.TEST_ROOT = old_root
+            TestCaseWithMemoryTransport.TEST_ROOT = old_root
         self.assertContainsRe(out, 'Ran 0 tests.*\n\nOK')
         self.assertEqual(
             'running tests...\ntests passed\n',
@@ -132,6 +191,7 @@
         self.stdin = stdin
         self.factory_stdin = getattr(bzrlib.ui.ui_factory, "stdin", None)
         self.factory = bzrlib.ui.ui_factory
+        self.working_dir = osutils.getcwd()
         stdout.write('foo\n')
         stderr.write('bar\n')
         return 0
@@ -162,6 +222,32 @@
         self.assertEqual('bar\n', self.factory.stderr.getvalue())
         self.assertIsInstance(self.factory, bzrlib.tests.blackbox.TestUIFactory)
 
+    def test_working_dir(self):
+        self.build_tree(['one/', 'two/'])
+        cwd = osutils.getcwd()
+
+        # Default is to work in the current directory
+        self.run_bzr_captured(['foo', 'bar'])
+        self.assertEqual(cwd, self.working_dir)
+
+        self.run_bzr_captured(['foo', 'bar'], working_dir=None)
+        self.assertEqual(cwd, self.working_dir)
+
+        # The function should be run in the alternative directory
+        # but afterwards the current working dir shouldn't be changed
+        self.run_bzr_captured(['foo', 'bar'], working_dir='one')
+        self.assertNotEqual(cwd, self.working_dir)
+        self.assertEndsWith(self.working_dir, 'one')
+        self.assertEqual(cwd, osutils.getcwd())
+
+        self.run_bzr_captured(['foo', 'bar'], working_dir='two')
+        self.assertNotEqual(cwd, self.working_dir)
+        self.assertEndsWith(self.working_dir, 'two')
+        self.assertEqual(cwd, osutils.getcwd())
+
+
+class TestRunBzrSubprocess(TestCaseWithTransport):
+
     def test_run_bzr_subprocess(self):
         """The run_bzr_helper_external comand behaves nicely."""
         result = self.run_bzr_subprocess('--version')
@@ -185,7 +271,7 @@
         self.assertEqual(None, os.environ.get('BZR_EMAIL'))
         out, err = self.run_bzr_subprocess('whoami', env_changes={
                                             'BZR_EMAIL':'Joe Foo <joe@foo.com>'
-                                          })
+                                          }, universal_newlines=True)
         self.assertEqual('', err)
         self.assertEqual('Joe Foo <joe@foo.com>\n', out)
         # And it should not be modified
@@ -195,7 +281,7 @@
         # it is actually changing
         out, err = self.run_bzr_subprocess('whoami', env_changes={
                                             'BZR_EMAIL':'Barry <bar@foo.com>'
-                                          })
+                                          }, universal_newlines=True)
         self.assertEqual('', err)
         self.assertEqual('Barry <bar@foo.com>\n', out)
         self.assertEqual(None, os.environ.get('BZR_EMAIL'))
@@ -209,20 +295,22 @@
         os.environ['EMAIL'] = rand_email
         try:
             # By default, the child will inherit the current env setting
-            out, err = self.run_bzr_subprocess('whoami')
+            out, err = self.run_bzr_subprocess('whoami', universal_newlines=True)
             self.assertEqual('', err)
             self.assertEqual(rand_bzr_email + '\n', out)
 
             # Now that BZR_EMAIL is not set, it should fall back to EMAIL
             out, err = self.run_bzr_subprocess('whoami',
-                                               env_changes={'BZR_EMAIL':None})
+                                               env_changes={'BZR_EMAIL':None},
+                                               universal_newlines=True)
             self.assertEqual('', err)
             self.assertEqual(rand_email + '\n', out)
 
             # This switches back to the default email guessing logic
             # Which shouldn't match either of the above addresses
             out, err = self.run_bzr_subprocess('whoami',
-                           env_changes={'BZR_EMAIL':None, 'EMAIL':None})
+                           env_changes={'BZR_EMAIL':None, 'EMAIL':None},
+                           universal_newlines=True)
 
             self.assertEqual('', err)
             self.assertNotEqual(rand_bzr_email + '\n', out)
@@ -236,9 +324,123 @@
         """run_bzr_subprocess won't fail if deleting a nonexistant env var"""
         self.failIf('NON_EXISTANT_ENV_VAR' in os.environ)
         out, err = self.run_bzr_subprocess('rocks',
-                        env_changes={'NON_EXISTANT_ENV_VAR':None})
-        self.assertEquals('it sure does!\n', out)
-
+                        env_changes={'NON_EXISTANT_ENV_VAR':None},
+                        universal_newlines=True)
+        self.assertEqual('it sure does!\n', out)
+        self.assertEqual('', err)
+
+    def test_run_bzr_subprocess_working_dir(self):
+        """Test that we can specify the working dir for the child"""
+        cwd = osutils.getcwd()
+
+        self.make_branch_and_tree('.')
+        self.make_branch_and_tree('one')
+        self.make_branch_and_tree('two')
+
+        def get_root(**kwargs):
+            """Spawn a process to get the 'root' of the tree.
+
+            You can pass in arbitrary new arguments. This just makes
+            sure that the returned path doesn't have trailing whitespace.
+            """
+            return self.run_bzr_subprocess('root', **kwargs)[0].rstrip()
+
+        self.assertEqual(cwd, get_root())
+        self.assertEqual(cwd, get_root(working_dir=None))
+        # Has our path changed?
+        self.assertEqual(cwd, osutils.getcwd())
+
+        dir1 = get_root(working_dir='one')
+        self.assertEndsWith(dir1, 'one')
+        self.assertEqual(cwd, osutils.getcwd())
+
+        dir2 = get_root(working_dir='two')
+        self.assertEndsWith(dir2, 'two')
+        self.assertEqual(cwd, osutils.getcwd())
+
+
+class _DontSpawnProcess(Exception):
+    """A simple exception which just allows us to skip unnecessary steps"""
+
+
+class TestRunBzrSubprocessCommands(TestCaseWithTransport):
+
+    def _popen(self, *args, **kwargs):
+        """Record the command that is run, so that we can ensure it is correct"""
+        self._popen_args = args
+        self._popen_kwargs = kwargs
+        raise _DontSpawnProcess()
+
+    def test_run_bzr_subprocess_no_plugins(self):
+        self.assertRaises(_DontSpawnProcess, self.run_bzr_subprocess)
+        command = self._popen_args[0]
+        self.assertEqual(sys.executable, command[0])
+        self.assertEqual(self.get_bzr_path(), command[1])
+        self.assertEqual(['--no-plugins'], command[2:])
+
+    def test_allow_plugins(self):
+        self.assertRaises(_DontSpawnProcess,
+                          self.run_bzr_subprocess, allow_plugins=True)
+        command = self._popen_args[0]
+        self.assertEqual([], command[2:])
+
+
+class TestBzrSubprocess(TestCaseWithTransport):
+
+    def test_start_and_stop_bzr_subprocess(self):
+        """We can start and perform other test actions while that process is
+        still alive.
+        """
+        process = self.start_bzr_subprocess(['--version'])
+        result = self.finish_bzr_subprocess(process)
+        self.assertContainsRe(result[0], 'is free software')
+        self.assertEqual('', result[1])
+
+    def test_start_and_stop_bzr_subprocess_with_error(self):
+        """finish_bzr_subprocess allows specification of the desired exit code.
+        """
+        process = self.start_bzr_subprocess(['--versionn'])
+        result = self.finish_bzr_subprocess(process, retcode=3)
+        self.assertEqual('', result[0])
+        self.assertContainsRe(result[1], 'unknown command')
+
+    def test_start_and_stop_bzr_subprocess_ignoring_retcode(self):
+        """finish_bzr_subprocess allows the exit code to be ignored."""
+        process = self.start_bzr_subprocess(['--versionn'])
+        result = self.finish_bzr_subprocess(process, retcode=None)
+        self.assertEqual('', result[0])
+        self.assertContainsRe(result[1], 'unknown command')
+
+    def test_start_and_stop_bzr_subprocess_with_unexpected_retcode(self):
+        """finish_bzr_subprocess raises self.failureException if the retcode is
+        not the expected one.
+        """
+        process = self.start_bzr_subprocess(['--versionn'])
+        self.assertRaises(self.failureException, self.finish_bzr_subprocess,
+                          process, retcode=0)
+        
+    def test_start_and_stop_bzr_subprocess_send_signal(self):
+        """finish_bzr_subprocess raises self.failureException if the retcode is
+        not the expected one.
+        """
+        process = self.start_bzr_subprocess(['wait-until-signalled'],
+                                            skip_if_plan_to_signal=True)
+        self.assertEqual('running\n', process.stdout.readline())
+        result = self.finish_bzr_subprocess(process, send_signal=signal.SIGINT,
+                                            retcode=3)
+        self.assertEqual('', result[0])
+        self.assertEqual('bzr: interrupted\n', result[1])
+
+    def test_start_and_stop_working_dir(self):
+        cwd = osutils.getcwd()
+
+        self.make_branch_and_tree('one')
+
+        process = self.start_bzr_subprocess(['root'], working_dir='one')
+        result = self.finish_bzr_subprocess(process)
+        self.assertEndsWith(result[0], 'one\n')
+        self.assertEqual('', result[1])
+        
 
 class TestRunBzrError(ExternalBase):
 

=== added file 'bzrlib/tests/blackbox/test_serve.py'
--- bzrlib/tests/blackbox/test_serve.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_serve.py	2006-09-21 09:15:20 +0000
@@ -0,0 +1,219 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests of the bzr serve command."""
+
+import os
+import signal
+import subprocess
+import threading
+
+from bzrlib import errors
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.errors import ParamikoNotPresent
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.transport import get_transport, smart
+
+
+class TestBzrServe(TestCaseWithTransport):
+
+    def assertInetServerShutsdownCleanly(self, process):
+        """Shutdown the server process looking for errors."""
+        # Shutdown the server: the server should shut down when it cannot read
+        # from stdin anymore.
+        process.stdin.close()
+        # Hide stdin from the subprocess module, so it won't fail to close it.
+        process.stdin = None
+        result = self.finish_bzr_subprocess(process, retcode=0)
+        self.assertEqual('', result[0])
+        self.assertEqual('', result[1])
+    
+    def assertServerFinishesCleanly(self, process):
+        """Shutdown the bzr serve instance process looking for errors."""
+        # Shutdown the server
+        result = self.finish_bzr_subprocess(process, retcode=3,
+                                            send_signal=signal.SIGINT)
+        self.assertEqual('', result[0])
+        self.assertEqual('bzr: interrupted\n', result[1])
+
+    def start_server_inet(self, extra_options=()):
+        """Start a bzr server subprocess using the --inet option.
+
+        :param extra_options: extra options to give the server.
+        :return: a tuple with the bzr process handle for passing to
+            finish_bzr_subprocess, a client for the server, and a transport.
+        """
+        # Serve from the current directory
+        process = self.start_bzr_subprocess(['serve', '--inet'])
+
+        # Connect to the server
+        # We use this url because while this is no valid URL to connect to this
+        # server instance, the transport needs a URL.
+        medium = smart.SmartSimplePipesClientMedium(
+            process.stdout, process.stdin)
+        transport = smart.SmartTransport('bzr://localhost/', medium=medium)
+        return process, transport
+
+    def start_server_port(self, extra_options=()):
+        """Start a bzr server subprocess.
+
+        :param extra_options: extra options to give the server.
+        :return: a tuple with the bzr process handle for passing to
+            finish_bzr_subprocess, and the base url for the server.
+        """
+        # Serve from the current directory
+        args = ['serve', '--port', 'localhost:0']
+        args.extend(extra_options)
+        process = self.start_bzr_subprocess(args, skip_if_plan_to_signal=True)
+        port_line = process.stdout.readline()
+        prefix = 'listening on port: '
+        self.assertStartsWith(port_line, prefix)
+        port = int(port_line[len(prefix):])
+        return process,'bzr://localhost:%d/' % port
+
+    def test_bzr_serve_inet_readonly(self):
+        """bzr server should provide a read only filesystem by default."""
+        process, transport = self.start_server_inet()
+        self.assertRaises(errors.TransportNotPossible, transport.mkdir, 'adir')
+        self.assertInetServerShutsdownCleanly(process)
+
+    def test_bzr_serve_inet_readwrite(self):
+        # Make a branch
+        self.make_branch('.')
+
+        process, transport = self.start_server_inet(['--allow-writes'])
+
+        # We get a working branch
+        branch = BzrDir.open_from_transport(transport).open_branch()
+        branch.repository.get_revision_graph()
+        self.assertEqual(None, branch.last_revision())
+        self.assertInetServerShutsdownCleanly(process)
+
+    def test_bzr_serve_port_readonly(self):
+        """bzr server should provide a read only filesystem by default."""
+        process, url = self.start_server_port()
+        transport = get_transport(url)
+        self.assertRaises(errors.TransportNotPossible, transport.mkdir, 'adir')
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_port_readwrite(self):
+        # Make a branch
+        self.make_branch('.')
+
+        process, url = self.start_server_port(['--allow-writes'])
+
+        # Connect to the server
+        branch = Branch.open(url)
+
+        # We get a working branch
+        branch.repository.get_revision_graph()
+        self.assertEqual(None, branch.last_revision())
+
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_no_args(self):
+        """'bzr serve' with no arguments or options should not traceback."""
+        out, err = self.run_bzr_error(
+            ['bzr serve requires one of --inet or --port'], 'serve')
+
+    def test_bzr_connect_to_bzr_ssh(self):
+        """User acceptance that get_transport of a bzr+ssh:// behaves correctly.
+
+        bzr+ssh:// should cause bzr to run a remote bzr smart server over SSH.
+        """
+        try:
+            from bzrlib.transport.sftp import SFTPServer
+        except ParamikoNotPresent:
+            raise TestSkipped('Paramiko not installed')
+        from bzrlib.tests.stub_sftp import StubServer
+        
+        # Make a branch
+        self.make_branch('a_branch')
+
+        # Start an SSH server
+        self.command_executed = []
+        # XXX: This is horrible -- we define a really dumb SSH server that
+        # executes commands, and manage the hooking up of stdin/out/err to the
+        # SSH channel ourselves.  Surely this has already been implemented
+        # elsewhere?
+        class StubSSHServer(StubServer):
+
+            test = self
+
+            def check_channel_exec_request(self, channel, command):
+                self.test.command_executed.append(command)
+                proc = subprocess.Popen(
+                    command, shell=True, stdin=subprocess.PIPE,
+                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                
+                # XXX: horribly inefficient, not to mention ugly.
+                # Start a thread for each of stdin/out/err, and relay bytes from
+                # the subprocess to channel and vice versa.
+                def ferry_bytes(read, write, close):
+                    while True:
+                        bytes = read(1)
+                        if bytes == '':
+                            close()
+                            break
+                        write(bytes)
+
+                file_functions = [
+                    (channel.recv, proc.stdin.write, proc.stdin.close),
+                    (proc.stdout.read, channel.sendall, channel.close),
+                    (proc.stderr.read, channel.sendall_stderr, channel.close)]
+                for read, write, close in file_functions:
+                    t = threading.Thread(
+                        target=ferry_bytes, args=(read, write, close))
+                    t.start()
+
+                return True
+
+        ssh_server = SFTPServer(StubSSHServer)
+        # XXX: We *don't* want to override the default SSH vendor, so we set
+        # _vendor to what _get_ssh_vendor returns.
+        ssh_server.setUp()
+        self.addCleanup(ssh_server.tearDown)
+        port = ssh_server._listener.port
+
+        # Access the branch via a bzr+ssh URL.  The BZR_REMOTE_PATH environment
+        # variable is used to tell bzr what command to run on the remote end.
+        path_to_branch = os.path.abspath('a_branch')
+        
+        orig_bzr_remote_path = os.environ.get('BZR_REMOTE_PATH')
+        os.environ['BZR_REMOTE_PATH'] = self.get_bzr_path()
+        try:
+            branch = Branch.open(
+                'bzr+ssh://fred:secret@localhost:%d%s' % (port, path_to_branch))
+            
+            branch.repository.get_revision_graph()
+            self.assertEqual(None, branch.last_revision())
+            # Check we can perform write operations
+            branch.bzrdir.root_transport.mkdir('foo')
+        finally:
+            # Restore the BZR_REMOTE_PATH environment variable back to its
+            # original state.
+            if orig_bzr_remote_path is None:
+                del os.environ['BZR_REMOTE_PATH']
+            else:
+                os.environ['BZR_REMOTE_PATH'] = orig_bzr_remote_path
+
+        self.assertEqual(
+            ['%s serve --inet --directory=/ --allow-writes'
+             % self.get_bzr_path()],
+            self.command_executed)
+        

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2006-08-31 18:18:09 +0000
+++ bzrlib/tests/blackbox/test_status.py	2006-09-07 21:37:43 +0000
@@ -28,7 +28,7 @@
 import sys
 from tempfile import TemporaryFile
 
-from bzrlib import bzrdir, errors, ignores
+from bzrlib import bzrdir, errors
 import bzrlib.branch
 from bzrlib.builtins import merge
 from bzrlib.osutils import pathjoin
@@ -62,8 +62,6 @@
         """Test basic branch status"""
         wt = self.make_branch_and_tree('.')
 
-        ignores._set_user_ignores(['./.bazaar'])
-
         # status with no commits or files - it must
         # work and show no output. We do this with no
         # commits to be sure that it's not going to fail
@@ -95,8 +93,6 @@
         """Tests branch status with revisions"""
         wt = self.make_branch_and_tree('.')
 
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.build_tree(['hello.c', 'bye.c'])
         wt.add('hello.c')
         wt.add('bye.c')
@@ -153,8 +149,6 @@
         wt = self.make_branch_and_tree('.')
         b = wt.branch
 
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
         wt.add('directory')
         wt.add('test.c')
@@ -215,8 +209,6 @@
 class TestStatus(TestCaseWithTransport):
 
     def test_status(self):
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.run_bzr("init")
         self.build_tree(['hello.txt'])
         result = self.run_bzr("status")[0]

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-08-29 20:47:02 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-10-06 05:53:44 +0000
@@ -40,13 +40,14 @@
 import sys
 
 import bzrlib
+from bzrlib import (
+    osutils,
+    )
 from bzrlib.branch import Branch
-import bzrlib.bzrdir as bzrdir
 from bzrlib.errors import BzrCommandError
 from bzrlib.osutils import (
     has_symlinks,
     pathjoin,
-    rmtree,
     terminal_width,
     )
 from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
@@ -147,75 +148,6 @@
         test.runbzr('add goodbye')
         test.runbzr('commit -m setup goodbye')
 
-    def test_export(self):
-        os.mkdir('branch')
-        os.chdir('branch')
-        self.example_branch()
-        self.runbzr('export ../latest')
-        self.assertEqual(file('../latest/goodbye', 'rt').read(), 'baz')
-        self.runbzr('export ../first -r 1')
-        self.assert_(not os.path.exists('../first/goodbye'))
-        self.assertEqual(file('../first/hello', 'rt').read(), 'foo')
-        self.runbzr('export ../first.gz -r 1')
-        self.assertEqual(file('../first.gz/hello', 'rt').read(), 'foo')
-        self.runbzr('export ../first.bz2 -r 1')
-        self.assertEqual(file('../first.bz2/hello', 'rt').read(), 'foo')
-
-        from tarfile import TarFile
-        self.runbzr('export ../first.tar -r 1')
-        self.assert_(os.path.isfile('../first.tar'))
-        tf = TarFile('../first.tar')
-        self.assert_('first/hello' in tf.getnames(), tf.getnames())
-        self.assertEqual(tf.extractfile('first/hello').read(), 'foo')
-        self.runbzr('export ../first.tar.gz -r 1')
-        self.assert_(os.path.isfile('../first.tar.gz'))
-        self.runbzr('export ../first.tbz2 -r 1')
-        self.assert_(os.path.isfile('../first.tbz2'))
-        self.runbzr('export ../first.tar.bz2 -r 1')
-        self.assert_(os.path.isfile('../first.tar.bz2'))
-        self.runbzr('export ../first.tar.tbz2 -r 1')
-        self.assert_(os.path.isfile('../first.tar.tbz2'))
-
-        from bz2 import BZ2File
-        tf = TarFile('../first.tar.tbz2', 
-                     fileobj=BZ2File('../first.tar.tbz2', 'r'))
-        self.assert_('first.tar/hello' in tf.getnames(), tf.getnames())
-        self.assertEqual(tf.extractfile('first.tar/hello').read(), 'foo')
-        self.runbzr('export ../first2.tar -r 1 --root pizza')
-        tf = TarFile('../first2.tar')
-        self.assert_('pizza/hello' in tf.getnames(), tf.getnames())
-
-        from zipfile import ZipFile
-        self.runbzr('export ../first.zip -r 1')
-        self.failUnlessExists('../first.zip')
-        zf = ZipFile('../first.zip')
-        self.assert_('first/hello' in zf.namelist(), zf.namelist())
-        self.assertEqual(zf.read('first/hello'), 'foo')
-
-        self.runbzr('export ../first2.zip -r 1 --root pizza')
-        zf = ZipFile('../first2.zip')
-        self.assert_('pizza/hello' in zf.namelist(), zf.namelist())
-        
-        self.runbzr('export ../first-zip --format=zip -r 1')
-        zf = ZipFile('../first-zip')
-        self.assert_('first-zip/hello' in zf.namelist(), zf.namelist())
-
-    def test_inventory(self):
-        bzr = self.runbzr
-        def output_equals(value, *args):
-            out = self.runbzr(['inventory'] + list(args), backtick=True)
-            self.assertEquals(out, value)
-
-        bzr('init')
-        open('a', 'wb').write('hello\n')
-        os.mkdir('b')
-
-        bzr('add a b')
-        bzr('commit -m add')
-
-        output_equals('a\n', '--kind', 'file')
-        output_equals('b\n', '--kind', 'directory')
-
     def test_pull_verbose(self):
         """Pull changes from one branch to another and watch the output."""
 
@@ -277,7 +209,7 @@
         self.runbzr('pull')
         self.runbzr('pull ../c')
         self.runbzr('branch ../c ../d')
-        rmtree('../c')
+        osutils.rmtree('../c')
         self.runbzr('pull')
         os.chdir('../b')
         self.runbzr('pull')

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2006-09-11 01:03:59 +0000
@@ -48,7 +48,7 @@
         self.assertContainsRe(out, 'second commit')
 
         # Nothing has changed
-        self.assertEqual('a2', wt.last_revision())
+        self.assertEqual(['a2'], wt.get_parent_ids())
 
         # Uncommit, don't prompt
         out, err = self.run_bzr('uncommit', '--force')
@@ -56,17 +56,15 @@
         self.assertContainsRe(out, 'second commit')
 
         # This should look like we are back in revno 1
-        self.assertEqual('a1', wt.last_revision())
+        self.assertEqual(['a1'], wt.get_parent_ids())
         out, err = self.run_bzr('status')
         self.assertEquals(out, 'modified:\n  a\n')
 
     def test_uncommit_checkout(self):
         wt = self.create_simple_tree()
-
-        checkout_tree = wt.bzrdir.sprout('checkout').open_workingtree()
-        checkout_tree.branch.bind(wt.branch)
-
-        self.assertEqual('a2', checkout_tree.last_revision())
+        checkout_tree = wt.branch.create_checkout('checkout')
+
+        self.assertEqual(['a2'], checkout_tree.get_parent_ids())
 
         os.chdir('checkout')
         out, err = self.run_bzr('uncommit', '--dry-run', '--force')
@@ -74,7 +72,7 @@
         self.assertNotContainsRe(out, 'initial commit')
         self.assertContainsRe(out, 'second commit')
 
-        self.assertEqual('a2', checkout_tree.last_revision())
+        self.assertEqual(['a2'], checkout_tree.get_parent_ids())
 
         out, err = self.run_bzr('uncommit', '--force')
         self.assertNotContainsRe(out, 'initial commit')
@@ -82,9 +80,9 @@
 
         # uncommit in a checkout should uncommit the parent branch
         # (but doesn't effect the other working tree)
-        self.assertEquals('a1', checkout_tree.last_revision())
+        self.assertEquals(['a1'], checkout_tree.get_parent_ids())
         self.assertEquals('a1', wt.branch.last_revision())
-        self.assertEquals('a2', wt.last_revision())
+        self.assertEquals(['a2'], wt.get_parent_ids())
 
     def test_uncommit_bound(self):
         os.mkdir('a')
@@ -95,8 +93,7 @@
         t_a.commit('commit 1')
         t_a.commit('commit 2')
         t_a.commit('commit 3')
-        b = t_a.bzrdir.sprout('b').open_branch()
-        b.bind(t_a.branch)
+        b = t_a.branch.create_checkout('b').branch
         uncommit.uncommit(b)
         self.assertEqual(len(b.revision_history()), 2)
         self.assertEqual(len(t_a.branch.revision_history()), 2)
@@ -115,7 +112,7 @@
 
         self.assertNotContainsRe(out, 'initial commit')
         self.assertContainsRe(out, 'second commit')
-        self.assertEqual('a1', wt.last_revision())
+        self.assertEqual(['a1'], wt.get_parent_ids())
         self.assertEqual('a1', wt.branch.last_revision())
 
     def test_uncommit_neg_1(self):
@@ -152,8 +149,7 @@
 
         os.chdir('tree')
         out, err = self.run_bzr('uncommit', '--force')
-        self.assertEqual('a1', wt.last_revision())
-        self.assertEqual(['b3'], wt.pending_merges())
+        self.assertEqual(['a1', 'b3'], wt.get_parent_ids())
 
     def test_uncommit_multiple_merge(self):
         wt = self.create_simple_tree()
@@ -191,14 +187,12 @@
         wt.branch.fetch(tree2.branch)
         wt.set_pending_merges(['b4'])
 
-        self.assertEqual('a3', wt.last_revision())
-        self.assertEqual(['b4'], wt.pending_merges())
+        self.assertEqual(['a3', 'b4'], wt.get_parent_ids())
 
         os.chdir('tree')
         out, err = self.run_bzr('uncommit', '--force', '-r', '2')
 
-        self.assertEqual('a2', wt.last_revision())
-        self.assertEqual(['b3', 'b4'], wt.pending_merges())
+        self.assertEqual(['a2', 'b3', 'b4'], wt.get_parent_ids())
 
     def test_uncommit_octopus_merge(self):
         # Check that uncommit keeps the pending merges in the same order

=== added file 'bzrlib/tests/blackbox/test_version_info.py'
--- bzrlib/tests/blackbox/test_version_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_version_info.py	2006-09-22 15:19:55 +0000
@@ -0,0 +1,139 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Blackbox tests for version_info"""
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestVersionInfo(TestCaseWithTransport):
+
+    def test_invalid_format(self):
+        self.run_bzr('version-info', '--format', 'quijibo', retcode=3)
+
+    def create_tree(self):
+        wt = self.make_branch_and_tree('branch')
+
+        self.build_tree(['branch/a'])
+        wt.add('a')
+        wt.commit('adding a', rev_id='r1')
+
+        self.build_tree(['branch/b'])
+        wt.add('b')
+        wt.commit('adding b', rev_id='r2')
+
+        self.revisions = wt.branch.revision_history()
+        return wt
+
+    def test_basic(self):
+        self.create_tree()
+
+        txt = self.run_bzr('version-info', 'branch')[0]
+        self.assertContainsRe(txt, 'date:')
+        self.assertContainsRe(txt, 'build-date:')
+        self.assertContainsRe(txt, 'revno: 2')
+        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+
+    def test_all(self):
+        """'--all' includes clean, revision history, and file revisions"""
+        self.create_tree()
+        txt = self.run_bzr('version-info', 'branch',
+                           '--all')[0]
+        self.assertContainsRe(txt, 'date:')
+        self.assertContainsRe(txt, 'revno: 2')
+        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+        self.assertContainsRe(txt, 'clean: True')
+        self.assertContainsRe(txt, 'revisions:')
+        for rev_id in self.revisions:
+            self.assertContainsRe(txt, 'id: ' + rev_id)
+        self.assertContainsRe(txt, 'message: adding a')
+        self.assertContainsRe(txt, 'message: adding b')
+        self.assertContainsRe(txt, 'file-revisions:')
+        self.assertContainsRe(txt, 'path: a')
+        self.assertContainsRe(txt, 'path: b')
+
+    def test_clean(self):
+        """Test that --check-clean includes the right info"""
+        self.create_tree()
+
+        txt = self.run_bzr('version-info', 'branch',
+                           '--check-clean')[0]
+        self.assertContainsRe(txt, 'clean: True')
+
+        self.build_tree_contents([('branch/c', 'now unclean\n')])
+        txt = self.run_bzr('version-info', 'branch',
+                           '--check-clean')[0]
+        self.assertContainsRe(txt, 'clean: False')
+
+        txt = self.run_bzr('version-info', 'branch',
+                           '--check-clean', '--include-file-revisions')[0]
+        self.assertContainsRe(txt, 'revision: unversioned')
+
+        os.remove('branch/c')
+
+    def test_no_working_tree(self):
+        tree = self.create_tree()
+        branch = self.make_branch('just_branch')
+        branch.pull(tree.branch)
+
+        txt = self.run_bzr('version-info', 'just_branch')[0]
+        self.assertStartsWith(txt, 'revision-id: r2\n')
+
+    def assertEqualNoBuildDate(self, text1, text2):
+        """Compare 2 texts, but ignore the build-date field.
+
+        build-date is the current timestamp, accurate to seconds. But the
+        clock is always ticking, and it may have ticked between the time
+        that text1 and text2 were generated.
+        """
+        lines1 = text1.splitlines(True)
+        lines2 = text2.splitlines(True)
+        for line1, line2 in zip(lines1, lines2):
+            if line1.startswith('build-date: '):
+                self.assertStartsWith(line2, 'build-date: ')
+            else:
+                self.assertEqual(line1, line2)
+        self.assertEqual(len(lines1), len(lines2))
+
+    def test_no_branch(self):
+        """Test that bzr defaults to the local working directory"""
+        self.create_tree()
+
+        txt1 = self.run_bzr('version-info', 'branch')[0]
+
+        os.chdir('branch')
+        txt2 = self.run_bzr('version-info')[0]
+        self.assertEqualNoBuildDate(txt1, txt2)
+
+    def test_rio(self):
+        """Test that we can pass --format=rio"""
+        self.create_tree()
+
+        txt = self.run_bzr('version-info', 'branch')[0]
+        txt1 = self.run_bzr('version-info', '--format', 'rio', 'branch')[0]
+        txt2 = self.run_bzr('version-info', '--format=rio', 'branch')[0]
+        self.assertEqualNoBuildDate(txt, txt1)
+        self.assertEqualNoBuildDate(txt, txt2)
+
+    def test_python(self):
+        """Test that we can do --format=python"""
+        self.create_tree()
+
+        txt = self.run_bzr('version-info', '--format', 'python', 'branch')[0]
+
+        self.assertContainsRe(txt, 'version_info = {')

=== modified file 'bzrlib/tests/branch_implementations/test_bound_sftp.py'
--- bzrlib/tests/branch_implementations/test_bound_sftp.py	2006-08-25 05:03:17 +0000
+++ bzrlib/tests/branch_implementations/test_bound_sftp.py	2006-09-11 01:03:59 +0000
@@ -50,10 +50,9 @@
         wt_child = b_base.bzrdir.sprout('child').open_workingtree()
         self.sftp_base = Branch.open(self.get_url('base'))
         wt_child.branch.bind(self.sftp_base)
-
+        # check the branch histories are ready for using in tests.
         self.assertEqual(['r@b-1'], b_base.revision_history())
         self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
-
         return b_base, wt_child
 
     def tearDown(self):
@@ -70,6 +69,9 @@
         wt_base.commit('first', rev_id='r@b-1')
 
         b_base = wt_base.branch
+        # manually make a branch we can bind, because the default format
+        # may not be bindable-from, and we want to test the side effects etc
+        # of bondage.
         old_format = BzrDirFormat.get_default_format()
         BzrDirFormat.set_default_format(BzrDirMetaFormat1())
         try:
@@ -82,7 +84,14 @@
         sftp_b_base = Branch.open(self.get_url('base'))
         b_child.bind(sftp_b_base)
         self.assertEqual(sftp_b_base.base, b_child.get_bound_location())
-        # this line is more of a working tree test line, but - what the hey.
+        # the bind must not have given b_child history:
+        self.assertEqual([], b_child.revision_history())
+        # we should be able to update the branch at this point:
+        self.assertEqual(None, b_child.update())
+        # and now there must be history.
+        self.assertEqual(['r@b-1'], b_child.revision_history())
+        # this line is more of a working tree test line, but - what the hey,
+        # it has work to do.
         b_child.bzrdir.open_workingtree().update()
         self.failUnlessExists('child/a')
         self.failUnlessExists('child/b')
@@ -99,7 +108,7 @@
         self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
         self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
 
-    def test_bound_fail(self):
+    def test_bound_commit_fails_when_out_of_date(self):
         # Make sure commit fails if out of date.
         b_base, wt_child = self.create_branches()
 
@@ -127,9 +136,7 @@
     def test_double_binding(self):
         b_base, wt_child = self.create_branches()
 
-        wt_child2 = wt_child.bzrdir.sprout('child2').open_workingtree()
-
-        wt_child2.branch.bind(wt_child.branch)
+        wt_child2 = wt_child.branch.create_checkout('child2')
 
         open('child2/a', 'wb').write('new contents\n')
         self.assertRaises(errors.CommitToDoubleBoundBranch,
@@ -162,8 +169,8 @@
                 wt_child.branch.bind, sftp_b_base)
 
     def test_commit_remote_bound(self):
-        # Make sure it is detected if the current base
-        # suddenly is bound when child goes to commit
+        # Make sure it is detected if the current base is bound during the
+        # objects lifetime, when the child goes to commit.
         b_base, wt_child = self.create_branches()
 
         b_base.bzrdir.sprout('newbase')
@@ -214,29 +221,19 @@
         self.assertRaises(errors.DivergedBranches,
                 wt_child.branch.bind, sftp_b_base)
 
-        # TODO: jam 20051230 merge_inner doesn't set pending merges
-        #       Is this on purpose?
-        #       merge_inner also doesn't fetch any missing revisions
-        #merge_inner(wt_child.branch, sftp_b_base.revision_tree('r@b-2'), 
-        #        wt_child.branch.revision_tree('r@b-1'))
-        # TODO: jam 20051230 merge(..., (None, None), ...) seems to
-        #       cause an infinite loop of some sort. It definitely doesn't
-        #       work, you have to use list notation
-        merge((sftp_b_base.base, 2), [None, None], this_dir=wt_child.branch.base)
-
+        wt_child.merge_from_branch(sftp_b_base)
         self.assertEqual([wt_child_rev, 'r@b-2'], wt_child.get_parent_ids())
         wt_child.commit('merged', rev_id='r@c-3')
 
-        # After a merge, trying to bind again should succeed
-        # by pushing the new change to base
+        # After a merge, trying to bind again should succeed but not push the
+        # new change.
         wt_child.branch.bind(sftp_b_base)
 
-        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3'],
-                b_base.revision_history())
-        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3'],
-                wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2'], b_base.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3'],
+            wt_child.branch.revision_history())
 
-    def test_bind_parent_ahead(self):
+    def test_bind_parent_ahead_preserves_parent(self):
         b_base, wt_child = self.create_branches()
 
         wt_child.branch.unbind()
@@ -250,7 +247,7 @@
         sftp_b_base = Branch.open(self.get_url('base'))
         wt_child.branch.bind(sftp_b_base)
 
-        self.assertEqual(['r@b-1', 'r@b-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
 
         wt_child.branch.unbind()
 
@@ -262,13 +259,12 @@
         self.assertEqual(['r@b-1', 'r@b-2', 'r@b-3', 'r@b-4', 'r@b-5'],
                 b_base.revision_history())
 
-        self.assertEqual(['r@b-1', 'r@b-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
 
         wt_child.branch.bind(sftp_b_base)
-        self.assertEqual(['r@b-1', 'r@b-2', 'r@b-3', 'r@b-4', 'r@b-5'],
-                wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
 
-    def test_bind_child_ahead(self):
+    def test_bind_child_ahead_preserves_child(self):
         b_base, wt_child = self.create_branches()
 
         wt_child.branch.unbind()
@@ -280,7 +276,7 @@
         sftp_b_base = Branch.open(self.get_url('base'))
         wt_child.branch.bind(sftp_b_base)
 
-        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], b_base.revision_history())
 
         # Check and make sure it also works if child is ahead multiple
         wt_child.branch.unbind()
@@ -290,11 +286,10 @@
 
         self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3', 'r@c-4', 'r@c-5'],
                 wt_child.branch.revision_history())
-        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], b_base.revision_history())
 
         wt_child.branch.bind(sftp_b_base)
-        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3', 'r@c-4', 'r@c-5'],
-                b_base.revision_history())
+        self.assertEqual(['r@b-1'], b_base.revision_history())
 
     def test_commit_after_merge(self):
         from bzrlib.builtins import merge
@@ -315,12 +310,7 @@
         self.failIf(wt_child.branch.repository.has_revision('r@d-2'))
         self.failIf(b_base.repository.has_revision('r@d-2'))
 
-        # TODO: jam 20051230 merge_inner doesn't set pending merges
-        #       Is this on purpose?
-        #       merge_inner also doesn't fetch any missing revisions
-        #merge_inner(wt_child.branch, b_other.revision_tree('r@d-2'),
-        #        wt_child.branch.revision_tree('r@b-1'))
-        merge((wt_other.branch.base, 2), [None, None], this_dir=wt_child.branch.base)
+        wt_child.merge_from_branch(wt_other.branch)
 
         self.failUnlessExists('child/c')
         self.assertEqual(['r@d-2'], wt_child.get_parent_ids()[1:])
@@ -328,7 +318,7 @@
         self.failIf(b_base.repository.has_revision('r@d-2'))
 
         # Commit should succeed, and cause merged revisions to
-        # be pulled into base
+        # be pushed into base
         wt_child.commit('merge other', rev_id='r@c-2')
         self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
         self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
@@ -339,7 +329,9 @@
 
         open('a', 'ab').write('child adds some text\n')
 
+        # this deletes the branch from memory
         del b_base
+        # and this moves it out of the way on disk
         os.rename('base', 'hidden_base')
 
         self.assertRaises(errors.BoundBranchConnectionFailure,
@@ -356,7 +348,9 @@
         self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
         self.assertEqual(['r@b-1', 'r@d-2'], wt_other.branch.revision_history())
 
+        # this deletes the branch from memory
         del b_base
+        # and this moves it out of the way on disk
         os.rename('base', 'hidden_base')
 
         self.assertRaises(errors.BoundBranchConnectionFailure,

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-08-23 10:30:07 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-09-11 01:03:59 +0000
@@ -164,7 +164,7 @@
         repo_b = self.make_repository('b')
         wt_a.bzrdir.open_repository().copy_content_into(repo_b)
         br_b = wt_a.bzrdir.open_branch().clone(repo_b.bzrdir, revision_id='1')
-        self.assertEqual(br_b.last_revision(), '1')
+        self.assertEqual('1', br_b.last_revision())
 
     def test_sprout_partial(self):
         # test sprouting with a prefix of the revision-history.
@@ -179,7 +179,7 @@
         repo_b = self.make_repository('b')
         wt_a.bzrdir.open_repository().copy_content_into(repo_b)
         br_b = wt_a.bzrdir.open_branch().sprout(repo_b.bzrdir, revision_id='1')
-        self.assertEqual(br_b.last_revision(), '1')
+        self.assertEqual('1', br_b.last_revision())
 
     def get_parented_branch(self):
         wt_a = self.make_branch_and_tree('a')
@@ -391,19 +391,47 @@
         tree_a = self.make_branch_and_tree('a')
         branch_a = tree_a.branch
         checkout_b = branch_a.create_checkout('b')
+        self.assertEqual(None, checkout_b.last_revision())
         checkout_b.commit('rev1', rev_id='rev1')
         self.assertEqual('rev1', branch_a.last_revision())
         self.assertNotEqual(checkout_b.branch.base, branch_a.base)
 
         checkout_c = branch_a.create_checkout('c', lightweight=True)
+        self.assertEqual('rev1', checkout_c.last_revision())
         checkout_c.commit('rev2', rev_id='rev2')
         self.assertEqual('rev2', branch_a.last_revision())
         self.assertEqual(checkout_c.branch.base, branch_a.base)
 
         os.mkdir('d')
         checkout_d = branch_a.create_checkout('d', lightweight=True)
+        self.assertEqual('rev2', checkout_d.last_revision())
         os.mkdir('e')
         checkout_e = branch_a.create_checkout('e')
+        self.assertEqual('rev2', checkout_e.last_revision())
+
+    def test_create_anonymous_lightweight_checkout(self):
+        """A lightweight checkout from a readonly branch should succeed."""
+        tree_a = self.make_branch_and_tree('a')
+        rev_id = tree_a.commit('put some content in the branch')
+        source_branch = bzrlib.branch.Branch.open(
+            'readonly+' + tree_a.bzrdir.root_transport.base)
+        # sanity check that the test will be valid
+        self.assertRaises((errors.LockError, errors.TransportNotPossible),
+            source_branch.lock_write)
+        checkout = source_branch.create_checkout('c', lightweight=True)
+        self.assertEqual(rev_id, checkout.last_revision())
+
+    def test_create_anonymous_heavyweight_checkout(self):
+        """A regular checkout from a readonly branch should succeed."""
+        tree_a = self.make_branch_and_tree('a')
+        rev_id = tree_a.commit('put some content in the branch')
+        source_branch = bzrlib.branch.Branch.open(
+            'readonly+' + tree_a.bzrdir.root_transport.base)
+        # sanity check that the test will be valid
+        self.assertRaises((errors.LockError, errors.TransportNotPossible),
+            source_branch.lock_write)
+        checkout = source_branch.create_checkout('c')
+        self.assertEqual(rev_id, checkout.last_revision())
 
 
 class ChrootedTests(TestCaseWithBranch):

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-08-30 13:20:22 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-10-02 13:05:18 +0000
@@ -17,22 +17,30 @@
 """Tests for bzrdir implementations - tests a bzrdir format."""
 
 from cStringIO import StringIO
+import errno
 import os
 from stat import S_ISDIR
 import sys
 
 import bzrlib.branch
-import bzrlib.bzrdir as bzrdir
+from bzrlib import (
+    bzrdir,
+    errors,
+    lockdir,
+    repository,
+    transactions,
+    transport,
+    ui,
+    workingtree,
+    )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.check import check
-import bzrlib.errors as errors
 from bzrlib.errors import (FileExists,
                            NoSuchRevision,
                            NoSuchFile,
                            UninitializableFormat,
                            NotBranchError,
                            )
-import bzrlib.repository as repository
 import bzrlib.revision
 from bzrlib.tests import (
                           ChrootedTestCase,
@@ -41,12 +49,8 @@
                           TestSkipped,
                           )
 from bzrlib.trace import mutter
-import bzrlib.transactions as transactions
-import bzrlib.transport as transport
 from bzrlib.transport import get_transport
-import bzrlib.ui as ui
 from bzrlib.upgrade import upgrade
-import bzrlib.workingtree as workingtree
 
 
 class TestCaseWithBzrDir(TestCaseWithTransport):
@@ -148,8 +152,26 @@
         dir.create_repository()
         dir.create_branch()
         wt = dir.create_workingtree(revision_id=bzrlib.revision.NULL_REVISION)
-        self.assertIs(wt.last_revision(), None)
+        self.assertEqual([], wt.get_parent_ids())
 
+    def test_destroy_workingtree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        tree.add('file')
+        tree.commit('first commit')
+        bzrdir = tree.bzrdir
+        try:
+            bzrdir.destroy_workingtree()
+        except errors.UnsupportedOperation:
+            raise TestSkipped('Format does not support destroying tree')
+        self.failIfExists('tree/file')
+        self.assertRaises(errors.NoWorkingTree, bzrdir.open_workingtree)
+        bzrdir.create_workingtree()
+        self.failUnlessExists('tree/file')
+        bzrdir.destroy_workingtree_metadata()
+        self.failUnlessExists('tree/file')
+        self.assertRaises(errors.NoWorkingTree, bzrdir.open_workingtree)
+            
     def test_clone_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
         target = dir.clone(self.get_url('target'))
@@ -445,7 +467,7 @@
         dir = tree.bzrdir
         target = dir.clone(self.get_url('target'), revision_id='1')
         self.skipIfNoWorkingTree(target)
-        self.assertEqual('1', target.open_workingtree().last_revision())
+        self.assertEqual(['1'], target.open_workingtree().get_parent_ids())
 
     def test_clone_bzrdir_incomplete_source_with_basis(self):
         # ensure that basis really does grab from the basis by having incomplete source
@@ -464,7 +486,7 @@
         target = dir.clone(self.get_url('target'), basis=tree.bzrdir)
         self.assertEqual('2', target.open_branch().last_revision())
         try:
-            self.assertEqual('2', target.open_workingtree().last_revision())
+            self.assertEqual(['2'], target.open_workingtree().get_parent_ids())
         except errors.NoWorkingTree:
             # It should have a working tree if it's able to have one, so if
             # we're here make sure it really can't have one.
@@ -515,11 +537,28 @@
         repo = dir.create_repository()
         repo.fetch(tree.branch.repository)
         self.assertTrue(repo.has_revision('1'))
+        try:
+            self.assertIs(dir.open_branch().last_revision(), None)
+        except errors.NotBranchError:
+            pass
         target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
+        # testing inventory isn't reasonable for repositories
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/repository/inventory.knit',
+                                     './.bzr/inventory'
                                      ])
+        try:
+            # If we happen to have a tree, we'll guarantee everything
+            # except for the tree root is the same.
+            inventory_f = file(dir.transport.base+'inventory', 'rb')
+            self.assertContainsRe(inventory_f.read(), 
+                                  '<inventory file_id="TREE_ROOT[^"]*"'
+                                  ' format="5">\n</inventory>\n')
+            inventory_f.close()
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
 
     def test_sprout_bzrdir_with_repository_to_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
@@ -837,7 +876,7 @@
         # place
         target.open_repository()
         # we trust that the working tree sprouting works via the other tests.
-        self.assertEqual('1', target.open_workingtree().last_revision())
+        self.assertEqual(['1'], target.open_workingtree().get_parent_ids())
         self.assertEqual('1', target.open_branch().last_revision())
 
     def test_sprout_bzrdir_tree_revision(self):
@@ -853,7 +892,7 @@
         tree.commit('revision 2', rev_id='2', allow_pointless=True)
         dir = tree.bzrdir
         target = self.sproutOrSkip(dir, self.get_url('target'), revision_id='1')
-        self.assertEqual('1', target.open_workingtree().last_revision())
+        self.assertEqual(['1'], target.open_workingtree().get_parent_ids())
 
     def test_sprout_bzrdir_incomplete_source_with_basis(self):
         # ensure that basis really does grab from the basis by having incomplete source
@@ -872,7 +911,7 @@
         target = self.sproutOrSkip(dir, self.get_url('target'),
                                    basis=tree.bzrdir)
         self.assertEqual('2', target.open_branch().last_revision())
-        self.assertEqual('2', target.open_workingtree().last_revision())
+        self.assertEqual(['2'], target.open_workingtree().get_parent_ids())
         self.assertTrue(target.open_branch().repository.has_revision('2'))
 
     def test_format_initialize_find_open(self):
@@ -1019,7 +1058,7 @@
             made_tree = made_control.create_workingtree(revision_id='a')
         except errors.NotLocalUrl:
             raise TestSkipped("Can't make working tree on transport %r" % t)
-        self.assertEqual('a', made_tree.last_revision())
+        self.assertEqual(['a'], made_tree.get_parent_ids())
         
     def test_open_workingtree(self):
         if not self.bzrdir_format.is_supported():
@@ -1317,23 +1356,30 @@
         unused_repo = thisdir.create_repository()
         master.lock_write()
         unused_repo.lock_write()
-        # two yes's : branch and repository. If the repo in this
-        # dir is inappropriately accessed, 3 will be needed, and
-        # we'll see that because the stream will be fully consumed
-        bzrlib.ui.ui_factory.stdin = StringIO("y\ny\ny\n")
-        master.bzrdir.break_lock()
-        # only two ys should have been read
-        self.assertEqual("y\n", bzrlib.ui.ui_factory.stdin.read())
-        # we should be able to lock a newly opened branch now
-        branch = master.bzrdir.open_branch()
-        branch.lock_write()
-        branch.unlock()
-        # we should not be able to lock the repository in thisdir as its still
-        # held by the explicit lock we took, and the break lock should not have
-        # touched it.
-        repo = thisdir.open_repository()
-        self.assertRaises(errors.LockContention, repo.lock_write)
-        unused_repo.unlock()
+        try:
+            # two yes's : branch and repository. If the repo in this
+            # dir is inappropriately accessed, 3 will be needed, and
+            # we'll see that because the stream will be fully consumed
+            bzrlib.ui.ui_factory.stdin = StringIO("y\ny\ny\n")
+            master.bzrdir.break_lock()
+            # only two ys should have been read
+            self.assertEqual("y\n", bzrlib.ui.ui_factory.stdin.read())
+            # we should be able to lock a newly opened branch now
+            branch = master.bzrdir.open_branch()
+            branch.lock_write()
+            branch.unlock()
+            # we should not be able to lock the repository in thisdir as its still
+            # held by the explicit lock we took, and the break lock should not have
+            # touched it.
+            repo = thisdir.open_repository()
+            orig_default = lockdir._DEFAULT_TIMEOUT_SECONDS
+            try:
+                lockdir._DEFAULT_TIMEOUT_SECONDS = 1
+                self.assertRaises(errors.LockContention, repo.lock_write)
+            finally:
+                lockdir._DEFAULT_TIMEOUT_SECONDS = orig_default
+        finally:
+            unused_repo.unlock()
         self.assertRaises(errors.LockBroken, master.unlock)
 
     def test_break_lock_tree(self):

=== modified file 'bzrlib/tests/interrepository_implementations/test_interrepository.py'
--- bzrlib/tests/interrepository_implementations/test_interrepository.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/interrepository_implementations/test_interrepository.py	2006-09-10 20:12:00 +0000
@@ -186,6 +186,21 @@
         from_tree.add('filename', 'funky-chars<>%&;"\'')
         from_tree.commit('commit filename')
         to_repo = self.make_to_repository('to')
+        to_repo.fetch(from_tree.branch.repository, from_tree.get_parent_ids()[0])
+
+    def test_fetch_no_inventory_revision(self):
+        """Old inventories lack revision_ids, so simulate this"""
+        from_tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/filename'])
+        from_tree.add('filename', 'funky-chars<>%&;"\'')
+        from_tree.commit('commit filename')
+        old_deserialise = from_tree.branch.repository.deserialise_inventory
+        def deserialise(revision_id, text):
+            inventory = old_deserialise(revision_id, text)
+            inventory.revision_id = None
+            return inventory
+        from_tree.branch.repository.deserialise_inventory = deserialise
+        to_repo = self.make_to_repository('to')
         to_repo.fetch(from_tree.branch.repository, from_tree.last_revision())
 
 
@@ -248,7 +263,8 @@
         # repository.
 
         # 'ghost' is a ghost in missing_ghost and not in with_ghost_rev
-        inv = Inventory()
+        inv = Inventory(revision_id='ghost')
+        inv.root.revision = 'ghost'
         repo = self.make_repository('with_ghost_rev')
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
@@ -261,6 +277,8 @@
         repo.add_revision('ghost', rev)
          
         repo = self.make_to_repository('missing_ghost')
+        inv = Inventory(revision_id='with_ghost')
+        inv.root.revision = 'with_ghost'
         sha1 = repo.add_inventory('with_ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,

=== modified file 'bzrlib/tests/repository_implementations/test_commit_builder.py'
--- bzrlib/tests/repository_implementations/test_commit_builder.py	2006-09-05 23:01:33 +0000
+++ bzrlib/tests/repository_implementations/test_commit_builder.py	2006-09-27 02:28:44 +0000
@@ -19,6 +19,7 @@
 from bzrlib import inventory
 from bzrlib.errors import UnsupportedOperation
 from bzrlib.repository import CommitBuilder
+from bzrlib import tests
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
 
 
@@ -33,7 +34,7 @@
         if builder.record_root_entry is True:
             ie = tree.inventory.root
             parent_tree = tree.branch.repository.revision_tree(None)
-            parent_invs = [parent_tree.inventory]
+            parent_invs = []
             builder.record_entry_contents(ie, parent_invs, '', tree)
 
     def test_finish_inventory(self):
@@ -70,6 +71,8 @@
     def test_commit_without_root(self):
         """This should cause a deprecation warning, not an assertion failure"""
         tree = self.make_branch_and_tree(".")
+        if tree.branch.repository._format.rich_root_data:
+            raise tests.TestSkipped('Format requires root')
         self.build_tree(['foo'])
         tree.add('foo', 'foo-id')
         entry = tree.inventory['foo-id']
@@ -92,3 +95,15 @@
         # precisely test that - a repository that wants to can add it on deserialisation,
         # but thats all the current contract guarantees anyway.
         self.assertEqual(rev_id, tree.branch.repository.get_inventory(rev_id).revision_id)
+
+    def test_revision_tree(self):
+        tree = self.make_branch_and_tree(".")
+        builder = tree.branch.get_commit_builder([])
+        self.record_root(builder, tree)
+        builder.finish_inventory()
+        rev_id = builder.commit('foo bar')
+        rev_tree = builder.revision_tree()
+        # Just a couple simple tests to ensure that it actually follows
+        # the RevisionTree api.
+        self.assertEqual(rev_id, rev_tree.get_revision_id())
+        self.assertEqual([], rev_tree.get_parent_ids())

=== modified file 'bzrlib/tests/repository_implementations/test_fileid_involved.py'
--- bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-09-10 20:12:00 +0000
@@ -173,13 +173,22 @@
             self.branch.repository.fileids_altered_by_revision_ids(
                 ['rev-G', 'rev-F', 'rev-C', 'rev-B', 'rev-<D>', 'rev-K', 'rev-J']))
 
+    def fileids_altered_by_revision_ids(self, revision_ids):
+        """This is a wrapper to strip TREE_ROOT if it occurs"""
+        repo = self.branch.repository
+        root_id = self.branch.basis_tree().inventory.root.file_id
+        result = repo.fileids_altered_by_revision_ids(revision_ids)
+        if root_id in result:
+            del result[root_id]
+        return result
+
     def test_fileids_altered_by_revision_ids(self):
         self.assertEqual(
             {'a-file-id-2006-01-01-abcd':set(['rev-A']),
              'b-file-id-2006-01-01-defg': set(['rev-A']),
              'c-funky<file-id> quiji%bo': set(['rev-A']),
              }, 
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-A"]))
+            self.fileids_altered_by_revision_ids(["rev-A"]))
         self.assertEqual(
             {'a-file-id-2006-01-01-abcd':set(['rev-B'])
              }, 

=== modified file 'bzrlib/tests/repository_implementations/test_reconcile.py'
--- bzrlib/tests/repository_implementations/test_reconcile.py	2006-08-04 04:41:02 +0000
+++ bzrlib/tests/repository_implementations/test_reconcile.py	2006-08-23 13:15:45 +0000
@@ -56,7 +56,8 @@
         t = get_transport(self.get_url())
         # an empty inventory with no revision for testing with.
         repo = self.make_repository('inventory_without_revision')
-        inv = Inventory()
+        inv = Inventory(revision_id='missing')
+        inv.root.revision = 'missing'
         repo.add_inventory('missing', inv, [])
 
         # an empty inventory with no revision for testing with.
@@ -64,8 +65,9 @@
         # that all the cached data is correctly converted into ghost links
         # and the referenced inventory still cleaned.
         repo = self.make_repository('inventory_without_revision_and_ghost')
-        inv = Inventory()
         repo.add_inventory('missing', inv, [])
+        inv = Inventory(revision_id='references_missing')
+        inv.root.revision = 'references_missing'
         sha1 = repo.add_inventory('references_missing', inv, ['missing'])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -79,6 +81,8 @@
         # a inventory with no parents and the revision has parents..
         # i.e. a ghost.
         repo = self.make_repository('inventory_one_ghost')
+        inv = Inventory(revision_id='ghost')
+        inv.root.revision = 'ghost'
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -92,6 +96,8 @@
         # a inventory with a ghost that can be corrected now.
         t.copy_tree('inventory_one_ghost', 'inventory_ghost_present')
         repo = bzrlib.repository.Repository.open('inventory_ghost_present')
+        inv = Inventory(revision_id='the_ghost')
+        inv.root.revision = 'the_ghost'
         sha1 = repo.add_inventory('the_ghost', inv, [])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -265,7 +271,8 @@
 
         # now setup the wrong-first parent case
         repo = tree.branch.repository
-        inv = Inventory()
+        inv = Inventory(revision_id='wrong-first-parent')
+        inv.root.revision = 'wrong-first-parent'
         sha1 = repo.add_inventory('wrong-first-parent', inv, ['2', '1'])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -278,7 +285,8 @@
 
         # now setup the wrong-secondary parent case
         repo = repo_secondary
-        inv = Inventory()
+        inv = Inventory(revision_id='wrong-secondary-parent')
+        inv.root.revision = 'wrong-secondary-parent'
         sha1 = repo.add_inventory('wrong-secondary-parent', inv, ['1', '3', '2'])
         rev = Revision(timestamp=0,
                        timezone=None,

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-08-23 10:30:07 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-10-02 13:05:18 +0000
@@ -30,7 +30,7 @@
                            UninitializableFormat,
                            NotBranchError,
                            )
-from bzrlib.inventory import Inventory
+from bzrlib.inventory import Inventory, InventoryDirectory
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
@@ -71,6 +71,15 @@
         tree_b.get_file_text('file1')
         rev1 = repo_b.get_revision('rev1')
 
+    def test_supports_rich_root(self):
+        tree = self.make_branch_and_tree('a')
+        tree.commit('')
+        second_revision = tree.commit('')
+        inv = tree.branch.repository.revision_tree(second_revision).inventory
+        rich_root = (inv.root.revision != second_revision)
+        self.assertEqual(rich_root, 
+                         tree.branch.repository.supports_rich_root())
+
     def test_clone_specific_format(self):
         """todo"""
 
@@ -144,13 +153,18 @@
 
     def test_revision_tree(self):
         wt = self.make_branch_and_tree('.')
+        wt.set_root_id('fixed-root')
         wt.commit('lala!', rev_id='revision-1', allow_pointless=True)
         tree = wt.branch.repository.revision_tree('revision-1')
-        self.assertEqual(list(tree.list_files()), [])
+        self.assertEqual('revision-1', tree.inventory.root.revision) 
+        expected = InventoryDirectory('fixed-root', '', None)
+        expected.revision = 'revision-1'
+        self.assertEqual([('', 'V', 'directory', 'fixed-root', expected)],
+                         list(tree.list_files(include_root=True)))
         tree = wt.branch.repository.revision_tree(None)
-        self.assertEqual([], list(tree.list_files()))
+        self.assertEqual([], list(tree.list_files(include_root=True)))
         tree = wt.branch.repository.revision_tree(NULL_REVISION)
-        self.assertEqual([], list(tree.list_files()))
+        self.assertEqual([], list(tree.list_files(include_root=True)))
 
     def test_fetch(self):
         # smoke test fetch to ensure that the convenience function works.
@@ -162,9 +176,36 @@
         tree_a.commit('rev1', rev_id='rev1')
         # fetch with a default limit (grab everything)
         repo = bzrdir.BzrDir.create_repository(self.get_url('b'))
-        repo.fetch(tree_a.branch.repository,
-                   revision_id=None,
-                   pb=bzrlib.progress.DummyProgress())
+        if (tree_a.branch.repository._format.rich_root_data and not
+            repo._format.rich_root_data):
+            raise TestSkipped('Cannot fetch from model2 to model1')
+        repo.fetch(tree_a.branch.repository,
+                   revision_id=None,
+                   pb=bzrlib.progress.DummyProgress())
+
+    def test_fetch_knit2(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        # fetch with a default limit (grab everything)
+        f = bzrdir.BzrDirMetaFormat1()
+        f._repository_format = repository.RepositoryFormatKnit2()
+        os.mkdir('b')
+        b_bzrdir = f.initialize(self.get_url('b'))
+        repo = b_bzrdir.create_repository()
+        repo.fetch(tree_a.branch.repository,
+                   revision_id=None,
+                   pb=bzrlib.progress.DummyProgress())
+        rev1_tree = repo.revision_tree('rev1')
+        lines = rev1_tree.get_file_lines(rev1_tree.inventory.root.file_id)
+        self.assertEqual([], lines)
+        b_branch = b_bzrdir.create_branch()
+        b_branch.pull(tree_a.branch)
+        tree_b = b_bzrdir.create_workingtree()
+        tree_b.commit('no change', rev_id='rev2')
+        rev2_tree = repo.revision_tree('rev2')
+        self.assertEqual('rev1', rev2_tree.inventory.root.revision)
 
     def test_get_revision_delta(self):
         tree_a = self.make_branch_and_tree('a')
@@ -240,6 +281,8 @@
         except errors.UpToDateFormat:
             # this is in the most current format already.
             return
+        except errors.BadConversionTarget, e:
+            raise TestSkipped(str(e))
         wt = WorkingTree.open(wt.basedir)
         new_signature = wt.branch.repository.get_signature_text('A')
         self.assertEqual(old_signature, new_signature)
@@ -317,9 +360,59 @@
         tree = self.make_branch_and_tree('.')
         tree.commit('message', rev_id='rev_id')
         self.assertEqual('rev_id', tree.basis_tree().inventory.root.revision)
-        rev_tree = tree.branch.repository.revision_tree(tree.last_revision())
+        rev_tree = tree.branch.repository.revision_tree(tree.get_parent_ids()[0])
         self.assertEqual('rev_id', rev_tree.inventory.root.revision)
 
+    def test_create_basis_inventory(self):
+        # Needs testing here because differences between repo and working tree
+        # basis inventory formats can lead to bugs.
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+
+        t._control_files.get_utf8('basis-inventory-cache')
+
+        basis_inv = t.basis_tree().inventory
+        self.assertEquals('r1', basis_inv.revision_id)
+        
+        store_inv = b.repository.get_inventory('r1')
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+        open('b', 'wb').write('b\n')
+        t.add('b')
+        t.commit('b', rev_id='r2')
+
+        t._control_files.get_utf8('basis-inventory-cache')
+
+        basis_inv_txt = t.read_basis_inventory()
+        basis_inv = bzrlib.xml6.serializer_v6.read_inventory_from_string(basis_inv_txt)
+        self.assertEquals('r2', basis_inv.revision_id)
+        store_inv = b.repository.get_inventory('r2')
+
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+    def test_upgrade_from_format4(self):
+        from bzrlib.tests.test_upgrade import _upgrade_dir_template
+        if self.repository_format.__class__ == repository.RepositoryFormat4:
+            raise TestSkipped('Cannot convert format-4 to itself')
+        self.build_tree_contents(_upgrade_dir_template)
+        old_repodir = bzrlib.bzrdir.BzrDir.open_unsupported('.')
+        old_repo_format = old_repodir.open_repository()._format
+        format = self.repository_format._matchingbzrdir
+        try:
+            format.repository_format = self.repository_format
+        except AttributeError:
+            pass
+        upgrade('.', format)
+
+    def test_pointless_commit(self):
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(errors.PointlessCommit, tree.commit, 'pointless',
+                          allow_pointless=False)
+        tree.commit('pointless', allow_pointless=True)
+
 
 class TestCaseWithComplexRepository(TestCaseWithRepository):
 
@@ -434,7 +527,8 @@
         # a inventory with no parents and the revision has parents..
         # i.e. a ghost.
         repo = self.make_repository('inventory_with_unnecessary_ghost')
-        inv = Inventory()
+        inv = Inventory(revision_id = 'ghost')
+        inv.root.revision = 'ghost'
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,
@@ -445,6 +539,8 @@
         rev.parent_ids = ['the_ghost']
         repo.add_revision('ghost', rev)
          
+        inv = Inventory(revision_id = 'the_ghost')
+        inv.root.revision = 'the_ghost'
         sha1 = repo.add_inventory('the_ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,
@@ -474,3 +570,29 @@
     def test_corrupt_revision_get_revision_reconcile(self):
         repo = repository.Repository.open('inventory_with_unnecessary_ghost')
         repo.get_revision_reconcile('ghost')
+
+
+class TestEscaping(TestCaseWithTransport):
+    """Test that repositories can be stored correctly on VFAT transports.
+    
+    Makes sure we have proper escaping of invalid characters, etc.
+
+    It'd be better to test all operations on the FakeVFATTransportDecorator,
+    but working trees go straight to the os not through the Transport layer.
+    Therefore we build some history first in the regular way and then 
+    check it's safe to access for vfat.
+    """
+
+    def test_on_vfat(self):
+        FOO_ID = 'foo<:>ID'
+        REV_ID = 'revid-1'
+        wt = self.make_branch_and_tree('repo')
+        self.build_tree(["repo/foo"], line_endings='binary')
+        # add file with id containing wierd characters
+        wt.add(['foo'], [FOO_ID])
+        wt.commit('this is my new commit', rev_id=REV_ID)
+        # now access over vfat; should be safe
+        branch = bzrdir.BzrDir.open('vfat+' + self.get_url('repo')).open_branch()
+        revtree = branch.repository.revision_tree(REV_ID)
+        contents = revtree.get_file_text(FOO_ID)
+        self.assertEqual(contents, 'contents of repo/foo\n')

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/stub_sftp.py	2006-09-08 18:52:17 +0000
@@ -153,12 +153,13 @@
     def open(self, path, flags, attr):
         path = self._realpath(path)
         try:
-            if hasattr(os, 'O_BINARY'):
-                flags |= os.O_BINARY
+            flags |= getattr(os, 'O_BINARY', 0)
             if getattr(attr, 'st_mode', None):
                 fd = os.open(path, flags, attr.st_mode)
             else:
-                fd = os.open(path, flags)
+                # os.open() defaults to 0777 which is
+                # an odd default mode for files
+                fd = os.open(path, flags, 0666)
         except OSError, e:
             return SFTPServer.convert_errno(e.errno)
 

=== modified file 'bzrlib/tests/test_ancestry.py'
--- bzrlib/tests/test_ancestry.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_ancestry.py	2006-10-15 15:55:32 +0000
@@ -19,42 +19,43 @@
 import os
 import sys
 
-from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.tests import TestCaseWithMemoryTransport
 from bzrlib.branch import Branch
 from bzrlib.revision import is_ancestor
 
 
-class TestAncestry(TestCaseWithTransport):
+class TestAncestry(TestCaseWithMemoryTransport):
+
+    def assertAncestryEqual(self, expected, revision_id, branch):
+        """Assert that the ancestry of revision_id in branch is as expected."""
+        ancestry = branch.repository.get_ancestry(revision_id)
+        self.assertEqual(expected, ancestry)
 
     def test_straightline_ancestry(self):
         """Test ancestry file when just committing."""
-        wt = self.make_branch_and_tree('.')
-        b = wt.branch
-
-        wt.commit(message='one',
-                  allow_pointless=True,
-                  rev_id='tester@foo--1')
-
-        wt.commit(message='two',
-                  allow_pointless=True,
-                  rev_id='tester@foo--2')
-
-        ancs = b.repository.get_ancestry('tester@foo--2')
-        self.assertEqual([None, 'tester@foo--1', 'tester@foo--2'], ancs)
-        self.assertEqual([None, 'tester@foo--1'], 
-                         b.repository.get_ancestry('tester@foo--1'))
+        tree = self.make_branch_and_memory_tree('.')
+        branch = tree.branch
+        tree.lock_write()
+        tree.add('')
+        rev_id_one = tree.commit('one')
+        rev_id_two = tree.commit('two', allow_pointless=True)
+        tree.unlock()
+
+        self.assertAncestryEqual([None, rev_id_one, rev_id_two],
+            rev_id_two, branch)
+        self.assertAncestryEqual([None, rev_id_one], rev_id_one, branch)
 
     def test_none_is_always_an_ancestor(self):
-        wt = self.make_branch_and_tree('.')
-        b = wt.branch
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
         # note this is tested before any commits are done.
-        self.assertEqual(True, is_ancestor(None, None, b))
-        wt.commit(message='one',
-                  allow_pointless=True,
-                  rev_id='tester@foo--1')
-        self.assertEqual(True, is_ancestor(None, None, b))
-        self.assertEqual(True, is_ancestor('tester@foo--1', None, b))
-        self.assertEqual(False, is_ancestor(None, 'tester@foo--1', b))
+        self.assertTrue(is_ancestor(None, None, tree.branch))
+        rev_id = tree.commit('one')
+        tree.unlock()
+        self.assertTrue(is_ancestor(None, None, tree.branch))
+        self.assertTrue(is_ancestor(rev_id, None, tree.branch))
+        self.assertFalse(is_ancestor(None, rev_id, tree.branch))
 
 
 # TODO: check that ancestry is updated to include indirectly merged revisions

=== modified file 'bzrlib/tests/test_bad_files.py'
--- bzrlib/tests/test_bad_files.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_bad_files.py	2006-09-08 18:52:17 +0000
@@ -18,7 +18,6 @@
 """Tests being able to ignore mad filetypes.
 """
 
-from bzrlib import ignores
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.errors import BadFileKindError
 import os
@@ -58,14 +57,11 @@
         # put some bogus stuff in the tree
 
         # We can only continue if we have mkfifo
-        if not hasattr(os, 'mkfifo'):
+        if getattr(os, 'mkfifo', None) is None:
             # TODO: Ultimately this should be TestSkipped
             # or PlatformDeficiency
             return
 
-        # Ignore the .bazaar/ home directory that is created
-        ignores._set_user_ignores(['./.bazaar'])
-
         # status with nothing
         verify_status(self, wt, [])
 

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2006-09-05 20:08:52 +0000
+++ bzrlib/tests/test_branch.py	2006-09-07 23:31:28 +0000
@@ -77,43 +77,6 @@
             branch.unlock()
 
 
-class TestBranchEscaping(TestCaseWithTransport):
-    """Test a branch can be correctly stored and used on a vfat-like transport
-    
-    Makes sure we have proper escaping of invalid characters, etc.
-
-    It'd be better to test all operations on the FakeVFATTransportDecorator,
-    but working trees go straight to the os not through the Transport layer.
-    Therefore we build some history first in the regular way and then 
-    check it's safe to access for vfat.
-    """
-
-    FOO_ID = 'foo<:>ID'
-    REV_ID = 'revid-1'
-
-    def setUp(self):
-        super(TestBranchEscaping, self).setUp()
-        from bzrlib.repository import RepositoryFormatKnit1
-        bzrdir = BzrDirMetaFormat1().initialize(self.get_url())
-        repo = RepositoryFormatKnit1().initialize(bzrdir)
-        branch = bzrdir.create_branch()
-        wt = bzrdir.create_workingtree()
-        self.build_tree_contents([("foo", "contents of foo")])
-        # add file with id containing wierd characters
-        wt.add(['foo'], [self.FOO_ID])
-        wt.commit('this is my new commit', rev_id=self.REV_ID)
-
-    def test_branch_on_vfat(self):
-        from bzrlib.transport.fakevfat import FakeVFATTransportDecorator
-        # now access over vfat; should be safe
-        transport = FakeVFATTransportDecorator('vfat+' + self.get_url())
-        bzrdir, junk = BzrDir.open_containing_from_transport(transport)
-        branch = bzrdir.open_branch()
-        revtree = branch.repository.revision_tree(self.REV_ID)
-        contents = revtree.get_file_text(self.FOO_ID)
-        self.assertEqual(contents, 'contents of foo')
-
-
 class SampleBranchFormat(bzrlib.branch.BranchFormat):
     """A sample format
 

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2006-08-17 14:16:46 +0000
+++ bzrlib/tests/test_bundle.py	2006-10-06 05:53:44 +0000
@@ -19,12 +19,20 @@
 import sys
 import tempfile
 
-from bzrlib import inventory
-from bzrlib.builtins import merge
+from bzrlib import (
+    bzrdir,
+    errors,
+    inventory,
+    repository,
+    treebuilder,
+    )
+from bzrlib.builtins import _merge_helper
 from bzrlib.bzrdir import BzrDir
 from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.bundle.bundle_data import BundleTree
 from bzrlib.bundle.serializer import write_bundle, read_bundle
+from bzrlib.bundle.serializer.v08 import BundleSerializerV08
+from bzrlib.bundle.serializer.v09 import BundleSerializerV09
 from bzrlib.branch import Branch
 from bzrlib.diff import internal_diff
 from bzrlib.errors import (BzrError, TestamentMismatch, NotABundle, BadBundle, 
@@ -301,15 +309,69 @@
             [inventory.ROOT_ID, 'a', 'b', 'd', 'e'])
 
 
-class BundleTester(TestCaseWithTransport):
+class BundleTester1(TestCaseWithTransport):
+
+    def test_mismatched_bundle(self):
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        serializer = BundleSerializerV08('0.8')
+        b = self.make_branch('.', format=format)
+        self.assertRaises(errors.IncompatibleFormat, serializer.write, 
+                          b.repository, [], {}, StringIO())
+
+    def test_matched_bundle(self):
+        """Don't raise IncompatibleFormat for knit2 and bundle0.9"""
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        serializer = BundleSerializerV09('0.9')
+        b = self.make_branch('.', format=format)
+        serializer.write(b.repository, [], {}, StringIO())
+
+    def test_mismatched_model(self):
+        """Try copying a bundle from knit2 to knit1"""
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        source = self.make_branch_and_tree('source', format=format)
+        source.commit('one', rev_id='one-id')
+        source.commit('two', rev_id='two-id')
+        text = StringIO()
+        write_bundle(source.branch.repository, 'two-id', None, text, 
+                     format='0.9')
+        text.seek(0)
+
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit1()
+        target = self.make_branch('target', format=format)
+        self.assertRaises(errors.IncompatibleRevision, install_bundle, 
+                          target.repository, read_bundle(text))
+
+
+class V08BundleTester(TestCaseWithTransport):
+
+    format = '0.8'
+
+    def bzrdir_format(self):
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit1()
+        return format
+
+    def make_branch_and_tree(self, path, format=None):
+        if format is None:
+            format = self.bzrdir_format()
+        return TestCaseWithTransport.make_branch_and_tree(self, path, format)
+
+    def make_branch(self, path, format=None):
+        if format is None:
+            format = self.bzrdir_format()
+        return TestCaseWithTransport.make_branch(self, path, format)
 
     def create_bundle_text(self, base_rev_id, rev_id):
         bundle_txt = StringIO()
         rev_ids = write_bundle(self.b1.repository, rev_id, base_rev_id, 
-                               bundle_txt)
+                               bundle_txt, format=self.format)
         bundle_txt.seek(0)
         self.assertEqual(bundle_txt.readline(), 
-                         '# Bazaar revision bundle v0.8\n')
+                         '# Bazaar revision bundle v%s\n' % self.format)
         self.assertEqual(bundle_txt.readline(), '#\n')
 
         rev = self.b1.repository.get_revision(rev_id)
@@ -391,9 +453,10 @@
         else:
             if not os.path.exists(checkout_dir):
                 os.mkdir(checkout_dir)
-        tree = BzrDir.create_standalone_workingtree(checkout_dir)
+        tree = self.make_branch_and_tree(checkout_dir)
         s = StringIO()
-        ancestors = write_bundle(self.b1.repository, rev_id, None, s)
+        ancestors = write_bundle(self.b1.repository, rev_id, None, s,
+                                 format=self.format)
         s.seek(0)
         assert isinstance(s.getvalue(), str), (
             "Bundle isn't a bytestring:\n %s..." % repr(s.getvalue())[:40])
@@ -571,9 +634,14 @@
                           verbose=False)
         bundle = self.get_valid_bundle('a@cset-0-5', 'a@cset-0-6')
         other = self.get_checkout('a@cset-0-5')
+        tree1_inv = self.tree1.branch.repository.get_inventory_xml(
+            'a@cset-0-5')
+        tree2_inv = other.branch.repository.get_inventory_xml('a@cset-0-5')
+        self.assertEqualDiff(tree1_inv, tree2_inv)
         other.rename_one('sub/dir/nolastnewline.txt', 'sub/nolastnewline.txt')
         other.commit('rename file', rev_id='a@cset-0-6b')
-        merge([other.basedir, -1], [None, None], this_dir=self.tree1.basedir)
+        _merge_helper([other.basedir, -1], [None, None],
+                      this_dir=self.tree1.basedir)
         self.tree1.commit(u'Merge', rev_id='a@cset-0-7',
                           verbose=False)
         bundle = self.get_valid_bundle('a@cset-0-6', 'a@cset-0-7')
@@ -581,7 +649,7 @@
     def test_symlink_bundle(self):
         if not has_symlinks():
             raise TestSkipped("No symlink support")
-        self.tree1 = BzrDir.create_standalone_workingtree('b1')
+        self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
         tt = TreeTransform(self.tree1)
         tt.new_symlink('link', tt.root, 'bar/foo', 'link-1')
@@ -611,7 +679,7 @@
         self.get_valid_bundle('l@cset-0-3', 'l@cset-0-4')
 
     def test_binary_bundle(self):
-        self.tree1 = BzrDir.create_standalone_workingtree('b1')
+        self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
         tt = TreeTransform(self.tree1)
         
@@ -653,7 +721,7 @@
         self.get_valid_bundle(None, 'b@cset-0-4')
 
     def test_last_modified(self):
-        self.tree1 = BzrDir.create_standalone_workingtree('b1')
+        self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
         tt = TreeTransform(self.tree1)
         tt.new_file('file', tt.root, 'file', 'file')
@@ -674,14 +742,15 @@
         tt.create_file('file2', trans_id)
         tt.apply()
         other.commit('modify text in another tree', rev_id='a@lmod-0-2b')
-        merge([other.basedir, -1], [None, None], this_dir=self.tree1.basedir)
+        _merge_helper([other.basedir, -1], [None, None],
+                      this_dir=self.tree1.basedir)
         self.tree1.commit(u'Merge', rev_id='a@lmod-0-3',
                           verbose=False)
         self.tree1.commit(u'Merge', rev_id='a@lmod-0-4')
         bundle = self.get_valid_bundle('a@lmod-0-2a', 'a@lmod-0-4')
 
     def test_hide_history(self):
-        self.tree1 = BzrDir.create_standalone_workingtree('b1')
+        self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
 
         open('b1/one', 'wb').write('one\n')
@@ -693,7 +762,7 @@
         self.tree1.commit('modify', rev_id='a@cset-0-3')
         bundle_file = StringIO()
         rev_ids = write_bundle(self.tree1.branch.repository, 'a@cset-0-3',
-                               'a@cset-0-1', bundle_file)
+                               'a@cset-0-1', bundle_file, format=self.format)
         self.assertNotContainsRe(bundle_file.getvalue(), 'two')
         self.assertContainsRe(bundle_file.getvalue(), 'one')
         self.assertContainsRe(bundle_file.getvalue(), 'three')
@@ -785,11 +854,14 @@
         bundle = self.get_valid_bundle(None, 'white-4')
 
     def test_alt_timezone_bundle(self):
-        self.tree1 = self.make_branch_and_tree('b1')
+        self.tree1 = self.make_branch_and_memory_tree('b1')
         self.b1 = self.tree1.branch
+        builder = treebuilder.TreeBuilder()
 
-        self.build_tree(['b1/newfile'])
-        self.tree1.add(['newfile'])
+        self.tree1.lock_write()
+        builder.start_tree(self.tree1)
+        builder.build(['newfile'])
+        builder.finish_tree()
 
         # Asia/Colombo offset = 5 hours 30 minutes
         self.tree1.commit('non-hour offset timezone', rev_id='tz-1',
@@ -801,6 +873,7 @@
         self.assertEqual('Mon 2006-07-10 20:51:26.000000000 +0530', rev.date)
         self.assertEqual(19800, rev.timezone)
         self.assertEqual(1152544886.0, rev.timestamp)
+        self.tree1.unlock()
 
     def test_bundle_root_id(self):
         self.tree1 = self.make_branch_and_tree('b1')
@@ -811,6 +884,26 @@
         self.assertEqual('revid1', tree.inventory.root.revision)
 
 
+class V09BundleKnit2Tester(V08BundleTester):
+
+    format = '0.9'
+
+    def bzrdir_format(self):
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        return format
+
+
+class V09BundleKnit1Tester(V08BundleTester):
+
+    format = '0.9'
+
+    def bzrdir_format(self):
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit1()
+        return format
+
+
 class MungedBundleTester(TestCaseWithTransport):
 
     def build_test_bundle(self):

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2006-09-05 20:58:28 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-09-14 04:55:30 +0000
@@ -392,6 +392,28 @@
             get_transport(self.get_readonly_url('g/p/q')))
         self.assertEqual('g/p/q', relpath)
 
+    def test_open_from_transport(self):
+        # transport pointing at bzrdir should give a bzrdir with root transport
+        # set to the given transport
+        control = bzrdir.BzrDir.create(self.get_url())
+        transport = get_transport(self.get_url())
+        opened_bzrdir = bzrdir.BzrDir.open_from_transport(transport)
+        self.assertEqual(transport.base, opened_bzrdir.root_transport.base)
+        self.assertIsInstance(opened_bzrdir, bzrdir.BzrDir)
+        
+    def test_open_from_transport_no_bzrdir(self):
+        transport = get_transport(self.get_url())
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport,
+                          transport)
+
+    def test_open_from_transport_bzrdir_in_parent(self):
+        control = bzrdir.BzrDir.create(self.get_url())
+        transport = get_transport(self.get_url())
+        transport.mkdir('subdir')
+        transport = transport.clone('subdir')
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport,
+                          transport)
+
 
 class TestMeta1DirFormat(TestCaseWithTransport):
     """Tests specific to the meta1 dir format."""

=== modified file 'bzrlib/tests/test_command.py'
--- bzrlib/tests/test_command.py	2006-08-22 21:31:23 +0000
+++ bzrlib/tests/test_command.py	2006-09-01 03:55:07 +0000
@@ -21,7 +21,7 @@
     errors,
     )
 from bzrlib.commands import display_command
-from bzrlib.tests import TestCase
+from bzrlib.tests import TestCase, TestSkipped
 
 
 class TestCommands(TestCase):
@@ -49,6 +49,9 @@
     def test_unicode_option(self):
         # This error is actually thrown by optparse, when it
         # can't find the given option
+        import optparse
+        if optparse.__version__ == "1.5.3":
+            raise TestSkipped("optparse 1.5.3 can't handle unicode options")
         self.assertRaises(errors.BzrCommandError,
                           commands.run_bzr, ['log', u'--option\xb5'])
 

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/test_commit.py	2006-09-27 22:12:53 +0000
@@ -18,14 +18,18 @@
 import os
 
 import bzrlib
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib import (
+    errors,
+    lockdir,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
-from bzrlib.workingtree import WorkingTree
 from bzrlib.commit import Commit, NullCommitReporter
 from bzrlib.config import BranchConfig
 from bzrlib.errors import (PointlessCommit, BzrError, SigningFailed, 
                            LockContention)
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.workingtree import WorkingTree
 
 
 # TODO: Test commit with some added, and added-but-missing files
@@ -407,10 +411,14 @@
         bound = master.sprout('bound')
         wt = bound.open_workingtree()
         wt.branch.set_bound_location(os.path.realpath('master'))
+
+        orig_default = lockdir._DEFAULT_TIMEOUT_SECONDS
         master_branch.lock_write()
         try:
+            lockdir._DEFAULT_TIMEOUT_SECONDS = 1
             self.assertRaises(LockContention, wt.commit, 'silly')
         finally:
+            lockdir._DEFAULT_TIMEOUT_SECONDS = orig_default
             master_branch.unlock()
 
     def test_commit_bound_merge(self):
@@ -543,3 +551,9 @@
         timestamp = rev.timestamp
         timestamp_1ms = round(timestamp, 3)
         self.assertEqual(timestamp_1ms, timestamp)
+
+    def test_commit_unversioned_specified(self):
+        """Commit should raise if specified files isn't in basis or worktree"""
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(errors.PathsNotVersionedError, tree.commit, 
+                          'message', specific_files=['bogus'])

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/test_commit_merge.py	2006-09-10 19:04:48 +0000
@@ -63,7 +63,9 @@
         wtx = self.make_branch_and_tree('x')
         base_rev = wtx.commit('common parent')
         bx = wtx.branch
+        wtx.commit('establish root id')
         wty = wtx.bzrdir.sprout('y').open_workingtree()
+        self.assertEqual(wtx.get_root_id(), wty.get_root_id())
         by = wty.branch
 
         self.build_tree(['x/ecks', 'y/why'])

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2006-08-06 23:35:09 +0000
+++ bzrlib/tests/test_config.py	2006-10-12 01:07:42 +0000
@@ -64,8 +64,11 @@
 sample_branches_text = ("[http://www.example.com]\n"
                         "# Top level policy\n"
                         "email=Robert Collins <robertc@example.org>\n"
-                        "[http://www.example.com/useglobal]\n"
-                        "# different project, forces global lookup\n"
+                        "[http://www.example.com/ignoreparent]\n"
+                        "# different project: ignore parent dir config\n"
+                        "ignore_parents=true\n"
+                        "[http://www.example.com/norecurse]\n"
+                        "# configuration items that only apply to this dir\n"
                         "recurse=false\n"
                         "[/b/]\n"
                         "check_signatures=require\n"
@@ -77,7 +80,6 @@
                         "# test trailing / matching\n"
                         "[/a/*]\n"
                         "#subdirs will match but not the parent\n"
-                        "recurse=False\n"
                         "[/a/c]\n"
                         "check_signatures=ignore\n"
                         "post_commit=bzrlib.tests.test_config.post_commit\n"
@@ -535,58 +537,78 @@
         self.failUnless(isinstance(global_config, config.GlobalConfig))
         self.failUnless(global_config is my_config._get_global_config())
 
-    def test__get_section_no_match(self):
+    def test__get_matching_sections_no_match(self):
         self.get_branch_config('/')
-        self.assertEqual(None, self.my_location_config._get_section())
+        self.assertEqual([], self.my_location_config._get_matching_sections())
         
-    def test__get_section_exact(self):
+    def test__get_matching_sections_exact(self):
         self.get_branch_config('http://www.example.com')
-        self.assertEqual('http://www.example.com',
-                         self.my_location_config._get_section())
+        self.assertEqual([('http://www.example.com', '')],
+                         self.my_location_config._get_matching_sections())
    
-    def test__get_section_suffix_does_not(self):
+    def test__get_matching_sections_suffix_does_not(self):
         self.get_branch_config('http://www.example.com-com')
-        self.assertEqual(None, self.my_location_config._get_section())
+        self.assertEqual([], self.my_location_config._get_matching_sections())
 
-    def test__get_section_subdir_recursive(self):
+    def test__get_matching_sections_subdir_recursive(self):
         self.get_branch_config('http://www.example.com/com')
-        self.assertEqual('http://www.example.com',
-                         self.my_location_config._get_section())
-
-    def test__get_section_subdir_matches(self):
-        self.get_branch_config('http://www.example.com/useglobal')
-        self.assertEqual('http://www.example.com/useglobal',
-                         self.my_location_config._get_section())
-
-    def test__get_section_subdir_nonrecursive(self):
-        self.get_branch_config(
-            'http://www.example.com/useglobal/childbranch')
-        self.assertEqual('http://www.example.com',
-                         self.my_location_config._get_section())
-
-    def test__get_section_subdir_trailing_slash(self):
+        self.assertEqual([('http://www.example.com', 'com')],
+                         self.my_location_config._get_matching_sections())
+
+    def test__get_matching_sections_ignoreparent(self):
+        self.get_branch_config('http://www.example.com/ignoreparent')
+        self.assertEqual([('http://www.example.com/ignoreparent', '')],
+                         self.my_location_config._get_matching_sections())
+
+    def test__get_matching_sections_ignoreparent_subdir(self):
+        self.get_branch_config(
+            'http://www.example.com/ignoreparent/childbranch')
+        self.assertEqual([('http://www.example.com/ignoreparent', 'childbranch')],
+                         self.my_location_config._get_matching_sections())
+
+    def test__get_matching_sections_norecurse(self):
+        self.get_branch_config('http://www.example.com/norecurse')
+        self.assertEqual([('http://www.example.com/norecurse', ''),
+                          ('http://www.example.com', 'norecurse')],
+                         self.my_location_config._get_matching_sections())
+
+    def test__get_matching_sections_norecurse_subdir(self):
+        self.get_branch_config(
+            'http://www.example.com/norecurse/childbranch')
+        self.assertEqual([('http://www.example.com', 'norecurse/childbranch')],
+                         self.my_location_config._get_matching_sections())
+
+    def test__get_matching_sections_subdir_trailing_slash(self):
         self.get_branch_config('/b')
-        self.assertEqual('/b/', self.my_location_config._get_section())
+        self.assertEqual([('/b/', '')],
+                         self.my_location_config._get_matching_sections())
 
-    def test__get_section_subdir_child(self):
+    def test__get_matching_sections_subdir_child(self):
         self.get_branch_config('/a/foo')
-        self.assertEqual('/a/*', self.my_location_config._get_section())
+        self.assertEqual([('/a/*', ''), ('/a/', 'foo')],
+                         self.my_location_config._get_matching_sections())
 
-    def test__get_section_subdir_child_child(self):
+    def test__get_matching_sections_subdir_child_child(self):
         self.get_branch_config('/a/foo/bar')
-        self.assertEqual('/a/', self.my_location_config._get_section())
+        self.assertEqual([('/a/*', 'bar'), ('/a/', 'foo/bar')],
+                         self.my_location_config._get_matching_sections())
 
-    def test__get_section_trailing_slash_with_children(self):
+    def test__get_matching_sections_trailing_slash_with_children(self):
         self.get_branch_config('/a/')
-        self.assertEqual('/a/', self.my_location_config._get_section())
+        self.assertEqual([('/a/', '')],
+                         self.my_location_config._get_matching_sections())
 
-    def test__get_section_explicit_over_glob(self):
+    def test__get_matching_sections_explicit_over_glob(self):
+        # XXX: 2006-09-08 jamesh
+        # This test only passes because ord('c') > ord('*').  If there
+        # was a config section for '/a/?', it would get precedence
+        # over '/a/c'.
         self.get_branch_config('/a/c')
-        self.assertEqual('/a/c', self.my_location_config._get_section())
-
+        self.assertEqual([('/a/c', ''), ('/a/*', ''), ('/a/', 'c')],
+                         self.my_location_config._get_matching_sections())
 
     def test_location_without_username(self):
-        self.get_branch_config('http://www.example.com/useglobal')
+        self.get_branch_config('http://www.example.com/ignoreparent')
         self.assertEqual(u'Erik B\u00e5gfors <erik@bagfors.nu>',
                          self.my_config.username())
 
@@ -641,7 +663,7 @@
         self.get_branch_config('/a')
         self.assertEqual('local',
                          self.my_config.get_user_option('user_local_option'))
-        
+
     def test_post_commit_default(self):
         self.get_branch_config('/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
@@ -824,5 +846,5 @@
     def test_extract_email_address(self):
         self.assertEqual('jane@test.com',
                          config.extract_email_address('Jane <jane@test.com>'))
-        self.assertRaises(errors.BzrError,
+        self.assertRaises(errors.NoEmailInUsername,
                           config.extract_email_address, 'Jane Tester')

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2006-08-21 19:05:37 +0000
+++ bzrlib/tests/test_errors.py	2006-10-16 01:13:59 +0000
@@ -26,11 +26,48 @@
 
 class TestErrors(TestCaseWithTransport):
 
+    def test_inventory_modified(self):
+        error = errors.InventoryModified("a tree to be repred")
+        self.assertEqualDiff("The current inventory for the tree 'a tree to "
+            "be repred' has been modified, so a clean inventory cannot be "
+            "read without data loss.",
+            str(error))
+
+    def test_medium_not_connected(self):
+        error = errors.MediumNotConnected("a medium")
+        self.assertEqualDiff(
+            "The medium 'a medium' is not connected.", str(error))
+        
     def test_no_repo(self):
         dir = bzrdir.BzrDir.create(self.get_url())
         error = errors.NoRepositoryPresent(dir)
         self.assertNotEqual(-1, str(error).find((dir.transport.clone('..').base)))
         self.assertEqual(-1, str(error).find((dir.transport.base)))
+        
+    def test_no_smart_medium(self):
+        error = errors.NoSmartMedium("a transport")
+        self.assertEqualDiff("The transport 'a transport' cannot tunnel the "
+            "smart protocol.",
+            str(error))
+
+    def test_no_such_id(self):
+        error = errors.NoSuchId("atree", "anid")
+        self.assertEqualDiff("The file id anid is not present in the tree "
+            "atree.",
+            str(error))
+
+    def test_not_write_locked(self):
+        error = errors.NotWriteLocked('a thing to repr')
+        self.assertEqualDiff("'a thing to repr' is not write locked but needs "
+            "to be.",
+            str(error))
+
+    def test_too_many_concurrent_requests(self):
+        error = errors.TooManyConcurrentRequests("a medium")
+        self.assertEqualDiff("The medium 'a medium' has reached its concurrent "
+            "request limit. Be sure to finish_writing and finish_reading on "
+            "the current request that is open.",
+            str(error))
 
     def test_up_to_date(self):
         error = errors.UpToDateFormat(bzrdir.BzrDirFormat4())
@@ -47,6 +84,27 @@
                              repo.bzrdir.root_transport.base,
                              str(error))
 
+    def test_reading_completed(self):
+        error = errors.ReadingCompleted("a request")
+        self.assertEqualDiff("The MediumRequest 'a request' has already had "
+            "finish_reading called upon it - the request has been completed and"
+            " no more data may be read.",
+            str(error))
+
+    def test_writing_completed(self):
+        error = errors.WritingCompleted("a request")
+        self.assertEqualDiff("The MediumRequest 'a request' has already had "
+            "finish_writing called upon it - accept bytes may not be called "
+            "anymore.",
+            str(error))
+
+    def test_writing_not_completed(self):
+        error = errors.WritingNotComplete("a request")
+        self.assertEqualDiff("The MediumRequest 'a request' has not has "
+            "finish_writing called upon it - until the write phase is complete"
+            " no data may be read.",
+            str(error))
+
 
 class PassThroughError(errors.BzrNewError):
     """Pass through %(foo)s and %(bar)s"""
@@ -55,6 +113,10 @@
         errors.BzrNewError.__init__(self, foo=foo, bar=bar)
 
 
+class ErrorWithBadFormat(errors.BzrNewError):
+    """One format specifier: %(thing)s"""
+
+
 class TestErrorFormatting(TestCase):
     
     def test_always_str(self):
@@ -66,3 +128,19 @@
         # returned from e.__str__(), and it has non ascii characters
         s = str(e)
         self.assertEqual('Pass through \xc2\xb5 and bar', s)
+
+    def test_mismatched_format_args(self):
+        # Even though ErrorWithBadFormat's format string does not match the
+        # arguments we constructing it with, we can still stringify an instance
+        # of this exception. The resulting string will say its unprintable.
+        e = ErrorWithBadFormat(not_thing='x')
+        self.assertStartsWith(
+            str(e), 'Unprintable exception ErrorWithBadFormat(')
+
+
+class TestSpecificErrors(TestCase):
+    
+    def test_transport_not_possible(self):
+        e = errors.TransportNotPossible('readonly', 'original error')
+        self.assertEqual('Transport operation not possible:'
+                         ' readonly original error', str(e))

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_hashcache.py	2006-09-08 18:46:29 +0000
@@ -112,7 +112,7 @@
     def test_hashcache_raise(self):
         """check that hashcache can raise BzrError"""
         hc = self.make_hashcache()
-        if not hasattr(os, 'mkfifo'):
+        if getattr(os, 'mkfifo', None) is None:
             raise TestSkipped('filesystem fifos not supported on this system')
         os.mkfifo('a')
         # It's possible that the system supports fifos but the filesystem

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-07-14 19:25:24 +0000
+++ bzrlib/tests/test_http.py	2006-10-11 05:55:32 +0000
@@ -19,16 +19,21 @@
 
 # TODO: Should be renamed to bzrlib.transport.http.tests?
 
+import errno
+import select
+import socket
+import threading
+
 import bzrlib
-from bzrlib.errors import DependencyNotPresent
+from bzrlib.errors import DependencyNotPresent, UnsupportedProtocol
 from bzrlib.tests import TestCase, TestSkipped
-from bzrlib.transport import Transport
+from bzrlib.transport import get_transport, Transport
 from bzrlib.transport.http import extract_auth, HttpTransportBase
 from bzrlib.transport.http._urllib import HttpTransport_urllib
 from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 
 
-class FakeManager (object):
+class FakeManager(object):
 
     def __init__(self):
         self.credentials = []
@@ -37,6 +42,60 @@
         self.credentials.append([realm, host, username, password])
 
 
+class RecordingServer(object):
+    """A fake HTTP server.
+    
+    It records the bytes sent to it, and replies with a 200.
+    """
+
+    def __init__(self, expect_body_tail=None):
+        """Constructor.
+
+        :type expect_body_tail: str
+        :param expect_body_tail: a reply won't be sent until this string is
+            received.
+        """
+        self._expect_body_tail = expect_body_tail
+        self.host = None
+        self.port = None
+        self.received_bytes = ''
+
+    def setUp(self):
+        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._sock.bind(('127.0.0.1', 0))
+        self.host, self.port = self._sock.getsockname()
+        self._ready = threading.Event()
+        self._thread = threading.Thread(target=self._accept_read_and_reply)
+        self._thread.setDaemon(True)
+        self._thread.start()
+        self._ready.wait(5)
+
+    def _accept_read_and_reply(self):
+        self._sock.listen(1)
+        self._ready.set()
+        self._sock.settimeout(5)
+        try:
+            conn, address = self._sock.accept()
+            # On win32, the accepted connection will be non-blocking to start
+            # with because we're using settimeout.
+            conn.setblocking(True)
+            while not self.received_bytes.endswith(self._expect_body_tail):
+                self.received_bytes += conn.recv(4096)
+            conn.sendall('HTTP/1.1 200 OK\r\n')
+        except socket.timeout:
+            # Make sure the client isn't stuck waiting for us to e.g. accept.
+            self._sock.close()
+
+    def tearDown(self):
+        try:
+            self._sock.close()
+        except socket.error:
+            # We might have already closed it.  We don't care.
+            pass
+        self.host = None
+        self.port = None
+
+
 class TestHttpUrls(TestCase):
 
     def test_url_parsing(self):
@@ -68,9 +127,6 @@
         self.assertRaises(ValueError,
             t.abspath,
             '.bzr/')
-        self.assertRaises(ValueError,
-            t.abspath,
-            '/.bzr')
 
     def test_http_root_urls(self):
         """Construction of URLs from server root"""
@@ -126,6 +182,13 @@
         self.assertTrue(server.logs[0].find(
             '"GET /foo/bar HTTP/1.1" 200 - "-" "bzr/%s' % bzrlib.__version__) > -1)
 
+    def test_get_smart_medium(self):
+        # For HTTP, get_smart_medium should return the transport object.
+        server = self.get_readonly_server()
+        http_transport = self._transport(server.get_url())
+        medium = http_transport.get_smart_medium()
+        self.assertIs(medium, http_transport)
+        
 
 class TestHttpConnections_urllib(TestCaseWithWebserver, TestHttpMixins):
 
@@ -135,6 +198,14 @@
         TestCaseWithWebserver.setUp(self)
         self._prep_tree()
 
+    def test_has_on_bogus_host(self):
+        import urllib2
+        # Get a random address, so that we can be sure there is no
+        # http handler there.
+        s = socket.socket()
+        s.bind(('localhost', 0))
+        t = self._transport('http://%s:%s/' % s.getsockname())
+        self.assertRaises(urllib2.URLError, t.has, 'foo/bar')
 
 
 class TestHttpConnections_pycurl(TestCaseWithWebserver, TestHttpMixins):
@@ -153,7 +224,6 @@
         self._prep_tree()
 
 
-
 class TestHttpTransportRegistration(TestCase):
     """Test registrations of various http implementations"""
 
@@ -190,6 +260,34 @@
         self.assertEqual([[10, 12], [22, 26]], ranges)
 
 
+class TestPost(TestCase):
+
+    def _test_post_body_is_received(self, scheme):
+        server = RecordingServer(expect_body_tail='end-of-body')
+        server.setUp()
+        self.addCleanup(server.tearDown)
+        url = '%s://%s:%s/' % (scheme, server.host, server.port)
+        try:
+            http_transport = get_transport(url)
+        except UnsupportedProtocol:
+            raise TestSkipped('%s not available' % scheme)
+        code, response = http_transport._post('abc def end-of-body')
+        self.assertTrue(
+            server.received_bytes.startswith('POST /.bzr/smart HTTP/1.'))
+        self.assertTrue('content-length: 19\r' in server.received_bytes.lower())
+        # The transport should not be assuming that the server can accept
+        # chunked encoding the first time it connects, because HTTP/1.1, so we
+        # check for the literal string.
+        self.assertTrue(
+            server.received_bytes.endswith('\r\n\r\nabc def end-of-body'))
+
+    def test_post_body_is_received_urllib(self):
+        self._test_post_body_is_received('http+urllib')
+
+    def test_post_body_is_received_pycurl(self):
+        self._test_post_body_is_received('http+pycurl')
+
+
 class TestRangeHeader(TestCase):
     """Test range_header method"""
 
@@ -213,3 +311,34 @@
         self.check_header('0-9,300-5000,-50',
                           ranges=[(0,9), (300,5000)],
                           tail=50)
+
+        
+class TestRecordingServer(TestCase):
+
+    def test_create(self):
+        server = RecordingServer(expect_body_tail=None)
+        self.assertEqual('', server.received_bytes)
+        self.assertEqual(None, server.host)
+        self.assertEqual(None, server.port)
+
+    def test_setUp_and_tearDown(self):
+        server = RecordingServer(expect_body_tail=None)
+        server.setUp()
+        try:
+            self.assertNotEqual(None, server.host)
+            self.assertNotEqual(None, server.port)
+        finally:
+            server.tearDown()
+        self.assertEqual(None, server.host)
+        self.assertEqual(None, server.port)
+
+    def test_send_receive_bytes(self):
+        server = RecordingServer(expect_body_tail='c')
+        server.setUp()
+        self.addCleanup(server.tearDown)
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.connect((server.host, server.port))
+        sock.sendall('abc')
+        self.assertEqual('HTTP/1.1 200 OK\r\n',
+                         sock.recv(4096, socket.MSG_WAITALL))
+        self.assertEqual('abc', server.received_bytes)

=== modified file 'bzrlib/tests/test_http_response.py'
--- bzrlib/tests/test_http_response.py	2006-09-01 17:05:01 +0000
+++ bzrlib/tests/test_http_response.py	2006-10-11 08:51:52 +0000
@@ -351,6 +351,20 @@
 mbp@sourcefrog.net-20050309040929-eee0eb3e6d1e762""")
 
 
+_single_range_no_content_type = (206, """HTTP/1.1 206 Partial Content\r
+Date: Tue, 11 Jul 2006 04:45:22 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Thu, 06 Jul 2006 20:22:05 GMT\r
+ETag: "238a3c-16ec2-805c5540"\r
+Accept-Ranges: bytes\r
+Content-Length: 100\r
+Content-Range: bytes 100-199/93890\r
+Connection: close\r
+\r
+""", """mbp@sourcefrog.net-20050309040815-13242001617e4a06
+mbp@sourcefrog.net-20050309040929-eee0eb3e6d1e762""")
+
+
 _multipart_range_response = (206, """HTTP/1.1 206 Partial Content\r
 Date: Tue, 11 Jul 2006 04:49:48 GMT\r
 Server: Apache/2.0.54 (Fedora)\r
@@ -508,6 +522,12 @@
         self.check_header('Content-Range', 'bytes 100-199/93890')
         self.check_header('Content-Type', 'text/plain; charset=UTF-8')
 
+    def test_single_range_no_content(self):
+        self.use_response(_single_range_no_content_type)
+
+        self.check_header('Content-Length', '100')
+        self.check_header('Content-Range', 'bytes 100-199/93890')
+
     def test_multi_range(self):
         self.use_response(_multipart_range_response)
 
@@ -597,6 +617,15 @@
         out.seek(100)
         self.assertEqual(_single_range_response[2], out.read(100))
 
+    def test_single_range_no_content(self):
+        out = self.get_response(_single_range_no_content_type)
+        self.assertIsInstance(out, response.HttpRangeResponse)
+
+        self.assertRaises(errors.InvalidRange, out.read, 20)
+
+        out.seek(100)
+        self.assertEqual(_single_range_no_content_type[2], out.read(100))
+
     def test_multi_range(self):
         out = self.get_response(_multipart_range_response)
         self.assertIsInstance(out, response.HttpMultipartRangeResponse)
@@ -641,14 +670,6 @@
             response.handle_response, 'http://missing', a_response[0], headers,
                                       StringIO(a_response[2]))
 
-    def test_missing_content_type(self):
-        a_response = _single_range_response
-        headers = http._extract_headers(a_response[1], 'http://nocontent')
-        del headers['Content-Type']
-        self.assertRaises(errors.InvalidHttpContentType,
-            response.handle_response, 'http://nocontent', a_response[0],
-                                      headers, StringIO(a_response[2]))
-
     def test_missing_content_range(self):
         a_response = _single_range_response
         headers = http._extract_headers(a_response[1], 'http://nocontent')

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-08-23 10:30:07 +0000
+++ bzrlib/tests/test_inv.py	2006-09-03 20:41:45 +0000
@@ -544,3 +544,19 @@
         os.unlink('b1/a')
         wt.revert([])
         self.assertEqual(len(wt.inventory), 1)
+
+
+class TestIsRoot(TestCase):
+    """Ensure our root-checking code is accurate."""
+
+    def test_is_root(self):
+        inv = Inventory('TREE_ROOT')
+        self.assertTrue(inv.is_root('TREE_ROOT'))
+        self.assertFalse(inv.is_root('booga'))
+        inv.root.file_id = 'booga'
+        self.assertFalse(inv.is_root('TREE_ROOT'))
+        self.assertTrue(inv.is_root('booga'))
+        # works properly even if no root is set
+        inv.root = None
+        self.assertFalse(inv.is_root('TREE_ROOT'))
+        self.assertFalse(inv.is_root('booga'))

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2006-07-24 20:05:07 +0000
+++ bzrlib/tests/test_knit.py	2006-09-26 13:40:43 +0000
@@ -20,7 +20,7 @@
 import difflib
 
 
-from bzrlib.errors import KnitError, RevisionAlreadyPresent
+from bzrlib.errors import KnitError, RevisionAlreadyPresent, NoSuchFile
 from bzrlib.knit import (
     KnitVersionedFile,
     KnitPlainFactory,
@@ -36,12 +36,14 @@
 class KnitTests(TestCaseWithTransport):
     """Class containing knit test helper routines."""
 
-    def make_test_knit(self, annotate=False):
+    def make_test_knit(self, annotate=False, delay_create=False):
         if not annotate:
             factory = KnitPlainFactory()
         else:
             factory = None
-        return KnitVersionedFile('test', get_transport('.'), access_mode='w', factory=factory, create=True)
+        return KnitVersionedFile('test', get_transport('.'), access_mode='w',
+                                 factory=factory, create=True,
+                                 delay_create=delay_create)
 
 
 class BasicKnitTests(KnitTests):
@@ -312,6 +314,7 @@
         # this tests that a new knit index file has the expected content
         # and that is writes the data we expect as records are added.
         knit = self.make_test_knit(True)
+        # Now knit files are not created until we first add data to them
         self.assertFileEqual("# bzr knit index 8\n", 'test.kndx')
         knit.add_lines_with_ghosts('revid', ['a_ghost'], ['a\n'])
         self.assertFileEqual(
@@ -342,6 +345,91 @@
         self.assertEqual(['revid', 'revid2', 'revid3'], knit.versions())
         self.assertEqual(['revid2'], knit.get_parents('revid3'))
 
+    def test_delay_create(self):
+        """Test that passing delay_create=True creates files late"""
+        knit = self.make_test_knit(annotate=True, delay_create=True)
+        self.failIfExists('test.knit')
+        self.failIfExists('test.kndx')
+        knit.add_lines_with_ghosts('revid', ['a_ghost'], ['a\n'])
+        self.failUnlessExists('test.knit')
+        self.assertFileEqual(
+            "# bzr knit index 8\n"
+            "\n"
+            "revid fulltext 0 84 .a_ghost :",
+            'test.kndx')
+
+    def test_create_parent_dir(self):
+        """create_parent_dir can create knits in nonexistant dirs"""
+        # Has no effect if we don't set 'delay_create'
+        trans = get_transport('.')
+        self.assertRaises(NoSuchFile, KnitVersionedFile, 'dir/test',
+                          trans, access_mode='w', factory=None,
+                          create=True, create_parent_dir=True)
+        # Nothing should have changed yet
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True)
+        self.failIfExists('dir/test.knit')
+        self.failIfExists('dir/test.kndx')
+        self.failIfExists('dir')
+        knit.add_lines('revid', [], ['a\n'])
+        self.failUnlessExists('dir')
+        self.failUnlessExists('dir/test.knit')
+        self.assertFileEqual(
+            "# bzr knit index 8\n"
+            "\n"
+            "revid fulltext 0 84  :",
+            'dir/test.kndx')
+
+    def test_create_mode_700(self):
+        trans = get_transport('.')
+        if not trans._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True,
+                                 file_mode=0600,
+                                 dir_mode=0700)
+        knit.add_lines('revid', [], ['a\n'])
+        self.assertTransportMode(trans, 'dir', 0700)
+        self.assertTransportMode(trans, 'dir/test.knit', 0600)
+        self.assertTransportMode(trans, 'dir/test.kndx', 0600)
+
+    def test_create_mode_770(self):
+        trans = get_transport('.')
+        if not trans._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True,
+                                 file_mode=0660,
+                                 dir_mode=0770)
+        knit.add_lines('revid', [], ['a\n'])
+        self.assertTransportMode(trans, 'dir', 0770)
+        self.assertTransportMode(trans, 'dir/test.knit', 0660)
+        self.assertTransportMode(trans, 'dir/test.kndx', 0660)
+
+    def test_create_mode_777(self):
+        trans = get_transport('.')
+        if not trans._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True,
+                                 file_mode=0666,
+                                 dir_mode=0777)
+        knit.add_lines('revid', [], ['a\n'])
+        self.assertTransportMode(trans, 'dir', 0777)
+        self.assertTransportMode(trans, 'dir/test.knit', 0666)
+        self.assertTransportMode(trans, 'dir/test.kndx', 0666)
+
     def test_plan_merge(self):
         my_knit = self.make_test_knit(annotate=True)
         my_knit.add_lines('text1', [], split_lines(TEXT_1))

=== added file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_lazy_import.py	2006-09-13 02:16:02 +0000
@@ -0,0 +1,1043 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test the lazy_import functionality."""
+
+import os
+import sys
+
+from bzrlib import (
+    errors,
+    lazy_import,
+    osutils,
+    )
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+
+class InstrumentedReplacer(lazy_import.ScopeReplacer):
+    """Track what actions are done"""
+
+    @staticmethod
+    def use_actions(actions):
+        InstrumentedReplacer.actions = actions
+
+    def _replace(self):
+        InstrumentedReplacer.actions.append('_replace')
+        return lazy_import.ScopeReplacer._replace(self)
+
+    def __getattribute__(self, attr):
+        InstrumentedReplacer.actions.append(('__getattribute__', attr))
+        return lazy_import.ScopeReplacer.__getattribute__(self, attr)
+
+    def __call__(self, *args, **kwargs):
+        InstrumentedReplacer.actions.append(('__call__', args, kwargs))
+        return lazy_import.ScopeReplacer.__call__(self, *args, **kwargs)
+
+
+class InstrumentedImportReplacer(lazy_import.ImportReplacer):
+
+    @staticmethod
+    def use_actions(actions):
+        InstrumentedImportReplacer.actions = actions
+
+    def _import(self, scope, name):
+        InstrumentedImportReplacer.actions.append(('_import', name))
+        return lazy_import.ImportReplacer._import(self, scope, name)
+
+    def _replace(self):
+        InstrumentedImportReplacer.actions.append('_replace')
+        return lazy_import.ScopeReplacer._replace(self)
+
+    def __getattribute__(self, attr):
+        InstrumentedImportReplacer.actions.append(('__getattribute__', attr))
+        return lazy_import.ScopeReplacer.__getattribute__(self, attr)
+
+    def __call__(self, *args, **kwargs):
+        InstrumentedImportReplacer.actions.append(('__call__', args, kwargs))
+        return lazy_import.ScopeReplacer.__call__(self, *args, **kwargs)
+
+
+class TestClass(object):
+    """Just a simple test class instrumented for the test cases"""
+
+    class_member = 'class_member'
+
+    @staticmethod
+    def use_actions(actions):
+        TestClass.actions = actions
+
+    def __init__(self):
+        TestClass.actions.append('init')
+
+    def foo(self, x):
+        TestClass.actions.append(('foo', x))
+        return 'foo'
+
+
+class TestScopeReplacer(TestCase):
+    """Test the ability of the replacer to put itself into the correct scope.
+
+    In these tests we use the global scope, because we cannot replace
+    variables in the local scope. This means that we need to be careful
+    and not have the replacing objects use the same name, or we would
+    get collisions.
+    """
+
+    def test_object(self):
+        """ScopeReplacer can create an instance in local scope.
+        
+        An object should appear in globals() by constructing a ScopeReplacer,
+        and it will be replaced with the real object upon the first request.
+        """
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass()
+
+        try:
+            test_obj1
+        except NameError:
+            # test_obj1 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj1 was not supposed to exist yet')
+
+        orig_globals = set(globals().keys())
+
+        InstrumentedReplacer(scope=globals(), name='test_obj1',
+                             factory=factory)
+
+        new_globals = set(globals().keys())
+
+        # We can't use isinstance() because that uses test_obj1.__class__
+        # and that goes through __getattribute__ which would activate
+        # the replacement
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj1, '__class__'))
+        self.assertEqual('foo', test_obj1.foo(1))
+        self.assertIsInstance(test_obj1, TestClass)
+        self.assertEqual('foo', test_obj1.foo(2))
+        self.assertEqual([('__getattribute__', 'foo'),
+                          '_replace',
+                          'factory',
+                          'init',
+                          ('foo', 1),
+                          ('foo', 2),
+                         ], actions)
+
+    def test_replace_side_effects(self):
+        """Creating a new object should only create one entry in globals.
+
+        And only that entry even after replacement.
+        """
+        try:
+            test_scope1
+        except NameError:
+            # test_scope1 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_scope1 was not supposed to exist yet')
+
+        # ignore the logged actions
+        TestClass.use_actions([])
+
+        def factory(replacer, scope, name):
+            return TestClass()
+
+        orig_globals = set(globals().keys())
+
+        lazy_import.ScopeReplacer(scope=globals(), name='test_scope1',
+                                  factory=factory)
+
+        new_globals = set(globals().keys())
+
+        self.assertEqual(lazy_import.ScopeReplacer,
+                         object.__getattribute__(test_scope1, '__class__'))
+        self.assertEqual('foo', test_scope1.foo(1))
+        self.assertIsInstance(test_scope1, TestClass)
+
+        final_globals = set(globals().keys())
+
+        self.assertEqual(set(['test_scope1']), new_globals - orig_globals)
+        self.assertEqual(set(), orig_globals - new_globals)
+        self.assertEqual(set(), final_globals - new_globals)
+        self.assertEqual(set(), new_globals - final_globals)
+
+    def test_class(self):
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass
+
+        try:
+            test_class1
+        except NameError:
+            # test_class2 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_class1 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_class1',
+                             factory=factory)
+
+        self.assertEqual('class_member', test_class1.class_member)
+        self.assertEqual(test_class1, TestClass)
+        self.assertEqual([('__getattribute__', 'class_member'),
+                          '_replace',
+                          'factory',
+                         ], actions)
+
+    def test_call_class(self):
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass
+
+        try:
+            test_class2
+        except NameError:
+            # test_class2 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_class2 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_class2',
+                             factory=factory)
+
+        self.failIf(test_class2 is TestClass)
+        obj = test_class2()
+        self.assertIs(test_class2, TestClass)
+        self.assertIsInstance(obj, TestClass)
+        self.assertEqual('class_member', obj.class_member)
+        self.assertEqual([('__call__', (), {}),
+                          '_replace',
+                          'factory',
+                          'init',
+                         ], actions)
+
+    def test_call_func(self):
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+
+        def func(a, b, c=None):
+            actions.append('func')
+            return (a, b, c)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return func
+
+        try:
+            test_func1
+        except NameError:
+            # test_func1 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_func1 was not supposed to exist yet')
+        InstrumentedReplacer(scope=globals(), name='test_func1',
+                             factory=factory)
+
+        self.failIf(test_func1 is func)
+        val = test_func1(1, 2, c='3')
+        self.assertIs(test_func1, func)
+
+        self.assertEqual((1,2,'3'), val)
+        self.assertEqual([('__call__', (1,2), {'c':'3'}),
+                          '_replace',
+                          'factory',
+                          'func',
+                         ], actions)
+
+    def test_other_variable(self):
+        """Test when a ScopeReplacer is assigned to another variable.
+
+        This test could be updated if we find a way to trap '=' rather
+        than just giving a belated exception.
+        ScopeReplacer only knows about the variable it was created as,
+        so until the object is replaced, it is illegal to pass it to
+        another variable. (Though discovering this may take a while)
+        """
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass()
+
+        try:
+            test_obj2
+        except NameError:
+            # test_obj2 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj2 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_obj2',
+                             factory=factory)
+
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj2, '__class__'))
+        # This is technically not allowed, but we don't have a way to
+        # test it until later.
+        test_obj3 = test_obj2
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj2, '__class__'))
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj3, '__class__'))
+        
+        # The first use of the alternate variable causes test_obj2 to
+        # be replaced.
+        self.assertEqual('foo', test_obj3.foo(1))
+        # test_obj2 has been replaced, but the ScopeReplacer has no
+        # idea of test_obj3
+        self.assertEqual(TestClass,
+                         object.__getattribute__(test_obj2, '__class__'))
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj3, '__class__'))
+        # We should be able to access test_obj2 attributes normally
+        self.assertEqual('foo', test_obj2.foo(2))
+        self.assertEqual('foo', test_obj2.foo(3))
+
+        # However, the next access on test_obj3 should raise an error
+        # because only now are we able to detect the problem.
+        self.assertRaises(errors.IllegalUseOfScopeReplacer,
+                          getattr, test_obj3, 'foo')
+        
+        # However, the 
+        self.assertEqual([('__getattribute__', 'foo'),
+                          '_replace',
+                          'factory',
+                          'init',
+                          ('foo', 1),
+                          ('foo', 2),
+                          ('foo', 3),
+                          ('__getattribute__', 'foo'),
+                          '_replace',
+                         ], actions)
+
+
+class ImportReplacerHelper(TestCaseInTempDir):
+    """Test the ability to have a lazily imported module or object"""
+
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        self.create_modules()
+        base_path = self.test_dir + '/base'
+
+        self.actions = []
+        InstrumentedImportReplacer.use_actions(self.actions)
+
+        original_import = __import__
+        def instrumented_import(mod, scope1, scope2, fromlist):
+            self.actions.append(('import', mod, fromlist))
+            return original_import(mod, scope1, scope2, fromlist)
+
+        def cleanup():
+            if base_path in sys.path:
+                sys.path.remove(base_path)
+            __builtins__['__import__'] = original_import
+        self.addCleanup(cleanup)
+        sys.path.append(base_path)
+        __builtins__['__import__'] = instrumented_import
+
+    def create_modules(self):
+        """Create some random modules to be imported.
+
+        Each entry has a random suffix, and the full names are saved
+
+        These are setup as follows:
+         base/ <= used to ensure not in default search path
+            root-XXX/
+                __init__.py <= This will contain var1, func1
+                mod-XXX.py <= This will contain var2, func2
+                sub-XXX/
+                    __init__.py <= Contains var3, func3
+                    submoda-XXX.py <= contains var4, func4
+                    submodb-XXX.py <= containse var5, func5
+        """
+        rand_suffix = osutils.rand_chars(4)
+        root_name = 'root_' + rand_suffix
+        mod_name = 'mod_' + rand_suffix
+        sub_name = 'sub_' + rand_suffix
+        submoda_name = 'submoda_' + rand_suffix
+        submodb_name = 'submodb_' + rand_suffix
+
+        os.mkdir('base')
+        root_path = osutils.pathjoin('base', root_name)
+        os.mkdir(root_path)
+        root_init = osutils.pathjoin(root_path, '__init__.py')
+        f = open(osutils.pathjoin(root_path, '__init__.py'), 'wb')
+        try:
+            f.write('var1 = 1\ndef func1(a):\n  return a\n')
+        finally:
+            f.close()
+        mod_path = osutils.pathjoin(root_path, mod_name + '.py')
+        f = open(mod_path, 'wb')
+        try:
+            f.write('var2 = 2\ndef func2(a):\n  return a\n')
+        finally:
+            f.close()
+
+        sub_path = osutils.pathjoin(root_path, sub_name)
+        os.mkdir(sub_path)
+        f = open(osutils.pathjoin(sub_path, '__init__.py'), 'wb')
+        try:
+            f.write('var3 = 3\ndef func3(a):\n  return a\n')
+        finally:
+            f.close()
+        submoda_path = osutils.pathjoin(sub_path, submoda_name + '.py')
+        f = open(submoda_path, 'wb')
+        try:
+            f.write('var4 = 4\ndef func4(a):\n  return a\n')
+        finally:
+            f.close()
+        submodb_path = osutils.pathjoin(sub_path, submodb_name + '.py')
+        f = open(submodb_path, 'wb')
+        try:
+            f.write('var5 = 5\ndef func5(a):\n  return a\n')
+        finally:
+            f.close()
+        self.root_name = root_name
+        self.mod_name = mod_name
+        self.sub_name = sub_name
+        self.submoda_name = submoda_name
+        self.submodb_name = submodb_name
+
+
+class TestImportReplacerHelper(ImportReplacerHelper):
+
+    def test_basic_import(self):
+        """Test that a real import of these modules works"""
+        sub_mod_path = '.'.join([self.root_name, self.sub_name,
+                                  self.submoda_name])
+        root = __import__(sub_mod_path, globals(), locals(), [])
+        self.assertEqual(1, root.var1)
+        self.assertEqual(3, getattr(root, self.sub_name).var3)
+        self.assertEqual(4, getattr(getattr(root, self.sub_name),
+                                    self.submoda_name).var4)
+
+        mod_path = '.'.join([self.root_name, self.mod_name])
+        root = __import__(mod_path, globals(), locals(), [])
+        self.assertEqual(2, getattr(root, self.mod_name).var2)
+
+        self.assertEqual([('import', sub_mod_path, []),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+
+class TestImportReplacer(ImportReplacerHelper):
+
+    def test_import_root(self):
+        """Test 'import root-XXX as root1'"""
+        try:
+            root1
+        except NameError:
+            # root1 shouldn't exist yet
+            pass
+        else:
+            self.fail('root1 was not supposed to exist yet')
+
+        # This should replicate 'import root-xxyyzz as root1'
+        InstrumentedImportReplacer(scope=globals(), name='root1',
+                                   module_path=[self.root_name],
+                                   member=None, children={})
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root1, '__class__'))
+        self.assertEqual(1, root1.var1)
+        self.assertEqual('x', root1.func1('x'))
+
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root1'),
+                          ('import', self.root_name, []),
+                         ], self.actions)
+
+    def test_import_mod(self):
+        """Test 'import root-XXX.mod-XXX as mod2'"""
+        try:
+            mod1
+        except NameError:
+            # mod1 shouldn't exist yet
+            pass
+        else:
+            self.fail('mod1 was not supposed to exist yet')
+
+        mod_path = self.root_name + '.' + self.mod_name
+        InstrumentedImportReplacer(scope=globals(), name='mod1',
+                                   module_path=[self.root_name, self.mod_name],
+                                   member=None, children={})
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(mod1, '__class__'))
+        self.assertEqual(2, mod1.var2)
+        self.assertEqual('y', mod1.func2('y'))
+
+        self.assertEqual([('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod1'),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+    def test_import_mod_from_root(self):
+        """Test 'from root-XXX import mod-XXX as mod2'"""
+        try:
+            mod2
+        except NameError:
+            # mod2 shouldn't exist yet
+            pass
+        else:
+            self.fail('mod2 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(), name='mod2',
+                                   module_path=[self.root_name],
+                                   member=self.mod_name, children={})
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(mod2, '__class__'))
+        self.assertEqual(2, mod2.var2)
+        self.assertEqual('y', mod2.func2('y'))
+
+        self.assertEqual([('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod2'),
+                          ('import', self.root_name, [self.mod_name]),
+                         ], self.actions)
+
+    def test_import_root_and_mod(self):
+        """Test 'import root-XXX.mod-XXX' remapping both to root3.mod3"""
+        try:
+            root3
+        except NameError:
+            # root3 shouldn't exist yet
+            pass
+        else:
+            self.fail('root3 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(),
+            name='root3', module_path=[self.root_name], member=None,
+            children={'mod3':([self.root_name, self.mod_name], None, {})})
+
+        # So 'root3' should be a lazy import
+        # and once it is imported, mod3 should also be lazy until
+        # actually accessed.
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root3, '__class__'))
+        self.assertEqual(1, root3.var1)
+
+        # There is a mod3 member, but it is also lazy
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root3.mod3, '__class__'))
+        self.assertEqual(2, root3.mod3.var2)
+
+        mod_path = self.root_name + '.' + self.mod_name
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root3'),
+                          ('import', self.root_name, []),
+                          ('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod3'),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+    def test_import_root_and_root_mod(self):
+        """Test that 'import root, root.mod' can be done.
+
+        The second import should re-use the first one, and just add
+        children to be imported.
+        """
+        try:
+            root4
+        except NameError:
+            # root4 shouldn't exist yet
+            pass
+        else:
+            self.fail('root4 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(),
+            name='root4', module_path=[self.root_name], member=None,
+            children={})
+
+        # So 'root4' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root4, '__class__'))
+
+        # Lets add a new child to be imported on demand
+        # This syntax of using object.__getattribute__ is the correct method
+        # for accessing the _import_replacer_children member
+        children = object.__getattribute__(root4, '_import_replacer_children')
+        children['mod4'] = ([self.root_name, self.mod_name], None, {})
+
+        # Accessing root4.mod4 should import root, but mod should stay lazy
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root4.mod4, '__class__'))
+        self.assertEqual(2, root4.mod4.var2)
+
+        mod_path = self.root_name + '.' + self.mod_name
+        self.assertEqual([('__getattribute__', 'mod4'),
+                          '_replace',
+                          ('_import', 'root4'),
+                          ('import', self.root_name, []),
+                          ('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod4'),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+    def test_import_root_sub_submod(self):
+        """Test import root.mod, root.sub.submoda, root.sub.submodb
+        root should be a lazy import, with multiple children, who also
+        have children to be imported.
+        And when root is imported, the children should be lazy, and
+        reuse the intermediate lazy object.
+        """
+        try:
+            root5
+        except NameError:
+            # root5 shouldn't exist yet
+            pass
+        else:
+            self.fail('root5 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(),
+            name='root5', module_path=[self.root_name], member=None,
+            children={'mod5':([self.root_name, self.mod_name], None, {}),
+                      'sub5':([self.root_name, self.sub_name], None,
+                            {'submoda5':([self.root_name, self.sub_name,
+                                         self.submoda_name], None, {}),
+                             'submodb5':([self.root_name, self.sub_name,
+                                          self.submodb_name], None, {})
+                            }),
+                     })
+
+        # So 'root5' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root5, '__class__'))
+
+        # Accessing root5.mod5 should import root, but mod should stay lazy
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root5.mod5, '__class__'))
+        # root5.sub5 should still be lazy, but not re-import root5
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root5.sub5, '__class__'))
+
+        # Accessing root5.sub5.submoda5 should import sub5, but not either
+        # of the sub objects (they should be available as lazy objects
+        self.assertEqual(InstrumentedImportReplacer,
+                     object.__getattribute__(root5.sub5.submoda5, '__class__'))
+        self.assertEqual(InstrumentedImportReplacer,
+                     object.__getattribute__(root5.sub5.submodb5, '__class__'))
+
+        # This should import mod5
+        self.assertEqual(2, root5.mod5.var2)
+        # These should import submoda5 and submodb5
+        self.assertEqual(4, root5.sub5.submoda5.var4)
+        self.assertEqual(5, root5.sub5.submodb5.var5)
+
+        mod_path = self.root_name + '.' + self.mod_name
+        sub_path = self.root_name + '.' + self.sub_name
+        submoda_path = sub_path + '.' + self.submoda_name
+        submodb_path = sub_path + '.' + self.submodb_name
+
+        self.assertEqual([('__getattribute__', 'mod5'),
+                          '_replace',
+                          ('_import', 'root5'),
+                          ('import', self.root_name, []),
+                          ('__getattribute__', 'submoda5'),
+                          '_replace',
+                          ('_import', 'sub5'),
+                          ('import', sub_path, []),
+                          ('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod5'),
+                          ('import', mod_path, []),
+                          ('__getattribute__', 'var4'),
+                          '_replace',
+                          ('_import', 'submoda5'),
+                          ('import', submoda_path, []),
+                          ('__getattribute__', 'var5'),
+                          '_replace',
+                          ('_import', 'submodb5'),
+                          ('import', submodb_path, []),
+                         ], self.actions)
+
+
+class TestConvertImportToMap(TestCase):
+    """Directly test the conversion from import strings to maps"""
+
+    def check(self, expected, import_strings):
+        proc = lazy_import.ImportProcessor()
+        for import_str in import_strings:
+            proc._convert_import_str(import_str)
+        self.assertEqual(expected, proc.imports,
+                         'Import of %r was not converted correctly'
+                         ' %s != %s' % (import_strings, expected,
+                                        proc.imports))
+
+    def test_import_one(self):
+        self.check({'one':(['one'], None, {}),
+                   }, ['import one'])
+
+    def test_import_one_two(self):
+        one_two_map = {'one':(['one'], None,
+                              {'two':(['one', 'two'], None, {}),
+                              }),
+                      }
+        self.check(one_two_map, ['import one.two'])
+        self.check(one_two_map, ['import one, one.two'])
+        self.check(one_two_map, ['import one', 'import one.two'])
+        self.check(one_two_map, ['import one.two', 'import one'])
+
+    def test_import_one_two_three(self):
+        one_two_three_map = {
+            'one':(['one'], None,
+                   {'two':(['one', 'two'], None,
+                           {'three':(['one', 'two', 'three'], None, {}),
+                           }),
+                   }),
+        }
+        self.check(one_two_three_map, ['import one.two.three'])
+        self.check(one_two_three_map, ['import one, one.two.three'])
+        self.check(one_two_three_map, ['import one',
+                                              'import one.two.three'])
+        self.check(one_two_three_map, ['import one.two.three',
+                                              'import one'])
+
+    def test_import_one_as_x(self):
+        self.check({'x':(['one'], None, {}),
+                          }, ['import one as x'])
+
+    def test_import_one_two_as_x(self):
+        self.check({'x':(['one', 'two'], None, {}),
+                   }, ['import one.two as x'])
+
+    def test_import_mixed(self):
+        mixed = {'x':(['one', 'two'], None, {}),
+                 'one':(['one'], None,
+                       {'two':(['one', 'two'], None, {}),
+                       }),
+                }
+        self.check(mixed, ['import one.two as x, one.two'])
+        self.check(mixed, ['import one.two as x', 'import one.two'])
+        self.check(mixed, ['import one.two', 'import one.two as x'])
+
+    def test_import_with_as(self):
+        self.check({'fast':(['fast'], None, {})}, ['import fast'])
+
+
+class TestFromToMap(TestCase):
+    """Directly test the conversion of 'from foo import bar' syntax"""
+
+    def check_result(self, expected, from_strings):
+        proc = lazy_import.ImportProcessor()
+        for from_str in from_strings:
+            proc._convert_from_str(from_str)
+        self.assertEqual(expected, proc.imports,
+                         'Import of %r was not converted correctly'
+                         ' %s != %s' % (from_strings, expected, proc.imports))
+
+    def test_from_one_import_two(self):
+        self.check_result({'two':(['one'], 'two', {})},
+                          ['from one import two'])
+
+    def test_from_one_import_two_as_three(self):
+        self.check_result({'three':(['one'], 'two', {})},
+                          ['from one import two as three'])
+
+    def test_from_one_import_two_three(self):
+        two_three_map = {'two':(['one'], 'two', {}),
+                         'three':(['one'], 'three', {}),
+                        }
+        self.check_result(two_three_map,
+                          ['from one import two, three'])
+        self.check_result(two_three_map,
+                          ['from one import two',
+                           'from one import three'])
+
+    def test_from_one_two_import_three(self):
+        self.check_result({'three':(['one', 'two'], 'three', {})},
+                          ['from one.two import three'])
+
+
+class TestCanonicalize(TestCase):
+    """Test that we can canonicalize import texts"""
+
+    def check(self, expected, text):
+        proc = lazy_import.ImportProcessor()
+        parsed = proc._canonicalize_import_text(text)
+        self.assertEqual(expected, parsed,
+                         'Incorrect parsing of text:\n%s\n%s\n!=\n%s'
+                         % (text, expected, parsed))
+
+    def test_import_one(self):
+        self.check(['import one'], 'import one')
+        self.check(['import one'], '\nimport one\n\n')
+
+    def test_import_one_two(self):
+        self.check(['import one, two'], 'import one, two')
+        self.check(['import one, two'], '\nimport one, two\n\n')
+
+    def test_import_one_as_two_as(self):
+        self.check(['import one as x, two as y'], 'import one as x, two as y')
+        self.check(['import one as x, two as y'],
+                   '\nimport one as x, two as y\n')
+
+    def test_from_one_import_two(self):
+        self.check(['from one import two'], 'from one import two')
+        self.check(['from one import two'], '\nfrom one import two\n\n')
+        self.check(['from one import two'], '\nfrom one import (two)\n')
+        self.check(['from one import  two '], '\nfrom one import (\n\ttwo\n)\n')
+
+    def test_multiple(self):
+        self.check(['import one', 'import two, three', 'from one import four'],
+                   'import one\nimport two, three\nfrom one import four')
+        self.check(['import one', 'import two, three', 'from one import four'],
+                   'import one\nimport (two, three)\nfrom one import four')
+        self.check(['import one', 'import two, three', 'from one import four'],
+                   'import one\n'
+                   'import two, three\n'
+                   'from one import four')
+        self.check(['import one',
+                    'import two, three', 'from one import  four, '],
+                   'import one\n'
+                   'import two, three\n'
+                   'from one import (\n'
+                   '    four,\n'
+                   '    )\n'
+                   )
+
+    def test_missing_trailing(self):
+        proc = lazy_import.ImportProcessor()
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._canonicalize_import_text,
+                          "from foo import (\n  bar\n")
+
+
+class TestImportProcessor(TestCase):
+    """Test that ImportProcessor can turn import texts into lazy imports"""
+
+    def check(self, expected, text):
+        proc = lazy_import.ImportProcessor()
+        proc._build_map(text)
+        self.assertEqual(expected, proc.imports,
+                         'Incorrect processing of:\n%s\n%s\n!=\n%s'
+                         % (text, expected, proc.imports))
+
+    def test_import_one(self):
+        exp = {'one':(['one'], None, {})}
+        self.check(exp, 'import one')
+        self.check(exp, '\nimport one\n')
+
+    def test_import_one_two(self):
+        exp = {'one':(['one'], None,
+                      {'two':(['one', 'two'], None, {}),
+                      }),
+              }
+        self.check(exp, 'import one.two')
+        self.check(exp, 'import one, one.two')
+        self.check(exp, 'import one\nimport one.two')
+
+    def test_import_as(self):
+        exp = {'two':(['one'], None, {})}
+        self.check(exp, 'import one as two')
+
+    def test_import_many(self):
+        exp = {'one':(['one'], None,
+                      {'two':(['one', 'two'], None,
+                              {'three':(['one', 'two', 'three'], None, {}),
+                              }),
+                       'four':(['one', 'four'], None, {}),
+                      }),
+               'five':(['one', 'five'], None, {}),
+              }
+        self.check(exp, 'import one.two.three, one.four, one.five as five')
+        self.check(exp, 'import one.five as five\n'
+                        'import one\n'
+                        'import one.two.three\n'
+                        'import one.four\n')
+
+    def test_from_one_import_two(self):
+        exp = {'two':(['one'], 'two', {})}
+        self.check(exp, 'from one import two\n')
+        self.check(exp, 'from one import (\n'
+                        '    two,\n'
+                        '    )\n')
+
+    def test_from_one_import_two(self):
+        exp = {'two':(['one'], 'two', {})}
+        self.check(exp, 'from one import two\n')
+        self.check(exp, 'from one import (two)\n')
+        self.check(exp, 'from one import (two,)\n')
+        self.check(exp, 'from one import two as two\n')
+        self.check(exp, 'from one import (\n'
+                        '    two,\n'
+                        '    )\n')
+
+    def test_from_many(self):
+        exp = {'two':(['one'], 'two', {}),
+               'three':(['one', 'two'], 'three', {}),
+               'five':(['one', 'two'], 'four', {}),
+              }
+        self.check(exp, 'from one import two\n'
+                        'from one.two import three, four as five\n')
+        self.check(exp, 'from one import two\n'
+                        'from one.two import (\n'
+                        '    three,\n'
+                        '    four as five,\n'
+                        '    )\n')
+
+    def test_mixed(self):
+        exp = {'two':(['one'], 'two', {}),
+               'three':(['one', 'two'], 'three', {}),
+               'five':(['one', 'two'], 'four', {}),
+               'one':(['one'], None,
+                      {'two':(['one', 'two'], None, {}),
+                      }),
+              }
+        self.check(exp, 'from one import two\n'
+                        'from one.two import three, four as five\n'
+                        'import one.two')
+        self.check(exp, 'from one import two\n'
+                        'from one.two import (\n'
+                        '    three,\n'
+                        '    four as five,\n'
+                        '    )\n'
+                        'import one\n'
+                        'import one.two\n')
+
+    def test_incorrect_line(self):
+        proc = lazy_import.ImportProcessor()
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'foo bar baz')
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'improt foo')
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'importfoo')
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'fromimport')
+
+    def test_name_collision(self):
+        proc = lazy_import.ImportProcessor()
+        proc._build_map('import foo')
+
+        # All of these would try to create an object with the
+        # same name as an existing object.
+        self.assertRaises(errors.ImportNameCollision,
+                          proc._build_map, 'import bar as foo')
+        self.assertRaises(errors.ImportNameCollision,
+                          proc._build_map, 'from foo import bar as foo')
+        self.assertRaises(errors.ImportNameCollision,
+                          proc._build_map, 'from bar import foo')
+
+
+class TestLazyImportProcessor(ImportReplacerHelper):
+
+    def test_root(self):
+        try:
+            root6
+        except NameError:
+            pass # root6 should not be defined yet
+        else:
+            self.fail('root6 was not supposed to exist yet')
+
+        text = 'import %s as root6' % (self.root_name,)
+        proc = lazy_import.ImportProcessor(InstrumentedImportReplacer)
+        proc.lazy_import(scope=globals(), text=text)
+
+        # So 'root6' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root6, '__class__'))
+
+        self.assertEqual(1, root6.var1)
+        self.assertEqual('x', root6.func1('x'))
+
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root6'),
+                          ('import', self.root_name, []),
+                         ], self.actions)
+
+    def test_import_deep(self):
+        """Test import root.mod, root.sub.submoda, root.sub.submodb
+        root should be a lazy import, with multiple children, who also
+        have children to be imported.
+        And when root is imported, the children should be lazy, and
+        reuse the intermediate lazy object.
+        """
+        try:
+            submoda7
+        except NameError:
+            pass # submoda7 should not be defined yet
+        else:
+            self.fail('submoda7 was not supposed to exist yet')
+
+        text = """\
+import %(root_name)s.%(sub_name)s.%(submoda_name)s as submoda7
+""" % self.__dict__
+        proc = lazy_import.ImportProcessor(InstrumentedImportReplacer)
+        proc.lazy_import(scope=globals(), text=text)
+
+        # So 'submoda7' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(submoda7, '__class__'))
+
+        # This should import submoda7
+        self.assertEqual(4, submoda7.var4)
+
+        sub_path = self.root_name + '.' + self.sub_name
+        submoda_path = sub_path + '.' + self.submoda_name
+
+        self.assertEqual([('__getattribute__', 'var4'),
+                          '_replace',
+                          ('_import', 'submoda7'),
+                          ('import', submoda_path, []),
+                         ], self.actions)
+
+    def test_lazy_import(self):
+        """Smoke test that lazy_import() does the right thing"""
+        try:
+            root8
+        except NameError:
+            pass # root8 should not be defined yet
+        else:
+            self.fail('root8 was not supposed to exist yet')
+        lazy_import.lazy_import(globals(),
+                                'import %s as root8' % (self.root_name,),
+                                lazy_import_class=InstrumentedImportReplacer)
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root8, '__class__'))
+
+        self.assertEqual(1, root8.var1)
+        self.assertEqual(1, root8.var1)
+        self.assertEqual(1, root8.func1(1))
+
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root8'),
+                          ('import', self.root_name, []),
+                         ], self.actions)

=== added file 'bzrlib/tests/test_lazy_regex.py'
--- bzrlib/tests/test_lazy_regex.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_lazy_regex.py	2006-10-09 09:41:01 +0000
@@ -0,0 +1,129 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test that lazy regexes are not compiled right away"""
+
+import re
+
+from bzrlib import (
+    lazy_regex,
+    tests,
+    )
+
+
+class InstrumentedLazyRegex(lazy_regex.LazyRegex):
+    """Keep track of actions on the lazy regex"""
+
+    _actions = []
+
+    @classmethod
+    def use_actions(cls, actions):
+        cls._actions = actions
+
+    def __getattr__(self, attr):
+        self._actions.append(('__getattr__', attr))
+        return super(InstrumentedLazyRegex, self).__getattr__(attr)
+
+    def _real_re_compile(self, *args, **kwargs):
+        self._actions.append(('_real_re_compile',
+                                               args, kwargs))
+        return super(InstrumentedLazyRegex, self)._real_re_compile(*args, **kwargs)
+
+
+class TestLazyRegex(tests.TestCase):
+
+    def test_lazy_compile(self):
+        """Make sure that LazyRegex objects compile at the right time"""
+        actions = []
+        InstrumentedLazyRegex.use_actions(actions)
+
+        pattern = InstrumentedLazyRegex(args=('foo',))
+        actions.append(('created regex', 'foo'))
+        # This match call should compile the regex and go through __getattr__
+        pattern.match('foo')
+        # But a further call should not go through __getattr__ because it has
+        # been bound locally.
+        pattern.match('foo')
+
+        self.assertEqual([('created regex', 'foo'),
+                          ('__getattr__', 'match'),
+                          ('_real_re_compile', ('foo',), {}),
+                         ], actions)
+
+
+class TestLazyCompile(tests.TestCase):
+
+    def test_simple_acts_like_regex(self):
+        """Test that the returned object has basic regex like functionality"""
+        pattern = lazy_regex.lazy_compile('foo')
+        self.assertIsInstance(pattern, lazy_regex.LazyRegex)
+        self.assertTrue(pattern.match('foo'))
+        self.assertIs(None, pattern.match('bar'))
+
+    def test_extra_args(self):
+        """Test that extra arguments are also properly passed"""
+        pattern = lazy_regex.lazy_compile('foo', re.I)
+        self.assertIsInstance(pattern, lazy_regex.LazyRegex)
+        self.assertTrue(pattern.match('foo'))
+        self.assertTrue(pattern.match('Foo'))
+
+    def test_findall(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        self.assertEqual(['f', 'fo', 'foo', 'fooo'],
+                         pattern.findall('f fo foo fooo'))
+
+    def test_finditer(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        matches = [(m.start(), m.end(), m.group())
+                   for m in pattern.finditer('foo bar fop')]
+        self.assertEqual([(0, 3, 'foo'), (8, 10, 'fo')], matches)
+
+    def test_match(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        self.assertIs(None, pattern.match('baz foo'))
+        self.assertEqual('fooo', pattern.match('fooo').group())
+
+    def test_search(self):
+        pattern = lazy_regex.lazy_compile('fo*')
+        self.assertEqual('foo', pattern.search('baz foo').group())
+        self.assertEqual('fooo', pattern.search('fooo').group())
+
+    def test_split(self):
+        pattern = lazy_regex.lazy_compile('[,;]*')
+        self.assertEqual(['x', 'y', 'z'], pattern.split('x,y;z'))
+
+
+class TestInstallLazyCompile(tests.TestCase):
+
+    def setUp(self):
+        super(TestInstallLazyCompile, self).setUp()
+        self.addCleanup(lazy_regex.reset_compile)
+
+    def test_install(self):
+        lazy_regex.install_lazy_compile()
+        pattern = re.compile('foo')
+        self.assertIsInstance(pattern, lazy_regex.LazyRegex)
+
+    def test_reset(self):
+        lazy_regex.install_lazy_compile()
+        lazy_regex.reset_compile()
+        pattern = re.compile('foo')
+        self.failIf(isinstance(pattern, lazy_regex.LazyRegex),
+                    'lazy_regex.reset_compile() did not restore the original'
+                    ' compile() function %s' % (type(pattern),))
+        # but the returned object should still support regex operations
+        m = pattern.match('foo')
+        self.assertEqual('foo', m.group())

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2006-08-24 20:04:13 +0000
+++ bzrlib/tests/test_lockdir.py	2006-09-29 20:12:58 +0000
@@ -17,17 +17,22 @@
 """Tests for LockDir"""
 
 from cStringIO import StringIO
-from threading import Thread
+from threading import Thread, Lock
 import time
 
 import bzrlib
+from bzrlib import (
+    config,
+    osutils,
+    )
 from bzrlib.errors import (
         LockBreakMismatch,
         LockContention, LockError, UnlockableTransport,
         LockNotHeld, LockBroken
         )
-from bzrlib.lockdir import LockDir
+from bzrlib.lockdir import LockDir, _DEFAULT_TIMEOUT_SECONDS
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.trace import note
 
 # These tests sometimes use threads to test the behaviour of lock files with
 # concurrent actors.  This is not a typical (or necessarily supported) use;
@@ -42,6 +47,13 @@
 class TestLockDir(TestCaseWithTransport):
     """Test LockDir operations"""
 
+    def logging_report_function(self, fmt, *args):
+        self._logged_reports.append((fmt, args))
+
+    def setup_log_reporter(self, lock_dir):
+        self._logged_reports = []
+        lock_dir._report_function = self.logging_report_function
+
     def test_00_lock_creation(self):
         """Creation of lock file on a transport"""
         t = self.get_transport()
@@ -156,6 +168,7 @@
         lf1 = LockDir(t, 'test_lock')
         lf1.create()
         lf2 = LockDir(t, 'test_lock')
+        self.setup_log_reporter(lf2)
         lf1.attempt_lock()
         try:
             before = time.time()
@@ -168,6 +181,19 @@
                     "took %f seconds to detect lock contention" % (after - before))
         finally:
             lf1.unlock()
+        lock_base = lf2.transport.abspath(lf2.path)
+        self.assertEqual(1, len(self._logged_reports))
+        self.assertEqual('%s %s\n'
+                         '%s\n%s\n'
+                         'Will continue to try until %s\n',
+                         self._logged_reports[0][0])
+        args = self._logged_reports[0][1]
+        self.assertEqual('Unable to obtain', args[0])
+        self.assertEqual('lock %s' % (lock_base,), args[1])
+        self.assertStartsWith(args[2], 'held by ')
+        self.assertStartsWith(args[3], 'locked ')
+        self.assertEndsWith(args[3], ' ago')
+        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
 
     def test_31_lock_wait_easy(self):
         """Succeed when waiting on a lock with no contention.
@@ -175,6 +201,7 @@
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
         lf1.create()
+        self.setup_log_reporter(lf1)
         try:
             before = time.time()
             lf1.wait_lock(timeout=0.4, poll=0.1)
@@ -182,6 +209,7 @@
             self.assertTrue(after - before <= 1.0)
         finally:
             lf1.unlock()
+        self.assertEqual([], self._logged_reports)
 
     def test_32_lock_wait_succeed(self):
         """Succeed when trying to acquire a lock that gets released
@@ -201,6 +229,7 @@
         unlocker.start()
         try:
             lf2 = LockDir(t, 'test_lock')
+            self.setup_log_reporter(lf2)
             before = time.time()
             # wait and then lock
             lf2.wait_lock(timeout=0.4, poll=0.1)
@@ -209,6 +238,22 @@
         finally:
             unlocker.join()
 
+        # There should be only 1 report, even though it should have to
+        # wait for a while
+        lock_base = lf2.transport.abspath(lf2.path)
+        self.assertEqual(1, len(self._logged_reports))
+        self.assertEqual('%s %s\n'
+                         '%s\n%s\n'
+                         'Will continue to try until %s\n',
+                         self._logged_reports[0][0])
+        args = self._logged_reports[0][1]
+        self.assertEqual('Unable to obtain', args[0])
+        self.assertEqual('lock %s' % (lock_base,), args[1])
+        self.assertStartsWith(args[2], 'held by ')
+        self.assertStartsWith(args[3], 'locked ')
+        self.assertEndsWith(args[3], ' ago')
+        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
+
     def test_33_wait(self):
         """Succeed when waiting on a lock that gets released
 
@@ -236,6 +281,162 @@
         finally:
             unlocker.join()
 
+    def test_34_lock_write_waits(self):
+        """LockDir.lock_write() will wait for the lock.""" 
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.attempt_lock()
+
+        def wait_and_unlock():
+            time.sleep(0.1)
+            lf1.unlock()
+        unlocker = Thread(target=wait_and_unlock)
+        unlocker.start()
+        try:
+            lf2 = LockDir(t, 'test_lock')
+            self.setup_log_reporter(lf2)
+            before = time.time()
+            # wait and then lock
+            lf2.lock_write()
+            after = time.time()
+        finally:
+            unlocker.join()
+
+        # There should be only 1 report, even though it should have to
+        # wait for a while
+        lock_base = lf2.transport.abspath(lf2.path)
+        self.assertEqual(1, len(self._logged_reports))
+        self.assertEqual('%s %s\n'
+                         '%s\n%s\n'
+                         'Will continue to try until %s\n',
+                         self._logged_reports[0][0])
+        args = self._logged_reports[0][1]
+        self.assertEqual('Unable to obtain', args[0])
+        self.assertEqual('lock %s' % (lock_base,), args[1])
+        self.assertStartsWith(args[2], 'held by ')
+        self.assertStartsWith(args[3], 'locked ')
+        self.assertEndsWith(args[3], ' ago')
+        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
+
+    def test_35_wait_lock_changing(self):
+        """LockDir.wait_lock() will report if the lock changes underneath.
+        
+        This is the stages we want to happen:
+
+        0) Synchronization locks are created and locked.
+        1) Lock1 obtains the lockdir, and releases the 'check' lock.
+        2) Lock2 grabs the 'check' lock, and checks the lockdir.
+           It sees the lockdir is already acquired, reports the fact, 
+           and unsets the 'checked' lock.
+        3) Thread1 blocks on acquiring the 'checked' lock, and then tells
+           Lock1 to release and acquire the lockdir. This resets the 'check'
+           lock.
+        4) Lock2 acquires the 'check' lock, and checks again. It notices
+           that the holder of the lock has changed, and so reports a new 
+           lock holder.
+        5) Thread1 blocks on the 'checked' lock, this time, it completely
+           unlocks the lockdir, allowing Lock2 to acquire the lock.
+        """
+
+        wait_to_check_lock = Lock()
+        wait_until_checked_lock = Lock()
+
+        wait_to_check_lock.acquire()
+        wait_until_checked_lock.acquire()
+        note('locked check and checked locks')
+
+        class LockDir1(LockDir):
+            """Use the synchronization points for the first lock."""
+
+            def attempt_lock(self):
+                # Once we have acquired the lock, it is okay for
+                # the other lock to check it
+                try:
+                    return super(LockDir1, self).attempt_lock()
+                finally:
+                    note('lock1: releasing check lock')
+                    wait_to_check_lock.release()
+
+        class LockDir2(LockDir):
+            """Use the synchronization points for the second lock."""
+
+            def attempt_lock(self):
+                note('lock2: waiting for check lock')
+                wait_to_check_lock.acquire()
+                note('lock2: acquired check lock')
+                try:
+                    return super(LockDir2, self).attempt_lock()
+                finally:
+                    note('lock2: releasing checked lock')
+                    wait_until_checked_lock.release()
+
+        t = self.get_transport()
+        lf1 = LockDir1(t, 'test_lock')
+        lf1.create()
+
+        lf2 = LockDir2(t, 'test_lock')
+        self.setup_log_reporter(lf2)
+
+        def wait_and_switch():
+            lf1.attempt_lock()
+            # Block until lock2 has had a chance to check
+            note('lock1: waiting 1 for checked lock')
+            wait_until_checked_lock.acquire()
+            note('lock1: acquired for checked lock')
+            note('lock1: released lockdir')
+            lf1.unlock()
+            note('lock1: acquiring lockdir')
+            # Create a new nonce, so the lock looks different.
+            lf1.nonce = osutils.rand_chars(20)
+            lf1.lock_write()
+            note('lock1: acquired lockdir')
+
+            # Block until lock2 has peeked again
+            note('lock1: waiting 2 for checked lock')
+            wait_until_checked_lock.acquire()
+            note('lock1: acquired for checked lock')
+            # Now unlock, and let lock 2 grab the lock
+            lf1.unlock()
+            wait_to_check_lock.release()
+
+        unlocker = Thread(target=wait_and_switch)
+        unlocker.start()
+        try:
+            # Wait and play against the other thread
+            lf2.wait_lock(timeout=1.0, poll=0.01)
+        finally:
+            unlocker.join()
+        lf2.unlock()
+
+        # There should be 2 reports, because the lock changed
+        lock_base = lf2.transport.abspath(lf2.path)
+        self.assertEqual(2, len(self._logged_reports))
+
+        self.assertEqual('%s %s\n'
+                         '%s\n%s\n'
+                         'Will continue to try until %s\n',
+                         self._logged_reports[0][0])
+        args = self._logged_reports[0][1]
+        self.assertEqual('Unable to obtain', args[0])
+        self.assertEqual('lock %s' % (lock_base,), args[1])
+        self.assertStartsWith(args[2], 'held by ')
+        self.assertStartsWith(args[3], 'locked ')
+        self.assertEndsWith(args[3], ' ago')
+        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
+
+        self.assertEqual('%s %s\n'
+                         '%s\n%s\n'
+                         'Will continue to try until %s\n',
+                         self._logged_reports[1][0])
+        args = self._logged_reports[1][1]
+        self.assertEqual('Lock owner changed for', args[0])
+        self.assertEqual('lock %s' % (lock_base,), args[1])
+        self.assertStartsWith(args[2], 'held by ')
+        self.assertStartsWith(args[3], 'locked ')
+        self.assertEndsWith(args[3], ' ago')
+        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
+
     def test_40_confirm_easy(self):
         """Confirm a lock that's already held"""
         t = self.get_transport()
@@ -376,3 +577,26 @@
 
         lf1.unlock()
         self.failIf(t.has('test_lock/held/info'))
+
+    def test__format_lock_info(self):
+        ld1 = self.get_lock()
+        ld1.create()
+        ld1.lock_write()
+        try:
+            info_list = ld1._format_lock_info(ld1.peek())
+        finally:
+            ld1.unlock()
+        self.assertEqual('lock %s' % (ld1.transport.abspath(ld1.path),),
+                         info_list[0])
+        self.assertContainsRe(info_list[1],
+                              r'^held by .* on host .* \[process #\d*\]$')
+        self.assertContainsRe(info_list[2], r'locked \d+ seconds? ago$')
+
+    def test_lock_without_email(self):
+        global_config = config.GlobalConfig()
+        # Intentionally has no email address
+        global_config.set_user_option('email', 'User Identity')
+        ld1 = self.get_lock()
+        ld1.create()
+        ld1.lock_write()
+        ld1.unlock()

=== added file 'bzrlib/tests/test_memorytree.py'
--- bzrlib/tests/test_memorytree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_memorytree.py	2006-10-15 15:55:32 +0000
@@ -0,0 +1,156 @@
+# Copyright (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the MemoryTree class."""
+
+from bzrlib import errors
+from bzrlib.memorytree import MemoryTree
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.treebuilder import TreeBuilder
+
+
+class TestMemoryTree(TestCaseWithTransport):
+    
+    def test_create_on_branch(self):
+        """Creating a mutable tree on a trivial branch works."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        self.assertEqual(branch.bzrdir, tree.bzrdir)
+        self.assertEqual(branch, tree.branch)
+        self.assertEqual([], tree.get_parent_ids())
+    
+    def test_create_on_branch_with_content(self):
+        """Creating a mutable tree on a non-trivial branch works."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        # build some content
+        tree.lock_write()
+        builder = TreeBuilder()
+        builder.start_tree(tree)
+        builder.build(['foo'])
+        builder.finish_tree()
+        rev_id = tree.commit('first post')
+        tree.unlock()
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_read()
+        self.assertEqual([rev_id], tree.get_parent_ids())
+        self.assertEqual('contents of foo\n',
+            tree.get_file(tree.path2id('foo')).read())
+        tree.unlock()
+
+    def test_lock_tree_write(self):
+        """Check we can lock_tree_write and unlock MemoryTrees."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_tree_write()
+        tree.unlock()
+
+    def test_lock_tree_write_after_read_fails(self):
+        """Check that we error when trying to upgrade a read lock to write."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_read()
+        self.assertRaises(errors.ReadOnlyError, tree.lock_tree_write)
+        tree.unlock()
+
+    def test_lock_write(self):
+        """Check we can lock_write and unlock MemoryTrees."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_write()
+        tree.unlock()
+
+    def test_lock_write_after_read_fails(self):
+        """Check that we error when trying to upgrade a read lock to write."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_read()
+        self.assertRaises(errors.ReadOnlyError, tree.lock_write)
+        tree.unlock()
+
+    def test_add_with_kind(self):
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_write()
+        tree.add(['', 'afile', 'adir'], None, 
+                 ['directory', 'file', 'directory'])
+        self.assertEqual('afile', tree.id2path(tree.path2id('afile')))
+        self.assertEqual('adir', tree.id2path(tree.path2id('adir')))
+        self.assertFalse(tree.has_filename('afile'))
+        self.assertFalse(tree.has_filename('adir'))
+        tree.unlock()
+
+    def test_put_new_file(self):
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_write()
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                  kinds=['directory', 'file'])
+        tree.put_file_bytes_non_atomic('foo-id', 'barshoom')
+        self.assertEqual('barshoom', tree.get_file('foo-id').read())
+        tree.unlock()
+
+    def test_put_existing_file(self):
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_write()
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                 kinds=['directory', 'file'])
+        tree.put_file_bytes_non_atomic('foo-id', 'first-content')
+        tree.put_file_bytes_non_atomic('foo-id', 'barshoom')
+        self.assertEqual('barshoom', tree.get_file('foo-id').read())
+        tree.unlock()
+
+    def test_commit_trivial(self):
+        """Smoke test for commit on a MemoryTree.
+
+        Becamse of commits design and layering, if this works, all commit
+        logic should work quite reliably.
+        """
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_write()
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                 kinds=['directory', 'file'])
+        tree.put_file_bytes_non_atomic('foo-id', 'barshoom')
+        revision_id = tree.commit('message baby')
+        # the parents list for the tree should have changed.
+        self.assertEqual([revision_id], tree.get_parent_ids())
+        tree.unlock()
+        # and we should have a revision that is accessible outside the tree lock
+        revtree = tree.branch.repository.revision_tree(revision_id)
+        self.assertEqual('barshoom', revtree.get_file('foo-id').read())
+
+    def test_unversion(self):
+        """Some test for unversion of a memory tree."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        tree.lock_write()
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                 kinds=['directory', 'file'])
+        tree.unversion(['foo-id'])
+        self.assertFalse(tree.has_id('foo-id'))
+        tree.unlock()
+
+    def test_last_revision(self):
+        """There should be a last revision method we can call."""
+        tree = self.make_branch_and_memory_tree('branch')
+        tree.lock_write()
+        tree.add('')
+        rev_id = tree.commit('first post')
+        tree.unlock()
+        self.assertEqual(rev_id, tree.last_revision())

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2006-09-06 02:47:54 +0000
+++ bzrlib/tests/test_merge.py	2006-09-21 16:03:07 +0000
@@ -81,7 +81,7 @@
         self.assertEqual([tip], wt2.get_parent_ids())
         
     def test_pending_with_null(self):
-        """When base is forced to revno 0, pending_merges is set"""
+        """When base is forced to revno 0, parent_ids are set"""
         wt2 = self.test_unrelated()
         wt1 = WorkingTree.open('.')
         br1 = wt1.branch
@@ -127,7 +127,7 @@
         transform_tree(tree, tree.branch.basis_tree())
 
     def test_ignore_zero_merge_inner(self):
-        # Test that merge_inner's ignore zero paramter is effective
+        # Test that merge_inner's ignore zero parameter is effective
         tree_a =self.make_branch_and_tree('a')
         tree_a.commit(message="hello")
         dir_b = tree_a.bzrdir.sprout('b')
@@ -136,12 +136,12 @@
         log = StringIO()
         merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(), 
                     this_tree=tree_b, ignore_zero=True)
-        log = self._get_log()
+        log = self._get_log(keep_log_file=True)
         self.failUnless('All changes applied successfully.\n' not in log)
         tree_b.revert([])
         merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(), 
                     this_tree=tree_b, ignore_zero=False)
-        log = self._get_log()
+        log = self._get_log(keep_log_file=True)
         self.failUnless('All changes applied successfully.\n' in log)
 
     def test_merge_inner_conflicts(self):

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2006-08-30 14:37:31 +0000
+++ bzrlib/tests/test_merge_core.py	2006-10-12 01:50:46 +0000
@@ -3,6 +3,9 @@
 import sys
 
 import bzrlib
+from bzrlib import (
+    osutils,
+    )
 from bzrlib.add import smart_add_tree
 from bzrlib.builtins import merge
 from bzrlib.conflicts import ContentsConflict, TextConflict, PathConflict
@@ -10,24 +13,28 @@
                            WorkingTreeNotRevision, BzrCommandError, NoDiff3)
 import bzrlib.inventory as inventory
 from bzrlib.merge import Merge3Merger, Diff3Merger, WeaveMerger
-from bzrlib.osutils import (file_kind, getcwd, mkdtemp, pathjoin, rename, rmtree,
-                            sha_file, 
+from bzrlib.osutils import (file_kind, getcwd, pathjoin, rename,
+                            sha_file,
                             )
 from bzrlib.transform import TreeTransform
 from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
-from bzrlib.workingtree import WorkingTree
+from bzrlib.workingtree import WorkingTree, gen_root_id
 
 
 class MergeBuilder(object):
     def __init__(self, dir=None):
-        self.dir = mkdtemp(prefix="merge-test", dir=dir)
+        self.dir = osutils.mkdtemp(prefix="merge-test", dir=dir)
+        self.tree_root = gen_root_id()
         def wt(name):
            path = pathjoin(self.dir, name)
            os.mkdir(path)
            wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(path)
+           # the tests perform pulls, so need a branch that is writeable.
+           wt.lock_write()
+           wt.set_root_id(self.tree_root)
            tt = TreeTransform(wt)
            return wt, tt
-        self.base, self.base_tt = wt('base') 
+        self.base, self.base_tt = wt('base')
         self.this, self.this_tt = wt('this')
         self.other, self.other_tt = wt('other')
 
@@ -162,8 +169,14 @@
             new_inventory[file_id] = path
         return new_inventory
 
+    def unlock(self):
+        self.base.unlock()
+        self.this.unlock()
+        self.other.unlock()
+
     def cleanup(self):
-        rmtree(self.dir)
+        self.unlock()
+        osutils.rmtree(self.dir)
 
 
 class MergeTest(TestCaseWithTransport):
@@ -171,16 +184,16 @@
     def test_change_name(self):
         """Test renames"""
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
+        builder.add_file("1", builder.tree_root, "name1", "hello1", True)
         builder.change_name("1", other="name2")
-        builder.add_file("2", "TREE_ROOT", "name3", "hello2", True)
+        builder.add_file("2", builder.tree_root, "name3", "hello2", True)
         builder.change_name("2", base="name4")
-        builder.add_file("3", "TREE_ROOT", "name5", "hello3", True)
+        builder.add_file("3", builder.tree_root, "name5", "hello3", True)
         builder.change_name("3", this="name6")
         builder.merge()
         builder.cleanup()
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "hello1", False)
+        builder.add_file("1", builder.tree_root, "name1", "hello1", False)
         builder.change_name("1", other="name2", this="name3")
         conflicts = builder.merge()
         self.assertEqual(conflicts, [PathConflict('name3', 'name2', '1')])
@@ -188,19 +201,20 @@
 
     def test_merge_one(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
+        builder.add_file("1", builder.tree_root, "name1", "hello1", True)
         builder.change_contents("1", other="text4")
-        builder.add_file("2", "TREE_ROOT", "name2", "hello1", True)
+        builder.add_file("2", builder.tree_root, "name2", "hello1", True)
         builder.change_contents("2", other="text4")
         builder.merge(interesting_ids=["1"])
         self.assertEqual(builder.this.get_file("1").read(), "text4" )
         self.assertEqual(builder.this.get_file("2").read(), "hello1" )
+        builder.cleanup()
         
     def test_file_moves(self):
         """Test moves"""
         builder = MergeBuilder(getcwd())
-        builder.add_dir("1", "TREE_ROOT", "dir1")
-        builder.add_dir("2", "TREE_ROOT", "dir2")
+        builder.add_dir("1", builder.tree_root, "dir1")
+        builder.add_dir("2", builder.tree_root, "dir2")
         builder.add_file("3", "1", "file1", "hello1", True)
         builder.add_file("4", "1", "file2", "hello2", True)
         builder.add_file("5", "1", "file3", "hello3", True)
@@ -211,9 +225,9 @@
         builder.cleanup()
 
         builder = MergeBuilder(getcwd())
-        builder.add_dir("1", "TREE_ROOT", "dir1")
-        builder.add_dir("2", "TREE_ROOT", "dir2")
-        builder.add_dir("3", "TREE_ROOT", "dir3")
+        builder.add_dir("1", builder.tree_root, "dir1")
+        builder.add_dir("2", builder.tree_root, "dir2")
+        builder.add_dir("3", builder.tree_root, "dir3")
         builder.add_file("4", "1", "file1", "hello1", False)
         builder.change_parent("4", other="2", this="3")
         conflicts = builder.merge()
@@ -240,7 +254,7 @@
     def test_reprocess_weave(self):
         # Reprocess works on weaves, and behaves as expected
         builder = MergeBuilder(getcwd())
-        builder.add_file('a', 'TREE_ROOT', 'blah', 'a', False)
+        builder.add_file('a', builder.tree_root, 'blah', 'a', False)
         builder.change_contents('a', this='b\nc\nd\ne\n', other='z\nc\nd\ny\n')
         builder.merge(WeaveMerger, reprocess=True)
         expected = """<<<<<<< TREE
@@ -257,8 +271,7 @@
 >>>>>>> MERGE-SOURCE
 """
         self.assertEqualDiff(builder.this.get_file("a").read(), expected)
-
- 
+        builder.cleanup()
 
     def do_contents_test(self, merge_factory):
         """Test merging with specified ContentsChange factory"""
@@ -268,33 +281,39 @@
 
     def contents_test_success(self, merge_factory):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_contents("1", other="text4")
-        builder.add_file("2", "TREE_ROOT", "name3", "text2", False)
+        builder.add_file("2", builder.tree_root, "name3", "text2", False)
         builder.change_contents("2", base="text5")
-        builder.add_file("3", "TREE_ROOT", "name5", "text3", True)
-        builder.add_file("4", "TREE_ROOT", "name6", "text4", True)
+        builder.add_file("3", builder.tree_root, "name5", "text3", True)
+        builder.add_file("4", builder.tree_root, "name6", "text4", True)
         builder.remove_file("4", base=True)
-        builder.add_file("5", "TREE_ROOT", "name7", "a\nb\nc\nd\ne\nf\n", True)
+        builder.add_file("5", builder.tree_root, "name7", "a\nb\nc\nd\ne\nf\n",
+                         True)
         builder.change_contents("5", other="a\nz\nc\nd\ne\nf\n", 
                                      this="a\nb\nc\nd\ne\nz\n")
-        builder.merge(merge_factory)
-        self.assertEqual(builder.this.get_file("1").read(), "text4" )
-        self.assertEqual(builder.this.get_file("2").read(), "text2" )
-        self.assertEqual(builder.this.get_file("5").read(), 
-                         "a\nz\nc\nd\ne\nz\n")
-        self.assertIs(builder.this.is_executable("1"), True)
-        self.assertIs(builder.this.is_executable("2"), False)
-        self.assertIs(builder.this.is_executable("3"), True)
+        conflicts = builder.merge(merge_factory)
+        try:
+            self.assertEqual([], conflicts)
+            self.assertEqual("text4", builder.this.get_file("1").read())
+            self.assertEqual("text2", builder.this.get_file("2").read())
+            self.assertEqual("a\nz\nc\nd\ne\nz\n", 
+                             builder.this.get_file("5").read())
+            self.assertTrue(builder.this.is_executable("1"))
+            self.assertFalse(builder.this.is_executable("2"))
+            self.assertTrue(builder.this.is_executable("3"))
+        except:
+            builder.unlock()
+            raise
         return builder
 
     def contents_test_conflicts(self, merge_factory):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_contents("1", other="text4", this="text3")
-        builder.add_file("2", "TREE_ROOT", "name2", "text1", True)
+        builder.add_file("2", builder.tree_root, "name2", "text1", True)
         builder.change_contents("2", other="\x00", this="text3")
-        builder.add_file("3", "TREE_ROOT", "name3", "text5", False)
+        builder.add_file("3", builder.tree_root, "name3", "text5", False)
         builder.change_perms("3", this=True)
         builder.change_contents('3', this='moretext')
         builder.remove_file('3', other=True)
@@ -308,7 +327,7 @@
     def test_symlink_conflicts(self):
         if sys.platform != "win32":
             builder = MergeBuilder(getcwd())
-            builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
+            builder.add_symlink("2", builder.tree_root, "name2", "target1")
             builder.change_target("2", other="target4", base="text3")
             conflicts = builder.merge()
             self.assertEqual(conflicts, [ContentsConflict('name2', 
@@ -318,9 +337,9 @@
     def test_symlink_merge(self):
         if sys.platform != "win32":
             builder = MergeBuilder(getcwd())
-            builder.add_symlink("1", "TREE_ROOT", "name1", "target1")
-            builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
-            builder.add_symlink("3", "TREE_ROOT", "name3", "target1")
+            builder.add_symlink("1", builder.tree_root, "name1", "target1")
+            builder.add_symlink("2", builder.tree_root, "name2", "target1")
+            builder.add_symlink("3", builder.tree_root, "name3", "target1")
             builder.change_target("1", this="target2")
             builder.change_target("2", base="target2")
             builder.change_target("3", other="target2")
@@ -332,20 +351,20 @@
 
     def test_no_passive_add(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.remove_file("1", this=True)
         builder.merge()
         builder.cleanup()
 
     def test_perms_merge(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_perms("1", other=False)
-        builder.add_file("2", "TREE_ROOT", "name2", "text2", True)
+        builder.add_file("2", builder.tree_root, "name2", "text2", True)
         builder.change_perms("2", base=False)
-        builder.add_file("3", "TREE_ROOT", "name3", "text3", True)
+        builder.add_file("3", builder.tree_root, "name3", "text3", True)
         builder.change_perms("3", this=False)
-        builder.add_file('4', 'TREE_ROOT', 'name4', 'text4', False)
+        builder.add_file('4', builder.tree_root, 'name4', 'text4', False)
         builder.change_perms('4', this=True)
         builder.remove_file('4', base=True)
         builder.merge()
@@ -356,21 +375,22 @@
 
     def test_new_suffix(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_contents("1", other="text3")
-        builder.add_file("2", "TREE_ROOT", "name1.new", "text2", True)
+        builder.add_file("2", builder.tree_root, "name1.new", "text2", True)
         builder.merge()
         os.lstat(builder.this.id2abspath("2"))
         builder.cleanup()
 
     def test_spurious_conflict(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", False)
+        builder.add_file("1", builder.tree_root, "name1", "text1", False)
         builder.remove_file("1", other=True)
-        builder.add_file("2", "TREE_ROOT", "name1", "text1", False, this=False, 
-                         base=False)
+        builder.add_file("2", builder.tree_root, "name1", "text1", False, 
+                         this=False, base=False)
         conflicts = builder.merge()
         self.assertEqual(conflicts, []) 
+        builder.cleanup()
 
 
 class FunctionalMergeTest(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2006-06-10 13:58:01 +0000
+++ bzrlib/tests/test_msgeditor.py	2006-09-08 18:46:29 +0000
@@ -56,10 +56,10 @@
         self._bzr_editor = os.environ.get('BZR_EDITOR', None)
 
     def tearDown(self):
-        if self._bzr_editor != None:
+        if self._bzr_editor is not None:
             os.environ['BZR_EDITOR'] = self._bzr_editor
         else:
-            if os.environ.get('BZR_EDITOR', None) != None:
+            if os.environ.get('BZR_EDITOR', None) is not None:
                 del os.environ['BZR_EDITOR']
         super(MsgEditorTest, self).tearDown()
 

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-08-14 22:53:25 +0000
+++ bzrlib/tests/test_osutils.py	2006-10-06 05:53:44 +0000
@@ -23,8 +23,11 @@
 import sys
 
 import bzrlib
+from bzrlib import (
+    errors,
+    osutils,
+    )
 from bzrlib.errors import BzrBadParameterNotUnicode, InvalidURL
-import bzrlib.osutils as osutils
 from bzrlib.tests import (
         StringIOWrapper,
         TestCase, 
@@ -78,6 +81,14 @@
         self.assertEqual(type(result), str)
         self.assertContainsRe(result, r'^[a-z0-9]{100}$')
 
+    def test_is_inside(self):
+        is_inside = osutils.is_inside
+        self.assertTrue(is_inside('src', 'src/foo.c'))
+        self.assertFalse(is_inside('src', 'srccontrol'))
+        self.assertTrue(is_inside('src', 'src/a/a/a/foo.c'))
+        self.assertTrue(is_inside('foo.c', 'foo.c'))
+        self.assertFalse(is_inside('foo.c', ''))
+        self.assertTrue(is_inside('', 'foo.c'))
 
     def test_rmtree(self):
         # Check to remove tree with read-only files/dirs
@@ -150,6 +161,42 @@
         finally:
             os.umask(orig_umask)
 
+    def assertFormatedDelta(self, expected, seconds):
+        """Assert osutils.format_delta formats as expected"""
+        actual = osutils.format_delta(seconds)
+        self.assertEqual(expected, actual)
+
+    def test_format_delta(self):
+        self.assertFormatedDelta('0 seconds ago', 0)
+        self.assertFormatedDelta('1 second ago', 1)
+        self.assertFormatedDelta('10 seconds ago', 10)
+        self.assertFormatedDelta('59 seconds ago', 59)
+        self.assertFormatedDelta('89 seconds ago', 89)
+        self.assertFormatedDelta('1 minute, 30 seconds ago', 90)
+        self.assertFormatedDelta('3 minutes, 0 seconds ago', 180)
+        self.assertFormatedDelta('3 minutes, 1 second ago', 181)
+        self.assertFormatedDelta('10 minutes, 15 seconds ago', 615)
+        self.assertFormatedDelta('30 minutes, 59 seconds ago', 1859)
+        self.assertFormatedDelta('31 minutes, 0 seconds ago', 1860)
+        self.assertFormatedDelta('60 minutes, 0 seconds ago', 3600)
+        self.assertFormatedDelta('89 minutes, 59 seconds ago', 5399)
+        self.assertFormatedDelta('1 hour, 30 minutes ago', 5400)
+        self.assertFormatedDelta('2 hours, 30 minutes ago', 9017)
+        self.assertFormatedDelta('10 hours, 0 minutes ago', 36000)
+        self.assertFormatedDelta('24 hours, 0 minutes ago', 86400)
+        self.assertFormatedDelta('35 hours, 59 minutes ago', 129599)
+        self.assertFormatedDelta('36 hours, 0 minutes ago', 129600)
+        self.assertFormatedDelta('36 hours, 0 minutes ago', 129601)
+        self.assertFormatedDelta('36 hours, 1 minute ago', 129660)
+        self.assertFormatedDelta('36 hours, 1 minute ago', 129661)
+        self.assertFormatedDelta('84 hours, 10 minutes ago', 303002)
+
+        # We handle when time steps the wrong direction because computers
+        # don't have synchronized clocks.
+        self.assertFormatedDelta('84 hours, 10 minutes in the future', -303002)
+        self.assertFormatedDelta('1 second in the future', -1)
+        self.assertFormatedDelta('2 seconds in the future', -2)
+
 
 class TestSafeUnicode(TestCase):
 
@@ -275,6 +322,18 @@
         except (IOError, OSError), e:
             self.assertEqual(errno.ENOENT, e.errno)
 
+    def test_splitpath(self):
+        def check(expected, path):
+            self.assertEqual(expected, osutils.splitpath(path))
+
+        check(['a'], 'a')
+        check(['a', 'b'], 'a/b')
+        check(['a', 'b'], 'a/./b')
+        check(['a', '.b'], 'a/.b')
+        check(['a', '.b'], 'a\\.b')
+
+        self.assertRaises(errors.BzrError, osutils.splitpath, 'a/../b')
+
 
 class TestMacFuncsDirs(TestCaseInTempDir):
     """Test mac special functions that require directories."""
@@ -300,6 +359,7 @@
         os.chdir(u'Ba\u030agfors')
         self.assertEndsWith(osutils._mac_getcwd(), u'B\xe5gfors')
 
+
 class TestSplitLines(TestCase):
 
     def test_split_unicode(self):
@@ -529,3 +589,63 @@
         # and in the worst case, use bzrlib.user_encoding
         self.assertEqual('user_encoding', osutils.get_terminal_encoding())
 
+
+class TestSetUnsetEnv(TestCase):
+    """Test updating the environment"""
+
+    def setUp(self):
+        super(TestSetUnsetEnv, self).setUp()
+
+        self.assertEqual(None, os.environ.get('BZR_TEST_ENV_VAR'),
+                         'Environment was not cleaned up properly.'
+                         ' Variable BZR_TEST_ENV_VAR should not exist.')
+        def cleanup():
+            if 'BZR_TEST_ENV_VAR' in os.environ:
+                del os.environ['BZR_TEST_ENV_VAR']
+
+        self.addCleanup(cleanup)
+
+    def test_set(self):
+        """Test that we can set an env variable"""
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'foo')
+        self.assertEqual(None, old)
+        self.assertEqual('foo', os.environ.get('BZR_TEST_ENV_VAR'))
+
+    def test_double_set(self):
+        """Test that we get the old value out"""
+        osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'foo')
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'bar')
+        self.assertEqual('foo', old)
+        self.assertEqual('bar', os.environ.get('BZR_TEST_ENV_VAR'))
+
+    def test_unicode(self):
+        """Environment can only contain plain strings
+        
+        So Unicode strings must be encoded.
+        """
+        # Try a few different characters, to see if we can get
+        # one that will be valid in the user_encoding
+        possible_vals = [u'm\xb5', u'\xe1', u'\u0410']
+        for uni_val in possible_vals:
+            try:
+                env_val = uni_val.encode(bzrlib.user_encoding)
+            except UnicodeEncodeError:
+                # Try a different character
+                pass
+            else:
+                break
+        else:
+            raise TestSkipped('Cannot find a unicode character that works in'
+                              ' encoding %s' % (bzrlib.user_encoding,))
+
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', uni_val)
+        self.assertEqual(env_val, os.environ.get('BZR_TEST_ENV_VAR'))
+
+    def test_unset(self):
+        """Test that passing None will remove the env var"""
+        osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'foo')
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', None)
+        self.assertEqual('foo', old)
+        self.assertEqual(None, os.environ.get('BZR_TEST_ENV_VAR'))
+        self.failIf('BZR_TEST_ENV_VAR' in os.environ)
+

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2006-09-05 20:58:28 +0000
+++ bzrlib/tests/test_permissions.py	2006-09-09 18:52:57 +0000
@@ -34,6 +34,7 @@
 import sys
 import stat
 from cStringIO import StringIO
+import urllib
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
@@ -72,10 +73,11 @@
         test.assertTransportMode(t, base, dir_mode)
     for root, dirs, files in os.walk(base):
         for d in dirs:
-            p = os.path.join(root, d)
+            p = '/'.join([urllib.quote(x) for x in root.split('/\\') + [d]])
             test.assertTransportMode(t, p, dir_mode)
         for f in files:
             p = os.path.join(root, f)
+            p = '/'.join([urllib.quote(x) for x in root.split('/\\') + [f]])
             test.assertTransportMode(t, p, file_mode)
 
 
@@ -88,7 +90,8 @@
         t = self.make_branch_and_tree('.')
         b = t.branch
         open('a', 'wb').write('foo\n')
-        t.add('a')
+        # ensure check_mode_r works with capital-letter file-ids like TREE_ROOT
+        t.add('a', 'CAPS-ID')
         t.commit('foo')
 
         chmod_r('.bzr', 0644, 0755)

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_plugins.py	2006-09-25 21:54:28 +0000
@@ -148,8 +148,8 @@
         help = {}
         current = None
         for line in self.capture('help commands').splitlines():
-            if line.startswith('bzr '):
-                current = line.split()[1]
+            if not line.startswith(' '):
+                current = line.split()[0]
             help[current] = help.get(current, '') + line
 
         return help
@@ -189,7 +189,7 @@
             help = self.capture('help myplug')
             self.assertContainsRe(help, 'From plugin "myplug"')
             help = self.split_help_commands()['myplug']
-            self.assertContainsRe(help, 'From plugin "myplug"')
+            self.assertContainsRe(help, '\[myplug\]')
         finally:
             # remove the plugin 'plugin'
             if getattr(bzrlib.plugins, 'plugin', None):

=== modified file 'bzrlib/tests/test_read_bundle.py'
--- bzrlib/tests/test_read_bundle.py	2006-09-05 20:42:31 +0000
+++ bzrlib/tests/test_read_bundle.py	2006-09-08 00:19:48 +0000
@@ -57,7 +57,7 @@
 
         out = cStringIO.StringIO()
         rev_ids = write_bundle(wt.branch.repository,
-                               wt.last_revision(), None, out)
+                               wt.get_parent_ids()[0], None, out)
         out.seek(0)
         if self._transport.is_readonly():
             f = open('test_bundle', 'wb')

=== added file 'bzrlib/tests/test_registry.py'
--- bzrlib/tests/test_registry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_registry.py	2006-09-21 22:25:19 +0000
@@ -0,0 +1,262 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the Registry classes"""
+
+import os
+import sys
+
+from bzrlib import (
+    errors,
+    registry,
+    osutils,
+    )
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+
+class TestRegistry(TestCase):
+
+    def register_stuff(self, a_registry):
+        a_registry.register('one', 1)
+        a_registry.register('two', 2)
+        a_registry.register('four', 4)
+        a_registry.register('five', 5)
+
+    def test_registry(self):
+        a_registry = registry.Registry()
+        self.register_stuff(a_registry)
+
+        self.failUnless(a_registry.default_key is None)
+
+        # test get() (self.default_key == None)
+        self.assertRaises(KeyError, a_registry.get)
+        self.assertRaises(KeyError, a_registry.get, None)
+        self.assertEqual(2, a_registry.get('two'))
+        self.assertRaises(KeyError, a_registry.get, 'three')
+
+        # test _set_default_key
+        a_registry.default_key = 'five'
+        self.failUnless(a_registry.default_key == 'five')
+        self.assertEqual(5, a_registry.get())
+        self.assertEqual(5, a_registry.get(None))
+        # If they ask for a specific entry, they should get KeyError
+        # not the default value. They can always pass None if they prefer
+        self.assertRaises(KeyError, a_registry.get, 'six')
+        self.assertRaises(KeyError, a_registry._set_default_key, 'six')
+
+        # test keys()
+        self.assertEqual(['five', 'four', 'one', 'two'], a_registry.keys())
+
+    def test_registry_funcs(self):
+        a_registry = registry.Registry()
+        self.register_stuff(a_registry)
+
+        self.failUnless('one' in a_registry)
+        a_registry.remove('one')
+        self.failIf('one' in a_registry)
+        self.assertRaises(KeyError, a_registry.get, 'one')
+
+        a_registry.register('one', 'one')
+
+        self.assertEqual(['five', 'four', 'one', 'two'],
+                         sorted(a_registry.keys()))
+        self.assertEqual([('five', 5), ('four', 4),
+                          ('one', 'one'), ('two', 2)],
+                         sorted(a_registry.iteritems()))
+
+    def test_register_override(self):
+        a_registry = registry.Registry()
+        a_registry.register('one', 'one')
+        self.assertRaises(KeyError, a_registry.register, 'one', 'two')
+        self.assertRaises(KeyError, a_registry.register, 'one', 'two',
+                                    override_existing=False)
+
+        a_registry.register('one', 'two', override_existing=True)
+        self.assertEqual('two', a_registry.get('one'))
+
+        self.assertRaises(KeyError, a_registry.register_lazy,
+                          'one', 'three', 'four')
+
+        a_registry.register_lazy('one', 'module', 'member',
+                                 override_existing=True)
+
+    def test_registry_help(self):
+        a_registry = registry.Registry()
+        a_registry.register('one', 1, help='help text for one')
+        # We should not have to import the module to return the help
+        # information
+        a_registry.register_lazy('two', 'nonexistent_module', 'member',
+                                 help='help text for two')
+
+        # We should be able to handle a callable to get information
+        help_calls = []
+        def generic_help(reg, key):
+            help_calls.append(key)
+            return 'generic help for %s' % (key,)
+        a_registry.register('three', 3, help=generic_help)
+        a_registry.register_lazy('four', 'nonexistent_module', 'member2',
+                                 help=generic_help)
+        a_registry.register('five', 5)
+
+        def help_from_object(reg, key):
+            obj = reg.get(key)
+            return obj.help()
+
+        class SimpleObj(object):
+            def help(self):
+                return 'this is my help'
+        a_registry.register('six', SimpleObj(), help=help_from_object)
+
+        self.assertEqual('help text for one', a_registry.get_help('one'))
+        self.assertEqual('help text for two', a_registry.get_help('two'))
+        self.assertEqual('generic help for three',
+                         a_registry.get_help('three'))
+        self.assertEqual(['three'], help_calls)
+        self.assertEqual('generic help for four',
+                         a_registry.get_help('four'))
+        self.assertEqual(['three', 'four'], help_calls)
+        self.assertEqual(None, a_registry.get_help('five'))
+        self.assertEqual('this is my help', a_registry.get_help('six'))
+
+        self.assertRaises(KeyError, a_registry.get_help, None)
+        self.assertRaises(KeyError, a_registry.get_help, 'seven')
+
+        a_registry.default_key = 'one'
+        self.assertEqual('help text for one', a_registry.get_help(None))
+        self.assertRaises(KeyError, a_registry.get_help, 'seven')
+
+        self.assertEqual([('five', None),
+                          ('four', 'generic help for four'),
+                          ('one', 'help text for one'),
+                          ('six', 'this is my help'),
+                          ('three', 'generic help for three'),
+                          ('two', 'help text for two'),
+                         ], sorted((key, a_registry.get_help(key))
+                                    for key in a_registry.keys()))
+
+        # We don't know what order it was called in, but we should get
+        # 2 more calls to three and four
+        self.assertEqual(['four', 'four', 'three', 'three'],
+                         sorted(help_calls))
+
+    def test_registry_info(self):
+        a_registry = registry.Registry()
+        a_registry.register('one', 1, info='string info')
+        # We should not have to import the module to return the info
+        a_registry.register_lazy('two', 'nonexistent_module', 'member',
+                                 info=2)
+
+        # We should be able to handle a callable to get information
+        a_registry.register('three', 3, info=['a', 'list'])
+        obj = object()
+        a_registry.register_lazy('four', 'nonexistent_module', 'member2',
+                                 info=obj)
+        a_registry.register('five', 5)
+
+        self.assertEqual('string info', a_registry.get_info('one'))
+        self.assertEqual(2, a_registry.get_info('two'))
+        self.assertEqual(['a', 'list'], a_registry.get_info('three'))
+        self.assertIs(obj, a_registry.get_info('four'))
+        self.assertIs(None, a_registry.get_info('five'))
+
+        self.assertRaises(KeyError, a_registry.get_info, None)
+        self.assertRaises(KeyError, a_registry.get_info, 'six')
+
+        a_registry.default_key = 'one'
+        self.assertEqual('string info', a_registry.get_info(None))
+        self.assertRaises(KeyError, a_registry.get_info, 'six')
+
+        self.assertEqual([('five', None),
+                          ('four', obj),
+                          ('one', 'string info'),
+                          ('three', ['a', 'list']),
+                          ('two', 2),
+                         ], sorted((key, a_registry.get_info(key))
+                                    for key in a_registry.keys()))
+
+
+class TestRegistryWithDirs(TestCaseInTempDir):
+    """Registry tests that require temporary dirs"""
+
+    def create_plugin_file(self, contents):
+        """Create a file to be used as a plugin.
+
+        This is created in a temporary directory, so that we
+        are sure that it doesn't start in the plugin path.
+        """
+        os.mkdir('tmp')
+        plugin_name = 'bzr_plugin_a_%s' % (osutils.rand_chars(4),)
+        open('tmp/'+plugin_name+'.py', 'wb').write(contents)
+        return plugin_name
+
+    def create_simple_plugin(self):
+        return self.create_plugin_file(
+            'object1 = "foo"\n'
+            '\n\n'
+            'def function(a,b,c):\n'
+            '    return a,b,c\n'
+            '\n\n'
+            'class MyClass(object):\n'
+            '    def __init__(self, a):\n'
+            '      self.a = a\n'
+            '\n\n'
+        )
+
+    def test_lazy_import_registry(self):
+        plugin_name = self.create_simple_plugin()
+        a_registry = registry.Registry()
+        a_registry.register_lazy('obj', plugin_name, 'object1')
+        a_registry.register_lazy('function', plugin_name, 'function')
+        a_registry.register_lazy('klass', plugin_name, 'MyClass')
+        a_registry.register_lazy('module', plugin_name, None)
+
+        self.assertEqual(['function', 'klass', 'module', 'obj'],
+                         sorted(a_registry.keys()))
+        # The plugin should not be loaded until we grab the first object
+        self.failIf(plugin_name in sys.modules)
+
+        # By default the plugin won't be in the search path
+        self.assertRaises(ImportError, a_registry.get, 'obj')
+
+        plugin_path = os.getcwd() + '/tmp'
+        sys.path.append(plugin_path)
+        try:
+            obj = a_registry.get('obj')
+            self.assertEqual('foo', obj)
+            self.failUnless(plugin_name in sys.modules)
+
+            # Now grab another object
+            func = a_registry.get('function')
+            self.assertEqual(plugin_name, func.__module__)
+            self.assertEqual('function', func.__name__)
+            self.assertEqual((1, [], '3'), func(1, [], '3'))
+
+            # And finally a class
+            klass = a_registry.get('klass')
+            self.assertEqual(plugin_name, klass.__module__)
+            self.assertEqual('MyClass', klass.__name__)
+
+            inst = klass(1)
+            self.assertIsInstance(inst, klass)
+            self.assertEqual(1, inst.a)
+
+            module = a_registry.get('module')
+            self.assertIs(obj, module.object1)
+            self.assertIs(func, module.function)
+            self.assertIs(klass, module.MyClass)
+        finally:
+            sys.path.remove(plugin_path)
+

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-09-05 20:58:28 +0000
+++ bzrlib/tests/test_repository.py	2006-09-09 18:52:57 +0000
@@ -38,13 +38,15 @@
 from bzrlib.transport import get_transport
 from bzrlib.transport.http import HttpServer
 from bzrlib.transport.memory import MemoryServer
+from bzrlib import upgrade, workingtree
 
 
 class TestDefaultFormat(TestCase):
 
     def test_get_set_default_format(self):
+        private_default = repository._default_format.__class__
         old_format = repository.RepositoryFormat.get_default_format()
-        self.assertTrue(isinstance(old_format, repository.RepositoryFormatKnit1))
+        self.assertTrue(isinstance(old_format, private_default))
         repository.RepositoryFormat.set_default_format(SampleRepositoryFormat())
         # creating a repository should now create an instrumented dir.
         try:
@@ -435,3 +437,25 @@
     def test_unescape_xml(self):
         """We get some kind of error when malformed entities are passed"""
         self.assertRaises(KeyError, repository._unescape_xml, 'foo&bar;') 
+
+
+class TestRepositoryFormatKnit2(TestCaseWithTransport):
+
+    def test_convert(self):
+        """Ensure the upgrade adds weaves for roots"""
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit1()
+        tree = self.make_branch_and_tree('.', format)
+        tree.commit("Dull commit", rev_id="dull")
+        revision_tree = tree.branch.repository.revision_tree('dull')
+        self.assertRaises(errors.NoSuchFile, revision_tree.get_file_lines,
+            revision_tree.inventory.root.file_id)
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        upgrade.Convert('.', format)
+        tree = workingtree.WorkingTree.open('.')
+        revision_tree = tree.branch.repository.revision_tree('dull')
+        revision_tree.get_file_lines(revision_tree.inventory.root.file_id)
+        tree.commit("Another dull commit", rev_id='dull2')
+        revision_tree = tree.branch.repository.revision_tree('dull2')
+        self.assertEqual('dull', revision_tree.inventory.root.revision)

=== modified file 'bzrlib/tests/test_revert.py'
--- bzrlib/tests/test_revert.py	2006-08-28 19:55:25 +0000
+++ bzrlib/tests/test_revert.py	2006-09-27 20:28:32 +0000
@@ -36,3 +36,44 @@
         target_tree.revert([])
         self.failIfExists('target/dir/contents')
         self.failIfExists('target/dir')
+
+    def test_revert_new(self):
+        """Only locally-changed new files should be preserved when reverting
+
+        When a file isn't present in revert's target tree:
+        If a file hasn't been committed, revert should unversion it, but not
+        delete it.
+        If a file has local changes, revert should unversion it, but not
+        delete it.
+        If a file has no changes from the last commit, revert should delete it.
+        If a file has changes due to a merge, revert should delete it.
+        """
+        tree = self.make_branch_and_tree('tree')
+        tree.commit('empty tree')
+        merge_target = tree.bzrdir.sprout('merge_target').open_workingtree()
+        self.build_tree(['tree/new_file'])
+
+        # newly-added files should not be deleted
+        tree.add('new_file')
+        basis_tree = tree.basis_tree()
+        tree.revert([])
+        self.failUnlessExists('tree/new_file')
+
+        # unchanged files should be deleted
+        tree.add('new_file')
+        tree.commit('add new_file')
+        tree.revert([], old_tree=basis_tree)
+        self.failIfExists('tree/new_file')
+        
+        # files should be deleted if their changes came from merges
+        merge_target.merge_from_branch(tree.branch)
+        self.failUnlessExists('merge_target/new_file')
+        merge_target.revert([])
+        self.failIfExists('merge_target/new_file')
+
+        # files should not be deleted if changed after a merge
+        merge_target.merge_from_branch(tree.branch)
+        self.failUnlessExists('merge_target/new_file')
+        self.build_tree_contents([('merge_target/new_file', 'new_contents')])
+        merge_target.revert([])
+        self.failUnlessExists('merge_target/new_file')

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/test_revision.py	2006-09-05 08:14:23 +0000
@@ -234,7 +234,7 @@
         """
         br1, br2 = make_branches(self)
         source = MultipleRevisionSources(br1.repository, br2.repository)
-        combined_1 = combined_graph(br1.last_revision(), 
+        combined_1 = combined_graph(br1.last_revision(),
                                     br2.last_revision(), source)
         combined_2 = combined_graph(br2.last_revision(),
                                     br1.last_revision(), source)

=== modified file 'bzrlib/tests/test_revisiontree.py'
--- bzrlib/tests/test_revisiontree.py	2006-07-14 05:02:37 +0000
+++ bzrlib/tests/test_revisiontree.py	2006-09-30 00:25:31 +0000
@@ -17,6 +17,9 @@
 
 """Tests for the RevisionTree class."""
 
+from bzrlib import (
+    revision,
+    )
 import bzrlib
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tree import RevisionTree
@@ -47,3 +50,12 @@
             [self.rev_id],
             self.t.branch.repository.revision_tree(revid_2).get_parent_ids())
         # TODO commit a merge and check it is reported correctly.
+
+        # the parents for a revision_tree(None) are []:
+        self.assertEqual([],
+            self.t.branch.repository.revision_tree(None).get_parent_ids())
+
+    def test_empty_no_root(self):
+        null_tree = self.t.branch.repository.revision_tree(
+            revision.NULL_REVISION)
+        self.assertIs(None, null_tree.inventory.root)

=== modified file 'bzrlib/tests/test_rio.py'
--- bzrlib/tests/test_rio.py	2006-03-09 18:47:40 +0000
+++ bzrlib/tests/test_rio.py	2006-09-22 19:37:57 +0000
@@ -27,6 +27,9 @@
 import sys
 from tempfile import TemporaryFile
 
+from bzrlib import (
+    rio,
+    )
 from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.rio import (RioWriter, Stanza, read_stanza, read_stanzas, rio_file,
                         RioReader)
@@ -320,8 +323,6 @@
         self.assertRaises(TypeError, s.add, 10, {})
 
     def test_rio_unicode(self):
-        # intentionally use cStringIO which doesn't accomodate unencoded unicode objects
-        sio = cStringIO.StringIO()
         uni_data = u'\N{KATAKANA LETTER O}'
         s = Stanza(foo=uni_data)
         self.assertEquals(s.get('foo'), uni_data)
@@ -331,3 +332,24 @@
         new_s = read_stanza(raw_lines)
         self.assertEquals(new_s.get('foo'), uni_data)
 
+    def test_rio_to_unicode(self):
+        uni_data = u'\N{KATAKANA LETTER O}'
+        s = Stanza(foo=uni_data)
+        unicode_str = s.to_unicode()
+        self.assertEqual(u'foo: %s\n' % (uni_data,), unicode_str)
+        new_s = rio.read_stanza_unicode(unicode_str.splitlines(True))
+        self.assertEqual(uni_data, new_s.get('foo'))
+
+    def test_nested_rio_unicode(self):
+        uni_data = u'\N{KATAKANA LETTER O}'
+        s = Stanza(foo=uni_data)
+        parent_stanza = Stanza(child=s.to_unicode())
+        raw_lines = parent_stanza.to_lines()
+        self.assertEqual(['child: foo: ' + uni_data.encode('utf-8') + '\n',
+                          '\t\n',
+                         ], raw_lines)
+        new_parent = read_stanza(raw_lines)
+        child_text = new_parent.get('child')
+        self.assertEqual(u'foo: %s\n' % uni_data, child_text)
+        new_child = rio.read_stanza_unicode(child_text.splitlines(True))
+        self.assertEqual(uni_data, new_child.get('foo'))

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-09-05 23:01:33 +0000
+++ bzrlib/tests/test_selftest.py	2006-10-13 07:22:23 +0000
@@ -15,6 +15,7 @@
 
 """Tests for the test framework."""
 
+import cStringIO
 import os
 from StringIO import StringIO
 import sys
@@ -22,23 +23,31 @@
 import unittest
 import warnings
 
-from bzrlib import osutils
 import bzrlib
+from bzrlib import (
+    bzrdir,
+    memorytree,
+    osutils,
+    repository,
+    )
 from bzrlib.progress import _BaseProgressBar
 from bzrlib.tests import (
                           ChrootedTestCase,
                           TestCase,
                           TestCaseInTempDir,
+                          TestCaseWithMemoryTransport,
                           TestCaseWithTransport,
                           TestSkipped,
                           TestSuite,
                           TextTestRunner,
                           )
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.tests.TestUtil import _load_module_by_name
 import bzrlib.errors as errors
 from bzrlib import symbol_versioning
 from bzrlib.symbol_versioning import zero_ten, zero_eleven
 from bzrlib.trace import note
+from bzrlib.transport.memory import MemoryServer, MemoryTransport
 from bzrlib.version import _get_bzr_source_tree
 
 
@@ -60,7 +69,8 @@
         """Test logs are captured when a test fails."""
         self.log('a test message')
         self._log_file.flush()
-        self.assertContainsRe(self._get_log(), 'a test message\n')
+        self.assertContainsRe(self._get_log(keep_log_file=True),
+                              'a test message\n')
 
 
 class TestTreeShape(TestCaseInTempDir):
@@ -420,6 +430,57 @@
         self.assertEqual(tests[1].transport_server, server1)
         self.assertEqual(tests[1].transport_readonly_server, server2)
 
+
+class TestTestCaseInTempDir(TestCaseInTempDir):
+
+    def test_home_is_not_working(self):
+        self.assertNotEqual(self.test_dir, self.test_home_dir)
+        cwd = osutils.getcwd()
+        self.assertEqual(self.test_dir, cwd)
+        self.assertEqual(self.test_home_dir, os.environ['HOME'])
+
+
+class TestTestCaseWithMemoryTransport(TestCaseWithMemoryTransport):
+
+    def test_home_is_non_existant_dir_under_root(self):
+        """The test_home_dir for TestCaseWithMemoryTransport is missing.
+
+        This is because TestCaseWithMemoryTransport is for tests that do not
+        need any disk resources: they should be hooked into bzrlib in such a 
+        way that no global settings are being changed by the test (only a 
+        few tests should need to do that), and having a missing dir as home is
+        an effective way to ensure that this is the case.
+        """
+        self.assertEqual(self.TEST_ROOT + "/MemoryTransportMissingHomeDir",
+            self.test_home_dir)
+        self.assertEqual(self.test_home_dir, os.environ['HOME'])
+        
+    def test_cwd_is_TEST_ROOT(self):
+        self.assertEqual(self.test_dir, self.TEST_ROOT)
+        cwd = osutils.getcwd()
+        self.assertEqual(self.test_dir, cwd)
+
+    def test_make_branch_and_memory_tree(self):
+        """In TestCaseWithMemoryTransport we should not make the branch on disk.
+
+        This is hard to comprehensively robustly test, so we settle for making
+        a branch and checking no directory was created at its relpath.
+        """
+        tree = self.make_branch_and_memory_tree('dir')
+        self.failIfExists('dir')
+        self.assertIsInstance(tree, memorytree.MemoryTree)
+
+    def test_make_branch_and_memory_tree_with_format(self):
+        """make_branch_and_memory_tree should accept a format option."""
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormat7()
+        tree = self.make_branch_and_memory_tree('dir', format=format)
+        self.failIfExists('dir')
+        self.assertIsInstance(tree, memorytree.MemoryTree)
+        self.assertEqual(format.repository_format.__class__,
+            tree.branch.repository._format.__class__)
+
+
 class TestTestCaseWithTransport(TestCaseWithTransport):
     """Tests for the convenience functions TestCaseWithTransport introduces."""
 
@@ -464,6 +525,21 @@
         self.assertRaises(AssertionError, self.assertIsDirectory, 'not_here', t)
 
 
+class TestTestCaseTransports(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestTestCaseTransports, self).setUp()
+        self.transport_server = MemoryServer
+
+    def test_make_bzrdir_preserves_transport(self):
+        t = self.get_transport()
+        result_bzrdir = self.make_bzrdir('subdir')
+        self.assertIsInstance(result_bzrdir.transport, 
+                              MemoryTransport)
+        # should not be on disk, should only be in memory
+        self.failIfExists('subdir')
+
+
 class TestChrootedTest(ChrootedTestCase):
 
     def test_root_is_root(self):
@@ -719,9 +795,71 @@
         output_string = output.getvalue()
         self.assertContainsRe(output_string, "--date [0-9.]+")
         if workingtree is not None:
-            revision_id = workingtree.last_revision()
+            revision_id = workingtree.get_parent_ids()[0]
             self.assertEndsWith(output_string.rstrip(), revision_id)
 
+    def test_success_log_deleted(self):
+        """Successful tests have their log deleted"""
+
+        class LogTester(TestCase):
+
+            def test_success(self):
+                self.log('this will be removed\n')
+
+        sio = cStringIO.StringIO()
+        runner = TextTestRunner(stream=sio)
+        test = LogTester('test_success')
+        result = self.run_test_runner(runner, test)
+
+        log = test._get_log()
+        self.assertEqual("DELETED log file to reduce memory footprint", log)
+        self.assertEqual('', test._log_contents)
+        self.assertIs(None, test._log_file_name)
+
+    def test_fail_log_kept(self):
+        """Failed tests have their log kept"""
+
+        class LogTester(TestCase):
+
+            def test_fail(self):
+                self.log('this will be kept\n')
+                self.fail('this test fails')
+
+        sio = cStringIO.StringIO()
+        runner = TextTestRunner(stream=sio)
+        test = LogTester('test_fail')
+        result = self.run_test_runner(runner, test)
+
+        text = sio.getvalue()
+        self.assertContainsRe(text, 'this will be kept')
+        self.assertContainsRe(text, 'this test fails')
+
+        log = test._get_log()
+        self.assertContainsRe(log, 'this will be kept')
+        self.assertEqual(log, test._log_contents)
+
+    def test_error_log_kept(self):
+        """Tests with errors have their log kept"""
+
+        class LogTester(TestCase):
+
+            def test_error(self):
+                self.log('this will be kept\n')
+                raise ValueError('random exception raised')
+
+        sio = cStringIO.StringIO()
+        runner = TextTestRunner(stream=sio)
+        test = LogTester('test_error')
+        result = self.run_test_runner(runner, test)
+
+        text = sio.getvalue()
+        self.assertContainsRe(text, 'this will be kept')
+        self.assertContainsRe(text, 'random exception raised')
+
+        log = test._get_log()
+        self.assertContainsRe(log, 'this will be kept')
+        self.assertEqual(log, test._log_contents)
+
 
 class TestTestCase(TestCase):
     """Tests that test the core bzrlib TestCase."""
@@ -893,6 +1031,27 @@
         self.assertIsInstance(bzrlib.bzrdir.BzrDir.open('b')._format,
                               bzrlib.bzrdir.BzrDirFormat6)
 
+    def test_make_branch_and_memory_tree(self):
+        # we should be able to get a new branch and a mutable tree from
+        # TestCaseWithTransport
+        tree = self.make_branch_and_memory_tree('a')
+        self.assertIsInstance(tree, bzrlib.memorytree.MemoryTree)
+
+
+class TestSFTPMakeBranchAndTree(TestCaseWithSFTPServer):
+
+    def test_make_tree_for_sftp_branch(self):
+        """Transports backed by local directories create local trees."""
+
+        tree = self.make_branch_and_tree('t1')
+        base = tree.bzrdir.root_transport.base
+        self.failIf(base.startswith('sftp'),
+                'base %r is on sftp but should be local' % base)
+        self.assertEquals(tree.bzrdir.root_transport,
+                tree.branch.bzrdir.root_transport)
+        self.assertEquals(tree.bzrdir.root_transport,
+                tree.branch.repository.bzrdir.root_transport)
+
 
 class TestSelftest(TestCase):
     """Tests of bzrlib.tests.selftest."""

=== modified file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	2006-08-15 13:19:12 +0000
+++ bzrlib/tests/test_setup.py	2006-09-09 16:43:09 +0000
@@ -46,6 +46,15 @@
         """
         if not os.path.isfile('setup.py'):
             raise TestSkipped('There is no setup.py file in current directory')
+        try:
+            import distutils.sysconfig
+            makefile_path = distutils.sysconfig.get_makefile_filename()
+            if not os.path.exists(makefile_path):
+                raise TestSkipped('You must have the python Makefile installed to run this test.'
+                                  ' Usually this can be found by installing "python-dev"')
+        except ImportError:
+            raise TestSkipped('You must have distutils installed to run this test.'
+                              ' Usually this can be found by installing "python-dev"')
         self.log('test_build running in %s' % os.getcwd())
         install_dir = tempfile.mkdtemp()
         # setup.py must be run from the root source directory, but the tests

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2006-08-25 07:31:08 +0000
+++ bzrlib/tests/test_sftp_transport.py	2006-09-16 06:44:47 +0000
@@ -21,9 +21,10 @@
 
 import bzrlib.bzrdir as bzrdir
 import bzrlib.errors as errors
-from bzrlib.osutils import pathjoin, lexists
+from bzrlib.osutils import pathjoin, lexists, set_or_unset_env
 from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
 import bzrlib.transport
+from bzrlib.transport import get_transport
 import bzrlib.transport.http
 from bzrlib.workingtree import WorkingTree
 
@@ -115,16 +116,18 @@
         ### FIXME decide and then test for all transports. RBC20051208
 
 
-class SFTPTransportTestRelative(TestCaseWithSFTPServer):
+class SFTPTransportTestRelativeRoot(TestCaseWithSFTPServer):
     """Test the SFTP transport with homedir based relative paths."""
 
     def setUp(self):
         self._get_remote_is_absolute = False
-        super(SFTPTransportTestRelative, self).setUp()
+        super(SFTPTransportTestRelativeRoot, self).setUp()
 
     def test__remote_path_relative_root(self):
         # relative paths are preserved
         t = self.get_transport('')
+        # the remote path should be ''
+        self.assertEqual('', t._path)
         self.assertEqual('a', t._remote_path('a'))
 
 
@@ -192,8 +195,37 @@
                               clone_from=fake)
             self.fail('expected exception not raised')
         except TransportError, e:
-            self.assertEquals(str(e), 
-                    'Transport error: ~janneke: invalid port number ')
+            self.assertEquals(str(e),
+                    'Transport error: '
+                    'invalid port number ~janneke in url:\n'
+                    'sftp://lilypond.org:~janneke/public_html/bzr/gub ')
+
+    def test_get_paramiko_vendor(self):
+        """Test that if no 'ssh' is available we get builtin paramiko"""
+        from bzrlib.transport import ssh
+        # set '.' as the only location in the path, forcing no 'ssh' to exist
+        orig_vendor = ssh._ssh_vendor
+        orig_path = set_or_unset_env('PATH', '.')
+        try:
+            # No vendor defined yet, query for one
+            ssh._ssh_vendor = None
+            vendor = ssh._get_ssh_vendor()
+            self.assertIsInstance(vendor, ssh.ParamikoVendor)
+        finally:
+            set_or_unset_env('PATH', orig_path)
+            ssh._ssh_vendor = orig_vendor
+
+    def test_abspath_root_sibling_server(self):
+        from bzrlib.transport.sftp import SFTPSiblingAbsoluteServer
+        server = SFTPSiblingAbsoluteServer()
+        server.setUp()
+        try:
+            transport = get_transport(server.get_url())
+            self.assertFalse(transport.abspath('/').endswith('/~/'))
+            self.assertTrue(transport.abspath('/').endswith('/'))
+            del transport
+        finally:
+            server.tearDown()
 
 
 class SFTPBranchTest(TestCaseWithSFTPServer):
@@ -403,6 +435,8 @@
 
     def setUp(self):
         TestCase.setUp(self)
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
 
     def test_delay(self):
         from bzrlib.transport.sftp import SocketDelay

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2006-08-15 15:50:31 +0000
+++ bzrlib/tests/test_smart_add.py	2006-09-10 19:04:48 +0000
@@ -121,7 +121,6 @@
     def test_add_dry_run(self):
         """Test a dry run add, make sure nothing is added."""
         from bzrlib.commands import run_bzr
-        ignores._set_user_ignores(['./.bazaar'])
         eq = self.assertEqual
         wt = self.make_branch_and_tree('.')
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
@@ -140,7 +139,7 @@
         from bzrlib.commands import run_bzr
         wt = self.make_branch_and_tree('.')
         # The default ignore list includes '*.py[co]', but not CVS
-        ignores._set_user_ignores(['./.bazaar', '*.py[co]'])
+        ignores._set_user_ignores(['*.py[co]'])
         self.build_tree(['inertiatic/', 'inertiatic/esp', 'inertiatic/CVS',
                         'inertiatic/foo.pyc'])
         added, ignored = smart_add_tree(wt, u'.')
@@ -320,14 +319,12 @@
 
         self.build_tree(['new/a', 'new/b', 'new/c',
                          'new/subdir/', 'new/subdir/b', 'new/subdir/d'])
+        new_tree.set_root_id(self.base_tree.get_root_id())
         self.add_helper(self.base_tree, 'dir', new_tree, ['new'])
 
-        # We 'a' and 'b' exist in the root, and they are being added
-        # in a new 'root'. Since ROOT ids are not unique, we will
+        # We know 'a' and 'b' exist in the root, and they are being added
+        # in a new 'root'. Since ROOT ids have been set as the same, we will
         # use those ids
-        # TODO: This will probably change once trees have a unique root id
-        # It is definitely arguable that 'a' should get the id of
-        # 'dir/a' not of 'a'.
         self.assertEqual(self.base_tree.path2id('a'),
                          new_tree.path2id('a'))
         self.assertEqual(self.base_tree.path2id('b'),

=== added file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_smart_transport.py	2006-10-09 07:34:15 +0000
@@ -0,0 +1,1539 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for smart transport"""
+
+# all of this deals with byte strings so this is safe
+from cStringIO import StringIO
+import os
+import socket
+import threading
+import urllib2
+
+from bzrlib import (
+        bzrdir,
+        errors,
+        osutils,
+        tests,
+        urlutils,
+        )
+from bzrlib.transport import (
+        get_transport,
+        local,
+        memory,
+        smart,
+        )
+from bzrlib.transport.http import (
+        HTTPServerWithSmarts,
+        SmartClientHTTPMediumRequest,
+        SmartRequestHandler,
+        )
+
+
+class StringIOSSHVendor(object):
+    """A SSH vendor that uses StringIO to buffer writes and answer reads."""
+
+    def __init__(self, read_from, write_to):
+        self.read_from = read_from
+        self.write_to = write_to
+        self.calls = []
+
+    def connect_ssh(self, username, password, host, port, command):
+        self.calls.append(('connect_ssh', username, password, host, port,
+            command))
+        return StringIOSSHConnection(self)
+
+
+class StringIOSSHConnection(object):
+    """A SSH connection that uses StringIO to buffer writes and answer reads."""
+
+    def __init__(self, vendor):
+        self.vendor = vendor
+    
+    def close(self):
+        self.vendor.calls.append(('close', ))
+        
+    def get_filelike_channels(self):
+        return self.vendor.read_from, self.vendor.write_to
+
+
+
+class SmartClientMediumTests(tests.TestCase):
+    """Tests for SmartClientMedium.
+
+    We should create a test scenario for this: we need a server module that
+    construct the test-servers (like make_loopsocket_and_medium), and the list
+    of SmartClientMedium classes to test.
+    """
+
+    def make_loopsocket_and_medium(self):
+        """Create a loopback socket for testing, and a medium aimed at it."""
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.bind(('127.0.0.1', 0))
+        sock.listen(1)
+        port = sock.getsockname()[1]
+        medium = smart.SmartTCPClientMedium('127.0.0.1', port)
+        return sock, medium
+
+    def receive_bytes_on_server(self, sock, bytes):
+        """Accept a connection on sock and read 3 bytes.
+
+        The bytes are appended to the list bytes.
+
+        :return: a Thread which is running to do the accept and recv.
+        """
+        def _receive_bytes_on_server():
+            connection, address = sock.accept()
+            bytes.append(connection.recv(3, socket.MSG_WAITALL))
+            connection.close()
+        t = threading.Thread(target=_receive_bytes_on_server)
+        t.start()
+        return t
+    
+    def test_construct_smart_stream_medium_client(self):
+        # make a new instance of the common base for Stream-like Mediums.
+        # this just ensures that the constructor stays parameter-free which
+        # is important for reuse : some subclasses will dynamically connect,
+        # others are always on, etc.
+        medium = smart.SmartClientStreamMedium()
+
+    def test_construct_smart_client_medium(self):
+        # the base client medium takes no parameters
+        medium = smart.SmartClientMedium()
+    
+    def test_construct_smart_simple_pipes_client_medium(self):
+        # the SimplePipes client medium takes two pipes:
+        # readable pipe, writeable pipe.
+        # Constructing one should just save these and do nothing.
+        # We test this by passing in None.
+        medium = smart.SmartSimplePipesClientMedium(None, None)
+        
+    def test_simple_pipes_client_request_type(self):
+        # SimplePipesClient should use SmartClientStreamMediumRequest's.
+        medium = smart.SmartSimplePipesClientMedium(None, None)
+        request = medium.get_request()
+        self.assertIsInstance(request, smart.SmartClientStreamMediumRequest)
+
+    def test_simple_pipes_client_get_concurrent_requests(self):
+        # the simple_pipes client does not support pipelined requests:
+        # but it does support serial requests: we construct one after 
+        # another is finished. This is a smoke test testing the integration
+        # of the SmartClientStreamMediumRequest and the SmartClientStreamMedium
+        # classes - as the sibling classes share this logic, they do not have
+        # explicit tests for this.
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        request = medium.get_request()
+        request.finished_writing()
+        request.finished_reading()
+        request2 = medium.get_request()
+        request2.finished_writing()
+        request2.finished_reading()
+
+    def test_simple_pipes_client__accept_bytes_writes_to_writable(self):
+        # accept_bytes writes to the writeable pipe.
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        medium._accept_bytes('abc')
+        self.assertEqual('abc', output.getvalue())
+    
+    def test_simple_pipes_client_disconnect_does_nothing(self):
+        # calling disconnect does nothing.
+        input = StringIO()
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        # send some bytes to ensure disconnecting after activity still does not
+        # close.
+        medium._accept_bytes('abc')
+        medium.disconnect()
+        self.assertFalse(input.closed)
+        self.assertFalse(output.closed)
+
+    def test_simple_pipes_client_accept_bytes_after_disconnect(self):
+        # calling disconnect on the client does not alter the pipe that
+        # accept_bytes writes to.
+        input = StringIO()
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        medium._accept_bytes('abc')
+        medium.disconnect()
+        medium._accept_bytes('abc')
+        self.assertFalse(input.closed)
+        self.assertFalse(output.closed)
+        self.assertEqual('abcabc', output.getvalue())
+    
+    def test_simple_pipes_client_ignores_disconnect_when_not_connected(self):
+        # Doing a disconnect on a new (and thus unconnected) SimplePipes medium
+        # does nothing.
+        medium = smart.SmartSimplePipesClientMedium(None, None)
+        medium.disconnect()
+
+    def test_simple_pipes_client_can_always_read(self):
+        # SmartSimplePipesClientMedium is never disconnected, so read_bytes
+        # always tries to read from the underlying pipe.
+        input = StringIO('abcdef')
+        medium = smart.SmartSimplePipesClientMedium(input, None)
+        self.assertEqual('abc', medium.read_bytes(3))
+        medium.disconnect()
+        self.assertEqual('def', medium.read_bytes(3))
+        
+    def test_simple_pipes_client_supports__flush(self):
+        # invoking _flush on a SimplePipesClient should flush the output 
+        # pipe. We test this by creating an output pipe that records
+        # flush calls made to it.
+        from StringIO import StringIO # get regular StringIO
+        input = StringIO()
+        output = StringIO()
+        flush_calls = []
+        def logging_flush(): flush_calls.append('flush')
+        output.flush = logging_flush
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        # this call is here to ensure we only flush once, not on every
+        # _accept_bytes call.
+        medium._accept_bytes('abc')
+        medium._flush()
+        medium.disconnect()
+        self.assertEqual(['flush'], flush_calls)
+
+    def test_construct_smart_ssh_client_medium(self):
+        # the SSH client medium takes:
+        # host, port, username, password, vendor
+        # Constructing one should just save these and do nothing.
+        # we test this by creating a empty bound socket and constructing
+        # a medium.
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.bind(('127.0.0.1', 0))
+        unopened_port = sock.getsockname()[1]
+        # having vendor be invalid means that if it tries to connect via the
+        # vendor it will blow up.
+        medium = smart.SmartSSHClientMedium('127.0.0.1', unopened_port,
+            username=None, password=None, vendor="not a vendor")
+        sock.close()
+
+    def test_ssh_client_connects_on_first_use(self):
+        # The only thing that initiates a connection from the medium is giving
+        # it bytes.
+        output = StringIO()
+        vendor = StringIOSSHVendor(StringIO(), output)
+        medium = smart.SmartSSHClientMedium('a hostname', 'a port', 'a username',
+            'a password', vendor)
+        medium._accept_bytes('abc')
+        self.assertEqual('abc', output.getvalue())
+        self.assertEqual([('connect_ssh', 'a username', 'a password',
+            'a hostname', 'a port',
+            ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes'])],
+            vendor.calls)
+    
+    def test_ssh_client_changes_command_when_BZR_REMOTE_PATH_is_set(self):
+        # The only thing that initiates a connection from the medium is giving
+        # it bytes.
+        output = StringIO()
+        vendor = StringIOSSHVendor(StringIO(), output)
+        orig_bzr_remote_path = os.environ.get('BZR_REMOTE_PATH')
+        def cleanup_environ():
+            osutils.set_or_unset_env('BZR_REMOTE_PATH', orig_bzr_remote_path)
+        self.addCleanup(cleanup_environ)
+        os.environ['BZR_REMOTE_PATH'] = 'fugly'
+        medium = smart.SmartSSHClientMedium('a hostname', 'a port', 'a username',
+            'a password', vendor)
+        medium._accept_bytes('abc')
+        self.assertEqual('abc', output.getvalue())
+        self.assertEqual([('connect_ssh', 'a username', 'a password',
+            'a hostname', 'a port',
+            ['fugly', 'serve', '--inet', '--directory=/', '--allow-writes'])],
+            vendor.calls)
+    
+    def test_ssh_client_disconnect_does_so(self):
+        # calling disconnect should disconnect both the read_from and write_to
+        # file-like object it from the ssh connection.
+        input = StringIO()
+        output = StringIO()
+        vendor = StringIOSSHVendor(input, output)
+        medium = smart.SmartSSHClientMedium('a hostname', vendor=vendor)
+        medium._accept_bytes('abc')
+        medium.disconnect()
+        self.assertTrue(input.closed)
+        self.assertTrue(output.closed)
+        self.assertEqual([
+            ('connect_ssh', None, None, 'a hostname', None,
+            ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+            ('close', ),
+            ],
+            vendor.calls)
+
+    def test_ssh_client_disconnect_allows_reconnection(self):
+        # calling disconnect on the client terminates the connection, but should
+        # not prevent additional connections occuring.
+        # we test this by initiating a second connection after doing a
+        # disconnect.
+        input = StringIO()
+        output = StringIO()
+        vendor = StringIOSSHVendor(input, output)
+        medium = smart.SmartSSHClientMedium('a hostname', vendor=vendor)
+        medium._accept_bytes('abc')
+        medium.disconnect()
+        # the disconnect has closed output, so we need a new output for the
+        # new connection to write to.
+        input2 = StringIO()
+        output2 = StringIO()
+        vendor.read_from = input2
+        vendor.write_to = output2
+        medium._accept_bytes('abc')
+        medium.disconnect()
+        self.assertTrue(input.closed)
+        self.assertTrue(output.closed)
+        self.assertTrue(input2.closed)
+        self.assertTrue(output2.closed)
+        self.assertEqual([
+            ('connect_ssh', None, None, 'a hostname', None,
+            ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+            ('close', ),
+            ('connect_ssh', None, None, 'a hostname', None,
+            ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+            ('close', ),
+            ],
+            vendor.calls)
+    
+    def test_ssh_client_ignores_disconnect_when_not_connected(self):
+        # Doing a disconnect on a new (and thus unconnected) SSH medium
+        # does not fail.  It's ok to disconnect an unconnected medium.
+        medium = smart.SmartSSHClientMedium(None)
+        medium.disconnect()
+
+    def test_ssh_client_raises_on_read_when_not_connected(self):
+        # Doing a read on a new (and thus unconnected) SSH medium raises
+        # MediumNotConnected.
+        medium = smart.SmartSSHClientMedium(None)
+        self.assertRaises(errors.MediumNotConnected, medium.read_bytes, 0)
+        self.assertRaises(errors.MediumNotConnected, medium.read_bytes, 1)
+
+    def test_ssh_client_supports__flush(self):
+        # invoking _flush on a SSHClientMedium should flush the output 
+        # pipe. We test this by creating an output pipe that records
+        # flush calls made to it.
+        from StringIO import StringIO # get regular StringIO
+        input = StringIO()
+        output = StringIO()
+        flush_calls = []
+        def logging_flush(): flush_calls.append('flush')
+        output.flush = logging_flush
+        vendor = StringIOSSHVendor(input, output)
+        medium = smart.SmartSSHClientMedium('a hostname', vendor=vendor)
+        # this call is here to ensure we only flush once, not on every
+        # _accept_bytes call.
+        medium._accept_bytes('abc')
+        medium._flush()
+        medium.disconnect()
+        self.assertEqual(['flush'], flush_calls)
+        
+    def test_construct_smart_tcp_client_medium(self):
+        # the TCP client medium takes a host and a port.  Constructing it won't
+        # connect to anything.
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.bind(('127.0.0.1', 0))
+        unopened_port = sock.getsockname()[1]
+        medium = smart.SmartTCPClientMedium('127.0.0.1', unopened_port)
+        sock.close()
+
+    def test_tcp_client_connects_on_first_use(self):
+        # The only thing that initiates a connection from the medium is giving
+        # it bytes.
+        sock, medium = self.make_loopsocket_and_medium()
+        bytes = []
+        t = self.receive_bytes_on_server(sock, bytes)
+        medium.accept_bytes('abc')
+        t.join()
+        sock.close()
+        self.assertEqual(['abc'], bytes)
+    
+    def test_tcp_client_disconnect_does_so(self):
+        # calling disconnect on the client terminates the connection.
+        # we test this by forcing a short read during a socket.MSG_WAITALL
+        # call: write 2 bytes, try to read 3, and then the client disconnects.
+        sock, medium = self.make_loopsocket_and_medium()
+        bytes = []
+        t = self.receive_bytes_on_server(sock, bytes)
+        medium.accept_bytes('ab')
+        medium.disconnect()
+        t.join()
+        sock.close()
+        self.assertEqual(['ab'], bytes)
+        # now disconnect again: this should not do anything, if disconnection
+        # really did disconnect.
+        medium.disconnect()
+    
+    def test_tcp_client_ignores_disconnect_when_not_connected(self):
+        # Doing a disconnect on a new (and thus unconnected) TCP medium
+        # does not fail.  It's ok to disconnect an unconnected medium.
+        medium = smart.SmartTCPClientMedium(None, None)
+        medium.disconnect()
+
+    def test_tcp_client_raises_on_read_when_not_connected(self):
+        # Doing a read on a new (and thus unconnected) TCP medium raises
+        # MediumNotConnected.
+        medium = smart.SmartTCPClientMedium(None, None)
+        self.assertRaises(errors.MediumNotConnected, medium.read_bytes, 0)
+        self.assertRaises(errors.MediumNotConnected, medium.read_bytes, 1)
+
+    def test_tcp_client_supports__flush(self):
+        # invoking _flush on a TCPClientMedium should do something useful.
+        # RBC 20060922 not sure how to test/tell in this case.
+        sock, medium = self.make_loopsocket_and_medium()
+        bytes = []
+        t = self.receive_bytes_on_server(sock, bytes)
+        # try with nothing buffered
+        medium._flush()
+        medium._accept_bytes('ab')
+        # and with something sent.
+        medium._flush()
+        medium.disconnect()
+        t.join()
+        sock.close()
+        self.assertEqual(['ab'], bytes)
+        # now disconnect again : this should not do anything, if disconnection
+        # really did disconnect.
+        medium.disconnect()
+
+
+class TestSmartClientStreamMediumRequest(tests.TestCase):
+    """Tests the for SmartClientStreamMediumRequest.
+    
+    SmartClientStreamMediumRequest is a helper for the three stream based 
+    mediums: TCP, SSH, SimplePipes, so we only test it once, and then test that
+    those three mediums implement the interface it expects.
+    """
+
+    def test_accept_bytes_after_finished_writing_errors(self):
+        # calling accept_bytes after calling finished_writing raises 
+        # WritingCompleted to prevent bad assumptions on stream environments
+        # breaking the needs of message-based environments.
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        request.finished_writing()
+        self.assertRaises(errors.WritingCompleted, request.accept_bytes, None)
+
+    def test_accept_bytes(self):
+        # accept bytes should invoke _accept_bytes on the stream medium.
+        # we test this by using the SimplePipes medium - the most trivial one
+        # and checking that the pipes get the data.
+        input = StringIO()
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        request.accept_bytes('123')
+        request.finished_writing()
+        request.finished_reading()
+        self.assertEqual('', input.getvalue())
+        self.assertEqual('123', output.getvalue())
+
+    def test_construct_sets_stream_request(self):
+        # constructing a SmartClientStreamMediumRequest on a StreamMedium sets
+        # the current request to the new SmartClientStreamMediumRequest
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        self.assertIs(medium._current_request, request)
+
+    def test_construct_while_another_request_active_throws(self):
+        # constructing a SmartClientStreamMediumRequest on a StreamMedium with
+        # a non-None _current_request raises TooManyConcurrentRequests.
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        medium._current_request = "a"
+        self.assertRaises(errors.TooManyConcurrentRequests,
+            smart.SmartClientStreamMediumRequest, medium)
+
+    def test_finished_read_clears_current_request(self):
+        # calling finished_reading clears the current request from the requests
+        # medium
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        request.finished_writing()
+        request.finished_reading()
+        self.assertEqual(None, medium._current_request)
+
+    def test_finished_read_before_finished_write_errors(self):
+        # calling finished_reading before calling finished_writing triggers a
+        # WritingNotComplete error.
+        medium = smart.SmartSimplePipesClientMedium(None, None)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        self.assertRaises(errors.WritingNotComplete, request.finished_reading)
+        
+    def test_read_bytes(self):
+        # read bytes should invoke _read_bytes on the stream medium.
+        # we test this by using the SimplePipes medium - the most trivial one
+        # and checking that the data is supplied. Its possible that a 
+        # faulty implementation could poke at the pipe variables them selves,
+        # but we trust that this will be caught as it will break the integration
+        # smoke tests.
+        input = StringIO('321')
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        request.finished_writing()
+        self.assertEqual('321', request.read_bytes(3))
+        request.finished_reading()
+        self.assertEqual('', input.read())
+        self.assertEqual('', output.getvalue())
+
+    def test_read_bytes_before_finished_write_errors(self):
+        # calling read_bytes before calling finished_writing triggers a
+        # WritingNotComplete error because the Smart protocol is designed to be
+        # compatible with strict message based protocols like HTTP where the
+        # request cannot be submitted until the writing has completed.
+        medium = smart.SmartSimplePipesClientMedium(None, None)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        self.assertRaises(errors.WritingNotComplete, request.read_bytes, None)
+
+    def test_read_bytes_after_finished_reading_errors(self):
+        # calling read_bytes after calling finished_reading raises 
+        # ReadingCompleted to prevent bad assumptions on stream environments
+        # breaking the needs of message-based environments.
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        request = smart.SmartClientStreamMediumRequest(medium)
+        request.finished_writing()
+        request.finished_reading()
+        self.assertRaises(errors.ReadingCompleted, request.read_bytes, None)
+
+
+class RemoteTransportTests(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(RemoteTransportTests, self).setUp()
+        # We're allowed to set  the transport class here, so that we don't use
+        # the default or a parameterized class, but rather use the
+        # TestCaseWithTransport infrastructure to set up a smart server and
+        # transport.
+        self.transport_server = smart.SmartTCPServer_for_testing
+
+    def test_plausible_url(self):
+        self.assert_(self.get_url().startswith('bzr://'))
+
+    def test_probe_transport(self):
+        t = self.get_transport()
+        self.assertIsInstance(t, smart.SmartTransport)
+
+    def test_get_medium_from_transport(self):
+        """Remote transport has a medium always, which it can return."""
+        t = self.get_transport()
+        medium = t.get_smart_medium()
+        self.assertIsInstance(medium, smart.SmartClientMedium)
+
+
+class ErrorRaisingProtocol(object):
+
+    def __init__(self, exception):
+        self.exception = exception
+
+    def next_read_size(self):
+        raise self.exception
+
+
+class SampleRequest(object):
+    
+    def __init__(self, expected_bytes):
+        self.accepted_bytes = ''
+        self._finished_reading = False
+        self.expected_bytes = expected_bytes
+        self.excess_buffer = ''
+
+    def accept_bytes(self, bytes):
+        self.accepted_bytes += bytes
+        if self.accepted_bytes.startswith(self.expected_bytes):
+            self._finished_reading = True
+            self.excess_buffer = self.accepted_bytes[len(self.expected_bytes):]
+
+    def next_read_size(self):
+        if self._finished_reading:
+            return 0
+        else:
+            return 1
+
+
+class TestSmartServerStreamMedium(tests.TestCase):
+
+    def portable_socket_pair(self):
+        """Return a pair of TCP sockets connected to each other.
+        
+        Unlike socket.socketpair, this should work on Windows.
+        """
+        listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        listen_sock.bind(('127.0.0.1', 0))
+        listen_sock.listen(1)
+        client_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        client_sock.connect(listen_sock.getsockname())
+        server_sock, addr = listen_sock.accept()
+        listen_sock.close()
+        return server_sock, client_sock
+    
+    def test_smart_query_version(self):
+        """Feed a canned query version to a server"""
+        # wire-to-wire, using the whole stack
+        to_server = StringIO('hello\n')
+        from_server = StringIO()
+        transport = local.LocalTransport(urlutils.local_path_to_url('/'))
+        server = smart.SmartServerPipeStreamMedium(
+            to_server, from_server, transport)
+        protocol = smart.SmartServerRequestProtocolOne(transport,
+                from_server.write)
+        server._serve_one_request(protocol)
+        self.assertEqual('ok\0011\n',
+                         from_server.getvalue())
+
+    def test_canned_get_response(self):
+        transport = memory.MemoryTransport('memory:///')
+        transport.put_bytes('testfile', 'contents\nof\nfile\n')
+        to_server = StringIO('get\001./testfile\n')
+        from_server = StringIO()
+        server = smart.SmartServerPipeStreamMedium(
+            to_server, from_server, transport)
+        protocol = smart.SmartServerRequestProtocolOne(transport,
+                from_server.write)
+        server._serve_one_request(protocol)
+        self.assertEqual('ok\n'
+                         '17\n'
+                         'contents\nof\nfile\n'
+                         'done\n',
+                         from_server.getvalue())
+
+    def test_pipe_like_stream_with_bulk_data(self):
+        sample_request_bytes = 'command\n9\nbulk datadone\n'
+        to_server = StringIO(sample_request_bytes)
+        from_server = StringIO()
+        server = smart.SmartServerPipeStreamMedium(to_server, from_server, None)
+        sample_protocol = SampleRequest(expected_bytes=sample_request_bytes)
+        server._serve_one_request(sample_protocol)
+        self.assertEqual('', from_server.getvalue())
+        self.assertEqual(sample_request_bytes, sample_protocol.accepted_bytes)
+        self.assertFalse(server.finished)
+
+    def test_socket_stream_with_bulk_data(self):
+        sample_request_bytes = 'command\n9\nbulk datadone\n'
+        server_sock, client_sock = self.portable_socket_pair()
+        server = smart.SmartServerSocketStreamMedium(
+            server_sock, None)
+        sample_protocol = SampleRequest(expected_bytes=sample_request_bytes)
+        client_sock.sendall(sample_request_bytes)
+        server._serve_one_request(sample_protocol)
+        server_sock.close()
+        self.assertEqual('', client_sock.recv(1))
+        self.assertEqual(sample_request_bytes, sample_protocol.accepted_bytes)
+        self.assertFalse(server.finished)
+
+    def test_pipe_like_stream_shutdown_detection(self):
+        to_server = StringIO('')
+        from_server = StringIO()
+        server = smart.SmartServerPipeStreamMedium(to_server, from_server, None)
+        server._serve_one_request(SampleRequest('x'))
+        self.assertTrue(server.finished)
+        
+    def test_socket_stream_shutdown_detection(self):
+        server_sock, client_sock = self.portable_socket_pair()
+        client_sock.close()
+        server = smart.SmartServerSocketStreamMedium(
+            server_sock, None)
+        server._serve_one_request(SampleRequest('x'))
+        self.assertTrue(server.finished)
+        
+    def test_pipe_like_stream_with_two_requests(self):
+        # If two requests are read in one go, then two calls to
+        # _serve_one_request should still process both of them as if they had
+        # been received seperately.
+        sample_request_bytes = 'command\n'
+        to_server = StringIO(sample_request_bytes * 2)
+        from_server = StringIO()
+        server = smart.SmartServerPipeStreamMedium(to_server, from_server, None)
+        first_protocol = SampleRequest(expected_bytes=sample_request_bytes)
+        server._serve_one_request(first_protocol)
+        self.assertEqual(0, first_protocol.next_read_size())
+        self.assertEqual('', from_server.getvalue())
+        self.assertFalse(server.finished)
+        # Make a new protocol, call _serve_one_request with it to collect the
+        # second request.
+        second_protocol = SampleRequest(expected_bytes=sample_request_bytes)
+        server._serve_one_request(second_protocol)
+        self.assertEqual('', from_server.getvalue())
+        self.assertEqual(sample_request_bytes, second_protocol.accepted_bytes)
+        self.assertFalse(server.finished)
+        
+    def test_socket_stream_with_two_requests(self):
+        # If two requests are read in one go, then two calls to
+        # _serve_one_request should still process both of them as if they had
+        # been received seperately.
+        sample_request_bytes = 'command\n'
+        server_sock, client_sock = self.portable_socket_pair()
+        server = smart.SmartServerSocketStreamMedium(
+            server_sock, None)
+        first_protocol = SampleRequest(expected_bytes=sample_request_bytes)
+        # Put two whole requests on the wire.
+        client_sock.sendall(sample_request_bytes * 2)
+        server._serve_one_request(first_protocol)
+        self.assertEqual(0, first_protocol.next_read_size())
+        self.assertFalse(server.finished)
+        # Make a new protocol, call _serve_one_request with it to collect the
+        # second request.
+        second_protocol = SampleRequest(expected_bytes=sample_request_bytes)
+        stream_still_open = server._serve_one_request(second_protocol)
+        self.assertEqual(sample_request_bytes, second_protocol.accepted_bytes)
+        self.assertFalse(server.finished)
+        server_sock.close()
+        self.assertEqual('', client_sock.recv(1))
+
+    def test_pipe_like_stream_error_handling(self):
+        # Use plain python StringIO so we can monkey-patch the close method to
+        # not discard the contents.
+        from StringIO import StringIO
+        to_server = StringIO('')
+        from_server = StringIO()
+        self.closed = False
+        def close():
+            self.closed = True
+        from_server.close = close
+        server = smart.SmartServerPipeStreamMedium(to_server, from_server, None)
+        fake_protocol = ErrorRaisingProtocol(Exception('boom'))
+        server._serve_one_request(fake_protocol)
+        self.assertEqual('', from_server.getvalue())
+        self.assertTrue(self.closed)
+        self.assertTrue(server.finished)
+        
+    def test_socket_stream_error_handling(self):
+        # Use plain python StringIO so we can monkey-patch the close method to
+        # not discard the contents.
+        from StringIO import StringIO
+        server_sock, client_sock = self.portable_socket_pair()
+        server = smart.SmartServerSocketStreamMedium(
+            server_sock, None)
+        fake_protocol = ErrorRaisingProtocol(Exception('boom'))
+        server._serve_one_request(fake_protocol)
+        # recv should not block, because the other end of the socket has been
+        # closed.
+        self.assertEqual('', client_sock.recv(1))
+        self.assertTrue(server.finished)
+        
+    def test_pipe_like_stream_keyboard_interrupt_handling(self):
+        # Use plain python StringIO so we can monkey-patch the close method to
+        # not discard the contents.
+        to_server = StringIO('')
+        from_server = StringIO()
+        server = smart.SmartServerPipeStreamMedium(to_server, from_server, None)
+        fake_protocol = ErrorRaisingProtocol(KeyboardInterrupt('boom'))
+        self.assertRaises(
+            KeyboardInterrupt, server._serve_one_request, fake_protocol)
+        self.assertEqual('', from_server.getvalue())
+
+    def test_socket_stream_keyboard_interrupt_handling(self):
+        server_sock, client_sock = self.portable_socket_pair()
+        server = smart.SmartServerSocketStreamMedium(
+            server_sock, None)
+        fake_protocol = ErrorRaisingProtocol(KeyboardInterrupt('boom'))
+        self.assertRaises(
+            KeyboardInterrupt, server._serve_one_request, fake_protocol)
+        server_sock.close()
+        self.assertEqual('', client_sock.recv(1))
+        
+
+class TestSmartTCPServer(tests.TestCase):
+
+    def test_get_error_unexpected(self):
+        """Error reported by server with no specific representation"""
+        class FlakyTransport(object):
+            def get_bytes(self, path):
+                raise Exception("some random exception from inside server")
+        server = smart.SmartTCPServer(backing_transport=FlakyTransport())
+        server.start_background_thread()
+        try:
+            transport = smart.SmartTCPTransport(server.get_url())
+            try:
+                transport.get('something')
+            except errors.TransportError, e:
+                self.assertContainsRe(str(e), 'some random exception')
+            else:
+                self.fail("get did not raise expected error")
+        finally:
+            server.stop_background_thread()
+
+
+class SmartTCPTests(tests.TestCase):
+    """Tests for connection/end to end behaviour using the TCP server.
+
+    All of these tests are run with a server running on another thread serving
+    a MemoryTransport, and a connection to it already open.
+
+    the server is obtained by calling self.setUpServer(readonly=False).
+    """
+
+    def setUpServer(self, readonly=False):
+        """Setup the server.
+
+        :param readonly: Create a readonly server.
+        """
+        self.backing_transport = memory.MemoryTransport()
+        if readonly:
+            self.real_backing_transport = self.backing_transport
+            self.backing_transport = get_transport("readonly+" + self.backing_transport.abspath('.'))
+        self.server = smart.SmartTCPServer(self.backing_transport)
+        self.server.start_background_thread()
+        self.transport = smart.SmartTCPTransport(self.server.get_url())
+
+    def tearDown(self):
+        if getattr(self, 'transport', None):
+            self.transport.disconnect()
+        if getattr(self, 'server', None):
+            self.server.stop_background_thread()
+        super(SmartTCPTests, self).tearDown()
+        
+
+class WritableEndToEndTests(SmartTCPTests):
+    """Client to server tests that require a writable transport."""
+
+    def setUp(self):
+        super(WritableEndToEndTests, self).setUp()
+        self.setUpServer()
+
+    def test_start_tcp_server(self):
+        url = self.server.get_url()
+        self.assertContainsRe(url, r'^bzr://127\.0\.0\.1:[0-9]{2,}/')
+
+    def test_smart_transport_has(self):
+        """Checking for file existence over smart."""
+        self.backing_transport.put_bytes("foo", "contents of foo\n")
+        self.assertTrue(self.transport.has("foo"))
+        self.assertFalse(self.transport.has("non-foo"))
+
+    def test_smart_transport_get(self):
+        """Read back a file over smart."""
+        self.backing_transport.put_bytes("foo", "contents\nof\nfoo\n")
+        fp = self.transport.get("foo")
+        self.assertEqual('contents\nof\nfoo\n', fp.read())
+
+    def test_get_error_enoent(self):
+        """Error reported from server getting nonexistent file."""
+        # The path in a raised NoSuchFile exception should be the precise path
+        # asked for by the client. This gives meaningful and unsurprising errors
+        # for users.
+        try:
+            self.transport.get('not%20a%20file')
+        except errors.NoSuchFile, e:
+            self.assertEqual('not%20a%20file', e.path)
+        else:
+            self.fail("get did not raise expected error")
+
+    def test_simple_clone_conn(self):
+        """Test that cloning reuses the same connection."""
+        # we create a real connection not a loopback one, but it will use the
+        # same server and pipes
+        conn2 = self.transport.clone('.')
+        self.assertIs(self.transport._medium, conn2._medium)
+
+    def test__remote_path(self):
+        self.assertEquals('/foo/bar',
+                          self.transport._remote_path('foo/bar'))
+
+    def test_clone_changes_base(self):
+        """Cloning transport produces one with a new base location"""
+        conn2 = self.transport.clone('subdir')
+        self.assertEquals(self.transport.base + 'subdir/',
+                          conn2.base)
+
+    def test_open_dir(self):
+        """Test changing directory"""
+        transport = self.transport
+        self.backing_transport.mkdir('toffee')
+        self.backing_transport.mkdir('toffee/apple')
+        self.assertEquals('/toffee', transport._remote_path('toffee'))
+        toffee_trans = transport.clone('toffee')
+        # Check that each transport has only the contents of its directory
+        # directly visible. If state was being held in the wrong object, it's
+        # conceivable that cloning a transport would alter the state of the
+        # cloned-from transport.
+        self.assertTrue(transport.has('toffee'))
+        self.assertFalse(toffee_trans.has('toffee'))
+        self.assertFalse(transport.has('apple'))
+        self.assertTrue(toffee_trans.has('apple'))
+
+    def test_open_bzrdir(self):
+        """Open an existing bzrdir over smart transport"""
+        transport = self.transport
+        t = self.backing_transport
+        bzrdir.BzrDirFormat.get_default_format().initialize_on_transport(t)
+        result_dir = bzrdir.BzrDir.open_containing_from_transport(transport)
+
+
+class ReadOnlyEndToEndTests(SmartTCPTests):
+    """Tests from the client to the server using a readonly backing transport."""
+
+    def test_mkdir_error_readonly(self):
+        """TransportNotPossible should be preserved from the backing transport."""
+        self.setUpServer(readonly=True)
+        self.assertRaises(errors.TransportNotPossible, self.transport.mkdir,
+            'foo')
+        
+
+class SmartServerRequestHandlerTests(tests.TestCaseWithTransport):
+    """Test that call directly into the handler logic, bypassing the network."""
+
+    def test_construct_request_handler(self):
+        """Constructing a request handler should be easy and set defaults."""
+        handler = smart.SmartServerRequestHandler(None)
+        self.assertFalse(handler.finished_reading)
+
+    def test_hello(self):
+        handler = smart.SmartServerRequestHandler(None)
+        handler.dispatch_command('hello', ())
+        self.assertEqual(('ok', '1'), handler.response.args)
+        self.assertEqual(None, handler.response.body)
+        
+    def test_get_bundle(self):
+        from bzrlib.bundle import serializer
+        wt = self.make_branch_and_tree('.')
+        self.build_tree_contents([('hello', 'hello world')])
+        wt.add('hello')
+        rev_id = wt.commit('add hello')
+        
+        handler = smart.SmartServerRequestHandler(self.get_transport())
+        handler.dispatch_command('get_bundle', ('.', rev_id))
+        bundle = serializer.read_bundle(StringIO(handler.response.body))
+        self.assertEqual((), handler.response.args)
+
+    def test_readonly_exception_becomes_transport_not_possible(self):
+        """The response for a read-only error is ('ReadOnlyError')."""
+        handler = smart.SmartServerRequestHandler(self.get_readonly_transport())
+        # send a mkdir for foo, with no explicit mode - should fail.
+        handler.dispatch_command('mkdir', ('foo', ''))
+        # and the failure should be an explicit ReadOnlyError
+        self.assertEqual(("ReadOnlyError", ), handler.response.args)
+        # XXX: TODO: test that other TransportNotPossible errors are
+        # presented as TransportNotPossible - not possible to do that
+        # until I figure out how to trigger that relatively cleanly via
+        # the api. RBC 20060918
+
+    def test_hello_has_finished_body_on_dispatch(self):
+        """The 'hello' command should set finished_reading."""
+        handler = smart.SmartServerRequestHandler(None)
+        handler.dispatch_command('hello', ())
+        self.assertTrue(handler.finished_reading)
+        self.assertNotEqual(None, handler.response)
+
+    def test_put_bytes_non_atomic(self):
+        """'put_...' should set finished_reading after reading the bytes."""
+        handler = smart.SmartServerRequestHandler(self.get_transport())
+        handler.dispatch_command('put_non_atomic', ('a-file', '', 'F', ''))
+        self.assertFalse(handler.finished_reading)
+        handler.accept_body('1234')
+        self.assertFalse(handler.finished_reading)
+        handler.accept_body('5678')
+        handler.end_of_body()
+        self.assertTrue(handler.finished_reading)
+        self.assertEqual(('ok', ), handler.response.args)
+        self.assertEqual(None, handler.response.body)
+        
+    def test_readv_accept_body(self):
+        """'readv' should set finished_reading after reading offsets."""
+        self.build_tree(['a-file'])
+        handler = smart.SmartServerRequestHandler(self.get_readonly_transport())
+        handler.dispatch_command('readv', ('a-file', ))
+        self.assertFalse(handler.finished_reading)
+        handler.accept_body('2,')
+        self.assertFalse(handler.finished_reading)
+        handler.accept_body('3')
+        handler.end_of_body()
+        self.assertTrue(handler.finished_reading)
+        self.assertEqual(('readv', ), handler.response.args)
+        # co - nte - nt of a-file is the file contents we are extracting from.
+        self.assertEqual('nte', handler.response.body)
+
+    def test_readv_short_read_response_contents(self):
+        """'readv' when a short read occurs sets the response appropriately."""
+        self.build_tree(['a-file'])
+        handler = smart.SmartServerRequestHandler(self.get_readonly_transport())
+        handler.dispatch_command('readv', ('a-file', ))
+        # read beyond the end of the file.
+        handler.accept_body('100,1')
+        handler.end_of_body()
+        self.assertTrue(handler.finished_reading)
+        self.assertEqual(('ShortReadvError', 'a-file', '100', '1', '0'),
+            handler.response.args)
+        self.assertEqual(None, handler.response.body)
+
+
+class SmartTransportRegistration(tests.TestCase):
+
+    def test_registration(self):
+        t = get_transport('bzr+ssh://example.com/path')
+        self.assertIsInstance(t, smart.SmartSSHTransport)
+        self.assertEqual('example.com', t._host)
+
+
+class TestSmartTransport(tests.TestCase):
+        
+    def test_use_connection_factory(self):
+        # We want to be able to pass a client as a parameter to SmartTransport.
+        input = StringIO("ok\n3\nbardone\n")
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        transport = smart.SmartTransport('bzr://localhost/', medium=medium)
+
+        # We want to make sure the client is used when the first remote
+        # method is called.  No data should have been sent, or read.
+        self.assertEqual(0, input.tell())
+        self.assertEqual('', output.getvalue())
+
+        # Now call a method that should result in a single request : as the
+        # transport makes its own protocol instances, we check on the wire.
+        # XXX: TODO: give the transport a protocol factory, which can make
+        # an instrumented protocol for us.
+        self.assertEqual('bar', transport.get_bytes('foo'))
+        # only the needed data should have been sent/received.
+        self.assertEqual(13, input.tell())
+        self.assertEqual('get\x01/foo\n', output.getvalue())
+
+    def test__translate_error_readonly(self):
+        """Sending a ReadOnlyError to _translate_error raises TransportNotPossible."""
+        medium = smart.SmartClientMedium()
+        transport = smart.SmartTransport('bzr://localhost/', medium=medium)
+        self.assertRaises(errors.TransportNotPossible,
+            transport._translate_error, ("ReadOnlyError", ))
+
+
+class InstrumentedServerProtocol(smart.SmartServerStreamMedium):
+    """A smart server which is backed by memory and saves its write requests."""
+
+    def __init__(self, write_output_list):
+        smart.SmartServerStreamMedium.__init__(self, memory.MemoryTransport())
+        self._write_output_list = write_output_list
+
+
+class TestSmartProtocol(tests.TestCase):
+    """Tests for the smart protocol.
+
+    Each test case gets a smart_server and smart_client created during setUp().
+
+    It is planned that the client can be called with self.call_client() giving
+    it an expected server response, which will be fed into it when it tries to
+    read. Likewise, self.call_server will call a servers method with a canned
+    serialised client request. Output done by the client or server for these
+    calls will be captured to self.to_server and self.to_client. Each element
+    in the list is a write call from the client or server respectively.
+    """
+
+    def setUp(self):
+        super(TestSmartProtocol, self).setUp()
+        self.server_to_client = []
+        self.to_server = StringIO()
+        self.to_client = StringIO()
+        self.client_medium = smart.SmartSimplePipesClientMedium(self.to_client,
+            self.to_server)
+        self.client_protocol = smart.SmartClientRequestProtocolOne(
+            self.client_medium)
+        self.smart_server = InstrumentedServerProtocol(self.server_to_client)
+        self.smart_server_request = smart.SmartServerRequestHandler(None)
+
+    def assertOffsetSerialisation(self, expected_offsets, expected_serialised,
+        client, smart_server_request):
+        """Check that smart (de)serialises offsets as expected.
+        
+        We check both serialisation and deserialisation at the same time
+        to ensure that the round tripping cannot skew: both directions should
+        be as expected.
+        
+        :param expected_offsets: a readv offset list.
+        :param expected_seralised: an expected serial form of the offsets.
+        :param smart_server_request: a SmartServerRequestHandler instance.
+        """
+        # XXX: 'smart_server_request' should be a SmartServerRequestProtocol in
+        # future.
+        offsets = smart_server_request._deserialise_offsets(expected_serialised)
+        self.assertEqual(expected_offsets, offsets)
+        serialised = client._serialise_offsets(offsets)
+        self.assertEqual(expected_serialised, serialised)
+
+    def build_protocol_waiting_for_body(self):
+        out_stream = StringIO()
+        protocol = smart.SmartServerRequestProtocolOne(None, out_stream.write)
+        protocol.has_dispatched = True
+        protocol.request = smart.SmartServerRequestHandler(None)
+        def handle_end_of_bytes():
+            self.end_received = True
+            self.assertEqual('abcdefg', protocol.request._body_bytes)
+            protocol.request.response = smart.SmartServerResponse(('ok', ))
+        protocol.request._end_of_body_handler = handle_end_of_bytes
+        # Call accept_bytes to make sure that internal state like _body_decoder
+        # is initialised.  This test should probably be given a clearer
+        # interface to work with that will not cause this inconsistency.
+        #   -- Andrew Bennetts, 2006-09-28
+        protocol.accept_bytes('')
+        return protocol
+
+    def test_construct_version_one_server_protocol(self):
+        protocol = smart.SmartServerRequestProtocolOne(None, None)
+        self.assertEqual('', protocol.excess_buffer)
+        self.assertEqual('', protocol.in_buffer)
+        self.assertFalse(protocol.has_dispatched)
+        self.assertEqual(1, protocol.next_read_size())
+
+    def test_construct_version_one_client_protocol(self):
+        # we can construct a client protocol from a client medium request
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(None, output)
+        request = medium.get_request()
+        client_protocol = smart.SmartClientRequestProtocolOne(request)
+
+    def test_server_offset_serialisation(self):
+        """The Smart protocol serialises offsets as a comma and \n string.
+
+        We check a number of boundary cases are as expected: empty, one offset,
+        one with the order of reads not increasing (an out of order read), and
+        one that should coalesce.
+        """
+        self.assertOffsetSerialisation([], '',
+            self.client_protocol, self.smart_server_request)
+        self.assertOffsetSerialisation([(1,2)], '1,2',
+            self.client_protocol, self.smart_server_request)
+        self.assertOffsetSerialisation([(10,40), (0,5)], '10,40\n0,5',
+            self.client_protocol, self.smart_server_request)
+        self.assertOffsetSerialisation([(1,2), (3,4), (100, 200)],
+            '1,2\n3,4\n100,200', self.client_protocol, self.smart_server_request)
+
+    def test_accept_bytes_to_protocol(self):
+        out_stream = StringIO()
+        protocol = smart.SmartServerRequestProtocolOne(None, out_stream.write)
+        protocol.accept_bytes('abc')
+        self.assertEqual('abc', protocol.in_buffer)
+        protocol.accept_bytes('\n')
+        self.assertEqual("error\x01Generic bzr smart protocol error: bad request"
+            " u'abc'\n", out_stream.getvalue())
+        self.assertTrue(protocol.has_dispatched)
+        self.assertEqual(1, protocol.next_read_size())
+
+    def test_accept_bytes_with_invalid_utf8_to_protocol(self):
+        out_stream = StringIO()
+        protocol = smart.SmartServerRequestProtocolOne(None, out_stream.write)
+        # the byte 0xdd is not a valid UTF-8 string.
+        protocol.accept_bytes('\xdd\n')
+        self.assertEqual(
+            "error\x01Generic bzr smart protocol error: "
+            "one or more arguments of request '\\xdd\\n' are not valid UTF-8\n",
+            out_stream.getvalue())
+        self.assertTrue(protocol.has_dispatched)
+        self.assertEqual(1, protocol.next_read_size())
+
+    def test_accept_body_bytes_to_protocol(self):
+        protocol = self.build_protocol_waiting_for_body()
+        self.assertEqual(6, protocol.next_read_size())
+        protocol.accept_bytes('7\nabc')
+        self.assertEqual(9, protocol.next_read_size())
+        protocol.accept_bytes('defgd')
+        protocol.accept_bytes('one\n')
+        self.assertEqual(0, protocol.next_read_size())
+        self.assertTrue(self.end_received)
+
+    def test_accept_request_and_body_all_at_once(self):
+        mem_transport = memory.MemoryTransport()
+        mem_transport.put_bytes('foo', 'abcdefghij')
+        out_stream = StringIO()
+        protocol = smart.SmartServerRequestProtocolOne(mem_transport,
+                out_stream.write)
+        protocol.accept_bytes('readv\x01foo\n3\n3,3done\n')
+        self.assertEqual(0, protocol.next_read_size())
+        self.assertEqual('readv\n3\ndefdone\n', out_stream.getvalue())
+        self.assertEqual('', protocol.excess_buffer)
+        self.assertEqual('', protocol.in_buffer)
+
+    def test_accept_excess_bytes_are_preserved(self):
+        out_stream = StringIO()
+        protocol = smart.SmartServerRequestProtocolOne(None, out_stream.write)
+        protocol.accept_bytes('hello\nhello\n')
+        self.assertEqual("ok\x011\n", out_stream.getvalue())
+        self.assertEqual("hello\n", protocol.excess_buffer)
+        self.assertEqual("", protocol.in_buffer)
+
+    def test_accept_excess_bytes_after_body(self):
+        protocol = self.build_protocol_waiting_for_body()
+        protocol.accept_bytes('7\nabcdefgdone\nX')
+        self.assertTrue(self.end_received)
+        self.assertEqual("X", protocol.excess_buffer)
+        self.assertEqual("", protocol.in_buffer)
+        protocol.accept_bytes('Y')
+        self.assertEqual("XY", protocol.excess_buffer)
+        self.assertEqual("", protocol.in_buffer)
+
+    def test_accept_excess_bytes_after_dispatch(self):
+        out_stream = StringIO()
+        protocol = smart.SmartServerRequestProtocolOne(None, out_stream.write)
+        protocol.accept_bytes('hello\n')
+        self.assertEqual("ok\x011\n", out_stream.getvalue())
+        protocol.accept_bytes('hel')
+        self.assertEqual("hel", protocol.excess_buffer)
+        protocol.accept_bytes('lo\n')
+        self.assertEqual("hello\n", protocol.excess_buffer)
+        self.assertEqual("", protocol.in_buffer)
+
+    def test_sync_with_request_sets_finished_reading(self):
+        protocol = smart.SmartServerRequestProtocolOne(None, None)
+        request = smart.SmartServerRequestHandler(None)
+        protocol.sync_with_request(request)
+        self.assertEqual(1, protocol.next_read_size())
+        request.finished_reading = True
+        protocol.sync_with_request(request)
+        self.assertEqual(0, protocol.next_read_size())
+
+    def test_query_version(self):
+        """query_version on a SmartClientProtocolOne should return a number.
+        
+        The protocol provides the query_version because the domain level clients
+        may all need to be able to probe for capabilities.
+        """
+        # What we really want to test here is that SmartClientProtocolOne calls
+        # accept_bytes(tuple_based_encoding_of_hello) and reads and parses the
+        # response of tuple-encoded (ok, 1).  Also, seperately we should test
+        # the error if the response is a non-understood version.
+        input = StringIO('ok\x011\n')
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        protocol = smart.SmartClientRequestProtocolOne(medium.get_request())
+        self.assertEqual(1, protocol.query_version())
+
+    def assertServerToClientEncoding(self, expected_bytes, expected_tuple,
+            input_tuples):
+        """Assert that each input_tuple serialises as expected_bytes, and the
+        bytes deserialise as expected_tuple.
+        """
+        # check the encoding of the server for all input_tuples matches
+        # expected bytes
+        for input_tuple in input_tuples:
+            server_output = StringIO()
+            server_protocol = smart.SmartServerRequestProtocolOne(
+                None, server_output.write)
+            server_protocol._send_response(input_tuple)
+            self.assertEqual(expected_bytes, server_output.getvalue())
+        # check the decoding of the client protocol from expected_bytes:
+        input = StringIO(expected_bytes)
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        protocol = smart.SmartClientRequestProtocolOne(medium.get_request())
+        protocol.call('foo')
+        self.assertEqual(expected_tuple, protocol.read_response_tuple())
+
+    def test_client_call_empty_response(self):
+        # protocol.call() can get back an empty tuple as a response. This occurs
+        # when the parsed line is an empty line, and results in a tuple with
+        # one element - an empty string.
+        self.assertServerToClientEncoding('\n', ('', ), [(), ('', )])
+
+    def test_client_call_three_element_response(self):
+        # protocol.call() can get back tuples of other lengths. A three element
+        # tuple should be unpacked as three strings.
+        self.assertServerToClientEncoding('a\x01b\x0134\n', ('a', 'b', '34'),
+            [('a', 'b', '34')])
+
+    def test_client_call_with_body_bytes_uploads(self):
+        # protocol.call_with_upload should length-prefix the bytes onto the 
+        # wire.
+        expected_bytes = "foo\n7\nabcdefgdone\n"
+        input = StringIO("\n")
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        protocol = smart.SmartClientRequestProtocolOne(medium.get_request())
+        protocol.call_with_body_bytes(('foo', ), "abcdefg")
+        self.assertEqual(expected_bytes, output.getvalue())
+
+    def test_client_call_with_body_readv_array(self):
+        # protocol.call_with_upload should encode the readv array and then
+        # length-prefix the bytes onto the wire.
+        expected_bytes = "foo\n7\n1,2\n5,6done\n"
+        input = StringIO("\n")
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        protocol = smart.SmartClientRequestProtocolOne(medium.get_request())
+        protocol.call_with_body_readv_array(('foo', ), [(1,2),(5,6)])
+        self.assertEqual(expected_bytes, output.getvalue())
+
+    def test_client_read_body_bytes_all(self):
+        # read_body_bytes should decode the body bytes from the wire into
+        # a response.
+        expected_bytes = "1234567"
+        server_bytes = "ok\n7\n1234567done\n"
+        input = StringIO(server_bytes)
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        protocol = smart.SmartClientRequestProtocolOne(medium.get_request())
+        protocol.call('foo')
+        protocol.read_response_tuple(True)
+        self.assertEqual(expected_bytes, protocol.read_body_bytes())
+
+    def test_client_read_body_bytes_incremental(self):
+        # test reading a few bytes at a time from the body
+        # XXX: possibly we should test dribbling the bytes into the stringio
+        # to make the state machine work harder: however, as we use the
+        # LengthPrefixedBodyDecoder that is already well tested - we can skip
+        # that.
+        expected_bytes = "1234567"
+        server_bytes = "ok\n7\n1234567done\n"
+        input = StringIO(server_bytes)
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        protocol = smart.SmartClientRequestProtocolOne(medium.get_request())
+        protocol.call('foo')
+        protocol.read_response_tuple(True)
+        self.assertEqual(expected_bytes[0:2], protocol.read_body_bytes(2))
+        self.assertEqual(expected_bytes[2:4], protocol.read_body_bytes(2))
+        self.assertEqual(expected_bytes[4:6], protocol.read_body_bytes(2))
+        self.assertEqual(expected_bytes[6], protocol.read_body_bytes())
+
+    def test_client_cancel_read_body_does_not_eat_body_bytes(self):
+        # cancelling the expected body needs to finish the request, but not
+        # read any more bytes.
+        expected_bytes = "1234567"
+        server_bytes = "ok\n7\n1234567done\n"
+        input = StringIO(server_bytes)
+        output = StringIO()
+        medium = smart.SmartSimplePipesClientMedium(input, output)
+        protocol = smart.SmartClientRequestProtocolOne(medium.get_request())
+        protocol.call('foo')
+        protocol.read_response_tuple(True)
+        protocol.cancel_read_body()
+        self.assertEqual(3, input.tell())
+        self.assertRaises(errors.ReadingCompleted, protocol.read_body_bytes)
+
+
+class LengthPrefixedBodyDecoder(tests.TestCase):
+
+    # XXX: TODO: make accept_reading_trailer invoke translate_response or 
+    # something similar to the ProtocolBase method.
+
+    def test_construct(self):
+        decoder = smart.LengthPrefixedBodyDecoder()
+        self.assertFalse(decoder.finished_reading)
+        self.assertEqual(6, decoder.next_read_size())
+        self.assertEqual('', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+
+    def test_accept_bytes(self):
+        decoder = smart.LengthPrefixedBodyDecoder()
+        decoder.accept_bytes('')
+        self.assertFalse(decoder.finished_reading)
+        self.assertEqual(6, decoder.next_read_size())
+        self.assertEqual('', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+        decoder.accept_bytes('7')
+        self.assertFalse(decoder.finished_reading)
+        self.assertEqual(6, decoder.next_read_size())
+        self.assertEqual('', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+        decoder.accept_bytes('\na')
+        self.assertFalse(decoder.finished_reading)
+        self.assertEqual(11, decoder.next_read_size())
+        self.assertEqual('a', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+        decoder.accept_bytes('bcdefgd')
+        self.assertFalse(decoder.finished_reading)
+        self.assertEqual(4, decoder.next_read_size())
+        self.assertEqual('bcdefg', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+        decoder.accept_bytes('one')
+        self.assertFalse(decoder.finished_reading)
+        self.assertEqual(1, decoder.next_read_size())
+        self.assertEqual('', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+        decoder.accept_bytes('\nblarg')
+        self.assertTrue(decoder.finished_reading)
+        self.assertEqual(1, decoder.next_read_size())
+        self.assertEqual('', decoder.read_pending_data())
+        self.assertEqual('blarg', decoder.unused_data)
+        
+    def test_accept_bytes_all_at_once_with_excess(self):
+        decoder = smart.LengthPrefixedBodyDecoder()
+        decoder.accept_bytes('1\nadone\nunused')
+        self.assertTrue(decoder.finished_reading)
+        self.assertEqual(1, decoder.next_read_size())
+        self.assertEqual('a', decoder.read_pending_data())
+        self.assertEqual('unused', decoder.unused_data)
+
+    def test_accept_bytes_exact_end_of_body(self):
+        decoder = smart.LengthPrefixedBodyDecoder()
+        decoder.accept_bytes('1\na')
+        self.assertFalse(decoder.finished_reading)
+        self.assertEqual(5, decoder.next_read_size())
+        self.assertEqual('a', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+        decoder.accept_bytes('done\n')
+        self.assertTrue(decoder.finished_reading)
+        self.assertEqual(1, decoder.next_read_size())
+        self.assertEqual('', decoder.read_pending_data())
+        self.assertEqual('', decoder.unused_data)
+
+
+class FakeHTTPMedium(object):
+    def __init__(self):
+        self.written_request = None
+        self._current_request = None
+    def send_http_smart_request(self, bytes):
+        self.written_request = bytes
+        return None
+
+
+class HTTPTunnellingSmokeTest(tests.TestCaseWithTransport):
+    
+    def _test_bulk_data(self, url_protocol):
+        # We should be able to send and receive bulk data in a single message.
+        # The 'readv' command in the smart protocol both sends and receives bulk
+        # data, so we use that.
+        self.build_tree(['data-file'])
+        http_server = HTTPServerWithSmarts()
+        http_server._url_protocol = url_protocol
+        http_server.setUp()
+        self.addCleanup(http_server.tearDown)
+
+        http_transport = get_transport(http_server.get_url())
+
+        medium = http_transport.get_smart_medium()
+        #remote_transport = RemoteTransport('fake_url', medium)
+        remote_transport = smart.SmartTransport('/', medium=medium)
+        self.assertEqual(
+            [(0, "c")], list(remote_transport.readv("data-file", [(0,1)])))
+
+    def test_bulk_data_pycurl(self):
+        try:
+            self._test_bulk_data('http+pycurl')
+        except errors.UnsupportedProtocol, e:
+            raise tests.TestSkipped(str(e))
+    
+    def test_bulk_data_urllib(self):
+        self._test_bulk_data('http+urllib')
+
+    def test_smart_http_medium_request_accept_bytes(self):
+        medium = FakeHTTPMedium()
+        request = SmartClientHTTPMediumRequest(medium)
+        request.accept_bytes('abc')
+        request.accept_bytes('def')
+        self.assertEqual(None, medium.written_request)
+        request.finished_writing()
+        self.assertEqual('abcdef', medium.written_request)
+
+    def _test_http_send_smart_request(self, url_protocol):
+        http_server = HTTPServerWithSmarts()
+        http_server._url_protocol = url_protocol
+        http_server.setUp()
+        self.addCleanup(http_server.tearDown)
+
+        post_body = 'hello\n'
+        expected_reply_body = 'ok\x011\n'
+
+        http_transport = get_transport(http_server.get_url())
+        medium = http_transport.get_smart_medium()
+        response = medium.send_http_smart_request(post_body)
+        reply_body = response.read()
+        self.assertEqual(expected_reply_body, reply_body)
+
+    def test_http_send_smart_request_pycurl(self):
+        try:
+            self._test_http_send_smart_request('http+pycurl')
+        except errors.UnsupportedProtocol, e:
+            raise tests.TestSkipped(str(e))
+
+    def test_http_send_smart_request_urllib(self):
+        self._test_http_send_smart_request('http+urllib')
+
+    def test_http_server_with_smarts(self):
+        http_server = HTTPServerWithSmarts()
+        http_server.setUp()
+        self.addCleanup(http_server.tearDown)
+
+        post_body = 'hello\n'
+        expected_reply_body = 'ok\x011\n'
+
+        smart_server_url = http_server.get_url() + '.bzr/smart'
+        reply = urllib2.urlopen(smart_server_url, post_body).read()
+
+        self.assertEqual(expected_reply_body, reply)
+
+    def test_smart_http_server_post_request_handler(self):
+        http_server = HTTPServerWithSmarts()
+        http_server.setUp()
+        self.addCleanup(http_server.tearDown)
+        httpd = http_server._get_httpd()
+
+        socket = SampleSocket(
+            'POST /.bzr/smart HTTP/1.0\r\n'
+            # HTTP/1.0 posts must have a Content-Length.
+            'Content-Length: 6\r\n'
+            '\r\n'
+            'hello\n')
+        request_handler = SmartRequestHandler(
+            socket, ('localhost', 80), httpd)
+        response = socket.writefile.getvalue()
+        self.assertStartsWith(response, 'HTTP/1.0 200 ')
+        # This includes the end of the HTTP headers, and all the body.
+        expected_end_of_response = '\r\n\r\nok\x011\n'
+        self.assertEndsWith(response, expected_end_of_response)
+
+
+class SampleSocket(object):
+    """A socket-like object for use in testing the HTTP request handler."""
+    
+    def __init__(self, socket_read_content):
+        """Constructs a sample socket.
+
+        :param socket_read_content: a byte sequence
+        """
+        # Use plain python StringIO so we can monkey-patch the close method to
+        # not discard the contents.
+        from StringIO import StringIO
+        self.readfile = StringIO(socket_read_content)
+        self.writefile = StringIO()
+        self.writefile.close = lambda: None
+        
+    def makefile(self, mode='r', bufsize=None):
+        if 'r' in mode:
+            return self.readfile
+        else:
+            return self.writefile
+
+        
+# TODO: Client feature that does get_bundle and then installs that into a
+# branch; this can be used in place of the regular pull/fetch operation when
+# coming from a smart server.
+#
+# TODO: Eventually, want to do a 'branch' command by fetching the whole
+# history as one big bundle.  How?  
+#
+# The branch command does 'br_from.sprout', which tries to preserve the same
+# format.  We don't necessarily even want that.  
+#
+# It might be simpler to handle cmd_pull first, which does a simpler fetch()
+# operation from one branch into another.  It already has some code for
+# pulling from a bundle, which it does by trying to see if the destination is
+# a bundle file.  So it seems the logic for pull ought to be:
+# 
+#  - if it's a smart server, get a bundle from there and install that
+#  - if it's a bundle, install that
+#  - if it's a branch, pull from there
+#
+# Getting a bundle from a smart server is a bit different from reading a
+# bundle from a URL:
+#
+#  - we can reasonably remember the URL we last read from 
+#  - you can specify a revision number to pull, and we need to pass it across
+#    to the server as a limit on what will be requested
+#
+# TODO: Given a URL, determine whether it is a smart server or not (or perhaps
+# otherwise whether it's a bundle?)  Should this be a property or method of
+# the transport?  For the ssh protocol, we always know it's a smart server.
+# For http, we potentially need to probe.  But if we're explicitly given
+# bzr+http:// then we can skip that for now. 

=== modified file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	2006-08-16 16:55:54 +0000
+++ bzrlib/tests/test_testament.py	2006-09-28 01:57:43 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005-2006 by Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 from sha import sha
 
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.testament import Testament, StrictTestament
+from bzrlib.testament import Testament, StrictTestament, StrictTestament3
 from bzrlib.transform import TreeTransform
 from bzrlib.osutils import has_symlinks
 
@@ -32,6 +32,7 @@
     def setUp(self):
         super(TestamentSetup, self).setUp()
         self.wt = self.make_branch_and_tree('.')
+        self.wt.set_root_id('TREE_ROT')
         b = self.b = self.wt.branch
         b.nick = "test branch"
         self.wt.commit(message='initial null commit',
@@ -57,9 +58,18 @@
 
 class TestamentTests(TestamentSetup):
 
+    def testament_class(self):
+        return Testament
+
+    def expected(self, key):
+        return texts[self.testament_class()][key]
+
+    def from_revision(self, repository, revision_id):
+        return self.testament_class().from_revision(repository, revision_id)
+
     def test_null_testament(self):
         """Testament for a revision with no contents."""
-        t = Testament.from_revision(self.b.repository, 'test@user-1')
+        t = self.from_revision(self.b.repository, 'test@user-1')
         ass = self.assertTrue
         eq = self.assertEqual
         ass(isinstance(t, Testament))
@@ -70,35 +80,21 @@
 
     def test_testment_text_form(self):
         """Conversion of testament to canonical text form."""
-        t = Testament.from_revision(self.b.repository, 'test@user-1')
-        text_form = t.as_text()
-        self.log('testament text form:\n' + text_form)
-        self.assertEqual(text_form, REV_1_TESTAMENT)
-
-    def test_strict_testment_text_form(self):
-        """Conversion of testament to canonical text form."""
-        t = StrictTestament.from_revision(self.b.repository, 'test@user-1')
-        text_form = t.as_text()
-        self.log('testament text form:\n' + text_form)
-        self.assertEqualDiff(text_form, REV_1_STRICT_TESTAMENT)
+        t = self.from_revision(self.b.repository, 'test@user-1')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqualDiff(text_form, self.expected('rev_1'))
+        short_text_form = t.as_short_text()
+        self.assertEqualDiff(short_text_form, self.expected('rev_1_short'))
 
     def test_testament_with_contents(self):
         """Testament containing a file and a directory."""
-        t = Testament.from_revision(self.b.repository, 'test@user-2')
-        text_form = t.as_text()
-        self.log('testament text form:\n' + text_form)
-        self.assertEqualDiff(text_form, REV_2_TESTAMENT)
-        actual_short = t.as_short_text()
-        self.assertEqualDiff(actual_short, REV_2_SHORT)
-
-    def test_strict_testament_with_contents(self):
-        """Testament containing a file and a directory."""
-        t = StrictTestament.from_revision(self.b.repository, 'test@user-2')
-        text_form = t.as_text()
-        self.log('testament text form:\n' + text_form)
-        self.assertEqualDiff(text_form, REV_2_STRICT_TESTAMENT)
-        actual_short = t.as_short_text()
-        self.assertEqualDiff(actual_short, REV_2_SHORT_STRICT)
+        t = self.from_revision(self.b.repository, 'test@user-2')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqualDiff(text_form, self.expected('rev_2'))
+        actual_short = t.as_short_text()
+        self.assertEqualDiff(actual_short, self.expected('rev_2_short'))
 
     def test_testament_symlinks(self):
         """Testament containing symlink (where possible)"""
@@ -111,8 +107,8 @@
                  timezone=36000,
                  rev_id='test@user-3',
                  committer='test@user')
-        t = Testament.from_revision(self.b.repository, 'test@user-3')
-        self.assertEqualDiff(t.as_text(), REV_3_TESTAMENT)
+        t = self.from_revision(self.b.repository, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), self.expected('rev_3'))
 
     def test_testament_revprops(self):
         """Testament to revision with extra properties"""
@@ -126,8 +122,8 @@
                       rev_id='test@user-3',
                       committer='test@user',
                       revprops=props)
-        t = Testament.from_revision(self.b.repository, 'test@user-3')
-        self.assertEqualDiff(t.as_text(), REV_PROPS_TESTAMENT)
+        t = self.from_revision(self.b.repository, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), self.expected('rev_props'))
 
     def test_testament_unicode_commit_message(self):
         self.wt.commit(
@@ -138,19 +134,32 @@
             committer='Erik B\xe5gfors <test@user>',
             revprops={'uni':u'\xb5'}
             )
-        t = Testament.from_revision(self.b.repository, 'test@user-3')
+        t = self.from_revision(self.b.repository, 'test@user-3')
         self.assertEqualDiff(
-            SAMPLE_UNICODE_TESTAMENT.encode('utf-8'), t.as_text())
+            self.expected('sample_unicode').encode('utf-8'), t.as_text())
 
     def test___init__(self):
         revision = self.b.repository.get_revision('test@user-2')
         inventory = self.b.repository.get_inventory('test@user-2')
-        testament_1 = Testament(revision, inventory).as_short_text()
-        testament_2 = Testament.from_revision(self.b.repository, 
-                                              'test@user-2').as_short_text()
-        self.assertEqual(testament_1, testament_2)
+        testament_1 = self.testament_class()(revision, inventory)
+        text_1 = testament_1.as_short_text()
+        text_2 = self.from_revision(self.b.repository, 
+                                    'test@user-2').as_short_text()
+        self.assertEqual(text_1, text_2)
                     
 
+class TestamentTestsStrict(TestamentTests):
+    
+    def testament_class(self):
+        return StrictTestament
+
+
+class TestamentTestsStrict2(TestamentTests):
+    
+    def testament_class(self):
+        return StrictTestament3
+
+
 REV_1_TESTAMENT = """\
 bazaar-ng testament version 1
 revision-id: test@user-1
@@ -166,6 +175,7 @@
     test branch
 """
 
+
 REV_1_STRICT_TESTAMENT = """\
 bazaar-ng testament version 2.1
 revision-id: test@user-1
@@ -182,6 +192,23 @@
 """
 
 
+REV_1_STRICT_TESTAMENT3 = """\
+bazaar testament version 3 strict
+revision-id: test@user-1
+committer: test@user
+timestamp: 1129025423
+timezone: 0
+parents:
+message:
+  initial null commit
+inventory:
+  directory . TREE_ROT test@user-1 no
+properties:
+  branch-nick:
+    test branch
+"""
+
+
 REV_1_SHORT = """\
 bazaar-ng testament short form 1
 revision-id: test@user-1
@@ -196,6 +223,13 @@
 """ % sha(REV_1_STRICT_TESTAMENT).hexdigest()
 
 
+REV_1_SHORT_STRICT3 = """\
+bazaar testament short form 3 strict
+revision-id: test@user-1
+sha1: %s
+""" % sha(REV_1_STRICT_TESTAMENT3).hexdigest()
+
+
 REV_2_TESTAMENT = """\
 bazaar-ng testament version 1
 revision-id: test@user-2
@@ -236,6 +270,27 @@
 """
 
 
+REV_2_STRICT_TESTAMENT3 = """\
+bazaar testament version 3 strict
+revision-id: test@user-2
+committer: test@user
+timestamp: 1129025483
+timezone: 36000
+parents:
+  test@user-1
+message:
+  add files and directories
+inventory:
+  directory . TREE_ROT test@user-2 no
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
+  directory src src-id test@user-2 no
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
+properties:
+  branch-nick:
+    test branch
+"""
+
+
 REV_2_SHORT = """\
 bazaar-ng testament short form 1
 revision-id: test@user-2
@@ -250,6 +305,13 @@
 """ % sha(REV_2_STRICT_TESTAMENT).hexdigest()
 
 
+REV_2_SHORT_STRICT3 = """\
+bazaar testament short form 3 strict
+revision-id: test@user-2
+sha1: %s
+""" % sha(REV_2_STRICT_TESTAMENT3).hexdigest()
+
+
 REV_PROPS_TESTAMENT = """\
 bazaar-ng testament version 1
 revision-id: test@user-3
@@ -276,6 +338,59 @@
 """
 
 
+REV_PROPS_TESTAMENT_STRICT = """\
+bazaar-ng testament version 2.1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  revision with properties
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
+  directory src src-id test@user-2 no
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
+properties:
+  branch-nick:
+    test branch
+  empty:
+  flavor:
+    sour cherry
+    cream cheese
+  size:
+    medium
+"""
+
+
+REV_PROPS_TESTAMENT_STRICT3 = """\
+bazaar testament version 3 strict
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  revision with properties
+inventory:
+  directory . TREE_ROT test@user-3 no
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
+  directory src src-id test@user-2 no
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
+properties:
+  branch-nick:
+    test branch
+  empty:
+  flavor:
+    sour cherry
+    cream cheese
+  size:
+    medium
+"""
+
+
 REV_3_TESTAMENT = """\
 bazaar-ng testament version 1
 revision-id: test@user-3
@@ -297,6 +412,49 @@
 """
 
 
+REV_3_TESTAMENT_STRICT = """\
+bazaar-ng testament version 2.1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  add symlink
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
+  symlink link link-id wibble/linktarget test@user-3 no
+  directory src src-id test@user-2 no
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
+properties:
+  branch-nick:
+    test branch
+"""
+
+
+REV_3_TESTAMENT_STRICT3 = """\
+bazaar testament version 3 strict
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  add symlink
+inventory:
+  directory . TREE_ROT test@user-3 no
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
+  symlink link link-id wibble/linktarget test@user-3 no
+  directory src src-id test@user-2 no
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
+properties:
+  branch-nick:
+    test branch
+"""
+
+
 SAMPLE_UNICODE_TESTAMENT = u"""\
 bazaar-ng testament version 1
 revision-id: test@user-3
@@ -317,3 +475,76 @@
   uni:
     \xb5
 """
+
+
+SAMPLE_UNICODE_TESTAMENT_STRICT = u"""\
+bazaar-ng testament version 2.1
+revision-id: test@user-3
+committer: Erik B\xe5gfors <test@user>
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  non-ascii commit \N{COPYRIGHT SIGN} me
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
+  directory src src-id test@user-2 no
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
+properties:
+  branch-nick:
+    test branch
+  uni:
+    \xb5
+"""
+
+
+SAMPLE_UNICODE_TESTAMENT_STRICT3 = u"""\
+bazaar testament version 3 strict
+revision-id: test@user-3
+committer: Erik B\xe5gfors <test@user>
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  non-ascii commit \N{COPYRIGHT SIGN} me
+inventory:
+  directory . TREE_ROT test@user-3 no
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
+  directory src src-id test@user-2 no
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
+properties:
+  branch-nick:
+    test branch
+  uni:
+    \xb5
+"""
+
+
+texts = {
+    Testament: { 'rev_1': REV_1_TESTAMENT,
+                 'rev_1_short': REV_1_SHORT,
+                 'rev_2': REV_2_TESTAMENT,
+                 'rev_2_short': REV_2_SHORT,
+                 'rev_3': REV_3_TESTAMENT,
+                 'rev_props': REV_PROPS_TESTAMENT,
+                 'sample_unicode': SAMPLE_UNICODE_TESTAMENT,
+    },
+    StrictTestament: {'rev_1': REV_1_STRICT_TESTAMENT,
+                      'rev_1_short': REV_1_SHORT_STRICT,
+                      'rev_2': REV_2_STRICT_TESTAMENT,
+                      'rev_2_short': REV_2_SHORT_STRICT,
+                      'rev_3': REV_3_TESTAMENT_STRICT,
+                      'rev_props': REV_PROPS_TESTAMENT_STRICT,
+                      'sample_unicode': SAMPLE_UNICODE_TESTAMENT_STRICT,
+    },
+    StrictTestament3: {'rev_1': REV_1_STRICT_TESTAMENT3,
+                      'rev_1_short': REV_1_SHORT_STRICT3,
+                      'rev_2': REV_2_STRICT_TESTAMENT3,
+                      'rev_2_short': REV_2_SHORT_STRICT3,
+                      'rev_3': REV_3_TESTAMENT_STRICT3,
+                      'rev_props': REV_PROPS_TESTAMENT_STRICT3,
+                      'sample_unicode': SAMPLE_UNICODE_TESTAMENT_STRICT3,
+    },
+}

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2006-08-22 21:39:33 +0000
+++ bzrlib/tests/test_trace.py	2006-09-06 21:17:22 +0000
@@ -87,22 +87,22 @@
 
     def test_trace_unicode(self):
         """Write Unicode to trace log"""
-        mutter(u'the unicode character for benzene is \N{BENZENE RING}')
-        self._log_file.flush()
-        self.assertContainsRe(self._get_log(), 'the unicode character',)
+        self.log(u'the unicode character for benzene is \N{BENZENE RING}')
+        self.assertContainsRe(self._get_log(keep_log_file=True),
+                              "the unicode character for benzene is")
     
     def test_trace_argument_unicode(self):
         """Write a Unicode argument to the trace log"""
         mutter(u'the unicode character for benzene is %s', u'\N{BENZENE RING}')
-        self._log_file.flush()
-        self.assertContainsRe(self._get_log(), 'the unicode character')
+        self.assertContainsRe(self._get_log(keep_log_file=True),
+                              'the unicode character')
 
     def test_trace_argument_utf8(self):
         """Write a Unicode argument to the trace log"""
         mutter(u'the unicode character for benzene is %s',
                u'\N{BENZENE RING}'.encode('utf-8'))
-        self._log_file.flush()
-        self.assertContainsRe(self._get_log(), 'the unicode character')
+        self.assertContainsRe(self._get_log(keep_log_file=True),
+                              'the unicode character')
 
     def test_report_broken_pipe(self):
         try:
@@ -119,11 +119,7 @@
         mutter(u'Writing a greek mu (\xb5) works in a unicode string')
         mutter('But fails in an ascii string \xb5')
         mutter('and in an ascii argument: %s', '\xb5')
-        # TODO: jam 20051227 mutter() doesn't flush the log file, and
-        #       self._get_log() opens the file directly and reads it.
-        #       So we need to manually flush the log file
-        self._log_file.flush()
-        log = self._get_log()
+        log = self._get_log(keep_log_file=True)
         self.assertContainsRe(log, 'Writing a greek mu')
         self.assertContainsRe(log, "But fails in an ascii string")
         self.assertContainsRe(log, u"ascii argument: \xb5")

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-09-06 12:00:01 +0000
+++ bzrlib/tests/test_transform.py	2006-09-27 02:28:44 +0000
@@ -15,8 +15,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import stat
+import sys
 
-from bzrlib import tests
+from bzrlib import (
+    tests,
+    urlutils,
+    )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.conflicts import (DuplicateEntry, DuplicateID, MissingParent,
                               UnversionedParent, ParentLoop, DeletingParent,)
@@ -30,7 +35,8 @@
 from bzrlib.transform import (TreeTransform, ROOT_PARENT, FinalPaths, 
                               resolve_conflicts, cook_conflicts, 
                               find_interesting, build_tree, get_backup_name)
-import bzrlib.urlutils as urlutils
+from bzrlib.workingtree import gen_root_id
+
 
 class TestTreeTransform(TestCaseInTempDir):
 
@@ -42,7 +48,7 @@
     def get_transform(self):
         transform = TreeTransform(self.wt)
         #self.addCleanup(transform.finalize)
-        return transform, transform.trans_id_tree_file_id(self.wt.get_root_id())
+        return transform, transform.root
 
     def test_existing_limbo(self):
         limbo_name = urlutils.local_path_from_url(
@@ -192,7 +198,7 @@
         transform3.delete_contents(oz_id)
         self.assertEqual(transform3.find_conflicts(), 
                          [('missing parent', oz_id)])
-        root_id = transform3.trans_id_tree_file_id('TREE_ROOT')
+        root_id = transform3.root
         tip_id = transform3.trans_id_tree_file_id('tip-id')
         transform3.adjust_path('tip', root_id, tip_id)
         transform3.apply()
@@ -224,7 +230,7 @@
     def test_name_invariants(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.new_file('name2', root, 'hello2', 'name2')
         ddir = create_tree.new_directory('dying_directory', root, 'ddir')
@@ -234,7 +240,7 @@
         create_tree.apply()
 
         mangle_tree,root = self.get_transform()
-        root = mangle_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = mangle_tree.root
         #swap names
         name1 = mangle_tree.trans_id_tree_file_id('name1')
         name2 = mangle_tree.trans_id_tree_file_id('name2')
@@ -319,7 +325,7 @@
     def test_move_dangling_ie(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.apply()
         delete_contents, root = self.get_transform()
@@ -335,7 +341,7 @@
     def test_replace_dangling_ie(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.apply()
         delete_contents = TreeTransform(self.wt)
@@ -535,12 +541,58 @@
         self.assertTrue(wt.is_executable('soc'))
         self.assertTrue(wt.is_executable('sac'))
 
+    def test_preserve_mode(self):
+        """File mode is preserved when replacing content"""
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        transform, root = self.get_transform()
+        transform.new_file('file1', root, 'contents', 'file1-id', True)
+        transform.apply()
+        self.assertTrue(self.wt.is_executable('file1-id'))
+        transform, root = self.get_transform()
+        file1_id = transform.trans_id_tree_file_id('file1-id')
+        transform.delete_contents(file1_id)
+        transform.create_file('contents2', file1_id)
+        transform.apply()
+        self.assertTrue(self.wt.is_executable('file1-id'))
+
+    def test__set_mode_stats_correctly(self):
+        """_set_mode stats to determine file mode."""
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        stat_paths = []
+        real_stat = os.stat
+        def instrumented_stat(path):
+            stat_paths.append(path)
+            return real_stat(path)
+
+        transform, root = self.get_transform()
+
+        bar1_id = transform.new_file('bar', root, 'bar contents 1\n',
+                                     file_id='bar-id-1', executable=False)
+        transform.apply()
+
+        transform, root = self.get_transform()
+        bar1_id = transform.trans_id_tree_path('bar')
+        bar2_id = transform.trans_id_tree_path('bar2')
+        try:
+            os.stat = instrumented_stat
+            transform.create_file('bar2 contents\n', bar2_id, mode_id=bar1_id)
+        finally:
+            os.stat = real_stat
+            transform.finalize()
+
+        bar1_abspath = self.wt.abspath('bar')
+        self.assertEqual([bar1_abspath], stat_paths)
+
 
 class TransformGroup(object):
-    def __init__(self, dirname):
+    def __init__(self, dirname, root_id):
         self.name = dirname
         os.mkdir(dirname)
         self.wt = BzrDir.create_standalone_workingtree(dirname)
+        self.wt.set_root_id(root_id)
         self.b = self.wt.branch
         self.tt = TreeTransform(self.wt)
         self.root = self.tt.trans_id_tree_file_id(self.wt.get_root_id())
@@ -552,7 +604,8 @@
 
 class TestTransformMerge(TestCaseInTempDir):
     def test_text_merge(self):
-        base = TransformGroup("base")
+        root_id = gen_root_id()
+        base = TransformGroup("base", root_id)
         base.tt.new_file('a', base.root, 'a\nb\nc\nd\be\n', 'a')
         base.tt.new_file('b', base.root, 'b1', 'b')
         base.tt.new_file('c', base.root, 'c', 'c')
@@ -562,7 +615,7 @@
         base.tt.new_directory('g', base.root, 'g')
         base.tt.new_directory('h', base.root, 'h')
         base.tt.apply()
-        other = TransformGroup("other")
+        other = TransformGroup("other", root_id)
         other.tt.new_file('a', other.root, 'y\nb\nc\nd\be\n', 'a')
         other.tt.new_file('b', other.root, 'b2', 'b')
         other.tt.new_file('c', other.root, 'c2', 'c')
@@ -573,7 +626,7 @@
         other.tt.new_file('h', other.root, 'h\ni\nj\nk\n', 'h')
         other.tt.new_file('i', other.root, 'h\ni\nj\nk\n', 'i')
         other.tt.apply()
-        this = TransformGroup("this")
+        this = TransformGroup("this", root_id)
         this.tt.new_file('a', this.root, 'a\nb\nc\nd\bz\n', 'a')
         this.tt.new_file('b', this.root, 'b', 'b')
         this.tt.new_file('c', this.root, 'c', 'c')
@@ -630,9 +683,10 @@
     def test_file_merge(self):
         if not has_symlinks():
             raise TestSkipped('Symlinks are not supported on this platform')
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         for tg in this, base, other:
             tg.tt.new_directory('a', tg.root, 'a')
             tg.tt.new_symlink('b', tg.root, 'b', 'b')
@@ -670,9 +724,10 @@
         self.assertIs(os.path.lexists(this.wt.abspath('h.OTHER')), True)
 
     def test_filename_merge(self):
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
                                    for t in [base, this, other]]
         base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 
@@ -702,9 +757,10 @@
         self.assertEqual(this.wt.id2path('f'), pathjoin('b/f1'))
 
     def test_filename_merge_conflicts(self):
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
                                    for t in [base, this, other]]
         base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 
@@ -731,6 +787,7 @@
         self.assertIs(os.path.lexists(this.wt.abspath('b/h1.OTHER')), False)
         self.assertEqual(this.wt.id2path('i'), pathjoin('b/i1.OTHER'))
 
+
 class TestBuildTree(tests.TestCaseWithTransport):
 
     def test_build_tree(self):

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2006-09-05 21:25:15 +0000
+++ bzrlib/tests/test_transport.py	2006-10-06 05:53:44 +0000
@@ -100,6 +100,13 @@
         finally:
             _set_protocol_handlers(saved_handlers)
 
+    def test__combine_paths(self):
+        t = Transport('/')
+        self.assertEqual('/home/sarah/project/foo',
+                         t._combine_paths('/home/sarah', 'project/foo'))
+        self.assertEqual('/etc',
+                         t._combine_paths('/home/sarah', '../../etc'))
+
 
 class TestCoalesceOffsets(TestCase):
     
@@ -168,11 +175,17 @@
     def test_clone(self):
         transport = MemoryTransport()
         self.assertTrue(isinstance(transport, MemoryTransport))
+        self.assertEqual("memory:///", transport.clone("/").base)
 
     def test_abspath(self):
         transport = MemoryTransport()
         self.assertEqual("memory:///relpath", transport.abspath('relpath'))
 
+    def test_abspath_of_root(self):
+        transport = MemoryTransport()
+        self.assertEqual("memory:///", transport.base)
+        self.assertEqual("memory:///", transport.abspath('/'))
+
     def test_relpath(self):
         transport = MemoryTransport()
 
@@ -409,7 +422,7 @@
         base_url = self._server.get_url()
         # try getting the transport via the regular interface:
         t = get_transport(base_url)
-        if not isinstance(t, self.transport_class): 
+        if not isinstance(t, self.transport_class):
             # we did not get the correct transport class type. Override the
             # regular connection behaviour by direct construction.
             t = self.transport_class(base_url)

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-09-06 14:37:18 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-09-22 04:39:58 +0000
@@ -26,18 +26,19 @@
 import sys
 
 from bzrlib import (
+    errors,
     osutils,
     urlutils,
     )
 from bzrlib.errors import (DirectoryNotEmpty, NoSuchFile, FileExists,
-                           LockError, PathError,
+                           LockError, NoSmartServer, PathError,
                            TransportNotPossible, ConnectionError,
                            InvalidURL)
 from bzrlib.osutils import getcwd
 from bzrlib.symbol_versioning import zero_eleven
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.tests.test_transport import TestTransportImplementation
-from bzrlib.transport import memory
+from bzrlib.transport import memory, smart
 import bzrlib.transport
 
 
@@ -68,8 +69,10 @@
         except excClass:
             return
         else:
-            if hasattr(excClass,'__name__'): excName = excClass.__name__
-            else: excName = str(excClass)
+            if getattr(excClass,'__name__', None) is not None:
+                excName = excClass.__name__
+            else:
+                excName = str(excClass)
             raise self.failureException, "%s not raised" % excName
 
     def test_has(self):
@@ -89,6 +92,13 @@
         self.assertEqual(False, t.has_any(['c', 'c', 'c']))
         self.assertEqual(True, t.has_any(['b', 'b', 'b']))
 
+    def test_has_root_works(self):
+        current_transport = self.get_transport()
+        # import pdb;pdb.set_trace()
+        self.assertTrue(current_transport.has('/'))
+        root = current_transport.clone('/')
+        self.assertTrue(root.has(''))
+
     def test_get(self):
         t = self.get_transport()
 
@@ -142,6 +152,11 @@
                              t.put, 'b', StringIO('file-like\ncontents\n'))
         self.check_transport_contents('file-like\ncontents\n', t, 'b')
 
+        self.assertRaises(NoSuchFile,
+            self.applyDeprecated,
+            zero_eleven,
+            t.put, 'path/doesnt/exist/c', StringIO('contents'))
+
     def test_put_bytes(self):
         t = self.get_transport()
 
@@ -243,6 +258,18 @@
         umask = osutils.get_umask()
         t.put_bytes_non_atomic('nomode', 'test text\n', mode=None)
         self.assertTransportMode(t, 'nomode', 0666 & ~umask)
+
+        # We should also be able to set the mode for a parent directory
+        # when it is created
+        t.put_bytes_non_atomic('dir700/mode664', 'test text\n', mode=0664,
+                               dir_mode=0700, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir700', 0700)
+        t.put_bytes_non_atomic('dir770/mode664', 'test text\n', mode=0664,
+                               dir_mode=0770, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir770', 0770)
+        t.put_bytes_non_atomic('dir777/mode664', 'test text\n', mode=0664,
+                               dir_mode=0777, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir777', 0777)
         
     def test_put_file(self):
         t = self.get_transport()
@@ -352,6 +379,19 @@
         t.put_file_non_atomic('nomode', StringIO('test text\n'), mode=None)
         self.assertTransportMode(t, 'nomode', 0666 & ~umask)
         
+        # We should also be able to set the mode for a parent directory
+        # when it is created
+        sio = StringIO()
+        t.put_file_non_atomic('dir700/mode664', sio, mode=0664,
+                              dir_mode=0700, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir700', 0700)
+        t.put_file_non_atomic('dir770/mode664', sio, mode=0664,
+                              dir_mode=0770, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir770', 0770)
+        t.put_file_non_atomic('dir777/mode664', sio, mode=0664,
+                              dir_mode=0777, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir777', 0777)
+
     def test_put_multi(self):
         t = self.get_transport()
 
@@ -373,6 +413,37 @@
         self.check_transport_contents('diff\ncontents for\na\n', t, 'a')
         self.check_transport_contents('another contents\nfor d\n', t, 'd')
 
+    def test_put_permissions(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        if not t._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        self.applyDeprecated(zero_eleven, t.put, 'mode644',
+                             StringIO('test text\n'), mode=0644)
+        self.assertTransportMode(t, 'mode644', 0644)
+        self.applyDeprecated(zero_eleven, t.put, 'mode666',
+                             StringIO('test text\n'), mode=0666)
+        self.assertTransportMode(t, 'mode666', 0666)
+        self.applyDeprecated(zero_eleven, t.put, 'mode600',
+                             StringIO('test text\n'), mode=0600)
+        self.assertTransportMode(t, 'mode600', 0600)
+        # Yes, you can put a file such that it becomes readonly
+        self.applyDeprecated(zero_eleven, t.put, 'mode400',
+                             StringIO('test text\n'), mode=0400)
+        self.assertTransportMode(t, 'mode400', 0400)
+        self.applyDeprecated(zero_eleven, t.put_multi,
+                             [('mmode644', StringIO('text\n'))], mode=0644)
+        self.assertTransportMode(t, 'mmode644', 0644)
+
+        # The default permissions should be based on the current umask
+        umask = osutils.get_umask()
+        self.applyDeprecated(zero_eleven, t.put, 'nomode',
+                             StringIO('test text\n'), mode=None)
+        self.assertTransportMode(t, 'nomode', 0666 & ~umask)
+        
     def test_mkdir(self):
         t = self.get_transport()
 
@@ -724,6 +795,8 @@
         t.mkdir('adir/asubdir')
         t.mkdir('bdir')
         t.mkdir('bdir/bsubdir')
+        # any kind of PathError would be OK, though we normally expect
+        # DirectoryNotEmpty
         self.assertRaises(PathError, t.rename, 'bdir', 'adir')
         # nothing was changed so it should still be as before
         self.assertTrue(t.has('bdir/bsubdir'))
@@ -814,12 +887,18 @@
         # TODO: test copy_multi
 
     def test_connection_error(self):
-        """ConnectionError is raised when connection is impossible"""
+        """ConnectionError is raised when connection is impossible.
+        
+        The error may be raised from either the constructor or the first
+        operation on the transport.
+        """
         try:
             url = self._server.get_bogus_url()
         except NotImplementedError:
             raise TestSkipped("Transport %s has no bogus URL support." %
                               self._server.__class__)
+        # This should be:  but SSH still connects on construction. No COOKIE!
+        # self.assertRaises((ConnectionError, NoSuchFile), t.get, '.bzr/branch')
         try:
             t = bzrlib.transport.get_transport(url)
             t.get('.bzr/branch')
@@ -965,6 +1044,44 @@
         self.failUnless(t2.has('d'))
         self.failUnless(t3.has('b/d'))
 
+    def test_clone_to_root(self):
+        orig_transport = self.get_transport()
+        # Repeatedly go up to a parent directory until we're at the root
+        # directory of this transport
+        root_transport = orig_transport
+        new_transport = root_transport.clone("..")
+        # as we are walking up directories, the path must be must be 
+        # growing less, except at the top
+        self.assertTrue(len(new_transport.base) < len(root_transport.base)
+            or new_transport.base == root_transport.base)
+        while new_transport.base != root_transport.base:
+            root_transport = new_transport
+            new_transport = root_transport.clone("..")
+            # as we are walking up directories, the path must be must be 
+            # growing less, except at the top
+            self.assertTrue(len(new_transport.base) < len(root_transport.base)
+                or new_transport.base == root_transport.base)
+
+        # Cloning to "/" should take us to exactly the same location.
+        self.assertEqual(root_transport.base, orig_transport.clone("/").base)
+        # the abspath of "/" from the original transport should be the same
+        # as the base at the root:
+        self.assertEqual(orig_transport.abspath("/"), root_transport.base)
+
+        # At the root, the URL must still end with / as its a directory
+        self.assertEqual(root_transport.base[-1], '/')
+
+    def test_clone_from_root(self):
+        """At the root, cloning to a simple dir should just do string append."""
+        orig_transport = self.get_transport()
+        root_transport = orig_transport.clone('/')
+        self.assertEqual(root_transport.base + '.bzr/',
+            root_transport.clone('.bzr').base)
+
+    def test_base_url(self):
+        t = self.get_transport()
+        self.assertEqual('/', t.base[-1])
+
     def test_relpath(self):
         t = self.get_transport()
         self.assertEqual('', t.relpath(t.base))
@@ -993,11 +1110,15 @@
         # specific test cases.
         transport = self.get_transport()
         
-        # disabled because some transports might normalize urls in generating
-        # the abspath - eg http+pycurl-> just http -- mbp 20060308 
         self.assertEqual(transport.base + 'relpath',
                          transport.abspath('relpath'))
 
+        # This should work without raising an error.
+        transport.abspath("/")
+
+        # the abspath of "/" and "/foo/.." should result in the same location
+        self.assertEqual(transport.abspath("/"), transport.abspath("/foo/.."))
+
     def test_local_abspath(self):
         transport = self.get_transport()
         try:
@@ -1129,23 +1250,37 @@
         self.check_transport_contents('bar', transport2, 'foo')
 
     def test_lock_write(self):
+        """Test transport-level write locks.
+
+        These are deprecated and transports may decline to support them.
+        """
         transport = self.get_transport()
         if transport.is_readonly():
             self.assertRaises(TransportNotPossible, transport.lock_write, 'foo')
             return
         transport.put_bytes('lock', '')
-        lock = transport.lock_write('lock')
+        try:
+            lock = transport.lock_write('lock')
+        except TransportNotPossible:
+            return
         # TODO make this consistent on all platforms:
         # self.assertRaises(LockError, transport.lock_write, 'lock')
         lock.unlock()
 
     def test_lock_read(self):
+        """Test transport-level read locks.
+
+        These are deprecated and transports may decline to support them.
+        """
         transport = self.get_transport()
         if transport.is_readonly():
             file('lock', 'w').close()
         else:
             transport.put_bytes('lock', '')
-        lock = transport.lock_read('lock')
+        try:
+            lock = transport.lock_read('lock')
+        except TransportNotPossible:
+            return
         # TODO make this consistent on all platforms:
         # self.assertRaises(LockError, transport.lock_read, 'lock')
         lock.unlock()
@@ -1175,3 +1310,31 @@
         self.assertEqual(d[1], (9, '9'))
         self.assertEqual(d[2], (0, '0'))
         self.assertEqual(d[3], (3, '34'))
+
+    def test_get_smart_medium(self):
+        """All transports must either give a smart medium, or know they can't.
+        """
+        transport = self.get_transport()
+        try:
+            medium = transport.get_smart_medium()
+            self.assertIsInstance(medium, smart.SmartClientMedium)
+        except errors.NoSmartMedium:
+            # as long as we got it we're fine
+            pass
+
+    def test_readv_short_read(self):
+        transport = self.get_transport()
+        if transport.is_readonly():
+            file('a', 'w').write('0123456789')
+        else:
+            transport.put_bytes('a', '01234567890')
+
+        # This is intentionally reading off the end of the file
+        # since we are sure that it cannot get there
+        self.assertListRaises((errors.ShortReadvError, AssertionError),
+                              transport.readv, 'a', [(1,1), (8,10)])
+
+        # This is trying to seek past the end of the file, it should
+        # also raise a special error
+        self.assertListRaises(errors.ShortReadvError,
+                              transport.readv, 'a', [(12,2)])

=== modified file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/test_tree.py	2006-09-28 01:53:23 +0000
@@ -62,10 +62,11 @@
     calls = []
 
     def compare(self, want_unchanged=False, specific_files=None,
-        extra_trees=None, require_versioned=False):
+        extra_trees=None, require_versioned=False, include_root=False):
         self.calls.append(
             ('compare', self.source, self.target, want_unchanged,
-             specific_files, extra_trees, require_versioned)
+             specific_files, extra_trees, require_versioned, 
+             include_root)
             )
     
     @classmethod
@@ -79,7 +80,7 @@
         """This test tests the way Tree.compare() uses InterTree."""
         old_optimisers = InterTree._optimisers
         try:
-            InterTree._optimisers = set()
+            InterTree._optimisers = []
             RecordingOptimiser.calls = []
             InterTree.register_optimiser(RecordingOptimiser)
             tree = self.make_branch_and_tree('1')
@@ -88,19 +89,32 @@
             # trivial usage
             tree.changes_from(tree2)
             # pass in all optional arguments by position
-            tree.changes_from(tree2, 'unchanged', 'specific', 'extra', 'require')
+            tree.changes_from(tree2, 'unchanged', 'specific', 'extra', 
+                              'require', True)
             # pass in all optional arguments by keyword
             tree.changes_from(tree2,
                 specific_files='specific',
                 want_unchanged='unchanged',
                 extra_trees='extra',
                 require_versioned='require',
+                include_root=True,
                 )
         finally:
             InterTree._optimisers = old_optimisers
         self.assertEqual(
             [
-             ('compare', tree2, tree, False, None, None, False),
-             ('compare', tree2, tree, 'unchanged', 'specific', 'extra', 'require'),
-             ('compare', tree2, tree, 'unchanged', 'specific', 'extra', 'require'),
+             ('compare', tree2, tree, False, None, None, False, False),
+             ('compare', tree2, tree, 'unchanged', 'specific', 'extra',
+              'require', True),
+             ('compare', tree2, tree, 'unchanged', 'specific', 'extra',
+              'require', True),
             ], RecordingOptimiser.calls)
+
+    def test_changes_from_with_root(self):
+        """Ensure the include_root option does what's expected."""
+        wt = self.make_branch_and_tree('.')
+        delta = wt.changes_from(wt.basis_tree())
+        self.assertEqual(len(delta.added), 0)
+        delta = wt.changes_from(wt.basis_tree(), wt, include_root=True)
+        self.assertEqual(len(delta.added), 1)
+        self.assertEqual(delta.added[0][0], '')

=== added file 'bzrlib/tests/test_treebuilder.py'
--- bzrlib/tests/test_treebuilder.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_treebuilder.py	2006-09-15 22:46:02 +0000
@@ -0,0 +1,94 @@
+# Copyright (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the TreeBuilder helper class."""
+
+from bzrlib import errors
+from bzrlib.memorytree import MemoryTree
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.treebuilder import TreeBuilder
+
+
+class FakeTree(object):
+    """A pretend tree to test the calls made by TreeBuilder."""
+
+    def __init__(self):
+        self._calls = []
+
+    def lock_tree_write(self):
+        self._calls.append("lock_tree_write")
+
+    def unlock(self):
+        self._calls.append("unlock")
+
+
+class TestFakeTree(TestCaseWithTransport):
+    
+    def testFakeTree(self):
+        """Check that FakeTree works as required for the TreeBuilder tests."""
+        tree = FakeTree()
+        self.assertEqual([], tree._calls)
+        tree.lock_tree_write()
+        self.assertEqual(["lock_tree_write"], tree._calls)
+        tree.unlock()
+        self.assertEqual(["lock_tree_write", "unlock"], tree._calls)
+
+
+class TestTreeBuilder(TestCaseWithTransport):
+    
+    def test_create(self):
+        builder = TreeBuilder()
+
+    def test_start_tree_locks_write(self):
+        builder = TreeBuilder()
+        tree = FakeTree()
+        builder.start_tree(tree)
+        self.assertEqual(["lock_tree_write"], tree._calls)
+
+    def test_start_tree_when_started_fails(self):
+        builder = TreeBuilder()
+        tree = FakeTree()
+        builder.start_tree(tree)
+        self.assertRaises(errors.AlreadyBuilding, builder.start_tree, tree)
+        
+    def test_finish_tree_not_started_errors(self):
+        builder = TreeBuilder()
+        self.assertRaises(errors.NotBuilding, builder.finish_tree)
+
+    def test_finish_tree_unlocks(self):
+        builder = TreeBuilder()
+        tree = FakeTree()
+        builder.start_tree(tree)
+        builder.finish_tree()
+        self.assertEqual(["lock_tree_write", "unlock"], tree._calls)
+
+    def test_build_tree_not_started_errors(self):
+        builder = TreeBuilder()
+        self.assertRaises(errors.NotBuilding, builder.build, "foo")
+
+    def test_build_tree(self):
+        """Test building works using a MemoryTree."""
+        branch = self.make_branch('branch')
+        tree = MemoryTree.create_on_branch(branch)
+        builder = TreeBuilder()
+        builder.start_tree(tree)
+        builder.build(['foo', "bar/", "bar/file"])
+        self.assertEqual('contents of foo\n',
+            tree.get_file(tree.path2id('foo')).read())
+        self.assertEqual('contents of bar/file\n',
+            tree.get_file(tree.path2id('bar/file')).read())
+        builder.finish_tree()

=== modified file 'bzrlib/tests/test_upgrade.py'
--- bzrlib/tests/test_upgrade.py	2006-08-25 21:59:48 +0000
+++ bzrlib/tests/test_upgrade.py	2006-09-05 08:14:23 +0000
@@ -131,8 +131,8 @@
         self.build_tree_contents(_upgrade_dir_template)
         upgrade('.', bzrdir.BzrDirMetaFormat1())
         tree = workingtree.WorkingTree.open('.')
-        self.assertEqual(tree.last_revision(),
-                         tree.branch.revision_history()[-1])
+        self.assertEqual([tree.branch.revision_history()[-1]],
+            tree.get_parent_ids())
 
     def test_upgrade_v6_to_meta_no_workingtree(self):
         # Some format6 branches do not have checkout files. Upgrading

=== modified file 'bzrlib/tests/test_version.py'
--- bzrlib/tests/test_version.py	2006-08-16 02:53:40 +0000
+++ bzrlib/tests/test_version.py	2006-09-05 08:14:23 +0000
@@ -17,7 +17,7 @@
 
 """Tests for versioning of bzrlib."""
 
-from bzrlib import version
+from bzrlib import version, workingtree
 from bzrlib.tests import TestCase, TestSkipped
 
 class TestBzrlibVersioning(TestCase):
@@ -32,4 +32,5 @@
         if src_tree is None:
             raise TestSkipped("bzr tests aren't run from a bzr working tree")
         else:
-            source_version = src_tree.last_revision()
+            # ensure that what we got was in fact a working tree instance.
+            self.assertIsInstance(src_tree, workingtree.WorkingTree)

=== added file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_version_info.py	2006-09-27 02:28:44 +0000
@@ -0,0 +1,222 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for version_info"""
+
+from cStringIO import StringIO
+import imp
+import os
+import sys
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.rio import read_stanzas
+
+from bzrlib.version_info_formats.format_rio import RioVersionInfoBuilder
+from bzrlib.version_info_formats.format_python import PythonVersionInfoBuilder
+
+
+class TestVersionInfo(TestCaseWithTransport):
+
+    def create_branch(self):
+        wt = self.make_branch_and_tree('branch')
+
+        self.build_tree(['branch/a'])
+        wt.add('a')
+        wt.commit('a', rev_id='r1')
+
+        self.build_tree(['branch/b'])
+        wt.add('b')
+        wt.commit('b', rev_id='r2')
+
+        self.build_tree_contents([('branch/a', 'new contents\n')])
+        wt.commit(u'\xe52', rev_id='r3')
+
+        return wt
+
+    def test_rio_version_text(self):
+        wt = self.create_branch()
+
+        def regen(**kwargs):
+            sio = StringIO()
+            builder = RioVersionInfoBuilder(wt.branch, working_tree=wt,
+                                            **kwargs)
+            builder.generate(sio)
+            val = sio.getvalue()
+            return val
+
+        val = regen()
+        self.assertContainsRe(val, 'build-date:')
+        self.assertContainsRe(val, 'date:')
+        self.assertContainsRe(val, 'revno: 3')
+        self.assertContainsRe(val, 'revision-id: r3')
+
+        val = regen(check_for_clean=True)
+        self.assertContainsRe(val, 'clean: True')
+
+        self.build_tree(['branch/c'])
+        val = regen(check_for_clean=True)
+        self.assertContainsRe(val, 'clean: False')
+        os.remove('branch/c')
+
+        val = regen(include_revision_history=True)
+        self.assertContainsRe(val, 'id: r1')
+        self.assertContainsRe(val, 'message: a')
+        self.assertContainsRe(val, 'id: r2')
+        self.assertContainsRe(val, 'message: b')
+        self.assertContainsRe(val, 'id: r3')
+        self.assertContainsRe(val, 'message: \xc3\xa52') # utf8 encoding '\xe5'
+
+    def test_rio_version(self):
+        wt = self.create_branch()
+
+        def regen(**kwargs):
+            sio = StringIO()
+            builder = RioVersionInfoBuilder(wt.branch, working_tree=wt,
+                                            **kwargs)
+            builder.generate(sio)
+            sio.seek(0)
+            stanzas = list(read_stanzas(sio))
+            self.assertEqual(1, len(stanzas))
+            return stanzas[0]
+
+        def get_one_stanza(stanza, key):
+            new_stanzas = list(read_stanzas(
+                                StringIO(stanza[key].encode('utf8'))))
+            self.assertEqual(1, len(new_stanzas))
+            return new_stanzas[0]
+
+        stanza = regen()
+        self.failUnless('date' in stanza)
+        self.failUnless('build-date' in stanza)
+        self.assertEqual(['3'], stanza.get_all('revno'))
+        self.assertEqual(['r3'], stanza.get_all('revision-id'))
+
+        stanza = regen(check_for_clean=True)
+        self.assertEqual(['True'], stanza.get_all('clean'))
+
+        self.build_tree(['branch/c'])
+        stanza = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(['False'], stanza.get_all('clean'))
+
+        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['', 'a', 'b', 'c'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r3', 'r3', 'r2', 'unversioned'],
+            file_rev_stanza.get_all('revision'))
+        os.remove('branch/c')
+
+        stanza = regen(include_revision_history=True)
+        revision_stanza = get_one_stanza(stanza, 'revisions')
+        self.assertEqual(['r1', 'r2', 'r3'], revision_stanza.get_all('id'))
+        self.assertEqual(['a', 'b', u'\xe52'], revision_stanza.get_all('message'))
+        self.assertEqual(3, len(revision_stanza.get_all('date')))
+
+        # a was modified, so it should show up modified again
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+        stanza = regen(check_for_clean=True, include_file_revisions=True)
+        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['', 'a', 'b', 'c', 'd'], 
+                          file_rev_stanza.get_all('path'))
+        self.assertEqual(['r3', 'modified', 'renamed to d', 'new', 
+                          'renamed from b'],
+                         file_rev_stanza.get_all('revision'))
+
+        wt.commit('modified', rev_id='r4')
+        wt.remove(['c', 'd'])
+        os.remove('branch/d')
+        stanza = regen(check_for_clean=True, include_file_revisions=True)
+        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['', 'a', 'c', 'd'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r4', 'r4', 'unversioned', 'removed'],
+                         file_rev_stanza.get_all('revision'))
+
+    def test_python_version(self):
+        wt = self.create_branch()
+
+        def regen(**kwargs):
+            """Create a test module, import and return it"""
+            outf = open('test_version_information.py', 'wb')
+            try:
+                builder = PythonVersionInfoBuilder(wt.branch, working_tree=wt,
+                                                   **kwargs)
+                builder.generate(outf)
+            finally:
+                outf.close()
+            module_info = imp.find_module('test_version_information',
+                                          [os.getcwdu()])
+            tvi = imp.load_module('tvi', *module_info)
+            # Make sure the module isn't cached
+            sys.modules.pop('tvi', None)
+            sys.modules.pop('test_version_information', None)
+            # Delete the compiled versions, because we are generating
+            # a new file fast enough that python doesn't detect it
+            # needs to recompile, and using sleep() just makes the
+            # test slow
+            if os.path.exists('test_version_information.pyc'):
+                os.remove('test_version_information.pyc')
+            if os.path.exists('test_version_information.pyo'):
+                os.remove('test_version_information.pyo')
+            return tvi
+
+        tvi = regen()
+        self.assertEqual(3, tvi.version_info['revno'])
+        self.assertEqual('r3', tvi.version_info['revision_id'])
+        self.failUnless(tvi.version_info.has_key('date'))
+        self.assertEqual(None, tvi.version_info['clean'])
+
+        tvi = regen(check_for_clean=True)
+        self.assertEqual(True, tvi.version_info['clean'])
+
+        self.build_tree(['branch/c'])
+        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(False, tvi.version_info['clean'])
+        self.assertEqual(['', 'a', 'b', 'c'], 
+                         sorted(tvi.file_revisions.keys()))
+        self.assertEqual('r3', tvi.file_revisions['a'])
+        self.assertEqual('r2', tvi.file_revisions['b'])
+        self.assertEqual('unversioned', tvi.file_revisions['c'])
+        os.remove('branch/c')
+
+        tvi = regen(include_revision_history=True)
+
+        rev_info = [(rev, message) for rev, message, timestamp, timezone
+                                   in tvi.revisions]
+        self.assertEqual([('r1', 'a'), ('r2', 'b'), ('r3', u'\xe52')], rev_info)
+
+        # a was modified, so it should show up modified again
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(['', 'a', 'b', 'c', 'd'], 
+                          sorted(tvi.file_revisions.keys()))
+        self.assertEqual('modified', tvi.file_revisions['a'])
+        self.assertEqual('renamed to d', tvi.file_revisions['b'])
+        self.assertEqual('new', tvi.file_revisions['c'])
+        self.assertEqual('renamed from b', tvi.file_revisions['d'])
+
+        wt.commit('modified', rev_id='r4')
+        wt.remove(['c', 'd'])
+        os.remove('branch/d')
+        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(['', 'a', 'c', 'd'], 
+                          sorted(tvi.file_revisions.keys()))
+        self.assertEqual('r4', tvi.file_revisions['a'])
+        self.assertEqual('unversioned', tvi.file_revisions['c'])
+        self.assertEqual('removed', tvi.file_revisions['d'])
+
+

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_versionedfile.py	2006-09-25 18:30:50 +0000
@@ -24,7 +24,10 @@
 from StringIO import StringIO
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    progress,
+    )
 from bzrlib.errors import (
                            RevisionNotPresent, 
                            RevisionAlreadyPresent,
@@ -543,6 +546,17 @@
         # versions in the weave 
         # the ordering here is to make a tree so that dumb searches have
         # more changes to muck up.
+
+        class InstrumentedProgress(progress.DummyProgress):
+
+            def __init__(self):
+
+                progress.DummyProgress.__init__(self)
+                self.updates = []
+
+            def update(self, msg=None, current=None, total=None):
+                self.updates.append((msg, current, total))
+
         vf = self.get_file()
         # add a base to get included
         vf.add_lines('base', [], ['base\n'])
@@ -556,7 +570,7 @@
         vf.add_lines('otherchild',
                      ['lancestor', 'base'],
                      ['base\n', 'lancestor\n', 'otherchild\n'])
-        def iter_with_versions(versions):
+        def iter_with_versions(versions, expected):
             # now we need to see what lines are returned, and how often.
             lines = {'base\n':0,
                      'lancestor\n':0,
@@ -564,18 +578,32 @@
                      'child\n':0,
                      'otherchild\n':0,
                      }
+            progress = InstrumentedProgress()
             # iterate over the lines
-            for line in vf.iter_lines_added_or_present_in_versions(versions):
+            for line in vf.iter_lines_added_or_present_in_versions(versions, 
+                pb=progress):
                 lines[line] += 1
+            if []!= progress.updates: 
+                self.assertEqual(expected, progress.updates)
             return lines
-        lines = iter_with_versions(['child', 'otherchild'])
+        lines = iter_with_versions(['child', 'otherchild'], 
+                                   [('Walking content.', 0, 2), 
+                                    ('Walking content.', 0, 2), 
+                                    ('Walking content.', 3, 2), 
+                                    ('Walking content.', 2, 2)])
         # we must see child and otherchild
         self.assertTrue(lines['child\n'] > 0)
         self.assertTrue(lines['otherchild\n'] > 0)
         # we dont care if we got more than that.
         
         # test all lines
-        lines = iter_with_versions(None)
+        lines = iter_with_versions(None, [('Walking content.', 0, 5), 
+                                          ('Walking content.', 0, 5), 
+                                          ('Walking content.', 1, 5), 
+                                          ('Walking content.', 2, 5), 
+                                          ('Walking content.', 2, 5), 
+                                          ('Walking content.', 3, 5), 
+                                          ('Walking content.', 5, 5)])
         # all lines must be seen at least once
         self.assertTrue(lines['base\n'] > 0)
         self.assertTrue(lines['lancestor\n'] > 0)

=== modified file 'bzrlib/tests/test_weave.py'
--- bzrlib/tests/test_weave.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_weave.py	2006-09-20 22:32:33 +0000
@@ -25,11 +25,13 @@
 
 from pprint import pformat
 
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    )
+from bzrlib.osutils import sha_string
+from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.weave import Weave, WeaveFormatError, WeaveError, reweave
 from bzrlib.weavefile import write_weave, read_weave
-from bzrlib.tests import TestCase
-from bzrlib.osutils import sha_string
 
 
 # texts for use in testing
@@ -890,3 +892,14 @@
         self.assertFalse(w1._compatible_parents(set(), set([1])))
         self.assertFalse(w1._compatible_parents(my_parents, set([1, 2, 3, 4])))
         self.assertFalse(w1._compatible_parents(my_parents, set([4])))
+
+
+class TestWeaveFile(TestCaseInTempDir):
+    
+    def test_empty_file(self):
+        f = open('empty.weave', 'wb+')
+        try:
+            self.assertRaises(errors.WeaveFormatError,
+                              read_weave, f)
+        finally:
+            f.close()

=== modified file 'bzrlib/tests/test_whitebox.py'
--- bzrlib/tests/test_whitebox.py	2006-06-20 07:55:43 +0000
+++ bzrlib/tests/test_whitebox.py	2006-10-05 22:42:09 +0000
@@ -17,6 +17,9 @@
 import os
 import unittest
 
+from bzrlib import (
+    osutils,
+    )
 from bzrlib.tests import TestCaseWithTransport, TestCase
 from bzrlib.branch import Branch
 from bzrlib.errors import PathNotChild
@@ -33,7 +36,6 @@
         if it is inside a branch and return the path relative to the base.
         """
         import tempfile
-        from bzrlib.osutils import rmtree
         
         savedir = os.getcwdu()
         dtmp = tempfile.mkdtemp()
@@ -81,4 +83,4 @@
 
         finally:
             os.chdir(savedir)
-            rmtree(dtmp)
+            osutils.rmtree(dtmp)

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-09-05 20:58:28 +0000
+++ bzrlib/tests/test_workingtree.py	2006-09-22 04:52:17 +0000
@@ -25,12 +25,18 @@
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import NotBranchError, NotVersionedError
 from bzrlib.lockdir import LockDir
+from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.trace import mutter
 from bzrlib.transport import get_transport
-from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
-                                WorkingTree)
+from bzrlib.workingtree import (
+    TreeEntry,
+    TreeDirectory,
+    TreeFile,
+    TreeLink,
+    WorkingTree,
+    )
 
 class TestTreeDirectory(TestCaseWithTransport):
 
@@ -167,9 +173,10 @@
         t = control.get_workingtree_transport(None)
         self.assertEqualDiff('Bazaar-NG Working Tree format 3',
                              t.get('format').read())
-        self.assertEqualDiff('<inventory format="5">\n'
-                             '</inventory>\n',
-                             t.get('inventory').read())
+        self.assertContainsRe(t.get('inventory').read(), 
+                              '<inventory file_id="[^"]*" format="5">\n'
+                              '</inventory>\n',
+                             )
         self.assertEqualDiff('### bzr hashcache v5\n',
                              t.get('stat-cache').read())
         self.assertFalse(t.has('inventory.basis'))
@@ -364,3 +371,49 @@
         finally:
             bzrlib.DEFAULT_IGNORE = orig_default
             ignores._runtime_ignores = orig_runtime
+
+
+class InstrumentedTree(object):
+    """A instrumented tree to check the needs_tree_write_lock decorator."""
+
+    def __init__(self):
+        self._locks = []
+
+    def lock_tree_write(self):
+        self._locks.append('t')
+
+    @needs_tree_write_lock
+    def method_with_tree_write_lock(self, *args, **kwargs):
+        """A lock_tree_write decorated method that returns its arguments."""
+        return args, kwargs
+
+    @needs_tree_write_lock
+    def method_that_raises(self):
+        """This method causes an exception when called with parameters.
+        
+        This allows the decorator code to be checked - it should still call
+        unlock.
+        """
+
+    def unlock(self):
+        self._locks.append('u')
+
+
+class TestInstrumentedTree(TestCase):
+
+    def test_needs_tree_write_lock(self):
+        """@needs_tree_write_lock should be semantically transparent."""
+        tree = InstrumentedTree()
+        self.assertEqual(
+            'method_with_tree_write_lock',
+            tree.method_with_tree_write_lock.__name__)
+        self.assertEqual(
+            "A lock_tree_write decorated method that returns its arguments.",
+            tree.method_with_tree_write_lock.__doc__)
+        args = (1, 2, 3)
+        kwargs = {'a':'b'}
+        result = tree.method_with_tree_write_lock(1,2,3, a='b')
+        self.assertEqual((args, kwargs), result)
+        self.assertEqual(['t', 'u'], tree._locks)
+        self.assertRaises(TypeError, tree.method_that_raises, 'foo')
+        self.assertEqual(['t', 'u', 't', 'u'], tree._locks)

=== modified file 'bzrlib/tests/tree_implementations/__init__.py'
--- bzrlib/tests/tree_implementations/__init__.py	2006-09-05 11:52:44 +0000
+++ bzrlib/tests/tree_implementations/__init__.py	2006-09-10 19:04:48 +0000
@@ -86,13 +86,15 @@
         :param empty_tree: A working tree with no content and no parents to
             modify.
         """
+        empty_tree.set_root_id('empty-root-id')
         return self._convert_tree(empty_tree, converter)
 
     def _make_abc_tree(self, tree):
         """setup an abc content tree."""
         files = ['a', 'b/', 'b/c']
-        self.build_tree(files, line_endings='binary',
+        self.build_tree(files, line_endings='binary', 
                         transport=tree.bzrdir.root_transport)
+        tree.set_root_id('root-id')
         tree.add(files, ['a-id', 'b-id', 'c-id'])
 
     def get_tree_no_parents_abc_content(self, tree, converter=None):

=== modified file 'bzrlib/tests/tree_implementations/test_test_trees.py'
--- bzrlib/tests/tree_implementations/test_test_trees.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/tree_implementations/test_test_trees.py	2006-08-06 00:20:32 +0000
@@ -16,7 +16,6 @@
 
 """Tests for the test trees used by the tree_implementations tests."""
 
-from bzrlib import inventory 
 from bzrlib.tests.tree_implementations import TestCaseWithTree
 
 
@@ -28,9 +27,9 @@
         self.assertEqual([], tree.get_parent_ids())
         self.assertEqual([], tree.conflicts())
         self.assertEqual([], list(tree.unknowns()))
-        self.assertEqual([inventory.ROOT_ID], list(iter(tree)))
+        self.assertEqual(['empty-root-id'], list(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID)],
+            [('', 'empty-root-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
 
     def test_abc_tree_no_parents(self):
@@ -41,10 +40,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -57,10 +56,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('foobar\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -73,10 +72,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertTrue(tree.is_executable('c-id'))
@@ -89,10 +88,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -105,10 +104,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('bar\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -121,10 +120,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('e', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('e', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertTrue(tree.is_executable('c-id'))

=== modified file 'bzrlib/tests/workingtree_implementations/__init__.py'
--- bzrlib/tests/workingtree_implementations/__init__.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/__init__.py	2006-10-05 07:34:58 +0000
@@ -56,6 +56,7 @@
         'bzrlib.tests.workingtree_implementations.test_changes_from',
         'bzrlib.tests.workingtree_implementations.test_commit',
         'bzrlib.tests.workingtree_implementations.test_executable',
+        'bzrlib.tests.workingtree_implementations.test_flush',
         'bzrlib.tests.workingtree_implementations.test_get_parent_ids',
         'bzrlib.tests.workingtree_implementations.test_is_control_filename',
         'bzrlib.tests.workingtree_implementations.test_is_ignored',
@@ -63,6 +64,11 @@
         'bzrlib.tests.workingtree_implementations.test_merge_from_branch',
         'bzrlib.tests.workingtree_implementations.test_parents',
         'bzrlib.tests.workingtree_implementations.test_pull',
+        'bzrlib.tests.workingtree_implementations.test_put_file',
+        'bzrlib.tests.workingtree_implementations.test_mkdir',
+        'bzrlib.tests.workingtree_implementations.test_read_working_inventory',
+        'bzrlib.tests.workingtree_implementations.test_set_root_id',
+        'bzrlib.tests.workingtree_implementations.test_unversion',
         'bzrlib.tests.workingtree_implementations.test_workingtree',
         ]
     adapter = WorkingTreeTestProviderAdapter(

=== modified file 'bzrlib/tests/workingtree_implementations/test_basis_inventory.py'
--- bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-08-17 14:16:46 +0000
+++ bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-09-09 18:52:57 +0000
@@ -18,8 +18,9 @@
 
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 from bzrlib.branch import Branch
+from bzrlib import inventory
 from bzrlib.revision import Revision
-import bzrlib.xml5
+import bzrlib.xml6
 
 
 class TestBasisInventory(TestCaseWithWorkingTree):
@@ -35,7 +36,7 @@
         t.add('a')
         t.commit('a', rev_id='r1')
 
-        t._control_files.get_utf8('basis-inventory')
+        t._control_files.get_utf8('basis-inventory-cache')
 
         basis_inv = t.basis_tree().inventory
         self.assertEquals('r1', basis_inv.revision_id)
@@ -47,16 +48,32 @@
         t.add('b')
         t.commit('b', rev_id='r2')
 
-        t._control_files.get_utf8('basis-inventory')
+        t._control_files.get_utf8('basis-inventory-cache')
 
         basis_inv_txt = t.read_basis_inventory()
-        basis_inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(basis_inv_txt)
-        basis_inv.root.revision = 'r2'
+        basis_inv = bzrlib.xml6.serializer_v6.read_inventory_from_string(basis_inv_txt)
         self.assertEquals('r2', basis_inv.revision_id)
         store_inv = b.repository.get_inventory('r2')
 
         self.assertEquals(store_inv._byid, basis_inv._byid)
 
+    def test_wrong_format(self):
+        """WorkingTree.basis safely ignores junk basis inventories"""
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+        t._control_files.put_utf8('basis-inventory-cache', 'booga')
+        t.basis_tree()
+        t._control_files.put_utf8('basis-inventory-cache', '<xml/>')
+        t.basis_tree()
+        t._control_files.put_utf8('basis-inventory-cache', '<inventory />')
+        t.basis_tree()
+        t._control_files.put_utf8('basis-inventory-cache', 
+                                  '<inventory format="pi"/>')
+        t.basis_tree()
+
     def test_basis_inv_gets_revision(self):
         """When the inventory of the basis tree has no revision id it gets set.
 
@@ -65,11 +82,13 @@
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         # TODO change this to use CommitBuilder
+        inv = inventory.Inventory(revision_id='r1')
+        inv.root.revision = 'r1'
+        inv_lines = tree.branch.repository.serialise_inventory(inv).split('\n')
+        inv_lines = [(l + '\n') for l in inv_lines if l is not None]
         tree.branch.repository.control_weaves.get_weave('inventory',
             tree.branch.repository.get_transaction()
-            ).add_lines('r1', [], [
-                '<inventory format="5">\n',
-                '</inventory>\n'])
+            ).add_lines('r1', [], inv_lines)
         rev = Revision(timestamp=0,
                        timezone=None,
                        committer="Foo Bar <foo@example.com>",
@@ -86,6 +105,6 @@
         # without parsing, but to do this properly needs a serialiser on the
         # tree object that abstracts whether it is xml/rio/etc.
         self.assertContainsRe(
-            tree._control_files.get_utf8('basis-inventory').read(),
+            tree._control_files.get_utf8('basis-inventory-cache').read(),
             'revision_id="r1"')
         

=== modified file 'bzrlib/tests/workingtree_implementations/test_commit.py'
--- bzrlib/tests/workingtree_implementations/test_commit.py	2006-09-05 21:14:35 +0000
+++ bzrlib/tests/workingtree_implementations/test_commit.py	2006-09-08 00:19:48 +0000
@@ -83,7 +83,7 @@
     def test_commit_sets_last_revision(self):
         tree = self.make_branch_and_tree('tree')
         committed_id = tree.commit('foo', rev_id='foo', allow_pointless=True)
-        self.assertEqual('foo', tree.last_revision())
+        self.assertEqual(['foo'], tree.get_parent_ids())
         # the commit should have returned the same id we asked for.
         self.assertEqual('foo', committed_id)
 
@@ -163,6 +163,43 @@
             'wibble@fofof--20050401--1928390812'],
             rev.parent_ids)
 
+    def test_commit_deleted_subtree_and_files_updates_workingtree(self):
+        """The working trees inventory may be adjusted by commit."""
+        wt = self.make_branch_and_tree('.')
+        wt.lock_write()
+        self.build_tree(['a', 'b/', 'b/c', 'd'])
+        wt.add(['a', 'b', 'b/c', 'd'], ['a-id', 'b-id', 'c-id', 'd-id'])
+        this_dir = self.get_transport()
+        this_dir.delete_tree('b')
+        this_dir.delete('d')
+        # now we have a tree with a through d in the inventory, but only
+        # a present on disk. After commit b-id, c-id and d-id should be
+        # missing from the inventory, within the same tree transaction.
+        wt.commit('commit stuff')
+        self.assertTrue(wt.has_id('a-id'))
+        self.assertFalse(wt.has_or_had_id('b-id'))
+        self.assertFalse(wt.has_or_had_id('c-id'))
+        self.assertFalse(wt.has_or_had_id('d-id'))
+        self.assertTrue(wt.has_filename('a'))
+        self.assertFalse(wt.has_filename('b'))
+        self.assertFalse(wt.has_filename('b/c'))
+        self.assertFalse(wt.has_filename('d'))
+        wt.unlock()
+        # the changes should have persisted to disk - reopen the workingtree
+        # to be sure.
+        wt = wt.bzrdir.open_workingtree()
+        wt.lock_read()
+        self.assertTrue(wt.has_id('a-id'))
+        self.assertFalse(wt.has_or_had_id('b-id'))
+        self.assertFalse(wt.has_or_had_id('c-id'))
+        self.assertFalse(wt.has_or_had_id('d-id'))
+        self.assertTrue(wt.has_filename('a'))
+        self.assertFalse(wt.has_filename('b'))
+        self.assertFalse(wt.has_filename('b/c'))
+        self.assertFalse(wt.has_filename('d'))
+        wt.unlock()
+        
+
 class TestCommitProgress(TestCaseWithWorkingTree):
     
     def restoreDefaults(self):

=== modified file 'bzrlib/tests/workingtree_implementations/test_executable.py'
--- bzrlib/tests/workingtree_implementations/test_executable.py	2006-07-24 04:40:06 +0000
+++ bzrlib/tests/workingtree_implementations/test_executable.py	2006-09-05 08:14:23 +0000
@@ -75,7 +75,7 @@
         # TODO: Maybe this should be a blackbox test
         dir2 = self.wt.branch.bzrdir.clone('b2', revision_id='r1')
         wt2 = dir2.open_workingtree()
-        self.assertEqual('r1', wt2.last_revision())
+        self.assertEqual(['r1'], wt2.get_parent_ids())
         self.assertEqual('r1', wt2.branch.last_revision())
         return wt2
 
@@ -149,7 +149,7 @@
 
         # Make sure pull will delete the files
         wt2.pull(self.wt.branch)
-        self.assertEquals('r2', wt2.last_revision())
+        self.assertEquals(['r2'], wt2.get_parent_ids())
         self.assertEquals('r2', wt2.branch.last_revision())
         self.check_empty(wt2)
 
@@ -163,7 +163,7 @@
         self.check_exist(self.wt)
 
         wt2.pull(self.wt.branch)
-        self.assertEquals('r3', wt2.last_revision())
+        self.assertEquals(['r3'], wt2.get_parent_ids())
         self.assertEquals('r3', wt2.branch.last_revision())
         self.check_exist(wt2)
 

=== added file 'bzrlib/tests/workingtree_implementations/test_flush.py'
--- bzrlib/tests/workingtree_implementations/test_flush.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_flush.py	2006-10-05 07:34:58 +0000
@@ -0,0 +1,68 @@
+# (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for WorkingTree.flush."""
+
+from bzrlib import errors, inventory
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestFlush(TestCaseWithWorkingTree):
+
+    def test_flush_fresh_tree(self):
+        tree = self.make_branch_and_tree('t1')
+        tree.lock_write()
+        try:
+            tree.flush()
+        finally:
+            tree.unlock()
+
+    def test_flush_when_inventory_is_modified(self):
+        # when doing a flush the inventory should be written if needed.
+        # we test that by changing the inventory (using
+        # _set_inventory for now until add etc have lazy writes of
+        # the inventory on unlock).
+        tree = self.make_branch_and_tree('tree')
+        # prepare for a series of changes that will modify the 
+        # inventory
+        tree.lock_write()
+        try:
+            old_root = tree.get_root_id()
+            tree.set_root_id('new-root')
+            # to detect that the inventory is written by flush, we
+            # first check that it was not written yet.
+            reference_tree = tree.bzrdir.open_workingtree()
+            self.assertEqual(old_root, reference_tree.get_root_id())
+            # now flush the tree which should write the inventory.
+            tree.flush()
+            # and check it was written using another reference tree
+            reference_tree = tree.bzrdir.open_workingtree()
+            self.assertEqual('new-root', reference_tree.get_root_id())
+        finally:
+            tree.unlock()
+            
+    def test_flush_with_read_lock_fails(self):
+        """Flush cannot be used during a read lock."""
+        tree = self.make_branch_and_tree('t1')
+        tree.lock_read()
+        try:
+            self.assertRaises(errors.NotWriteLocked, tree.flush)
+        finally:
+            tree.unlock()
+            
+    def test_flush_with_no_lock_fails(self):
+        tree = self.make_branch_and_tree('t1')
+        self.assertRaises(errors.NotWriteLocked, tree.flush)

=== modified file 'bzrlib/tests/workingtree_implementations/test_get_parent_ids.py'
--- bzrlib/tests/workingtree_implementations/test_get_parent_ids.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_get_parent_ids.py	2006-09-05 09:49:40 +0000
@@ -34,18 +34,15 @@
         self.assertEqual([rev1_id, rev2_id], t.get_parent_ids())
 
     def test_pending_merges(self):
-        """Test the correspondence between pending merges and get_parent_ids."""
+        """Test the correspondence between set pending merges and get_parent_ids."""
         wt = self.make_branch_and_tree('.')
         self.assertEqual([], wt.get_parent_ids())
-        self.assertEqual([], wt.pending_merges())
         # the first pending merge replaces the 'last revision' because
         # 'last revision' is shorthand for 'left most parent'
         wt.add_pending_merge('foo@azkhazan-123123-abcabc')
-        self.assertEqual([], wt.pending_merges())
         self.assertEqual(['foo@azkhazan-123123-abcabc'], wt.get_parent_ids())
         # adding a merge which is already in the parents list gets ignored.
         wt.add_pending_merge('foo@azkhazan-123123-abcabc')
-        self.assertEqual([], wt.pending_merges())
         self.assertEqual(['foo@azkhazan-123123-abcabc'], wt.get_parent_ids())
         # adding a different merge results in it being appended to the list -
         # order is preserved.
@@ -53,5 +50,3 @@
         self.assertEqual(['foo@azkhazan-123123-abcabc',
             'wibble@fofof--20050401--1928390812'],
             wt.get_parent_ids())
-        self.assertEqual(['wibble@fofof--20050401--1928390812'],
-            wt.pending_merges())

=== modified file 'bzrlib/tests/workingtree_implementations/test_locking.py'
--- bzrlib/tests/workingtree_implementations/test_locking.py	2006-07-24 02:07:58 +0000
+++ bzrlib/tests/workingtree_implementations/test_locking.py	2006-10-13 07:39:25 +0000
@@ -16,8 +16,11 @@
 
 """Tests for the (un)lock interfaces on all working tree implemenations."""
 
-import bzrlib.branch as branch
-import bzrlib.errors as errors
+from bzrlib import (
+    branch,
+    errors,
+    lockdir,
+    )
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 
 
@@ -53,6 +56,96 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
         
+    def test_trivial_lock_tree_write_unlock(self):
+        """Locking for tree write is ok when the branch is not locked."""
+        wt = self.make_branch_and_tree('.')
+
+        self.assertFalse(wt.is_locked())
+        self.assertFalse(wt.branch.is_locked())
+        wt.lock_tree_write()
+        try:
+            self.assertTrue(wt.is_locked())
+            self.assertTrue(wt.branch.is_locked())
+        finally:
+            wt.unlock()
+        self.assertFalse(wt.is_locked())
+        self.assertFalse(wt.branch.is_locked())
+        
+    def test_trivial_lock_tree_write_branch_read_locked(self):
+        """It is ok to lock_tree_write when the branch is read locked."""
+        wt = self.make_branch_and_tree('.')
+
+        self.assertFalse(wt.is_locked())
+        self.assertFalse(wt.branch.is_locked())
+        wt.branch.lock_read()
+        try:
+            wt.lock_tree_write()
+        except errors.ReadOnlyError:
+            # When ReadOnlyError is raised, it indicates that the 
+            # workingtree shares its lock with the branch, which is what
+            # the git/hg/bzr0.6 formats do.
+            # in this case, no lock should have been taken - but the tree
+            # will have been locked because they share a lock. Unlocking
+            # just the branch should make everything match again correctly.
+            wt.branch.unlock()
+            self.assertFalse(wt.is_locked())
+            self.assertFalse(wt.branch.is_locked())
+            return
+        try:
+            self.assertTrue(wt.is_locked())
+            self.assertTrue(wt.branch.is_locked())
+        finally:
+            wt.unlock()
+        self.assertFalse(wt.is_locked())
+        self.assertTrue(wt.branch.is_locked())
+        wt.branch.unlock()
+        
+    def _test_unlock_with_lock_method(self, methodname):
+        """Create a tree and then test its unlocking behaviour.
+
+        :param methodname: The lock method to use to establish locks.
+        """
+        # when unlocking the last lock count from tree_write_lock,
+        # the tree should do a flush().
+        # we test that by changing the inventory using set_root_id
+        tree = self.make_branch_and_tree('tree')
+        # prepare for a series of changes that will modify the 
+        # inventory
+        getattr(tree, methodname)()
+        # note that we dont have a try:finally here because of two reasons:
+        # firstly there will only be errors reported if the test fails, and 
+        # when it fails thats ok as long as the test suite cleanup still works,
+        # which it will as the lock objects are released (thats where the 
+        # warning comes from.  Secondly, it is hard in this test to be 
+        # sure that we've got the right interactions between try:finally
+        # and the lock/unlocks we are doing.
+        getattr(tree, methodname)()
+        # this should really do something within the public api
+        # e.g. mkdir('foo') but all the mutating methods at the
+        # moment trigger inventory writes and thus will not 
+        # let us trigger a read-when-dirty situation.
+        old_root = tree.get_root_id()
+        tree.set_root_id('new-root')
+        # to detect that the inventory is written by unlock, we
+        # first check that it was not written yet.
+        reference_tree = tree.bzrdir.open_workingtree()
+        self.assertEqual(old_root, reference_tree.get_root_id())
+        # now unlock the second held lock, which should do nothing.
+        tree.unlock()
+        reference_tree = tree.bzrdir.open_workingtree()
+        self.assertEqual(old_root, reference_tree.get_root_id())
+        # unlocking the first lock we took will now flush.
+        tree.unlock()
+        # and check it was written using another reference tree
+        reference_tree = tree.bzrdir.open_workingtree()
+        self.assertEqual('new-root', reference_tree.get_root_id())
+
+    def test_unlock_from_tree_write_lock_flushes(self):
+        self._test_unlock_with_lock_method("lock_tree_write")
+        
+    def test_unlock_from_write_lock_flushes(self):
+        self._test_unlock_with_lock_method("lock_write")
+        
     def test_unlock_branch_failures(self):
         """If the branch unlock fails the tree must still unlock."""
         # The public interface for WorkingTree requires a branch, but
@@ -135,7 +228,13 @@
         branch_copy.lock_write()
         try:
             try:
-                self.assertRaises(errors.LockError, wt.lock_write)
+                orig_default = lockdir._DEFAULT_TIMEOUT_SECONDS
+                try:
+                    lockdir._DEFAULT_TIMEOUT_SECONDS = 1
+                    self.assertRaises(errors.LockError, wt.lock_write)
+                finally:
+                    lockdir._DEFAULT_TIMEOUT_SECONDS = orig_default
+
                 self.assertFalse(wt.is_locked())
                 self.assertFalse(wt.branch.is_locked())
             finally:
@@ -143,3 +242,31 @@
                     wt.unlock()
         finally:
             branch_copy.unlock()
+
+    def test_failing_to_lock_tree_write_branch_does_not_lock(self):
+        """If the branch cannot be read locked, dont lock the tree."""
+        # Many implementations treat read-locks as non-blocking, but some
+        # treat them as blocking with writes.. Accordingly we test this by
+        # opening the branch twice, and locking the branch for write in the
+        # second instance.  Our lock contract requires separate instances to
+        # mutually exclude if a lock is exclusive at all: If we get no error
+        # locking, the test still passes.
+        wt = self.make_branch_and_tree('.')
+        branch_copy = branch.Branch.open('.')
+
+        branch_copy.lock_write()
+        try:
+            try:
+                wt.lock_tree_write()
+            except errors.LockError:
+                # any error here means the locks are exclusive in some 
+                # manner
+                self.assertFalse(wt.is_locked())
+                self.assertFalse(wt.branch.is_locked())
+                return
+            else:
+                # no error - the branch allows read locks while writes
+                # are taken, just pass.
+                wt.unlock()
+        finally:
+            branch_copy.unlock()

=== added file 'bzrlib/tests/workingtree_implementations/test_mkdir.py'
--- bzrlib/tests/workingtree_implementations/test_mkdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_mkdir.py	2006-09-07 23:13:08 +0000
@@ -0,0 +1,34 @@
+# (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for interface conformance of 'workingtree.put_mkdir'"""
+
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestMkdir(TestCaseWithWorkingTree):
+
+    def test_mkdir_no_id(self):
+        t = self.make_branch_and_tree('t1')
+        file_id = t.mkdir('path')
+        self.assertEqual('directory', t.kind(file_id))
+
+    def test_mkdir_with_id(self):
+        t = self.make_branch_and_tree('t1')
+        file_id = t.mkdir('path', 'my-id')
+        self.assertEqual('my-id', file_id)
+        self.assertEqual('directory', t.kind(file_id))

=== modified file 'bzrlib/tests/workingtree_implementations/test_parents.py'
--- bzrlib/tests/workingtree_implementations/test_parents.py	2006-08-25 02:21:46 +0000
+++ bzrlib/tests/workingtree_implementations/test_parents.py	2006-09-08 00:19:48 +0000
@@ -18,7 +18,7 @@
 
 import os
 
-from bzrlib import errors
+from bzrlib import errors, symbol_versioning
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 from bzrlib.uncommit import uncommit
 
@@ -26,12 +26,19 @@
 class TestParents(TestCaseWithWorkingTree):
 
     def assertConsistentParents(self, expected, tree):
+        """Check that the parents found are as expected.
+
+        This test helper also checks that they are consistent with
+        the pre-get_parent_ids() api - which is now deprecated.
+        """
         self.assertEqual(expected, tree.get_parent_ids())
         if expected == []:
             self.assertEqual(None, tree.last_revision())
         else:
             self.assertEqual(expected[0], tree.last_revision())
-        self.assertEqual(expected[1:], tree.pending_merges())
+        self.assertEqual(expected[1:],
+            self.applyDeprecated(symbol_versioning.zero_eleven,
+                tree.pending_merges))
 
 
 class TestSetParents(TestParents):

=== modified file 'bzrlib/tests/workingtree_implementations/test_pull.py'
--- bzrlib/tests/workingtree_implementations/test_pull.py	2006-06-20 03:57:11 +0000
+++ bzrlib/tests/workingtree_implementations/test_pull.py	2006-09-05 08:14:23 +0000
@@ -41,7 +41,7 @@
         tree_a, tree_b = self.get_pullable_trees()
         tree_b.pull(tree_a.branch)
         self.failUnless(tree_b.branch.repository.has_revision('A'))
-        self.assertEqual('A', tree_b.last_revision())
+        self.assertEqual(['A'], tree_b.get_parent_ids())
 
     def test_pull_overwrites(self):
         tree_a, tree_b = self.get_pullable_trees()
@@ -50,7 +50,7 @@
         tree_b.pull(tree_a.branch, overwrite=True)
         self.failUnless(tree_b.branch.repository.has_revision('A'))
         self.failUnless(tree_b.branch.repository.has_revision('B'))
-        self.assertEqual('A', tree_b.last_revision())
+        self.assertEqual(['A'], tree_b.get_parent_ids())
 
     def test_pull_merges_tree_content(self):
         tree_a, tree_b = self.get_pullable_trees()

=== added file 'bzrlib/tests/workingtree_implementations/test_put_file.py'
--- bzrlib/tests/workingtree_implementations/test_put_file.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_put_file.py	2006-09-07 23:13:08 +0000
@@ -0,0 +1,37 @@
+# (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for interface conformance of 'workingtree.put_file*'"""
+
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestPutFileBytesNonAtomic(TestCaseWithWorkingTree):
+
+    def test_put_new_file(self):
+        t = self.make_branch_and_tree('t1')
+        t.add(['foo'], ids=['foo-id'], kinds=['file'])
+        t.put_file_bytes_non_atomic('foo-id', 'barshoom')
+        self.assertEqual('barshoom', t.get_file('foo-id').read())
+
+    def test_put_existing_file(self):
+        t = self.make_branch_and_tree('t1')
+        t.add(['foo'], ids=['foo-id'], kinds=['file'])
+        t.put_file_bytes_non_atomic('foo-id', 'first-content')
+        t.put_file_bytes_non_atomic('foo-id', 'barshoom')
+        self.assertEqual('barshoom', t.get_file('foo-id').read())
+

=== added file 'bzrlib/tests/workingtree_implementations/test_read_working_inventory.py'
--- bzrlib/tests/workingtree_implementations/test_read_working_inventory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_read_working_inventory.py	2006-10-05 07:34:58 +0000
@@ -0,0 +1,40 @@
+# (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for WorkingTree.read_working_inventory."""
+
+from bzrlib import errors, inventory
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestReadWorkingInventory(TestCaseWithWorkingTree):
+
+    def test_trivial_read(self):
+        tree = self.make_branch_and_tree('t1')
+        self.assertIsInstance(tree.read_working_inventory(), inventory.Inventory)
+
+    def test_read_after_inventory_modification(self):
+        tree = self.make_branch_and_tree('tree')
+        # prepare for a series of changes that will modify the 
+        # inventory
+        tree.lock_write()
+        try:
+            tree.set_root_id('new-root')
+            # having dirtied the inventory, we can now expect an 
+            # InventoryModified exception when doing a read_working_inventory()
+            self.assertRaises(errors.InventoryModified, tree.read_working_inventory)
+        finally:
+            tree.unlock()

=== added file 'bzrlib/tests/workingtree_implementations/test_set_root_id.py'
--- bzrlib/tests/workingtree_implementations/test_set_root_id.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_set_root_id.py	2006-10-05 07:34:58 +0000
@@ -0,0 +1,57 @@
+# (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for WorkingTree.set_root_id"""
+
+from bzrlib import inventory
+from bzrlib.symbol_versioning import zero_twelve
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestSetRootId(TestCaseWithWorkingTree):
+
+    def test_set_None(self):
+        # setting the root_id to None is equivalent to setting it
+        # to inventory.ROOT_ID
+        tree = self.make_branch_and_tree('a-tree')
+        self.callDeprecated([
+            'WorkingTree.set_root_id with fileid=None was deprecated in version'
+            ' 0.12.'],
+            tree.set_root_id, None)
+        self.assertEqual(inventory.ROOT_ID, tree.get_root_id())
+
+    def test_set_and_read_unicode(self):
+        tree = self.make_branch_and_tree('a-tree')
+        # setting the root id allows it to be read via get_root_id.
+        tree.lock_write()
+        try:
+            old_id = tree.get_root_id()
+            tree.set_root_id(u'\xe5n id')
+            self.assertEqual(u'\xe5n id', tree.get_root_id())
+            # set root id should not have triggered a flush of the tree,
+            # so check a new tree sees the old state.
+            reference_tree = tree.bzrdir.open_workingtree()
+            self.assertEqual(old_id, reference_tree.get_root_id())
+        finally:
+            tree.unlock()
+        # having unlocked the tree, the value should have been 
+        # preserved into the next lock, which is an implicit read
+        # lock around the get_root_id call.
+        self.assertEqual(u'\xe5n id', tree.get_root_id())
+        # and if we get a new working tree instance, then the value
+        # should still be retained
+        tree = tree.bzrdir.open_workingtree()
+        self.assertEqual(u'\xe5n id', tree.get_root_id())

=== added file 'bzrlib/tests/workingtree_implementations/test_unversion.py'
--- bzrlib/tests/workingtree_implementations/test_unversion.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_unversion.py	2006-09-07 08:13:22 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests of the WorkingTree.unversion API."""
+
+from bzrlib import errors
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestUnversion(TestCaseWithWorkingTree):
+
+    def test_unversion_requires_write_lock(self):
+        """WT.unversion([]) in a read lock raises ReadOnlyError."""
+        tree = self.make_branch_and_tree('.')
+        tree.lock_read()
+        self.assertRaises(errors.ReadOnlyError, tree.unversion, [])
+        tree.unlock()
+
+    def test_unversion_missing_file(self):
+        """WT.unversion(['missing-id']) raises NoSuchId."""
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(errors.NoSuchId, tree.unversion, ['missing-id'])
+
+    def test_unversion_several_files(self):
+        """After unversioning several files, they should not be versioned."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b', 'c'])
+        tree.add(['a', 'b', 'c'], ['a-id', 'b-id', 'c-id'])
+        # within a lock unversion should take effect
+        tree.lock_write()
+        tree.unversion(['a-id', 'b-id'])
+        self.assertFalse(tree.has_id('a-id'))
+        self.assertFalse(tree.has_id('b-id'))
+        self.assertTrue(tree.has_id('c-id'))
+        self.assertTrue(tree.has_filename('a'))
+        self.assertTrue(tree.has_filename('b'))
+        self.assertTrue(tree.has_filename('c'))
+        tree.unlock()
+        # the changes should have persisted to disk - reopen the workingtree
+        # to be sure.
+        tree = tree.bzrdir.open_workingtree()
+        self.assertFalse(tree.has_id('a-id'))
+        self.assertFalse(tree.has_id('b-id'))
+        self.assertTrue(tree.has_id('c-id'))
+        self.assertTrue(tree.has_filename('a'))
+        self.assertTrue(tree.has_filename('b'))
+        self.assertTrue(tree.has_filename('c'))
+        
+    def test_unversion_subtree(self):
+        """Unversioning the root of a subtree unversions the entire subtree."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'c'])
+        tree.add(['a', 'a/b', 'c'], ['a-id', 'b-id', 'c-id'])
+        # within a lock unversion should take effect
+        tree.lock_write()
+        tree.unversion(['a-id'])
+        self.assertFalse(tree.has_id('a-id'))
+        self.assertFalse(tree.has_id('b-id'))
+        self.assertTrue(tree.has_id('c-id'))
+        self.assertTrue(tree.has_filename('a'))
+        self.assertTrue(tree.has_filename('a/b'))
+        self.assertTrue(tree.has_filename('c'))
+        tree.unlock()

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-05 21:14:35 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-10 19:04:48 +0000
@@ -19,7 +19,6 @@
 import os
 import sys
 
-from bzrlib import ignores
 import bzrlib
 from bzrlib import branch, bzrdir, errors, osutils, urlutils, workingtree
 from bzrlib.errors import (NotBranchError, NotVersionedError,
@@ -49,14 +48,12 @@
 
     def test_list_files_sorted(self):
         tree = self.make_branch_and_tree('.')
-        ignores._set_user_ignores(['./.bazaar'])
         self.build_tree(['dir/', 'file', 'dir/file', 'dir/b',
                          'dir/subdir/', 'a', 'dir/subfile',
                          'zz_dir/', 'zz_dir/subfile'])
         files = [(path, kind) for (path, v, kind, file_id, entry)
                                in tree.list_files()]
         self.assertEqual([
-            ('.bazaar', 'directory'),
             ('a', 'file'),
             ('dir', 'directory'),
             ('file', 'file'),
@@ -67,7 +64,6 @@
         files = [(path, kind) for (path, v, kind, file_id, entry)
                                in tree.list_files()]
         self.assertEqual([
-            ('.bazaar', 'directory'),
             ('a', 'file'),
             ('dir', 'directory'),
             ('dir/b', 'file'),
@@ -267,32 +263,31 @@
         wt = self.make_branch_and_tree('source')
         cloned_dir = wt.bzrdir.clone('target')
         cloned = cloned_dir.open_workingtree()
-        self.assertEqual(cloned.last_revision(), wt.last_revision())
+        self.assertEqual(cloned.get_parent_ids(), wt.get_parent_ids())
 
     def test_last_revision(self):
         wt = self.make_branch_and_tree('source')
-        self.assertEqual(None, wt.last_revision())
+        self.assertEqual([], wt.get_parent_ids())
         wt.commit('A', allow_pointless=True, rev_id='A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
 
     def test_set_last_revision(self):
         wt = self.make_branch_and_tree('source')
-        self.assertEqual(None, wt.last_revision())
         # set last-revision to one not in the history
         wt.set_last_revision('A')
         # set it back to None for an empty tree.
         wt.set_last_revision(None)
         wt.commit('A', allow_pointless=True, rev_id='A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
         # None is aways in the branch
         wt.set_last_revision(None)
-        self.assertEqual(None, wt.last_revision())
+        self.assertEqual([], wt.get_parent_ids())
         # and now we can set it to 'A'
         # because some formats mutate the branch to set it on the tree
         # we need to alter the branch to let this pass.
         wt.branch.set_revision_history(['A', 'B'])
         wt.set_last_revision('A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
 
     def test_set_last_revision_different_to_branch(self):
         # working tree formats from the meta-dir format and newer support
@@ -313,19 +308,20 @@
         wt = branch.bzrdir.create_workingtree()
         wt.commit('A', allow_pointless=True, rev_id='A')
         wt.set_last_revision(None)
-        self.assertEqual(None, wt.last_revision())
+        self.assertEqual([], wt.get_parent_ids())
         self.assertEqual('A', wt.branch.last_revision())
         # and now we can set it back to 'A'
         wt.set_last_revision('A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
         self.assertEqual('A', wt.branch.last_revision())
 
     def test_clone_and_commit_preserves_last_revision(self):
+        """Doing a commit into a clone tree does not affect the source."""
         wt = self.make_branch_and_tree('source')
         cloned_dir = wt.bzrdir.clone('target')
         wt.commit('A', allow_pointless=True, rev_id='A')
-        self.assertNotEqual(cloned_dir.open_workingtree().last_revision(),
-                            wt.last_revision())
+        self.assertNotEqual(cloned_dir.open_workingtree().get_parent_ids(),
+                            wt.get_parent_ids())
 
     def test_clone_preserves_content(self):
         wt = self.make_branch_and_tree('source')
@@ -384,7 +380,7 @@
         source.branch.repository.clone(made_control)
         source.branch.clone(made_control)
         made_tree = self.workingtree_format.initialize(made_control, revision_id='a')
-        self.assertEqual('a', made_tree.last_revision())
+        self.assertEqual(['a'], made_tree.get_parent_ids())
 
     def test_update_sets_last_revision(self):
         # working tree formats from the meta-dir format and newer support
@@ -415,7 +411,29 @@
         # and update old_tree
         self.assertEqual(0, old_tree.update())
         self.failUnlessExists('checkout/file')
-        self.assertEqual('A', old_tree.last_revision())
+        self.assertEqual(['A'], old_tree.get_parent_ids())
+
+    def test_update_sets_root_id(self):
+        """Ensure tree root is set properly by update.
+        
+        Since empty trees don't have root_ids, but workingtrees do,
+        an update of a checkout of revision 0 to a new revision,  should set
+        the root id.
+        """
+        wt = self.make_branch_and_tree('tree')
+        main_branch = wt.branch
+        # create an out of date working tree by making a checkout in this
+        # current format
+        self.build_tree(['checkout/', 'tree/file'])
+        checkout = main_branch.create_checkout('checkout')
+        # now commit to 'tree'
+        wt.add('file')
+        wt.commit('A', rev_id='A')
+        # and update checkout 
+        self.assertEqual(0, checkout.update())
+        self.failUnlessExists('checkout/file')
+        self.assertEqual(wt.get_root_id(), checkout.get_root_id())
+        self.assertNotEqual(None, wt.get_root_id())
 
     def test_update_returns_conflict_count(self):
         # working tree formats from the meta-dir format and newer support
@@ -448,7 +466,7 @@
         old_tree.add('file')
         # and update old_tree
         self.assertEqual(1, old_tree.update())
-        self.assertEqual('A', old_tree.last_revision())
+        self.assertEqual(['A'], old_tree.get_parent_ids())
 
     def test_merge_revert(self):
         from bzrlib.merge import merge_inner
@@ -487,7 +505,7 @@
             return
         master_tree.commit('foo', rev_id='foo', allow_pointless=True)
         tree.update()
-        self.assertEqual('foo', tree.last_revision())
+        self.assertEqual(['foo'], tree.get_parent_ids())
         self.assertEqual('foo', tree.branch.last_revision())
 
     def test_update_turns_local_commit_into_merge(self):
@@ -516,9 +534,10 @@
         # sync with master prepatory to committing
         tree.update()
         # which should have pivoted the local tip into a merge
-        self.assertEqual(master_tip, tree.last_revision())
-        self.assertEqual([master_tip], tree.branch.revision_history())
         self.assertEqual([master_tip, 'bar'], tree.get_parent_ids())
+        # and the local branch history should match the masters now.
+        self.assertEqual(master_tree.branch.revision_history(),
+            tree.branch.revision_history())
 
     def test_merge_modified(self):
         tree = self.make_branch_and_tree('master')

=== modified file 'bzrlib/textinv.py'
--- bzrlib/textinv.py	2006-08-07 23:14:45 +0000
+++ bzrlib/textinv.py	2006-08-07 23:19:38 +0000
@@ -52,9 +52,10 @@
 def write_text_inventory(inv, outf):
     """Write out inv in a simple trad-unix text format."""
     outf.write(START_MARK)
-    entries = inv.iter_entries()
-    entries.next()
-    for path, ie in entries:
+    for path, ie in inv.iter_entries():
+        if inv.is_root(ie.file_id):
+            continue
+        
         outf.write(ie.file_id + ' ')
         outf.write(escape(ie.name) + ' ')
         outf.write(ie.kind + ' ')

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2006-08-22 21:39:33 +0000
+++ bzrlib/trace.py	2006-10-06 07:14:31 +0000
@@ -50,14 +50,17 @@
 # is quite expensive, even when the message is not printed by any handlers.
 # We should perhaps change back to just simply doing it here.
 
-
-import errno
 import os
 import sys
+import re
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import errno
 import logging
+""")
 
 import bzrlib
-from bzrlib.errors import BzrError, BzrNewError
 from bzrlib.symbol_versioning import (deprecated_function,
         zero_nine,
         )
@@ -93,7 +96,7 @@
 def mutter(fmt, *args):
     if _trace_file is None:
         return
-    if hasattr(_trace_file, 'closed') and _trace_file.closed:
+    if (getattr(_trace_file, 'closed', None) is not None) and _trace_file.closed:
         return
 
     if isinstance(fmt, unicode):
@@ -168,9 +171,6 @@
     """
     if msg:
         error(msg)
-    else:
-        exc_str = format_exception_short(sys.exc_info())
-        error(exc_str)
     log_exception_quietly()
 
 
@@ -288,7 +288,8 @@
     """Report an exception that probably indicates a bug in bzr"""
     import traceback
     exc_type, exc_object, exc_tb = exc_info
-    print >>err_file, "bzr: ERROR: %s: %s" % (exc_type, exc_object)
+    print >>err_file, "bzr: ERROR: %s.%s: %s" % (
+        exc_type.__module__, exc_type.__name__, exc_object)
     print >>err_file
     traceback.print_exception(exc_type, exc_object, exc_tb, file=err_file)
     print >>err_file

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-09-06 02:47:54 +0000
+++ bzrlib/transform.py	2006-09-28 01:53:23 +0000
@@ -21,7 +21,7 @@
 from bzrlib import bzrdir, errors
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
                            ReusingTransform, NotVersionedError, CantMoveRoot,
-                           ExistingLimbo, ImmortalLimbo)
+                           ExistingLimbo, ImmortalLimbo, NoFinalPath)
 from bzrlib.inventory import InventoryEntry
 from bzrlib.osutils import (file_kind, supports_executable, pathjoin, lexists,
                             delete_any)
@@ -73,13 +73,13 @@
      * set_executability
     """
     def __init__(self, tree, pb=DummyProgress()):
-        """Note: a write lock is taken on the tree.
+        """Note: a tree_write lock is taken on the tree.
         
         Use TreeTransform.finalize() to release the lock
         """
         object.__init__(self)
         self._tree = tree
-        self._tree.lock_write()
+        self._tree.lock_tree_write()
         try:
             control_files = self._tree._control_files
             self._limbodir = urlutils.local_path_from_url(
@@ -292,7 +292,7 @@
         except KeyError:
             return
         try:
-            mode = os.stat(old_path).st_mode
+            mode = os.stat(self._tree.abspath(old_path)).st_mode
         except OSError, e:
             if e.errno == errno.ENOENT:
                 return
@@ -457,7 +457,10 @@
         try:
             return self._new_name[trans_id]
         except KeyError:
-            return os.path.basename(self._tree_id_paths[trans_id])
+            try:
+                return os.path.basename(self._tree_id_paths[trans_id])
+            except KeyError:
+                raise NoFinalPath(trans_id, self)
 
     def by_parent(self):
         """Return a map of parent: children for known parents.
@@ -570,7 +573,10 @@
             parent_id = trans_id
             while parent_id is not ROOT_PARENT:
                 seen.add(parent_id)
-                parent_id = self.final_parent(parent_id)
+                try:
+                    parent_id = self.final_parent(parent_id)
+                except KeyError:
+                    break
                 if parent_id == trans_id:
                     conflicts.append(('parent loop', trans_id))
                 if parent_id in seen:
@@ -958,6 +964,8 @@
     file_trans_id = {}
     top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
+    if tree.inventory.root is not None:
+        wt.set_root_id(tree.inventory.root.file_id)
     tt = TreeTransform(wt)
     divert = set()
     try:
@@ -1086,7 +1094,7 @@
 def create_by_entry(tt, entry, tree, trans_id, lines=None, mode_id=None):
     """Create new file contents according to an inventory entry."""
     if entry.kind == "file":
-        if lines == None:
+        if lines is None:
             lines = tree.get_file(entry.file_id).readlines()
         tt.create_file(lines, trans_id, mode_id=mode_id)
     elif entry.kind == "symlink":
@@ -1229,8 +1237,12 @@
         pp.next_phase()
         wt_interesting = [i for i in working_tree.inventory if interesting(i)]
         child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        basis_tree = None
         try:
             for id_num, file_id in enumerate(wt_interesting):
+                if (working_tree.inventory.is_root(file_id) and 
+                    len(target_tree.inventory) == 0):
+                    continue
                 child_pb.update("New file check", id_num+1, 
                                 len(sorted_interesting))
                 if file_id not in target_tree:
@@ -1240,18 +1252,21 @@
                         file_kind = working_tree.kind(file_id)
                     except NoSuchFile:
                         file_kind = None
+                    delete_merge_modified = (file_id in merge_modified)
                     if file_kind != 'file' and file_kind is not None:
                         keep_contents = False
-                        delete_merge_modified = False
                     else:
+                        if basis_tree is None:
+                            basis_tree = working_tree.basis_tree()
+                        wt_sha1 = working_tree.get_file_sha1(file_id)
                         if (file_id in merge_modified and 
-                            merge_modified[file_id] == 
-                            working_tree.get_file_sha1(file_id)):
-                            keep_contents = False
-                            delete_merge_modified = True
+                            merge_modified[file_id] == wt_sha1):
+                            keep_contents = False
+                        elif (file_id in basis_tree and 
+                            basis_tree.get_file_sha1(file_id) == wt_sha1):
+                            keep_contents = False
                         else:
                             keep_contents = True
-                            delete_merge_modified = False
                     if not keep_contents:
                         tt.delete_contents(trans_id)
                     if delete_merge_modified:

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-09-06 02:57:51 +0000
+++ bzrlib/transport/__init__.py	2006-10-13 06:41:04 +0000
@@ -26,14 +26,17 @@
 it.
 """
 
+from cStringIO import StringIO
+import re
+import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import errno
 from collections import deque
 from copy import deepcopy
-from cStringIO import StringIO
-import re
 from stat import S_ISDIR
-import sys
-from unittest import TestSuite
+import unittest
 import urllib
 import urlparse
 import warnings
@@ -45,8 +48,8 @@
     symbol_versioning,
     urlutils,
     )
-from bzrlib.errors import DependencyNotPresent
-from bzrlib.osutils import pumpfile
+""")
+
 from bzrlib.symbol_versioning import (
         deprecated_passed,
         deprecated_method,
@@ -165,7 +168,8 @@
             port = int(port)
         except ValueError:
             # TODO: Should this be ConnectionError?
-            raise errors.TransportError('%s: invalid port number' % port)
+            raise errors.TransportError(
+                'invalid port number %s in url:\n%s' % (port, url))
     host = urllib.unquote(host)
 
     path = urllib.unquote(path)
@@ -198,6 +202,8 @@
     implementations can do pipelining.
     In general implementations should support having a generator or a list
     as an argument (ie always iterate, never index)
+
+    :ivar base: Base URL for the transport; should always end in a slash.
     """
 
     # implementations can override this if it is more efficient
@@ -220,7 +226,7 @@
 
         This handles things like ENOENT, ENOTDIR, EEXIST, and EACCESS
         """
-        if hasattr(e, 'errno'):
+        if getattr(e, 'errno', None) is not None:
             if e.errno in (errno.ENOENT, errno.ENOTDIR):
                 raise errors.NoSuchFile(path, extra=e)
             # I would rather use errno.EFOO, but there doesn't seem to be
@@ -259,7 +265,7 @@
         """
         assert not isinstance(from_file, basestring), \
             '_pump should only be called on files not %s' % (type(from_file,))
-        pumpfile(from_file, to_file)
+        osutils.pumpfile(from_file, to_file)
 
     def _get_total(self, multi):
         """Try to figure out how many entries are in multi,
@@ -304,13 +310,59 @@
 
     def abspath(self, relpath):
         """Return the full url to the given relative path.
-        This can be supplied with a string or a list
 
-        XXX: Robert Collins 20051016 - is this really needed in the public
-             interface ?
+        :param relpath: a string of a relative path
         """
+
+        # XXX: Robert Collins 20051016 - is this really needed in the public
+        # interface ?
         raise NotImplementedError(self.abspath)
 
+    def _combine_paths(self, base_path, relpath):
+        """Transform a Transport-relative path to a remote absolute path.
+
+        This does not handle substitution of ~ but does handle '..' and '.'
+        components.
+
+        Examples::
+
+            t._combine_paths('/home/sarah', 'project/foo')
+                => '/home/sarah/project/foo'
+            t._combine_paths('/home/sarah', '../../etc')
+                => '/etc'
+
+        :param base_path: urlencoded path for the transport root; typically a 
+             URL but need not contain scheme/host/etc.
+        :param relpath: relative url string for relative part of remote path.
+        :return: urlencoded string for final path.
+        """
+        # FIXME: share the common code across more transports; variants of
+        # this likely occur in http and sftp too.
+        #
+        # TODO: Also need to consider handling of ~, which might vary between
+        # transports?
+        if not isinstance(relpath, str):
+            raise errors.InvalidURL("not a valid url: %r" % relpath)
+        if relpath.startswith('/'):
+            base_parts = []
+        else:
+            base_parts = base_path.split('/')
+        if len(base_parts) > 0 and base_parts[-1] == '':
+            base_parts = base_parts[:-1]
+        for p in relpath.split('/'):
+            if p == '..':
+                if len(base_parts) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                base_parts.pop()
+            elif p == '.':
+                continue # No-op
+            elif p != '':
+                base_parts.append(p)
+        path = '/'.join(base_parts)
+        return path
+
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.
 
@@ -342,6 +394,8 @@
         Note that some transports MAY allow querying on directories, but this
         is not part of the protocol.  In other words, the results of 
         t.has("a_directory_name") are undefined.
+
+        :rtype: bool
         """
         raise NotImplementedError(self.has)
 
@@ -378,6 +432,7 @@
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
+        :rtype: File-like object.
         """
         raise NotImplementedError(self.get)
 
@@ -388,6 +443,26 @@
         """
         return self.get(relpath).read()
 
+    def get_smart_client(self):
+        """Return a smart client for this transport if possible.
+
+        A smart client doesn't imply the presence of a smart server: it implies
+        that the smart protocol can be tunnelled via this transport.
+
+        :raises NoSmartServer: if no smart server client is available.
+        """
+        raise errors.NoSmartServer(self.base)
+
+    def get_smart_medium(self):
+        """Return a smart client medium for this transport if possible.
+
+        A smart medium doesn't imply the presence of a smart server: it implies
+        that the smart protocol can be tunnelled via this transport.
+
+        :raises NoSmartMedium: if no smart server medium is available.
+        """
+        raise errors.NoSmartMedium(self)
+
     def readv(self, relpath, offsets):
         """Get parts of the file at the given relative path.
 
@@ -398,9 +473,9 @@
             return
 
         fp = self.get(relpath)
-        return self._seek_and_read(fp, offsets)
+        return self._seek_and_read(fp, offsets, relpath)
 
-    def _seek_and_read(self, fp, offsets):
+    def _seek_and_read(self, fp, offsets, relpath='<unknown>'):
         """An implementation of readv that uses fp.seek and fp.read.
 
         This uses _coalesce_offsets to issue larger reads and fewer seeks.
@@ -428,6 +503,9 @@
             #       benchmarked.
             fp.seek(c_offset.start)
             data = fp.read(c_offset.length)
+            if len(data) < c_offset.length:
+                raise errors.ShortReadvError(relpath, c_offset.start,
+                            c_offset.length, actual=len(data))
             for suboffset, subsize in c_offset.ranges:
                 key = (c_offset.start+suboffset, subsize)
                 data_map[key] = data[suboffset:suboffset+subsize]
@@ -526,7 +604,8 @@
         return self.put_file(relpath, StringIO(bytes), mode=mode)
 
     def put_bytes_non_atomic(self, relpath, bytes, mode=None,
-                             create_parent_dir=False):
+                             create_parent_dir=False,
+                             dir_mode=None):
         """Copy the string into the target location.
 
         This function is not strictly safe to use. See 
@@ -540,11 +619,13 @@
         :param create_parent_dir: If we cannot create the target file because
                         the parent directory does not exist, go ahead and
                         create it, and then try again.
+        :param dir_mode: Possible access permissions for new directories.
         """
         assert isinstance(bytes, str), \
             'bytes must be a plain string, not %s' % type(bytes)
         self.put_file_non_atomic(relpath, StringIO(bytes), mode=mode,
-                                 create_parent_dir=create_parent_dir)
+                                 create_parent_dir=create_parent_dir,
+                                 dir_mode=dir_mode)
 
     def put_file(self, relpath, f, mode=None):
         """Copy the file-like object into the location.
@@ -565,7 +646,8 @@
         #raise NotImplementedError(self.put_file)
 
     def put_file_non_atomic(self, relpath, f, mode=None,
-                            create_parent_dir=False):
+                            create_parent_dir=False,
+                            dir_mode=None):
         """Copy the file-like object into the target location.
 
         This function is not strictly safe to use. It is only meant to
@@ -580,6 +662,7 @@
         :param create_parent_dir: If we cannot create the target file because
                         the parent directory does not exist, go ahead and
                         create it, and then try again.
+        :param dir_mode: Possible access permissions for new directories.
         """
         # Default implementation just does an atomic put.
         try:
@@ -589,7 +672,7 @@
                 raise
             parent_dir = osutils.dirname(relpath)
             if parent_dir:
-                self.mkdir(parent_dir)
+                self.mkdir(parent_dir, mode=dir_mode)
                 return self.put_file(relpath, f, mode=mode)
 
     @deprecated_method(zero_eleven)
@@ -629,11 +712,15 @@
         return self.append_file(relpath, f, mode=mode)
 
     def append_file(self, relpath, f, mode=None):
-        """Append the text in the file-like object to the supplied location.
-
-        returns the length of relpath before the content was written to it.
-        
-        If the file does not exist, it is created with the supplied mode.
+        """Append bytes from a file-like object to a file at relpath.
+
+        The file is created if it does not already exist.
+
+        :param f: a file-like object of the bytes to append.
+        :param mode: Unix mode for newly created files.  This is not used for
+            existing files.
+
+        :returns: the length of relpath before the content was written to it.
         """
         symbol_versioning.warn('Transport %s should implement append_file,'
                                ' rather than implementing append() as of'
@@ -643,11 +730,16 @@
         return self.append(relpath, f, mode=mode)
 
     def append_bytes(self, relpath, bytes, mode=None):
-        """Append the text in the string object to the supplied location.
-
-        returns the length of relpath before the content was written to it.
-        
-        If the file does not exist, it is created with the supplied mode.
+        """Append bytes to a file at relpath.
+
+        The file is created if it does not already exist.
+
+        :type f: str
+        :param f: a string of the bytes to append.
+        :param mode: Unix mode for newly created files.  This is not used for
+            existing files.
+
+        :returns: the length of relpath before the content was written to it.
         """
         assert isinstance(bytes, str), \
             'bytes must be a plain string, not %s' % type(bytes)
@@ -843,26 +935,37 @@
         WARNING: many transports do not support this, so trying avoid using
         it if at all possible.
         """
-        raise errors.TransportNotPossible("This transport has not "
+        raise errors.TransportNotPossible("Transport %r has not "
                                           "implemented list_dir "
                                           "(but must claim to be listable "
-                                          "to trigger this error).")
+                                          "to trigger this error)."
+                                          % (self))
 
     def lock_read(self, relpath):
         """Lock the given file for shared (read) access.
-        WARNING: many transports do not support this, so trying avoid using it
+
+        WARNING: many transports do not support this, so trying avoid using it.
+        These methods may be removed in the future.
+
+        Transports may raise TransportNotPossible if OS-level locks cannot be
+        taken over this transport.  
 
         :return: A lock object, which should contain an unlock() function.
         """
-        raise NotImplementedError(self.lock_read)
+        raise errors.TransportNotPossible("transport locks not supported on %s" % self)
 
     def lock_write(self, relpath):
         """Lock the given file for exclusive (write) access.
-        WARNING: many transports do not support this, so trying avoid using it
+
+        WARNING: many transports do not support this, so trying avoid using it.
+        These methods may be removed in the future.
+
+        Transports may raise TransportNotPossible if OS-level locks cannot be
+        taken over this transport.
 
         :return: A lock object, which should contain an unlock() function.
         """
-        raise NotImplementedError(self.lock_write)
+        raise errors.TransportNotPossible("transport locks not supported on %s" % self)
 
     def is_readonly(self):
         """Return true if this connection cannot be written to."""
@@ -943,7 +1046,7 @@
     for factory in factory_list:
         try:
             return factory(base), None
-        except DependencyNotPresent, e:
+        except errors.DependencyNotPresent, e:
             mutter("failed to instantiate transport %r for %r: %r" %
                     (factory, base, e))
             last_err = e
@@ -996,7 +1099,7 @@
     """
 
     def adapt(self, test):
-        result = TestSuite()
+        result = unittest.TestSuite()
         for klass, server_factory in self._test_permutations():
             new_test = deepcopy(test)
             new_test.transport_class = klass
@@ -1010,7 +1113,7 @@
 
     def get_transport_test_permutations(self, module):
         """Get the permutations module wants to have tested."""
-        if not hasattr(module, 'get_test_permutations'):
+        if getattr(module, 'get_test_permutations', None) is None:
             warning("transport module %s doesn't provide get_test_permutations()"
                     % module.__name__)
             return []
@@ -1078,6 +1181,12 @@
 register_lazy_transport('memory://', 'bzrlib.transport.memory', 'MemoryTransport')
 register_lazy_transport('readonly+', 'bzrlib.transport.readonly', 'ReadonlyTransportDecorator')
 register_lazy_transport('fakenfs+', 'bzrlib.transport.fakenfs', 'FakeNFSTransportDecorator')
-register_lazy_transport('vfat+', 
+register_lazy_transport('vfat+',
                         'bzrlib.transport.fakevfat',
                         'FakeVFATTransportDecorator')
+register_lazy_transport('bzr://',
+                        'bzrlib.transport.smart',
+                        'SmartTCPTransport')
+register_lazy_transport('bzr+ssh://',
+                        'bzrlib.transport.smart',
+                        'SmartSSHTransport')

=== modified file 'bzrlib/transport/decorator.py'
--- bzrlib/transport/decorator.py	2006-09-05 21:20:24 +0000
+++ bzrlib/transport/decorator.py	2006-09-13 13:43:21 +0000
@@ -88,6 +88,9 @@
         """See Transport.get()."""
         return self._decorated.get(relpath)
 
+    def get_smart_client(self):
+        return self._decorated.get_smart_client()
+
     def has(self, relpath):
         """See Transport.has()."""
         return self._decorated.has(relpath)

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2006-09-05 21:20:24 +0000
+++ bzrlib/transport/ftp.py	2006-09-15 01:12:18 +0000
@@ -196,15 +196,13 @@
     def _abspath(self, relpath):
         assert isinstance(relpath, basestring)
         relpath = urlutils.unescape(relpath)
-        relpath_parts = relpath.split('/')
-        if len(relpath_parts) > 1:
-            if relpath_parts[0] == '':
-                raise ValueError("path %r within branch %r seems to be absolute"
-                                 % (relpath, self._path))
-        basepath = self._path.split('/')
+        if relpath.startswith('/'):
+            basepath = []
+        else:
+            basepath = self._path.split('/')
         if len(basepath) > 0 and basepath[-1] == '':
             basepath = basepath[:-1]
-        for p in relpath_parts:
+        for p in relpath.split('/'):
             if p == '..':
                 if len(basepath) == 0:
                     # In most filesystems, a request for the parent
@@ -307,7 +305,7 @@
         abspath = self._abspath(relpath)
         tmp_abspath = '%s.tmp.%.9f.%d.%d' % (abspath, time.time(),
                         os.getpid(), random.randint(0,0x7FFFFFFF))
-        if not hasattr(fp, 'read'):
+        if getattr(fp, 'read', None) is None:
             fp = StringIO(fp)
         try:
             mutter("FTP put: %s", abspath)

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2006-09-05 21:20:24 +0000
+++ bzrlib/transport/http/__init__.py	2006-10-09 07:34:15 +0000
@@ -39,7 +39,13 @@
                            TransportError, ConnectionError, InvalidURL)
 from bzrlib.branch import Branch
 from bzrlib.trace import mutter
-from bzrlib.transport import Transport, register_transport, Server
+from bzrlib.transport import (
+    get_transport,
+    register_transport,
+    Server,
+    smart,
+    Transport,
+    )
 from bzrlib.transport.http.response import (HttpMultipartRangeResponse,
                                             HttpRangeResponse)
 from bzrlib.ui import ui_factory
@@ -119,7 +125,7 @@
     return m
 
 
-class HttpTransportBase(Transport):
+class HttpTransportBase(Transport, smart.SmartClientMedium):
     """Base class for http implementations.
 
     Does URL parsing, etc, but not any network IO.
@@ -169,19 +175,17 @@
         else:
             # TODO: Don't call this with an array - no magic interfaces
             relpath_parts = relpath[:]
-        if len(relpath_parts) > 1:
-            # TODO: Check that the "within branch" part of the
-            # error messages below is relevant in all contexts
-            if relpath_parts[0] == '':
-                raise ValueError("path %r within branch %r seems to be absolute"
-                                 % (relpath, self._path))
-            # read only transports never manipulate directories
-            if self.is_readonly() and relpath_parts[-1] == '':
+        if relpath.startswith('/'):
+            basepath = []
+        else:
+            # Except for the root, no trailing slashes are allowed
+            if len(relpath_parts) > 1 and relpath_parts[-1] == '':
                 raise ValueError("path %r within branch %r seems to be a directory"
                                  % (relpath, self._path))
-        basepath = self._path.split('/')
-        if len(basepath) > 0 and basepath[-1] == '':
-            basepath = basepath[:-1]
+            basepath = self._path.split('/')
+            if len(basepath) > 0 and basepath[-1] == '':
+                basepath = basepath[:-1]
+
         for p in relpath_parts:
             if p == '..':
                 if len(basepath) == 0:
@@ -240,6 +244,17 @@
         """
         raise NotImplementedError(self._get)
 
+    def get_request(self):
+        return SmartClientHTTPMediumRequest(self)
+
+    def get_smart_medium(self):
+        """See Transport.get_smart_medium.
+
+        HttpTransportBase directly implements the minimal interface of
+        SmartMediumClient, so this returns self.
+        """
+        return self
+
     def readv(self, relpath, offsets):
         """Get parts of the file at the given relative path.
 
@@ -254,7 +269,9 @@
             f.seek(start, (start < 0) and 2 or 0)
             start = f.tell()
             data = f.read(size)
-            assert len(data) == size
+            if len(data) != size:
+                raise errors.ShortReadvError(relpath, start, size,
+                                             actual=len(data))
             yield start, data
 
     @staticmethod
@@ -284,6 +301,17 @@
 
         return combined
 
+    def _post(self, body_bytes):
+        """POST body_bytes to .bzr/smart on this transport.
+        
+        :returns: (response code, response body file-like object).
+        """
+        # TODO: Requiring all the body_bytes to be available at the beginning of
+        # the POST may require large client buffers.  It would be nice to have
+        # an interface that allows streaming via POST when possible (and
+        # degrades to a local buffer when not).
+        raise NotImplementedError(self._post)
+
     def put_file(self, relpath, f, mode=None):
         """Copy the file-like object into the location.
 
@@ -371,13 +399,14 @@
 
     def clone(self, offset=None):
         """Return a new HttpTransportBase with root at self.base + offset
-        For now HttpTransportBase does not actually connect, so just return
-        a new HttpTransportBase object.
+
+        We leave the daughter classes take advantage of the hint
+        that it's a cloning not a raw creation.
         """
         if offset is None:
-            return self.__class__(self.base)
+            return self.__class__(self.base, self)
         else:
-            return self.__class__(self.abspath(offset))
+            return self.__class__(self.abspath(offset), self)
 
     @staticmethod
     def range_header(ranges, tail_amount):
@@ -397,6 +426,33 @@
 
         return ','.join(strings)
 
+    def send_http_smart_request(self, bytes):
+        code, body_filelike = self._post(bytes)
+        assert code == 200, 'unexpected HTTP response code %r' % (code,)
+        return body_filelike
+
+
+class SmartClientHTTPMediumRequest(smart.SmartClientMediumRequest):
+    """A SmartClientMediumRequest that works with an HTTP medium."""
+
+    def __init__(self, medium):
+        smart.SmartClientMediumRequest.__init__(self, medium)
+        self._buffer = ''
+
+    def _accept_bytes(self, bytes):
+        self._buffer += bytes
+
+    def _finished_writing(self):
+        data = self._medium.send_http_smart_request(self._buffer)
+        self._response_body = data
+
+    def _read_bytes(self, count):
+        return self._response_body.read(count)
+        
+    def _finished_reading(self):
+        """See SmartClientMediumRequest._finished_reading."""
+        pass
+        
 
 #---------------- test server facilities ----------------
 # TODO: load these only when running tests
@@ -449,7 +505,7 @@
         if not self.parse_request(): # An error code has been sent, just exit
             return
         mname = 'do_' + self.command
-        if not hasattr(self, mname):
+        if getattr(self, mname, None) is None:
             self.send_error(501, "Unsupported method (%r)" % self.command)
             return
         method = getattr(self, mname)
@@ -502,11 +558,13 @@
         Server.__init__(self)
         self.request_handler = request_handler
 
-    def _http_start(self):
-        httpd = None
-        httpd = TestingHTTPServer(('localhost', 0),
+    def _get_httpd(self):
+        return TestingHTTPServer(('localhost', 0),
                                   self.request_handler,
                                   self)
+
+    def _http_start(self):
+        httpd = self._get_httpd()
         host, port = httpd.socket.getsockname()
         self._http_base_url = '%s://localhost:%s/' % (self._url_protocol, port)
         self._http_starting.release()
@@ -570,3 +628,45 @@
         # etc
         return 'http://127.0.0.1:1/'
 
+
+class HTTPServerWithSmarts(HttpServer):
+    """HTTPServerWithSmarts extends the HttpServer with POST methods that will
+    trigger a smart server to execute with a transport rooted at the rootdir of
+    the HTTP server.
+    """
+
+    def __init__(self):
+        HttpServer.__init__(self, SmartRequestHandler)
+
+
+class SmartRequestHandler(TestingHTTPRequestHandler):
+    """Extend TestingHTTPRequestHandler to support smart client POSTs."""
+
+    def do_POST(self):
+        """Hand the request off to a smart server instance."""
+        self.send_response(200)
+        self.send_header("Content-type", "application/octet-stream")
+        transport = get_transport(self.server.test_case._home_dir)
+        # TODO: We might like to support streaming responses.  1.0 allows no
+        # Content-length in this case, so for integrity we should perform our
+        # own chunking within the stream.
+        # 1.1 allows chunked responses, and in this case we could chunk using
+        # the HTTP chunking as this will allow HTTP persistence safely, even if
+        # we have to stop early due to error, but we would also have to use the
+        # HTTP trailer facility which may not be widely available.
+        out_buffer = StringIO()
+        smart_protocol_request = smart.SmartServerRequestProtocolOne(
+                transport, out_buffer.write)
+        # if this fails, we should return 400 bad request, but failure is
+        # failure for now - RBC 20060919
+        data_length = int(self.headers['Content-Length'])
+        # Perhaps there should be a SmartServerHTTPMedium that takes care of
+        # feeding the bytes in the http request to the smart_protocol_request,
+        # but for now it's simpler to just feed the bytes directly.
+        smart_protocol_request.accept_bytes(self.rfile.read(data_length))
+        assert smart_protocol_request.next_read_size() == 0, (
+            "not finished reading, but all data sent to protocol.")
+        self.send_header("Content-Length", str(len(out_buffer.getvalue())))
+        self.end_headers()
+        self.wfile.write(out_buffer.getvalue())
+

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-08-31 21:34:20 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-10-11 07:16:15 +0000
@@ -95,6 +95,7 @@
         abspath = self._real_abspath(relpath)
         curl.setopt(pycurl.URL, abspath)
         self._set_curl_options(curl)
+        curl.setopt(pycurl.HTTPGET, 1)
         # don't want the body - ie just do a HEAD request
         # This means "NO BODY" not 'nobody'
         curl.setopt(pycurl.NOBODY, 1)
@@ -115,6 +116,14 @@
             return self._get_full(relpath)
     
     def _setup_get_request(self, curl, relpath):
+        # Make sure we do a GET request. versions > 7.14.1 also set the
+        # NO BODY flag, but we'll do it ourselves in case it is an older
+        # pycurl version
+        curl.setopt(pycurl.NOBODY, 0)
+        curl.setopt(pycurl.HTTPGET, 1)
+        return self._setup_request(curl, relpath)
+
+    def _setup_request(self, curl, relpath):
         """Do the common setup stuff for making a request
 
         :param curl: The curl object to place the request on
@@ -127,11 +136,6 @@
         abspath = self._real_abspath(relpath)
         curl.setopt(pycurl.URL, abspath)
         self._set_curl_options(curl)
-        # Make sure we do a GET request. versions > 7.14.1 also set the
-        # NO BODY flag, but we'll do it ourselves in case it is an older
-        # pycurl version
-        curl.setopt(pycurl.NOBODY, 0)
-        curl.setopt(pycurl.HTTPGET, 1)
 
         data = StringIO()
         header = StringIO()
@@ -179,11 +183,21 @@
         # handle_response will raise NoSuchFile, etc based on the response code
         return code, response.handle_response(abspath, code, headers, data)
 
-    def _raise_curl_connection_error(self, curl):
-        curl_errno = curl.getinfo(pycurl.OS_ERRNO)
-        url = curl.getinfo(pycurl.EFFECTIVE_URL)
-        raise ConnectionError('curl connection error (%s) on %s'
-                              % (os.strerror(curl_errno), url))
+    def _post(self, body_bytes):
+        fake_file = StringIO(body_bytes)
+        curl = self._base_curl
+        # Other places that use _base_curl for GET requests explicitly set
+        # HTTPGET, so it should be safe to re-use the same object for both GETs
+        # and POSTs.
+        curl.setopt(pycurl.POST, 1)
+        curl.setopt(pycurl.POSTFIELDSIZE, len(body_bytes))
+        curl.setopt(pycurl.READFUNCTION, fake_file.read)
+        abspath, data, header = self._setup_request(curl, '.bzr/smart')
+        self._curl_perform(curl)
+        data.seek(0)
+        code = curl.getinfo(pycurl.HTTP_CODE)
+        headers = _extract_headers(header.getvalue(), abspath)
+        return code, response.handle_response(abspath, code, headers, data)
 
     def _raise_curl_http_error(self, curl, info=None):
         code = curl.getinfo(pycurl.HTTP_CODE)
@@ -200,9 +214,12 @@
         # There's no way in http/1.0 to say "must revalidate"; we don't want
         # to force it to always retrieve.  so just turn off the default Pragma
         # provided by Curl.
+        # Also, we override the Expect: header so that pycurl will send the POST
+        # body immediately.
         headers = ['Cache-control: max-age=0',
                    'Pragma: no-cache',
-                   'Connection: Keep-Alive']
+                   'Connection: Keep-Alive',
+                   'Expect: ',]
         ## curl.setopt(pycurl.VERBOSE, 1)
         # TODO: maybe include a summary of the pycurl version
         ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__,)
@@ -220,8 +237,10 @@
             mutter('got pycurl error: %s, %s, %s, url: %s ',
                     e[0], _pycurl_errors.errorcode[e[0]], e, url)
             if e[0] in (_pycurl_errors.CURLE_COULDNT_RESOLVE_HOST,
-                        _pycurl_errors.CURLE_COULDNT_CONNECT):
-                self._raise_curl_connection_error(curl)
+                        _pycurl_errors.CURLE_COULDNT_CONNECT,
+                        _pycurl_errors.CURLE_COULDNT_RESOLVE_PROXY):
+                raise ConnectionError('curl connection error (%s)\non %s'
+                              % (e[1], url))
             # jam 20060713 The code didn't use to re-raise the exception here
             # but that seemed bogus
             raise

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2006-07-14 04:58:29 +0000
+++ bzrlib/transport/http/_urllib.py	2006-09-25 16:41:46 +0000
@@ -54,11 +54,14 @@
         # so nothing to do with from_transport
 
     def _get(self, relpath, ranges, tail_amount=0):
+        return self._request(relpath, 'GET', ranges, tail_amount=tail_amount)
+
+    def _request(self, relpath, method, ranges, tail_amount=0, body=None):
         path = relpath
         try:
             path = self._real_abspath(relpath)
-            resp = self._get_url_impl(path, method='GET', ranges=ranges,
-                                      tail_amount=tail_amount)
+            resp = self._get_url_impl(path, method=method, ranges=ranges,
+                                      tail_amount=tail_amount, body=body)
             return resp.code, response.handle_response(path,
                                 resp.code, resp.headers, resp)
         except urllib2.HTTPError, e:
@@ -67,16 +70,16 @@
                 raise NoSuchFile(path, extra=e)
             raise
         except (BzrError, IOError), e:
-            if hasattr(e, 'errno'):
+            if getattr(e, 'errno', None) is not None:
                 mutter('io error: %s %s for has url: %r',
                     e.errno, errno.errorcode.get(e.errno), path)
                 if e.errno == errno.ENOENT:
                     raise NoSuchFile(path, extra=e)
-            raise ConnectionError(msg = "Error retrieving %s: %s" 
-                             % (self.abspath(relpath), str(e)),
-                             orig_error=e)
+            raise ConnectionError(msg = "Error retrieving %s: %s"
+                                  % (self.abspath(relpath), str(e)),
+                                  orig_error=e)
 
-    def _get_url_impl(self, url, method, ranges, tail_amount=0):
+    def _get_url_impl(self, url, method, ranges, tail_amount=0, body=None):
         """Actually pass get request into urllib
 
         :returns: urllib Response object
@@ -87,6 +90,8 @@
         opener = urllib2.build_opener(auth_handler)
         request = Request(url)
         request.method = method
+        if body is not None:
+            request.add_data(body)
         request.add_header('Pragma', 'no-cache')
         request.add_header('Cache-control', 'max-age=0')
         request.add_header('User-Agent',
@@ -97,6 +102,9 @@
         response = opener.open(request)
         return response
 
+    def _post(self, body_bytes):
+        return self._request('.bzr/smart', 'POST', [], body=body_bytes)
+
     def should_cache(self):
         """Return True if the data pulled across should be cached locally.
         """
@@ -113,10 +121,13 @@
             f.read()
             f.close()
             return True
+        except urllib2.HTTPError, e:
+            mutter('url error code: %s, for has url: %r', e.code, abspath)
+            if e.code == 404:
+                return False
+            raise
         except urllib2.URLError, e:
-            mutter('url error code: %s for has url: %r', e.code, abspath)
-            if e.code == 404:
-                return False
+            mutter('url error: %s, for has url: %r', e.reason, abspath)
             raise
         except IOError, e:
             mutter('io error: %s %s for has url: %r',

=== modified file 'bzrlib/transport/http/response.py'
--- bzrlib/transport/http/response.py	2006-09-01 17:05:01 +0000
+++ bzrlib/transport/http/response.py	2006-10-13 06:36:44 +0000
@@ -251,10 +251,16 @@
         try:
             content_type = headers['Content-Type']
         except KeyError:
-            raise errors.InvalidHttpContentType(url, '',
-                msg='Missing Content-Type')
+            # When there is no content-type header we treat
+            # the response as being of type 'application/octet-stream' as per
+            # RFC2616 section 7.2.1.
+            # Therefore it is obviously not multipart
+            content_type = 'application/octet-stream'
+            is_multipart = False
+        else:
+            is_multipart = _is_multipart(content_type)
 
-        if _is_multipart(content_type):
+        if is_multipart:
             # Full fledged multipart response
             return HttpMultipartRangeResponse(url, content_type, data)
         else:

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2006-09-06 02:57:51 +0000
+++ bzrlib/transport/local.py	2006-09-13 03:21:03 +0000
@@ -19,22 +19,24 @@
 This is a fairly thin wrapper on regular file IO.
 """
 
+import os
+from stat import ST_MODE, S_ISDIR, ST_SIZE, S_IMODE
+import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import errno
-import os
 import shutil
-import sys
-from stat import ST_MODE, S_ISDIR, ST_SIZE, S_IMODE
-import tempfile
 
 from bzrlib import (
     atomicfile,
     osutils,
     urlutils,
+    symbol_versioning,
     )
-from bzrlib.osutils import (abspath, realpath, normpath, pathjoin, rename,
-                            check_legal_path, rmtree)
-from bzrlib.symbol_versioning import warn
 from bzrlib.trace import mutter
+""")
+
 from bzrlib.transport import Transport, Server
 
 
@@ -48,7 +50,8 @@
     def __init__(self, base):
         """Set the base path where files will be stored."""
         if not base.startswith('file://'):
-            warn("Instantiating LocalTransport with a filesystem path"
+            symbol_versioning.warn(
+                "Instantiating LocalTransport with a filesystem path"
                 " is deprecated as of bzr 0.8."
                 " Please use bzrlib.transport.get_transport()"
                 " or pass in a file:// url.",
@@ -91,7 +94,8 @@
         assert isinstance(relpath, basestring), (type(relpath), relpath)
         # jam 20060426 Using normpath on the real path, because that ensures
         #       proper handling of stuff like
-        path = normpath(pathjoin(self._local_base, urlutils.unescape(relpath)))
+        path = osutils.normpath(osutils.pathjoin(
+                    self._local_base, urlutils.unescape(relpath)))
         return urlutils.local_path_to_url(path)
 
     def local_abspath(self, relpath):
@@ -145,7 +149,7 @@
         path = relpath
         try:
             path = self._abspath(relpath)
-            check_legal_path(path)
+            osutils.check_legal_path(path)
             fp = atomicfile.AtomicFile(path, 'wb', new_mode=mode)
         except (IOError, OSError),e:
             self._translate_error(e, path)
@@ -165,7 +169,7 @@
         path = relpath
         try:
             path = self._abspath(relpath)
-            check_legal_path(path)
+            osutils.check_legal_path(path)
             fp = atomicfile.AtomicFile(path, 'wb', new_mode=mode)
         except (IOError, OSError),e:
             self._translate_error(e, path)
@@ -177,7 +181,8 @@
 
     def _put_non_atomic_helper(self, relpath, writer,
                                mode=None,
-                               create_parent_dir=False):
+                               create_parent_dir=False,
+                               dir_mode=None):
         """Common functionality information for the put_*_non_atomic.
 
         This tracks all the create_parent_dir stuff.
@@ -206,10 +211,7 @@
             parent_dir = os.path.dirname(abspath)
             if not parent_dir:
                 self._translate_error(e, relpath)
-            try:
-                os.mkdir(parent_dir)
-            except (IOError, OSError), e:
-                self._translate_error(e, relpath)
+            self._mkdir(parent_dir, mode=dir_mode)
             # We created the parent directory, lets try to open the
             # file again
             try:
@@ -227,7 +229,8 @@
             os.close(fd)
 
     def put_file_non_atomic(self, relpath, f, mode=None,
-                            create_parent_dir=False):
+                            create_parent_dir=False,
+                            dir_mode=None):
         """Copy the file-like object into the target location.
 
         This function is not strictly safe to use. It is only meant to
@@ -246,14 +249,16 @@
         def writer(fd):
             self._pump_to_fd(f, fd)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def put_bytes_non_atomic(self, relpath, bytes, mode=None,
-                             create_parent_dir=False):
+                             create_parent_dir=False, dir_mode=None):
         def writer(fd):
             os.write(fd, bytes)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def iter_files_recursive(self):
         """Iter the relative paths of files in the transports sub-tree."""
@@ -267,23 +272,25 @@
             else:
                 yield relpath
 
-    def mkdir(self, relpath, mode=None):
-        """Create a directory at the given path."""
-        path = relpath
+    def _mkdir(self, abspath, mode=None):
+        """Create a real directory, filtering through mode"""
+        if mode is None:
+            # os.mkdir() will filter through umask
+            local_mode = 0777
+        else:
+            local_mode = mode
         try:
-            if mode is None:
-                # os.mkdir() will filter through umask
-                local_mode = 0777
-            else:
-                local_mode = mode
-            path = self._abspath(relpath)
-            os.mkdir(path, local_mode)
+            os.mkdir(abspath, local_mode)
             if mode is not None:
                 # It is probably faster to just do the chmod, rather than
                 # doing a stat, and then trying to compare
-                os.chmod(path, mode)
+                os.chmod(abspath, mode)
         except (IOError, OSError),e:
-            self._translate_error(e, path)
+            self._translate_error(e, abspath)
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        self._mkdir(self._abspath(relpath), mode=mode)
 
     def _get_append_file(self, relpath, mode=None):
         """Call os.open() for the given relpath"""
@@ -363,7 +370,7 @@
 
         try:
             # this version will delete the destination if necessary
-            rename(path_from, path_to)
+            osutils.rename(path_from, path_to)
         except (IOError, OSError),e:
             # TODO: What about path_to?
             self._translate_error(e, path_from)

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2006-09-05 21:25:15 +0000
+++ bzrlib/transport/memory.py	2006-09-16 13:36:47 +0000
@@ -20,7 +20,6 @@
 so this is primarily useful for testing.
 """
 
-from copy import copy
 import os
 import errno
 import re
@@ -59,7 +58,9 @@
         if url[-1] != '/':
             url = url + '/'
         super(MemoryTransport, self).__init__(url)
-        self._cwd = url[url.find(':') + 3:]
+        split = url.find(':') + 3
+        self._scheme = url[:split]
+        self._cwd = url[split:]
         # dictionaries from absolute path to file mode
         self._dirs = {'/':None}
         self._files = {}
@@ -67,20 +68,10 @@
 
     def clone(self, offset=None):
         """See Transport.clone()."""
-        if offset is None or offset == '':
-            return copy(self)
-        segments = offset.split('/')
-        cwdsegments = self._cwd.split('/')[:-1]
-        while len(segments):
-            segment = segments.pop(0)
-            if segment == '.':
-                continue
-            if segment == '..':
-                if len(cwdsegments) > 1:
-                    cwdsegments.pop()
-                continue
-            cwdsegments.append(segment)
-        url = self.base[:self.base.find(':') + 3] + '/'.join(cwdsegments) + '/'
+        path = self._combine_paths(self._cwd, offset)
+        if len(path) == 0 or path[-1] != '/':
+            path += '/'
+        url = self._scheme + path
         result = MemoryTransport(url)
         result._dirs = self._dirs
         result._files = self._files
@@ -236,6 +227,10 @@
         relpath = urlutils.unescape(relpath)
         if relpath.find('..') != -1:
             raise AssertionError('relpath contains ..')
+        if relpath == '':
+            return '/'
+        if relpath[0] == '/':
+            return relpath
         if relpath == '.':
             if (self._cwd == '/'):
                 return self._cwd

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-09-06 02:57:51 +0000
+++ bzrlib/transport/sftp.py	2006-09-18 07:58:41 +0000
@@ -17,26 +17,35 @@
 
 """Implementation of Transport over SFTP, using paramiko."""
 
+# TODO: Remove the transport-based lock_read and lock_write methods.  They'll
+# then raise TransportNotPossible, which will break remote access to any
+# formats which rely on OS-level locks.  That should be fine as those formats
+# are pretty old, but these combinations may have to be removed from the test
+# suite.  Those formats all date back to 0.7; so we should be able to remove
+# these methods when we officially drop support for those formats.
+
 import errno
 import os
 import random
 import select
 import socket
 import stat
-import subprocess
 import sys
 import time
 import urllib
 import urlparse
 import weakref
 
-from bzrlib.errors import (FileExists, 
+from bzrlib import (
+    errors,
+    urlutils,
+    )
+from bzrlib.errors import (FileExists,
                            NoSuchFile, PathNotChild,
                            TransportError,
-                           LockError, 
+                           LockError,
                            PathError,
                            ParamikoNotPresent,
-                           UnknownSSH,
                            )
 from bzrlib.osutils import pathjoin, fancy_rename, getcwd
 from bzrlib.trace import mutter, warning
@@ -47,7 +56,6 @@
     ssh,
     Transport,
     )
-import bzrlib.urlutils as urlutils
 
 try:
     import paramiko
@@ -85,8 +93,15 @@
 
 
 class SFTPLock(object):
-    """This fakes a lock in a remote location."""
+    """This fakes a lock in a remote location.
+    
+    A present lock is indicated just by the existence of a file.  This
+    doesn't work well on all transports and they are only used in 
+    deprecated storage formats.
+    """
+    
     __slots__ = ['path', 'lock_path', 'lock_file', 'transport']
+
     def __init__(self, path, transport):
         assert isinstance(transport, SFTPTransport)
 
@@ -119,7 +134,69 @@
             pass
 
 
-class SFTPTransport(Transport):
+class SFTPUrlHandling(Transport):
+    """Mix-in that does common handling of SSH/SFTP URLs."""
+
+    def __init__(self, base):
+        self._parse_url(base)
+        base = self._unparse_url(self._path)
+        if base[-1] != '/':
+            base += '/'
+        super(SFTPUrlHandling, self).__init__(base)
+
+    def _parse_url(self, url):
+        (self._scheme,
+         self._username, self._password,
+         self._host, self._port, self._path) = self._split_url(url)
+
+    def _unparse_url(self, path):
+        """Return a URL for a path relative to this transport.
+        """
+        path = urllib.quote(path)
+        # handle homedir paths
+        if not path.startswith('/'):
+            path = "/~/" + path
+        netloc = urllib.quote(self._host)
+        if self._username is not None:
+            netloc = '%s@%s' % (urllib.quote(self._username), netloc)
+        if self._port is not None:
+            netloc = '%s:%d' % (netloc, self._port)
+        return urlparse.urlunparse((self._scheme, netloc, path, '', '', ''))
+
+    def _split_url(self, url):
+        (scheme, username, password, host, port, path) = split_url(url)
+        ## assert scheme == 'sftp'
+
+        # the initial slash should be removed from the path, and treated
+        # as a homedir relative path (the path begins with a double slash
+        # if it is absolute).
+        # see draft-ietf-secsh-scp-sftp-ssh-uri-03.txt
+        # RBC 20060118 we are not using this as its too user hostile. instead
+        # we are following lftp and using /~/foo to mean '~/foo'.
+        # handle homedir paths
+        if path.startswith('/~/'):
+            path = path[3:]
+        elif path == '/~':
+            path = ''
+        return (scheme, username, password, host, port, path)
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        
+        @param relpath: the relative path or path components
+        @type relpath: str or list
+        """
+        return self._unparse_url(self._remote_path(relpath))
+    
+    def _remote_path(self, relpath):
+        """Return the path to be passed along the sftp protocol for relpath.
+        
+        :param relpath: is a urlencoded string.
+        """
+        return self._combine_paths(self._path, relpath)
+
+
+class SFTPTransport(SFTPUrlHandling):
     """Transport implementation for SFTP access."""
 
     _do_prefetch = _default_do_prefetch
@@ -141,11 +218,6 @@
     _max_request_size = 32768
 
     def __init__(self, base, clone_from=None):
-        assert base.startswith('sftp://')
-        self._parse_url(base)
-        base = self._unparse_url()
-        if base[-1] != '/':
-            base += '/'
         super(SFTPTransport, self).__init__(base)
         if clone_from is None:
             self._sftp_connect()
@@ -171,45 +243,117 @@
         else:
             return SFTPTransport(self.abspath(offset), self)
 
-    def abspath(self, relpath):
-        """
-        Return the full url to the given relative path.
-        
-        @param relpath: the relative path or path components
-        @type relpath: str or list
-        """
-        return self._unparse_url(self._remote_path(relpath))
-    
     def _remote_path(self, relpath):
         """Return the path to be passed along the sftp protocol for relpath.
         
         relpath is a urlencoded string.
+
+        :return: a path prefixed with / for regular abspath-based urls, or a
+            path that does not begin with / for urls which begin with /~/.
         """
-        # FIXME: share the common code across transports
+        # how does this work? 
+        # it processes relpath with respect to 
+        # our state:
+        # firstly we create a path to evaluate: 
+        # if relpath is an abspath or homedir path, its the entire thing
+        # otherwise we join our base with relpath
+        # then we eliminate all empty segments (double //'s) outside the first
+        # two elements of the list. This avoids problems with trailing 
+        # slashes, or other abnormalities.
+        # finally we evaluate the entire path in a single pass
+        # '.'s are stripped,
+        # '..' result in popping the left most already 
+        # processed path (which can never be empty because of the check for
+        # abspath and homedir meaning that its not, or that we've used our
+        # path. If the pop would pop the root, we ignore it.
+
+        # Specific case examinations:
+        # remove the special casefor ~: if the current root is ~/ popping of it
+        # = / thus our seed for a ~ based path is ['', '~']
+        # and if we end up with [''] then we had basically ('', '..') (which is
+        # '/..' so we append '' if the length is one, and assert that the first
+        # element is still ''. Lastly, if we end with ['', '~'] as a prefix for
+        # the output, we've got a homedir path, so we strip that prefix before
+        # '/' joining the resulting list.
+        #
+        # case one: '/' -> ['', ''] cannot shrink
+        # case two: '/' + '../foo' -> ['', 'foo'] (take '', '', '..', 'foo')
+        #           and pop the second '' for the '..', append 'foo'
+        # case three: '/~/' -> ['', '~', ''] 
+        # case four: '/~/' + '../foo' -> ['', '~', '', '..', 'foo'],
+        #           and we want to get '/foo' - the empty path in the middle
+        #           needs to be stripped, then normal path manipulation will 
+        #           work.
+        # case five: '/..' ['', '..'], we want ['', '']
+        #            stripping '' outside the first two is ok
+        #            ignore .. if its too high up
+        #
+        # lastly this code is possibly reusable by FTP, but not reusable by
+        # local paths: ~ is resolvable correctly, nor by HTTP or the smart
+        # server: ~ is resolved remotely.
+        # 
+        # however, a version of this that acts on self.base is possible to be
+        # written which manipulates the URL in canonical form, and would be
+        # reusable for all transports, if a flag for allowing ~/ at all was
+        # provided.
         assert isinstance(relpath, basestring)
-        relpath = urlutils.unescape(relpath).split('/')
-        basepath = self._path.split('/')
-        if len(basepath) > 0 and basepath[-1] == '':
-            basepath = basepath[:-1]
-
-        for p in relpath:
-            if p == '..':
-                if len(basepath) == 0:
-                    # In most filesystems, a request for the parent
-                    # of root, just returns root.
-                    continue
-                basepath.pop()
-            elif p == '.':
-                continue # No-op
-            else:
-                basepath.append(p)
-
-        path = '/'.join(basepath)
-        # mutter('relpath => remotepath %s => %s', relpath, path)
+        relpath = urlutils.unescape(relpath)
+
+        # case 1)
+        if relpath.startswith('/'):
+            # abspath - normal split is fine.
+            current_path = relpath.split('/')
+        elif relpath.startswith('~/'):
+            # root is homedir based: normal split and prefix '' to remote the
+            # special case
+            current_path = [''].extend(relpath.split('/'))
+        else:
+            # root is from the current directory:
+            if self._path.startswith('/'):
+                # abspath, take the regular split
+                current_path = []
+            else:
+                # homedir based, add the '', '~' not present in self._path
+                current_path = ['', '~']
+            # add our current dir
+            current_path.extend(self._path.split('/'))
+            # add the users relpath
+            current_path.extend(relpath.split('/'))
+        # strip '' segments that are not in the first one - the leading /.
+        to_process = current_path[:1]
+        for segment in current_path[1:]:
+            if segment != '':
+                to_process.append(segment)
+
+        # process '.' and '..' segments into output_path.
+        output_path = []
+        for segment in to_process:
+            if segment == '..':
+                # directory pop. Remove a directory 
+                # as long as we are not at the root
+                if len(output_path) > 1:
+                    output_path.pop()
+                # else: pass
+                # cannot pop beyond the root, so do nothing
+            elif segment == '.':
+                continue # strip the '.' from the output.
+            else:
+                # this will append '' to output_path for the root elements,
+                # which is appropriate: its why we strip '' in the first pass.
+                output_path.append(segment)
+
+        # check output special cases:
+        if output_path == ['']:
+            # [''] -> ['', '']
+            output_path = ['', '']
+        elif output_path[:2] == ['', '~']:
+            # ['', '~', ...] -> ...
+            output_path = output_path[2:]
+        path = '/'.join(output_path)
         return path
 
     def relpath(self, abspath):
-        username, password, host, port, path = self._split_url(abspath)
+        scheme, username, password, host, port, path = self._split_url(abspath)
         error = []
         if (username != self._username):
             error.append('username mismatch')
@@ -263,13 +407,13 @@
             fp = self._sftp.file(path, mode='rb')
             readv = getattr(fp, 'readv', None)
             if readv:
-                return self._sftp_readv(fp, offsets)
+                return self._sftp_readv(fp, offsets, relpath)
             mutter('seek and read %s offsets', len(offsets))
-            return self._seek_and_read(fp, offsets)
+            return self._seek_and_read(fp, offsets, relpath)
         except (IOError, paramiko.SSHException), e:
             self._translate_io_exception(e, path, ': error retrieving')
 
-    def _sftp_readv(self, fp, offsets):
+    def _sftp_readv(self, fp, offsets, relpath='<unknown>'):
         """Use the readv() member of fp to do async readv.
 
         And then read them using paramiko.readv(). paramiko.readv()
@@ -362,10 +506,16 @@
                 yield cur_offset_and_size[0], this_data
                 cur_offset_and_size = offset_stack.next()
 
+            # We read a coalesced entry, so mark it as done
+            cur_coalesced = None
             # Now that we've read all of the data for this coalesced section
             # on to the next
             cur_coalesced = cur_coalesced_stack.next()
 
+        if cur_coalesced is not None:
+            raise errors.ShortReadvError(relpath, cur_coalesced.start,
+                cur_coalesced.length, len(data))
+
     def put_file(self, relpath, f, mode=None):
         """
         Copy the file-like object into the location.
@@ -426,7 +576,8 @@
             raise
 
     def _put_non_atomic_helper(self, relpath, writer, mode=None,
-                               create_parent_dir=False):
+                               create_parent_dir=False,
+                               dir_mode=None):
         abspath = self._remote_path(relpath)
 
         # TODO: jam 20060816 paramiko doesn't publicly expose a way to
@@ -465,14 +616,12 @@
             # Try to create the parent directory, and then go back to
             # writing the file
             parent_dir = os.path.dirname(abspath)
-            try:
-                self._sftp.mkdir(parent_dir)
-            except (paramiko.SSHException, IOError), e:
-                self._translate_io_exception(e, abspath, ': unable to open')
+            self._mkdir(parent_dir, dir_mode)
             _open_and_write_file()
 
     def put_file_non_atomic(self, relpath, f, mode=None,
-                            create_parent_dir=False):
+                            create_parent_dir=False,
+                            dir_mode=None):
         """Copy the file-like object into the target location.
 
         This function is not strictly safe to use. It is only meant to
@@ -491,14 +640,17 @@
         def writer(fout):
             self._pump(f, fout)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def put_bytes_non_atomic(self, relpath, bytes, mode=None,
-                             create_parent_dir=False):
+                             create_parent_dir=False,
+                             dir_mode=None):
         def writer(fout):
             fout.write(bytes)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def iter_files_recursive(self):
         """Walk the relative paths of all files in this transport."""
@@ -512,16 +664,22 @@
             else:
                 yield relpath
 
+    def _mkdir(self, abspath, mode=None):
+        if mode is None:
+            local_mode = 0777
+        else:
+            local_mode = mode
+        try:
+            self._sftp.mkdir(abspath, local_mode)
+            if mode is not None:
+                self._sftp.chmod(abspath, mode=mode)
+        except (paramiko.SSHException, IOError), e:
+            self._translate_io_exception(e, abspath, ': unable to mkdir',
+                failure_exc=FileExists)
+
     def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
-        path = self._remote_path(relpath)
-        try:
-            self._sftp.mkdir(path)
-            if mode is not None:
-                self._sftp.chmod(path, mode=mode)
-        except (paramiko.SSHException, IOError), e:
-            self._translate_io_exception(e, path, ': unable to mkdir',
-                failure_exc=FileExists)
+        self._mkdir(self._remote_path(relpath), mode=mode)
 
     def _translate_io_exception(self, e, path, more_info='', 
                                 failure_exc=PathError):
@@ -539,7 +697,7 @@
         """
         # paramiko seems to generate detailless errors.
         self._translate_error(e, path, raise_generic=False)
-        if hasattr(e, 'args'):
+        if getattr(e, 'args', None) is not None:
             if (e.args == ('No such file or directory',) or
                 e.args == ('No such file',)):
                 raise NoSuchFile(path, str(e) + more_info)
@@ -549,7 +707,7 @@
             if (e.args == ('Failure',)):
                 raise failure_exc(path, str(e) + more_info)
             mutter('Raising exception with args %s', e.args)
-        if hasattr(e, 'errno'):
+        if getattr(e, 'errno', None) is not None:
             mutter('Raising exception with errno %s', e.errno)
         raise e
 
@@ -665,41 +823,6 @@
         # that we have taken the lock.
         return SFTPLock(relpath, self)
 
-    def _unparse_url(self, path=None):
-        if path is None:
-            path = self._path
-        path = urllib.quote(path)
-        # handle homedir paths
-        if not path.startswith('/'):
-            path = "/~/" + path
-        netloc = urllib.quote(self._host)
-        if self._username is not None:
-            netloc = '%s@%s' % (urllib.quote(self._username), netloc)
-        if self._port is not None:
-            netloc = '%s:%d' % (netloc, self._port)
-        return urlparse.urlunparse(('sftp', netloc, path, '', '', ''))
-
-    def _split_url(self, url):
-        (scheme, username, password, host, port, path) = split_url(url)
-        assert scheme == 'sftp'
-
-        # the initial slash should be removed from the path, and treated
-        # as a homedir relative path (the path begins with a double slash
-        # if it is absolute).
-        # see draft-ietf-secsh-scp-sftp-ssh-uri-03.txt
-        # RBC 20060118 we are not using this as its too user hostile. instead
-        # we are following lftp and using /~/foo to mean '~/foo'.
-        # handle homedir paths
-        if path.startswith('/~/'):
-            path = path[3:]
-        elif path == '/~':
-            path = ''
-        return (username, password, host, port, path)
-
-    def _parse_url(self, url):
-        (self._username, self._password,
-         self._host, self._port, self._path) = self._split_url(url)
-
     def _sftp_connect(self):
         """Connect to the remote sftp server.
         After this, self._sftp should have a valid connection (or
@@ -745,6 +868,12 @@
             self._translate_io_exception(e, abspath, ': unable to open',
                 failure_exc=FileExists)
 
+    def _can_roundtrip_unix_modebits(self):
+        if sys.platform == 'win32':
+            # anyone else?
+            return False
+        else:
+            return True
 
 # ------------- server test implementation --------------
 import threading
@@ -893,13 +1022,14 @@
 class SFTPServer(Server):
     """Common code for SFTP server facilities."""
 
-    def __init__(self):
+    def __init__(self, server_interface=StubServer):
         self._original_vendor = None
         self._homedir = None
         self._server_homedir = None
         self._listener = None
         self._root = None
         self._vendor = ssh.ParamikoVendor()
+        self._server_interface = server_interface
         # sftp server logs
         self.logs = []
         self.add_latency = 0
@@ -930,7 +1060,7 @@
         f.close()
         host_key = paramiko.RSAKey.from_private_key_file(key_file)
         ssh_server.add_server_key(host_key)
-        server = StubServer(self)
+        server = self._server_interface(self)
         ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer,
                                          StubSFTPServer, root=self._root,
                                          home=self._server_homedir)
@@ -990,7 +1120,6 @@
         # Re-import these as locals, so that they're still accessible during
         # interpreter shutdown (when all module globals get set to None, leading
         # to confusing errors like "'NoneType' object has no attribute 'error'".
-        import socket, errno
         class FakeChannel(object):
             def get_transport(self):
                 return self

=== added file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/smart.py	2006-10-09 07:34:15 +0000
@@ -0,0 +1,1772 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Smart-server protocol, client and server.
+
+Requests are sent as a command and list of arguments, followed by optional
+bulk body data.  Responses are similarly a response and list of arguments,
+followed by bulk body data. ::
+
+  SEP := '\001'
+    Fields are separated by Ctrl-A.
+  BULK_DATA := CHUNK TRAILER
+    Chunks can be repeated as many times as necessary.
+  CHUNK := CHUNK_LEN CHUNK_BODY
+  CHUNK_LEN := DIGIT+ NEWLINE
+    Gives the number of bytes in the following chunk.
+  CHUNK_BODY := BYTE[chunk_len]
+  TRAILER := SUCCESS_TRAILER | ERROR_TRAILER
+  SUCCESS_TRAILER := 'done' NEWLINE
+  ERROR_TRAILER := 
+
+Paths are passed across the network.  The client needs to see a namespace that
+includes any repository that might need to be referenced, and the client needs
+to know about a root directory beyond which it cannot ascend.
+
+Servers run over ssh will typically want to be able to access any path the user 
+can access.  Public servers on the other hand (which might be over http, ssh
+or tcp) will typically want to restrict access to only a particular directory 
+and its children, so will want to do a software virtual root at that level.
+In other words they'll want to rewrite incoming paths to be under that level
+(and prevent escaping using ../ tricks.)
+
+URLs that include ~ should probably be passed across to the server verbatim
+and the server can expand them.  This will proably not be meaningful when 
+limited to a directory?
+
+At the bottom level socket, pipes, HTTP server.  For sockets, we have the idea
+that you have multiple requests and get a read error because the other side did
+shutdown.  For pipes we have read pipe which will have a zero read which marks
+end-of-file.  For HTTP server environment there is not end-of-stream because
+each request coming into the server is independent.
+
+So we need a wrapper around pipes and sockets to seperate out requests from
+substrate and this will give us a single model which is consist for HTTP,
+sockets and pipes.
+
+Server-side
+-----------
+
+ MEDIUM  (factory for protocol, reads bytes & pushes to protocol,
+          uses protocol to detect end-of-request, sends written
+          bytes to client) e.g. socket, pipe, HTTP request handler.
+  ^
+  | bytes.
+  v
+
+PROTOCOL  (serialization, deserialization)  accepts bytes for one
+          request, decodes according to internal state, pushes
+          structured data to handler.  accepts structured data from
+          handler and encodes and writes to the medium.  factory for
+          handler.
+  ^
+  | structured data
+  v
+
+HANDLER   (domain logic) accepts structured data, operates state
+          machine until the request can be satisfied,
+          sends structured data to the protocol.
+
+
+Client-side
+-----------
+
+ CLIENT             domain logic, accepts domain requests, generated structured
+                    data, reads structured data from responses and turns into
+                    domain data.  Sends structured data to the protocol.
+                    Operates state machines until the request can be delivered
+                    (e.g. reading from a bundle generated in bzrlib to deliver a
+                    complete request).
+
+                    Possibly this should just be RemoteBzrDir, RemoteTransport,
+                    ...
+  ^
+  | structured data
+  v
+
+PROTOCOL  (serialization, deserialization)  accepts structured data for one
+          request, encodes and writes to the medium.  Reads bytes from the
+          medium, decodes and allows the client to read structured data.
+  ^
+  | bytes.
+  v
+
+ MEDIUM  (accepts bytes from the protocol & delivers to the remote server.
+          Allows the potocol to read bytes e.g. socket, pipe, HTTP request.
+"""
+
+
+# TODO: _translate_error should be on the client, not the transport because
+#     error coding is wire protocol specific.
+
+# TODO: A plain integer from query_version is too simple; should give some
+# capabilities too?
+
+# TODO: Server should probably catch exceptions within itself and send them
+# back across the network.  (But shouldn't catch KeyboardInterrupt etc)
+# Also needs to somehow report protocol errors like bad requests.  Need to
+# consider how we'll handle error reporting, e.g. if we get halfway through a
+# bulk transfer and then something goes wrong.
+
+# TODO: Standard marker at start of request/response lines?
+
+# TODO: Make each request and response self-validatable, e.g. with checksums.
+#
+# TODO: get/put objects could be changed to gradually read back the data as it
+# comes across the network
+#
+# TODO: What should the server do if it hits an error and has to terminate?
+#
+# TODO: is it useful to allow multiple chunks in the bulk data?
+#
+# TODO: If we get an exception during transmission of bulk data we can't just
+# emit the exception because it won't be seen.
+#   John proposes:  I think it would be worthwhile to have a header on each
+#   chunk, that indicates it is another chunk. Then you can send an 'error'
+#   chunk as long as you finish the previous chunk.
+#
+# TODO: Clone method on Transport; should work up towards parent directory;
+# unclear how this should be stored or communicated to the server... maybe
+# just pass it on all relevant requests?
+#
+# TODO: Better name than clone() for changing between directories.  How about
+# open_dir or change_dir or chdir?
+#
+# TODO: Is it really good to have the notion of current directory within the
+# connection?  Perhaps all Transports should factor out a common connection
+# from the thing that has the directory context?
+#
+# TODO: Pull more things common to sftp and ssh to a higher level.
+#
+# TODO: The server that manages a connection should be quite small and retain
+# minimum state because each of the requests are supposed to be stateless.
+# Then we can write another implementation that maps to http.
+#
+# TODO: What to do when a client connection is garbage collected?  Maybe just
+# abruptly drop the connection?
+#
+# TODO: Server in some cases will need to restrict access to files outside of
+# a particular root directory.  LocalTransport doesn't do anything to stop you
+# ascending above the base directory, so we need to prevent paths
+# containing '..' in either the server or transport layers.  (Also need to
+# consider what happens if someone creates a symlink pointing outside the 
+# directory tree...)
+#
+# TODO: Server should rebase absolute paths coming across the network to put
+# them under the virtual root, if one is in use.  LocalTransport currently
+# doesn't do that; if you give it an absolute path it just uses it.
+# 
+# XXX: Arguments can't contain newlines or ascii; possibly we should e.g.
+# urlescape them instead.  Indeed possibly this should just literally be
+# http-over-ssh.
+#
+# FIXME: This transport, with several others, has imperfect handling of paths
+# within urls.  It'd probably be better for ".." from a root to raise an error
+# rather than return the same directory as we do at present.
+#
+# TODO: Rather than working at the Transport layer we want a Branch,
+# Repository or BzrDir objects that talk to a server.
+#
+# TODO: Probably want some way for server commands to gradually produce body
+# data rather than passing it as a string; they could perhaps pass an
+# iterator-like callback that will gradually yield data; it probably needs a
+# close() method that will always be closed to do any necessary cleanup.
+#
+# TODO: Split the actual smart server from the ssh encoding of it.
+#
+# TODO: Perhaps support file-level readwrite operations over the transport
+# too.
+#
+# TODO: SmartBzrDir class, proxying all Branch etc methods across to another
+# branch doing file-level operations.
+#
+
+from cStringIO import StringIO
+import os
+import socket
+import tempfile
+import threading
+import urllib
+import urlparse
+
+from bzrlib import (
+    bzrdir,
+    errors,
+    revision,
+    transport,
+    trace,
+    urlutils,
+    )
+from bzrlib.bundle.serializer import write_bundle
+try:
+    from bzrlib.transport import ssh
+except errors.ParamikoNotPresent:
+    # no paramiko.  SmartSSHClientMedium will break.
+    pass
+
+# must do this otherwise urllib can't parse the urls properly :(
+for scheme in ['ssh', 'bzr', 'bzr+loopback', 'bzr+ssh']:
+    transport.register_urlparse_netloc_protocol(scheme)
+del scheme
+
+
+def _recv_tuple(from_file):
+    req_line = from_file.readline()
+    return _decode_tuple(req_line)
+
+
+def _decode_tuple(req_line):
+    if req_line == None or req_line == '':
+        return None
+    if req_line[-1] != '\n':
+        raise errors.SmartProtocolError("request %r not terminated" % req_line)
+    try:
+        return tuple((a.decode('utf-8') for a in req_line[:-1].split('\x01')))
+    except UnicodeDecodeError:
+        raise errors.SmartProtocolError(
+            "one or more arguments of request %r are not valid UTF-8"
+            % req_line)
+
+
+def _encode_tuple(args):
+    """Encode the tuple args to a bytestream."""
+    return '\x01'.join((a.encode('utf-8') for a in args)) + '\n'
+
+
+class SmartProtocolBase(object):
+    """Methods common to client and server"""
+
+    # TODO: this only actually accomodates a single block; possibly should
+    # support multiple chunks?
+    def _encode_bulk_data(self, body):
+        """Encode body as a bulk data chunk."""
+        return ''.join(('%d\n' % len(body), body, 'done\n'))
+
+    def _serialise_offsets(self, offsets):
+        """Serialise a readv offset list."""
+        txt = []
+        for start, length in offsets:
+            txt.append('%d,%d' % (start, length))
+        return '\n'.join(txt)
+        
+
+class SmartServerRequestProtocolOne(SmartProtocolBase):
+    """Server-side encoding and decoding logic for smart version 1."""
+    
+    def __init__(self, backing_transport, write_func):
+        self._backing_transport = backing_transport
+        self.excess_buffer = ''
+        self._finished_reading = False
+        self.in_buffer = ''
+        self.has_dispatched = False
+        self.request = None
+        self._body_decoder = None
+        self._write_func = write_func
+
+    def accept_bytes(self, bytes):
+        """Take bytes, and advance the internal state machine appropriately.
+        
+        :param bytes: must be a byte string
+        """
+        assert isinstance(bytes, str)
+        self.in_buffer += bytes
+        if not self.has_dispatched:
+            if '\n' not in self.in_buffer:
+                # no command line yet
+                return
+            self.has_dispatched = True
+            try:
+                first_line, self.in_buffer = self.in_buffer.split('\n', 1)
+                first_line += '\n'
+                req_args = _decode_tuple(first_line)
+                self.request = SmartServerRequestHandler(
+                    self._backing_transport)
+                self.request.dispatch_command(req_args[0], req_args[1:])
+                if self.request.finished_reading:
+                    # trivial request
+                    self.excess_buffer = self.in_buffer
+                    self.in_buffer = ''
+                    self._send_response(self.request.response.args,
+                        self.request.response.body)
+                self.sync_with_request(self.request)
+            except KeyboardInterrupt:
+                raise
+            except Exception, exception:
+                # everything else: pass to client, flush, and quit
+                self._send_response(('error', str(exception)))
+                return None
+
+        if self.has_dispatched:
+            if self._finished_reading:
+                # nothing to do.XXX: this routine should be a single state 
+                # machine too.
+                self.excess_buffer += self.in_buffer
+                self.in_buffer = ''
+                return
+            if self._body_decoder is None:
+                self._body_decoder = LengthPrefixedBodyDecoder()
+            self._body_decoder.accept_bytes(self.in_buffer)
+            self.in_buffer = self._body_decoder.unused_data
+            body_data = self._body_decoder.read_pending_data()
+            self.request.accept_body(body_data)
+            if self._body_decoder.finished_reading:
+                self.request.end_of_body()
+                assert self.request.finished_reading, \
+                    "no more body, request not finished"
+            self.sync_with_request(self.request)
+            if self.request.response is not None:
+                self._send_response(self.request.response.args,
+                    self.request.response.body)
+                self.excess_buffer = self.in_buffer
+                self.in_buffer = ''
+            else:
+                assert not self.request.finished_reading, \
+                    "no response and we have finished reading."
+
+    def _send_response(self, args, body=None):
+        """Send a smart server response down the output stream."""
+        self._write_func(_encode_tuple(args))
+        if body is not None:
+            assert isinstance(body, str), 'body must be a str'
+            bytes = self._encode_bulk_data(body)
+            self._write_func(bytes)
+
+    def sync_with_request(self, request):
+        self._finished_reading = request.finished_reading
+        
+    def next_read_size(self):
+        if self._finished_reading:
+            return 0
+        if self._body_decoder is None:
+            return 1
+        else:
+            return self._body_decoder.next_read_size()
+
+
+class LengthPrefixedBodyDecoder(object):
+    """Decodes the length-prefixed bulk data."""
+    
+    def __init__(self):
+        self.bytes_left = None
+        self.finished_reading = False
+        self.unused_data = ''
+        self.state_accept = self._state_accept_expecting_length
+        self.state_read = self._state_read_no_data
+        self._in_buffer = ''
+        self._trailer_buffer = ''
+    
+    def accept_bytes(self, bytes):
+        """Decode as much of bytes as possible.
+
+        If 'bytes' contains too much data it will be appended to
+        self.unused_data.
+
+        finished_reading will be set when no more data is required.  Further
+        data will be appended to self.unused_data.
+        """
+        # accept_bytes is allowed to change the state
+        current_state = self.state_accept
+        self.state_accept(bytes)
+        while current_state != self.state_accept:
+            current_state = self.state_accept
+            self.state_accept('')
+
+    def next_read_size(self):
+        if self.bytes_left is not None:
+            # Ideally we want to read all the remainder of the body and the
+            # trailer in one go.
+            return self.bytes_left + 5
+        elif self.state_accept == self._state_accept_reading_trailer:
+            # Just the trailer left
+            return 5 - len(self._trailer_buffer)
+        elif self.state_accept == self._state_accept_expecting_length:
+            # There's still at least 6 bytes left ('\n' to end the length, plus
+            # 'done\n').
+            return 6
+        else:
+            # Reading excess data.  Either way, 1 byte at a time is fine.
+            return 1
+        
+    def read_pending_data(self):
+        """Return any pending data that has been decoded."""
+        return self.state_read()
+
+    def _state_accept_expecting_length(self, bytes):
+        self._in_buffer += bytes
+        pos = self._in_buffer.find('\n')
+        if pos == -1:
+            return
+        self.bytes_left = int(self._in_buffer[:pos])
+        self._in_buffer = self._in_buffer[pos+1:]
+        self.bytes_left -= len(self._in_buffer)
+        self.state_accept = self._state_accept_reading_body
+        self.state_read = self._state_read_in_buffer
+
+    def _state_accept_reading_body(self, bytes):
+        self._in_buffer += bytes
+        self.bytes_left -= len(bytes)
+        if self.bytes_left <= 0:
+            # Finished with body
+            if self.bytes_left != 0:
+                self._trailer_buffer = self._in_buffer[self.bytes_left:]
+                self._in_buffer = self._in_buffer[:self.bytes_left]
+            self.bytes_left = None
+            self.state_accept = self._state_accept_reading_trailer
+        
+    def _state_accept_reading_trailer(self, bytes):
+        self._trailer_buffer += bytes
+        # TODO: what if the trailer does not match "done\n"?  Should this raise
+        # a ProtocolViolation exception?
+        if self._trailer_buffer.startswith('done\n'):
+            self.unused_data = self._trailer_buffer[len('done\n'):]
+            self.state_accept = self._state_accept_reading_unused
+            self.finished_reading = True
+    
+    def _state_accept_reading_unused(self, bytes):
+        self.unused_data += bytes
+
+    def _state_read_no_data(self):
+        return ''
+
+    def _state_read_in_buffer(self):
+        result = self._in_buffer
+        self._in_buffer = ''
+        return result
+
+
+class SmartServerStreamMedium(object):
+    """Handles smart commands coming over a stream.
+
+    The stream may be a pipe connected to sshd, or a tcp socket, or an
+    in-process fifo for testing.
+
+    One instance is created for each connected client; it can serve multiple
+    requests in the lifetime of the connection.
+
+    The server passes requests through to an underlying backing transport, 
+    which will typically be a LocalTransport looking at the server's filesystem.
+    """
+
+    def __init__(self, backing_transport):
+        """Construct new server.
+
+        :param backing_transport: Transport for the directory served.
+        """
+        # backing_transport could be passed to serve instead of __init__
+        self.backing_transport = backing_transport
+        self.finished = False
+
+    def serve(self):
+        """Serve requests until the client disconnects."""
+        # Keep a reference to stderr because the sys module's globals get set to
+        # None during interpreter shutdown.
+        from sys import stderr
+        try:
+            while not self.finished:
+                protocol = SmartServerRequestProtocolOne(self.backing_transport,
+                                                         self._write_out)
+                self._serve_one_request(protocol)
+        except Exception, e:
+            stderr.write("%s terminating on exception %s\n" % (self, e))
+            raise
+
+    def _serve_one_request(self, protocol):
+        """Read one request from input, process, send back a response.
+        
+        :param protocol: a SmartServerRequestProtocol.
+        """
+        try:
+            self._serve_one_request_unguarded(protocol)
+        except KeyboardInterrupt:
+            raise
+        except Exception, e:
+            self.terminate_due_to_error()
+
+    def terminate_due_to_error(self):
+        """Called when an unhandled exception from the protocol occurs."""
+        raise NotImplementedError(self.terminate_due_to_error)
+
+
+class SmartServerSocketStreamMedium(SmartServerStreamMedium):
+
+    def __init__(self, sock, backing_transport):
+        """Constructor.
+
+        :param sock: the socket the server will read from.  It will be put
+            into blocking mode.
+        """
+        SmartServerStreamMedium.__init__(self, backing_transport)
+        self.push_back = ''
+        sock.setblocking(True)
+        self.socket = sock
+
+    def _serve_one_request_unguarded(self, protocol):
+        while protocol.next_read_size():
+            if self.push_back:
+                protocol.accept_bytes(self.push_back)
+                self.push_back = ''
+            else:
+                bytes = self.socket.recv(4096)
+                if bytes == '':
+                    self.finished = True
+                    return
+                protocol.accept_bytes(bytes)
+        
+        self.push_back = protocol.excess_buffer
+    
+    def terminate_due_to_error(self):
+        """Called when an unhandled exception from the protocol occurs."""
+        # TODO: This should log to a server log file, but no such thing
+        # exists yet.  Andrew Bennetts 2006-09-29.
+        self.socket.close()
+        self.finished = True
+
+    def _write_out(self, bytes):
+        self.socket.sendall(bytes)
+
+
+class SmartServerPipeStreamMedium(SmartServerStreamMedium):
+
+    def __init__(self, in_file, out_file, backing_transport):
+        """Construct new server.
+
+        :param in_file: Python file from which requests can be read.
+        :param out_file: Python file to write responses.
+        :param backing_transport: Transport for the directory served.
+        """
+        SmartServerStreamMedium.__init__(self, backing_transport)
+        self._in = in_file
+        self._out = out_file
+
+    def _serve_one_request_unguarded(self, protocol):
+        while True:
+            bytes_to_read = protocol.next_read_size()
+            if bytes_to_read == 0:
+                # Finished serving this request.
+                self._out.flush()
+                return
+            bytes = self._in.read(bytes_to_read)
+            if bytes == '':
+                # Connection has been closed.
+                self.finished = True
+                self._out.flush()
+                return
+            protocol.accept_bytes(bytes)
+
+    def terminate_due_to_error(self):
+        # TODO: This should log to a server log file, but no such thing
+        # exists yet.  Andrew Bennetts 2006-09-29.
+        self._out.close()
+        self.finished = True
+
+    def _write_out(self, bytes):
+        self._out.write(bytes)
+
+
+class SmartServerResponse(object):
+    """Response generated by SmartServerRequestHandler."""
+
+    def __init__(self, args, body=None):
+        self.args = args
+        self.body = body
+
+# XXX: TODO: Create a SmartServerRequestHandler which will take the responsibility
+# for delivering the data for a request. This could be done with as the
+# StreamServer, though that would create conflation between request and response
+# which may be undesirable.
+
+
+class SmartServerRequestHandler(object):
+    """Protocol logic for smart server.
+    
+    This doesn't handle serialization at all, it just processes requests and
+    creates responses.
+    """
+
+    # IMPORTANT FOR IMPLEMENTORS: It is important that SmartServerRequestHandler
+    # not contain encoding or decoding logic to allow the wire protocol to vary
+    # from the object protocol: we will want to tweak the wire protocol separate
+    # from the object model, and ideally we will be able to do that without
+    # having a SmartServerRequestHandler subclass for each wire protocol, rather
+    # just a Protocol subclass.
+
+    # TODO: Better way of representing the body for commands that take it,
+    # and allow it to be streamed into the server.
+    
+    def __init__(self, backing_transport):
+        self._backing_transport = backing_transport
+        self._converted_command = False
+        self.finished_reading = False
+        self._body_bytes = ''
+        self.response = None
+
+    def accept_body(self, bytes):
+        """Accept body data.
+
+        This should be overriden for each command that desired body data to
+        handle the right format of that data. I.e. plain bytes, a bundle etc.
+
+        The deserialisation into that format should be done in the Protocol
+        object. Set self.desired_body_format to the format your method will
+        handle.
+        """
+        # default fallback is to accumulate bytes.
+        self._body_bytes += bytes
+        
+    def _end_of_body_handler(self):
+        """An unimplemented end of body handler."""
+        raise NotImplementedError(self._end_of_body_handler)
+        
+    def do_hello(self):
+        """Answer a version request with my version."""
+        return SmartServerResponse(('ok', '1'))
+
+    def do_has(self, relpath):
+        r = self._backing_transport.has(relpath) and 'yes' or 'no'
+        return SmartServerResponse((r,))
+
+    def do_get(self, relpath):
+        backing_bytes = self._backing_transport.get_bytes(relpath)
+        return SmartServerResponse(('ok',), backing_bytes)
+
+    def _deserialise_optional_mode(self, mode):
+        # XXX: FIXME this should be on the protocol object.
+        if mode == '':
+            return None
+        else:
+            return int(mode)
+
+    def do_append(self, relpath, mode):
+        self._converted_command = True
+        self._relpath = relpath
+        self._mode = self._deserialise_optional_mode(mode)
+        self._end_of_body_handler = self._handle_do_append_end
+    
+    def _handle_do_append_end(self):
+        old_length = self._backing_transport.append_bytes(
+            self._relpath, self._body_bytes, self._mode)
+        self.response = SmartServerResponse(('appended', '%d' % old_length))
+
+    def do_delete(self, relpath):
+        self._backing_transport.delete(relpath)
+
+    def do_iter_files_recursive(self, abspath):
+        # XXX: the path handling needs some thought.
+        #relpath = self._backing_transport.relpath(abspath)
+        transport = self._backing_transport.clone(abspath)
+        filenames = transport.iter_files_recursive()
+        return SmartServerResponse(('names',) + tuple(filenames))
+
+    def do_list_dir(self, relpath):
+        filenames = self._backing_transport.list_dir(relpath)
+        return SmartServerResponse(('names',) + tuple(filenames))
+
+    def do_mkdir(self, relpath, mode):
+        self._backing_transport.mkdir(relpath,
+                                      self._deserialise_optional_mode(mode))
+
+    def do_move(self, rel_from, rel_to):
+        self._backing_transport.move(rel_from, rel_to)
+
+    def do_put(self, relpath, mode):
+        self._converted_command = True
+        self._relpath = relpath
+        self._mode = self._deserialise_optional_mode(mode)
+        self._end_of_body_handler = self._handle_do_put
+
+    def _handle_do_put(self):
+        self._backing_transport.put_bytes(self._relpath,
+                self._body_bytes, self._mode)
+        self.response = SmartServerResponse(('ok',))
+
+    def _deserialise_offsets(self, text):
+        # XXX: FIXME this should be on the protocol object.
+        offsets = []
+        for line in text.split('\n'):
+            if not line:
+                continue
+            start, length = line.split(',')
+            offsets.append((int(start), int(length)))
+        return offsets
+
+    def do_put_non_atomic(self, relpath, mode, create_parent, dir_mode):
+        self._converted_command = True
+        self._end_of_body_handler = self._handle_put_non_atomic
+        self._relpath = relpath
+        self._dir_mode = self._deserialise_optional_mode(dir_mode)
+        self._mode = self._deserialise_optional_mode(mode)
+        # a boolean would be nicer XXX
+        self._create_parent = (create_parent == 'T')
+
+    def _handle_put_non_atomic(self):
+        self._backing_transport.put_bytes_non_atomic(self._relpath,
+                self._body_bytes,
+                mode=self._mode,
+                create_parent_dir=self._create_parent,
+                dir_mode=self._dir_mode)
+        self.response = SmartServerResponse(('ok',))
+
+    def do_readv(self, relpath):
+        self._converted_command = True
+        self._end_of_body_handler = self._handle_readv_offsets
+        self._relpath = relpath
+
+    def end_of_body(self):
+        """No more body data will be received."""
+        self._run_handler_code(self._end_of_body_handler, (), {})
+        # cannot read after this.
+        self.finished_reading = True
+
+    def _handle_readv_offsets(self):
+        """accept offsets for a readv request."""
+        offsets = self._deserialise_offsets(self._body_bytes)
+        backing_bytes = ''.join(bytes for offset, bytes in
+            self._backing_transport.readv(self._relpath, offsets))
+        self.response = SmartServerResponse(('readv',), backing_bytes)
+        
+    def do_rename(self, rel_from, rel_to):
+        self._backing_transport.rename(rel_from, rel_to)
+
+    def do_rmdir(self, relpath):
+        self._backing_transport.rmdir(relpath)
+
+    def do_stat(self, relpath):
+        stat = self._backing_transport.stat(relpath)
+        return SmartServerResponse(('stat', str(stat.st_size), oct(stat.st_mode)))
+        
+    def do_get_bundle(self, path, revision_id):
+        # open transport relative to our base
+        t = self._backing_transport.clone(path)
+        control, extra_path = bzrdir.BzrDir.open_containing_from_transport(t)
+        repo = control.open_repository()
+        tmpf = tempfile.TemporaryFile()
+        base_revision = revision.NULL_REVISION
+        write_bundle(repo, revision_id, base_revision, tmpf)
+        tmpf.seek(0)
+        return SmartServerResponse((), tmpf.read())
+
+    def dispatch_command(self, cmd, args):
+        """Deprecated compatibility method.""" # XXX XXX
+        func = getattr(self, 'do_' + cmd, None)
+        if func is None:
+            raise errors.SmartProtocolError("bad request %r" % (cmd,))
+        self._run_handler_code(func, args, {})
+
+    def _run_handler_code(self, callable, args, kwargs):
+        """Run some handler specific code 'callable'.
+
+        If a result is returned, it is considered to be the commands response,
+        and finished_reading is set true, and its assigned to self.response.
+
+        Any exceptions caught are translated and a response object created
+        from them.
+        """
+        result = self._call_converting_errors(callable, args, kwargs)
+        if result is not None:
+            self.response = result
+            self.finished_reading = True
+        # handle unconverted commands
+        if not self._converted_command:
+            self.finished_reading = True
+            if result is None:
+                self.response = SmartServerResponse(('ok',))
+
+    def _call_converting_errors(self, callable, args, kwargs):
+        """Call callable converting errors to Response objects."""
+        try:
+            return callable(*args, **kwargs)
+        except errors.NoSuchFile, e:
+            return SmartServerResponse(('NoSuchFile', e.path))
+        except errors.FileExists, e:
+            return SmartServerResponse(('FileExists', e.path))
+        except errors.DirectoryNotEmpty, e:
+            return SmartServerResponse(('DirectoryNotEmpty', e.path))
+        except errors.ShortReadvError, e:
+            return SmartServerResponse(('ShortReadvError',
+                e.path, str(e.offset), str(e.length), str(e.actual)))
+        except UnicodeError, e:
+            # If it is a DecodeError, than most likely we are starting
+            # with a plain string
+            str_or_unicode = e.object
+            if isinstance(str_or_unicode, unicode):
+                val = u'u:' + str_or_unicode
+            else:
+                val = u's:' + str_or_unicode.encode('base64')
+            # This handles UnicodeEncodeError or UnicodeDecodeError
+            return SmartServerResponse((e.__class__.__name__,
+                    e.encoding, val, str(e.start), str(e.end), e.reason))
+        except errors.TransportNotPossible, e:
+            if e.msg == "readonly transport":
+                return SmartServerResponse(('ReadOnlyError', ))
+            else:
+                raise
+
+
+class SmartTCPServer(object):
+    """Listens on a TCP socket and accepts connections from smart clients"""
+
+    def __init__(self, backing_transport, host='127.0.0.1', port=0):
+        """Construct a new server.
+
+        To actually start it running, call either start_background_thread or
+        serve.
+
+        :param host: Name of the interface to listen on.
+        :param port: TCP port to listen on, or 0 to allocate a transient port.
+        """
+        self._server_socket = socket.socket()
+        self._server_socket.bind((host, port))
+        self.port = self._server_socket.getsockname()[1]
+        self._server_socket.listen(1)
+        self._server_socket.settimeout(1)
+        self.backing_transport = backing_transport
+
+    def serve(self):
+        # let connections timeout so that we get a chance to terminate
+        # Keep a reference to the exceptions we want to catch because the socket
+        # module's globals get set to None during interpreter shutdown.
+        from socket import timeout as socket_timeout
+        from socket import error as socket_error
+        self._should_terminate = False
+        while not self._should_terminate:
+            try:
+                self.accept_and_serve()
+            except socket_timeout:
+                # just check if we're asked to stop
+                pass
+            except socket_error, e:
+                trace.warning("client disconnected: %s", e)
+                pass
+
+    def get_url(self):
+        """Return the url of the server"""
+        return "bzr://%s:%d/" % self._server_socket.getsockname()
+
+    def accept_and_serve(self):
+        conn, client_addr = self._server_socket.accept()
+        # For WIN32, where the timeout value from the listening socket
+        # propogates to the newly accepted socket.
+        conn.setblocking(True)
+        conn.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        handler = SmartServerSocketStreamMedium(conn, self.backing_transport)
+        connection_thread = threading.Thread(None, handler.serve, name='smart-server-child')
+        connection_thread.setDaemon(True)
+        connection_thread.start()
+
+    def start_background_thread(self):
+        self._server_thread = threading.Thread(None,
+                self.serve,
+                name='server-' + self.get_url())
+        self._server_thread.setDaemon(True)
+        self._server_thread.start()
+
+    def stop_background_thread(self):
+        self._should_terminate = True
+        # self._server_socket.close()
+        # we used to join the thread, but it's not really necessary; it will
+        # terminate in time
+        ## self._server_thread.join()
+
+
+class SmartTCPServer_for_testing(SmartTCPServer):
+    """Server suitable for use by transport tests.
+    
+    This server is backed by the process's cwd.
+    """
+
+    def __init__(self):
+        self._homedir = urlutils.local_path_to_url(os.getcwd())[7:]
+        # The server is set up by default like for ssh access: the client
+        # passes filesystem-absolute paths; therefore the server must look
+        # them up relative to the root directory.  it might be better to act
+        # a public server and have the server rewrite paths into the test
+        # directory.
+        SmartTCPServer.__init__(self,
+            transport.get_transport(urlutils.local_path_to_url('/')))
+        
+    def setUp(self):
+        """Set up server for testing"""
+        self.start_background_thread()
+
+    def tearDown(self):
+        self.stop_background_thread()
+
+    def get_url(self):
+        """Return the url of the server"""
+        host, port = self._server_socket.getsockname()
+        return "bzr://%s:%d%s" % (host, port, urlutils.escape(self._homedir))
+
+    def get_bogus_url(self):
+        """Return a URL which will fail to connect"""
+        return 'bzr://127.0.0.1:1/'
+
+
+class SmartStat(object):
+
+    def __init__(self, size, mode):
+        self.st_size = size
+        self.st_mode = mode
+
+
+class SmartTransport(transport.Transport):
+    """Connection to a smart server.
+
+    The connection holds references to pipes that can be used to send requests
+    to the server.
+
+    The connection has a notion of the current directory to which it's
+    connected; this is incorporated in filenames passed to the server.
+    
+    This supports some higher-level RPC operations and can also be treated 
+    like a Transport to do file-like operations.
+
+    The connection can be made over a tcp socket, or (in future) an ssh pipe
+    or a series of http requests.  There are concrete subclasses for each
+    type: SmartTCPTransport, etc.
+    """
+
+    # IMPORTANT FOR IMPLEMENTORS: SmartTransport MUST NOT be given encoding
+    # responsibilities: Put those on SmartClient or similar. This is vital for
+    # the ability to support multiple versions of the smart protocol over time:
+    # SmartTransport is an adapter from the Transport object model to the 
+    # SmartClient model, not an encoder.
+
+    def __init__(self, url, clone_from=None, medium=None):
+        """Constructor.
+
+        :param medium: The medium to use for this RemoteTransport. This must be
+            supplied if clone_from is None.
+        """
+        ### Technically super() here is faulty because Transport's __init__
+        ### fails to take 2 parameters, and if super were to choose a silly
+        ### initialisation order things would blow up. 
+        if not url.endswith('/'):
+            url += '/'
+        super(SmartTransport, self).__init__(url)
+        self._scheme, self._username, self._password, self._host, self._port, self._path = \
+                transport.split_url(url)
+        if clone_from is None:
+            self._medium = medium
+        else:
+            # credentials may be stripped from the base in some circumstances
+            # as yet to be clearly defined or documented, so copy them.
+            self._username = clone_from._username
+            # reuse same connection
+            self._medium = clone_from._medium
+        assert self._medium is not None
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        
+        @param relpath: the relative path or path components
+        @type relpath: str or list
+        """
+        return self._unparse_url(self._remote_path(relpath))
+    
+    def clone(self, relative_url):
+        """Make a new SmartTransport related to me, sharing the same connection.
+
+        This essentially opens a handle on a different remote directory.
+        """
+        if relative_url is None:
+            return SmartTransport(self.base, self)
+        else:
+            return SmartTransport(self.abspath(relative_url), self)
+
+    def is_readonly(self):
+        """Smart server transport can do read/write file operations."""
+        return False
+                                                   
+    def get_smart_client(self):
+        return self._medium
+
+    def get_smart_medium(self):
+        return self._medium
+                                                   
+    def _unparse_url(self, path):
+        """Return URL for a path.
+
+        :see: SFTPUrlHandling._unparse_url
+        """
+        # TODO: Eventually it should be possible to unify this with
+        # SFTPUrlHandling._unparse_url?
+        if path == '':
+            path = '/'
+        path = urllib.quote(path)
+        netloc = urllib.quote(self._host)
+        if self._username is not None:
+            netloc = '%s@%s' % (urllib.quote(self._username), netloc)
+        if self._port is not None:
+            netloc = '%s:%d' % (netloc, self._port)
+        return urlparse.urlunparse((self._scheme, netloc, path, '', '', ''))
+
+    def _remote_path(self, relpath):
+        """Returns the Unicode version of the absolute path for relpath."""
+        return self._combine_paths(self._path, relpath)
+
+    def _call(self, method, *args):
+        resp = self._call2(method, *args)
+        self._translate_error(resp)
+
+    def _call2(self, method, *args):
+        """Call a method on the remote server."""
+        protocol = SmartClientRequestProtocolOne(self._medium.get_request())
+        protocol.call(method, *args)
+        return protocol.read_response_tuple()
+
+    def _call_with_body_bytes(self, method, args, body):
+        """Call a method on the remote server with body bytes."""
+        protocol = SmartClientRequestProtocolOne(self._medium.get_request())
+        protocol.call_with_body_bytes((method, ) + args, body)
+        return protocol.read_response_tuple()
+
+    def has(self, relpath):
+        """Indicate whether a remote file of the given name exists or not.
+
+        :see: Transport.has()
+        """
+        resp = self._call2('has', self._remote_path(relpath))
+        if resp == ('yes', ):
+            return True
+        elif resp == ('no', ):
+            return False
+        else:
+            self._translate_error(resp)
+
+    def get(self, relpath):
+        """Return file-like object reading the contents of a remote file.
+        
+        :see: Transport.get_bytes()/get_file()
+        """
+        return StringIO(self.get_bytes(relpath))
+
+    def get_bytes(self, relpath):
+        remote = self._remote_path(relpath)
+        protocol = SmartClientRequestProtocolOne(self._medium.get_request())
+        protocol.call('get', remote)
+        resp = protocol.read_response_tuple(True)
+        if resp != ('ok', ):
+            protocol.cancel_read_body()
+            self._translate_error(resp, relpath)
+        return protocol.read_body_bytes()
+
+    def _serialise_optional_mode(self, mode):
+        if mode is None:
+            return ''
+        else:
+            return '%d' % mode
+
+    def mkdir(self, relpath, mode=None):
+        resp = self._call2('mkdir', self._remote_path(relpath),
+            self._serialise_optional_mode(mode))
+        self._translate_error(resp)
+
+    def put_bytes(self, relpath, upload_contents, mode=None):
+        # FIXME: upload_file is probably not safe for non-ascii characters -
+        # should probably just pass all parameters as length-delimited
+        # strings?
+        resp = self._call_with_body_bytes('put',
+            (self._remote_path(relpath), self._serialise_optional_mode(mode)),
+            upload_contents)
+        self._translate_error(resp)
+
+    def put_bytes_non_atomic(self, relpath, bytes, mode=None,
+                             create_parent_dir=False,
+                             dir_mode=None):
+        """See Transport.put_bytes_non_atomic."""
+        # FIXME: no encoding in the transport!
+        create_parent_str = 'F'
+        if create_parent_dir:
+            create_parent_str = 'T'
+
+        resp = self._call_with_body_bytes(
+            'put_non_atomic',
+            (self._remote_path(relpath), self._serialise_optional_mode(mode),
+             create_parent_str, self._serialise_optional_mode(dir_mode)),
+            bytes)
+        self._translate_error(resp)
+
+    def put_file(self, relpath, upload_file, mode=None):
+        # its not ideal to seek back, but currently put_non_atomic_file depends
+        # on transports not reading before failing - which is a faulty
+        # assumption I think - RBC 20060915
+        pos = upload_file.tell()
+        try:
+            return self.put_bytes(relpath, upload_file.read(), mode)
+        except:
+            upload_file.seek(pos)
+            raise
+
+    def put_file_non_atomic(self, relpath, f, mode=None,
+                            create_parent_dir=False,
+                            dir_mode=None):
+        return self.put_bytes_non_atomic(relpath, f.read(), mode=mode,
+                                         create_parent_dir=create_parent_dir,
+                                         dir_mode=dir_mode)
+
+    def append_file(self, relpath, from_file, mode=None):
+        return self.append_bytes(relpath, from_file.read(), mode)
+        
+    def append_bytes(self, relpath, bytes, mode=None):
+        resp = self._call_with_body_bytes(
+            'append',
+            (self._remote_path(relpath), self._serialise_optional_mode(mode)),
+            bytes)
+        if resp[0] == 'appended':
+            return int(resp[1])
+        self._translate_error(resp)
+
+    def delete(self, relpath):
+        resp = self._call2('delete', self._remote_path(relpath))
+        self._translate_error(resp)
+
+    def readv(self, relpath, offsets):
+        if not offsets:
+            return
+
+        offsets = list(offsets)
+
+        sorted_offsets = sorted(offsets)
+        # turn the list of offsets into a stack
+        offset_stack = iter(offsets)
+        cur_offset_and_size = offset_stack.next()
+        coalesced = list(self._coalesce_offsets(sorted_offsets,
+                               limit=self._max_readv_combine,
+                               fudge_factor=self._bytes_to_read_before_seek))
+
+        protocol = SmartClientRequestProtocolOne(self._medium.get_request())
+        protocol.call_with_body_readv_array(
+            ('readv', self._remote_path(relpath)),
+            [(c.start, c.length) for c in coalesced])
+        resp = protocol.read_response_tuple(True)
+
+        if resp[0] != 'readv':
+            # This should raise an exception
+            protocol.cancel_read_body()
+            self._translate_error(resp)
+            return
+
+        # FIXME: this should know how many bytes are needed, for clarity.
+        data = protocol.read_body_bytes()
+        # Cache the results, but only until they have been fulfilled
+        data_map = {}
+        for c_offset in coalesced:
+            if len(data) < c_offset.length:
+                raise errors.ShortReadvError(relpath, c_offset.start,
+                            c_offset.length, actual=len(data))
+            for suboffset, subsize in c_offset.ranges:
+                key = (c_offset.start+suboffset, subsize)
+                data_map[key] = data[suboffset:suboffset+subsize]
+            data = data[c_offset.length:]
+
+            # Now that we've read some data, see if we can yield anything back
+            while cur_offset_and_size in data_map:
+                this_data = data_map.pop(cur_offset_and_size)
+                yield cur_offset_and_size[0], this_data
+                cur_offset_and_size = offset_stack.next()
+
+    def rename(self, rel_from, rel_to):
+        self._call('rename',
+                   self._remote_path(rel_from),
+                   self._remote_path(rel_to))
+
+    def move(self, rel_from, rel_to):
+        self._call('move',
+                   self._remote_path(rel_from),
+                   self._remote_path(rel_to))
+
+    def rmdir(self, relpath):
+        resp = self._call('rmdir', self._remote_path(relpath))
+
+    def _translate_error(self, resp, orig_path=None):
+        """Raise an exception from a response"""
+        if resp is None:
+            what = None
+        else:
+            what = resp[0]
+        if what == 'ok':
+            return
+        elif what == 'NoSuchFile':
+            if orig_path is not None:
+                error_path = orig_path
+            else:
+                error_path = resp[1]
+            raise errors.NoSuchFile(error_path)
+        elif what == 'error':
+            raise errors.SmartProtocolError(unicode(resp[1]))
+        elif what == 'FileExists':
+            raise errors.FileExists(resp[1])
+        elif what == 'DirectoryNotEmpty':
+            raise errors.DirectoryNotEmpty(resp[1])
+        elif what == 'ShortReadvError':
+            raise errors.ShortReadvError(resp[1], int(resp[2]),
+                                         int(resp[3]), int(resp[4]))
+        elif what in ('UnicodeEncodeError', 'UnicodeDecodeError'):
+            encoding = str(resp[1]) # encoding must always be a string
+            val = resp[2]
+            start = int(resp[3])
+            end = int(resp[4])
+            reason = str(resp[5]) # reason must always be a string
+            if val.startswith('u:'):
+                val = val[2:]
+            elif val.startswith('s:'):
+                val = val[2:].decode('base64')
+            if what == 'UnicodeDecodeError':
+                raise UnicodeDecodeError(encoding, val, start, end, reason)
+            elif what == 'UnicodeEncodeError':
+                raise UnicodeEncodeError(encoding, val, start, end, reason)
+        elif what == "ReadOnlyError":
+            raise errors.TransportNotPossible('readonly transport')
+        else:
+            raise errors.SmartProtocolError('unexpected smart server error: %r' % (resp,))
+
+    def disconnect(self):
+        self._medium.disconnect()
+
+    def delete_tree(self, relpath):
+        raise errors.TransportNotPossible('readonly transport')
+
+    def stat(self, relpath):
+        resp = self._call2('stat', self._remote_path(relpath))
+        if resp[0] == 'stat':
+            return SmartStat(int(resp[1]), int(resp[2], 8))
+        else:
+            self._translate_error(resp)
+
+    ## def lock_read(self, relpath):
+    ##     """Lock the given file for shared (read) access.
+    ##     :return: A lock object, which should be passed to Transport.unlock()
+    ##     """
+    ##     # The old RemoteBranch ignore lock for reading, so we will
+    ##     # continue that tradition and return a bogus lock object.
+    ##     class BogusLock(object):
+    ##         def __init__(self, path):
+    ##             self.path = path
+    ##         def unlock(self):
+    ##             pass
+    ##     return BogusLock(relpath)
+
+    def listable(self):
+        return True
+
+    def list_dir(self, relpath):
+        resp = self._call2('list_dir', self._remote_path(relpath))
+        if resp[0] == 'names':
+            return [name.encode('ascii') for name in resp[1:]]
+        else:
+            self._translate_error(resp)
+
+    def iter_files_recursive(self):
+        resp = self._call2('iter_files_recursive', self._remote_path(''))
+        if resp[0] == 'names':
+            return resp[1:]
+        else:
+            self._translate_error(resp)
+
+
+class SmartClientMediumRequest(object):
+    """A request on a SmartClientMedium.
+
+    Each request allows bytes to be provided to it via accept_bytes, and then
+    the response bytes to be read via read_bytes.
+
+    For instance:
+    request.accept_bytes('123')
+    request.finished_writing()
+    result = request.read_bytes(3)
+    request.finished_reading()
+
+    It is up to the individual SmartClientMedium whether multiple concurrent
+    requests can exist. See SmartClientMedium.get_request to obtain instances 
+    of SmartClientMediumRequest, and the concrete Medium you are using for 
+    details on concurrency and pipelining.
+    """
+
+    def __init__(self, medium):
+        """Construct a SmartClientMediumRequest for the medium medium."""
+        self._medium = medium
+        # we track state by constants - we may want to use the same
+        # pattern as BodyReader if it gets more complex.
+        # valid states are: "writing", "reading", "done"
+        self._state = "writing"
+
+    def accept_bytes(self, bytes):
+        """Accept bytes for inclusion in this request.
+
+        This method may not be be called after finished_writing() has been
+        called.  It depends upon the Medium whether or not the bytes will be
+        immediately transmitted. Message based Mediums will tend to buffer the
+        bytes until finished_writing() is called.
+
+        :param bytes: A bytestring.
+        """
+        if self._state != "writing":
+            raise errors.WritingCompleted(self)
+        self._accept_bytes(bytes)
+
+    def _accept_bytes(self, bytes):
+        """Helper for accept_bytes.
+
+        Accept_bytes checks the state of the request to determing if bytes
+        should be accepted. After that it hands off to _accept_bytes to do the
+        actual acceptance.
+        """
+        raise NotImplementedError(self._accept_bytes)
+
+    def finished_reading(self):
+        """Inform the request that all desired data has been read.
+
+        This will remove the request from the pipeline for its medium (if the
+        medium supports pipelining) and any further calls to methods on the
+        request will raise ReadingCompleted.
+        """
+        if self._state == "writing":
+            raise errors.WritingNotComplete(self)
+        if self._state != "reading":
+            raise errors.ReadingCompleted(self)
+        self._state = "done"
+        self._finished_reading()
+
+    def _finished_reading(self):
+        """Helper for finished_reading.
+
+        finished_reading checks the state of the request to determine if 
+        finished_reading is allowed, and if it is hands off to _finished_reading
+        to perform the action.
+        """
+        raise NotImplementedError(self._finished_reading)
+
+    def finished_writing(self):
+        """Finish the writing phase of this request.
+
+        This will flush all pending data for this request along the medium.
+        After calling finished_writing, you may not call accept_bytes anymore.
+        """
+        if self._state != "writing":
+            raise errors.WritingCompleted(self)
+        self._state = "reading"
+        self._finished_writing()
+
+    def _finished_writing(self):
+        """Helper for finished_writing.
+
+        finished_writing checks the state of the request to determine if 
+        finished_writing is allowed, and if it is hands off to _finished_writing
+        to perform the action.
+        """
+        raise NotImplementedError(self._finished_writing)
+
+    def read_bytes(self, count):
+        """Read bytes from this requests response.
+
+        This method will block and wait for count bytes to be read. It may not
+        be invoked until finished_writing() has been called - this is to ensure
+        a message-based approach to requests, for compatability with message
+        based mediums like HTTP.
+        """
+        if self._state == "writing":
+            raise errors.WritingNotComplete(self)
+        if self._state != "reading":
+            raise errors.ReadingCompleted(self)
+        return self._read_bytes(count)
+
+    def _read_bytes(self, count):
+        """Helper for read_bytes.
+
+        read_bytes checks the state of the request to determing if bytes
+        should be read. After that it hands off to _read_bytes to do the
+        actual read.
+        """
+        raise NotImplementedError(self._read_bytes)
+
+
+class SmartClientStreamMediumRequest(SmartClientMediumRequest):
+    """A SmartClientMediumRequest that works with an SmartClientStreamMedium."""
+
+    def __init__(self, medium):
+        SmartClientMediumRequest.__init__(self, medium)
+        # check that we are safe concurrency wise. If some streams start
+        # allowing concurrent requests - i.e. via multiplexing - then this
+        # assert should be moved to SmartClientStreamMedium.get_request,
+        # and the setting/unsetting of _current_request likewise moved into
+        # that class : but its unneeded overhead for now. RBC 20060922
+        if self._medium._current_request is not None:
+            raise errors.TooManyConcurrentRequests(self._medium)
+        self._medium._current_request = self
+
+    def _accept_bytes(self, bytes):
+        """See SmartClientMediumRequest._accept_bytes.
+        
+        This forwards to self._medium._accept_bytes because we are operating
+        on the mediums stream.
+        """
+        self._medium._accept_bytes(bytes)
+
+    def _finished_reading(self):
+        """See SmartClientMediumRequest._finished_reading.
+
+        This clears the _current_request on self._medium to allow a new 
+        request to be created.
+        """
+        assert self._medium._current_request is self
+        self._medium._current_request = None
+        
+    def _finished_writing(self):
+        """See SmartClientMediumRequest._finished_writing.
+
+        This invokes self._medium._flush to ensure all bytes are transmitted.
+        """
+        self._medium._flush()
+
+    def _read_bytes(self, count):
+        """See SmartClientMediumRequest._read_bytes.
+        
+        This forwards to self._medium._read_bytes because we are operating
+        on the mediums stream.
+        """
+        return self._medium._read_bytes(count)
+
+
+class SmartClientRequestProtocolOne(SmartProtocolBase):
+    """The client-side protocol for smart version 1."""
+
+    def __init__(self, request):
+        """Construct a SmartClientRequestProtocolOne.
+
+        :param request: A SmartClientMediumRequest to serialise onto and
+            deserialise from.
+        """
+        self._request = request
+        self._body_buffer = None
+
+    def call(self, *args):
+        bytes = _encode_tuple(args)
+        self._request.accept_bytes(bytes)
+        self._request.finished_writing()
+
+    def call_with_body_bytes(self, args, body):
+        """Make a remote call of args with body bytes 'body'.
+
+        After calling this, call read_response_tuple to find the result out.
+        """
+        bytes = _encode_tuple(args)
+        self._request.accept_bytes(bytes)
+        bytes = self._encode_bulk_data(body)
+        self._request.accept_bytes(bytes)
+        self._request.finished_writing()
+
+    def call_with_body_readv_array(self, args, body):
+        """Make a remote call with a readv array.
+
+        The body is encoded with one line per readv offset pair. The numbers in
+        each pair are separated by a comma, and no trailing \n is emitted.
+        """
+        bytes = _encode_tuple(args)
+        self._request.accept_bytes(bytes)
+        readv_bytes = self._serialise_offsets(body)
+        bytes = self._encode_bulk_data(readv_bytes)
+        self._request.accept_bytes(bytes)
+        self._request.finished_writing()
+
+    def cancel_read_body(self):
+        """After expecting a body, a response code may indicate one otherwise.
+
+        This method lets the domain client inform the protocol that no body
+        will be transmitted. This is a terminal method: after calling it the
+        protocol is not able to be used further.
+        """
+        self._request.finished_reading()
+
+    def read_response_tuple(self, expect_body=False):
+        """Read a response tuple from the wire.
+
+        This should only be called once.
+        """
+        result = self._recv_tuple()
+        if not expect_body:
+            self._request.finished_reading()
+        return result
+
+    def read_body_bytes(self, count=-1):
+        """Read bytes from the body, decoding into a byte stream.
+        
+        We read all bytes at once to ensure we've checked the trailer for 
+        errors, and then feed the buffer back as read_body_bytes is called.
+        """
+        if self._body_buffer is not None:
+            return self._body_buffer.read(count)
+        _body_decoder = LengthPrefixedBodyDecoder()
+
+        while not _body_decoder.finished_reading:
+            bytes_wanted = _body_decoder.next_read_size()
+            bytes = self._request.read_bytes(bytes_wanted)
+            _body_decoder.accept_bytes(bytes)
+        self._request.finished_reading()
+        self._body_buffer = StringIO(_body_decoder.read_pending_data())
+        # XXX: TODO check the trailer result.
+        return self._body_buffer.read(count)
+
+    def _recv_tuple(self):
+        """Receive a tuple from the medium request."""
+        line = ''
+        while not line or line[-1] != '\n':
+            # TODO: this is inefficient - but tuples are short.
+            new_char = self._request.read_bytes(1)
+            line += new_char
+            assert new_char != '', "end of file reading from server."
+        return _decode_tuple(line)
+
+    def query_version(self):
+        """Return protocol version number of the server."""
+        self.call('hello')
+        resp = self.read_response_tuple()
+        if resp == ('ok', '1'):
+            return 1
+        else:
+            raise errors.SmartProtocolError("bad response %r" % (resp,))
+
+
+class SmartClientMedium(object):
+    """Smart client is a medium for sending smart protocol requests over."""
+
+    def disconnect(self):
+        """If this medium maintains a persistent connection, close it.
+        
+        The default implementation does nothing.
+        """
+        
+
+class SmartClientStreamMedium(SmartClientMedium):
+    """Stream based medium common class.
+
+    SmartClientStreamMediums operate on a stream. All subclasses use a common
+    SmartClientStreamMediumRequest for their requests, and should implement
+    _accept_bytes and _read_bytes to allow the request objects to send and
+    receive bytes.
+    """
+
+    def __init__(self):
+        self._current_request = None
+
+    def accept_bytes(self, bytes):
+        self._accept_bytes(bytes)
+
+    def __del__(self):
+        """The SmartClientStreamMedium knows how to close the stream when it is
+        finished with it.
+        """
+        self.disconnect()
+
+    def _flush(self):
+        """Flush the output stream.
+        
+        This method is used by the SmartClientStreamMediumRequest to ensure that
+        all data for a request is sent, to avoid long timeouts or deadlocks.
+        """
+        raise NotImplementedError(self._flush)
+
+    def get_request(self):
+        """See SmartClientMedium.get_request().
+
+        SmartClientStreamMedium always returns a SmartClientStreamMediumRequest
+        for get_request.
+        """
+        return SmartClientStreamMediumRequest(self)
+
+    def read_bytes(self, count):
+        return self._read_bytes(count)
+
+
+class SmartSimplePipesClientMedium(SmartClientStreamMedium):
+    """A client medium using simple pipes.
+    
+    This client does not manage the pipes: it assumes they will always be open.
+    """
+
+    def __init__(self, readable_pipe, writeable_pipe):
+        SmartClientStreamMedium.__init__(self)
+        self._readable_pipe = readable_pipe
+        self._writeable_pipe = writeable_pipe
+
+    def _accept_bytes(self, bytes):
+        """See SmartClientStreamMedium.accept_bytes."""
+        self._writeable_pipe.write(bytes)
+
+    def _flush(self):
+        """See SmartClientStreamMedium._flush()."""
+        self._writeable_pipe.flush()
+
+    def _read_bytes(self, count):
+        """See SmartClientStreamMedium._read_bytes."""
+        return self._readable_pipe.read(count)
+
+
+class SmartSSHClientMedium(SmartClientStreamMedium):
+    """A client medium using SSH."""
+    
+    def __init__(self, host, port=None, username=None, password=None,
+            vendor=None):
+        """Creates a client that will connect on the first use.
+        
+        :param vendor: An optional override for the ssh vendor to use. See
+            bzrlib.transport.ssh for details on ssh vendors.
+        """
+        SmartClientStreamMedium.__init__(self)
+        self._connected = False
+        self._host = host
+        self._password = password
+        self._port = port
+        self._username = username
+        self._read_from = None
+        self._ssh_connection = None
+        self._vendor = vendor
+        self._write_to = None
+
+    def _accept_bytes(self, bytes):
+        """See SmartClientStreamMedium.accept_bytes."""
+        self._ensure_connection()
+        self._write_to.write(bytes)
+
+    def disconnect(self):
+        """See SmartClientMedium.disconnect()."""
+        if not self._connected:
+            return
+        self._read_from.close()
+        self._write_to.close()
+        self._ssh_connection.close()
+        self._connected = False
+
+    def _ensure_connection(self):
+        """Connect this medium if not already connected."""
+        if self._connected:
+            return
+        executable = os.environ.get('BZR_REMOTE_PATH', 'bzr')
+        if self._vendor is None:
+            vendor = ssh._get_ssh_vendor()
+        else:
+            vendor = self._vendor
+        self._ssh_connection = vendor.connect_ssh(self._username,
+                self._password, self._host, self._port,
+                command=[executable, 'serve', '--inet', '--directory=/',
+                         '--allow-writes'])
+        self._read_from, self._write_to = \
+            self._ssh_connection.get_filelike_channels()
+        self._connected = True
+
+    def _flush(self):
+        """See SmartClientStreamMedium._flush()."""
+        self._write_to.flush()
+
+    def _read_bytes(self, count):
+        """See SmartClientStreamMedium.read_bytes."""
+        if not self._connected:
+            raise errors.MediumNotConnected(self)
+        return self._read_from.read(count)
+
+
+class SmartTCPClientMedium(SmartClientStreamMedium):
+    """A client medium using TCP."""
+    
+    def __init__(self, host, port):
+        """Creates a client that will connect on the first use."""
+        SmartClientStreamMedium.__init__(self)
+        self._connected = False
+        self._host = host
+        self._port = port
+        self._socket = None
+
+    def _accept_bytes(self, bytes):
+        """See SmartClientMedium.accept_bytes."""
+        self._ensure_connection()
+        self._socket.sendall(bytes)
+
+    def disconnect(self):
+        """See SmartClientMedium.disconnect()."""
+        if not self._connected:
+            return
+        self._socket.close()
+        self._socket = None
+        self._connected = False
+
+    def _ensure_connection(self):
+        """Connect this medium if not already connected."""
+        if self._connected:
+            return
+        self._socket = socket.socket()
+        self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        result = self._socket.connect_ex((self._host, int(self._port)))
+        if result:
+            raise errors.ConnectionError("failed to connect to %s:%d: %s" %
+                    (self._host, self._port, os.strerror(result)))
+        self._connected = True
+
+    def _flush(self):
+        """See SmartClientStreamMedium._flush().
+        
+        For TCP we do no flushing. We may want to turn off TCP_NODELAY and 
+        add a means to do a flush, but that can be done in the future.
+        """
+
+    def _read_bytes(self, count):
+        """See SmartClientMedium.read_bytes."""
+        if not self._connected:
+            raise errors.MediumNotConnected(self)
+        return self._socket.recv(count)
+
+
+class SmartTCPTransport(SmartTransport):
+    """Connection to smart server over plain tcp.
+    
+    This is essentially just a factory to get 'RemoteTransport(url,
+        SmartTCPClientMedium).
+    """
+
+    def __init__(self, url):
+        _scheme, _username, _password, _host, _port, _path = \
+            transport.split_url(url)
+        try:
+            _port = int(_port)
+        except (ValueError, TypeError), e:
+            raise errors.InvalidURL(path=url, extra="invalid port %s" % _port)
+        medium = SmartTCPClientMedium(_host, _port)
+        super(SmartTCPTransport, self).__init__(url, medium=medium)
+
+
+class SmartSSHTransport(SmartTransport):
+    """Connection to smart server over SSH.
+
+    This is essentially just a factory to get 'RemoteTransport(url,
+        SmartSSHClientMedium).
+    """
+
+    def __init__(self, url):
+        _scheme, _username, _password, _host, _port, _path = \
+            transport.split_url(url)
+        try:
+            if _port is not None:
+                _port = int(_port)
+        except (ValueError, TypeError), e:
+            raise errors.InvalidURL(path=url, extra="invalid port %s" % 
+                _port)
+        medium = SmartSSHClientMedium(_host, _port, _username, _password)
+        super(SmartSSHTransport, self).__init__(url, medium=medium)
+
+
+def get_test_permutations():
+    """Return (transport, server) permutations for testing."""
+    ### We may need a little more test framework support to construct an
+    ### appropriate RemoteTransport in the future.
+    return [(SmartTCPTransport, SmartTCPServer_for_testing)]

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2006-08-30 13:50:53 +0000
+++ bzrlib/transport/ssh.py	2006-09-18 09:47:56 +0000
@@ -28,6 +28,7 @@
 from bzrlib.errors import (ConnectionError,
                            ParamikoNotPresent,
                            TransportError,
+                           UnknownSSH,
                            )
 
 from bzrlib.osutils import pathjoin
@@ -101,7 +102,7 @@
     # A check for putty's plink or lsh would go here.
 
     mutter('falling back to paramiko implementation')
-    _ssh_vendor = ssh.ParamikoVendor()
+    _ssh_vendor = ParamikoVendor()
     return _ssh_vendor
 
 
@@ -153,10 +154,10 @@
         raise NotImplementedError(self.connect_sftp)
 
     def connect_ssh(self, username, password, host, port, command):
-        """Make an SSH connection, and return a pipe-like object.
+        """Make an SSH connection.
         
-        (This is currently unused, it's just here to indicate future directions
-        for this code.)
+        :returns: something with a `close` method, and a `get_filelike_channels`
+            method that returns a pair of (read, write) filelike objects.
         """
         raise NotImplementedError(self.connect_ssh)
         
@@ -176,10 +177,21 @@
 register_ssh_vendor('loopback', LoopbackVendor())
 
 
+class _ParamikoSSHConnection(object):
+    def __init__(self, channel):
+        self.channel = channel
+
+    def get_filelike_channels(self):
+        return self.channel.makefile('rb'), self.channel.makefile('wb')
+
+    def close(self):
+        return self.channel.close()
+
+
 class ParamikoVendor(SSHVendor):
     """Vendor that uses paramiko."""
 
-    def connect_sftp(self, username, password, host, port):
+    def _connect(self, username, password, host, port):
         global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
         
         load_host_keys()
@@ -217,13 +229,26 @@
                 ['Try editing %s or %s' % (filename1, filename2)])
 
         _paramiko_auth(username, password, host, t)
+        return t
         
+    def connect_sftp(self, username, password, host, port):
+        t = self._connect(username, password, host, port)
         try:
-            sftp = t.open_sftp_client()
+            return t.open_sftp_client()
         except paramiko.SSHException, e:
             raise ConnectionError('Unable to start sftp client %s:%d' %
                                   (host, port), e)
-        return sftp
+
+    def connect_ssh(self, username, password, host, port, command):
+        t = self._connect(username, password, host, port)
+        try:
+            channel = t.open_session()
+            cmdline = ' '.join(command)
+            channel.exec_command(cmdline)
+            return _ParamikoSSHConnection(channel)
+        except paramiko.SSHException, e:
+            raise ConnectionError('Unable to invoke remote bzr %s:%d' %
+                                  (host, port), e)
 
 register_ssh_vendor('paramiko', ParamikoVendor())
 
@@ -231,15 +256,18 @@
 class SubprocessVendor(SSHVendor):
     """Abstract base class for vendors that use pipes to a subprocess."""
     
+    def _connect(self, argv):
+        proc = subprocess.Popen(argv,
+                                stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE,
+                                **os_specific_subprocess_params())
+        return SSHSubprocess(proc)
+
     def connect_sftp(self, username, password, host, port):
         try:
             argv = self._get_vendor_specific_argv(username, host, port,
                                                   subsystem='sftp')
-            proc = subprocess.Popen(argv,
-                                    stdin=subprocess.PIPE,
-                                    stdout=subprocess.PIPE,
-                                    **os_specific_subprocess_params())
-            sock = SSHSubprocess(proc)
+            sock = self._connect(argv)
             return SFTPClient(sock)
         except (EOFError, paramiko.SSHException), e:
             raise ConnectionError('Unable to connect to SSH host %s:%s: %s'
@@ -253,6 +281,23 @@
             raise ConnectionError('Unable to connect to SSH host %s:%s: %s'
                                   % (host, port, e))
 
+    def connect_ssh(self, username, password, host, port, command):
+        try:
+            argv = self._get_vendor_specific_argv(username, host, port,
+                                                  command=command)
+            return self._connect(argv)
+        except (EOFError), e:
+            raise ConnectionError('Unable to connect to SSH host %s:%s: %s'
+                                  % (host, port, e))
+        except (OSError, IOError), e:
+            # If the machine is fast enough, ssh can actually exit
+            # before we try and send it the sftp request, which
+            # raises a Broken Pipe
+            if e.errno not in (errno.EPIPE,):
+                raise
+            raise ConnectionError('Unable to connect to SSH host %s:%s: %s'
+                                  % (host, port, e))
+
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
         """Returns the argument list to run the subprocess with.
@@ -468,4 +513,6 @@
         self.proc.stdout.close()
         self.proc.wait()
 
+    def get_filelike_channels(self):
+        return (self.proc.stdout, self.proc.stdin)
 

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-08-30 13:54:51 +0000
+++ bzrlib/tree.py	2006-09-28 22:56:04 +0000
@@ -54,7 +54,7 @@
     """
     
     def changes_from(self, other, want_unchanged=False, specific_files=None,
-        extra_trees=None, require_versioned=False):
+        extra_trees=None, require_versioned=False, include_root=False):
         """Return a TreeDelta of the changes from other to this tree.
 
         :param other: A tree to compare with.
@@ -81,6 +81,7 @@
             specific_files=specific_files,
             extra_trees=extra_trees,
             require_versioned=require_versioned,
+            include_root=include_root
             )
     
     def conflicts(self):
@@ -119,6 +120,18 @@
     def id2path(self, file_id):
         return self.inventory.id2path(file_id)
 
+    def is_control_filename(self, filename):
+        """True if filename is the name of a control file in this tree.
+        
+        :param filename: A filename within the tree. This is a relative path
+        from the root of this tree.
+
+        This is true IF and ONLY IF the filename is part of the meta data
+        that bzr controls in this tree. I.E. a random .bzr directory placed
+        on disk will not be a control file for this tree.
+        """
+        return self.bzrdir.is_control_filename(filename)
+
     def iter_entries_by_dir(self):
         """Walk the tree in 'by_dir' order.
 
@@ -135,6 +148,10 @@
     def _get_inventory(self):
         return self._inventory
     
+    def get_file(self, file_id):
+        """Return a file object for the file file_id in the tree."""
+        raise NotImplementedError(self.get_file)
+    
     def get_file_by_path(self, path):
         return self.get_file(self._inventory.path2id(path))
 
@@ -147,7 +164,7 @@
         fp = fingerprint_file(f)
         f.seek(0)
         
-        if ie.text_size != None:
+        if ie.text_size is not None:
             if ie.text_size != fp['size']:
                 raise BzrError("mismatched size for file %r in %r" % (ie.file_id, self._store),
                         ["inventory expects %d bytes" % ie.text_size,
@@ -160,6 +177,9 @@
                      "file is actually %s" % fp['sha1'],
                      "store is probably damaged/corrupt"])
 
+    def path2id(self, path):
+        """Return the id for path in this tree."""
+        return self._inventory.path2id(path)
 
     def print_file(self, file_id):
         """Print file with id `file_id` to stdout."""
@@ -198,7 +218,7 @@
 class EmptyTree(Tree):
 
     def __init__(self):
-        self._inventory = Inventory()
+        self._inventory = Inventory(root_id=None)
         warn('EmptyTree is deprecated as of bzr 0.9 please use '
             'repository.revision_tree instead.',
             DeprecationWarning, stacklevel=2)
@@ -216,7 +236,7 @@
         assert self._inventory[file_id].kind == "directory"
         return "directory"
 
-    def list_files(self):
+    def list_files(self, include_root=False):
         return iter([])
     
     def __contains__(self, file_id):
@@ -375,11 +395,11 @@
     will pass through to InterTree as appropriate.
     """
 
-    _optimisers = set()
+    _optimisers = []
 
     @needs_read_lock
     def compare(self, want_unchanged=False, specific_files=None,
-        extra_trees=None, require_versioned=False):
+        extra_trees=None, require_versioned=False, include_root=False):
         """Return the changes from source to target.
 
         :return: A TreeDelta.
@@ -407,4 +427,4 @@
             # _compare_trees would think we want a complete delta
             return delta.TreeDelta()
         return delta._compare_trees(self.source, self.target, want_unchanged,
-            specific_file_ids)
+            specific_file_ids, include_root)

=== added file 'bzrlib/treebuilder.py'
--- bzrlib/treebuilder.py	1970-01-01 00:00:00 +0000
+++ bzrlib/treebuilder.py	2006-09-22 04:52:17 +0000
@@ -0,0 +1,79 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""TreeBuilder helper class.
+
+TreeBuilders are used to build trees of various shapres or properties. This 
+can be extremely useful in testing for instance.
+"""
+
+from bzrlib import errors
+
+
+class TreeBuilder(object):
+    """A TreeBuilder allows the creation of specific content in one tree at a
+    time.
+    """
+
+    def __init__(self):
+        """Construct a TreeBuilder."""
+        self._tree = None
+        self._root_done = False
+
+    def build(self, recipe):
+        """Build recipe into the current tree.
+
+        :param recipe: A sequence of paths. For each path, the corresponding
+            path in the current tree is created and added. If the path ends in
+            '/' then a directory is added, otherwise a regular file is added.
+        """
+        self._ensure_building()
+        if not self._root_done:
+            self._tree.add('', 'root-id', 'directory')
+            self._root_done = True
+        for name in recipe:
+            assert isinstance(name, basestring)
+            if name[-1] == '/':
+                self._tree.mkdir(name[:-1])
+            else:
+                end = '\n'
+                content = "contents of %s%s" % (name.encode('utf-8'), end)
+                self._tree.add(name, None, 'file')
+                file_id = self._tree.path2id(name)
+                self._tree.put_file_bytes_non_atomic(file_id, content)
+
+    def _ensure_building(self):
+        """Raise NotBuilding if there is no current tree being built."""
+        if self._tree is None:
+            raise errors.NotBuilding
+            
+    def finish_tree(self):
+        """Finish building the current tree."""
+        self._ensure_building()
+        tree = self._tree
+        self._tree = None
+        tree.unlock()
+
+    def start_tree(self, tree):
+        """Start building on tree.
+        
+        :param tree: A tree to start building on. It must provide the 
+            MutableTree interface.
+        """
+        if self._tree is not None:
+            raise errors.AlreadyBuilding
+        self._tree = tree
+        self._tree.lock_tree_write()

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2006-07-28 16:05:23 +0000
+++ bzrlib/ui/__init__.py	2006-10-06 07:29:17 +0000
@@ -30,7 +30,13 @@
 
 import sys
 
-import bzrlib.progress
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    progress,
+    )
+""")
+
 from bzrlib.symbol_versioning import (deprecated_method, zero_eight)
 
 
@@ -118,15 +124,15 @@
     @deprecated_method(zero_eight)
     def progress_bar(self):
         """See UIFactory.nested_progress_bar()."""
-        return bzrlib.progress.DummyProgress()
+        return progress.DummyProgress()
 
     def get_password(self, prompt='', **kwargs):
         return None
 
     def nested_progress_bar(self):
         if self._progress_bar_stack is None:
-            self._progress_bar_stack = bzrlib.progress.ProgressBarStack(
-                klass=bzrlib.progress.DummyProgress)
+            self._progress_bar_stack = progress.ProgressBarStack(
+                klass=progress.DummyProgress)
         return self._progress_bar_stack.get_nested()
 
     def clear_term(self):

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2006-07-28 16:05:23 +0000
+++ bzrlib/ui/text.py	2006-10-06 07:03:55 +0000
@@ -19,12 +19,21 @@
 """Text UI, write output to the console.
 """
 
+import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import getpass
-import sys
-
-import bzrlib.progress
-from bzrlib.symbol_versioning import (deprecated_method, 
-        zero_eight)
+
+from bzrlib import (
+    progress,
+    )
+""")
+
+from bzrlib.symbol_versioning import (
+    deprecated_method,
+    zero_eight,
+    )
 from bzrlib.ui import CLIUIFactory
 
 
@@ -61,7 +70,7 @@
         """See UIFactory.nested_progress_bar()."""
         # this in turn is abstract, and creates either a tty or dots
         # bar depending on what we think of the terminal
-        return bzrlib.progress.ProgressBar()
+        return progress.ProgressBar()
 
     def get_password(self, prompt='', **kwargs):
         """Prompt the user for a password.
@@ -86,7 +95,7 @@
         may return a tty or dots bar depending on the terminal.
         """
         if self._progress_bar_stack is None:
-            self._progress_bar_stack = bzrlib.progress.ProgressBarStack(
+            self._progress_bar_stack = progress.ProgressBarStack(
                 klass=self._bar_type)
         return self._progress_bar_stack.get_nested()
 

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2006-08-30 04:14:58 +0000
+++ bzrlib/uncommit.py	2006-09-05 09:49:40 +0000
@@ -40,7 +40,7 @@
 
         pending_merges = []
         if tree is not None:
-            pending_merges = tree.pending_merges()
+            pending_merges = tree.get_parent_ids()[1:]
 
         master = branch.get_master_branch()
         if master is not None:

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2006-03-23 14:51:52 +0000
+++ bzrlib/upgrade.py	2006-08-17 19:51:54 +0000
@@ -19,7 +19,7 @@
 # change upgrade from .bzr to create a '.bzr-new', then do a bait and switch.
 
 
-from bzrlib.bzrdir import ConvertBzrDir4To5, ConvertBzrDir5To6, BzrDir, BzrDirFormat4, BzrDirFormat5
+from bzrlib.bzrdir import ConvertBzrDir4To5, ConvertBzrDir5To6, BzrDir, BzrDirFormat4, BzrDirFormat5, BzrDirFormat
 import bzrlib.errors as errors
 from bzrlib.transport import get_transport
 import bzrlib.ui as ui
@@ -54,6 +54,11 @@
         if not self.bzrdir.can_convert_format():
             raise errors.BzrError("cannot upgrade from branch format %s" %
                            self.bzrdir._format)
+        if self.format is None:
+            target_format = BzrDirFormat.get_default_format()
+        else:
+            target_format = self.format
+        self.bzrdir.check_conversion_target(target_format)
         self.pb.note('starting upgrade of %s', self.transport.base)
         self._backup_control_dir()
         while self.bzrdir.needs_format_conversion(self.format):

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2006-07-30 06:52:39 +0000
+++ bzrlib/urlutils.py	2006-10-06 05:53:44 +0000
@@ -19,13 +19,19 @@
 """A collection of function for handling URL operations."""
 
 import os
-from posixpath import split as _posix_split, normpath as _posix_normpath
 import re
 import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from posixpath import split as _posix_split, normpath as _posix_normpath
 import urllib
 
-import bzrlib.errors as errors
-import bzrlib.osutils
+from bzrlib import (
+    errors,
+    osutils,
+    )
+""")
 
 
 def basename(url, exclude_trailing_slash=True):
@@ -75,7 +81,7 @@
 
     base = local_path_from_url(base)
     path = local_path_from_url(path)
-    return escape(bzrlib.osutils.relpath(base, path))
+    return escape(osutils.relpath(base, path))
 
 
 def _find_scheme_and_separator(url):
@@ -129,6 +135,8 @@
         if m:
             # Absolute URL
             scheme = m.group('scheme')
+            # this skips .. normalisation, making http://host/../../..
+            # be rather strange.
             path = m.group('path').split('/')
         else:
             for chunk in arg.split('/'):
@@ -166,7 +174,7 @@
     # importing directly from posixpath allows us to test this 
     # on non-posix platforms
     return 'file://' + escape(_posix_normpath(
-        bzrlib.osutils._posix_abspath(path)))
+        osutils._posix_abspath(path)))
 
 
 def _win32_local_path_from_url(url):
@@ -195,8 +203,8 @@
     #       which actually strips trailing space characters.
     #       The worst part is that under linux ntpath.abspath has different
     #       semantics, since 'nt' is not an available module.
-    win32_path = bzrlib.osutils._nt_normpath(
-        bzrlib.osutils._win32_abspath(path)).replace('\\', '/')
+    win32_path = osutils._nt_normpath(
+        osutils._win32_abspath(path)).replace('\\', '/')
     return 'file:///' + win32_path[0].upper() + ':' + escape(win32_path[2:])
 
 

=== modified file 'bzrlib/util/configobj/configobj.py'
--- bzrlib/util/configobj/configobj.py	2006-08-29 20:47:02 +0000
+++ bzrlib/util/configobj/configobj.py	2006-09-08 18:46:29 +0000
@@ -1024,7 +1024,7 @@
             else:
                 self.configspec = None
             return
-        elif hasattr(infile, 'read'):
+        elif getattr(infile, 'read', None) is not None:
             # This supports file like objects
             infile = infile.read() or []
             # needs splitting into lines - but needs doing *after* decoding
@@ -2852,7 +2852,7 @@
     >>> uc2 = ConfigObj(file_like)
     >>> uc2 == uc
     1
-    >>> uc2.filename == None
+    >>> uc2.filename is None
     1
     >>> uc2.newlines == '\\r'
     1

=== modified file 'bzrlib/util/elementtree/ElementTree.py'
--- bzrlib/util/elementtree/ElementTree.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/elementtree/ElementTree.py	2006-09-08 18:46:29 +0000
@@ -572,7 +572,7 @@
     # @defreturn Element
 
     def parse(self, source, parser=None):
-        if not hasattr(source, "read"):
+        if getattr(source, "read", None) is None:
             source = open(source, "rb")
         if not parser:
             parser = XMLTreeBuilder()
@@ -651,7 +651,7 @@
 
     def write(self, file, encoding="us-ascii"):
         assert self._root is not None
-        if not hasattr(file, "write"):
+        if getattr(file, "write", None) is None:
             file = open(file, "wb")
         if not encoding:
             encoding = "us-ascii"
@@ -723,7 +723,7 @@
 def iselement(element):
     # FIXME: not sure about this; might be a better idea to look
     # for tag/attrib/text attributes
-    return isinstance(element, _ElementInterface) or hasattr(element, "tag")
+    return isinstance(element, _ElementInterface) or (getattr(element, "tag", None) is not None)
 
 ##
 # Writes an element tree or element structure to sys.stdout.  This
@@ -871,7 +871,7 @@
 class iterparse:
 
     def __init__(self, source, events=None):
-        if not hasattr(source, "read"):
+        if getattr(source, "read", None) is None:
             source = open(source, "rb")
         self._file = source
         self._events = []
@@ -1037,7 +1037,7 @@
 
     def close(self):
         assert len(self._elem) == 0, "missing end tags"
-        assert self._last != None, "missing toplevel element"
+        assert self._last is not None, "missing toplevel element"
         return self._last
 
     def _flush(self):

=== added directory 'bzrlib/version_info_formats'
=== added file 'bzrlib/version_info_formats/__init__.py'
--- bzrlib/version_info_formats/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/version_info_formats/__init__.py	2006-09-29 00:04:13 +0000
@@ -0,0 +1,203 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Routines for extracting all version information from a bzr branch."""
+
+import time
+
+from bzrlib.osutils import local_time_offset, format_date
+
+
+# This contains a map of format id => formatter
+# None is considered the default formatter
+_version_formats = {}
+
+def create_date_str(timestamp=None, offset=None):
+    """Just a wrapper around format_date to provide the right format.
+
+    We don't want to use '%a' in the time string, because it is locale
+    dependant. We also want to force timezone original, and show_offset
+
+    Without parameters this function yields the current date in the local
+    time zone.
+    """
+    if timestamp is None and offset is None:
+        timestamp = time.time()
+        offset = local_time_offset()
+    return format_date(timestamp, offset, date_fmt='%Y-%m-%d %H:%M:%S',
+                       timezone='original', show_offset=True)
+
+
+class VersionInfoBuilder(object):
+    """A class which lets you build up information about a revision."""
+
+    def __init__(self, branch, working_tree=None,
+                check_for_clean=False,
+                include_revision_history=False,
+                include_file_revisions=False,
+                ):
+        """Build up information about the given branch.
+        If working_tree is given, it can be checked for changes.
+
+        :param branch: The branch to work on
+        :param working_tree: If supplied, preferentially check
+            the working tree for changes.
+        :param check_for_clean: If False, we will skip the expense
+            of looking for changes.
+        :param include_revision_history: If True, the output
+            will include the full mainline revision history, including
+            date and message
+        :param include_file_revisions: The output should
+            include the explicit last-changed revision for each file.
+        """
+        self._branch = branch
+        self._working_tree = working_tree
+        self._check = check_for_clean
+        self._include_history = include_revision_history
+        self._include_file_revs = include_file_revisions
+
+        self._clean = None
+        self._file_revisions = {}
+        self._revision_history_info= []
+
+    def _extract_file_revisions(self):
+        """Extract the working revisions for all files"""
+
+        # Things seem clean if we never look :)
+        self._clean = True
+
+        if self._working_tree is not None:
+            basis_tree = self._working_tree.basis_tree()
+        else:
+            basis_tree = self._branch.basis_tree()
+
+        # Build up the list from the basis inventory
+        for info in basis_tree.list_files(include_root=True):
+            self._file_revisions[info[0]] = info[-1].revision
+
+        if not self._check or self._working_tree is None:
+            return
+
+        delta = self._working_tree.changes_from(basis_tree, 
+                                                include_root=True)
+
+        # Using a 2-pass algorithm for renames. This is because you might have
+        # renamed something out of the way, and then created a new file
+        # in which case we would rather see the new marker
+        # Or you might have removed the target, and then renamed
+        # in which case we would rather see the renamed marker
+        for (old_path, new_path, file_id,
+             kind, text_mod, meta_mod) in delta.renamed:
+            self._clean = False
+            self._file_revisions[old_path] = u'renamed to %s' % (new_path,)
+        for path, file_id, kind in delta.removed:
+            self._clean = False
+            self._file_revisions[path] = 'removed'
+        for path, file_id, kind in delta.added:
+            self._clean = False
+            self._file_revisions[path] = 'new'
+        for (old_path, new_path, file_id,
+             kind, text_mod, meta_mod) in delta.renamed:
+            self._clean = False
+            self._file_revisions[new_path] = u'renamed from %s' % (old_path,)
+        for path, file_id, kind, text_mod, meta_mod in delta.modified:
+            self._clean = False
+            self._file_revisions[path] = 'modified'
+
+        for path in self._working_tree.unknowns():
+            self._clean = False
+            self._file_revisions[path] = 'unversioned'
+
+    def _extract_revision_history(self):
+        """Find the messages for all revisions in history."""
+
+        # Unfortunately, there is no WorkingTree.revision_history
+        rev_hist = self._branch.revision_history()
+        if self._working_tree is not None:
+            last_rev = self._working_tree.last_revision()
+            assert last_rev in rev_hist, \
+                "Working Tree's last revision not in branch.revision_history"
+            rev_hist = rev_hist[:rev_hist.index(last_rev)+1]
+
+        repository =  self._branch.repository
+        repository.lock_read()
+        try:
+            for revision_id in rev_hist:
+                rev = repository.get_revision(revision_id)
+                self._revision_history_info.append(
+                    (rev.revision_id, rev.message,
+                     rev.timestamp, rev.timezone))
+        finally:
+            repository.unlock()
+
+    def _get_revision_id(self):
+        """Get the revision id we are working on."""
+        if self._working_tree is not None:
+            return self._working_tree.last_revision()
+        return self._branch.last_revision()
+
+    def generate(self, to_file):
+        """Output the version information to the supplied file.
+
+        :param to_file: The file to write the stream to. The output
+                will already be encoded, so to_file should not try
+                to change encodings.
+        :return: None
+        """
+        raise NotImplementedError(VersionInfoBuilder.generate)
+
+
+
+def register_builder(format, module, class_name):
+    """Register a version info format.
+
+    :param format: The short name of the format, this will be used as the
+        lookup key.
+    :param module: The string name to the module where the format class
+        can be found
+    :param class_name: The string name of the class to instantiate
+    """
+    if len(_version_formats) == 0:
+        _version_formats[None] = (module, class_name)
+    _version_formats[format] = (module, class_name)
+
+
+def get_builder(format):
+    """Get a handle to the version info builder class
+
+    :param format: The lookup key supplied to register_builder
+    :return: A class, which follows the VersionInfoBuilder api.
+    """
+    builder_module, builder_class_name = _version_formats[format]
+    module = __import__(builder_module, globals(), locals(),
+                        [builder_class_name])
+    klass = getattr(module, builder_class_name)
+    return klass
+
+
+def get_builder_formats():
+    """Get the possible list of formats"""
+    formats = _version_formats.keys()
+    formats.remove(None)
+    return formats
+
+
+register_builder('rio',
+                 'bzrlib.version_info_formats.format_rio',
+                 'RioVersionInfoBuilder')
+register_builder('python',
+                 'bzrlib.version_info_formats.format_python',
+                 'PythonVersionInfoBuilder')

=== added file 'bzrlib/version_info_formats/format_python.py'
--- bzrlib/version_info_formats/format_python.py	1970-01-01 00:00:00 +0000
+++ bzrlib/version_info_formats/format_python.py	2006-09-20 14:51:03 +0000
@@ -0,0 +1,103 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A generator which creates a python script from the current tree info"""
+
+import pprint
+
+from bzrlib.version_info_formats import (
+    create_date_str,
+    VersionInfoBuilder,
+    )
+
+
+# Header and footer for the python format
+_py_version_header = '''#!/usr/bin/env python
+"""This file is automatically generated by generate_version_info
+It uses the current working tree to determine the revision.
+So don't edit it. :)
+"""
+
+'''
+
+
+_py_version_footer = '''
+
+if __name__ == '__main__':
+    print 'revision: %(revno)d' % version_info
+    print 'nick: %(branch_nick)s' % version_info
+    print 'revision id: %(revision_id)s' % version_info
+'''
+
+
+class PythonVersionInfoBuilder(VersionInfoBuilder):
+    """Create a version file which is a python source module."""
+
+    def generate(self, to_file):
+        info = {'build_date':create_date_str()
+                  , 'revno':None
+                  , 'revision_id':None
+                  , 'branch_nick':self._branch.nick
+                  , 'clean':None
+                  , 'date':None
+        }
+        revisions = []
+
+        revision_id = self._get_revision_id()
+        if revision_id is None:
+            info['revno'] = 0
+        else:
+            info['revno'] = self._branch.revision_id_to_revno(revision_id)
+            info['revision_id'] = revision_id
+            rev = self._branch.repository.get_revision(revision_id)
+            info['date'] = create_date_str(rev.timestamp, rev.timezone)
+
+        if self._check or self._include_file_revs:
+            self._extract_file_revisions()
+
+        if self._check:
+            if self._clean:
+                info['clean'] = True
+            else:
+                info['clean'] = False
+
+        info_str = pprint.pformat(info)
+        to_file.write(_py_version_header)
+        to_file.write('version_info = ')
+        to_file.write(info_str)
+        to_file.write('\n\n')
+
+        if self._include_history:
+            self._extract_revision_history()
+            revision_str = pprint.pformat(self._revision_history_info)
+            to_file.write('revisions = ')
+            to_file.write(revision_str)
+            to_file.write('\n\n')
+        else:
+            to_file.write('revisions = {}\n\n')
+
+        if self._include_file_revs:
+            file_rev_str = pprint.pformat(self._file_revisions)
+            to_file.write('file_revisions = ')
+            to_file.write(file_rev_str)
+            to_file.write('\n\n')
+        else:
+            to_file.write('file_revisions = {}\n\n')
+
+        to_file.write(_py_version_footer)
+
+
+

=== added file 'bzrlib/version_info_formats/format_rio.py'
--- bzrlib/version_info_formats/format_rio.py	1970-01-01 00:00:00 +0000
+++ bzrlib/version_info_formats/format_rio.py	2006-09-22 05:38:28 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A generator which creates a rio stanza of the current tree info"""
+
+from bzrlib.rio import RioWriter, Stanza
+
+from bzrlib.version_info_formats import (
+    create_date_str,
+    VersionInfoBuilder,
+    )
+
+
+class RioVersionInfoBuilder(VersionInfoBuilder):
+    """This writes a rio stream out."""
+
+    def generate(self, to_file):
+        info = Stanza()
+        revision_id = self._get_revision_id()
+        if revision_id is not None:
+            info.add('revision-id', revision_id)
+            rev = self._branch.repository.get_revision(revision_id)
+            info.add('date', create_date_str(rev.timestamp, rev.timezone))
+            revno = str(self._branch.revision_id_to_revno(revision_id))
+        else:
+            revno = '0'
+
+        info.add('build-date', create_date_str())
+        info.add('revno', revno)
+
+        if self._branch.nick is not None:
+            info.add('branch-nick', self._branch.nick)
+
+        if self._check or self._include_file_revs:
+            self._extract_file_revisions()
+
+        if self._check:
+            if self._clean:
+                info.add('clean', 'True')
+            else:
+                info.add('clean', 'False')
+
+        if self._include_history:
+            self._extract_revision_history()
+            log = Stanza()
+            for (revision_id, message,
+                 timestamp, timezone) in self._revision_history_info:
+                log.add('id', revision_id)
+                log.add('message', message)
+                log.add('date', create_date_str(timestamp, timezone))
+            info.add('revisions', log.to_unicode())
+
+        if self._include_file_revs:
+            files = Stanza()
+            for path in sorted(self._file_revisions.keys()):
+                files.add('path', path)
+                files.add('revision', self._file_revisions[path])
+            info.add('file-revisions', files.to_unicode())
+
+        writer = RioWriter(to_file=to_file)
+        writer.write_stanza(info)
+
+

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-07-28 16:05:23 +0000
+++ bzrlib/versionedfile.py	2006-10-06 05:53:44 +0000
@@ -19,17 +19,21 @@
 
 """Versioned text file storage api."""
 
-
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from copy import deepcopy
-from unittest import TestSuite
-
-
-import bzrlib.errors as errors
+import unittest
+
+from bzrlib import (
+    errors,
+    tsort,
+    ui,
+    )
+from bzrlib.transport.memory import MemoryTransport
+""")
+
 from bzrlib.inter import InterObject
 from bzrlib.textmerge import TextMerge
-from bzrlib.transport.memory import MemoryTransport
-from bzrlib.tsort import topo_sort
-from bzrlib import ui
 from bzrlib.symbol_versioning import (deprecated_function,
         deprecated_method,
         zero_eight,
@@ -403,7 +407,8 @@
             version_ids,
             ignore_missing)
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+    def iter_lines_added_or_present_in_versions(self, version_ids=None, 
+                                                pb=None):
         """Iterate over the lines in the versioned file from version_ids.
 
         This may return lines from other versions, and does not return the
@@ -412,6 +417,10 @@
         thinks is relevant, but given that such hints are just guesses,
         its better not to have it if we don't need it.
 
+        If a progress bar is supplied, it may be used to indicate progress.
+        The caller is responsible for cleaning up progress bars (because this
+        is an iterator).
+
         NOTES: Lines are normalised: they will all have \n terminators.
                Lines are returned in arbitrary order.
         """
@@ -468,7 +477,7 @@
         """
         raise NotImplementedError(VersionedFile.plan_merge)
         
-    def weave_merge(self, plan, a_marker=TextMerge.A_MARKER, 
+    def weave_merge(self, plan, a_marker=TextMerge.A_MARKER,
                     b_marker=TextMerge.B_MARKER):
         return PlanWeaveMerge(plan, a_marker, b_marker).merge_lines()[0]
 
@@ -558,7 +567,7 @@
     InterVersionedFile.get(other).method_name(parameters).
     """
 
-    _optimisers = set()
+    _optimisers = []
     """The available optimised InterVersionedFile types."""
 
     def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
@@ -585,7 +594,7 @@
             target = temp_source
         version_ids = self._get_source_version_ids(version_ids, ignore_missing)
         graph = self.source.get_graph(version_ids)
-        order = topo_sort(graph.items())
+        order = tsort.topo_sort(graph.items())
         pb = ui.ui_factory.nested_progress_bar()
         parent_texts = {}
         try:
@@ -672,7 +681,7 @@
         self._formats = formats
     
     def adapt(self, test):
-        result = TestSuite()
+        result = unittest.TestSuite()
         for (interversionedfile_class,
              versionedfile_factory,
              versionedfile_factory_to) in self._formats:

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-09-05 20:08:52 +0000
+++ bzrlib/weave.py	2006-09-25 14:52:38 +0000
@@ -75,6 +75,9 @@
 import time
 import warnings
 
+from bzrlib import (
+    progress,
+    )
 from bzrlib.trace import mutter
 from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
         RevisionAlreadyPresent,
@@ -664,7 +667,8 @@
         """_walk has become visit, a supported api."""
         return self._walk_internal()
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+    def iter_lines_added_or_present_in_versions(self, version_ids=None,
+                                                pb=None):
         """See VersionedFile.iter_lines_added_or_present_in_versions()."""
         if version_ids is None:
             version_ids = self.versions()

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2006-07-28 16:05:23 +0000
+++ bzrlib/weavefile.py	2006-09-20 22:32:33 +0000
@@ -45,7 +45,7 @@
 
 
 def write_weave(weave, f, format=None):
-    if format == None or format == 1:
+    if format is None or format == 1:
         return write_weave_v5(weave, f)
     else:
         raise ValueError("unknown weave format %r" % format)
@@ -124,7 +124,11 @@
 
     lines = iter(f.readlines())
     
-    l = lines.next()
+    try:
+        l = lines.next()
+    except StopIteration:
+        raise WeaveFormatError('invalid weave file: no header')
+
     if l != FORMAT_1:
         raise WeaveFormatError('invalid weave file header: %r' % l)
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-09-04 08:00:00 +0000
+++ bzrlib/workingtree.py	2006-10-16 01:13:59 +0000
@@ -29,9 +29,6 @@
 WorkingTree.open(dir).
 """
 
-MERGE_MODIFIED_HEADER_1 = "BZR merge-modified list format 1"
-CONFLICT_HEADER_1 = "BZR conflict list format 1"
-
 # TODO: Give the workingtree sole responsibility for the working inventory;
 # remove the variable and references to it from the branch.  This may require
 # updating the commit code so as to update the inventory within the working
@@ -39,23 +36,40 @@
 # At the moment they may alias the inventory and have old copies of it in
 # memory.  (Now done? -- mbp 20060309)
 
-from binascii import hexlify
+from cStringIO import StringIO
+import os
+import re
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import collections
 from copy import deepcopy
-from cStringIO import StringIO
 import errno
 import fnmatch
-import os
-import re
 import stat
 from time import time
 import warnings
 
 import bzrlib
-from bzrlib import bzrdir, errors, ignores, osutils, urlutils
-from bzrlib.atomicfile import AtomicFile
+from bzrlib import (
+    bzrdir,
+    conflicts as _mod_conflicts,
+    errors,
+    ignores,
+    merge,
+    osutils,
+    textui,
+    transform,
+    urlutils,
+    xml5,
+    xml6,
+    )
 import bzrlib.branch
-from bzrlib.conflicts import Conflict, ConflictList, CONFLICT_SUFFIXES
+from bzrlib.transport import get_transport
+import bzrlib.ui
+""")
+
+from bzrlib import symbol_versioning
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.errors import (BzrCheckError,
                            BzrError,
@@ -67,45 +81,41 @@
                            MergeModifiedFormatError,
                            UnsupportedOperation,
                            )
-from bzrlib.inventory import InventoryEntry, Inventory
+from bzrlib.inventory import InventoryEntry, Inventory, ROOT_ID
 from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.lockdir import LockDir
-from bzrlib.merge import merge_inner, transform_tree
+import bzrlib.mutabletree
+from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.osutils import (
-                            abspath,
-                            compact_date,
-                            file_kind,
-                            isdir,
-                            getcwd,
-                            pathjoin,
-                            pumpfile,
-                            safe_unicode,
-                            splitpath,
-                            rand_chars,
-                            normpath,
-                            realpath,
-                            relpath,
-                            rename,
-                            supports_executable,
-                            )
+    compact_date,
+    file_kind,
+    isdir,
+    pathjoin,
+    safe_unicode,
+    splitpath,
+    rand_chars,
+    normpath,
+    realpath,
+    supports_executable,
+    )
+from bzrlib.trace import mutter, note
+from bzrlib.transport.local import LocalTransport
+import bzrlib.tree
 from bzrlib.progress import DummyProgress, ProgressPhase
 from bzrlib.revision import NULL_REVISION
+import bzrlib.revisiontree
 from bzrlib.rio import RioReader, rio_file, Stanza
 from bzrlib.symbol_versioning import (deprecated_passed,
         deprecated_method,
         deprecated_function,
         DEPRECATED_PARAMETER,
         zero_eight,
+        zero_eleven,
         )
-from bzrlib.trace import mutter, note
-from bzrlib.transform import build_tree
-from bzrlib.transport import get_transport
-from bzrlib.transport.local import LocalTransport
-from bzrlib.textui import show_status
-import bzrlib.tree
-import bzrlib.ui
-import bzrlib.xml5
-
+
+
+MERGE_MODIFIED_HEADER_1 = "BZR merge-modified list format 1"
+CONFLICT_HEADER_1 = "BZR conflict list format 1"
 
 # the regex removes any weird characters; we don't escape them 
 # but rather just pull them out
@@ -155,7 +165,7 @@
 
 def gen_root_id():
     """Return a new tree-root file id."""
-    return gen_file_id('TREE_ROOT')
+    return gen_file_id('tree_root')
 
 
 class TreeEntry(object):
@@ -213,7 +223,7 @@
         return ''
 
 
-class WorkingTree(bzrlib.tree.Tree):
+class WorkingTree(bzrlib.mutabletree.MutableTree):
     """Working copy tree.
 
     The inventory is held in the `Branch` working-inventory, and the
@@ -250,7 +260,7 @@
             self.basedir = wt.basedir
             self._control_files = wt._control_files
             self._hashcache = wt._hashcache
-            self._set_inventory(wt._inventory)
+            self._set_inventory(wt._inventory, dirty=False)
             self._format = wt._format
             self.bzrdir = wt.bzrdir
         from bzrlib.hashcache import HashCache
@@ -298,9 +308,14 @@
             hc.write()
 
         if _inventory is None:
-            self._set_inventory(self.read_working_inventory())
+            self._inventory_is_modified = False
+            self.read_working_inventory()
         else:
-            self._set_inventory(_inventory)
+            # the caller of __init__ has provided an inventory,
+            # we assume they know what they are doing - as its only
+            # the Format factory and creation methods that are
+            # permitted to do this.
+            self._set_inventory(_inventory, dirty=False)
 
     branch = property(
         fget=lambda self: self._branch,
@@ -321,22 +336,19 @@
         self._control_files.break_lock()
         self.branch.break_lock()
 
-    def _set_inventory(self, inv):
+    def _set_inventory(self, inv, dirty):
+        """Set the internal cached inventory.
+
+        :param inv: The inventory to set.
+        :param dirty: A boolean indicating whether the inventory is the same
+            logical inventory as whats on disk. If True the inventory is not
+            the same and should be written to disk or data will be lost, if
+            False then the inventory is the same as that on disk and any
+            serialisation would be unneeded overhead.
+        """
         assert inv.root is not None
         self._inventory = inv
-        self.path2id = self._inventory.path2id
-
-    def is_control_filename(self, filename):
-        """True if filename is the name of a control file in this tree.
-        
-        :param filename: A filename within the tree. This is a relative path
-        from the root of this tree.
-
-        This is true IF and ONLY IF the filename is part of the meta data
-        that bzr controls in this tree. I.E. a random .bzr directory placed
-        on disk will not be a control file for this tree.
-        """
-        return self.bzrdir.is_control_filename(filename)
+        self._inventory_is_modified = dirty
 
     @staticmethod
     def open(path=None, _unsupported=False):
@@ -399,18 +411,25 @@
         If the left most parent is a ghost then the returned tree will be an
         empty tree - one obtained by calling repository.revision_tree(None).
         """
-        revision_id = self.last_revision()
-        if revision_id is not None:
+        try:
+            revision_id = self.get_parent_ids()[0]
+        except IndexError:
+            # no parents, return an empty revision tree.
+            # in the future this should return the tree for
+            # 'empty:' - the implicit root empty tree.
+            return self.branch.repository.revision_tree(None)
+        else:
             try:
                 xml = self.read_basis_inventory()
-                inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
-                inv.root.revision = revision_id
-            except NoSuchFile:
-                inv = None
-            if inv is not None and inv.revision_id == revision_id:
-                return bzrlib.tree.RevisionTree(self.branch.repository, inv,
-                                                revision_id)
-        # FIXME? RBC 20060403 should we cache the inventory here ?
+                inv = xml6.serializer_v6.read_inventory_from_string(xml)
+                if inv is not None and inv.revision_id == revision_id:
+                    return bzrlib.tree.RevisionTree(self.branch.repository,
+                                                    inv, revision_id)
+            except (NoSuchFile, errors.BadInventoryFormat):
+                pass
+        # No cached copy available, retrieve from the repository.
+        # FIXME? RBC 20060403 should we cache the inventory locally
+        # at this point ?
         try:
             return self.branch.repository.revision_tree(revision_id)
         except errors.RevisionNotPresent:
@@ -466,7 +485,7 @@
         The path may be absolute or relative. If its a relative path it is 
         interpreted relative to the python current working directory.
         """
-        return relpath(self.basedir, path)
+        return osutils.relpath(self.basedir, path)
 
     def has_filename(self, filename):
         return osutils.lexists(self.abspath(filename))
@@ -486,7 +505,7 @@
         This implementation reads the pending merges list and last_revision
         value and uses that to decide what the parents list should be.
         """
-        last_rev = self.last_revision()
+        last_rev = self._last_revision()
         if last_rev is None:
             parents = []
         else:
@@ -500,10 +519,10 @@
                 parents.append(l.rstrip('\n'))
         return parents
 
+    @needs_read_lock
     def get_root_id(self):
         """Return the id of this trees root"""
-        inv = self.read_working_inventory()
-        return inv.root.file_id
+        return self._inventory.root.file_id
         
     def _get_store_filename(self, file_id):
         ## XXX: badly named; this is not in the store at all
@@ -535,31 +554,15 @@
     @needs_read_lock
     def copy_content_into(self, tree, revision_id=None):
         """Copy the current content and user files of this tree into tree."""
+        tree.set_root_id(self.get_root_id())
         if revision_id is None:
-            transform_tree(tree, self)
+            merge.transform_tree(tree, self)
         else:
             # TODO now merge from tree.last_revision to revision (to preserve
             # user local changes)
-            transform_tree(tree, self)
+            merge.transform_tree(tree, self)
             tree.set_parent_ids([revision_id])
 
-    @needs_write_lock
-    def commit(self, message=None, revprops=None, *args, **kwargs):
-        # avoid circular imports
-        from bzrlib.commit import Commit
-        if revprops is None:
-            revprops = {}
-        if not 'branch-nick' in revprops:
-            revprops['branch-nick'] = self.branch.nick
-        # args for wt.commit start at message from the Commit.commit method,
-        # but with branch a kwarg now, passing in args as is results in the
-        #message being used for the branch
-        args = (DEPRECATED_PARAMETER, message, ) + args
-        committed_id = Commit().commit( working_tree=self, revprops=revprops,
-            *args, **kwargs)
-        self._set_inventory(self.read_working_inventory())
-        return committed_id
-
     def id2abspath(self, file_id):
         return self.abspath(self.id2path(file_id))
 
@@ -603,66 +606,33 @@
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
     @needs_write_lock
-    def add(self, files, ids=None):
-        """Make files versioned.
-
-        Note that the command line normally calls smart_add instead,
-        which can automatically recurse.
-
-        This adds the files to the inventory, so that they will be
-        recorded by the next commit.
-
-        files
-            List of paths to add, relative to the base of the tree.
-
-        ids
-            If set, use these instead of automatically generated ids.
-            Must be the same length as the list of files, but may
-            contain None for ids that are to be autogenerated.
-
-        TODO: Perhaps have an option to add the ids even if the files do
-              not (yet) exist.
-
-        TODO: Perhaps callback with the ids and paths as they're added.
-        """
+    def _add(self, files, ids, kinds):
+        """See MutableTree._add."""
         # TODO: Re-adding a file that is removed in the working copy
         # should probably put it back with the previous ID.
-        if isinstance(files, basestring):
-            assert(ids is None or isinstance(ids, basestring))
-            files = [files]
-            if ids is not None:
-                ids = [ids]
-
-        if ids is None:
-            ids = [None] * len(files)
-        else:
-            assert(len(ids) == len(files))
-
+        # the read and write working inventory should not occur in this 
+        # function - they should be part of lock_write and unlock.
         inv = self.read_working_inventory()
-        for f,file_id in zip(files, ids):
-            if self.is_control_filename(f):
-                raise errors.ForbiddenControlFileError(filename=f)
-
-            fp = splitpath(f)
-
-            if len(fp) == 0:
-                raise BzrError("cannot add top-level %r" % f)
-
-            fullpath = normpath(self.abspath(f))
-            try:
-                kind = file_kind(fullpath)
-            except OSError, e:
-                if e.errno == errno.ENOENT:
-                    raise NoSuchFile(fullpath)
-            if not InventoryEntry.versionable_kind(kind):
-                raise errors.BadFileKindError(filename=f, kind=kind)
+        for f, file_id, kind in zip(files, ids, kinds):
+            assert kind is not None
             if file_id is None:
                 inv.add_path(f, kind=kind)
             else:
                 inv.add_path(f, kind=kind, file_id=file_id)
-
         self._write_inventory(inv)
 
+    @needs_tree_write_lock
+    def _gather_kinds(self, files, kinds):
+        """See MutableTree._gather_kinds."""
+        for pos, f in enumerate(files):
+            if kinds[pos] is None:
+                fullpath = normpath(self.abspath(f))
+                try:
+                    kinds[pos] = file_kind(fullpath)
+                except OSError, e:
+                    if e.errno == errno.ENOENT:
+                        raise NoSuchFile(fullpath)
+
     @needs_write_lock
     def add_parent_tree_id(self, revision_id, allow_leftmost_as_ghost=False):
         """Add revision_id as a parent.
@@ -679,7 +649,7 @@
         self.set_parent_ids(parents,
             allow_leftmost_as_ghost=len(parents) > 1 or allow_leftmost_as_ghost)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add_parent_tree(self, parent_tuple, allow_leftmost_as_ghost=False):
         """Add revision_id, tree tuple as a parent.
 
@@ -701,7 +671,7 @@
         self.set_parent_ids(parent_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add_pending_merge(self, *revision_ids):
         # TODO: Perhaps should check at this point that the
         # history of the revision is actually present?
@@ -715,16 +685,36 @@
         if updated:
             self.set_parent_ids(parents, allow_leftmost_as_ghost=True)
 
+    @deprecated_method(zero_eleven)
     @needs_read_lock
     def pending_merges(self):
         """Return a list of pending merges.
 
         These are revisions that have been merged into the working
         directory but not yet committed.
+
+        As of 0.11 this is deprecated. Please see WorkingTree.get_parent_ids()
+        instead - which is available on all tree objects.
         """
         return self.get_parent_ids()[1:]
 
-    @needs_write_lock
+    def _check_parents_for_ghosts(self, revision_ids, allow_leftmost_as_ghost):
+        """Common ghost checking functionality from set_parent_*.
+
+        This checks that the left hand-parent exists if there are any
+        revisions present.
+        """
+        if len(revision_ids) > 0:
+            leftmost_id = revision_ids[0]
+            if (not allow_leftmost_as_ghost and not
+                self.branch.repository.has_revision(leftmost_id)):
+                raise errors.GhostRevisionUnusableHere(leftmost_id)
+
+    def _set_merges_from_parent_ids(self, parent_ids):
+        merges = parent_ids[1:]
+        self._control_files.put_utf8('pending-merges', '\n'.join(merges))
+
+    @needs_tree_write_lock
     def set_parent_ids(self, revision_ids, allow_leftmost_as_ghost=False):
         """Set the parent ids to revision_ids.
         
@@ -737,50 +727,62 @@
         :param revision_ids: The revision_ids to set as the parent ids of this
             working tree. Any of these may be ghosts.
         """
+        self._check_parents_for_ghosts(revision_ids,
+            allow_leftmost_as_ghost=allow_leftmost_as_ghost)
+
         if len(revision_ids) > 0:
-            leftmost_id = revision_ids[0]
-            if (not allow_leftmost_as_ghost and not
-                self.branch.repository.has_revision(leftmost_id)):
-                raise errors.GhostRevisionUnusableHere(leftmost_id)
-            self.set_last_revision(leftmost_id)
+            self.set_last_revision(revision_ids[0])
         else:
             self.set_last_revision(None)
-        merges = revision_ids[1:]
-        self._control_files.put_utf8('pending-merges', '\n'.join(merges))
-
-    @needs_write_lock
+
+        self._set_merges_from_parent_ids(revision_ids)
+
+    @needs_tree_write_lock
     def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
-        """Set the parents of the working tree.
+        """See MutableTree.set_parent_trees."""
+        parent_ids = [rev for (rev, tree) in parents_list]
 
-        :param parents_list: A list of (revision_id, tree) tuples. 
-            If tree is None, then that element is treated as an unreachable
-            parent tree - i.e. a ghost.
-        """
-        # parent trees are not used in current format trees, delegate to
-        # set_parent_ids
-        self.set_parent_ids([rev for (rev, tree) in parents_list],
+        self._check_parents_for_ghosts(parent_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
 
-    @needs_write_lock
+        if len(parent_ids) == 0:
+            leftmost_parent_id = None
+            leftmost_parent_tree = None
+        else:
+            leftmost_parent_id, leftmost_parent_tree = parents_list[0]
+
+        if self._change_last_revision(leftmost_parent_id):
+            if leftmost_parent_tree is None:
+                # If we don't have a tree, fall back to reading the
+                # parent tree from the repository.
+                self._cache_basis_inventory(leftmost_parent_id)
+            else:
+                inv = leftmost_parent_tree.inventory
+                xml = self._create_basis_xml_from_inventory(
+                                        leftmost_parent_id, inv)
+                self._write_basis_inventory(xml)
+        self._set_merges_from_parent_ids(parent_ids)
+
+    @needs_tree_write_lock
     def set_pending_merges(self, rev_list):
         parents = self.get_parent_ids()
         leftmost = parents[:1]
         new_parents = leftmost + rev_list
         self.set_parent_ids(new_parents)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_merge_modified(self, modified_hashes):
         def iter_stanzas():
             for file_id, hash in modified_hashes.iteritems():
                 yield Stanza(file_id=file_id, hash=hash)
         self._put_rio('merge-hashes', iter_stanzas(), MERGE_MODIFIED_HEADER_1)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def _put_rio(self, filename, stanzas, header):
         my_file = rio_file(stanzas, header)
         self._control_files.put(filename, my_file)
 
-    @needs_write_lock
+    @needs_write_lock # because merge pulls data into the branch.
     def merge_from_branch(self, branch, to_revision=None):
         """Merge from a branch into this working tree.
 
@@ -844,6 +846,15 @@
                 merge_hashes[file_id] = hash
         return merge_hashes
 
+    @needs_write_lock
+    def mkdir(self, path, file_id=None):
+        """See MutableTree.mkdir()."""
+        if file_id is None:
+            file_id = gen_file_id(os.path.basename(path))
+        os.mkdir(self.abspath(path))
+        self.add(path, file_id, 'directory')
+        return file_id
+
     def get_symlink_target(self, file_id):
         return os.readlink(self.id2abspath(file_id))
 
@@ -855,7 +866,18 @@
         else:
             return '?'
 
-    def list_files(self):
+    def flush(self):
+        """Write the in memory inventory to disk."""
+        # TODO: Maybe this should only write on dirty ?
+        if self._control_files._lock_mode != 'w':
+            raise errors.NotWriteLocked(self)
+        sio = StringIO()
+        xml5.serializer_v5.write_inventory(self._inventory, sio)
+        sio.seek(0)
+        self._control_files.put('inventory', sio)
+        self._inventory_is_modified = False
+
+    def list_files(self, include_root=False):
         """Recursively list all files as (path, class, kind, id, entry).
 
         Lists, but does not descend into unversioned directories.
@@ -866,6 +888,8 @@
         Skips the control directory.
         """
         inv = self._inventory
+        if include_root is True:
+            yield ('', 'V', 'directory', inv.root.file_id, inv.root)
         # Convert these into local objects to save lookup times
         pathjoin = osutils.pathjoin
         file_kind = osutils.file_kind
@@ -964,8 +988,7 @@
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
-
-    @needs_write_lock
+    @needs_tree_write_lock
     def move(self, from_paths, to_name):
         """Rename files.
 
@@ -990,7 +1013,7 @@
         if not self.has_filename(to_name):
             raise BzrError("destination %r not in working directory" % to_abs)
         to_dir_id = inv.path2id(to_name)
-        if to_dir_id == None and to_name != '':
+        if to_dir_id is None and to_name != '':
             raise BzrError("destination %r is not a versioned directory" % to_name)
         to_dir_ie = inv[to_dir_id]
         if to_dir_ie.kind != 'directory':
@@ -1002,7 +1025,7 @@
             if not self.has_filename(f):
                 raise BzrError("%r does not exist in working tree" % f)
             f_id = inv.path2id(f)
-            if f_id == None:
+            if f_id is None:
                 raise BzrError("%r is not versioned" % f)
             name_tail = splitpath(f)[-1]
             dest_path = pathjoin(to_name, name_tail)
@@ -1015,26 +1038,29 @@
         # create a file in this interval and then the rename might be
         # left half-done.  But we should have caught most problems.
         orig_inv = deepcopy(self.inventory)
+        original_modified = self._inventory_is_modified
         try:
+            if len(from_paths):
+                self._inventory_is_modified = True
             for f in from_paths:
                 name_tail = splitpath(f)[-1]
                 dest_path = pathjoin(to_name, name_tail)
                 result.append((f, dest_path))
                 inv.rename(inv.path2id(f), to_dir_id, name_tail)
                 try:
-                    rename(self.abspath(f), self.abspath(dest_path))
+                    osutils.rename(self.abspath(f), self.abspath(dest_path))
                 except OSError, e:
                     raise BzrError("failed to rename %r to %r: %s" %
                                    (f, dest_path, e[1]),
                             ["rename rolled back"])
         except:
             # restore the inventory on error
-            self._set_inventory(orig_inv)
+            self._set_inventory(orig_inv, dirty=original_modified)
             raise
         self._write_inventory(inv)
         return result
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def rename_one(self, from_rel, to_rel):
         """Rename one file.
 
@@ -1047,7 +1073,7 @@
             raise BzrError("can't rename: new working file %r already exists" % to_rel)
 
         file_id = inv.path2id(from_rel)
-        if file_id == None:
+        if file_id is None:
             raise BzrError("can't rename: old name %r is not versioned" % from_rel)
 
         entry = inv[file_id]
@@ -1059,7 +1085,7 @@
 
         to_dir, to_tail = os.path.split(to_rel)
         to_dir_id = inv.path2id(to_dir)
-        if to_dir_id == None and to_dir != '':
+        if to_dir_id is None and to_dir != '':
             raise BzrError("can't determine destination directory id for %r" % to_dir)
 
         mutter("rename_one:")
@@ -1074,7 +1100,7 @@
         from_abs = self.abspath(from_rel)
         to_abs = self.abspath(to_rel)
         try:
-            rename(from_abs, to_abs)
+            osutils.rename(from_abs, to_abs)
         except OSError, e:
             inv.rename(file_id, from_parent, from_name)
             raise BzrError("failed to rename %r to %r: %s"
@@ -1092,7 +1118,33 @@
         for subp in self.extras():
             if not self.is_ignored(subp):
                 yield subp
-
+    
+    @needs_tree_write_lock
+    def unversion(self, file_ids):
+        """Remove the file ids in file_ids from the current versioned set.
+
+        When a file_id is unversioned, all of its children are automatically
+        unversioned.
+
+        :param file_ids: The file ids to stop versioning.
+        :raises: NoSuchId if any fileid is not currently versioned.
+        """
+        for file_id in file_ids:
+            if self._inventory.has_id(file_id):
+                self._inventory.remove_recursive_id(file_id)
+            else:
+                raise errors.NoSuchId(self, file_id)
+        if len(file_ids):
+            # in the future this should just set a dirty bit to wait for the 
+            # final unlock. However, until all methods of workingtree start
+            # with the current in -memory inventory rather than triggering 
+            # a read, it is more complex - we need to teach read_inventory
+            # to know when to read, and when to not read first... and possibly
+            # to save first when the in memory one may be corrupted.
+            # so for now, we just only write it if it is indeed dirty.
+            # - RBC 20060907
+            self._write_inventory(self._inventory)
+    
     @deprecated_method(zero_eight)
     def iter_conflicts(self):
         """List all files in the tree that have text or content conflicts.
@@ -1131,11 +1183,15 @@
                 pb = bzrlib.ui.ui_factory.nested_progress_bar()
                 try:
                     new_basis_tree = self.branch.basis_tree()
-                    merge_inner(self.branch,
+                    merge.merge_inner(
+                                self.branch,
                                 new_basis_tree,
                                 basis_tree,
                                 this_tree=self,
                                 pb=pb)
+                    if (basis_tree.inventory.root is None and
+                        new_basis_tree.inventory.root is not None):
+                        self.set_root_id(new_basis_tree.inventory.root.file_id)
                 finally:
                     pb.finished()
                 # TODO - dedup parents list with things merged by pull ?
@@ -1155,6 +1211,16 @@
             source.unlock()
             top_pb.finished()
 
+    @needs_write_lock
+    def put_file_bytes_non_atomic(self, file_id, bytes):
+        """See MutableTree.put_file_bytes_non_atomic."""
+        stream = file(self.id2abspath(file_id), 'wb')
+        try:
+            stream.write(bytes)
+        finally:
+            stream.close()
+        # TODO: update the hashcache here ?
+
     def extras(self):
         """Yield all unknown files in this WorkingTree.
 
@@ -1248,7 +1314,7 @@
         """Yield list of PATH, IGNORE_PATTERN"""
         for subp in self.extras():
             pat = self.is_ignored(subp)
-            if pat != None:
+            if pat is not None:
                 yield subp, pat
 
     def get_ignore_list(self):
@@ -1320,14 +1386,19 @@
     def kind(self, file_id):
         return file_kind(self.id2abspath(file_id))
 
-    @needs_read_lock
     def last_revision(self):
-        """Return the last revision id of this working tree.
-
-        In early branch formats this was == the branch last_revision,
-        but that cannot be relied upon - for working tree operations,
-        always use tree.last_revision().
+        """Return the last revision of the branch for this tree.
+
+        This format tree does not support a separate marker for last-revision
+        compared to the branch.
+
+        See MutableTree.last_revision
         """
+        return self._last_revision()
+
+    @needs_read_lock
+    def _last_revision(self):
+        """helper for get_parent_ids."""
         return self.branch.last_revision()
 
     def is_locked(self):
@@ -1342,8 +1413,17 @@
             self.branch.unlock()
             raise
 
+    def lock_tree_write(self):
+        """See MutableTree.lock_tree_write, and WorkingTree.unlock."""
+        self.branch.lock_read()
+        try:
+            return self._control_files.lock_write()
+        except:
+            self.branch.unlock()
+            raise
+
     def lock_write(self):
-        """See Branch.lock_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_write, and WorkingTree.unlock."""
         self.branch.lock_write()
         try:
             return self._control_files.lock_write()
@@ -1355,9 +1435,9 @@
         return self._control_files.get_physical_lock_status()
 
     def _basis_inventory_name(self):
-        return 'basis-inventory'
+        return 'basis-inventory-cache'
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_last_revision(self, new_revision):
         """Change the last revision in the working tree."""
         if self._change_last_revision(new_revision):
@@ -1379,6 +1459,18 @@
             self.branch.set_revision_history([new_revision])
         return True
 
+    def _write_basis_inventory(self, xml):
+        """Write the basis inventory XML to the basis-inventory file"""
+        assert isinstance(xml, str), 'serialised xml must be bytestring.'
+        path = self._basis_inventory_name()
+        sio = StringIO(xml)
+        self._control_files.put(path, sio)
+
+    def _create_basis_xml_from_inventory(self, revision_id, inventory):
+        """Create the text that will be saved in basis-inventory"""
+        inventory.revision_id = revision_id
+        return xml6.serializer_v6.write_inventory_to_string(inventory)
+
     def _cache_basis_inventory(self, new_revision):
         """Cache new_revision as the basis inventory."""
         # TODO: this should allow the ready-to-use inventory to be passed in,
@@ -1396,15 +1488,13 @@
             # root node id can legitimately look like 'revision_id' but cannot
             # contain a '"'.
             xml = self.branch.repository.get_inventory_xml(new_revision)
-            if not 'revision_id="' in xml.split('\n', 1)[0]:
+            firstline = xml.split('\n', 1)[0]
+            if (not 'revision_id="' in firstline or 
+                'format="6"' not in firstline):
                 inv = self.branch.repository.deserialise_inventory(
                     new_revision, xml)
-                inv.revision_id = new_revision
-                xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
-            assert isinstance(xml, str), 'serialised xml must be bytestring.'
-            path = self._basis_inventory_name()
-            sio = StringIO(xml)
-            self._control_files.put(path, sio)
+                xml = self._create_basis_xml_from_inventory(new_revision, inv)
+            self._write_basis_inventory(xml)
         except (errors.NoSuchRevision, errors.RevisionNotPresent):
             pass
 
@@ -1415,15 +1505,23 @@
         
     @needs_read_lock
     def read_working_inventory(self):
-        """Read the working inventory."""
+        """Read the working inventory.
+        
+        :raises errors.InventoryModified: read_working_inventory will fail
+            when the current in memory inventory has been modified.
+        """
+        # conceptually this should be an implementation detail of the tree. 
+        # XXX: Deprecate this.
         # ElementTree does its own conversion from UTF-8, so open in
         # binary.
-        result = bzrlib.xml5.serializer_v5.read_inventory(
+        if self._inventory_is_modified:
+            raise errors.InventoryModified(self)
+        result = xml5.serializer_v5.read_inventory(
             self._control_files.get('inventory'))
-        self._set_inventory(result)
+        self._set_inventory(result, dirty=False)
         return result
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def remove(self, files, verbose=False, to_file=None):
         """Remove nominated files from the working inventory..
 
@@ -1458,19 +1556,19 @@
                     new_status = 'I'
                 else:
                     new_status = '?'
-                show_status(new_status, inv[fid].kind, f, to_file=to_file)
+                textui.show_status(new_status, inv[fid].kind, f,
+                                   to_file=to_file)
             del inv[fid]
 
         self._write_inventory(inv)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def revert(self, filenames, old_tree=None, backups=True, 
                pb=DummyProgress()):
-        from transform import revert
-        from conflicts import resolve
+        from bzrlib.conflicts import resolve
         if old_tree is None:
             old_tree = self.basis_tree()
-        conflicts = revert(self, old_tree, filenames, backups, pb)
+        conflicts = transform.revert(self, old_tree, filenames, backups, pb)
         if not len(filenames):
             self.set_parent_ids(self.get_parent_ids()[:1])
             resolve(self)
@@ -1480,7 +1578,7 @@
 
     # XXX: This method should be deprecated in favour of taking in a proper
     # new Inventory object.
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_inventory(self, new_inventory_list):
         from bzrlib.inventory import (Inventory,
                                       InventoryDirectory,
@@ -1503,19 +1601,34 @@
                 raise BzrError("unknown kind %r" % kind)
         self._write_inventory(inv)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_root_id(self, file_id):
         """Set the root id for this tree."""
-        inv = self.read_working_inventory()
+        # for compatability 
+        if file_id is None:
+            symbol_versioning.warn(symbol_versioning.zero_twelve
+                % 'WorkingTree.set_root_id with fileid=None',
+                DeprecationWarning,
+                stacklevel=3)
+            file_id = ROOT_ID
+        inv = self._inventory
         orig_root_id = inv.root.file_id
+        # TODO: it might be nice to exit early if there was nothing
+        # to do, saving us from trigger a sync on unlock.
+        self._inventory_is_modified = True
+        # we preserve the root inventory entry object, but
+        # unlinkit from the byid index
         del inv._byid[inv.root.file_id]
         inv.root.file_id = file_id
+        # and link it into the index with the new changed id.
         inv._byid[inv.root.file_id] = inv.root
+        # and finally update all children to reference the new id.
+        # XXX: this should be safe to just look at the root.children
+        # list, not the WHOLE INVENTORY.
         for fid in inv:
             entry = inv[fid]
             if entry.parent_id == orig_root_id:
                 entry.parent_id = inv.root.file_id
-        self._write_inventory(inv)
 
     def unlock(self):
         """See Branch.unlock.
@@ -1532,23 +1645,30 @@
     def update(self):
         """Update a working tree along its branch.
 
-        This will update the branch if its bound too, which means we have multiple trees involved:
-        The new basis tree of the master.
-        The old basis tree of the branch.
-        The old basis tree of the working tree.
-        The current working tree state.
-        pathologically all three may be different, and non ancestors of each other.
-        Conceptually we want to:
-        Preserve the wt.basis->wt.state changes
-        Transform the wt.basis to the new master basis.
-        Apply a merge of the old branch basis to get any 'local' changes from it into the tree.
-        Restore the wt.basis->wt.state changes.
+        This will update the branch if its bound too, which means we have
+        multiple trees involved:
+
+        - The new basis tree of the master.
+        - The old basis tree of the branch.
+        - The old basis tree of the working tree.
+        - The current working tree state.
+
+        Pathologically, all three may be different, and non-ancestors of each
+        other.  Conceptually we want to:
+
+        - Preserve the wt.basis->wt.state changes
+        - Transform the wt.basis to the new master basis.
+        - Apply a merge of the old branch basis to get any 'local' changes from
+          it into the tree.
+        - Restore the wt.basis->wt.state changes.
 
         There isn't a single operation at the moment to do that, so we:
-        Merge current state -> basis tree of the master w.r.t. the old tree basis.
-        Do a 'normal' merge of the old branch basis if it is relevant.
+        - Merge current state -> basis tree of the master w.r.t. the old tree
+          basis.
+        - Do a 'normal' merge of the old branch basis if it is relevant.
         """
         old_tip = self.branch.update()
+
         # here if old_tip is not None, it is the old tip of the branch before
         # it was updated from the master branch. This should become a pending
         # merge in the working tree to preserve the user existing work.  we
@@ -1560,11 +1680,18 @@
         # local work is unreferenced and will appear to have been lost.
         # 
         result = 0
-        if self.last_revision() != self.branch.last_revision():
+        try:
+            last_rev = self.get_parent_ids()[0]
+        except IndexError:
+            last_rev = None
+        if last_rev != self.branch.last_revision():
             # merge tree state up to new branch tip.
             basis = self.basis_tree()
             to_tree = self.branch.basis_tree()
-            result += merge_inner(self.branch,
+            if basis.inventory.root is None:
+                self.set_root_id(to_tree.inventory.root.file_id)
+            result += merge.merge_inner(
+                                  self.branch,
                                   to_tree,
                                   basis,
                                   this_tree=self)
@@ -1584,13 +1711,14 @@
                 parent_trees.append(
                     (old_tip, self.branch.repository.revision_tree(old_tip)))
             self.set_parent_trees(parent_trees)
+            last_rev = parent_trees[0][0]
         else:
             # the working tree had the same last-revision as the master
             # branch did. We may still have pivot local work from the local
             # branch into old_tip:
             if old_tip is not None:
                 self.add_parent_tree_id(old_tip)
-        if old_tip and old_tip != self.last_revision():
+        if old_tip and old_tip != last_rev:
             # our last revision was not the prior branch last revision
             # and we have converted that last revision to a pending merge.
             # base is somewhere between the branch tip now
@@ -1604,21 +1732,18 @@
                 base_rev_id = None
             base_tree = self.branch.repository.revision_tree(base_rev_id)
             other_tree = self.branch.repository.revision_tree(old_tip)
-            result += merge_inner(self.branch,
+            result += merge.merge_inner(
+                                  self.branch,
                                   other_tree,
                                   base_tree,
                                   this_tree=self)
         return result
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def _write_inventory(self, inv):
         """Write inventory as the current inventory."""
-        sio = StringIO()
-        bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
-        sio.seek(0)
-        self._control_files.put('inventory', sio)
-        self._set_inventory(inv)
-        mutter('wrote working inventory')
+        self._set_inventory(inv, dirty=True)
+        self.flush()
 
     def set_conflicts(self, arg):
         raise UnsupportedOperation(self.set_conflicts, self)
@@ -1628,7 +1753,7 @@
 
     @needs_read_lock
     def conflicts(self):
-        conflicts = ConflictList()
+        conflicts = _mod_conflicts.ConflictList()
         for conflicted in self._iter_conflicts():
             text = True
             try:
@@ -1647,7 +1772,8 @@
                     if text == False:
                         break
             ctype = {True: 'text conflict', False: 'contents conflict'}[text]
-            conflicts.append(Conflict.factory(ctype, path=conflicted,
+            conflicts.append(_mod_conflicts.Conflict.factory(ctype,
+                             path=conflicted,
                              file_id=self.path2id(conflicted)))
         return conflicts
 
@@ -1660,10 +1786,27 @@
      - uses the branch last-revision.
     """
 
+    def lock_tree_write(self):
+        """See WorkingTree.lock_tree_write().
+
+        In Format2 WorkingTrees we have a single lock for the branch and tree
+        so lock_tree_write() degrades to lock_write().
+        """
+        self.branch.lock_write()
+        try:
+            return self._control_files.lock_write()
+        except:
+            self.branch.unlock()
+            raise
+
     def unlock(self):
         # we share control files:
-        if self._hashcache.needs_write and self._control_files._lock_count==3:
-            self._hashcache.write()
+        if self._control_files._lock_count == 3:
+            # _inventory_is_modified is always False during a read lock.
+            if self._inventory_is_modified:
+                self.flush()
+            if self._hashcache.needs_write:
+                self._hashcache.write()
         # reverse order of locking.
         try:
             return self._control_files.unlock()
@@ -1682,8 +1825,8 @@
     """
 
     @needs_read_lock
-    def last_revision(self):
-        """See WorkingTree.last_revision."""
+    def _last_revision(self):
+        """See Mutable.last_revision."""
         try:
             return self._control_files.get_utf8('last-revision').read()
         except NoSuchFile:
@@ -1701,34 +1844,38 @@
             self._control_files.put_utf8('last-revision', revision_id)
             return True
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_conflicts(self, conflicts):
         self._put_rio('conflicts', conflicts.to_stanzas(), 
                       CONFLICT_HEADER_1)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add_conflicts(self, new_conflicts):
         conflict_set = set(self.conflicts())
         conflict_set.update(set(list(new_conflicts)))
-        self.set_conflicts(ConflictList(sorted(conflict_set,
-                                               key=Conflict.sort_key)))
+        self.set_conflicts(_mod_conflicts.ConflictList(sorted(conflict_set,
+                                       key=_mod_conflicts.Conflict.sort_key)))
 
     @needs_read_lock
     def conflicts(self):
         try:
             confile = self._control_files.get('conflicts')
         except NoSuchFile:
-            return ConflictList()
+            return _mod_conflicts.ConflictList()
         try:
             if confile.next() != CONFLICT_HEADER_1 + '\n':
                 raise ConflictFormatError()
         except StopIteration:
             raise ConflictFormatError()
-        return ConflictList.from_stanzas(RioReader(confile))
+        return _mod_conflicts.ConflictList.from_stanzas(RioReader(confile))
 
     def unlock(self):
-        if self._hashcache.needs_write and self._control_files._lock_count==1:
-            self._hashcache.write()
+        if self._control_files._lock_count == 1:
+            # _inventory_is_modified is always False during a read lock.
+            if self._inventory_is_modified:
+                self.flush()
+            if self._hashcache.needs_write:
+                self._hashcache.write()
         # reverse order of locking.
         try:
             return self._control_files.unlock()
@@ -1737,7 +1884,7 @@
 
 
 def get_conflicted_stem(path):
-    for suffix in CONFLICT_SUFFIXES:
+    for suffix in _mod_conflicts.CONFLICT_SUFFIXES:
         if path.endswith(suffix):
             return path[:-len(suffix)]
 
@@ -1849,7 +1996,7 @@
         """
         sio = StringIO()
         inv = Inventory()
-        bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
+        xml5.serializer_v5.write_inventory(inv, sio)
         sio.seek(0)
         control_files.put('inventory', sio)
 
@@ -1880,11 +2027,12 @@
                          _internal=True,
                          _format=self,
                          _bzrdir=a_bzrdir)
-        wt._write_inventory(inv)
-        wt.set_root_id(inv.root.file_id)
         basis_tree = branch.repository.revision_tree(revision)
+        if basis_tree.inventory.root is not None:
+            wt.set_root_id(basis_tree.inventory.root.file_id)
+        # set the parent list and cache the basis tree.
         wt.set_parent_trees([(revision, basis_tree)])
-        build_tree(basis_tree, wt)
+        transform.build_tree(basis_tree, wt)
         return wt
 
     def __init__(self):
@@ -1952,7 +2100,7 @@
         branch = a_bzrdir.open_branch()
         if revision_id is None:
             revision_id = branch.last_revision()
-        inv = Inventory() 
+        inv = Inventory(root_id=gen_root_id())
         wt = WorkingTree3(a_bzrdir.root_transport.local_abspath('.'),
                          branch,
                          inv,
@@ -1960,19 +2108,22 @@
                          _format=self,
                          _bzrdir=a_bzrdir,
                          _control_files=control_files)
-        wt.lock_write()
+        wt.lock_tree_write()
         try:
-            wt._write_inventory(inv)
-            wt.set_root_id(inv.root.file_id)
             basis_tree = branch.repository.revision_tree(revision_id)
-            if revision_id == bzrlib.revision.NULL_REVISION:
+            # only set an explicit root id if there is one to set.
+            if basis_tree.inventory.root is not None:
+                wt.set_root_id(basis_tree.inventory.root.file_id)
+            if revision_id == NULL_REVISION:
                 wt.set_parent_trees([])
             else:
                 wt.set_parent_trees([(revision_id, basis_tree)])
-            build_tree(basis_tree, wt)
+            transform.build_tree(basis_tree, wt)
         finally:
+            # Unlock in this order so that the unlock-triggers-flush in
+            # WorkingTree is given a chance to fire.
+            control_files.unlock()
             wt.unlock()
-            control_files.unlock()
         return wt
 
     def __init__(self):

=== modified file 'bzrlib/xml4.py'
--- bzrlib/xml4.py	2006-07-28 23:53:09 +0000
+++ bzrlib/xml4.py	2006-09-08 18:46:29 +0000
@@ -51,12 +51,12 @@
         e.set('file_id', ie.file_id)
         e.set('kind', ie.kind)
 
-        if ie.text_size != None:
+        if ie.text_size is not None:
             e.set('text_size', '%d' % ie.text_size)
 
         for f in ['text_id', 'text_sha1', 'symlink_target']:
             v = getattr(ie, f)
-            if v != None:
+            if v is not None:
                 e.set(f, v)
 
         # to be conservative, we don't externalize the root pointers
@@ -92,7 +92,7 @@
         ## nodes in the root directory, but it's cleaner to use one
         ## internally.
         parent_id = elt.get('parent_id')
-        if parent_id == None:
+        if parent_id is None:
             parent_id = ROOT_ID
 
         kind = elt.get('kind')

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2006-08-17 14:00:59 +0000
+++ bzrlib/xml5.py	2006-09-10 02:18:31 +0000
@@ -97,6 +97,10 @@
     
     __slots__ = []
 
+    support_altered_by_hack = True
+    # This format supports the altered-by hack that reads file ids directly out
+    # of the versionedfile, without doing XML parsing.
+
     def write_inventory_to_string(self, inv):
         """Just call write_inventory with a StringIO and return the value"""
         sio = cStringIO.StringIO()
@@ -150,7 +154,7 @@
         append(_encode_and_escape(ie.file_id))
         append(' name="')
         append(_encode_and_escape(ie.name))
-        if ie.parent_id != ROOT_ID:
+        if self._parent_condition(ie):
             assert isinstance(ie.parent_id, basestring)
             append(' parent_id="')
             append(_encode_and_escape(ie.parent_id))
@@ -169,6 +173,9 @@
         append(" />\n")
         return
 
+    def _parent_condition(self, ie):
+        return ie.parent_id != ROOT_ID
+
     def _pack_revision(self, rev):
         """Revision object -> xml tree"""
         root = Element('revision',
@@ -228,7 +235,7 @@
             inv.add(ie)
         return inv
 
-    def _unpack_entry(self, elt):
+    def _unpack_entry(self, elt, none_parents=False):
         kind = elt.tag
         if not InventoryEntry.versionable_kind(kind):
             raise AssertionError('unsupported entry kind %s' % kind)
@@ -236,7 +243,7 @@
         get_cached = cache_utf8.get_cached_unicode
 
         parent_id = elt.get('parent_id')
-        if parent_id == None:
+        if parent_id is None and not none_parents:
             parent_id = ROOT_ID
         # TODO: jam 20060817 At present, caching file ids costs us too 
         #       much time. It slows down overall read performances from

=== added file 'bzrlib/xml6.py'
--- bzrlib/xml6.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml6.py	2006-08-24 03:35:48 +0000
@@ -0,0 +1,37 @@
+from bzrlib import cache_utf8, inventory, errors, xml5
+
+
+class Serializer_v6(xml5.Serializer_v5):
+
+    def _append_inventory_root(self, append, inv):
+        """Append the inventory root to output."""
+        append('<inventory')
+        append(' format="6"')
+        if inv.revision_id is not None:
+            append(' revision_id="')
+            append(xml5._encode_and_escape(inv.revision_id))
+        append('>\n')
+        self._append_entry(append, inv.root)
+
+    def _parent_condition(self, ie):
+        return ie.parent_id is not None
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element"""
+        if elt.tag != 'inventory':
+            raise errors.UnexpectedInventoryFormat('Root tag is %r' % elt.tag)
+        format = elt.get('format')
+        if format != '6':
+            raise errors.UnexpectedInventoryFormat('Invalid format version %r'
+                                                   % format)
+        revision_id = elt.get('revision_id')
+        if revision_id is not None:
+            revision_id = cache_utf8.get_cached_unicode(revision_id)
+        inv = inventory.Inventory(root_id=None, revision_id=revision_id)
+        for e in elt:
+            ie = self._unpack_entry(e, none_parents=True)
+            inv.add(ie)
+        return inv
+
+
+serializer_v6 = Serializer_v6()

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2006-07-28 16:05:23 +0000
+++ bzrlib/xml_serializer.py	2006-09-28 05:43:24 +0000
@@ -25,18 +25,27 @@
 from bzrlib.trace import mutter, warning
 
 try:
-    from cElementTree import (ElementTree, SubElement, Element,
-                              XMLTreeBuilder, fromstring, tostring)
-    import elementtree
+    try:
+        # it's in this package in python2.5
+        from xml.etree.cElementTree import (ElementTree, SubElement, Element,
+            XMLTreeBuilder, fromstring, tostring)
+        import xml.etree as elementtree
+    except ImportError:
+        from cElementTree import (ElementTree, SubElement, Element,
+                                  XMLTreeBuilder, fromstring, tostring)
+        import elementtree
+    ParseError = SyntaxError
 except ImportError:
     mutter('WARNING: using slower ElementTree; consider installing cElementTree'
            " and make sure it's on your PYTHONPATH")
+    # this copy is shipped with bzr
     from util.elementtree.ElementTree import (ElementTree, SubElement,
                                               Element, XMLTreeBuilder,
                                               fromstring, tostring)
     import util.elementtree as elementtree
+    from xml.parsers.expat import ExpatError as ParseError
 
-from bzrlib.errors import BzrError
+from bzrlib import errors
 
 
 class Serializer(object):
@@ -50,10 +59,16 @@
         return tostring(self._pack_inventory(inv)) + '\n'
 
     def read_inventory_from_string(self, xml_string):
-        return self._unpack_inventory(fromstring(xml_string))
+        try:
+            return self._unpack_inventory(fromstring(xml_string))
+        except ParseError, e:
+            raise errors.UnexpectedInventoryFormat(e)
 
     def read_inventory(self, f):
-        return self._unpack_inventory(self._read_element(f))
+        try:
+            return self._unpack_inventory(self._read_element(f))
+        except ParseError, e:
+            raise errors.UnexpectedInventoryFormat(e)
 
     def write_revision(self, rev, f):
         self._write_element(self._pack_revision(rev), f)

=== modified file 'contrib/newinventory.py'
--- contrib/newinventory.py	2006-07-28 16:05:23 +0000
+++ contrib/newinventory.py	2006-09-08 18:46:29 +0000
@@ -35,7 +35,7 @@
                 el.set('text_id', ie.text_id)
             if ie.text_sha1:
                 el.set('text_sha1', ie.text_sha1)
-            if ie.text_size != None:
+            if ie.text_size is not None:
                 el.set('text_size', ('%d' % ie.text_size))
         elif kind != 'directory':
             raise BzrError('unknown InventoryEntry kind %r' % kind)
@@ -84,7 +84,7 @@
                 f.write('text_id="%s" ' % ie.text_id)
             if ie.text_sha1:
                 f.write('text_sha1="%s" ' % ie.text_sha1)
-            if ie.text_size != None:
+            if ie.text_size is not None:
                 f.write('text_size="%d" ' % ie.text_size)
             f.write('/>\n')
         elif kind == 'directory':

=== added file 'doc/centralized_workflow.txt'
--- doc/centralized_workflow.txt	1970-01-01 00:00:00 +0000
+++ doc/centralized_workflow.txt	2006-09-09 14:04:50 +0000
@@ -0,0 +1,279 @@
+====================
+Centralized Workflow
+====================
+
+Overview
+========
+
+This document describes some basic workflow for using Bazaar_. This doesn't
+try to explain *why* every step is done, but more gives recommendations
+about what is considered a good way to work with Bazaar_.
+Bazaar_ is designed to be very flexible in workflows, from fully
+decentralized to mostly centralized.
+The practices here are meant to help ease the user into more advanced usage
+of Bazaar_, and allowing them to work in a mix of centralized and
+decentralized operation.
+
+In general, this document is meant for people in a work setting. Where
+several people are working on the same codebase, and want to work with
+eachother and keep in sync. However, this workflow is also applicable to a
+single developer, who might work on several machines, and wants to keep in
+sync with themselves.
+
+.. _Bazaar: http://bazaar-vcs.org
+
+
+Initial Setup
+=============
+
+These are some reasonably simple steps to setup Bazaar_ so that it works
+well for you.
+
+
+Setting User Email
+------------------
+
+Your user identity is stored with each commit. While this doesn't have to
+be accurate, unique, or anything else, it will be used in log messages an
+annotations, so it is nice to have something real.::
+
+  % bzr whoami "John Doe <jdoe@organization.com>"
+
+
+Setting up a local Repository
+-----------------------------
+
+Bazaar_ branches generally copy the history information around with them,
+which is part of how you can work in a fully decentralized manner. As an
+optimization, it is possible for branches to combine their storage needs,
+so that you do not need to copy around all of this history information
+whenever you create a new branch.
+
+The best way to do this is to create a `Shared Repository`_. In general,
+branches will share their storage if they exist in a subdirectory of a
+`Shared Repository`_.  So lets setup a `Shared Repository`_ in our home
+directory, thus all branches we create underneath will share their history
+storage.::
+
+  % bzr init-repo --trees ~
+
+
+Setting up a remote Repository
+------------------------------
+
+Many times you want a location where data is stored separate from where
+you do your work. This workflow is required by centralized systems (CVS/SVN).
+Usually they are on separate machines, but not always. This is actually a
+pretty good setup, especially in a work environment. Because it ensures
+a central location where data can be backed up, and means that if something
+happens to a developer's machine, no committed work has been lost.
+
+So lets set up a shared location for our project. Again, we will use a
+`Shared Repository`_ to optimize disk usage.::
+
+  % bzr init-repo --no-trees sftp://centralhost/srv/bzr/
+
+You can think of this step as similar to setting up a new cvsroot, or
+subversion repository (only obviously it is a little bit simpler).
+
+
+Starting to version an existing project
+=======================================
+
+Now that we have a repository, lets get going with a new project. Most of
+the time, you will already have some code that you started working with,
+that you now want to version using Bazaar_. If the code was originally
+in source control, there are many ways to convert the project to Bazaar_
+without losing any history. However, this is outside of the scope of this
+document. See XXXReferenceConversionOfHistory_.
+
+
+Developer 1: Creating the first revision
+----------------------------------------
+
+So first, we want to create a branch in our remote Repository, where we
+will want to host the project.  Let's assume we have a project named
+"sigil" that we want to start versioning, and create an empty branch::
+
+  % bzr init sftp://centralhost/srv/bzr/sigil
+
+This can be thought of as the "HEAD" branch in CVS terms, or as the "trunk"
+in Subversion terms. We typically call this the ``dev`` branch.
+
+I prefer working in a subdirectory of ``$HOME`` to avoid collisions with all
+the other stuff that ends up in ``$HOME``. Also, we will want a project
+directory where we can hold all of the different branches we end up
+working on::
+
+  % cd ~
+  % mkdir work
+  % cd work
+  % mkdir sigil
+  % cd sigil
+  % bzr checkout sftp://centralhost/srv/bzr/sigil dev
+  % cd dev
+  % cp -ar ~/sigil/* .
+  % bzr add
+  % bzr commit -m "Initial import of Sigil"
+
+There are many ways to setup your working directory, but the above way
+will makes it easy to handle working with feature/bugfix branches. And one
+of the strong points of Bazaar_ is how well it works with branches.
+
+At this point, because you have a 'checkout' of the remote branch, any
+commits you make in ``dev/`` will automatically be saved both locally,
+and on ``centralhost``.
+
+
+Developer N: Getting a working copy of the project
+--------------------------------------------------
+
+Since the first developer did all of the work of creating the project,
+all other developers can just get a checkout of that branch. They should
+still follow `Setting User Email`_ and `Setting up a local Repository`_.
+
+Then to get a copy of the current tip::
+
+  % cd ~/work/sigil
+  % bzr checkout sftp://centralhost/srv/bzr/sigil dev
+
+Now that two people both have a checkout of
+``sftp://centralhost/srv/bzr/sigil``, there will be times when one of
+them is out of date with the current version. Bazaar_ will inform the user
+of this and prevent them from committing. To get up to date use ``bzr
+update``, which will update the tree with the remote changes. This may
+require resolving conflicts, in the case that the same files have been
+modified.
+
+
+Developing on separate branches
+===============================
+
+So far everyone is working and committing their changes into the same
+branch. Which means that everyone needs to update fairly regularly, and
+deal with other people's changes. Also, if one person commits something
+which breaks the codebase, then everyone has to deal with it.
+
+Usually, it is better to do development on individual branches, and then
+integrate those back into the main branch, once they are stable. This is
+one of the biggest changes from working with CVS/SVN. They both allow you
+to work on separate branches, but their merging algorithms are fairly
+weak, so it is difficult to keep things synchronized. Bazaar_ merges track
+what has already been merged, and can even apply changes to files that
+have been renamed.
+
+
+Creating and working on a new branch
+------------------------------------
+
+We want to keep our changes available for other people, even if they
+aren't quite complete yet. So we will create a new public branch, and
+track it locally.::
+
+  % cd ~/work/sigil
+  % bzr branch sftp://centralhost/srv/bzr/sigil \
+               sftp://centralhost/srv/bzr/sigil/doodle-fixes
+  % bzr checkout sftp://centralhost/srv/bzr/sigil/doodle-fixes doodle-fixes
+  % cd doodle-fixes
+
+We now have a place to make any fixes to doodles that we need. And we
+won't interrupt people who are working on other parts of the code.
+Because we have a checkout, any commits made in the ``doodle-fixes/`` will
+also show up on ``centralhost``. [#nestedbranches_]
+It is also completely possible to have 2 developers collaborate on one of
+these branches, just like they would have collaborated on the ``dev``
+branch.[#cbranch]_
+
+.. [#cbranch] When using lots of independent branches, having to retype
+   the full URL all the time takes a lot of typing. We are looking into
+   various methods to help with this, such as branch aliases, etc. For
+   now, though, the bzrtools_ plugin provides the ``bzr cbranch`` command.
+   Which is designed to take a base branch, create a new public branch,
+   and create a checkout of that branch, all with much less typing.
+   Configuring ``cbranch`` is outside the scope of this document, but the
+   final commands look like ``bzr cbranch dev my-feature-branch``
+
+.. [#nestedbranches] It may look odd to have a branch in a subdirectory of
+   another branch. However, this is just fine, and you can think of it as
+   a heirarchial namespace. Where the nested branch is derived from the
+   outer branch.
+
+.. _bzrtools: https://launchpad.net/products/bzrtools
+
+
+Merging changes back
+--------------------
+
+When it is decided that some of the changes in ``doodle-fixes`` is ready
+to be merged into the main tree, simply do::
+
+  % cd ~/work/sigil/dev
+  % bzr merge ../doodle-fixes
+
+Now the changes are available in the ``dev`` branch, but they haven't been
+committed yet. This is the time when you want to review the final changes,
+and make sure they are what you want. ``bzr status`` and ``bzr diff`` are
+good tools to use here. Also, there may be some conflicts in files, if
+there were changes made to the same file. Bazaar_ will prevent you from
+committing until you have resolved these conflicts. That way you don't
+accidentally commit the conflict markers. ``bzr status`` will show the
+conflicts along with the other changes, or you can use ``bzr conflicts``
+to just list conflicts. Use ``bzr resolve file/name`` or ``bzr resolve
+--all`` once conflicts have been handled.[#resolve]_
+If you have a conflict that is particularly difficult to solve you may
+want to use the ``bzr remerge`` command. It will let you try different
+merge algorithms, as well as let you see the original source lines
+(``--show-base``).
+
+.. [#resolve] Some systems make you resolve conflicts as part of the merge
+   process. We have found that it is usually easier to resolve conflicts
+   when you have the view of the entire tree, rather than just a single
+   file. It gives you much more context, and also lets you run tests as
+   you resolve the problems.
+
+
+Recommended Branching
+---------------------
+
+One very common way to handle all of these branches is to give each
+developer their own branch, and their own place to work in the central
+location. This can be done with::
+
+  % bzr branch sftp://centralhost/srv/bzr/sigil \
+               sftp://centralhost/srv/bzr/sigil/user-a
+  % bzr branch sftp://centralhost/srv/bzr/sigil \
+               sftp://centralhost/srv/bzr/sigil/user-b
+
+This gives each developer their own branch to work on. And, they can
+easily create a new feature branch for themselves with just[#cbranch]_::
+
+  % bzr branch sftp://centralhost/srv/bzr/sigil/user-a \
+               sftp://centralhost/srv/bzr/sigil/user-a/feature 
+  % cd ~/work/sigil
+  % bzr checkout sftp://centralhost/srv/bzr/sigil/user-a/feature myfeature
+
+
+Glossary
+========
+
+Shared Repository
+-----------------
+
+Bazaar_ has the concept of a `Shared Repository`_. This is similar to the
+concept of other RCS's repository. Such as in Subversion, where you have a
+remote repository, which is where all of the history is stored, and
+locally you don't have any history information, only a checkout of the
+working tree files. Note that "Shared" in this context means shared
+between branches. It *may* be shared between people, but standalone
+branches can also be shared between people.
+
+In Bazaar_ terms, a `Shared Repository`_ is a location where multiple
+branches can **share** their revision history information. Because Bazaar_
+wants to support decentralized workflows, it is possible for every branch
+to maintain its own revision history information. But this is often
+inefficient, since often branches share history, and they might as well
+share the storage as well.
+
+
+.. 
+   vim: tw=74 ft=rst spell spelllang=en_us

=== modified file 'doc/index.txt'
--- doc/index.txt	2006-07-30 06:52:39 +0000
+++ doc/index.txt	2006-09-13 04:48:33 +0000
@@ -19,30 +19,34 @@
 * `Configuration <configuration.htm>`_ 
   (wiki page: http://bazaar-vcs.org/ConfiguringBzr)
 
-    Information on how to configure Bazaar
+    Information on how to configure Bazaar.
 
 * `Setting up email <setting_up_email.htm>`_
   (wiki page: http://bazaar-vcs.org/BzrSettingEmail)
 
     The various ways that exist to tell Bazaar your email address for
-    commits
+    commits.
 
 * `Plugins <plugins.htm>`_
   (wiki page: http://bazaar-vcs.org/BzrPlugins)
     
-    Information on how to use plugins in Bazaar
+    Information on how to use plugins in Bazaar.
 
 * `Specifying revision <specifying_revisions.htm>`_
   (wiki page: http://bazaar-vcs.org/BzrRevisionSpec)
 
     Description of the subarguments for ``-r`` as used by commands such as
-    **bzr log** and **bzr merge**
+    **bzr log** and **bzr merge**.
 
 * `Using aliases <using_aliases.htm>`_
   (wiki page: http://bazaar-vcs.org/CommandAliases)
 
     How to setup and use command aliases.
 
+* `Running a Bazaar server <server.htm>`_
+  
+    How to run a server to allow remote access to Bazaar branches.
+
 This document automatically created based on output of **bzr help**:
 
 * `Man page (help for bzr commands) <bzr_man.htm>`_

=== added file 'doc/server.txt'
--- doc/server.txt	1970-01-01 00:00:00 +0000
+++ doc/server.txt	2006-09-18 09:58:40 +0000
@@ -0,0 +1,82 @@
+=======================
+Running a Bazaar Server
+=======================
+
+Bazaar does not require a specialised server because it operates over HTTP, FTP
+or SFTP.  There is an optional smart server that can be invoked over SSH, from
+inetd, or in a dedicated mode.
+
+Dumb servers
+============
+
+We describe HTTP, FTP, SFTP and HTTP-WebDAV as "dumb" servers because they do
+not offer any assistance to Bazaar.  If you make a Bazaar repository available
+over any of these protocols, Bazaar will allow you to read it remotely.  Just
+enter the URL to the branch in the Bazaar command you are running.::
+
+    bzr log http://bazaar-vcs.org/bzr/bzr.dev
+
+Bazaar supports writing over FTP, SFTP and via a plugin over HTTP-WebDAV.
+
+High performance server
+=======================
+
+**In development**
+
+The high performance server is currently in development.  The version of Bazaar
+that accompanies this documentation is able to use the servers underlying
+protocol as a dumb server.  This is the first stage high performance server
+functionality to be delivered.
+
+The current high performance server provides read only access by default for
+security. To enable read-write access, run it with --allow-writes. When using
+the SSH access method, bzr automatically runs with with the --allow-writes
+option.
+
+It can be configured to run in three different ways:
+
+SSH
+---
+
+Using Bazaar over SSH requires no special configuration on the server::
+
+    BZR_REMOTE_PATH=~/bin/bzr bzr log bzr+ssh://host/path/to/branch
+
+The `BZR_REMOTE_PATH` environment variable adjusts how `bzr` will be invoked on
+the remote system.  By default, just `bzr` will be invoked, which requires the
+`bzr` executable to be on the default search path.
+
+The `bzr+ssh://` URL scheme only supports absolute paths from the root of the
+filesystem.  Future versions are expected to support `~` in the same way as
+`sftp://` URLs.
+
+inetd
+-----
+
+This example shows how to run `bzr` with a dedicated user `bzruser` for a shared
+repository in `/srv/bzr/repo` which has a branch at `/srv/bzr/repo/branchname`
+
+Running a Bazaar server from inetd requires an inetd.conf entry::
+
+    1234  stream  tcp  nowait  bzruser  /usr/bin/bzr serve --inet --directory=/srv/bzr/repo
+
+When running client commands, the URL you supply is a `bzr://` URL relative to
+the `--directory` option given in inetd.conf::
+
+    bzr log bzr://host:1234/branchname
+
+Dedicated
+---------
+
+This mode has the same path and URL behaviour as the inetd mode.  To run as a
+specific, you should use `su` or login as that user.  This example runs bzr
+serve on `localhost` port `1234`.
+
+server::
+
+    bzr serve --port=localhost:1234 --directory=/srv/bzr/repo
+    
+client::
+
+    bzr log bzr://host:1234/branchname
+

=== added file 'doc/version_info.txt'
--- doc/version_info.txt	1970-01-01 00:00:00 +0000
+++ doc/version_info.txt	2006-09-21 22:12:28 +0000
@@ -0,0 +1,57 @@
+==========================
+Using ``bzr version-info``
+==========================
+
+Overview
+========
+
+This document describes ways of using ``bzr version-info`` as part of a
+build routine to embed version information into a final project.
+
+
+Python Project
+==============
+
+TODO: Figure out how to attach into ``setup.py``
+
+
+If using a Makefile to build your project, you can generate the version
+information file as simply as::
+
+  library/_version.py:
+        bzr version-info --format=python > library/_version.py
+
+This generates a file which contains 3 dictionaries:
+
+  `version_info`: A dictionary containing the basic information about the
+                  current state.
+
+  revisions: A dictionary listing all of the revisions in the history of
+             the tree, along with the commit times and commit message.
+             This defaults to being empty unless ``--all`` or
+             ``--include-history`` is supplied. This is useful if you
+             want to track what bugfixes, etc, might be included in the
+             released version. But for many projects it is more
+             information than they need.
+
+  `file_revisions`: A dictionary listing the last-modified revision for
+                    all files in the project. This can be used similarly
+                    to how ``$Id$`` keywords are used in CVS controlled
+                    files. The last modified date can be determined by
+                    looking in the ``revisions`` map. This is also empty by
+                    default, and enabled only by ``--all`` or
+                    ``--include-file-revisions``.
+
+Check Clean
+===========
+
+Most information about the contents of the project can be cheaply
+determined by just reading the revision entry. However, it can be useful
+to know if the working tree was completely up-to-date when it was
+packaged, or if there was a local modification. By supplying either
+``--all`` or ``--check-clean``, ``bzr`` will inspect the working tree, and
+set the ``clean`` flag in ``version_info``, as well as set entries in
+``file_revisions`` as ``modified`` where appropriate.
+
+.. 
+   vim: tw=74 ft=rst spell spelllang=en_us

=== modified file 'profile_imports.py'
--- profile_imports.py	2006-06-19 14:40:19 +0000
+++ profile_imports.py	2006-10-11 00:23:23 +0000
@@ -82,7 +82,7 @@
         # scope_name, frame_name, frame_lineno
         out_file.write('%5.1f %5.1f %s %-35s\t@ %s:%d\n'
             % (info[-1]*1000., mod_time*1000., '+'*info[0], 
-               cur[1][:40], info[1], info[2]))
+               cur[1][:35], info[1], info[2]))
 
         if sorted:
             c_times.sort()
@@ -115,16 +115,21 @@
     frame = sys._getframe(1)
     frame_name = frame.f_globals.get('__name__', '<unknown>')
     extra = ''
-    cur_frame = 1
     if frame_name.endswith('demandload'):
         # If this was demandloaded, we have 3 frames to ignore
-        extra = ' (demandload)'
-        frame = sys._getframe(4)
-        cur_frame = 4
-        frame_name = frame.f_globals.get('__name__', '<unknown>')
+        extra = '(demandload) '
+        frame = sys._getframe(4)
+        frame_name = frame.f_globals.get('__name__', '<unknown>')
+    elif frame_name.endswith('lazy_import'):
+        # If this was lazily imported, we have 3 frames to ignore
+        extra = '[l] '
+        frame = sys._getframe(4)
+        frame_name = frame.f_globals.get('__name__', '<unknown>')
+    if fromlist:
+        extra += ' [%s]' % (', '.join(map(str, fromlist)),)
     frame_lineno = frame.f_lineno
 
-    this = stack_add(name+extra, frame_name, frame_lineno, scope_name)
+    this = stack_add(extra + name, frame_name, frame_lineno, scope_name)
 
     tstart = time.time()
     try:
@@ -145,9 +150,15 @@
     # And who is requesting this?
     frame = sys._getframe(2)
     frame_name = frame.f_globals.get('__name__', '<unknown>')
+
+    extra = ''
+    if frame_name.endswith('lazy_regex'):
+        # If this was lazily compiled, we have 3 more frames to ignore
+        extra = '[l] '
+        frame = sys._getframe(5)
+        frame_name = frame.f_globals.get('__name__', '<unknown>')
     frame_lineno = frame.f_lineno
-
-    this = stack_add(repr(args[0]), frame_name, frame_lineno)
+    this = stack_add(extra+repr(args[0]), frame_name, frame_lineno)
 
     tstart = time.time()
     try:

=== modified file 'setup.py' (properties changed: +x to -x)
--- setup.py	2006-08-17 13:53:36 +0000
+++ setup.py	2006-09-08 18:46:29 +0000
@@ -58,7 +58,7 @@
     print >>sys.stderr, "bzr: error: cannot find a suitable python interpreter"
     print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
     sys.exit(1)
-if hasattr(os, "unsetenv"):
+if getattr(os, "unsetenv", None) is not None:
     os.unsetenv(REINVOKE)
 
 

=== modified file 'tools/http_client.py'
--- tools/http_client.py	2005-09-19 06:05:19 +0000
+++ tools/http_client.py	2006-09-08 18:46:29 +0000
@@ -235,7 +235,7 @@
         debug("can't handle response status %r" % r.status)
         raise NotImplementedError()
 
-    if r.content_length == None:
+    if r.content_length is None:
         raise NotImplementedError()
 
     if r.content_length == 0:

